/******************************************************************************
 * Copyright (c) 2006 PalmSource, Inc (an ACCESS company). All rights reserved.
 *****************************************************************************/
/**
 * @file gras_settings.h
 *
 * @brief GHostRAS Settings access exported functions
 *
 *****************************************************************************/

#ifndef ALP_PDK_GRAS_SETTINGS_H_
#define ALP_PDK_GRAS_SETTINGS_H_	1

#include <hiker/global_settings.h>
#include <alp/pdk/gras.h>

/**
 * @addtogroup ALP_GHOST_SETTING Settings Access
 * @ingroup ALP_GHOST
 * @{
 */

#define ALP_GRAS_TRACE_MODULE				ALP_TRACE_DEFINE( ALP_CLASS_GRAS, 5)

// Global setting stuff
#define ALP_GRAS_SETTINGS_ROOT_MODE			S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IXGRP | S_IROTH | S_IWOTH | S_IXOTH	// 0777
//#define ALP_GRAS_SETTINGS_ROOT_MODE			S_IRUSR | S_IWUSR | S_IXUSR | S_IRGRP | S_IWGRP | S_IROTH | S_IWOTH	// 0766
#define ALP_GRAS_SETTINGS_ROOT_DIR			"/bundles/bar:com.access.apps.gvm_panel/saved"
#define ALP_GRAS_SETTINGS_STORAGE_SIZE		ALP_GRAS_SETTINGS_ROOT_DIR"/StorageSize"
#define ALP_GRAS_SETTINGS_LOCALE			ALP_GRAS_SETTINGS_ROOT_DIR"/Locale"
#define ALP_GRAS_SETTINGS_SOUND_ON			ALP_GRAS_SETTINGS_ROOT_DIR"/SoundOn"
#define ALP_GRAS_SETTINGS_SD_NOTIF			ALP_GRAS_SETTINGS_ROOT_DIR"/SDNotification"
#define ALP_GRAS_SETTINGS_KEYS				ALP_GRAS_SETTINGS_ROOT_DIR"/keys"
#define ALP_GRAS_SETTINGS_ROOT_APPLS		ALP_GRAS_SETTINGS_ROOT_DIR"/appls/"

#define APPL_ID_PATH_SIZE					256

#define	ALP_GARNET_68K_APPL_TYPE			'appl'

#define FOUR_BYTES_LENGTH					4		// type or creator size in bytes

#define	UINT32_TO_STR(x,s)	do { \
								uint32_t _x = (uint32_t)(x); \
								s[0] = (char)(_x >> 24); \
								s[1] = (char)(_x >> 16); \
								s[2] = (char)(_x >> 8); \
								s[3] = (char)_x; \
								s[4] = 0; \
							} while(0)

#define	STR_TO_UINT32(s,x)	do { \
								uint8_t* _s = (uint8_t*)s; \
								x = (_s[0]<< 24) + (_s[1]<<16) + (_s[2]<<8) + _s[3]; \
							} while(0)

struct _Alp68KAppSettings {
	gint					screenSize;
	gint					dynHeapSize;
	gint					deviceIdx;
	uint32_t				deviceID;
	uint32_t				companyID;
	gint					soundActive;
	Alp68KAppKeysSettings	keys;
	gint					screenMode;
	gint					vmouse;
};
typedef struct _Alp68KAppSettings Alp68KAppSettings;

struct _Alp68KGeneralSettings {
	gint					storageSize;
	gint					locale;
	gboolean				sdNotification;
};
typedef struct _Alp68KGeneralSettings Alp68KGeneralSettings;

struct _Alp68KSettingsContext {
	int	opaque;
};
typedef struct _Alp68KSettingsContext Alp68KSettingsContext;


#ifdef __cplusplus
extern "C" {
#endif

/******************************************************************************
 * APIs to use for retrieve settings for use with alp_gras_app_launch API
 *****************************************************************************/
alp_status_t alp_gras_settings_init_context(Alp68KSettingsContext **context);

alp_status_t alp_gras_settings_deinit_context(Alp68KSettingsContext *context);

// Retrieves from settings storage all the necessary fields.
alp_status_t alp_gras_settings_retrieve_68kapp_launch_parameters(Alp68KSettingsContext *context, uint32_t app_type, uint32_t app_creator, char *app_dbname, Alp68KAppConfig *app_gras_config);

/******************************************************************************
 * Easy APIs to use for retrieve general settings for use with alp_gras_open_session API
 *****************************************************************************/
alp_status_t alp_gras_settings_retrieve_general_session_parameters(int* storageSize, uint32_t* locale);

alp_status_t alp_gras_settings_retrieve_autostart(int* autostart);

/******************************************************************************
 * APIs to manipulate/store/retrieve settings
 *****************************************************************************/
// Fills it with default values
void alp_gras_settings_general_default(Alp68KSettingsContext *settings_context, Alp68KGeneralSettings *ghost_settings);

// Stores provided general settings
alp_status_t alp_gras_settings_set_general(Alp68KSettingsContext *context, Alp68KGeneralSettings *ghost_settings);

// Retrieves general settings
alp_status_t alp_gras_settings_get_general(Alp68KSettingsContext *context, Alp68KGeneralSettings *ghost_settings);

// Fills it with default values for keys mapping
void alp_gras_settings_keys_default(Alp68KSettingsContext *settings_context, Alp68KAppKeysSettings *keys_settings);

// Stores general keys mapping
alp_status_t alp_gras_settings_set_general_keys(Alp68KSettingsContext *context, Alp68KAppKeysSettings *app_keys_settings);

// Retrieves general keys mapping
alp_status_t alp_gras_settings_get_general_keys(Alp68KSettingsContext *context, Alp68KAppKeysSettings *app_keys_settings);
		
// Fills it with default values for 68K applications
void alp_gras_settings_68kapp_default(Alp68KSettingsContext *settings_context, Alp68KAppSettings *app_settings);

// Stores 68K application settings
alp_status_t alp_gras_settings_set_68kapp(Alp68KSettingsContext *context, uint32_t app_type, uint32_t app_creator, char *app_dbname, Alp68KAppSettings *app_settings);

// Retrieves 68K application settings
alp_status_t alp_gras_settings_get_68kapp(Alp68KSettingsContext *context, uint32_t app_type, uint32_t app_creator, char *app_dbname, Alp68KAppSettings *app_settings);

/* @} */

#ifdef __cplusplus
}
#endif

#endif /* ALP_PDK_GRAS_SETTINGS_H_ */
