/******************************************************************************
 * Copyright (c) 2006 PalmSource, Inc (an ACCESS company). All rights reserved.
 *****************************************************************************/
/*!
 * @file <power.h>
 *
 * @brief Declarations for ALP Power Manager routines.
 *
 *****************************************************************************/

#ifndef ALP_POWER_H_
#define ALP_POWER_H_ 1

#ifdef __cplusplus
extern "C" {
#endif

/*! @addtogroup ALP_POWER_MANAGER Power Manager
 * @ingroup ALP_SYSTEM
 * @{
 */
	

#include <hiker/sysclass.h>
#include <glib.h>
#include <time.h>

// declare this here, to prevent everyone who uses the file from needing GTK
struct _GtkWindow;


/*! @name Power Manager Error Codes
 *	@{
 */

#define ALP_STATUS_POWER_INVALID_PARAM	(ALP_CLASS_POWER | 0x00010000)	//!< Invalid parameter value
#define ALP_STATUS_POWER_NO_FREE_RAM	(ALP_CLASS_POWER | 0x00020000)	//!< Memory allocation failed
#define ALP_STATUS_POWER_NO_MISER		(ALP_CLASS_POWER | 0x00030000)	//!< Can't connect to Power Miser daeomon
#define ALP_STATUS_POWER_NOT_ALLOWED	(ALP_CLASS_POWER | 0x00040000)	//!< Operation not allowed
#define ALP_STATUS_POWER_TIME_PASSED	(ALP_CLASS_POWER | 0x00050000)	//!< Requested wake-up time is in the past
#define ALP_STATUS_POWER_NO_WAKEUP		(ALP_CLASS_POWER | 0x00060000)	//!< Specified wake-up is not active

//! @}

/*! @name Backlight and Timeout Names
 *
 *  These constants can be used as the \a name arguments to
 *  alp_power_set_default_timeout() and alp_power_set_brightness(), to specify
 *  the backlight device(s) to which the new timeout/brightness setting should
 *  be applied.
 *  
 *  @{
 */

#define ALP_POWER_PRIMARY_LCD		"_primary-lcd"	//!< Primary LCD backlight
#define ALP_POWER_PRIMARY_KEYBOARD	"_primary-key"	//!< Primary keyboard/keypad backlight
#define ALP_POWER_ALL_PRIMARY		"_all-primary"	//!< All primary backlights (LCD and keypad)
#define ALP_POWER_AUTO_OFF			"_auto-off"		//!< Auto-off timer (timeout only)
#define ALP_POWER_ALL_TIMEOUTS		"_all-timeouts"	//!< Auto-off timer and all primary backlights (timeout only)

//! @}


/*! @name Wake State Names
 *  *
 *   *  These values describe the device's current "wake state": fully on,
 *    *  partially on, or asleep.
 *     *
 *      * @{
 *       */

typedef enum {
	        ALP_POWER_FULL_WAKE_STATE,      //!< Device fully awake, screen on
	        ALP_POWER_SEMI_WAKE_STATE,      //!< Device partially awake, screen off
	        ALP_POWER_SLEEP_STATE,          //!< Device asleep, can be re-awakened by hardware or software events
} AlpPowerWakeState;


/*! @brief Prototype for functions which handle Power Manager wake-up events */
typedef gboolean (*AlpPowerWakeupCallback)(gpointer data);



/*!
 @} // endgroup ALP_POWER_MANAGER
 */

#ifdef __cplusplus
} 	// extern "C"
#endif 



#endif	/* ALP_POWER_H_ */


