/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/

/*!
 * @file exgmgr.h 
 *
 * @brief Public headers for Exchange Manager
 *
 */


#ifndef EXGMGR_H_
#define EXGMGR_H_

#include <sys/types.h>
#include <hiker/types.h>
#include <hiker/sysclass.h>

#ifdef __cplusplus
 extern "C"
 {
#endif

/*! @addtogroup ALP_EXCHANGE_MGR Exchange Manager
 *  @ingroup ALP_SYSTEM
 *  @{
 */

/*******************************************************************************
 * Errors
 *******************************************************************************/

/*! @name Exchange Manager Errors
 *  @{
 */
//! operation completed successfully
#define ALP_EXGMGR_ERR_NO_ERROR						((alp_status_t) (0))
//! request pointer is NULL
#define ALP_EXGMGR_ERR_NULL_REQUEST_ERROR 			((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00010000))
//! parameter not found
#define ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00020000))
//! cannot allocate memory//!< cannot allocate memory
#define ALP_EXGMGR_ERR_OUT_OF_MEMORY_ERROR 			((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00030000))
//! buffer to small
#define ALP_EXGMGR_ERR_BUF_TOO_SMALL_ERROR 			((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00040000))
//! verb string is too long
#define ALP_EXGMGR_ERR_VERB_TOO_LONG_ERROR 			((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00050000))
//! MIME Type string is too long
#define ALP_EXGMGR_ERR_MIMETYPE_TOO_LONG_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00060000))
//! Package Name string is too long
#define ALP_EXGMGR_ERR_PACKAGENAME_TOO_LONG_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00070000))
//! cannot create socket
#define ALP_EXG_MGR_ERR_SOCKET_ERROR 				((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00080000))
//! cannot bind socket
#define ALP_EXG_MGR_ERR_BIND_ERROR					((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00090000))
//! cannot listen socket
#define ALP_EXG_MGR_ERR_LISTEN_ERROR 				((alp_status_t) (ALP_CLASS_EXCHANGE | 0x000a0000))
//! cannot connect socket
#define ALP_EXG_MGR_ERR_CONNECT_ERROR 				((alp_status_t) (ALP_CLASS_EXCHANGE | 0x000b0000))
//! cannot write to destination
#define ALP_EXG_MGR_ERR_WRITE_ERROR 				((alp_status_t) (ALP_CLASS_EXCHANGE | 0x000c0000))
//! cannot send fd to destination
#define ALP_EXG_MGR_ERR_WRITE_FD_ERROR 				((alp_status_t) (ALP_CLASS_EXCHANGE | 0x000d0000))
//! cannot read
#define ALP_EXG_MGR_ERR_READ_ERROR 					((alp_status_t) (ALP_CLASS_EXCHANGE | 0x000e0000))
//! cannot receive fd from daemon
#define ALP_EXG_MGR_ERR_READ_FD_ERROR 				((alp_status_t) (ALP_CLASS_EXCHANGE | 0x000f0000))
//! the request the handler is trying to receive does not exist
#define ALP_EXG_MGR_ERR_NO_SUCH_REQUEST_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00100000))
//! the request was not received before it is completed
#define ALP_EXG_MGR_ERR_REQUEST_NOT_RECEIVED_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00110000))
//! the transport the request is trying to use does not exist
#define ALP_EXG_MGR_ERR_NO_SUCH_TRANSPORT_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00120000))
//! Notification Type string is too long
#define ALP_EXGMGR_ERR_NOTIFICATIONTYPE_TOO_LONG_ERROR ((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00130000))
//! DBUS subsystem returned an error
#define ALP_EXGMGR_ERR_DBUS_ERROR 					((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00140000))
//! Cannot create a temp file
#define ALP_EXGMGR_ERR_CANNOT_CREATE_TMP_FILE_ERROR ((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00150000))
//! no data fd was found in the request (mandatory for legacy obex)
#define ALP_EXGMGR_ERR_NO_DATA_FD_ERROR 			((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00160000))
//! general file system error during operations
#define ALP_EXGMGR_ERR_FILE_SYSTEM_ERROR 			((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00170000))
//! localized verb string is too long
#define ALP_EXGMGR_ERR_LOCALIZED_VERB_TOO_LONG_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00180000))
//! OBEX specific parameters are allowed only in an ALP_EXGMGR_STORE_OBJECT_VERB  request
#define ALP_EXGMGR_ERR_OBEX_PARAM_NOT_ALLOWED_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00190000))
//! Request execution canceled by initiator
#define ALP_EXGMGR_ERR_USER_CANCEL_ERROR 				((alp_status_t) (ALP_CLASS_EXCHANGE | 0x001a0000))
//! Error while updating the registration cache
#define ALP_EXGMGR_ERR_REG_CACHE_ERROR 				((alp_status_t) (ALP_CLASS_EXCHANGE | 0x001b0000))
//! Too many concurent async requests executing
#define ALP_EXGMGR_ERR_TOO_MANY_ASYNC_EXECUTE_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x001c0000))
//! General error in API parameter usage
#define ALP_EXGMGR_ERR_API_ERR_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x001d0000))
//! Cannot open tmp data file
#define ALP_EXGMGR_ERR_CANNOT_OPEN_TMP_FILE_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x001e0000))
//! General daemon init error
#define ALP_EXGMGR_ERR_DAEMON_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x001f0000))

//! there is already a handler registered for this verb, MIMEType and package
#define ALP_EXG_MGR_ERR_HANDLER_ALREADY_REGISTERED_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00200000))
//! there is already a handler registered for this verb and MIMEType, and iMustBeUnique was true.
#define ALP_EXG_MGR_ERR_HANDLER_ALREADY_EXISTS_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00210000))
//! no handler exists for this verb and MIMEType
#define ALP_EXG_MGR_ERR_HANDLER_NOT_FOUND_ERROR 			((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00220000))
//! communication error between handler and daemon
#define ALP_EXG_MGR_ERR_HANDLER_COMMUNICATION_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00230000))
//! handler did not repond to invocation within the expected delay
#define ALP_EXG_MGR_ERR_HANDLER_TIMEOUT_ERROR 				((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00240000))
//! handler did not consume the request (used by handlers that accept multiple registrants)
#define ALP_EXG_MGR_ERR_HANDLER_DID_NOT_CONSUME_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00250000))
//! handler accepts requests from local device only
#define ALP_EXG_MGR_ERR_HANDLER_IS_LOCAL_ONLY_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00260000))

//! cannot open anymore transport plug-in
#define ALP_EXG_MGR_ERR_TOO_MANY_TRANSPORT_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00900000))
//! cannot open transport plug-in
#define ALP_EXG_MGR_ERR_CANNOT_OPEN_TRANSPORT_LIB_ERROR ((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00910000))
//! cannot get transport function pointer
#define ALP_EXG_MGR_ERR_CANNOT_GET_TRANSPORT_FUNC_ERROR ((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00920000))
//! cannot create a new GSource
#define ALP_EXG_MGR_ERR_CANNOT_CREATE_GSOURCE_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00930000))
//! cannot attach the GSource
#define ALP_EXG_MGR_ERR_CANNOT_ATTACH_GSOURCE_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00940000))
//! transport cannot create socket
#define ALP_EXG_MGR_ERR_TRANSPORT_SOCKET_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00b00000))
//! transport cannot bind socket
#define ALP_EXG_MGR_ERR_TRANSPORT_BIND_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00b10000))
//! transport cannot listen socket
#define ALP_EXG_MGR_ERR_TRANSPORT_LISTEN_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00b20000))
//! transport cannot connect socket
#define ALP_EXG_MGR_ERR_TRANSPORT_CONNECT_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00b30000))
//! transport cannot write to destination
#define ALP_EXG_MGR_ERR_TRANSPORT_WRITE_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00b40000))
//! transport cannot send fd to destination
#define ALP_EXG_MGR_ERR_TRANSPORT_WRITE_FD_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00b50000))
//! transport cannot read
#define ALP_EXG_MGR_ERR_TRANSPORT_READ_ERROR 		((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00b60000))
//! transport cannot receive fd from other side
#define ALP_EXG_MGR_ERR_TRANSPORT_READ_FD_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00b70000))
//! transport cannot determine destination address
#define ALP_EXG_MGR_ERR_TRANSPORT_DEST_NOT_FOUND_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00b80000))
//! cannot find a transport param item
#define ALP_EXG_MGR_ERR_TRANSPORT_DEST_ITEM_NOT_FOUND_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00b90000))
//! transport is busy and cannot send at this time
#define ALP_EXG_MGR_ERR_TRANSPORT_BUSY_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00ba0000))
//! transport cannot open a temp file
#define ALP_EXG_MGR_ERR_TRANSPORT_OPEN_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00bb0000))
//! Obex could not send the object correctly
#define ALP_EXG_MGR_ERR_TRANSPORT_OBEXSEND_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00bc0000))
//! Obex could not receive the object correctly
#define ALP_EXG_MGR_ERR_TRANSPORT_OBEXRCV_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00bd0000))
//! Obex reply was not received within timeout
#define ALP_EXG_MGR_ERR_TRANSPORT_OBEXREPLY_TIMEOUT_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00be0000))
//! Obex server did not respond to our send request (it may not be started)
#define ALP_EXG_MGR_ERR_TRANSPORT_OBEX_TIMEOUT_ERROR 	((alp_status_t) (ALP_CLASS_EXCHANGE | 0x00bf0000))
//! @}

/*! @name Exchange Manager Version
 *  @{
 */
//! This is useful for transport modules only
#define ALP_EXGMGR_VERSION_1	1
//! @}

/*! @name Request reference
 *  @{
 */
//! This is the public opaque type. A request is formed and accessed using Exchange Manager API only.
typedef void *AlpExgRequest;
//! This is the reference of the request passed to an exchange handler.
typedef void *AlpExgRequestId;
//! @}

/*! @name String Parameters Max Lens
 *  @{
 */
//! max len for the verb
#define ALP_EXGMGR_VERB_MAXLEN			20		
//! max len for the localized verb
#define ALP_EXGMGR_LOCALIZED_VERB_MAXLEN	60		
//! max len for the MIME Type
#define ALP_EXGMGR_MIMETYPE_MAXLEN		50
//! max len for the package name
#define ALP_EXGMGR_PACKAGENAME_MAXLEN	150
//! max len for the notification type
#define ALP_EXGMGR_NOTIFICATIONTYPE_MAXLEN	40
//! @}

/*! @name Send/Receive Obex compatible well known verb
 *  @{
 */
//!< When using this verb, IR and BT will use a standard OBEX send to transmit the data. Receiving objects through OBEX will also invoke a handler for this verb.
#define ALP_EXGMGR_STORE_OBJECT_VERB	"send_obex_legacy"
//! @}

/*! @name Generic request string parameter name
 *  @{
 */
//! When using a generic request, the data must be passed in either the request file descriptor OR in this string parameter.
#define ALP_EXGMGR_GENERIC_DATA_PARAMETER_NAME	"alp_exg_generic_dataparam"
//! @}

/*! @name Generic data types
 *  @{
 */
//! Use this type to register a handler that can send or attach any type of data.
#define ALP_EXGMGR_ATTACH_ALL_MIME_TYPE	"alp_exg_attach_all_types"
//! Use this type when querying verbs that can do something with a string reprensenting a phone number
#define ALP_EXGMGR_GENERIC_PHONE_STRING_MIME_TYPE	"alp_exg_generic_phone_string"
//! Use this type when querying verbs that can do something with a string reprensenting an url
#define ALP_EXGMGR_GENERIC_URL_STRING_MIME_TYPE		"alp_exg_generic_url_string"
//! Use this type when querying verbs that can do something with a string reprensenting an email
#define ALP_EXGMGR_GENERIC_EMAIL_STRING_MIME_TYPE	"alp_exg_generic_email_string"
//! @}

/*! @name Registering/Unregistering handlers
 *
 *  @{
 */
 
/*! @brief Declare an action handler for a specific verb/type of data combination

 An application must register with the exchange manager for all the verb/MIMEtype combinations it knows how to handle.
 The package specified will be launched (or relauched) when there is an incoming request ready to be handled. Only
 ALP Packages can be launched this way.
 
 @param[in] iVerb the verb this handler implements
 @param[in] iLocalizedVerb the localized verb to be displayed to the user when a list of generic verbs is built. If NULL, iVerb will be returned.
 @param[in] iMIMEType the type of the data associated to the verb
 @param[in] iHandlerPackageName the package name of the application that will be launched to execute a request. The name must include the bundle type (e.g. "bar:com.appname")
 @param[in] iLocalOnly specifies that this handler will execute requests originating from the local device only
 @param[in] iSerializeMe specifies that this handler may or may not be executed simultaneously to other handlers
 @param[in] iMustBeUnique specifies that this handler does not accept other apps to register for the same verb/data.
 			if the handler passes zero to allow other to register, then it must not return data or parameters as these
 			would not be sent back to the request client.
 @param[in] iAcceptGenericInvocation specifies that the handler understands the generic data passing convention

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/

alp_status_t alp_exg_handler_register (char *iVerb, char *iLocalizedVerb, char *iMIMEType,
									char *iHandlerPackageName,
									int iLocalOnly, int iSerializeMe, int iMustBeUnique, int iAcceptGenericInvocation);
									
/*! @brief Declare an action handler for a specific verb/type of data combination

 An application must register with the exchange manager for all the verb/MIMEtype combinations it knows how to handle.
 The notification type specified will be broadcasted when there is an incoming request ready to be handled.
 The notification type must have been previsouly registered with the notification manager. Using this handler registration
 mechanism allows daemons or other non ALP Packages to be called to process incoming exchange requests.
 
 @param[in] iVerb the verb this handler implements
 @param[in] iLocalizedVerb the localized verb to be displayed to the user when a list of generic verbs is built. If NULL, iVerb will be returned.
 @param[in] iMIMEType the type of the data associated to the verb
 @param[in] iNotificationType the notification type that will be broadcast when there is a request for this verb/data type
 @param[in] iLocalOnly specifies that this handler will execute requests originating from the local device only
 @param[in] iSerializeMe specifies that this handler may or may not be executed simultaneously to other handlers
 @param[in] iMustBeUnique specifies that this handler does not accept other apps to register for the same verb/data.
 			if the handler passes zero to allow other to register, then it must not return data or parameters as these
 			would not be sent back to the request client.
 @param[in] iAcceptGenericInvocation specifies that the handler understands the generic data passing convention

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/

alp_status_t alp_exg_handler_register_notification (char *iVerb, char *iLocalizedVerb, char *iMIMEType,
									char *iNotificationType,
									int iLocalOnly, int iSerializeMe, int iMustBeUnique, int iAcceptGenericInvocation);
										
/*! @brief Undeclare a previously registered action handler

 If an application registers a handler for its own usage, it will need to unregister it when it quits.

 @param[in] iVerb the verb this handler implements
 @param[in] iMIMEType the type of the data associated to the verb
 @param[in] iHandlerPackageName is the same package name which was used for registering 
 
 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_handler_unregister (char *iVerb, char *iMIMEType, char *iHandlerPackageName);


/*! @brief Undeclare a previously registered action handler by notification

 If an application or daemon registers a handler by notification, it will need to unregister it when it quits.

 @param[in] iVerb the verb this handler implements
 @param[in] iMIMEType the type of the data associated to the verb
 @param[in] iNotificationType is the same package name which was used for registering 
 
 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_handler_unregister_notification (char *iVerb, char *iMIMEType, char *iNotificationType);

//! @}

/*! @name Check Handler Registration
 *
 *  @{
 */
 
/*! @brief Determine if a handler exist for a verb/data type pair.
 
 An application can use this function to determine if the specified handler is registered on the local device.

 @param[in] iVerb the verb this handler implements
 @param[in] iMIMEType the type of the data associated to the verb
 
 @return
	#ALP_EXGMGR_ERR_NO_ERROR - there is at least one handler registered for this verb/data type<br>
	#ALP_EXG_MGR_ERR_HANDLER_NOT_FOUND_ERROR - there is no handler registered for this verb/data type<br>
*/
alp_status_t alp_exg_handler_check_registration (char *iVerb, char *iMIMEType);

//! @}

/*! @name Query Generic verbs
 *
 *  @{
 */
 
/*! @brief Query a list of verbs that can handle a given data type in a generic mode
 
 An application can use this function to build a dynamic menu of actions that can be performed on some data
 that it knows the type. Only the verbs that can handle generic requests will be returned. 

 @param[in] iMIMEType the data type for which you want the registered verbs
 @param[out] oEntryCount the number of verbs in the returned buffer
 @param[out] oDataP a malloc-ed buffer containing a pair of null terminated strings for each entry. The first string is the verb to use in the request, the second string is the localized verb to display to the user. This buffer must be freeed by caller.
 
 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_handler_query_verbs (char *iMIMEType, int *oEntryCount, char **oDataP);

//! @}

/*! @name Create and Delete Requests
 *
 *  @{
 */
 
 /*! @brief Create an empty request object

 Everything works around the request. You must always create a request before you can do anything useful.

 @param[out] oRequest pointer to an AlpExgRequest variable that will receive the created request reference.

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_create (AlpExgRequest *oRequest);

/*! @brief Delete a request object

 @param[in] iRequest the request to destroy

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_delete (AlpExgRequest iRequest);

//! @}

/*! @name Construct a Request
 *
 *  @{
 */
 
/*! @brief Set the request verb

 @param[in] iRequest the request you are working on
 @param[in] iVerb theVerb defining the action you want to perform

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_set_verb (AlpExgRequest iRequest, char *iVerb);

/*! @brief Set the request data source using a file descriptor

 This function can be used by the request initiator as well as by the action handler to return data.

 @param[in] iRequest the request you are working on
 @param[in] iFd the file descriptor from which the data is readable. Note that once this function returns no error, the file descriptor belongs to the request and will be closed when the request is deleted.

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_set_data_fd (AlpExgRequest iRequest, int iFd);

/*! @brief Description Set the request data source using an url

 @param[in] iRequest the request you are working on
 @param[in] iURL the url where the handler will find the data

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_set_data_url (AlpExgRequest iRequest, char *iURL);

/*! @brief Set the request data type

 @param[in] iRequest the request you are working on
 @param[in] iMIMEType the MIME type of the data

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_set_data_type (AlpExgRequest iRequest, char *iMIMEType);

/*! @brief Create/Set a request integer parameter

 The parameter is replaced if it was already existing

 @param[in] iRequest the request you are working on
 @param[in] iParamTag the parameter name
 @param[in] iParamValue the parameter value

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_set_int_parameter (AlpExgRequest iRequest, char *iParamTag, int iParamValue);

/*! @brief Create/Set a request blob parameter

 The parameter is replaced if it was already existing

 @param[in] iRequest the request you are working on
 @param[in] iParamTag the parameter name
 @param[in] iBlob a pointer to the binary blob
 @param[in] iBlobSize size of the binary blob buffer

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_set_blob_parameter (AlpExgRequest iRequest, char *iParamTag, uint8_t *iBlob, size_t iBlobSize);

/*! @brief Create/Set a request string parameter

 The parameter is replaced if it was already existing

 @param[in] iRequest the request you are working on
 @param[in] iParamTag the parameter name
 @param[in] iParamValue the parameter value

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_set_string_parameter (AlpExgRequest iRequest, char *iParamTag, char *iParamValue);

//! @}

/*! @name Transport parameters
 *
 *  @{
 */
 
/*! @brief Show UI to select a transport to use to reach the request destination

 This function will popup a transport selection dialog and let the user choose one. 
 Depending on the transport, the user may also be invited to select a destination (this 
 is transport dependant UI. For TCP, it will be a simple text field to enter a name or IP address).

 Once the user has selected the transport and destination address, the right parameters 
 and values will have been added to the request. An application can also use this function on 
 an empty request just for the purpose to subsequently save the transport and destination 
 parameters and create a 'favorite' destination.

 @param[in] iRequest the request you are working on
 @param[in] iPrompt the prompt to display on top of the transport list (e.g. "where do you want to lookup Eric's email?"). If NULL, a default prompt will be shown.

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_transport_select (AlpExgRequest iRequest, char *iPrompt);

/*! @brief Retrieve all the transport parameters at once.

 This function lets you retrieve the transport params string from the current request. This can be useful if you want to save this string somewhere in order
 to be able to use it unchanged in subsequent requests.

 @param[in] iRequest the request you are working on
 @param[out] oParamsValue pointer to a buffer that will receive the parameters string
 @param[in] iParamsValueBufLen size of the buffer

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_transport_get_params (AlpExgRequest iRequest, char *oParamsValue, size_t iParamsValueBufLen);

/*! @brief Set all the transport parameters at once.

 This function can be used to specify the transport to use and destination device where the request shall be executed. You can either either construct
 the string yourself if you know the syntax, or use a saved string that you previously built using alp_exg_transport_select(). For the built in transports,
 the syntax is the following:
 * local : "local;"
 * internet : "inet;destaddr:address_name_or_ip;" where address_name_or_ip is either a name that can be resolved through DNS or a decimal-dotted IP address
 * irda : "irda;"
 * bluetooth : TBD
 
 If no transport parameters are set, the request will be executed locally.

 @param[in] iRequest the request you are working on
 @param[in] iParamsValue a string containing all the parameters

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_transport_set_params (AlpExgRequest iRequest, char *iParamsValue);

//! @}

/*! @name Executing a request
 *
 *  @{
 */
 /*! @brief Hand the request to the system and execute it.

 When the request if formed, go execute it. This is a synchronous function.
 No UI is displayed. If the client wants to execute this function asynchronously (e.g. 
 phone app first displays incoming phone number, and then handle the phone call while the 
 request is being executed), it must call it from another thread
 Upon return of this function, you can get returned parameters from the request, and also 
 retrieve a file descriptor to returned data if any.
 
 @param[in] iRequest the request you are working on

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
	#ALP_EXGMGR_ERR_USER_CANCEL_ERROR - Execution was canceled by user
*/
alp_status_t alp_exg_request_execute (AlpExgRequest iRequest);

/*! @name Execution completion callback
 *
 *  @{
 */
 /*! @brief Execution completion callback

 This function is called back when an asynchronous request completes execution. You get the request back and you
 are responsible to delete it when done with it. This function is called in the context of your main event loop
 so you don't need to do anything to synchronize with it.
 
 @param[in] iRequest the request that has completed execution
 @param[in] iCompletionErr this is the execution result.

*/
typedef void (*alp_exg_execute_complete_func) (AlpExgRequest iRequest, alp_status_t iExecuteCompleteErr);

/*! @name Executing a request asynchronously
 *
 *  @{
 */
 /*! @brief Hand the request to the system and execute it asynchronously

 This is the asynchronous request execution function. See alp_exg_request_execute for general information.
 To use this function, you need to provide a main GMainLoop running in the current context. The exchange mgr
 will use it to handle its own events and callback your completion routine when execution has finished.
 
 @param[in] iRequest the request you are working on
 @param[in] iExecuteCompletion this function is called when the request is complete.

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_execute_async (AlpExgRequest iRequest, alp_exg_execute_complete_func iExecuteCompleteFunc);

//! @}

/*! @name Handle generic Mode
 *
 *  @{
 */
 
/*! @brief Indicate this request is to be executed in generic mode

 The generic mode is a very simplified mode. When using this mode, the client can only specify a data file descriptor and nothing else.
 The handler must also have specified that it accepts generic requests. When presented such a request, the handler will perform
 a default action.

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
	#ALP_EXG_MGR_ERR_HANDLER_NOT_FOUND_ERROR - there is no handler registered for this verb/data type<br>
	#ALP_EXG_MGR_ERR_HANDLER_DID_NOT_CONSUME_ERROR - None of the registered handlers consumed the request<br>
*/
alp_status_t alp_exg_request_set_generic_mode (AlpExgRequest iRequest);

/*! @brief ask whether the request is in generic mode or not.

 This function is used by the handler. When a handler has registered itself by indicating it supports generic mode, it must check
 each request it receives to determine if it must perform a full treatment or just the generic action.

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_get_generic_mode (AlpExgRequest iRequest, int *oGenericMode);

//! @}

/*! @name Progress dialog
 *
 *  @{
 */
 
/*! @brief Activate Progress Dialog

 Call this function to activate a modal progress dialog that will appear when applicable.
 Currently, this is only for sending over OBEX using Bluetooth or Irda.

 @param[in] iRequest the request you are working on

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_activate_progress (AlpExgRequest iRequest);

//! @}


/*! @name Extracting all the info from a Request
 *
 *  @{
 */
 
/*! @brief Get the verb associated to the request

 This function will mostly be used from the application main loop to dispatch execution to the right event handler.

 @param[in] iRequest the request you are working on
 @param[out] oVerb pointer to a buffer that will receive the verb
 @param[in] iVerbBufLen the size of the buffer

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_get_verb (AlpExgRequest iRequest, char *oVerb, size_t iVerbBufLen);

/*! @brief Get an integer parameter from the request

 @param[in] iRequest the request you are working on
 @param[in] iParamTag the name of the parameter you are looking for
 @param[out] oParamValue a pointer to an integer variable that will hold the value

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_get_int_parameter (AlpExgRequest iRequest, char *iParamTag, int *oParamValue);

/*! @brief Get a string parameter from the request

 @param[in] iRequest the request you are working on
 @param[in] iParamTag the name of the parameter you are looking for
 @param[out] oParamValue a pointer to a buffer  that will hold the value text
 @param[in] iParamValueBufLen the size of the buffer
 
 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_get_string_parameter (AlpExgRequest iRequest, char *iParamTag, char *oParamValue, size_t iParamValueBufLen);

/*! @brief Get a string parameter from the request
 
 Lets you get a pointer to the string value directly in the request storage. It is much easier and cheaper to use
 this function than to use alp_exg_request_get_string_parameter. The retrieved pointer will be valid until you use
 one of the "set" function on the same request, or you delete the request.

 @param[in] iRequest the request you are working on
 @param[in] iParamTag the name of the parameter you are looking for
 @param[out] oParamValue a pointer to a string pointer that will hold the parameter value in the request buffer
 
 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_get_string_parameter_const (AlpExgRequest iRequest, char *iParamTag, const char **oParamValue);

/*! @brief Get a binary blob parameter from the request
 
 This function will allocate (malloc) a chunk of memory and fill it with the binary data that was passed in this parameter.
 The caller is responsible to free the chunk.

 @param[in] iRequest the request you are working on
 @param[in] iParamTag the name of the parameter you are looking for
 @param[out] oBlob a pointer to an uint8_t* that will be allocated by this function
 @param[out] oBlobSize a pointer to a size_t that will contain the allocated size.
 
 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_get_blob_parameter (AlpExgRequest iRequest, char *iParamTag, uint8_t **oBlob, size_t *oBlobSize);

/*! @brief Get the data type of the data

 This function will mostly be used from the handler to dispatch execution to the right event handler.

 @param[in] iRequest the request you are working on
 @param[out] oMIMEType pointer to a buffer that will receive the MIME Type
 @param[in] iMIMETypeBufLen the size of the buffer

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_get_data_type (AlpExgRequest iRequest, char *oMIMEType, size_t iMIMETypeBufLen);

/*! @brief Get the data source file descriptor

 This function should be called from the action handler or from the request initiator after it has been executed.

 @param[in] iRequest the request you are working on
 @param[out] oFd a pointer to a variable that will receive the file descriptor to read the data from.
			 Note that the caller takes ownership of the file descriptor and he must therefore close it when done with it.

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_get_data_fd (AlpExgRequest iRequest, int *oFd);

/*! @brief Get the data source url

 @param[in] iRequest the request you are working on
 @param[out] oUrl a pointer to the buffer that will receive the url
 @param[in] iUrlBufLen the size of the buffer

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_get_data_url (AlpExgRequest iRequest, char *oUrl, size_t iUrlBufLen);

//! @}

/*! @name Receiving a Request in a Handler
 *
 *  @{
 */
 
/*! @brief Receive the request for which we have declared to be the handler.

 This function must be called by the handler application in response to the invocation of its ExchangeHandler() function.

 @param[out] oRequest pointer to a variable that will hold the request you have received
 @param[in] iRequestId a unique id passed from the exchange daemon to the application handler

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_receive (AlpExgRequest *oRequest, AlpExgRequestId iRequestId);

/*! @brief Tell the exchange manager the handler has completed execution

 This function must be called by the handler application when it is finished with execution of the requested operation.

 @param[in] iRequest the request you are working on
 @param[in] iResult the result of the handler execution. This will be passed back to the request initiator

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_complete (AlpExgRequest iRequest, alp_status_t iResult);

//! @}

/*! @name Cancel execution
 *
 *  @{
 */

 /*! @brief Cancel request while execution in progress.

 This function lets you cancel a request while it is executing.
 When cancel completes, the matching alp_exg_request_execute or execute completion function will
 return ALP_EXGMGR_ERR_USER_CANCEL_ERROR.
 
 @param[in] iRequest a request you have created or acquired in the process where you call this function

 @return
	#ALP_EXGMGR_ERR_NO_ERROR - success<br>
*/
alp_status_t alp_exg_request_cancel (AlpExgRequest iRequest);

//! @}

//! @} // endgroup ALP_EXCHANGEMGR

#ifdef __cplusplus
 }
#endif

#endif /*EXGMGR_H_*/
