/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <hiker/fail.h>
 *
 * @brief Header file for asserts equivalent.
 *
 ******************************************************************************/

#ifndef ALP_FAIL_H_
#define ALP_FAIL_H_

#include <stdarg.h>
#include <hiker/config.h>

#ifdef __cplusplus
extern "C" {
#endif

// ---------------------------------------------------------------------------
// I can't use ASSERT because the current meaning of ErrFatalDisplay is the
// exact opposite of assert: it fails when the condition is true.
// I want to keep the same logic to facilitate porting of existing code.

// ---------------------------------------------------------------------------
void alp_fail_raise(const char* file, int line, const char* cond, const char* fmt, ...);

#if defined(WIN32)

// Visual C compiler does not support macros with variable arguments.
// This handy feature is unfortunately not ANSI C++ either


#else

// ---------------------------------------------------------------------------
#if ALP_BUILD == ALP_BUILD_DEBUG
#	define ALP_FAIL_FATAL_IF(cond, msg, ...)	\
		do { if (cond) alp_fail_raise(__FILE__, __LINE__, #cond, msg, ## __VA_ARGS__); } while (0)
#	define ALP_FAIL_NON_FATAL_IF(cond, msg, ...) \
		do { if (cond) alp_fail_raise(__FILE__, __LINE__, #cond, msg, ## __VA_ARGS__); } while (0)
#else
#	define ALP_FAIL_FATAL_IF(cond, msg, ...)	\
		do { if (cond) alp_fail_raise(__FILE__, __LINE__, #cond, msg, ## __VA_ARGS__); } while (0)
#	define ALP_FAIL_NON_FATAL_IF(cond, msg, ...)
#endif
#endif

/* ------------------------------------------------------------------- */
#ifdef __cplusplus
}
#endif

#endif	// ALP_FAIL_H_
