/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file global_settings.h
 *
 * @brief File public header for the ALP Global Settings service APIs.
 *
 *****************************************************************************/

#ifndef ALP_GLOBAL_SETTINGS_H_
#define ALP_GLOBAL_SETTINGS_H_ 1

#ifdef __cplusplus
extern "C" {
#endif

/*! 
 * @addtogroup ALP_GLOBAL_SETTINGS Global Settings
 * @ingroup ALP_SYSTEM 
 * @{
*/

#include <glib.h>

#include <hiker/config.h>
#include <hiker/sysclass.h>
#include <hiker/types.h>
#include <sys/types.h>		/* for mode_t */

#define ALP_GLOBAL_SETTINGS_RPC
/* functions labelled with ALP_GLOBAL_SETTINGS_RPC invokes remote operations to the server */

#define ALP_GLOBAL_SETTINGS_VERSION 1

#define GLOBAL_SETTINGS_APP_ID  "com.access.sys.global_settings"
#define GLOBAL_SETTINGS_CHANNEL_NAME  "global settings"
#define GLOBAL_SETTINGS_TIMEOUT 500000


/* for key change notification via the Notification Manager */

#define ALP_NOTIFY_EVENT_GLOBAL_SETTINGS_KEY_CHANGE  "/alp/globalsettings/keychange"

/* status constants */

#define ALP_STATUS_GLOBAL_SETTINGS_ERROR ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x00010000)
#define ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x00020000)
#define ALP_STATUS_GLOBAL_SETTINGS_VALUE_NOT_FOUND ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x00030000)
#define ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_AT_END ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x00040000)
#define ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_INVALID ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x00050000)
#define ALP_STATUS_GLOBAL_SETTINGS_VALUE_NOT_DIR ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x00060000)
#define ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x00070000)
#define ALP_STATUS_GLOBAL_SETTINGS_ENTRY_INVALID ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x00080000)
#define ALP_STATUS_GLOBAL_SETTINGS_VALUE_INVALID ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x00090000)
#define ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x000a0000)
#define ALP_STATUS_GLOBAL_SETTINGS_BUFFER_TOO_SMALL ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x000b0000)
#define ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x000c0000)
#define ALP_STATUS_GLOBAL_SETTINGS_KEY_EXISTS ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x000d0000)
#define ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x000e0000)


//
//   Value types
//

/*!
@brief enums for types of preference values
*/
typedef enum { 
    ALP_GLOBAL_SETTINGS_TYPE_INVALID = 0,
    ALP_GLOBAL_SETTINGS_TYPE_STRING,
    ALP_GLOBAL_SETTINGS_TYPE_INT,
    ALP_GLOBAL_SETTINGS_TYPE_FLOAT,
    ALP_GLOBAL_SETTINGS_TYPE_BOOL,
    ALP_GLOBAL_SETTINGS_TYPE_LIST,
    ALP_GLOBAL_SETTINGS_TYPE_DIR,
    ALP_GLOBAL_SETTINGS_TYPE_BLOB,
}  AlpGlobalSettingsValueType; 
       

// 
// Opaque typedef's
//

/*!
@brief Opaque type for representing a preference service instance
*/
typedef struct _AlpGlobalSettingsContext AlpGlobalSettingsContext;


/*!
@brief type for representing a preference value; must be initialized to 0 before use
*/
typedef struct _AlpGlobalSettingsValue AlpGlobalSettingsValue ; 
struct _AlpGlobalSettingsValue 
{
   AlpGlobalSettingsValueType type;
       int int_value;
       double float_value;
       gboolean bool_value;
       GList *list_value;
       GString *string_value;
       GByteArray blob_value;
  
} ;

#define   GLOBAL_SETTINGS_VALUE_INITIALIZER { ALP_GLOBAL_SETTINGS_TYPE_INVALID, 0, 0, 0, 0, 0, { 0, 0 } } 

/*!
@brief a simple key-value preference entry; must be initialized to 0 before use
*/
typedef struct _AlpGlobalSettingsEntry  AlpGlobalSettingsEntry ; 
struct _AlpGlobalSettingsEntry
{
   GString *key;
   AlpGlobalSettingsValue value; 
} ;


/*!
@brief the iterator for traversing a dir tree key value
*/
typedef struct _AlpGlobalSettingsEntryIterator  AlpGlobalSettingsEntryIterator  ; 
  
  

/*!

@brief allocate a new entry struct
@return the pointer to a newly allocated entry object or NULL on failure
*/
AlpGlobalSettingsEntry*alp_global_settings_entry_alloc(void);

/*!

@brief decllocates a previously allocated entry object
@param[in] e the pointer to the entry object to be freed

*/
void alp_global_settings_entry_free(AlpGlobalSettingsEntry *e);


/*!

@brief allocate a new value struct
@return the pointer to a newly allocated value object or NULL on failure
*/
AlpGlobalSettingsValue *alp_global_settings_value_alloc(void);


/*!

@brief decllocates a previously allocated value object
@param[in] e the pointer to the value object to be freed

*/
void alp_global_settings_value_free(AlpGlobalSettingsValue *e);



/*!

@brief decllocates resources used by a value object but not the value object itself
@param[in] e the pointer to the value object to be cleaned

*/
void alp_global_settings_value_clean(AlpGlobalSettingsValue *e);



/*!

@brief decllocates a list of value obbjects; the list is used to represent a list value
@param[in] l the pointer to the list of value objects to be freed

*/
void alp_global_settings_value_list_free(GList *l);

/*!
@brief initialize the Global Settings system


@return ALP_STATUS_OK on success
@return ALP_STATUS_ERROR on failure
*/

alp_status_t 
alp_global_settings_init(void);



/*!
@brief obtain the preference service


@param[out] context on successful return, a reference to the preference service, which is created by the
Global Setting API and will be freed upon alp_global_settings_close() 
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_ERROR on failure
*/
alp_status_t 
alp_global_settings_open(AlpGlobalSettingsContext **context);

/*!
@brief frees the reference to the preference service 

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_ERROR on failure
*/
alp_status_t 
alp_global_settings_close(AlpGlobalSettingsContext *context);

/*!
@brief begins a transaction in which all changes will occur in one shot automatically

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_begin_transaction(AlpGlobalSettingsContext *context);

/*!
@brief commits a transaction which has accumulated a serious of previous write commands

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/

ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_complete_transaction(AlpGlobalSettingsContext *context);


/*!
@brief cancels a transaction which has accumulated a serious of previous write commands

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/

ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_cancel_transaction(AlpGlobalSettingsContext *context);


/*!
@brief set the current "directory" in the key space
       then following key paths which are not absolute will be treated as relative to the "directory"

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] cwd the new "current directory"

@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/
alp_status_t 
alp_global_settings_set_cwd(AlpGlobalSettingsContext *context, const char *cwd);


/*!
@brief get the current "directory" in the key space
       then following key paths which are not absolute are treated as relative to the "directory"

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[out] buf the buffer to contain the "current directory" on output
@param[in] buf_size the size of the buffer pointed to by the buffer parameter

@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_BUFFER_TOO_SMALL if provided buffer is too small
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/
alp_status_t 
alp_global_settings_get_cwd(AlpGlobalSettingsContext *context, char *buf, size_t buf_size);

/*!
@brief set the  permission used when creating new keys 

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] mode the new access permission 
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/

ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_set_default_mode(AlpGlobalSettingsContext *context, mode_t mode);


/*!
@brief get the  permission used when creating new keys

@param[in] context referencing the current preference service
@param[out] mode to hold the value of the mode
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_get_default_mode(AlpGlobalSettingsContext *context, mode_t *mode);


/*!
@brief set the permission on an existing key

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key whose access permission is to be set
@param[in] mode the new access permission
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist 
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/

ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_set_mode(AlpGlobalSettingsContext *context, const char *key, mode_t mode);


/*!
@brief get the permission on an existing key

@param[in] context referencing the current preference service
@param[in] key  the key string
@param[out] mode to hold the permission bits of the key
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_get_mode(AlpGlobalSettingsContext *context, const char *key, mode_t *mode);

/*!
@brief set the user id on an existing key

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key whose id is to be set
@param[in] id the new user id
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist 
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/

ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_set_uid(AlpGlobalSettingsContext *context, const char *key, uid_t id);


/*!
@brief get the current user id of a key

@param[in] context referencing the current preference service
@param[in] key  the key string
@param[out] id  to hold the current uid
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_get_uid(AlpGlobalSettingsContext *context, const char *key, uid_t *id);


/*!
@brief set the group id of an existing key

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key whose gid is to be set
@param[in] id the new gid
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist 
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/

ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_set_gid(AlpGlobalSettingsContext *context, const char *key, gid_t id);


/*!
@brief get the group id of an existing key

@param[in] context referencing the current preference service
@param[in] key  the key string
@param[out] id to hold the group id of the key
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_get_gid(AlpGlobalSettingsContext *context, const char *key, gid_t *id);





/*!
@brief set the value of a key; a new key is created if the key does not exist and the old value is overwritten if the
key already exists

@param[in] context referencing the current preference service
@param[in] key  the key string
@param[in] value the new value for the key to be set
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
@return ALP_STATUS_GLOBAL_SETTINGS_ERROR on failure


*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_set( AlpGlobalSettingsContext *context, 
                     const char *key, 
		     const AlpGlobalSettingsValue *value);

/*!
@brief get the current value of a key

@param[in] context referencing the current preference service
@param[in] key  the key string
@param[out] value the value for the key if the key exists
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_get( AlpGlobalSettingsContext *context, 
                     const char *key, 
                     AlpGlobalSettingsValue *value);


/*!
@brief deletes the current key setting, and reverts the key value to default if there is one

@param[in] context referencing the current preference service
@param[in] key  the key string

@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_delete( AlpGlobalSettingsContext *context, 
                           const char *key);  // revert to default if there is one


/*!
@brief deletes the all keys (subtree) below a given key as well as that key

@param[in] context referencing the current preference service
@param[in] key  the key string

@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_delete_tree( AlpGlobalSettingsContext *context, 
                           const char *key);  


/*!
@brief deletes the all keys (subtree) below a given key as well as that key, except keeping default keys intact

@param[in] context referencing the current preference service
@param[in] key  the key string

@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_delete_tree_keeping_defaults( AlpGlobalSettingsContext *context, 
                           const char *key);  



/*!
@brief get the type of the value of a key

@param[in] context referencing the current preference service
@param[in] key  the key string
@param[out] type the type of the value for the key if the key exists
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_get_type( AlpGlobalSettingsContext *context, 
                     const char *key, 
                     AlpGlobalSettingsValueType *type);


/*!
@brief get the type of the value of a key

@param[in] context referencing the current preference service
@param[in] key  the key string
@param[out] size the size of the value for the key if the key exists
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_get_value_size( AlpGlobalSettingsContext *context, 
                     const char *key, 
                     size_t *size);



/*!
@brief set the key change notification to be on or off

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] on_off turn the notification on (non-zero) or off (0)
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/

ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_set_notification_on_off(AlpGlobalSettingsContext *context, int on_off);


/*!
@brief begin the traversal of the value of a key which is a list of subkeys

@param[in] context referencing the current preference service
@param[in] dir_path the key whose value is a list of keys (a directory) to be traversed
@param[out] iterator an "iterator" object created by this function for later traversal operations; to be freed when
findkeys_end() is called
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if dir_path key not found
@return ALP_STATUS_GLOBAL_SETTINGS_VALUE_NOT_DIR if the dir_path key is not a directory
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t 
alp_global_settings_findkeys_start( AlpGlobalSettingsContext *context, 
                                 const char *dir_path,
                                 AlpGlobalSettingsEntryIterator **iterator);



/*!
@brief advances the iterator to the next key in the list

@param[in,out] iterator the iterator which represents a directory traversal
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if iterator is zero
@return ALP_STATUS_GLOBAL_SETTINGS_ITERATOR_AT_END if the iterator is already at the end of the list
@return ALP_STATUS_GLOBAL_SETTINGS_ITERATOR_INVALID if the iterator is invalid (not properly initialized with
alp_global_settings_findkeys_start)

*/
alp_status_t 
alp_global_settings_findkeys_next(AlpGlobalSettingsEntryIterator *iterator);

/*!
@brief completes the traversal represented by the iterator; the associated resources are freed

@param[in] iterator the iterator which represents a directory traversal, must have been created by a
previous call to alp_global_settings_findkeys_start()
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if iterator is zero
@return ALP_STATUS_GLOBAL_SETTINGS_ITERATOR_INVALID if the iterator is invalid (not properly initialized with alp_global_settings_findkeys_start)

*/
alp_status_t 
alp_global_settings_findkeys_end(AlpGlobalSettingsEntryIterator *iterator);


/*!
@brief returns the key of the key pointed by an iterator separately

@param[in] iterator the iterator
@param[in] key the string buffer to receive the key string
@param[out] key_size the size of the buffer key
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if iterator is zero
@return ALP_STATUS_GLOBAL_SETTINGS_ITERATOR_INVALID if the iterator is invalid (not properly initialized with alp_global_settings_findkeys_start)
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/
alp_status_t
alp_global_settings_findkeys_get_key(AlpGlobalSettingsEntryIterator  *iterator, 
                               char *key, 
			       unsigned int key_size);


/*!
@brief returns the key and the value of the key of an value entry

@param[in] entry the preference entry
@param[in] key the string buffer to receive the key string; can be NULL in that case the key is not returned
@param[in] key_size the size of the buffer key
@param[out] value the value of the key; can be NULL in that case the value is not returned 
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ENTRY_INVALID if the iterator is invalid (not properly initialized with alp_global_settings_findkeys_start)
@return ALP_STATUS_GLOBAL_SETTINGS_BUFFER_TOO_SMALL if the supplied buffer is too small

*/
alp_status_t
alp_global_settings_entry_get_key_value(const AlpGlobalSettingsEntry *entry, 
                               char *key, 
            			       unsigned int key_size, 
            			       AlpGlobalSettingsValue *value);


/*!
@brief sets the key and the value of the key of an value entry; old resources are freed, if exist

@param[in] entry the preference entry
@param[in] key the key string to be set
@param[in] value the value of the key to be set 
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ENTRY_INVALID if an entry cannot be successfully set (for example, if the key or the
value parameters are invalid


*/
alp_status_t
alp_global_settings_entry_set_key_value(AlpGlobalSettingsEntry *entry, 
                               const char *key, 
			                   const AlpGlobalSettingsValue *value);



/*!
@brief set a string preference value; the old value, if any,  is freed

@param[in,out] value the value to be set
@param[in] string the string to be set
@return ALP_STATUS_OK on success

*/
alp_status_t
alp_global_settings_value_set_string(AlpGlobalSettingsValue *value, 
                             const char *string);



/*!
@brief create an integer preference value; the old value, if any,  is freed

@param[in,out] value the value to be set
@param[in] i to integer to be set
@return ALP_STATUS_OK on success

*/
alp_status_t
alp_global_settings_value_set_int(AlpGlobalSettingsValue *value, 
                             int i);



/*!
@brief create a float preference value; the old value, if any,  is freed

@param[in,out] value the value to be set
@param[in] f the float to be set
@return ALP_STATUS_OK on success

*/
alp_status_t
alp_global_settings_value_set_float(AlpGlobalSettingsValue *value, 
                             double f);



/*!
@brief create a bool preference value; the old value, if any,  is freed

@param[in,out] value the value to be set
@param[in] b the boolean to be set
@return ALP_STATUS_OK on success

*/
alp_status_t
alp_global_settings_value_set_bool(AlpGlobalSettingsValue *value, 
                             gboolean b);



/*!
@brief set a list (dir) preference value; the old value, if any,  is freed

@param[in,out] value the value to be set
@param[in] list the directory or list of to be set; the list should contain AlpGlobalSettingsEntry_t's
@return ALP_STATUS_OK on success

*/
alp_status_t
alp_global_settings_value_set_list(AlpGlobalSettingsValue *value, 
                             const GList *list);




/*!
@brief get the string preference value contained in an value object

@param[in] value the value object
@param[out] string the string buffer to receive the string value
@param[in] string_size the size of the string buffer
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE  if the value is not a string
@return ALP_STATUS_GLOBAL_SETTINGS_VALUE_INVALID  if this value object contains no valid value data

*/
alp_status_t
alp_global_settings_value_get_string(AlpGlobalSettingsValue *value, 
                             char *string, 
                    		 unsigned int string_size);



/*!
@brief get the integer preference value contained in an value object

@param[in] value the value object
@param[out] i the integer to receive the integral value
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE  if the value is not an integer
@return ALP_STATUS_GLOBAL_SETTINGS_VALUE_INVALID  if this value object contains no valid value data

*/
alp_status_t
alp_global_settings_value_get_int(AlpGlobalSettingsValue *value, 
                             int *i);



/*!
@brief get the float preference value contained in an value object

@param[in] value the value object
@param[out] f the float to receive the integral value
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE  if the value is not a float
@return ALP_STATUS_GLOBAL_SETTINGS_VALUE_INVALID  if this value object contains no valid value data

*/
alp_status_t
alp_global_settings_value_get_float(AlpGlobalSettingsValue *value, 
                             double *f);



/*!
@brief get the boolean preference value contained in an value object

@param[in] value the value object
@param[out] b the integer to receive the integral value
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE  if the value is not an boolean
@return ALP_STATUS_GLOBAL_SETTINGS_VALUE_INVALID  if this value object contains no valid value data

*/
alp_status_t
alp_global_settings_value_get_bool(AlpGlobalSettingsValue *value, 
                             gboolean *b);



/*!
@brief get the dir (list) preference value contained in an value object

@param[in] value the value object
@param[out] list the list object to receive the list data (a list of AlpGlobalSettingsEntry_t).  A copy of the current list will be allocated
and created.  This list needs to be freed by the caller
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE  if the value is not a dir/list
@return ALP_STATUS_GLOBAL_SETTINGS_VALUE_INVALID  if this value object contains no valid value data

*/
alp_status_t
alp_global_settings_value_get_list(AlpGlobalSettingsValue *value, 
                             GList **list);



/*!
@brief get a string key-value pair

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string
@param[out] value the value object to receive the value
@param[in] value_size the size of the value buffer
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if a parameter value is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist 
@return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE  if the value is not a string
@return ALP_STATUS_GLOBAL_SETTINGS_BUFFER_TOO_SMALL if value buffer is too short
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
alp_status_t
alp_global_settings_get_key_string_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   char *value, 
                                   unsigned int value_size);
                             

/*!
@brief set a string key-value pair

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string
@param[in] value  the string value
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ERROR if failure
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
alp_status_t
alp_global_settings_set_key_string_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   const char *value);
                             


/*!
@brief get a boolean key-value pair

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string
@param[out] value the value variable to receive the value
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if a parameter value is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist
@return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE  if the value is not a bool
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/
alp_status_t
alp_global_settings_get_key_bool_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   gboolean *value);

/*!
@brief set a boolean key-value pair

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string
@param[in] value  the boolean  value
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ERROR if failure
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
alp_status_t
alp_global_settings_set_key_bool_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   gboolean value);
                             


/*!
@brief get an integer key-value pair

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string
@param[out] value the value variable to receive the value
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if a parameter value is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist 
@return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE  if the value is not an integer
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
alp_status_t
alp_global_settings_get_key_int_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   int *value);

/*!
@brief set an integer key-value pair

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string
@param[in] value  the integer value
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ERROR if failure
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
alp_status_t
alp_global_settings_set_key_int_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   int value);
                             


/*!
@brief get a float  key-value pair

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string
@param[out] value the value variable to receive the float value
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if a parameter value is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist 
@return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE  if the value is not a float
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
alp_status_t
alp_global_settings_get_key_float_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   double *value);



/*!
@brief set a float
@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string
@param[in] value  the float value
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ERROR if failure
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
alp_status_t
alp_global_settings_set_key_float_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   double value);




/*!
@brief get a Blob key-value pair

@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string
@param[out] value the pointer to the buffer to receive the Blob value
@param[in] buffer_size the size of the buffer
@param[out] value_size the actual size of the Blob returned to the caller
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if a parameter value is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if key does not exist 
@return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE  if the value is not a Blob
@return ALP_STATUS_GLOBAL_SETTINGS_BUFFER_TOO_SMALL 
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
alp_status_t
alp_global_settings_get_key_blob_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   char  *value,
                                   int buffer_size,
                                   int * value_size);

/*!
@brief set a float
@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string
@param[in] value  the address of the Blob
@param[in] value_size the size of the Blob
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ERROR if failure
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
alp_status_t
alp_global_settings_set_key_blob_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   const char *value,
                                   int value_size);


/*!
@brief create a directory: this call fails if a key with the same name already exists 
@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] key the key string


@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ERROR if failure
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_EXISTS if key exists already
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
alp_status_t
alp_global_settings_set_key_directory(AlpGlobalSettingsContext *context, 
                                   const char *key);

/*!
@brief to install a set of preferences from an XML file en masse
@param[in] context a reference to the preference service returned previously by alp_global_settings_open()
@param[in] file the XML file name


@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ERROR if failure
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation

*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t
alp_global_settings_import_from_file(AlpGlobalSettingsContext *context, 
                                   const char *file);

/*!
@brief get back a list of keys according to a glob pattern

@param[in] context referencing the current preference service
@param[in] pattern  the glob pattern for the keys to be retrieved
@param[out] iterator an "iterator" object created by this function for later traversal operations; to be freed when
findkeys_end() is called
@return ALP_STATUS_OK on success
@return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID if context is zero
@return ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND if no key fitting the pattern not found
@return ALP_STATUS_GLOBAL_SETTINGS_VALUE_NOT_DIR if the dir_path key is not a directory
@return ALP_STATUS_GLOBAL_SETTINGS_PERMISSION_DENIED if no permission for operation
*/
ALP_GLOBAL_SETTINGS_RPC
alp_status_t
alp_global_settings_list_keys( AlpGlobalSettingsContext *context, const gchar *pattern, AlpGlobalSettingsEntryIterator **iterator);


/*!
@}
*/

#ifdef __cplusplus
}       // extern "C"
#endif

#endif  // #ifndef ALP_GLOBAL_SETTINGS_H_
