/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <hiker/prv/IpcMessage.h>
 *   
 * @brief Private header file for the ALP IPC Message APIs.
 *
 ******************************************************************************/


#ifndef ALP_IPC_MESSAGE_H
#define ALP_IPC_MESSAGE_H

// Includes ------------------------------------------------------------------
#include <stdint.h>
#include <string>

#include <hiker/ipc.h>
#include <hiker/types.h>


// Typedefs ------------------------------------------------------------------
namespace alp {

class string;

namespace ipc {

struct Array
{
	unsigned char*	array;
	uint32_t		len;

	Array() : array(NULL), len(0) {;}
	Array(unsigned char* a, uint32_t l)
		:	array(a), len(l)
	{;}
};

class Message
{
	public:
		enum PackMode { none, send, receive };

		Message();
		Message(AlpMessage* msg);
		Message(int what);
		Message(const Message& rhs);
		~Message();

		Message&	operator=(const Message&);

		inline alp_status_t	Status() const	{ return fStatus; }
		inline PackMode		Mode() const	{ return fMode; }
		inline int			What() const	{ return fWhat; }

		alp_status_t	AddInt32(int32_t i);
		alp_status_t	AddUInt32(uint32_t i);
		alp_status_t	AddBool(bool b);
		alp_status_t	AddFloat(float f);
		alp_status_t	AddString(const char* s);
		alp_status_t	AddString(const std::string& s);
		alp_status_t	AddString(const alp::string& s);
		alp_status_t	AddByteArray(const unsigned char* a, uint32_t len);

		alp_status_t	GetInt32(int32_t& i);
		alp_status_t	GetUInt32(uint32_t& i);
		alp_status_t	GetBool(bool& b);
		alp_status_t	GetFloat(float& f);
		alp_status_t	GetString(char*& s);
		alp_status_t	GetString(std::string& s);
		alp_status_t	GetString(alp::string& s);
		alp_status_t	GetByteArray(unsigned char*& a, uint32_t& len);

	private:
		AlpMessage*		GetMessage();
		alp_status_t	Send(AlpConnection* channel);

		class MessageSharedPtr;

		MessageSharedPtr*	fMessage;
		PackMode			fMode;
		alp_status_t		fStatus;
		int					fWhat;

	friend class Server;
	friend class Client;
};


// Prototypes ----------------------------------------------------------------
Message& operator<<(Message& ipc, const int32_t i);
Message& operator<<(Message& ipc, const uint32_t i);
Message& operator<<(Message& ipc, const bool b);
Message& operator<<(Message& ipc, const float f);
Message& operator<<(Message& ipc, const char* s);
Message& operator<<(Message& ipc, const std::string& s);
Message& operator<<(Message& ipc, const Array& a);

Message& operator>>(Message& ipc, int32_t& i);
Message& operator>>(Message& ipc, uint32_t& i);
Message& operator>>(Message& ipc, bool& b);
Message& operator>>(Message& ipc, float& f);
Message& operator>>(Message& ipc, char*& s);
Message& operator>>(Message& ipc, std::string& s);
Message& operator>>(Message& ipc, Array& a);

}	// namespace ipc

}	// namespace alp



#endif	// ALP_IPC_MESSAGE_H
