/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <hiker/prv/IpcServer.h
 * 
 * @brief PRV header file for the IPC Server APIs.
 *   
 ******************************************************************************/


#ifndef ALP_IPC_SERVER_H
#define ALP_IPC_SERVER_H

// Includes ------------------------------------------------------------------
#include <string>

#include <hiker/ipc.h>
#include <hiker/types.h>


// Typedefs ------------------------------------------------------------------
namespace alp {

namespace ipc {

class Message;

class Server
{
	public:
		enum CallbackFlags
		{
			cfConnect = 0x01,
			cfDisconnect = 0x02,
			cfReceive = 0x04,
			cfAll = cfConnect | cfDisconnect | cfReceive
		};

		Server(const char* name, AlpIpcChannelAccessMode mode, int flags = cfAll);
		Server(const std::string& name, AlpIpcChannelAccessMode mode, int flags = cfAll);
		virtual ~Server();

		virtual alp_status_t Run();

		alp_status_t	Status() const;
		int				Flags() const;
		alp_status_t	Send(Message& msg);
		
		uid_t			GetConnectionUID() const;
		gid_t			GetConnectionGID() const;
		pid_t			GetConnectionPID() const;

	protected:
		virtual void OnChannelConnect(AlpConnection* connection) {;}
		virtual void OnChannelDisconnect(AlpConnection* connection) {;}
		virtual void OnMessageReceived(Message& message) {;}

	private:
		void Init(const char* name, AlpIpcChannelAccessMode mode);

		static void	ChannelConnectCB(AlpConnection* connection,
									 Server* server);
		static void ChannelDisconnectCB(AlpConnection* connection,
										Server* server);
		static void	MessageReceiveCB(AlpConnection* connection,
									 AlpMessage* message,
									 Server* server);

		AlpChannel*		fServerChannel;
		alp_status_t	fStatus;
		int				fFlags;
		AlpConnection*	fCurrentConnection;
};

}	// namespace ipc

}	// namespace alp



#endif	// ALP_IPC_SERVER_H
