/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/**
 * @file alarmmgr_prv.h
 *
 * @brief Internal to Alarm Manager implementation
 *
 *****************************************************************************/

#ifndef __ALP_PRV_ALARMMGR_PRV_H__
#define __ALP_PRV_ALARMMGR_PRV_H__

#include <glib.h>

#include <hiker/types.h>
#include <hiker/traces.h>

#include <hiker/alarmmgr.h>

#define ALP_ALARM_MGR_SERVER_CHANNEL "alp_alarm_server"

/* cmd parameter */
#define ALP_ALARM_CMD_SET    1
#define ALP_ALARM_CMD_CANCEL 2
#define ALP_ALARM_CMD_GET    3


/* debug trace */
#define ALP_ALARM_TRACE   ALP_TRACE_DEFINE(ALP_CLASS_ALARM, 0x000000)

#define ALP_ATL(str, ...) ALP_TM(TL(ALP_ALARM_TRACE, "%s(): " str, __PRETTY_FUNCTION__, ## __VA_ARGS__))
#define ALP_AT(str, ...)  ALP_TM(T(ALP_ALARM_TRACE, "%s(): " str, __PRETTY_FUNCTION__, ## __VA_ARGS__))
#define ALP_AB(str, ...)  ALP_TM(B(ALP_ALARM_TRACE, str, ## __VA_ARGS__))

#ifdef __cplusplus
extern "C" {
#endif


/*******************************************************************
 * Alarm Manager alarm table structures
 *
 *******************************************************************/

/* An entry in the alarm table */
typedef struct _AlpAlmEntryType AlpAlmEntryType;
struct _AlpAlmEntryType {
	guint16		cmd;
	guint16		size;
	guint32		ref;
	guint32 	alarmSeconds;
	gchar 		appID[ALP_ALARM_MAX_APPID_SIZE+1];
	guint16		triggered:1; //!< 1 indicates the alarm has been triggered;
	guint16		notified:1;	 //!< 1 indicates we notified the Application Server that the application's alarm went off;
};

typedef AlpAlmEntryType*		AlpAlmEntryTypePtr;

#ifdef __cplusplus
}
#endif

#endif /* __ALP_PRV_ALARMMGR_PRV_H__ */
