/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file appmgr_prv.h
 *   
 * @brief Header file for PRIVATE MAX Application Manager APIs.
 * Contains the private API for the application server and its associated client library.
 * This is really just the message packing & unpacking APIs which are shared between the server & the client library implementation.
 ******************************************************************************/

#ifndef __ALP_APP_MGR_PRV_H_
#define __ALP_APP_MGR_PRV_H_

#include <hiker/pdk/appmgr_pdk.h>

#ifdef __cplusplus
extern "C" {
#endif

/*! @addtogroup ALP_APPLICATION_MGR Application Manager
 *  @ingroup ALP_SYSTEM
 *  @{
 */

/*! @addtogroup ALP_APPLICATION_MGR_PRIVATE Private
 *  @{
 */
 
 /************************************************
	PRIVATE.  ALP internal use only.
 ************************************************/




// Message packing & unpacking.  
// These calls should always be used to pack & unpack messages to/from the app server.

/// Utility API to create a new AlpMessage containing a launch request.
AlpMessage* alp_app_prv_make_launch_msg(const char *appID, int argc,  char *const argv[], gboolean waitForPID, unsigned int securityID);

/// Utility API to unpack the contents of an AlpMessage containing a launch request.
alp_status_t alp_app_prv_unpack_launch_msg(AlpMessage* msg, int *outArgc, char*** outArgv, gboolean *waitForPID, unsigned int *outSecurityID);


/// Utility API to create a new AlpMessage containing an exit request.
AlpMessage* alp_app_prv_make_exit_msg(const char *pkgID);

/// Utility API to unpack the contents of an AlpMessage containing an exit request.
alp_status_t alp_app_prv_unpack_exit_msg(AlpMessage* msg, char** pkgID);


/// Utility API to create a new AlpMessage containing a retire request.
AlpMessage* alp_app_prv_make_retire_msg(const char *appID);

/// Utility API to unpack the contents of an AlpMessage containing an retire request.
alp_status_t alp_app_prv_unpack_retire_msg(AlpMessage* msg, char** appID);


/// Utility API to create a new AlpMessage containing a "set backgrounding" request.
AlpMessage* alp_app_prv_make_backgrounding_msg(AlpAppBackgroundingType newBG);

/// Utility API to unpack the contents of an AlpMessage containing a "set backgrounding" request.
alp_status_t alp_app_prv_unpack_backgrounding_msg(AlpMessage* msg,
								AlpAppBackgroundingType* outBG);




/// Utility API to create a new AlpMessage containing an app registration.
AlpMessage* alp_app_prv_make_register_msg(const char *pkgID, pid_t appPID, guint32 flags, const char *lpScheme);

/// Utility API to unpack the contents of an AlpMessage containing an app registration.
alp_status_t alp_app_prv_unpack_register_msg(AlpMessage* msg, char** appID, pid_t *appPID, guint32 *flags, char**lpScheme);


/// Utility API to create a new AlpMessage containing an unregistration message.
AlpMessage* alp_app_prv_make_unregister_msg(int result);

/// Utility API to unpack the contents of an AlpMessage containing an unregistration message.
alp_status_t alp_app_prv_unpack_unregister_msg(AlpMessage* msg, int *result);


/// Utility API to create a new AlpMessage containing a diagnostic message.
AlpMessage* alp_app_prv_make_diagnostic_msg(const char *appID, const char *str);

/// Utility API to unpack the contents of an AlpMessage containing a diagnostic message.
alp_status_t alp_app_prv_unpack_diagnostic_msg(AlpMessage* msg, char **pkgID, char **str);


/// Utility API to pack the contents of an AlpMessage containing a launch reply message.
AlpMessage* alp_app_prv_make_launch_reply_msg(AlpMessage* replyTo, const char *appID, alp_status_t launchErr, pid_t appPID);

/// Utility API to unpack the contents of an AlpMessage containing a launch reply message.
alp_status_t alp_app_prv_unpack_launch_reply_msg(AlpMessage* msg, char **appID, alp_status_t *launchErr, pid_t *appPID);

/// Utility API to pack the contents of an AlpMessage containing a flush message.
AlpMessage* alp_app_prv_make_flush_msg(AlpMessage *replyTo);

/// Utility API to pack the contents of an AlpMessage containing a request for the current jumplist apps.
AlpMessage* alp_app_prv_make_jump_list_request_msg(void);

/// Utility API to pack the contents of an AlpMessage containing the current jumplist apps.
AlpMessage* alp_app_prv_make_jump_list_msg(AlpMessage *replyTo, int count, char *const appIDs[], const gboolean *isRunnings);

/// Utility API to unpack the contents of an AlpMessage containing the current jumplist apps. 
alp_status_t alp_app_prv_unpack_jump_list_msg(AlpMessage* msg, 
								 int *count, char***appIDs, gboolean **isRunnings);

/// Utility API to pack the contents of an AlpMessage containing a request/response for the current primary app name.
AlpMessage* alp_app_prv_make_cur_app_msg(AlpMessage *replyTo, const char *pkgID);

/// Utility API to unpack the contents of an AlpMessage containing a request/response for the current primary app name.
alp_status_t alp_app_prv_unpack_cur_app_msg(AlpMessage* msg, char** outPkgID);

/// Utility API to pack the contents of an AlpMessage containing a request/response for the "watcher" APIs.
AlpMessage* alp_app_prv_make_watcher_msg(AlpMessage *replyTo, const char *pkgID, 
							  AlpWatcherRef watcherRef, gboolean wasRun, 
							  gboolean remove, alp_status_t result, gint blockMode);

/// Utility API to unpack the contents of an AlpMessage containing a request/response for the "watcher" APIs.
alp_status_t alp_app_prv_unpack_watcher_msg(AlpMessage* msg, char** outPkgID, 
								AlpWatcherRef *outWatcherRef, gboolean *outWasRun,
								gboolean *outRemove, alp_status_t *outResult, gint *outBlockMode);

/// Utility API to make a theme update message.
AlpMessage* alp_app_prv_make_theme_update_msg(void);

/// Utility API to make a system reset message.
AlpMessage* alp_app_prv_make_reset_msg(void);

// Message packing utility APIs.
// Not tied to any particular message type.

/// Pack an argc/argv style argument list from AlpMessage structures.
alp_status_t alp_app_prv_pack_argv(AlpMessage *msg, const char *pkgID, int argc,  char *const argv[]);

/// Unpacking an argc/argv style argument list from AlpMessage structures.
alp_status_t alp_app_prv_unpack_argv(AlpMessage *msg, int *outArgc, char*** outArgv);


/************************************************
	PRIVATE constants, ALP internal use only.  
 ************************************************/
typedef enum
{
	ALP_APP_SERVER_FLAG_NONE				= 0,
	ALP_APP_SERVER_FLAG_EXIT_CLEAN		= (1 << 0),
	ALP_APP_SERVER_FLAG_WAITING_FOR_REPLY	= (1 << 1),
	ALP_APP_SERVER_FLAG_NO_JUMP_LIST		= (1 << 2),
	ALP_APP_SERVER_FLAG_ALWAYS_JUMP_LIST	= (1 << 3)
} AlpAppServerFlags;



// Standard message IDs: 
typedef enum
{
	ALP_APP_MSG_ID_LAUNCH = 1,
	ALP_APP_MSG_ID_REGISTER,
	ALP_APP_MSG_ID_UNREGISTER,
	ALP_APP_MSG_ID_EXIT,
	ALP_APP_MSG_ID_DIAGNOSTIC,
	ALP_APP_MSG_ID_LAUNCH_REPLY,
	ALP_APP_MSG_ID_BACKGROUNDING,
	ALP_APP_MSG_ID_FLUSH,
	ALP_APP_MSG_ID_JUMP_LIST_REQ,
	ALP_APP_MSG_ID_JUMP_LIST,
	ALP_APP_MSG_ID_CUR_APP,
	ALP_APP_MSG_ID_WATCHER,
	ALP_APP_MSG_ID_RETIRE,
	ALP_APP_MSG_ID_THEME_UPDATE,
	ALP_APP_MSG_ID_RESET
} AlpAppMsgID;



// Values for the "tristate" 'block' value in watcher messages.
#define ALP_APP_WATCHER_UNBLOCK	0
#define ALP_APP_WATCHER_BLOCK		1
#define ALP_APP_WATCHER_IGNORE	2



/// The name of the communications channel used by the appserver.  For use with alp_channel_connect().
#define ALP_APP_SERVER_CHANNEL "ALP AppServer"


//! @}
//! @}

#ifdef __cplusplus
}	// extern "C"
#endif

#endif // __ALP_APP_MGR_PRV_H_


