/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/**
 * @file attnmgr_ipc.h
 *
 * @brief ALP attention manager API & definitions
 *
 *****************************************************************************/

#ifndef __ALP_ATTN_IPC_H__
#define __ALP_ATTN_IPC_H__

#include <hiker/types.h>
#include <hiker/ipc.h>
#include <hiker/attnmgr_common.h>

/**
 * @addtogroup ALP_ATTN_MGR_PRIVATE
 * @ingroup ALP_ATTN_MGR
 */
/**
 * @defgroup ALP_ATTN_MGR_IPC IPC Definitions and API
 * @ingroup ALP_ATTN_MGR_PRIVATE
 * @{
 */
/**
 * @brief Message IDs for Attention Manager client IPC interface
 */
typedef enum
{
    ALP_ATTN_MSG_ID_STATUS = 1,   //!< request server status string
    ALP_ATTN_MSG_ID_STATUS_REPLY, //!< response with server status string
    ALP_ATTN_MSG_ID_POST,         //!< post alert request
    ALP_ATTN_MSG_ID_POST_REPLY,   //!< post alert response
    ALP_ATTN_MSG_ID_UPDATE,       //!< update alert properties request
    ALP_ATTN_MSG_ID_UPDATE_REPLY, //!< update alert properties response
    ALP_ATTN_MSG_ID_DELETE,       //!< delete alert request
    ALP_ATTN_MSG_ID_DELETE_REPLY, //!< delete alert response
    ALP_ATTN_MSG_ID_LOOKUP,       //!< lookup alert request
    ALP_ATTN_MSG_ID_LOOKUP_REPLY  //!< lookup alert response
} AlpAttnMsgID;

/**
 * @brief Message ID
 */
typedef enum
{
    ALP_ATTN_MGR_MSG_ID_QUERY = 1
} AlpAttnMgrMsgID;
    
/**
 * @brief Status definitions
 * @{
 */
#define ALP_STATUS_ATTN_IPC_BAD_PARAM       (ALP_STATUS_ATTN_IPC | 0x00000100) //<! An invalid parameter was passed to the function call
#define ALP_STATUS_ATTN_IPC_NO_MEMORY       (ALP_STATUS_ATTN_IPC | 0x00000200) //<! Unable to allocate enough memory to complete the function call
#define ALP_STATUS_ATTN_IPC_INTERNAL        (ALP_STATUS_ATTN_IPC | 0x00000300) //<! An unexpected internal error occurred during the function call
#define ALP_STATUS_ATTN_IPC_TIMEOUT         (ALP_STATUS_ATTN_IPC | 0x00000400) //<! The IPC request timed out during the function call
/*@}*/

/**
 * @brief Channel definitions
 * @{
 */
#define ALP_ATTN_CALLBACK_CHANNEL_FORMAT   "alp-alert-callback-%d"
#define ALP_ATTN_CALLBACK_CHANNEL_PROPERTY "callback-channel"
/*@}*/

#ifdef __cplusplus
extern "C" {
#endif

    alp_status_t    alp_attn_ipc_init       (const char *channel);

    const char     *alp_attn_ipc_get_channel (void);

    alp_status_t    alp_attn_ipc_setup      (void);

    alp_status_t    alp_attn_ipc_teardown   (void);

    alp_status_t    alp_attn_ipc_send       (AlpMessage  *msg);
        
    alp_status_t
    alp_attn_ipc_send_with_response         (AlpMessage  *msg,
                                             AlpMessage **response);
        
    /*** unpacking routines ***/
    alp_status_t
    alp_attn_ipc_unpack_alert_post_msg      (AlpMessage    *message,
                                             AlpDmlItemH   *itemH);

    alp_status_t
    alp_attn_ipc_unpack_alert_post_reply    (AlpMessage    *response,
                                             alp_status_t  *status,
                                             AlpAttnId     *alertId);

    alp_status_t
    alp_attn_ipc_unpack_alert_update_msg    (AlpMessage    *message,
                                             AlpDmlItemH   *itemH);

    alp_status_t
    alp_attn_ipc_unpack_alert_update_reply  (AlpMessage    *response,
                                             alp_status_t  *status);

    alp_status_t
    alp_attn_ipc_unpack_alert_delete_msg    (AlpMessage    *message,
                                             AlpDmlItemH   *itemH);

    alp_status_t
    alp_attn_ipc_unpack_alert_delete_reply  (AlpMessage    *response,
                                             alp_status_t  *status);

#if 0
    alp_status_t
    alp_attn_ipc_unpack_lookup_id_msg       (AlpMessage    *message,
                                             AlpAttnAlertH *alertH);

    alp_status_t
    alp_attn_ipc_unpack_lookup_id_reply     (AlpMessage    *response,
                                             alp_status_t  *status,
                                             AlpAttnId     *alertId);
#endif

    alp_status_t
    alp_attn_ipc_unpack_status_reply_msg    (AlpMessage    *response,
                                             alp_status_t  *status,
                                             gchar        **srv_status);

    /*** packing routines ***/
    AlpMessage *
    alp_attn_ipc_pack_alert_post_msg        (const char     *source,
                                             const char     *name,
                                             const char     *handle,
                                             const char     *interface,
                                             gint32          priority,
                                             gint32          duration,
                                             guint32         nprops,
                                             AlpAttnProp    *props[],
                                             const char     *channel);
    AlpMessage *
    alp_attn_ipc_pack_alert_post_reply      (AlpMessage     *replyTo,
                                             guint32         status,
                                             guint32         id);
    AlpMessage *
    alp_attn_ipc_pack_alert_update_msg      (const char     *source,
                                             const char     *name,
                                             const char     *handle,
                                             guint32         nprops,
                                             AlpAttnProp    *props[]);
    AlpMessage *
    alp_attn_ipc_pack_alert_update_reply    (AlpMessage     *replyTo,
                                             guint32         status);


    AlpMessage *
    alp_attn_ipc_pack_alert_delete_msg      (const char     *source,
                                             const char     *name,
                                             const char     *handle);

    AlpMessage *
    alp_attn_ipc_pack_alert_delete_reply    (AlpMessage     *replyTo,
                                             guint32         status);

#if 0
    AlpMessage *
    alp_attn_ipc_pack_lookup_id_msg         (const char     *source,
                                             const char     *name,
                                             const char     *handle);
                                             
    AlpMessage *
    alp_attn_ipc_pack_lookup_id_reply       (AlpMessage     *replyTo,
                                             guint32         status,
                                             guint32         id);
#endif
    AlpMessage *
    alp_attn_ipc_pack_status_reply          (AlpMessage     *replyTo,
                                             guint32         status,
                                             gchar          *srv_status);

    alp_status_t
    alp_attn_ipc_send_callback (const char *channel,
                                const char *source,
                                const char *name,
                                const char *handle,
                                uint32_t    argc,
                                char       *argv[]);
/*@}*/

#ifdef __cplusplus
} // extern "C"
#endif

#endif /* __ALP_ATTN_IPC_H__ */
