/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <hiker/prv/exgmgr_prv.h>
 *
 * @brief Private Header file for the ALP Exchange Manager APIs.
 *
 ******************************************************************************/



#ifndef EXGMGR_PRV_H_
#define EXGMGR_PRV_H_

#include <unistd.h>
#include <hiker/exgmgr.h>

#ifdef __cplusplus
 extern "C"
 {
#endif

// name of AFUNIX socket listening for connections from exgmgr lib.
#define ALP_EXG_DAEMON_SOCKET_NAME "/tmp/alp_exg_mgr_daemon_socket"

// Opcodes for messages to the daemon
#define ALP_EXG_DAEMON_MSG_OPCODE_REGISTER_HANDLER 1
#define ALP_EXG_DAEMON_MSG_OPCODE_UNREGISTER_HANDLER 2
#define ALP_EXG_DAEMON_MSG_OPCODE_RECEIVE_REQUEST 3
#define ALP_EXG_DAEMON_MSG_OPCODE_IS_ALIVE 4
#define ALP_EXG_DAEMON_MSG_OPCODE_REGISTER_HANDLER_NOTIFICATION 5
#define ALP_EXG_DAEMON_MSG_OPCODE_UNREGISTER_HANDLER_NOTIFICATION 6
#define ALP_EXG_DAEMON_MSG_OPCODE_QUERY_GENERIC_VERBS 7
#define ALP_EXG_DAEMON_MSG_OPCODE_CHECK_HANDLER_REGISTRATION 8
#define ALP_EXG_DAEMON_MSG_OPCODE_CANCEL_REQUEST 9
#define ALP_EXG_DAEMON_MSG_OPCODE_GET_GUID 10

// Message passing between the exgmgr lib and daemon
typedef struct _AlpExgMgrdMessageType AlpExgMgrdMessageType;
struct _AlpExgMgrdMessageType {
	int itsOpcode;
	union
	{
		struct _register_handler_bundle
		{
			char itsVerb[ALP_EXGMGR_VERB_MAXLEN + 1];
			char itsLocalizedVerb[ALP_EXGMGR_LOCALIZED_VERB_MAXLEN + 1];
			char itsMIMEType[ALP_EXGMGR_MIMETYPE_MAXLEN + 1];
			char itsHandlerBundleName[ALP_EXGMGR_PACKAGENAME_MAXLEN + 1];
			int itsLocalOnly;
			int itsSerializeMe;
			int itsMustBeUnique;
			int itsAcceptGenericInvocation;
		} register_handler_bundle;
		
		struct _register_handler_notification
		{
			char itsVerb[ALP_EXGMGR_VERB_MAXLEN + 1];
			char itsLocalizedVerb[ALP_EXGMGR_LOCALIZED_VERB_MAXLEN + 1];
			char itsMIMEType[ALP_EXGMGR_MIMETYPE_MAXLEN + 1];
			char itsHandlerNotificationType[ALP_EXGMGR_NOTIFICATIONTYPE_MAXLEN + 1];
			int itsLocalOnly;
			int itsSerializeMe;
			int itsMustBeUnique;
			int itsAcceptGenericInvocation;
		} register_handler_notification;
		
		struct _unregister_handler
		{
			char itsVerb[ALP_EXGMGR_VERB_MAXLEN + 1];
			char itsMIMEType[ALP_EXGMGR_MIMETYPE_MAXLEN + 1];
			char itsHandlerBundleName[ALP_EXGMGR_PACKAGENAME_MAXLEN + 1];
		} unregister_handler;
		
		struct _unregister_handler_notification
		{
			char itsVerb[ALP_EXGMGR_VERB_MAXLEN + 1];
			char itsMIMEType[ALP_EXGMGR_MIMETYPE_MAXLEN + 1];
			char itsHandlerNotificationType[ALP_EXGMGR_NOTIFICATIONTYPE_MAXLEN + 1];
		} unregister_handler_notification;

		struct _check_handler_registration
		{
			char itsVerb[ALP_EXGMGR_VERB_MAXLEN + 1];
			char itsMIMEType[ALP_EXGMGR_MIMETYPE_MAXLEN + 1];
		} check_handler_registration;
		
		struct _query_generic_verbs
		{
			char itsMIMEType[ALP_EXGMGR_MIMETYPE_MAXLEN + 1];
		} query_generic_verbs;
				
		struct _receive_request
		{
			AlpExgRequest itsRequestId;	// this was passed to the handler at invoke time, and the handler sends it back to us when it connects
		} receive_request;

		struct _cancel_request
		{
			int itsGuid;
		} cancel_request;
	} params;
};

// Message passing between the exgmgr lib and daemon
typedef struct _AlpExgMgrdMessageReplyType AlpExgMgrdMessageReplyType;
struct _AlpExgMgrdMessageReplyType {
	alp_status_t itsReplyStatus;
	union
	{
		struct _get_request_guid
		{
			int itsRequestGuid;
		} get_request_guid;
	} params;
};

// This structure is just there to calculate the size needed to store any of the Msg or reply types above
typedef struct _AlpExgMgrdMessageOrReplyType AlpExgMgrdMessageOrReplyType;
struct _AlpExgMgrdMessageOrReplyType {
	union
	{
		AlpExgMgrdMessageType itsMsg;
		AlpExgMgrdMessageReplyType itsReply;
	} params;
};

// These are private parameter names used with the special store request. The request client
// or request handler should not use them directly (Look at alp_exg_utils_store_requestxxx in exgmgrutils.h)

// When specifying obex compatible data by filename, this variable contains the number of objects in the array
#define ALP_EXGMGR_OBEX_OBJECT_COUNT "obex_legacy_count"
// This is the basename of the first filename. Subsequent filename must increment the last character
#define ALP_EXGMGR_OBEX_OBJECT_FILE_BASE "obex_legacy_file0"
#define ALP_EXGMGR_OBEX_OBJECT_FILE_2ND "obex_legacy_file1"
// This is the basename of the first mime type. Subsequent mime type must increment the last character
#define ALP_EXGMGR_OBEX_OBJECT_TYPE_BASE "obex_legacy_type0"
// This is the basename of the first description. Subsequent description must increment the last character
#define ALP_EXGMGR_OBEX_OBJECT_DESC_BASE "obex_legacy_desc0"
// This is the basename of the first object name. Subsequent object name must increment the last character
#define ALP_EXGMGR_OBEX_OBJECT_NAME_BASE "obex_legacy_name0"

// Private parameter to hold the request progress notification type
#define ALP_EXGMGR_PROGRESS_NOTIF_PARAM_NAME "priv_progress_param"
#define ALP_EXGMGR_DEFAULT_PROGRESS_NOTIF "AlpProgressNotifType"

// Private parameter to hold the guid (system wide unique request id)
#define ALP_EXGMGR_REQUEST_GLOBAL_UID_PARAM_NAME "AlpPrivGlobalUid"
#define EXG_MGR_BT_INCOMING_UID 1	// special UID used to cancel an incoming BT object.
#define EXG_MGR_IR_INCOMING_UID 2
#define EXG_MGR_FIRST_GUID 20

// Private parameter to hold the 'progress dialog active' value
#define ALP_EXGMGR_REQUEST_PROGRESS_ACTIVE_PARAM_NAME "AlpPrivProgressActive"


int alp_exg_prv_daemon_is_alive ();
alp_status_t alp_exg_prv_extract_transport_params_item_value (char *iTransportParams, char *iItemName, char *oValue, size_t iValueLen);
void alp_exg_request_prv_get_version (AlpExgRequest iRequest, int *oVersion);
void alp_ext_request_prv_get_info (AlpExgRequest iRequest, char **oInfo, size_t *oInfoLen);
void alp_exg_request_prv_set_info (AlpExgRequest iRequest, char *iInfo, size_t iInfoLen);
alp_status_t alp_exg_request_prv_alloc_init_info (AlpExgRequest iRequest, size_t iInfoLen);

alp_status_t alp_exg_prv_sync_full_write (int iSocket, void *iBuf, size_t iBufLen);
alp_status_t alp_exg_prv_sync_full_read (int iSocket, void *iBuf, size_t iBufLen);
ssize_t alp_exg_prv_read_fd(int sockFD, void* payload, size_t maxPL, int* recvFD);
ssize_t alp_exg_prv_write_fd(int sockFD, void* payload, size_t nPL, int sendFD);
char *alp_exg_make_tmpfile (char *iPrefix, int *oFd);
alp_status_t alp_exg_prv_fd_to_file (int iFd, char *iPrefix, char **oFilename);
alp_status_t alp_exg_prv_cancel_incoming (int iGuid);

//! @}

#ifdef __cplusplus
 }
#endif

#endif /*EXGMGR_PRV_H_*/
