/* lsm_cmd.h

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

 * This header file contains #defines and structures common to both
 * kernel and user space SPF functions.
 */

#ifndef _LSM_CMD_H_
#define _LSM_CMD_H_

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define SPF_LSM_DEV_IN              "/dev/security_in"
#define LSMSPF_PRIMARY_INIT_FILE    "/var/opt/alp/data/lib/security/LSMSPF_init"

/* standard Input Command message structure */
typedef struct Alp_lsmCmd_unit {
    u_int32_t   command;        /* LSM_SPF command number */
    u_int32_t   subject_id;     /* Subject identifier */
    u_int32_t   object_id1;     /* (first) object identifier */
    u_int32_t   object_id2;     /* (second) object identifier */
    u_int32_t   object_id3;     /* (third) object identifier */
    u_int32_t   process_id;     /* thread identifier */
    u_int32_t   bit_flag;       /* Permission Field */
    int32_t     status;         /* return status for 'command' */        
} Alp_lsmCmd_unit;


/* SPF-LSM Command definitions */
/* ACL Set commands */
#define ALP_LSMCMD_SET_CREATE           1   /* Create New (but empty) ACL Set Container */
#define ALP_LSMCMD_SET_MODIFY           2   /* Modify bit_flag of existing ACL Set container */
#define ALP_LSMCMD_SET_DELETE           3   /* Delete an existing ACL Set container */
#define ALP_LSMCMD_SET_RESET_ALL        4   /* Removes ALL existing ACL sets AND D-Cache objs */
/* ACL Unit commands */
#define ALP_LSMCMD_ACL_INODE_ADD        5   /* Add Inode ACL to a Policy Set */
#define ALP_LSMCMD_ACL_INODE_MODIFY     6   /* Modify bit flag of a inode ACL */
#define ALP_LSMCMD_ACL_INODE_DELETE     7   /* Delete Inode ACL from Policy Set */
#define ALP_LSMCMD_ACL_INODE_RESET      8   /* remove ALL Inode ACL Set entries */
#define ALP_LSMCMD_ACL_NET_ADD          9   /* Add Net (IP) ACL to policy set */
#define ALP_LSMCMD_ACL_NET_MODIFY       10  /* Modify bit_flag of Net (IP) Perm Unit ACL */
#define ALP_LSMCMD_ACL_NET_DELETE       11  /* Delete ACL Net (IP) from Set */
#define ALP_LSMCMD_ACL_NET_RESET        12  /* remove ALL Dev ACL set entries */
#define ALP_LSMCMD_ACL_RESET_ALL        13  /* remove ALL Dev AND Inode ACL entries */
/* D-Cache commands */
#define ALP_LSMCMD_DCACHE_ADD           14  /* Add new App/Policy mapping */
#define ALP_LSMCMD_DCACHE_MODIFY        15  /* Modify App/Policy mapping */
#define ALP_LSMCMD_DCACHE_ALL_MOD       16  /* Change all matching Policy ID to new mapping */
#define ALP_LSMCMD_DCACHE_DELETE        17  /* Delete App/Policy mapping */
#define ALP_LSMCMD_DCACHE_RESET_ALL     18  /* Removes ALL EXISTING D-CACHE objects */
/* D-Cache Signal/Socket flag commands */
#define ALP_LSMCMD_DSIG_ADD             19  /* Add a bit mask bit to the D-Cache sig_field */
#define ALP_LSMCMD_DSIG_DEL             20  /* Delete a bit mask from the D-Cache sig_field */
#define ALP_LSMCMD_DSIG_RESET           21  /* Wipe the D-Cache sig_field */
/* DCL Unit commands */
#define ALP_LSMCMD_DCL_INODE_ADD        22  /* Add Inode DCL to a D-Cache */
#define ALP_LSMCMD_DCL_INODE_MODIFY     23  /* Modify bit flag of a inode DCL */
#define ALP_LSMCMD_DCL_INODE_DELETE     24  /* Delete Inode DCL from D-Cache */
#define ALP_LSMCMD_DCL_INODE_RESET      25  /* remove ALL Inode DCL from D-Cache */
#define ALP_LSMCMD_DCL_NET_ADD          26  /* Add Net (IP) DCL to D-Cache */
#define ALP_LSMCMD_DCL_NET_MODIFY       27  /* Modify bit_flag of Net DCL */
#define ALP_LSMCMD_DCL_NET_DELETE       28  /* Delete Net (IP) DCL from D-Cache */
#define ALP_LSMCMD_DCL_NET_RESET        29  /* remove ALL Net (IP) DCL D cache */
#define ALP_LSMCMD_DCL_RESET_ALL        30  /* remove ALL Dev & Inode DCL from D-Cache */
/* SPF-LSM PID commands */
#define ALP_LSMCMD_PID_MAP              31  /* Map a PID to a App Pkg ID (D-Cache) obj */
#define ALP_LSMCMD_PID_STAT             32  /* Get Security Properties of PID */
/* SPF-LSM Module commands */
#define ALP_LSMCMD_LSM_ENABLE           33  /* Enable LSM security functionality */
#define ALP_LSMCMD_LSM_DISABLE          34  /* Disable LSM security functionality */
#define ALP_LSMCMD_LSM_DUMP_META        35  /* (Reserved) Create Metadata file from LSM data */
#define ALP_LSMCMD_LSM_LOAD_META        36  /* Load Metadata file (alternate initialization) */

/* Security Properties Bit-Mask fields */
/* ACL Set Bit-Flags */
#define ALP_LSMSET_BFLAG_EXCLUSIVE  0   /* anything not denied is permitted */
#define ALP_LSMSET_BFLAG_INET_INCL  1   /* any IP addr not permitted is denied */
#define ALP_LSMSET_BFLAG_INET_ASK   2   /* any IP addr not permitted is asked */
#define ALP_LSMSET_BFLAG_INO_INCL   4   /* inode access not permitted is denied */
#define ALP_LSMSET_BFLAG_INO_ASK    8   /* inode access not permitted is asked */
#define ALP_LSMSET_BFLAG_SOCK_INCL  16  /* Socket access not permitted is denied */
#define ALP_LSMSET_BFLAG_SOCK_ASK   32  /* Socket access not permitted is asked */
/* ACL Set Sig-Flags */
#define ALP_LSMSET_SFLAG_BLOCK_ALL_SOCKET       0x00000001      /* Block all socket create */
#define ALP_LSMSET_SFLAG_PERMIT_SOCKET          0x00000002      /* Permit all sockets create */
#define ALP_LSMSET_SFLAG_BLOCK_INET             0x00000004      /* Block all IPv4 Sockets */
#define ALP_LSMSET_SFLAG_PERMIT_INET            0x00000008      /* Permit all IPv4 sockets */
#define ALP_LSMSET_SFLAG_BLOCK_IRDA             0x00000010      /* Block all IRDA Sockets */
#define ALP_LSMSET_SFLAG_PERMIT_IRDA            0x00000020      /* Permit IRDA Sockets */
#define ALP_LSMSET_SFLAG_BLOCK_BLUETOOTH        0x00000040      /* Block all Bluetooth Sockets */
#define ALP_LSMSET_SFLAG_PERMIT_BLUE            0x00000080      /* Permit all Bluetooth Sockets */
#define ALP_LSMSET_SFLAG_BLOCK_UNIX             0x00000100      /* Block all IPC Sockets */
#define ALP_LSMSET_SFLAG_PERMIT_UNIX            0x00000200      /* Permit all IPC sockets */
#define ALP_LSMSET_SFLAG_BLOCK_FD_RECEIVE       0x00000400      /* Block receiving File Desc via IPC */
#define ALP_LSMSET_SFLAG_PERMIT_FD_RECEIVE      0x00000800      /* Allow receiving file Desc via IPC */
#define ALP_LSMSET_SFLAG_NO_CONNECT             0x00001000      /* Block socket connect */
#define ALP_LSMSET_SFLAG_ASK_CONNECT            0x00002000      /* Ask for socket connect */
#define ALP_LSMSET_SFLAG_PERMIT_CONNECT         0x00004000      /* Permit Socket connect */
#define ALP_LSMSET_SFLAG_NO_LISTEN              0x00008000      /* Block socket listen */
#define ALP_LSMSET_SFLAG_ASK_LISTEN             0x00010000      /* Ask for socket listen */
#define ALP_LSMSET_SFLAG_PERMIT_LISTEN          0x00020000      /* Permit Socket listen */
/* D-Cache Obj Sig-Flags */
#define ALP_LSMDCH_SFLAG_CONNECT_INET_DENY          0x00000001  /* Deny ALL IPv4 Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_INET_BLOCK         0x00000002  /* Block once IPv4 Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_INET_ALLOW         0x00000004  /* Allow once IPv4 Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_INET_PERMIT        0x00000008  /* Permit ALL IPv4 Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_IRDA_DENY          0x00000010  /* Deny ALL IRDA Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_IRDA_BLOCK         0x00000020  /* Block once IRDA Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_IRDA_ALLOW         0x00000040  /* Allow Once IRDA Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_IRDA_PERMIT        0x00000080  /* Permit ALL IRDA Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_BLUETOOTH_DENY     0x00000100  /* Deny ALL BLUETOOTH Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_BLUETOOTH_BLOCK    0x00000200  /* Block once BLUETOOTH Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_BLUETOOTH_ALLOW    0x00000400  /* Allow Once BLUETOOTH Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_BLUETOOTH_PERMIT   0x00000800  /* Permit BLUETOOTH Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_UNIX_DENY          0x00001000  /* Deny ALL IPC Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_UNIX_BLOCK         0x00002000  /* Block once IPC Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_UNIX_ALLOW         0x00004000  /* Allow Once IPC Connect */
#define ALP_LSMDCH_SFLAG_CONNECT_UNIX_PERMIT        0x00008000  /* Permit IPC Connect */
#define ALP_LSMDCH_SFLAG_LISTEN_INET_DENY           0x00010000  /* Deny all IPv4 Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_INET_BLOCK          0x00020000  /* Block once IPv4 Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_INET_ALLOW          0x00040000  /* Allow once IPv4 Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_INET_PERMIT         0x00080000  /* Permit all IPv4 Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_IRDA_DENY           0x00100000  /* Deny all IRDA Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_IRDA_BLOCK          0x00200000  /* Block once IRDA Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_IRDA_ALLOW          0x00400000  /* Allow once IRDA Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_IRDA_PERMIT         0x00800000  /* Permit all IRDA Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_BLUETOOTH_DENY      0x01000000  /* Deny all BLUETOOTH Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_BLUETOOTH_BLOCK     0x02000000  /* Block once BLUETOOTH Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_BLUETOOTH_ALLOW     0x04000000  /* Allow once all BLUETOOTH Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_BLUETOOTH_PERMIT    0x08000000  /* Permit all BLUETOOTH Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_UNIX_DENY           0x10000000  /* Deny all IPC Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_UNIX_BLOCK          0x20000000  /* Block once IPC Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_UNIX_ALLOW          0x40000000  /* Allow Once all IPC Listen */
#define ALP_LSMDCH_SFLAG_LISTEN_UNIX_PERMIT         0x80000000  /* Permit all IPC Listen */
/* ACL Unit Bit-flags */
#define ALP_LSMACL_BFLAG_DENY       1   /* Default Deny */
#define ALP_LSMACL_BFLAG_PERMIT     2   /* Default Permit */
#define ALP_LSMACL_BFLAG_ASK        4   /* Default Ask */
/* DCL Unit Bit-Flags */
#define ALP_LSMDCL_BFLAG_DENY       1   /* unconditional deny */
#define ALP_LSMDCL_BFLAG_PERMIT     2   /* unconditional permit */
#define ALP_LSMDCL_BFLAG_BLOCK      4   /* block once only */
#define ALP_LSMDCL_BFLAG_ALLOW      8   /* allow ONCE ONLY */

/* Acknowledgement Message Error codes */
#ifndef ALP_CLASS_LSM
#define ALP_CLASS_LSM 0x1F000000
#endif /* ALP_CLASS_LSM */

#define ALP_STATUS_LSM_NO_SUB       (ALP_CLASS_LSM | 1 )    /* subject_id does not exist */
#define ALP_STATUS_LSM_DUP_SUB      (ALP_CLASS_LSM | 2 )    /* duplicate subject_id */
#define ALP_STATUS_LSM_INUSE_SUB    (ALP_CLASS_LSM | 3 )    /* subject_id in use, can't be deleted */
#define ALP_STATUS_LSM_NO_OBJ       (ALP_CLASS_LSM | 4 )    /* object_id does not exist */
#define ALP_STATUS_LSM_DUP_OBJ      (ALP_CLASS_LSM | 5 )    /* duplicate object_id */
#define ALP_STATUS_LSM_DCACHE_BUSY  (ALP_CLASS_LSM | 6 )    /* D-Cache in use by App */
#define ALP_STATUS_LSM_NO_THREAD    (ALP_CLASS_LSM | 7 )    /* Can't find thread from pid */
#define ALP_STATUS_LSM_FOPEN_FAIL   (ALP_CLASS_LSM | 8 )    /* Kernel File Open error */
#define ALP_STATUS_LSM_FWRITE_FAIL  (ALP_CLASS_LSM | 9 )    /* Kernel File Write error */
#define ALP_STATUS_LSM_BAD_INIT_CMD (ALP_CLASS_LSM | 10 )   /* Unexpected Init Cmd */

/* Kernel LSM Specific data structures, global variables and function declarations */
#ifdef __KERNEL__

/* Global ACL Set Linked List structures */
extern struct list_head 	glsmList_ACLset_anchor;	/* global ACL Set linked list anchor */
extern struct rw_semaphore	glsmList_ACLset_flag;   /* ACL Set linked list R/W Semaphore */

/* Global D-Cache Set Linked List structures */
extern struct list_head glsmList_DCACH_anchor;      /* global D-Cache linked list anchor */
extern struct rw_semaphore glsmList_DCACH_flag;     /* D-Cache list R/W Semaphore */

/* enable/disable LSM enforcement functionality */
extern atomic_t glsmCmd_LSM_toggle;


/* prv_lsmCmd_process_command
 * This function interpets and implements the SPF-LSM command interface.
 * 
 * Inputs:
 * Alp_lsmCmd_unit* cmd_unit_ptr: Ptr to Command structure for input
 *
 * Implied Inputs (Global Variables)
 * struct list_head glsmList_ACLset_anchor;     * global ACL Set linked list anchor *
 * struct rw_semaphore glsmList_ACLset_flag;    * ACL Set linked list R/W Semaphore *
 * struct list_head glsmList_DCACH_anchor;      * global D-Cache linked list anchor *
 * struct rw_semaphore glsmList_DCACH_flag;     * ACL Set/D-Cache list R/W Semaphore *
 * atomic_t glsmCmd_Msg_toggle;                 * Message Acknowledement toggle *
 * atomic_t glsmCmd_LSM_toggle;                 * LSM enforcement toggle *
 *
 * Side-Effects:
 * Depending on the Command, Kernel Memory may be allocated/deallocated.
 * Module global variables may be modified
 *
 * Outputs:
 * Alp_lsmCmd_unit::command: Value is set to 0 (SUCCESS) or error code.
 *
 * Returns int:
 * 0 = Success: Command successful
 * Anything else: error response of command/sys call
 */
int prv_lsmCmd_process_command(
        Alp_lsmCmd_unit* cmd_unit_ptr);/* Pointer to command unit buffer */


/* prv_lsmCmd_init()
 * Initializes static global structures 
 */
int prv_lsmCmd_init(void);


/* prv_lsmCmd_cleanup()
 * Cleanup static global structures
 */
int prv_lsmCmd_cleanup(void);


#endif /* __KERNEL__ */


#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* LSM_CMD */
