/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/

/*!
 * @file nlpd_bits.h
 *
 * @brief Header file for libalp_nlpd_bits.
 *
 ******************************************************************************/



#ifndef _ALP_NLPD_H
#define _ALP_NLPD_H 1

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>    // used for opendir()
#include <fcntl.h>     // used for open()
#include <signal.h>    // used for kill()
#include <grp.h>
#include <sys/prctl.h>
#include <sys/types.h> // used for opendir()
#include <sys/stat.h>  // used for open()
#include <hiker/security.h>
#include <hiker/bundlemgr.h>
#include <hiker/traces.h>
#include <hiker/config.h>

#ifdef __cplusplus
extern "C" {
#endif

#define OVERWRITE 1
#define BACKCHANNEL "alp_appserver_cookiecutter_channel"
#define MAX_PATH_LEN 512

//
// Given a null terminated string of the form
// [NAME=VALUE; | NAME;]+ this will set each
// environment NAME = VALUE or in the case of
// NAME; it will unset that environment variable.
//
// Example "HOME=/home/david;DISTCC;SHELL=/usr/bin/sh"
// Will set $HOME = /home/david
// Will unset $DISTCC and will
// set $SHELL = /usr/bin/sh
//
void alp_prv_SetUpEnv(char *env);

//
// Given a package and the length of that packages this will set up
// the runtime GID and UIDs for the process to run that package.
//

unsigned int alp_get_security_id( AlpBundle bundle );
int alp_set_security_parameters( unsigned int security_id );


//
// This function renames a running process.
// It does not change the /proc/procID/exe link.
// It does seem to change it for the "ps" command
// and the "top" command.
//
// This returns 0 if it succeed.  1 if it had
// a problem.  There is no error recovery possible
// so this is just informational.  An example of
// a 1 would be if the new name of the process is
// long than the old name.  In that case it only
// copies what will fit.
//
int alp_prv_rename_process( char *NewName, char *argv[] );

// Lets you signal all processes that are run from a binary
// who's path is pointed to by the char *.  This is usefull
// for killing NLPD because it changes it's name but it can't
// change it's backing store.
//
int alp_prv_signal_all( const char *, int );

// Private functions
static int alp_prv_file_is_digits( const struct dirent *);
static char *alp_prv_image_path( char *current );

#ifdef __cplusplus
}
#endif
#endif /* nlpd_bits.h */
