/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <hiker/prv/support/FileStream.h>
 *
 * @brief Private Header file for the ALP FileStream.
 *
 ******************************************************************************/



#ifndef ALP_FILESTREAM_H
#define ALP_FILESTREAM_H

// Includes ------------------------------------------------------------------
#include <stdio.h>
#include <string>


namespace alp {

// Typedefs ------------------------------------------------------------------
class FileStream
{
	public:
		enum openmode
		{
			app = 0x01,
			ate = 0x02,
			binary = 0x04,
			in = 0x08,
			out = 0x10,
			trunc = 0x20
		};
		enum iostate
		{
			goodbit = 0x00,
			badbit	= 0x01,
			eofbit	= 0x02,
			failbit	= 0x04
		};

		FileStream();
		FileStream(const char* path, int mode);
		FileStream(const std::string& path, int mode);
		~FileStream();

		void		open(const char* path, int mode);
		void		open(const std::string& path, int mode);
		void		close();

		FileStream& getline(char* s, size_t size);
		FileStream& getline(char* s, size_t size, char delim);
		FileStream& getline(std::string& s);
		FileStream& getline(std::string& s, char delim);

		FileStream& read(char* s, size_t size);
		FileStream&	write(const char* s, size_t size);

		bool		eof();
		bool		good();
		int			openmode();
		int			rdstate();
		void		clear(int state = goodbit);

	private:
		bool		getlinechar(int& c, char delim);
		void		getlinecheck(int& c, char delim);

		FILE*		fHandle;
		uint8_t		fOpenMode;
		uint8_t		fState;
};


// Prototypes ----------------------------------------------------------------
inline FileStream& getline(FileStream& fs, std::string& s) { return fs.getline(s); }
inline FileStream& getline(FileStream& fs, std::string& s, char delim) { return fs.getline(s, delim); }

FileStream& operator<<(FileStream& fs, const char* s);
FileStream& operator<<(FileStream& fs, const std::string& s);
FileStream& operator<<(FileStream& fs, int i);

}	// namespace alp


#endif	// ALP_FILESTREAM_H
