/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file Locker.h
 *
 * @brief Header File for ALP Locker. 
 *
 ******************************************************************************/

#ifndef _ALP_LOCKER_H
#define _ALP_LOCKER_H

#include <pthread.h>

namespace alp {

#ifdef LOCK_DEBUG
#include <stdio.h>
#define DEADLOCK_TRACE(x) x
#else
#define DEADLOCK_TRACE(x)
#endif


class Locker 
{
	public:
		Locker(bool _recursive = true)
		{ 
			if( true == _recursive )
			{
				pthread_mutex_t lock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
				fLock = lock;
			}
			else
			{
				pthread_mutex_t lock = PTHREAD_MUTEX_INITIALIZER;
				fLock = lock;
			}
		}	
		
		~Locker() { Unlock(); pthread_mutex_destroy(&fLock); }
		
	bool Lock() 
	{ 
		DEADLOCK_TRACE( fprintf(stderr, "%s: %p [%p]", __PRETTY_FUNCTION__, this, pthread_self()); );
		return 0 == pthread_mutex_lock(&fLock); 
	}
	
	void Unlock() 
	{ 
		pthread_mutex_unlock(&fLock); 
		DEADLOCK_TRACE( fprintf(stderr, "%s: %p [%p]", __PRETTY_FUNCTION__, this, pthread_self()) ); 
	}
	
	private:
		pthread_mutex_t	fLock;
};

// --------------------------------------------------------------
class Autolock
{
	public:
				Autolock(Locker& _lock) : fLock(_lock) { fLock.Lock(); }
				~Autolock() { fLock.Unlock(); }
		
	private:
		Locker& fLock;
};

}	// namespace alp

#endif
