/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*
 * Copyright (c) 2005 Palmsource, Inc.
 *
 * This software is licensed as described in the file LICENSE, which
 * you should have received as part of this distribution. The terms
 * are also available at http://www.openbinder.org/license.html.
 *
 * This software consists of voluntary contributions made by many
 * individuals. For the exact contribution history, see the revision
 * history and logs, available at http://www.openbinder.org
 */

#ifndef _SUPPORT_STRING_H
#define _SUPPORT_STRING_H

#include <string>

namespace alp {

class string : public std::string
{
	public:
					string(const std::string& _other) : std::string(_other), utf8(*this) { ; }
					string(const char* _other, size_type _pos = 0, size_type _n = npos) : std::string(NULL == _other ? "" : _other, _pos, _n), utf8(*this) { ; }
					string(const char* _other, size_t _length) : std::string(NULL == _other ? "" : _other, _length), utf8(*this) { ; }
					string(const string& _other) : std::string(_other.c_str()), utf8(*this) { ; }
					string() : utf8(*this) { ; }

			string& operator<<(const string& _other);
			string& operator<<(const char* _other);
			string& operator<<(uint32_t _val);
			string& operator<<(int32_t _val);
			string& operator<<(int64_t _val);
			string& operator<<(uint64_t _val);
			string& operator=(const string& _other) { std::string::operator=(_other); return *this; }
			
	public:		
			class utf8_impl
			{
				public:
					utf8_impl(alp::string& _self) : fSelf(_self) { }
					
					size_t length() const;
					
				private:
					alp::string& fSelf;
			};
					
	public:
			utf8_impl	utf8;
};

// ------------------------------------------------------------------------
inline size_t
utf8_char_len(uint8_t ch)
{
	return ((0xe5000000 >> ((ch >> 3) & 0x1e)) & 3) + 1;
}

// ------------------------------------------------------------------------
inline uint32_t
utf8_char_to_uint32(const uint8_t *src, uint32_t length)
{
	uint32_t result = 0;

	for (uint32_t index = 0; index < length; index++)
		result |= src[index] << (24 - index * 8);

	return result;
}



/** @brief Convert a uint32 packed wchar32 type to a const char* compatible utf8 string.
 * @param[in] _source a utf8 packed into a uint32 variable.
 * @param[out] _result a 4 byte uint8_t* array.
 */
size_t uint32_to_utf8(uint32_t _source, uint8_t* _result);


}	// namespace alp

std::string& operator<<(std::string& _lhs, const std::string& _other);
std::string& operator<<(std::string& _lhs, const char* _other);
std::string& operator<<(std::string& _lhs, uint32_t _val);
std::string& operator<<(std::string& _lhs, int32_t _val);


#endif /* _SUPPORT_STRING_H */
