/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file volumesvcs_prv.h
 *
 * @brief Private header file for the ALP Volume services APIs.
 *
 ******************************************************************************/




#ifndef VOLUMESVCS_PRV_H
#define VOLUMESVCS_PRV_H

#include <hiker/volumesvcs.h>

#define ALP_VOLUMESVCS_SERVER_CHANNEL "alp_volumesvcs_server" // user requests come in here
#define ALP_VOLUMESVCS_UDEV_CHANNEL "alp_volumesvcs_udev" // udev evnents come in here
#define ALP_VOLUMESVCS_APPID "com.access.servers.volumesvcsd"

/*
 * environemtn vars and codes for passing udev events
 */
#define ALP_VOLUMESVCS_DEVNAME_CODE 0
#define ALP_VOLUMESVCS_DEVPATH_CODE 1
#define ALP_VOLUMESVCS_PHYSBUS_CODE 2
#define ALP_VOLUMESVCS_MAJOR_CODE 3
#define ALP_VOLUMESVCS_MINOR_CODE 4
#define ALP_VOLUMESVCS_ID_FS_LABEL_CODE 5
#define ALP_VOLUMESVCS_ID_FS_TYPE_CODE 6
#define ALP_VOLUMESVCS_ID_FS_UUID_CODE 7
#define ALP_VOLUMESVCS_UDEV_CODES 8

typedef struct {
	short	code;	// from above list
	char	*envvar;
} prv_alpudevmap_t;

static const prv_alpudevmap_t sudevmap[]={
   	{ALP_VOLUMESVCS_DEVNAME_CODE,"DEVNAME"},
	{ALP_VOLUMESVCS_DEVPATH_CODE,"DEVPATH"},
	{ALP_VOLUMESVCS_PHYSBUS_CODE,"PHYSDEVBUS"},
	{ALP_VOLUMESVCS_MAJOR_CODE,"MAJOR"},
	{ALP_VOLUMESVCS_MINOR_CODE,"MINOR"},
	{ALP_VOLUMESVCS_ID_FS_LABEL_CODE,"ID_FS_LABEL"},
	{ALP_VOLUMESVCS_ID_FS_TYPE_CODE,"ID_FS_TYPE"},
	{ALP_VOLUMESVCS_ID_FS_UUID_CODE,"ID_FS_UUID"}
};


/*
 * commands for rpc requests, each packet starts with a cmd code.  typedefs
 * are not used for the packets to allow for char pointers insted
 * of fixed size arrays,  the resposne packet uses the msgID for the
 * returned status.
 * comments following each code are the (intneral only) args
 */
#define ALP_VOLUMESVCS_CMD_MOUNT ALP_CLASS_VOLUMESVCS + 1
// udi to mount
// response ID: status
#define ALP_VOLUMESVCS_USER_CMD_MIN ALP_VOLUMESVCS_CMD_MOUNT

#define ALP_VOLUMESVCS_CMD_UMOUNT ALP_CLASS_VOLUMESVCS + 2
// udi to unmount
// response ID: status

#define ALP_VOLUMESVCS_CMD_FSINFO ALP_CLASS_VOLUMESVCS + 3
// none
// response ID: status
// response msg data:
//		 N = number of following elements
// 	     if N>0, then N AlpVolumeSvcsFSinfo elemnets packed as strings
//     	         and a byte array for the fstat and flags

#define ALP_VOLUMESVCS_CMD_MAKEFS ALP_CLASS_VOLUMESVCS + 4
// udi of device to format
// name to use as mount point
// fstype to format
// response ID: status

#define ALP_VOLUMESVCS_CMD_BIND ALP_CLASS_VOLUMESVCS + 5
// name
// dirpath to mount --bind on /media/<name>
// response ID status

#define ALP_VOLUMESVCS_CMD_FSTYPES ALP_CLASS_VOLUMESVCS + 6
// none
// response ID: status
// response msg data:
//		 N = number of following elements
// 	     if N>0, then N char pointers follow for the fstypes supported

#define ALP_VOLUMESVCS_CMD_RENAME ALP_CLASS_VOLUMESVCS + 7
// udi of entry to rename
// new name
// response ID: status

#define ALP_VOLUMESVCS_USER_CMD_MAX ALP_VOLUMESVCS_CMD_RENAME

#define ALP_VOLUMESVCS_UDEV_ADD ALP_CLASS_VOLUMESVCS + 20
// UDEV_MAX_CODE pairs of env var key num from above
//                        matching string for that key
#define ALP_VOLUMESVCS_UDEV_CMD_MIN ALP_VOLUMESVCS_UDEV_ADD
// UDEV_MAX_CODE pairs of env var key num from above
//                        matching string for that key

#define ALP_VOLUMESVCS_UDEV_REMOVE ALP_CLASS_VOLUMESVCS + 21
#define ALP_VOLUMESVCS_CTRL_EXIT ALP_CLASS_VOLUMESVCS + 22
#define ALP_VOLUMESVCS_UDEV_CMD_MAX ALP_VOLUMESVCS_CTRL_EXIT

/* flags that are internal only
 */
#define ALP_VOLUMESVCS_FLAGS_NOTIFY_PENDING 0x80000000
#define ALP_VOLUMESVCS_FLAGS_NONOTIFY       0x40000000
#define ALP_VOLUMESVCS_FLAGS_HIDE           0x20000000
#define ALP_VOLUMESVCS_FLAGS_QUIET_KIDS     0x10000000
#define ALP_VOLUMESVCS_FLAGS_PRIVATE_MASK   0xFFFF0000

#define PACKSTART(as,msg,cmd,error) {if ((as=alp_ipc_message_pack_start(msg,cmd)) != ALP_STATUS_OK) goto error;}
#define PACKRSPSTART(as,msg,cmd,rqst_msg,error) {if ((as=alp_ipc_message_response_pack_start(msg,cmd,rqst_msg)) != ALP_STATUS_OK) goto error;}
#define PACKEND(as,msg,error) {if ((as=alp_ipc_message_pack_end(msg)) != ALP_STATUS_OK) goto error;}
#define UNPACKSTART(as,msg,error) {if ((as=alp_ipc_message_unpack_start(msg)) != ALP_STATUS_OK) goto error;}
#define UNPACKEND(as,msg,error) {if ((as=alp_ipc_message_unpack_end(msg)) != ALP_STATUS_OK) goto error;}
#define PACKU32(as,msg,val,error) {if ((as=alp_ipc_message_pack_uint32(msg, val)) != ALP_STATUS_OK) goto error;}
#define UNPACKU32(as,msg,val,error) {if ((as=alp_ipc_message_unpack_uint32(msg, val)) != ALP_STATUS_OK) goto error;}
#define PACKSTR(as,msg,val,error) {if ((as=alp_ipc_message_pack_string(msg, val)) != ALP_STATUS_OK) goto error;}
#define UNPACKSTR(as,msg,val,error) {if ((as=alp_ipc_message_unpack_string(msg, val)) != ALP_STATUS_OK) goto error;}
#define PACKBYTES(as,msg,val,len,error) {if ((as=alp_ipc_message_pack_byte_array(msg, val, len)) != ALP_STATUS_OK) goto error;}
#define UNPACKBYTES(as,msg,val,len,error) {if ((as=alp_ipc_message_unpack_byte_array(msg, val, len)) != ALP_STATUS_OK) goto error;}

#endif
