/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*! 
 * @file <hiker/sal.h>
 *
 * @brief Public header for the SQL abstraction layer.
 ****************************************************************************/

/*! @brief SQL Abstraction Layer */
/**
 * @defgroup ALP_SAL SQL Abstraction Layer
 * @ingroup ALP_SYSTEM
 * @{
*/


/*Includes********************************************************************/
#ifndef ALP_SAL_H_
#define ALP_SAL_H_       1

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <hiker/types.h>
#include <hiker/sysclass.h>


/*Defines*********************************************************************/
/*! @brief SQL Abstraction Layer Error Codes */
/**
 * @defgroup ALP_SAL_ERRORS Errors
 * @ingroup ALP_SAL
 * @{
 */
#define ALP_STATUS_SAL_BUSY			(ALP_CLASS_SAL | 0x00010000)
#define ALP_STATUS_SAL_LOCKED		(ALP_CLASS_SAL | 0x00020000)
#define ALP_STATUS_SAL_NOMEM 		(ALP_CLASS_SAL | 0x00030000)
#define ALP_STATUS_SAL_READONLY		(ALP_CLASS_SAL | 0x00040000)
#define ALP_STATUS_SAL_INTERRUPT	(ALP_CLASS_SAL | 0x00050000)
#define ALP_STATUS_SAL_IOERR    	(ALP_CLASS_SAL | 0x00060000)
#define ALP_STATUS_SAL_CORRUPT  	(ALP_CLASS_SAL | 0x00070000)
#define ALP_STATUS_SAL_FULL     	(ALP_CLASS_SAL | 0x00080000)
#define ALP_STATUS_SAL_CANTOPEN 	(ALP_CLASS_SAL | 0x00090000)
#define ALP_STATUS_SAL_TOOBIG   	(ALP_CLASS_SAL | 0x000A0000)
#define ALP_STATUS_SAL_CONSTRAINT	(ALP_CLASS_SAL | 0x000B0000)
#define ALP_STATUS_SAL_MISMATCH 	(ALP_CLASS_SAL | 0x000C0000)
#define ALP_STATUS_SAL_AUTH     	(ALP_CLASS_SAL | 0x000D0000)
#define ALP_STATUS_SAL_ERROR    	(ALP_CLASS_SAL | 0x000E0000)
#define ALP_STATUS_SAL_MISUSE   	(ALP_CLASS_SAL | 0x000F0000)
#define ALP_STATUS_SAL_NOLFS    	(ALP_CLASS_SAL | 0x00100000)
#define ALP_STATUS_SAL_PROTOCOL 	(ALP_CLASS_SAL | 0x00110000)
#define ALP_STATUS_SAL_EMPTY    	(ALP_CLASS_SAL | 0x00120000)
#define ALP_STATUS_SAL_SCHEMA   	(ALP_CLASS_SAL | 0x00130000)
#define ALP_STATUS_SAL_NOTFOUND 	(ALP_CLASS_SAL | 0x00140000)
#define ALP_STATUS_SAL_INTERNAL 	(ALP_CLASS_SAL | 0x00150000)
#define ALP_STATUS_SAL_PERM     	(ALP_CLASS_SAL | 0x00160000)
#define ALP_STATUS_SAL_ABORT    	(ALP_CLASS_SAL | 0x00170000)
#define ALP_STATUS_SAL_UNKNOWN  	(ALP_CLASS_SAL | 0x00180000)
#define ALP_STATUS_SAL_BADPARAM 	(ALP_CLASS_SAL | 0x00190000)
#define ALP_STATUS_SAL_NORECORDS	(ALP_CLASS_SAL | 0x001A0000)
#define ALP_STATUS_SAL_DMLEXISTS	(ALP_CLASS_SAL | 0x001B0000)
/*@}*/


/*Typedefs********************************************************************/
/*! The AlpSALHandle must be passed around as a reference to the database */
typedef void *AlpSALHandle;

/*! A AlpSALStatement is an opaque structure used to bind host variables to a 
 * sql statement to be prepared for execution, and ultimately passed to 
 * other alp_sal_statement_* calls.
 */
typedef void *AlpSALStatement;

/*Prototypes******************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

/*! @brief SQL Abstraction Layer Data Access */
/**
 * @defgroup ALP_SAL_DATA_ACCESS Data Access
 * @ingroup ALP_SAL
 * @{
 */

extern alp_status_t 	alp_sal_database_open(AlpSALHandle *sal, 
						  const char *dbName);
extern alp_status_t 	alp_sal_database_close(AlpSALHandle sal);
extern alp_status_t 	alp_sal_database_create(const char *dbFileName,
						 const char *schemaFileName);

extern alp_status_t 	alp_sal_transaction_commit(AlpSALHandle sal);
extern alp_status_t 	alp_sal_transaction_begin(AlpSALHandle sal, 
						  char exclusive);
extern alp_status_t		alp_sal_transaction_rollback(AlpSALHandle sal);

extern alp_status_t 	alp_sal_sql_execute(AlpSALHandle sal, const char *sql, ...);
extern alp_status_t     alp_sal_last_insert_rowid(AlpSALHandle sal, 
						  uint32_t *rowID);
extern alp_status_t 	alp_sal_execute_sql_script(AlpSALHandle sal, FILE *fd);
extern alp_status_t 	alp_sal_sql_get_value(AlpSALHandle sal, int32_t 
						  *value, const char *sql, ...);

extern const char   	*alp_sal_get_error_message(AlpSALHandle sal);

extern alp_status_t 	alp_sal_string_build(char **combined, 
						  const char *sql, ...);
extern void         	alp_sal_string_destroy(char *s);
/*@}*/

/*! @brief SQL Abstraction Layer AlpSALStatement APIs */
/**
 * @defgroup ALP_SAL_STATEMENTS  AlpSALStatement APIs
 * @ingroup ALP_SAL
 * @{
 */
/*alpSALStatement.c***********************************************************/
extern alp_status_t 	alp_sal_statement_initialize(AlpSALHandle sal, 
						  AlpSALStatement **stmt, const char *sql);
extern AlpSALHandle     alp_sal_statement_get_handle(AlpSALStatement *stmt);

extern alp_status_t 	alp_sal_statement_set_text(AlpSALStatement *stmt, 
						  int16_t pos, const char *s, bool copy);
extern alp_status_t 	alp_sal_statement_set_int(AlpSALStatement *stmt, 
						  int16_t pos, int32_t n);
extern alp_status_t 	alp_sal_statement_set_int64(AlpSALStatement *stmt, 
						  int16_t pos, int64_t n);
extern alp_status_t 	alp_sal_statement_set_blob(AlpSALStatement *stmt, 
						  int16_t pos, void *blob, int32_t size, bool copy);
extern alp_status_t 	alp_sal_statement_set_float(AlpSALStatement *stmt, 
						  int16_t pos, float f);

extern int32_t			alp_sal_statement_get_int(AlpSALStatement *stmt, 
						  int pos);
extern int64_t			alp_sal_statement_get_int64(AlpSALStatement *stmt, 
						  int pos);
extern float			alp_sal_statement_get_float(AlpSALStatement *stmt, 
						  int pos);
extern char 			*alp_sal_statement_get_text(AlpSALStatement *stmt, 
						  int pos, bool copy);
extern void         	*alp_sal_statement_get_blob(AlpSALStatement *stmt, 
						  int pos, int32_t *size, bool copy);
extern bool		        alp_sal_statement_is_null(AlpSALStatement *stmt, 
						  int pos);

extern alp_status_t 	alp_sal_statement_execute(AlpSALStatement *stmt);
extern alp_status_t 	alp_sal_statement_destroy(AlpSALStatement *stmt);
extern alp_status_t 	alp_sal_statement_next_result(AlpSALStatement *stmt);
extern alp_status_t 	alp_sal_statement_reset(AlpSALStatement *stmt);
/*!
@} //End of SAL Errors Group
*/

/*!
@} //End of SAL Group 
*/

#ifdef __cplusplus
}
#endif

#endif /* ALP_SAL_H_ */

