/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*! @file <hiker/security.h>
 *  @brief Security Policy Framework public header file for functions
 *	
 *	@addtogroup ALP_SECURITY Security Policy Framework
 *	@{
 */
 
#ifndef ALP_SECURITY_H_
#define ALP_SECURITY_H_ 

//-----------------------------------------------------------------------------
// INCLUDES
//-----------------------------------------------------------------------------

#include <hiker/types.h>
#include <sys/types.h>
#include <hiker/sysclass.h>
#include <unistd.h>
#include <stdlib.h>

#ifdef  __cplusplus
extern "C" {
#endif

//-----------------------------------------------------------------------------
// DEFINES
//-----------------------------------------------------------------------------

/*! @brief Security Policy Framework constants */

/*! error_code: no error */
#define ALP_STATUS_SPF_E_NOERR                  ( ALP_CLASS_SPF | 0x00 )

/*! error_code: client cannot connect to server */
#define ALP_STATUS_SPF_NO_SERVER                ( ALP_CLASS_SPF | 0x01 )

/*! error_code: null or invalid parameter passed to call */
#define ALP_STATUS_SPF_INVALID_PARAM            ( ALP_CLASS_SPF | 0x02 )

/*! error_code: error allocating memory */
#define ALP_STATUS_SPF_MEM_ERROR                ( ALP_CLASS_SPF | 0x03 )

/* todo: these error codes predate the move to ALP_STATUS_. We couldn't
** get approval from project management to include them. So we're going
** to have to do a complete documentation cycle before we can start
** using them. In the mean time, any calls that would have generated
** these return codes are now generating ALP_STATUS_SPF_INVALID_PARAM.
*/

/*
#define ALP_SPF_E_UPDATE_GENERAL		( ALP_CLASS_SPF | 0x01 )
#define ALP_SPF_E_UPDATE_BADSIG			( ALP_CLASS_SPF | 0x02 )
#define ALP_SPF_E_UPDATE_BADACTOR		( ALP_CLASS_SPF | 0x03 )
#define ALP_SPF_E_QUERY_BADASSERTION	        ( ALP_CLASS_SPF | 0x04 )
#define ALP_SPF_E_QUERY_BADPOLICY	        ( ALP_CLASS_SPF | 0x05 )
#define ALP_SPF_E_QUERY_NOSERVICE		( ALP_CLASS_SPF | 0x06 )
#define ALP_SPF_E_QUERY_BADFORMAT		( ALP_CLASS_SPF | 0x07 )
*/

/*! assertion type code : string */
#define ALP_SPF_T_STRING   0

/*! assertion type code : boolean */
#define ALP_SPF_T_BOOLEAN  1

/*! assertion type code : int */
#define ALP_SPF_T_INT      2

/*! assertion type code : IPv4 address */
#define ALP_SPF_T_IPV4ADDR 3

/*! assertion type code : file or directory path */
#define ALP_SPF_T_PATH     4

/*! assertion type code : url */
#define ALP_SPF_T_URL      5

/*! PKCS5 selector: pin */
#define ALP_SPF_PKCS5_PIN 0

/*! PKCS5 selector: puk */
#define ALP_SPF_PKCS5_PUK 1

//-----------------------------------------------------------------------------
// TYPE DEFINITIONS
//-----------------------------------------------------------------------------

/*! @brief  SPF data structures
 * 			AlpSecurityPolicy
 * 			AlpSecurityResponse
 */
typedef unsigned int AlpSPFAssertionType;

typedef unsigned int AlpPackageID;

typedef struct {
  char *key;
  unsigned int value;
} AlpSPFIntroInstance;

typedef struct {
  unsigned int version;
  AlpSPFIntroInstance **instances;
} AlpSPFIntroInfo;

/* AlpSecurityResponse is deprecated. Do not test.
**
** typedef struct _AlpSecurityResponse {
**   uint response;
**   char *facility;
**   uint facility_len;
**   char *subject;
**   uint subject_len;
**   char *object;
**   uint object_len;
**   char *operation;
**   uint operation_len;
** } AlpSecurityResponse;
*/

//-----------------------------------------------------------------------------
// UPDATE APIs
//-----------------------------------------------------------------------------

/*!
 * @brief   This function prepares the client for communication with the security service, creating message contexts and allocating buffers if necessary.
 *
 * @return  Returns ALP_STATUS_SPF_E_NOERR if successful, ALP_STATUS_SPF_NO_SERVER otherwise.
 *
 *
 */
alp_status_t alp_spf_update_open( void );

/*!
 * @brief   This function passes an update to the security service.
 *
 * @param [in] message: The policy to be updated in a signed message format.
 *         
 * @param [in] message_len: Then length of the passed in message.
 *    
 * @return  ALP_STATUS_SPF_E_NOERR if successful, ALP_STATUS_SPF_NO_SERVER if the security service is unreachable or ALP_STATUS_SPF_INVALID_PARAM for any other error.
 *          
 *
 */
alp_status_t alp_spf_update( char *message, uint message_len );

/*!
 * @brief   This function .....
 *
 * @return  Returns ALP_STATUS_SPF_E_NOERR .....
 *
 *
 */
alp_status_t alp_spf_update_close( void );

//-----------------------------------------------------------------------------
// POLICY STATUS APIs
//-----------------------------------------------------------------------------

/*!
 * @brief   This function .....
 *
 * @return  Returns ALP_STATUS_SPF_E_NOERR .....
 *
 *
 */
alp_status_t alp_spf_ps_open( void );
/*!
 * @brief  This function ....
 *
 * @param [in]  policy: The policy input to get the policyname from .......
 *         
 * @param [out] policyname: Reference to the extracted policy name ....
 *    
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_get_policyname_from_policy( unsigned int policy, char **policyname );
/*!
 * @brief This function ....
 *       
 * @param [out] policyname: The policyname input to get the policy from .......
 *    
 * @param [in]  policy:  Reference to the extracted policy ....
 * 
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_get_policy_from_policyname( char *policyname, unsigned int *policy);

/*!
 * @brief This function ....
 *       
 * @param [in] packageid: The PackageID of the package in question
 *
 * @param [out] out_count: The reference count of the package before incrementing
 * 
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_packageid_increment( unsigned int packageid, unsigned int *out_count );

/*!
 * @brief This function ....
 *       
 * @param [in] packageid: The PackageID of the package in question
 *
 * @param [out] out_count: The reference count of the package before decrementing.
 * 
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_packageid_decrement( unsigned int packageid, unsigned int *out_count );

/*!
 * @brief This function ....
 *       
 * @param [in] packageid: The PackageID of the package in question
 *
 * @param [out] out_count: The reference count of the package.
 * 
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_packageid_count( unsigned int packageid, unsigned int *out_count );

/*!
 * @brief This function ....
 *       
 * @param [in]  certificate: The RFC1113/BASE64 encoding of the certificate associated with the policy you're interested in finding.
 * 
 * @param [in]  certificate_length: The length certificate encoding
 *
 * @param [out] policy: The policy associated with the given certificate. Or 0 if the certificate given does not map to any policy.
 *  
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_get_policy_by_certificate( char *certificate, unsigned int certificate_length, unsigned int *policy);
/*!
 * @brief This function ....
 *       
 * @param [in]  gid:
 *
 * @param [out] policy: 
 *  
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_get_policy_by_gid( gid_t gid, unsigned int *policy);
/*!
 * @brief This function ....
 *       
 * @param [in]  policy:
 *
 * @param [out] certificate: 
 *  
 * @param [out] certificate_length: 
 * 
 * @param [out] certid: 
 *  
 * @param [out] certid_length: 
 * 
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_get_certificate_by_policy( unsigned int policy, char **certificate, unsigned int *certificate_length, char **certid, unsigned int *certid_length);
/*!
 * @brief This function ....
 *
 * @param [in]   policy: 
 *  
 * @param [out]  gid:
 * 
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_get_gid_by_policy( unsigned int policy, gid_t *gid );
/*!
 * @brief This function ....
 *       
 * @param [in]  uid:
 *
 * @param [out] policy: 
 *  
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_get_policy_by_uid( uid_t uid, unsigned int *policy );
/*!
 * @brief This function ....
 *
 * @param [in]   path: 
 *  
 * @param [in]   path_length: 
 *
 * @param [out]  out_policy:
 * 
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_get_policy_by_package( char *path, unsigned int path_length, unsigned int *out_policy);
/*!
 * @brief This function ....
 *
 * @param [in]   policy: 
 
 * @param [out]  size:
 * 
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_get_gids_length( unsigned int policy, unsigned int *size );
/*!
 * @brief This function ....
 *
 *  
 * @param [in] policy:
 *
 * @param [out] uid:
 *
 * @param [out] gid:
 *
 * @param [out] sgids:
 *
 * @param [out] sgids_length:
 * 
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_spf_ps_get_uid_gid_gids_by_policy( unsigned int policy, uid_t *uid, gid_t *gid, gid_t *sgids[], unsigned int *sgids_length );
/*!
 * @brief This function ....
 *
 * @param [in]   path: 
 *  
 * @param [in]   path_length: 
 *
 * @param [out]  out_packageid:
 * 
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */
alp_status_t alp_prv_spf_ps_enroll_package( char *path, unsigned int path_length, unsigned int *out_packageid );
/*!
 * @brief   This function .....
 *
 * @return  Returns ALP_STATUS_SPF_E_NOERR .....
 *
 *
 */
alp_status_t alp_spf_ps_close( void );

//-----------------------------------------------------------------------------
// QUERY APIs
//-----------------------------------------------------------------------------

/*!
 * @brief   This function .....
 *
 * @return  Returns ALP_STATUS_SPF_E_NOERR .....
 *
 *
 */
alp_status_t alp_spf_query_open( void );
alp_status_t alp_spf_query_set_subject_pid( pid_t subject_pid );
alp_status_t alp_spf_query_reset_subject_pid( void  );
/*!
 * @brief This function ....
 *
 * @param [in]  assertion:
 *   
 * @param [in]  assertion_len:
 * 
 * @param [out]	out_assertion_value:
 *     
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */ 
alp_status_t alp_spf_query_get_assertion_string( char *assertion, uint assertion_len, char **out_assertion_value);
alp_status_t alp_spf_query_get_assertion_int( char *assertion, uint assertion_len, int *out_assertion_value );
alp_status_t alp_spf_query_get_assertion_boolean( char *assertion, uint assertion_len, uint *out_assertion_value );

/*!
 * @brief This function ....
 *
 * @param [in]  assertion:
 *   
 * @param [in]  assertion_len:
 *     
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */ 
alp_status_t alp_spf_query_free_assertion_string( char *assertion, uint assertion_len );
/*!
 * @brief This function ....
 *
 * @param [in]  assertion:
 *   
 * @param [in]  assertion_len:
 * 
 * @param [out]	out_type:
 *     
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */ 
alp_status_t alp_spf_query_assertion_type( char *assertion, uint assertion_len, AlpSPFAssertionType *out_type );
/*!
 * @brief This function ....
 *
 * @param [in]  policy:
 *   
 * @param [out]  assertion:
 * 
 * @param [out]  assertion_len:
 * 
 * @param [out]	out_assertion_value:
 *     
 * @return  ALP_STATUS_SPF_E_NOERR  ....
 *          
 *
 */ 
alp_status_t alp_spf_query_get_assertion_string_from_policy( char *policy, char *assertion, uint assertion_len, char **out_assertion_value );
alp_status_t alp_spf_query_get_assertion_int_from_policy( char *policy, char *assertion, uint assertion_len, int *out_assertion_value );
alp_status_t alp_spf_query_get_assertion_boolean_from_policy( char *policy, char *assertion, uint assertion_len, uint *out_assertion_value );
alp_status_t alp_spf_query_assertion_type_from_policy( char *policy, char *assertion, uint assertion_len, AlpSPFAssertionType *out_type );
/*!
 * @brief   This function .....
 *
 * @return  Returns ALP_STATUS_SPF_E_NOERR .....
 *
 *
 */
 alp_status_t alp_spf_query_close( void );

//-----------------------------------------------------------------------------
// USER DIALOG MANAGER APIs
//-----------------------------------------------------------------------------

/* User Dialog Manager API calls are deprecated. Do not test.
** alp_status_t alp_spf_udm_open( void );
** alp_status_t alp_spf_udm_query( AlpSecurityResponse *response );
** alp_status_t alp_spf_udm_close( void );
*/

alp_status_t alp_spf_info(  AlpSPFIntroInfo **out_info );

  /* PKCS5 Calls */

  alp_status_t alp_spf_pkcs5_set( unsigned int which, char *phrase );
  alp_status_t alp_spf_pkcs5_check( unsigned int which, char *phrase, unsigned int *correct );
  alp_status_t alp_spf_pkcs5_params_get( unsigned int which, unsigned int *iteration, char **salt, unsigned int *active );
  alp_status_t alp_spf_pkcs5_params_set( unsigned int which, unsigned int iteration, char *salt, unsigned int active );

#ifdef  __cplusplus
}
#endif

/*! @} */

#endif /* ALP_SECURITY_H_ */
