/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <hiker/traces.h>
 *
 * @brief Header file for Traces.  Traces are active if ALP_TRACE == ALP_TRACE_ON
 *
 ******************************************************************************/

#ifndef __ALP_TRACES_H__
#define __ALP_TRACES_H__

#include <stdio.h>
#include <stdarg.h>
#include <hiker/config.h>
#include <hiker/sysclass.h>
#include <hiker/fail.h>

#ifdef __cplusplus
extern "C" {
#endif


/* ---- Prototyping ---------------------- */ 
/* These functions should not be called directly */

void alp_prv_trace_outputVT	(int traceModule, const char* aFormatString, va_list arglist);
void alp_prv_trace_outputTL	(int traceModule, const char* aFormatString, ...);
void alp_prv_trace_outputVTL	(int traceModule, const char* aFormatString, va_list arglist);
void alp_prv_trace_outputT		(int traceModule, const char* aFormatString, ...);
void alp_prv_trace_outputB	(int traceModule, unsigned char* aBuffer, size_t aBufferLen);

// New apis 
void alp_trace_init(unsigned long anInstance, const char* aTracerKind, const char* config);
void alp_trace_close(void);
void alp_trace_local_add(int traceModule);
void alp_trace_local_remove(int traceModule);
void alp_trace_reload(void);

void alp_trace_setoutput(const char* aTracerKind, const char* config) ;
void alp_trace_getoutput(char* aOutput,  int nBufferLen);
void alp_trace_get_capabilities(char *buffer, size_t* len);
long alp_trace_getoutputport_status(void);


/* ------------------------------------------------------------------- */
#if ALP_TRACE == ALP_TRACE_ON
	#define	ALP_TM(X)			alp_prv_trace_output##X
#else
	#define	ALP_TM(X)
#endif

/* ------------------------------------------------------------------- */
/*  Used for custom error classes	*/
#define ALP_TRACE_DEFINE(x,y) (x+y)			

#ifdef __cplusplus
}
#endif

#endif	/* __ALP_TRACES_H__ */
