/******************************************************************************
 *
 * Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the entire contents of this file.
 *
 * The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.
 *
 * Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
 * Rights Reserved.
 *
 * Contributor(s): none.
 *
 *****************************************************************************/
/**
 * @file alarm_client.c
 *
 * @brief Alarm Manager client API implementation
 *
 *****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <time.h>

#include <hiker/alarmmgr.h>
#include <hiker/prv/alarmmgr_prv.h>

#include "alarm_client_prv.h"

alp_status_t
alp_alm_set_alarm (const gchar *appID, guint32 ref, guint32 alarmSeconds)
{
	AlpAlmEntryType	entry;
	alp_status_t err;

	ALP_ATL("app %s, ref %x seconds %u", appID, ref, alarmSeconds);

	/* check for errors */
	if (!appID || strlen(appID) > ALP_ALARM_MAX_APPID_SIZE) {
        ALP_ATL ("app id <%s> is either NULL or too long", appID);
		return ALP_STATUS_ALARM_BAD_PARAM;
    }

	memset(&entry,0,sizeof(AlpAlmEntryType));

	entry.size = sizeof(AlpAlmEntryType);	

	if ( alarmSeconds ) {
        if ( alarmSeconds < (guint32) time(NULL) ) {
            ALP_ATL("alarm time %u has occurred (now %u)",
                    alarmSeconds, time(NULL));
            return ALP_STATUS_ALARM_BAD_PARAM;
        }
		entry.cmd = ALP_ALARM_CMD_SET;
		entry.triggered = 0;
		entry.notified = 0;
	} else {	
		entry.cmd = ALP_ALARM_CMD_CANCEL;
	}	

	entry.ref = ref;
	entry.alarmSeconds = alarmSeconds;
	strcpy(entry.appID, appID);

	return alp_alm_client_send_receive(&entry, entry.size, NULL, 0);

}

alp_status_t
alp_alm_get_alarm (const gchar *appID, guint32 ref, guint32 *alarmSeconds)
{
	AlpAlmEntryType	entry;
	AlpAlmEntryType	reply_entry;
	alp_status_t err;

	ALP_ATL ("app %s ref %d", appID, ref);

	/* check for errors */
	if (!appID || strlen(appID) > ALP_ALARM_MAX_APPID_SIZE) {
		return ALP_STATUS_ALARM_BAD_PARAM;
    }
    if (alarmSeconds == NULL) {
        return ALP_STATUS_ALARM_BAD_PARAM;
    }

	memset(&entry,0,sizeof(AlpAlmEntryType));

	entry.size = sizeof(AlpAlmEntryType);	


	entry.cmd = ALP_ALARM_CMD_GET;		
	entry.ref = ref;
	strcpy(entry.appID, appID);


	err = alp_alm_client_send_receive(&entry,
                                      entry.size,
                                      &reply_entry,
                                      sizeof(reply_entry));
	
	if (err == ALP_STATUS_OK)
        *alarmSeconds = reply_entry.alarmSeconds;

	ALP_ATL ("reply_entry.appID = %s", reply_entry.appID);
	ALP_ATL ("reply_entry.ref = %d", reply_entry.ref);
	ALP_ATL ("reply_entry.alarmSeconds = %d", reply_entry.alarmSeconds);

	return err;
}
