/******************************************************************************
 *
 * Copyright (C) 2006, ACCESS Systems Americas, Inc. All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the entire contents of this file.
 *
 * The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.
 *
 * Portions created by ACCESS Systems Americas, Inc. are Copyright (C) 2006.
 * All Rights Reserved.
 *
 * Contributor(s): none.
 *
 *****************************************************************************/
/**
 * @file alarm_client_ipc.c
 *
 * @brief Alarm Manager client IPC implementation
 *
 *****************************************************************************/
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

#include <glib.h>

#include <hiker/ipc.h>

#include <hiker/alarmmgr.h>
#include <hiker/prv/alarmmgr_prv.h>

#include "alarm_client_prv.h"

alp_status_t
alp_alm_client_send_receive (void *buf,
                             int length,
                             void *reply_buf,
                             size_t reply_buf_len)
{
	alp_status_t   err      = ALP_STATUS_OK;	
	AlpConnection *connect  = NULL;
	AlpMessage    *message  = NULL;
    AlpMessage    *response = NULL;

	/* establish connection to the server */
    ALP_ATL ("ipc channel %s", ALP_ALARM_MGR_SERVER_CHANNEL);
	err = alp_ipc_channel_connect_synch (
        ALP_ALARM_MGR_SERVER_CHANNEL,
        ALP_IPC_CONNECTION_USAGE_HINT_SYNCH_ONLY,
        ALP_ALARM_IPC_CONNECT_TIMEOUT,
        &connect);

	if (err != ALP_STATUS_OK) {
		ALP_ATL ("alp_ipc_channel_synch err = %x\n", err);
		goto Done;
	}

	/* create & pack the message */
	message = alp_ipc_message_create();
	if (!message) {
		ALP_ATL ("failed to create message");
		goto Done;
	}

	alp_ipc_message_pack_start (message, 0);
	alp_ipc_message_pack_byte_array (message, buf, length);
	alp_ipc_message_pack_end (message);
	
	/* send the message to the server */
	err = alp_ipc_connection_send_with_response (connect,
                                                 message,
                                                 ALP_ALARM_IPC_REQUEST_TIMEOUT,
                                                 &response);
	if (err != ALP_STATUS_OK) {
		ALP_ATL ("connection send: err %x", err);
		goto Done;
	}

    /* unpack and process the response */
	alp_ipc_message_unpack_start(response);
	alp_ipc_message_unpack_uint32(response, (guint32*)&err);

	if (reply_buf) {
		void *msg_reply_buf = NULL;
		size_t msg_reply_buf_len;

		alp_ipc_message_unpack_byte_array(response,
                                          (guchar**)&msg_reply_buf,
                                          &msg_reply_buf_len);

		if (!msg_reply_buf || (msg_reply_buf_len > reply_buf_len)) {
			err = ALP_STATUS_ALARM_IPC;
		} else {
			memcpy(reply_buf, msg_reply_buf, msg_reply_buf_len);
		}
		if (msg_reply_buf) g_free( msg_reply_buf );
	}
	alp_ipc_message_unpack_end(response);
	

Done:
	/* disconnect and cleanup */
	if (connect) alp_ipc_channel_disconnect(connect);
	if (message) alp_ipc_message_destroy(message);
    if (response) alp_ipc_message_destroy(response);

	/* return status */
    ALP_ATL ("returning with error %x", err);
	return err != ALP_STATUS_OK ? ALP_STATUS_ALARM_IPC : err;
}
