/******************************************************************************
 *
 * Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is the entire contents of this file.
 *
 * The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.
 *
 * Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
 * Rights Reserved.
 *
 * Contributor(s): none.
 *
 *****************************************************************************/
/**
 * @file alarm_client_prv.h
 *
 * @brief Private definitions for the alarm manager API client implementation.
 */
#ifndef __ALP_ALARM_CLIENT_PRV_H__
#define __ALP_ALARM_CLIENT_PRV_H__

/*
 * @cond INTERNAL
 */

#define ALP_ALARM_IPC_CONNECT_TIMEOUT  1000 /* milliseconds */
#define ALP_ALARM_IPC_REQUEST_TIMEOUT  7000 /* milliseconds */

#ifdef __cplusplus
extern "C" {
#endif

/**
 * @brief Send a alarm request to the alarm manager and wait for a response.
 *
 * @param[in]  buf            The request structure to be sent
 * @param[in]  length         The number of bytes in the request structure.
 * @param[out] reply_buf      The reply message buffer
 * @param[out] reply_buf_len  The number of bytes returned in the reply buffer.
 *
 * @return ALP_STATUS_OK        if no errors occurred
 * @return ALP_STATUS_ALARM_IPC if any other error occurred, either in IPC or
 *                              as returned by the alarm manager.
 */
extern alp_status_t
alp_alm_client_send_receive (void  *buf,
                             int    length,
                             void  *reply_buf,
                             size_t reply_buf_len);

#ifdef __cplusplus
}
#endif

/* @endcond */

#endif /* __ALP_ALARM_CLIENT_PRV_H__ */
