/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/**
 * @file attn_dml_prv.h
 *
 * @brief ALP Attention DML private definitions
 *
 ******************************************************************************/
#ifndef __ALP_ATTN_DML_PRV_H__
#define __ALP_ATTN_DML_PRV_H__

#include <hiker/syspaths.h>

/* Private strings used by attention manager DML */
#define kAttnRootDirectory                  "/"

#define kAttnDefDBDirectory                 ALP_SYSPATH_SYSTEM_CACHE "attnmgr"
#define kAttnDefDBFileName                  "attn_mgr.db"

#define kAttnResourceDirectory              ALP_SYSPATH_ETC "attnmgr"
#define kAttnSchemaResourceFileName         "AttnSchema.sql"

/* defaults for integer fields */
#define kAttnDefPriority          3     /* lowest priority */
#define kAttnDefDuration          0
#define kAttnDefUserId            65534 /* nobody */
#define kAttnDefGroupId           65534 /* nobody */
#define kAttnDefPending           true
#define kAttnDefAttributes        0     /* eventually visible,dialog,sound */
#define kAttnDefAlertLuid         0
#define kAttnDefTimestamp         0
#define kAttnDefPropType          G_TYPE_NONE

/* timestampt format */
#define kAttnTimestampFormat   "%Y-%m-%d %H:%M:%S"

/* attention table names */
#define kAttnAlertsTableName    "ALERTS"
#define kAttnPropsTableName     "PROPERTIES"

/* alert column names */
#define kAttnAlertNameAlertLuid  "AlertLUID"
#define kAttnAlertNameSource     "Source"
#define kAttnAlertNameName       "Name"
#define kAttnAlertNameHandle     "Handle"
#define kAttnAlertNamePriority   "Priority"
#define kAttnAlertNameDuration   "Duration"
#define kAttnAlertNamePending    "Pending"
#define kAttnAlertNameAttributes "Attributes"
#define kAttnAlertNameUserId     "UserID"
#define kAttnAlertNameGroupId    "GroupID"
#define kAttnAlertNameCreated    "Created"
#define kAttnAlertNameUpdated    "Updated"

#define kAttnAlertFields       kAttnAlertNameAlertLuid  ", " \
                               kAttnAlertNameSource     ", " \
                               kAttnAlertNameName       ", " \
                               kAttnAlertNameHandle     ", " \
                               kAttnAlertNamePriority   ", " \
                               kAttnAlertNameDuration   ", " \
                               kAttnAlertNamePending    ", " \
                               kAttnAlertNameAttributes ", " \
                               kAttnAlertNameUserId     ", " \
                               kAttnAlertNameGroupId    ", " \
                               kAttnAlertNameCreated    ", " \
                               kAttnAlertNameUpdated

/* alert column definitions */
#define kAttnAlertColAlertLuid  0
#define kAttnAlertColSource     1
#define kAttnAlertColName       2
#define kAttnAlertColHandle     3
#define kAttnAlertColPriority   4
#define kAttnAlertColDuration   5
#define kAttnAlertColPending    6
#define kAttnAlertColAttributes 7
#define kAttnAlertColUserId     8
#define kAttnAlertColGroupId    9
#define kAttnAlertColCreated    10
#define kAttnAlertColUpdated    11

/* property column names */
#define kAttnPropNamePropLuid  "PropLUID"
#define kAttnPropNameAlertLuid "AlertLUID"
#define kAttnPropNameName      "Name"
#define kAttnPropNameType      "Type"
#define kAttnPropNameValue     "Value"

#define kAttnPropFields        kAttnPropNamePropLuid  ", " \
                               kAttnPropNameAlertLuid ", " \
                               kAttnPropNameName      ", " \
                               kAttnPropNameType      ", " \
                               kAttnPropNameValue  

/* property column definitions */
#define kAttnPropColPropLuid    0
#define kAttnPropColAlertLuid   1
#define kAttnPropColName        2
#define kAttnPropColType        3
#define kAttnPropColValue       4


typedef struct _AlpAttnDmlStatementType AlpAttnDmlStatementType;
struct _AlpAttnDmlStatementType {
	AlpAttnDmlItemType itemType;
	AlpSALStatement    salStmtH;
};

/**
 * @Structure base type for all other item types
 */
typedef struct _AlpAttnBaseType AlpAttnBaseType;
struct _AlpAttnBaseType {
    AlpAttnDmlItemType itemType; //!< type field for this record
    AlpLuid            itemLuid; //!< LUID to reference row in associated table
};

/**
 * @Structure for attention PROPERTIES table
 */
typedef struct _AlpAttnPropType AlpAttnPropType;
struct _AlpAttnPropType {
    AlpAttnDmlItemType itemType;
    AlpLuid            itemLuid; //!< LUID to reference row in PROPERTIES table

    AlpLuid  alertLuid;       //!< LUID of alert prop is associated

    char     *name;            //!< property name string
    uint32_t  type;            //!< property type
    void     *value;           //!< property value blob
    uint32_t  size;            //!< property value size in bytes

    AlpAttnPropType *next;    //!< next property in the property list
};

/**
 * @Structure for attention PROPERTY LIST
 */
typedef struct _AlpAttnPropListType AlpAttnPropListType;
struct _AlpAttnPropListType {
    AlpAttnDmlItemType itemType;
    AlpLuid            itemLuid;

    /* identifier of alert */
    char              *source;    //!< source app id
    char              *name;      //!< alert type name
    char              *handle;    //!< string handle to refer to item

    uint32_t           count;     //!< number of properties in the list
    AlpAttnPropType   *list;      //!< property list
};

/**
 * @Structure for attention ALERT table
 */
typedef struct _AlpAttnAlertType AlpAttnAlertType;
struct _AlpAttnAlertType {
    AlpAttnDmlItemType itemType;
    AlpLuid            itemLuid; //!< LUID to reference row in ALERTS table

    /* identifier of alert */
    char    *source;          //!< alert source app id
    char    *name;            //!< alert type name
    char    *handle;          //!< string handle to refer to item

    /* alert data */
    int32_t  priority;        //!< attention manager priority
    int32_t  duration;        //!< Duration of alert, zero means no duration
    int32_t  pending;         //!< alert has yet to be seen
    uint32_t attributes;      //!< Attr controlling disposition of alert
    uid_t    userId;          //!< User Id of alert source
    gid_t    groupId;         //!< Group Id of alert source
    time_t   created;         //!< time alert was posted
    time_t   updated;         //!< time alert was posted

    uint32_t          pcount; //!< number of properties in the list
    AlpAttnPropType  *plist;  //!< list of property item associated with this alert
};

#ifdef __cplusplus
extern "C" {
#endif
    static const char *alp_attn_db_path;

    alp_status_t alp_prv_attn_dml_create_database (const char *dbpath);
    alp_status_t alp_prv_attn_dml_init_database (const char *dbpath);
    alp_status_t alp_prv_attn_dml_convert_sal_error (alp_status_t salError);
    alp_status_t alp_prv_attn_dml_validate_luid (AlpDmlH dmlH, 
                                                 AlpAttnDmlItemType itemType,
                                                 AlpLuid luid);
    char     *alp_prv_attn_dml_get_order_string (AlpAttnDmlOrderByType orderBy);
#ifdef __cplusplus
}
#endif

#endif /* __ALP_ATTN_DML_PRV_H__ */
