/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/**
 * @file attn_ipc.c
 *
 * @brief ALP attention manager IPC data routines
 *
 ******************************************************************************/

#include <stdarg.h>
#include <glib.h>

#include <hiker/types.h>
#include <hiker/prv/attnmgr_traces.h>
#include <hiker/prv/attnmgr_ipc.h>

#include "attn_ipc_prv.h"

const char *alp_prv_attn_ipc_channel = NULL;

AlpConnection *alp_prv_attn_ipc_connection;

alp_status_t
alp_attn_ipc_init (const char *channel)
{
    alp_status_t err = ALP_STATUS_ATTN_IPC_BAD_PARAM;

    ALP_ATLX(IPC, "set channel to %s (was %s)",
             channel ? channel : "<NULL>",
             alp_prv_attn_ipc_channel ? alp_prv_attn_ipc_channel : "<NULL>");
    if (channel) {
        if (alp_prv_attn_ipc_channel) {
            ALP_ATLX(IPC,
                     "channel already assigned to %s. cannot assign it to %s",
                     alp_prv_attn_ipc_channel, channel);
            err = ALP_STATUS_OK;
        } else {
            alp_prv_attn_ipc_channel = g_strdup(channel);
            err = ALP_STATUS_OK;
            ALP_ATLX(IPC, "assigning channel to %s", alp_prv_attn_ipc_channel);
        }
    } else {
        if (alp_prv_attn_ipc_channel) {
            ALP_ATLX(IPC, "unassigning channel from %s",
                     alp_prv_attn_ipc_channel);
            g_free ((char *)alp_prv_attn_ipc_channel);
            alp_prv_attn_ipc_channel = NULL;
            err = ALP_STATUS_OK;
        } else {
            alp_prv_attn_ipc_channel = g_strdup (ALP_ATTN_IPC_SERVER_CHANNEL);
            err = ALP_STATUS_OK;
            ALP_ATLX(IPC, "defaulting channel to %s",
                     alp_prv_attn_ipc_channel);
        }
    }

    return err;
}

const char *
alp_attn_ipc_get_channel (void)
{
    return alp_prv_attn_ipc_channel;
}

alp_status_t
alp_attn_ipc_setup ()
{
    alp_status_t err;

    if (alp_prv_attn_ipc_channel == NULL) {
        alp_attn_ipc_init (NULL);
    }
	err = alp_ipc_channel_connect_synch(alp_prv_attn_ipc_channel,
                                        ALP_IPC_CONNECTION_USAGE_HINT_NONE,
                                        3000,
                                        &alp_prv_attn_ipc_connection);

    return err;
}

alp_status_t
alp_attn_ipc_send (AlpMessage *msg)
{
    alp_status_t err;

    err = alp_ipc_connection_send (alp_prv_attn_ipc_connection, msg);

    return err;
}

alp_status_t
alp_attn_ipc_send_with_response (AlpMessage *msg, AlpMessage **response)
{
    alp_status_t err;

    err = alp_ipc_connection_send_with_response (alp_prv_attn_ipc_connection,
                                                 msg,
                                                 ALP_ATTN_IPC_RESPONSE_TIMEOUT,
                                                 response);
    switch (err) {    /* probably should put this in a translate routine */
    case ALP_STATUS_OK:
        break;

    case ALP_STATUS_IPC_TIMEOUT:
        err = ALP_STATUS_ATTN_IPC_TIMEOUT;
        break;

    default:
        break;
    }

    return err;
}

alp_status_t
alp_attn_ipc_teardown ()
{
    alp_status_t err;

	err = alp_ipc_channel_disconnect (alp_prv_attn_ipc_connection);

    return err;
}

