/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/**
 * @file attn_ipc_tests.c
 *
 * @brief ALP attention manager IPC unit tests
 *
 ******************************************************************************/

#include <stdlib.h>
#include <time.h>

#include <glib.h>  /* used for FALSE, TRUE */

#include <hiker/types.h>
#include <hiker/unit_test.h>
#include <hiker/prv/attnmgr_traces.h>
#include <hiker/prv/attnmgr_ipc.h>

#include "../attn_ipc_prv.h"

#define ARRAY_SIZE(x)  (sizeof(x)/sizeof(x[0]))

class AttnIPCTest: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE( AttnIPCTest );
    CPPUNIT_TEST( setup_teardown );
    CPPUNIT_TEST( packing );
    CPPUNIT_TEST_SUITE_END();

public:
    void setUp();         // overriding TestFixture method
    void tearDown();      // overriding TestFixture method

protected:
    void setup_teardown();
    void packing();

private:
};

ALP_ROOT_TEST_SUITE_REGISTRATION( "UI-007", CPPUNIT_REGISTRY_TYPE_AUTO );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( AttnIPCTest, "UI-007" );

//
// AttnDMLTest methods
//

void
AttnIPCTest::setUp()
{
}

void
AttnIPCTest::tearDown()
{
}

void
AttnIPCTest::setup_teardown(void)
{
    alp_status_t err;

    /* set */
    err = alp_attn_ipc_init ("test");
    CPPUNIT_ASSERT (err == ALP_STATUS_OK);
    CPPUNIT_ASSERT (strcmp("test", alp_prv_attn_ipc_channel) == 0);

    /* try to change */
    err = alp_attn_ipc_init ("foo");
    CPPUNIT_ASSERT (err == ALP_STATUS_OK);
    CPPUNIT_ASSERT (strcmp("test", alp_prv_attn_ipc_channel) == 0);

    /* reset */
    err = alp_attn_ipc_init (NULL);
    CPPUNIT_ASSERT (err == ALP_STATUS_OK);
    CPPUNIT_ASSERT (alp_prv_attn_ipc_channel == NULL);

    /* default */
    err = alp_attn_ipc_init (NULL);
    CPPUNIT_ASSERT (err == ALP_STATUS_OK);
    CPPUNIT_ASSERT (strcmp(ALP_ATTN_IPC_SERVER_CHANNEL, alp_prv_attn_ipc_channel) == 0);

}

void
AttnIPCTest::packing(void)
{
    CPPUNIT_ASSERT (true);
}
