/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/fail.h>
#include <hiker/ipc.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdarg.h>
#include <sqlite3.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/types.h>
#include <dirent.h>
#include <dlfcn.h>
#include <signal.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
              
#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"

static void breakup_url(char * url, char ** name, char ** path, char ** search, char ** file)
{
	char * c = url;
	if (name)
		*name = NULL;
	if (path)
		*path = NULL;
	if (search)
		*search = NULL;
	if (file)
		*file = NULL;
	
	if (strncmp(url, "file:///", 8) == 0) {
		if (file)
			*file = url+7;
		return;
	}
		
	int l = strcspn(c, ":?");

	if (url[l] == '\0') {
		if (path)
			*path = url;
		return;
	}
	
	if (url[l] == ':') {
		if (name)
			*name = url;
		c = url+l;
		l = strcspn(c, "/?")+l;
	}
	
	if (url[l] == '/') {
		if (path)
			*path = c = url + l + 1;
		url[l] = '\0';
		l = strcspn(c, "?")+l;
	}
	
	if (url[l] == '?') {
		if (search)
			*search = c = url + l + 1;
		url[l] = '\0';
	}
}

// helper for processing ^p/prop/key syntax
static gchar * translate_p(const char * old_path, AlpBundleRef ref)
{
	gchar * property = g_strdup(old_path+3);
	
	if (!property)
		return NULL;
	
	gchar * key = strchr(property, '/');
	
	if (!key) {
		g_free(property);
		return NULL;
	}
	
	*key++ = '\0';
	
	char * number = NULL;
	
	// You can also say ^p/prop#2/key
	
	if ((number = strchr(property, '#'))) {
		*number++ = '\0';
	}
	
	gchar * value = alp_bundle_property_value(alp_bundle_ref_bundle(ref), property, number ? atoi(number) : 0, key);
	
	g_free(property);
	
	return value;
}


gchar * alp_bundle_acquire_ro_pathname(const char * url, AlpBundleRef* openRef, AlpBundleRef relativeRef /* may be NULL */)
{
	char buf[PATH_MAX];
	char * name, * path, * file;
	
	if (!url) {
		name = path = file = NULL;
	} else {
		strcpy(buf, url);
	
		breakup_url(buf, &name, &path, NULL, &file);
	}
	
	if (file) {
		if (openRef)
			*openRef = NULL;
		
		return g_strdup(file);
	}
	
	AlpBundleRef ref = NULL;
	
	if (!openRef) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	if (name) {
		ref = alp_bundle_open(alp_bundle_by_name(name));
	} else {
		alp_prv_bundle_ref_incref(relativeRef);
		ref = relativeRef;
	}
	
	if (!ref) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	if (path && strncmp(path, "^p/", 3) == 0) {
		gchar * new_url = translate_p(path, ref);
		
		if (!new_url)
			goto fail;

		gchar * result = alp_bundle_acquire_ro_pathname(new_url, openRef, ref);

		alp_prv_bundle_ref_decref(ref);
		g_free(new_url);

		return result;	

	fail:
		alp_prv_bundle_ref_decref(ref);
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	// A NULL path is allowed, meaning get the root of the bundle, same as "".
	// This is particularly useful to allow a NULL URL
	gchar * result = alp_bundle_ref_ro_pathname(ref, path ?: "");
	
	if (!result) {
		alp_prv_bundle_ref_decref(ref);
		*openRef = NULL;
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	*openRef = ref;
	return result;
}

GIOChannel * alp_bundle_acquire_ro_channel(const char * url, AlpBundleRef relativeRef/* may be NULL */)
{
	char buf[PATH_MAX];
	char * name, * path, * file;
	
	if (!url) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	strcpy(buf, url);
	
	breakup_url(buf, &name, &path, NULL, &file);
	
	if (file) {
		GIOChannel * result = g_io_channel_new_file(file, "r", NULL);
		if (!result) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		return result;
	}
	
	AlpBundleRef ref = NULL;
		
	if (name) {
		ref = alp_bundle_open(alp_bundle_by_name(name));
	} else {
		alp_prv_bundle_ref_incref(relativeRef);
		ref = relativeRef;
	}
	
	if (!ref) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	if (path && strncmp(path, "^p/", 3) == 0) {
		gchar * new_url = translate_p(path, ref);
		
		if (!new_url)
			goto fail;

		GIOChannel * result = alp_bundle_acquire_ro_channel(new_url, ref);

		alp_prv_bundle_ref_decref(ref);
		g_free(new_url);

		// propagate potential error from alp_bundle_acquire_ro_channel
		return result;	

	fail:
		alp_prv_bundle_ref_decref(ref);
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	GIOChannel * result = alp_bundle_ref_ro_channel(ref, path);
	
	// the channel retains a ref if it succeeds, so we can get rid
	// of ours, now
	alp_prv_bundle_ref_decref(ref);
	
	// propagate potential error from alp_bundle_ref_ro_channel
	return result;
}

alp_status_t alp_bundle_acquire_ro_data(const char * url, gchar ** data, size_t * length, AlpBundleRef relativeRef /* may be NULL */)
{
	char buf[PATH_MAX];
	char * name, * path, * file;
	
	if (!url)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
	strcpy(buf, url);
	
	breakup_url(buf, &name, &path, NULL, &file);

	if (file) {
		gboolean result = g_file_get_contents(file, data, length, NULL);
		if (result)
			return ALP_STATUS_OK;
		else
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);	
	}
	
	AlpBundleRef ref = NULL;
		
	if (name) {
		ref = alp_bundle_open(alp_bundle_by_name(name));
	} else {
		alp_prv_bundle_ref_incref(relativeRef);
		ref = relativeRef;
	}
	
	if (!ref)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);

	if (path && strncmp(path, "^p/", 3) == 0) {
		gchar * new_url = translate_p(path, ref);
		
		if (!new_url)
			goto fail;

		alp_status_t result = alp_bundle_acquire_ro_data(new_url, data, length, ref);

		alp_prv_bundle_ref_decref(ref);
		g_free(new_url);

		return result;	

	fail:
		alp_prv_bundle_ref_decref(ref);
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}

	alp_status_t result = alp_bundle_ref_ro_data(ref, path, data, length);
	
	if (result != ALP_STATUS_OK) {
		alp_prv_bundle_ref_decref(ref);
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}

	alp_prv_bundle_ref_decref(ref);

	return result;
}

gchar * alp_bundle_acquire_ro_tempfile(const char * url, AlpBundleRef* openRef, gboolean keep, const char * ext, AlpBundleRef relativeRef /* may be NULL */)
{
	char buf[PATH_MAX];
	char * name, * path, * file;
	
	if (!url) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	strcpy(buf, url);
	
	breakup_url(buf, &name, &path, NULL, &file);
	
	if (file) {
		if (openRef)
			*openRef = NULL;
		
		return g_strdup(file);
	}
	
	AlpBundleRef ref = NULL;
	
	if (!openRef) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	if (name) {
		ref = alp_bundle_open(alp_bundle_by_name(name));
	} else {
		alp_prv_bundle_ref_incref(relativeRef);
		ref = relativeRef;
	}
	
	if (!ref) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	if (path && strncmp(path, "^p/", 3) == 0) {
		gchar * new_url = translate_p(path, ref);
		
		if (!new_url)
			goto fail;
		
		gchar * result = alp_bundle_acquire_ro_tempfile(new_url, openRef, keep, ext, ref);

		alp_prv_bundle_ref_decref(ref);
		g_free(new_url);

		// propagate potential error from alp_bundle_acquire_ro_tempfile
		return result;	

	fail:
		alp_prv_bundle_ref_decref(ref);
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	gchar * result = alp_bundle_ref_ro_tempfile(ref, path, keep, ext);
	
	if (!result) {
		alp_prv_bundle_ref_decref(ref);
		*openRef = NULL;
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	*openRef = ref;
	return result;
}


///////////// Entrypoint implementation -- this is separate from the rest of the logic, and
///////////// should be a clean consumer of the public APIs

// GClosure's returned by alp_bundle_acquire_entrypoint retain a reference to the
// package they were in, and the dynamic library that code was loaded from (in that
// order). When the closure is deref'd out of existance, it'll drop its library and
// package refs.


static void cleanup_entrypoint(gpointer data, GClosure * closure)
{
	AlpBundleEntrypointClosure * ec = (AlpBundleEntrypointClosure*)closure;
	if (ec->dlhandle)
		dlclose(ec->dlhandle);
	if (ec->ref)
		alp_prv_bundle_ref_decref(ec->ref);
	g_free(ec->path);
	g_free(ec->sym);
}


static void my_so_marshal(GClosure * closure,
		       GValue   * return_value,
		       guint	n_param_values,
		       const GValue * param_values,
		       gpointer	invocation_hint,
		       gpointer	marshal_data)
{
	typedef int (*f_t)(gint,gpointer,gpointer);
	
	f_t callback;
	GCClosure * cc = (GCClosure*)closure;
	
	g_return_if_fail (n_param_values == 3);
	
	callback = (f_t)(marshal_data ? marshal_data : cc->callback);
	
	int result = callback(g_value_get_int(param_values + 0),
			      g_value_get_pointer(param_values + 1),
			      g_value_get_pointer(param_values + 2));
	
	g_value_set_int(return_value, result);
}

static void my_exe_marshal(GClosure * closure,
		       GValue   * return_value,
		       guint	n_param_values,
		       const GValue * param_values,
		       gpointer	invocation_hint,
		       gpointer	marshal_data)
{
	typedef int (*f_t)(gint,gpointer,gpointer);
	
	f_t callback;
	AlpBundleEntrypointClosure * ec = (AlpBundleEntrypointClosure*)closure;
	
	g_return_if_fail (n_param_values == 3);
	
	int argc = g_value_get_int(param_values + 0);
	char ** argv = g_value_get_pointer(param_values + 1);
	char ** envp = g_value_get_pointer(param_values + 2);
	
	struct sigaction sa, save_quit, save_int;

	sa.sa_handler = SIG_IGN;
	sa.sa_flags = 0;
	sigemptyset(&sa.sa_mask);
	
	sigaction(SIGQUIT, &sa, &save_quit);
	sigaction(SIGINT, &sa, &save_int);	
	
	int child_pid;
	
	sigset_t chld;
	sigemptyset(&chld);
	sigaddset(&chld, SIGCHLD);

	sigprocmask(SIG_BLOCK, &chld, &chld);

	if ((child_pid = fork()) == 0) {
		
		sigaction(SIGINT, &save_int, NULL);
		sigaction(SIGQUIT, &save_quit, NULL);
		sigprocmask(SIG_SETMASK, &chld, NULL);
		
		execve(ec->path, argv, envp);
		_exit(127);
	}
	
	
	// if fork fails, then we're continuing with a child_pid == -1
	int status, result;
	while ((result = waitpid(child_pid, &status, 0)) == -1 && (errno == EINTR))
		;
	
	sigprocmask(SIG_SETMASK, &chld, NULL);
	
	sigaction(SIGINT, &save_int, NULL);
	sigaction(SIGQUIT, &save_quit, NULL);
	
	if (child_pid == -1) {
		ec->result_signal = -2;
		result = -1;
	} else if (result == -1) {
		ec->result_signal = -1;
		result = -1;
	} else if (WIFSIGNALED(status)) {
		ec->result_signal = WTERMSIG(status);
		result = -1;
	} else {
		ec->result_signal = 0;
		result = WEXITSTATUS(status);
	}
	
	g_value_set_int(return_value, result);
}

GClosure * alp_bundle_acquire_entrypoint(const char * url, AlpBundleRef relativeRef /* may be NULL */)
{
	// FIXME: what if the caller doesn't want to default to' main'? What if they
	// want C code to default to 'foozle', but Java code to default to SomeClass?
	alp_status_t status;
	
	char buf[PATH_MAX];
	char * name, * path, * search, * file;
	
	if (!url) {
		name = path = search = file = NULL;
	} else {
		strcpy(buf, url);
	
		breakup_url(buf, &name, &path, &search, &file);
	}
	
	AlpBundleRef ref = NULL;
	gchar * fullpath = NULL;
	
	// Use URL for explicit file reference, otherwise retrieve it
	// from the packages's metadata
	if (file) {
		fullpath = g_strdup(file);
	} else {		
		if (name) {
			ref = alp_bundle_open(alp_bundle_by_name(name));
		} else {
			alp_prv_bundle_ref_incref(relativeRef);
			ref = relativeRef;
		}
		
		if (!ref) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		
		if (path && path[0] != 0) {
			if (strncmp(path, "^p/", 3) == 0) {
				gchar * new_url = translate_p(path, ref);
							
				if (!new_url) {
					status = ALP_STATUS_BUNDLEMGR_FAILURE;
					goto fail;
				}

				GClosure * result = alp_bundle_acquire_entrypoint(new_url, ref);

				alp_prv_bundle_ref_decref(ref);
				g_free(new_url);

				// propagate potential error from alp_bundle_acquire_entrypoint
				return result;	
			}
			
			fullpath = alp_bundle_ref_ro_pathname(ref, path);
		} else {
			// Defaulting logic.
			// bar:com.access.Memo -> bar:com.access.Memo/Memo.so?main
			
			const gchar * package_name = alp_bundle_ref_name(ref);
			
			if (!package_name) {
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail;
			}
		
			// Transform a.b.c into c.so
			const gchar * dot = strrchr(package_name, '.');
			
			if (dot)
				dot++;
			else
				dot = package_name;
		
			gchar * libname = g_strdup_printf(BAR_LIBPATH, dot);
		
			fullpath = alp_bundle_ref_ro_pathname(ref, libname);
			
			g_free(libname);
		}
		
		if (!fullpath) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
	}

	if (strlen(fullpath) > 3 && strcmp(fullpath+strlen(fullpath)-3, ".so")==0) {
	
		char resolved[PATH_MAX];
		
		if (realpath(fullpath, resolved) == NULL) {
			strcpy(resolved, fullpath);
		}

		ALP_TMTL("Acquire entrypoint: dlopen('%s')->dlsym('%s') from url %s", resolved, search ?: BAR_DEFAULT_SEARCH, url);

		void * library_handle = dlopen(resolved, RTLD_LAZY | RTLD_GLOBAL);
	
		if (!library_handle) {
			gchar* err_msg = dlerror();
			ALP_TMTL("LINK ERROR: %s", err_msg);
			//ALP_FAIL_FATAL_IF(TRUE, "LINK ERROR: %s\n", err_msg);
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_so;
		}

			
		void * ptr = dlsym(library_handle, search ?: BAR_DEFAULT_SEARCH);
	
		if (!ptr) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_so;
		}
	
		GClosure * closure = g_closure_new_simple (sizeof (AlpBundleEntrypointClosure), NULL /*user_data*/);

		if (!closure) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_so;
		}

		g_closure_add_finalize_notifier(closure, NULL /*user_data*/, cleanup_entrypoint);

		AlpBundleEntrypointClosure * ec = (AlpBundleEntrypointClosure*)closure;
		
		ec->struct_version = ALP_BUNDLE_ENTRYPOINT_CLOSURE_VERSION;
		ec->ref = ref;
		ec->dlhandle = library_handle;
		ec->callback = ptr;
		//container was incref'd above, if a ref is being used
		ec->path = g_strdup(fullpath);
		ec->sym = g_strdup(search ?: BAR_DEFAULT_SEARCH);
				
		g_closure_set_marshal (closure, my_so_marshal);
		
		g_free(fullpath);
		
		return closure;
	
	fail_so:
		if (library_handle)
			dlclose(library_handle);
		goto fail;
	} else {
		// FIXME: uses effective IDs: is this correct?
		if (access(fullpath, X_OK) != 0) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_exe;
		}

		ALP_TMTL("Acquire entrypoint: exec('%s') from url %s\n", fullpath, url);

		GClosure * closure = g_closure_new_simple (sizeof (AlpBundleEntrypointClosure), NULL /*user_data*/);

		if (!closure) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_exe;
		}

		g_closure_add_finalize_notifier(closure, NULL /*user_data*/, cleanup_entrypoint);

		AlpBundleEntrypointClosure * ec = (AlpBundleEntrypointClosure*)closure;
		
		ec->struct_version = ALP_BUNDLE_ENTRYPOINT_CLOSURE_VERSION;
		ec->ref = ref;
		ec->dlhandle = NULL;
		ec->callback = NULL;
		//container was incref'd above, if a ref is being used
		ec->path = g_strdup(fullpath);
		ec->sym = NULL;
				
		g_closure_set_marshal (closure, my_exe_marshal);
		
		g_free(fullpath);
		
		return closure;
	
	fail_exe:
		goto fail;
	}
	
fail:
	if (ref)
		alp_prv_bundle_ref_decref(ref);
	g_free(fullpath);

	set_error(status);	
	return NULL;	
}

int alp_bundle_entrypoint_closure_execve(GClosure * closure, char * const argv[], char * const envp[])
{
	extern char ** environ;
	
	int argc=0;
	
	while (argv[argc])
		argc++;
	
	if (!envp)
		envp = environ;

        g_type_init();

	GValue return_value, params[3];

        memset(&return_value, '\0', sizeof(return_value));
        memset(&params[0], '\0', sizeof(GValue));
        memset(&params[1], '\0', sizeof(GValue));
        memset(&params[2], '\0', sizeof(GValue));

        g_value_init(&return_value, G_TYPE_INT);
        g_value_init(&params[0], G_TYPE_INT);
        g_value_init(&params[1], G_TYPE_POINTER);
        g_value_init(&params[2], G_TYPE_POINTER);

        g_value_set_int(&params[0], argc);
        g_value_set_pointer(&params[1], (void*)argv);
        g_value_set_pointer(&params[2], (void*)envp);

	g_closure_invoke(closure, &return_value, 3, params, NULL);

	int result = g_value_get_int(&return_value);
	
	g_value_unset(&return_value);
	g_value_unset(&params[0]);
	g_value_unset(&params[1]);
	g_value_unset(&params[2]);
	
	return result;
}

int alp_bundle_entrypoint_closure_execl(GClosure * closure, const char * arg, ...)
{
	GPtrArray * a = g_ptr_array_new();
	
	va_list ap;
	va_start(ap, arg);
	
	const char * p = arg;
	
	while (p) {
		g_ptr_array_add(a, (void*)p);
		
		p = va_arg(ap, char *);
	}
	
	va_end(ap);
	
	g_ptr_array_add(a, NULL);
	
	int result = alp_bundle_entrypoint_closure_execve(closure, (char * const *)(a->pdata), NULL);

	g_ptr_array_free(a, TRUE);
	
	return result;
}
