/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/notifymgr.h>
#include <hiker/ipc.h>
#include <hiker/fail.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <expat.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>


#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/mount.h>

//#include <linux/cramfs_fs.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "pm_bar.h"
#include "pm_traces.h"

#include "cramfs_interface.h"

/* expat callback material to support scanning a BAR manifest */

struct userData_t
{
	int depth;
	int state;
	int id;
	int seq_id;
	const char * property;
	const char * key;
	const char * expected_package_name;
	int refOptional;
	GString * text;
	GHashTable * currentProperty;

	GHashTable * bar_properties;
   	gchar * bar_version;
	gchar * bar_name;
	gchar * bar_searchpath;
	gchar * bar_textdomain_dir;
	gchar * bar_textdomain_name;
	
	int complete;
	XML_Parser	parser;
};

void free_list(gpointer data)
{
	GList * list = data;
	g_list_foreach(list, (GFunc)g_hash_table_destroy, NULL);
	g_list_free(list);
}                        

static void XMLCALL
startElement(void *userData_v, const char *name, const char **atts)
{
	int i;
	struct userData_t * userData = userData_v;
	
	if (userData->state == 0) {
		if (strcmp(name, "manifest") == 0) {
			userData->state = 1;
			
			const char * name = NULL;
			const char * version = NULL;
			const char * searchpath = NULL;
			const char * textdomain_name = NULL;
			const char * textdomain_dir = NULL;
			
			for (;atts[0];atts+=2) {
				if (strcmp(atts[0], "name") == 0) {
					if (userData->expected_package_name && 
						strcmp(atts[1], userData->expected_package_name) != 0) {
						ALP_TMTL("Reading package named %s, expected %s",
							atts[1], userData->expected_package_name);
                                                XML_StopParser(userData->parser, XML_FALSE);
                                                return;
					}
                                        name = atts[1];
                                } else if (strcmp(atts[0], "version") == 0) {
                                        version = atts[1];
				} else if (strcmp(atts[0], "searchpath") == 0) {
					searchpath = atts[1];
				} else if (strcmp(atts[0], "textdomain") == 0) {
					textdomain_name = atts[1];
				} else if (strcmp(atts[0], "bindtextdomain") == 0) {
					textdomain_dir = atts[1];
				}
			}
			
			if (!name) {
			  XML_StopParser(userData->parser, XML_FALSE);
			  return;
			}
			  
                        userData->bar_version = g_strdup(version);
                        userData->bar_name = g_strdup_printf("bar:%s", name);
                        userData->bar_searchpath = g_strdup(searchpath);
                        userData->bar_textdomain_name = g_strdup(textdomain_name);
                        userData->bar_textdomain_dir = g_strdup(textdomain_dir);

                        userData->bar_properties = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, NULL);

		} else {
		  /* no other root element is acceptable */
		  XML_StopParser(userData->parser, XML_FALSE);
		  return;
		}
	
	} else if (userData->state == 1 && userData->depth == 1) {
		userData->seq_id++;

		userData->currentProperty = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
		
		GList ** proplist = g_hash_table_lookup(userData->bar_properties, name);
		if (!proplist) {
			proplist = g_new0(GList*, 1);
			g_hash_table_insert(userData->bar_properties, g_strdup(name), proplist);
		}
		*proplist = g_list_prepend(*proplist, userData->currentProperty);

		userData->state = 2;
	
	} else if (userData->state == 2 && userData->depth == 2) {
		userData->key = g_strdup(name);
		userData->text = g_string_new("");
	}
	
	userData->depth++;
}

static void XMLCALL
endElement(void *userData_v, const char *name)
{
	struct userData_t * userData = userData_v;

	userData->depth--;
	
	if (userData->state == 2 && userData->depth == 2) {
	        g_hash_table_insert(userData->currentProperty, (gpointer)userData->key, g_strdup(userData->text->str));
		g_string_free(userData->text, TRUE);
		userData->text = NULL;
        } else if (userData->state == 2 && userData->depth == 1) {
		userData->state = 1;
        } else if (userData->state == 1 && userData->depth == 0) {
	        userData->complete = 1;
                userData->state = -1;
	}	
}

static void XMLCALL
Text(void * userData_v, const char * name, int len)
{
	struct userData_t * userData = userData_v;
	
	if (userData->state == 2 && userData->depth == 3 && userData->text) {
		g_string_append_len(userData->text, name, len);
	}	
}


XML_Parser setup_parser(const char * expected_package_name)
{
  struct userData_t * d = g_new0(struct userData_t, 1);
	
  memset(d, '\0', sizeof(d));

  d->expected_package_name = NULL;
		
  XML_Parser parser = XML_ParserCreate(NULL);
  XML_SetUserData(parser, d);
  XML_SetElementHandler(parser, startElement, endElement);
  XML_SetCharacterDataHandler(parser, Text);
  
  d->parser = parser;
  
  return parser;
}	

void fix(gpointer key, gpointer value, gpointer userdata)
{
	GHashTable * new_hash = userdata;
	GList ** list_p = value;
	GList * list = *list_p;
	g_free(list_p);
	
	list = g_list_reverse(list);

	g_hash_table_insert(new_hash, key, list);
}

void del(gpointer key, gpointer value, gpointer userdata)
{
	GList ** list_p = value;
	GList * list = *list_p;
	g_free(list_p);
	
	g_list_foreach(list, (GFunc)g_hash_table_destroy, NULL);
	g_list_free(list);
}

gboolean finish_parser(XML_Parser parser, GHashTable ** properties, GHashTable ** metadata)
{
  struct userData_t * d = XML_GetUserData(parser);
  
  int result = d->complete;
  
  XML_ParserFree(parser);
  
  if (properties)  {
  	*properties = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, free_list);
	
	if (d->bar_properties) {
		g_hash_table_foreach(d->bar_properties, fix, *properties);
		g_hash_table_destroy(d->bar_properties);
	}
  } else {
  	if (d->bar_properties) {
	  	g_hash_table_foreach(d->bar_properties, del, NULL);
	  	g_hash_table_destroy(d->bar_properties);
	}
  }
  
  if (metadata) {
	*metadata = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
  
	  if (d->bar_name)
		  g_hash_table_insert(*metadata, g_strdup("name"), d->bar_name);
	if (d->bar_version)
		g_hash_table_insert(*metadata, g_strdup("version"), d->bar_version);
	  if (d->bar_searchpath)
	  	g_hash_table_insert(*metadata, g_strdup("searchpath"), d->bar_searchpath);
	if (d->bar_textdomain_name)
		g_hash_table_insert(*metadata, g_strdup("textdomain"), d->bar_textdomain_name);
	if (d->bar_textdomain_dir)
		g_hash_table_insert(*metadata, g_strdup("bindtextdomain"), d->bar_textdomain_dir);
	
   }else {
	g_free(d->bar_name);
	g_free(d->bar_version);
	g_free(d->bar_searchpath);
	g_free(d->bar_textdomain_name);
	g_free(d->bar_textdomain_dir);
}	

  g_free(d);  

  return result;
    
}


/* Take a package file and check it for validity. The simple checks just look at the superblock,
   while more extensive checks run it through the cramfsck utility. */
gboolean verify_bar_file(int fd, gboolean more)
{
	// FIXME: should 'more' be boolean or integral level?
	// FIXME: SECURITY: drop privs on cramfsck, or do a security audit on it.
	
	// Check: 1. we can open the file
	//        2. we can read the superblock
	//        3. the superblock's magic is correct, including our name for the cramfs
	//        4. verify the superblock doesn't lie, and the rest of the cramfs is present
	// If 'more', then also check:
	//        5. run cramfsck to make sure the archive has internal consistentancy
	
	// FIXME: we should validate the manifest -- make sure it is present (has the correct
	// DTD?), and ideally make sure it is identical with the manifest inside the archive.
	
	struct cramfs_super super;
	
	if (lseek(fd, 0, SEEK_SET) == -1) {
		ALP_TMTL("Seek error on package file: %s", strerror(errno));
		return FALSE;
	}
	
	size_t len = read(fd, &super, sizeof(super));
	if (len < sizeof(super)) {
		ALP_TMTL("Unable to read superblock from package file");
		return FALSE;
	}
	
	if (super.magic != CRAMFS_MAGIC ||
	    memcmp(super.signature, CRAMFS_SIGNATURE, 16) != 0)
	{
		ALP_TMTL("Superblock mismatch");
	    	return FALSE;
	}

	if (memcmp(super.name, "Binder Archive 1", 16) != 0)
	{
	    	ALP_TMTL("Superblock doesn't match expected cramfs name");
	    	return FALSE;
	}

	if ((size_t)(super.root.offset << 2) >= super.size) {
		ALP_TMTL("Root of FS is outsize of FS bounds");
		return FALSE;
	}
	
	struct stat st;
	
	if (fstat(fd, &st) == -1) {
		ALP_TMTL("Unable to retrive package file size: %s\n", strerror(errno));
		return FALSE;
	}
	
	if (super.size > (size_t)st.st_size) {
		ALP_TMTL("Superblock length is longer than file length\n");
		return FALSE;
	}
	
	if (more) {
		int child_pid;
	
		sigset_t chld;
		sigemptyset(&chld);
		sigaddset(&chld, SIGCHLD);

		sigprocmask(SIG_BLOCK, &chld, &chld);
	
		// decide about SIGINT, HUP
	
		char fdname[64];
		sprintf(fdname, "/proc/self/fd/%d", fd);
		
		lseek(fd, 0, SEEK_SET);
		
		if ((child_pid = fork()) == 0) {
			int devnull=open("/dev/null", O_RDWR);
			if (devnull != 0) dup2(devnull, 0);
			if (devnull != 1) dup2(devnull, 1);
			if (devnull != 2) dup2(devnull, 2);
			int result = execlp("cramfsck", "cramfsck", fdname, NULL);
			_exit(127);
		}
	
		int status, result;
		while ((result = waitpid(child_pid, &status, 0)) == -1 && (errno == EINTR))
			;
	
		sigprocmask(SIG_SETMASK, &chld, NULL);
	
		if (result == -1) {
			ALP_TMTL("In CheckPackageFile, error waiting for child pid %d: %s", child_pid, strerror(errno));
			return FALSE; // failure
		}
	
		if (WIFSIGNALED(status)) {
			ALP_TMTL("In CheckPackageFile, child %d dies with signal %d", child_pid, WTERMSIG(status));
	 		return FALSE; // failure
		}
		
		if (WEXITSTATUS(status) != 0) {
			ALP_TMTL("cramfsck didn't like package file");
			return FALSE; // failure
		}
	}
	
	lseek(fd, 0, SEEK_SET);

	return TRUE; // success
}

