/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>

#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"

void alp_prv_bundle_dump(void)
{
	S_STMT s;

	ALP_TMTL("alp_prv_bundle_dump {");
	ALP_TMTL("----------");
	ALP_TMTL("Schemes:");
	S_EXEC("BEGIN");
	S_PREPARE(s, "SELECT scheme_id, scheme_name FROM schemes;");
	S_BEGIN_SELECT(s) {
		ALP_TMTL("SC-%d: %s", S_COL_INT(s,0), S_COL_TEXT(s, 1));
	}
	S_END_SELECT(s);
	
	ALP_TMT("----------\n");
	ALP_TMT("Stores:\n");
	S_PREPARE(s, "SELECT * FROM stores;");
	S_BEGIN_SELECT(s) {
		int i;
		ALP_TMT("\t");
		for (i=0;i < sqlite3_column_count(s);i++) {
			if (i>1)
				ALP_TMT(", ");
			ALP_TMT("%s: %s", sqlite3_column_name(s, i), S_COL_TEXT(s, i));
		}
		ALP_TMTL("");
	}
	S_END_SELECT(s);

	ALP_TMT("----------\n");
	ALP_TMT("Containers:\n");
	S_PREPARE(s, "SELECT containers.package_id, scheme_id, store_id, open_foldername, common_metadata.name, names.name, containers.stub FROM containers JOIN common_metadata ON (containers.package_id=common_metadata.package_id) LEFT OUTER JOIN names ON (containers.package_id=names.package_id);");
	S_BEGIN_SELECT(s) {
		ALP_TMT("C-%d: scheme_id %d, store_id %d, open_foldername '%s', my name '%s', official name '%s', is-stub: %d\n", S_COL_INT(s,0), S_COL_INT(s,1), S_COL_INT(s,2), S_COL_TEXT(s,3), S_COL_TEXT(s,4), S_COL_TEXT(s,5), S_COL_INT(s,6));
	
	
		S_STMT s2;
		S_PREPARE(s2, "SELECT * FROM merged_metadata WHERE package_id=?1");
		S_BIND_INT(s2, 1, S_COL_INT(s,0));
		S_BEGIN_SELECT(s2) {
			int i;
			for (i=0;i < sqlite3_column_count(s2);i++) {
				ALP_TMTL("	metadata '%s' -> '%s'", sqlite3_column_name(s2, i), S_COL_TEXT(s2, i));
			}
		}
		S_END_SELECT(s2);
	
		S_PREPARE(s2, "SELECT sequence, property FROM properties WHERE package_id=?1 AND key ISNULL AND value ISNULL");
		S_BIND_INT(s2, 1, S_COL_INT(s,0));
		S_BEGIN_SELECT(s2) {
		
			ALP_TMT("        property '%s' (%d):\n", S_COL_TEXT(s2, 1), S_COL_INT(s2, 0));
			
			S_STMT s3;
			S_PREPARE(s3, "SELECT key, value FROM properties WHERE key NOTNULL AND package_id=?1 AND sequence=?2");
			S_BIND_INT(s3, 1, S_COL_INT(s,0));
			S_BIND_INT(s3, 2, S_COL_INT(s2, 0));
			S_BEGIN_SELECT(s3) {
				ALP_TMT("	        '%s' -> '%s'\n", S_COL_TEXT(s3, 0), S_COL_TEXT(s3, 1));
			}
			S_END_SELECT(s3);
			
		}
		S_END_SELECT(s2);
	
		if (S_COL_INT(s, 1) == 1) {	//BAR
			ALP_TMT("	References:\n");
			S_PREPARE(s2, "SELECT refer_to,optional FROM bar_references WHERE package_id = ?1");
			S_BIND_INT(s2, 1, S_COL_INT(s,0));
			S_BEGIN_SELECT(s2) {
				ALP_TMTL("		-> '%s' (optional=%d)", S_COL_TEXT(s2, 0), S_COL_INT(s2, 1));
			}
			S_END_SELECT(s2);

			ALP_TMT("	Exports:\n");
			S_PREPARE(s2, "SELECT path,name,kind FROM bar_exports WHERE package_id = ?1");
			S_BIND_INT(s2, 1, S_COL_INT(s,0));
			S_BEGIN_SELECT(s2) {
				ALP_TMTL("		%s: %s (%s)", S_COL_TEXT(s2, 2), S_COL_TEXT(s2, 0), (const char*)S_COL_TEXT(s2, 1) ?: "none");
			}
			S_END_SELECT(s2);
		}
	
	}
	S_END_SELECT(s);

	ALP_TMT("----------\n");
	ALP_TMT("Names:\n");
	S_PREPARE(s, "SELECT name, package_id FROM names");
	S_BEGIN_SELECT(s) {
		ALP_TMTL("	name '%s' -> C-%d", S_COL_TEXT(s, 0), S_COL_INT(s, 1));
	}
	S_END_SELECT(s);
	ALP_TMT("GHost:\n");
	S_PREPARE(s, "SELECT package_id, ghost_db_name, ghost_db_type, ghost_db_creator, ghost_db_type  FROM ghost_metadata");
	S_BEGIN_SELECT(s) {
		ALP_TMTL("	dbname '%s' creator '%s' type '%s' package_id->%d\n",  S_COL_TEXT(s, 1), S_COL_TEXT(s, 3), S_COL_TEXT(s, 2), S_COL_INT(s, 0));		
	}
	S_END_SELECT(s);
	
	ALP_TMT("JAVA:\n");
	S_PREPARE(s, "SELECT package_id, midlet_name, midlet_id FROM java_metadata");
	S_BEGIN_SELECT(s) {
		ALP_TMTL("	midlet_name '%s' midlet_id '%s' type '%s' package_id->%d\n",  S_COL_TEXT(s, 1), S_COL_INT(s, 2),  S_COL_INT(s, 0));		
	}
	S_END_SELECT(s);
	
	S_EXEC("COMMIT");
	ALP_TMT("}\n");
}


