#include <unistd.h>
#include <signal.h>

#include <alp/pdk/gras.h>

#include "pm_globals.h"
#include "ghost_support.h"

//#include "pm_bar.h"
#include "pm_traces.h"

void * gGhostSession = 0;

int gGHost = 0; // presence of GHost, server client. Clients just assume

int ghost_available(void)
{
	return alp_gras_is_installed();
}

alp_status_t ghost_start(void)
{
	int err;
	
    if (pm_status == STATUS_SERVER && !gGHost) {
      ALP_TMTL("Somebody tried a GHost operation in the server without ghost_available()");
    }

	// Cancel any pending stop
	alarm(0);
	if (gGhostSession != 0)
		return ALP_STATUS_OK;
		
	err = alp_gras_open_local_session(ALP_GRAS_SESSION_TYPE_DB, 0, 1, &gGhostSession);
	if (err != ALP_STATUS_OK) 
	{
		ALP_TMTL("alp_gras_open_local_session failed (err=0x%x)", err);
		return err;
	}
	
	return ALP_STATUS_OK;
}

static void SigAlarmHandler(int signum)
{
	void* session = gGhostSession;
	gGhostSession = NULL;
	int ret = alp_gras_close_session(session, 0, ALP_GRAS_CLOSE_DELAY_NORMAL);
	if (ret != 0) 
		ALP_TMTL("Closing ghost session failed: %d", ret);
}

alp_status_t ghost_stop(int now)
{
	static int gSigHandlerSet = 0;
	
	if (!gGhostSession)
		return -1;
	
	if (now)
	{
		int ret = alp_gras_close_session(gGhostSession, 0, ALP_GRAS_CLOSE_DELAY_NORMAL);
		gGhostSession = NULL;
		if (ret != 0) 
		{
			ALP_TMTL("Closing ghost session failed: %d", ret);
			return -1;
		}
		return ALP_STATUS_OK;
	}
	
	if (!gSigHandlerSet)
	{
		signal(SIGALRM, SigAlarmHandler);
		gSigHandlerSet = 1;
	}
	
	alarm(ALP_GRAS_QUIT_DELAY_SEC);
	return ALP_STATUS_OK;
}
