/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/types.h>
#include <unistd.h>
//#include <linux/cramfs_fs.h>
#include <signal.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/sendfile.h>
#include <alp/pdk/gras.h>    
#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"
#include "pm_bar.h"
#include "pm_msg.h"
#include "pm_perms.h"

#include "ghost_support.h"
#include "java_support.h"

#include "cramfs_interface.h"

#define SIZE 65536

// FIXME: this is just bar for the moment, and both filename and mimetype are ignored
// FIXME: basic bug: progress not invoked
int alp_bundle_image_verify(const char * provide_filename, const char * provide_mimetype, GIOChannel * image, gboolean full_check, GHashTable ** metadata, GHashTable ** properties, AlpBundleProgressiveData * progress)
{
	char tmp[PATH_MAX];
	alp_status_t status;
	
	sprintf(tmp, "%s/tmp/alp-image.XXXXXX", getenv("ALP_RUN_TOP") ?: "");
	
	gchar * buffer = NULL;
	int fd = mkstemp(tmp);
	XML_Parser parser = NULL;
	
	if (fd == -1) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	buffer = g_malloc(SIZE);
	if (!buffer) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	size_t len = 0;
	for (;;) {
		gsize bytes = 0;
		GIOStatus result = g_io_channel_read_chars(image, buffer, SIZE, &bytes, NULL);
		if (bytes > 0) {
			int w = write(fd, buffer, bytes);
			if (w < (int)bytes) {
				ALP_TMTL("Problem streaming in image for verification: tried to write %d bytes to %s, only wrote %d",
					bytes, tmp, w);
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail;
			}
			len += bytes;
		}
		if (bytes == 0 && (result == G_IO_STATUS_NORMAL || result == G_IO_STATUS_EOF))
			break;
			
		if (result != G_IO_STATUS_NORMAL) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
	}
	
	gboolean verification = verify_bar_file(fd, full_check);

	if (verification == FALSE) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	struct cramfs_super super;

	len = read(fd, &super, sizeof(super));
	if (len < sizeof(super)) {
		ALP_TMTL("Unable to read super block from package file");
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	parser = setup_parser(NULL);

	size_t pos = sizeof(super);
	size_t end = super.root.offset << 2;
	
	// pull the manifest out of the gap in the cramfs image, feeding it to the bar xml parser
	while (pos < end) {
		len = end - pos;
		void * buf = XML_GetBuffer(parser, SIZE);
		
		int r = read(fd, buf, (len > SIZE) ? SIZE : len);
		
		if (r < 0) {
			ALP_TMTL("Error reading manifest");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		int done = (r == 0);
		
		if (!done) {
			char * term = memchr((char*)buf, '\0', r);
			if (term) {
				done = 1;
				r = term - (char*)buf;
			}
			pos += r;
			if (pos >= end)
				done = 1;
		}
		
		if (!XML_ParseBuffer(parser, r, done)) {
			ALP_TMTL("Manifest parser aborted with failure");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		
		if (done)
			break;
	}
	
	// finish up parsing, delivering results to caller
	
	// FIXME: if we have a failed parse, could there be data in properties or metadata? If so,
	// we oughn't to fill in the parameters until success is known -- don't forget to deallocate
	// if needed during cleanup
	gboolean parse_result = finish_parser(parser, properties, metadata);
	parser = NULL;
	
	if (!parse_result) {
		ALP_TMTL("Verification failed after manifest parsing");
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}

	close(fd);
	unlink(tmp);
	
	// no problems detected
	return ALP_STATUS_OK;
fail:
	if (fd != -1)
		close(fd);

	g_free(buffer);
	unlink(tmp);
	
	if (parser)
		finish_parser(parser, NULL, NULL);
		
	return set_error(status);
}

static void free_list(gpointer data)
{
	GList * list = data;
	g_list_foreach(list, (GFunc)g_hash_table_destroy, NULL);
	g_list_free(list);
}                        


/* Notice of grotesqueries: image_install needs to run in the server to do a lot of it's magic,
 * however the progress callback needs to be updated in the client, likewise invocation of the
 * callback functions on the input GIOChannel. To actually accomplish this will be messy, involving
 * a backchannel from the server to the client passing updates, and possibly going both ways to
 * pull new data from the GIOChannel. (An optimization would be to detect the channel is just an
 * fd, and pass the fd directly to the server).
 *
 * The internal alp_prv_bundle_image_install_fd() routine only exists to serve a limited version
 * of this code which doesn't know how to invoke progress callbacks, and copies the entire input
 * channel to a temp file before working on it. 
 */

// FIXME: basic bug: progress not invoked
AlpBundle alp_bundle_image_install(const char * provide_filename, const char * provide_mimetype, GIOChannel * image, const char * destination_store, int flags, GHashTable ** metadata_p, GHashTable ** properties_p,  AlpBundleProgressiveData * progress)
{
	char tmp[PATH_MAX];
	
	alp_status_t status;
	
	// FIXME: stream directly from channel, instead of loading to a temp file
	sprintf(tmp, "%s/tmp/alp-image.XXXXXX", getenv("ALP_RUN_TOP") ?: "");
	
	gchar * buffer = NULL;
	int fd = mkstemp(tmp);
	XML_Parser parser = NULL;
	
	if (fd == -1) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	buffer = g_malloc(SIZE);
	if (!buffer) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	size_t len = 0;
	for (;;) {
		gsize bytes = 0;
		GIOStatus result = g_io_channel_read_chars(image, buffer, SIZE, &bytes, NULL);
		if (bytes > 0) {
			int w = write(fd, buffer, bytes);
			if (w < (int)bytes) {
				ALP_TMTL("Problem streaming in image for installation: tried to write %d bytes to %s, only wrote %d",
					bytes, tmp, w);
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail;
			}
			len += bytes;
		}
		if (bytes == 0 && (result == G_IO_STATUS_NORMAL || result == G_IO_STATUS_EOF))
			break;
			
		if (result != G_IO_STATUS_NORMAL) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
	}
	g_free(buffer);
	
	AlpBundle result;

	if (pm_status == STATUS_CLIENT) {
	
		AlpMessage * message, *response;
			
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PRV_BUNDLE_IMAGE_INSTALL_TMPNAME);
		alp_ipc_message_pack_string(message, provide_filename);
		alp_ipc_message_pack_string(message, provide_mimetype);
		alp_ipc_message_pack_string(message, tmp); // filename, not fd
		alp_ipc_message_pack_string(message, destination_store);
		alp_ipc_message_pack_int32(message, flags);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			ALP_TMTL("some failure of IPC");
			// some failure of IPC
			result = ALP_BUNDLE_NULL;
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		alp_ipc_message_destroy(message);
		
		int i,j,k;
		int i_max,j_max,k_max;
		
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result.value);
		alp_ipc_message_unpack_int32(response, &status);
		
		// metadata key/value pair count
		alp_ipc_message_unpack_int32(response, &i_max);
		
		if (metadata_p)
			*metadata_p = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);

		for (i=0;i<i_max;i++) {
			gchar * metadata_key, * metadata_value;
			alp_ipc_message_unpack_string(response, &metadata_key);
			alp_ipc_message_unpack_string(response, &metadata_value);
			if (metadata_p)
				g_hash_table_insert(*metadata_p, metadata_key, metadata_value);
		}

		// property key count
		alp_ipc_message_unpack_int32(response, &i_max);

		if (i_max != -1) {

			GHashTable * all_properties = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, free_list);
			
			for (i=0;i<i_max;i++) {
				gchar * property_name;
				GList * sequence = NULL;
			
				alp_ipc_message_unpack_string(response, &property_name);
				
				// sequence count
				alp_ipc_message_unpack_int32(response, &j_max);

				for (j=0;j<j_max;j++) {
					// property key/value pair count
					alp_ipc_message_unpack_int32(response, &k_max);
					
					GHashTable * property = NULL;
					if (k_max != -1) {
						property = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);

						for (k=0;k<k_max;k++) {					

							gchar * property_key, * property_value;
							alp_ipc_message_unpack_string(response, &property_key);
							alp_ipc_message_unpack_string(response, &property_value);
							g_hash_table_insert(property, property_key, property_value);
						}
					}
							
					sequence = g_list_append(sequence, property);
				}

				g_hash_table_insert(all_properties, property_name, sequence);
			}
			
			if (properties_p)
				*properties_p = all_properties;
			else
				g_hash_table_destroy(all_properties); // oops


		} else {
			if (properties_p)
				*properties_p = NULL;
		}
		
		alp_ipc_message_unpack_end(response);

		alp_ipc_message_destroy(response);
		
		set_error(status);

	} else if (pm_status == STATUS_SERVER) {
		result = alp_prv_bundle_image_install_fd(provide_filename, provide_mimetype, fd, destination_store, flags, metadata_p, properties_p,  progress, NULL);
	}


	close(fd);
	unlink(tmp);
	
	// propagate potential set_error()
	return result;
fail:

	close(fd);
	unlink(tmp);

	set_error(status);
	return ALP_BUNDLE_NULL;
}

static void f1(gpointer key, gpointer value, gpointer user_data)
{
	alp_ipc_message_pack_string((AlpMessage*)user_data, (gchar*)key);
	alp_ipc_message_pack_string((AlpMessage*)user_data, (gchar*)value);
}

alp_status_t my_alp_ipc_message_pack_GHashTable_str_str(AlpMessage * response, GHashTable * h)
{
	if (!h)
		alp_ipc_message_pack_int32(response, -1);
	else {
		alp_ipc_message_pack_int32(response, g_hash_table_size(h));
		g_hash_table_foreach(h, f1, response);
	}

	return 0;
}

static void f2(gpointer key, gpointer value, gpointer user_data)
{
	alp_ipc_message_pack_string((AlpMessage*)user_data, (gchar*)key);
	GList * list = value;
	
	alp_ipc_message_pack_int32((AlpMessage*)user_data, g_list_length(list));
	
	GList * iter = list;
	
	for (iter = list; iter; iter=g_list_next(iter)) {
		my_alp_ipc_message_pack_GHashTable_str_str((AlpMessage*)user_data, (GHashTable*)(iter->data));
	}
}

alp_status_t my_alp_ipc_message_pack_GHashTable_str_list_GHashTable_str_str(AlpMessage * response, GHashTable * h)
{
	if (!h)
		alp_ipc_message_pack_int32(response, -1);
	else {
		alp_ipc_message_pack_int32(response, g_hash_table_size(h));
		g_hash_table_foreach(h, f2, response);
	}

	return 0;
}

void handle_client_alp_prv_bundle_image_install_tmpname(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	gchar * provide_filename, * provide_mimetype, *tmp, *destination_store;
	int flags;
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_string(message, &provide_filename);
	alp_ipc_message_unpack_string(message, &provide_mimetype);
	alp_ipc_message_unpack_string(message, &tmp); // filename, not fd
	alp_ipc_message_unpack_string(message, &destination_store);
	alp_ipc_message_unpack_int32(message, &flags);
	alp_ipc_message_unpack_end(message);
			
	AlpBundle result = {0};

	GHashTable * metadata = NULL;
	GHashTable * properties = NULL;
	
	int fd = open(tmp, O_RDONLY);
	if (fd == -1) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		result = ALP_BUNDLE_NULL;
	} else {
		// the routine checks permissions itself
		result = alp_prv_bundle_image_install_fd(provide_filename, provide_mimetype, fd, destination_store, flags, &metadata, &properties, NULL /* progress */, connection);
		close(fd);
	}
	
	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_MOVE_TO_STORE_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result.value); 
	alp_ipc_message_pack_int32(response, result.value ? ALP_STATUS_OK : alp_bundle_error());
	my_alp_ipc_message_pack_GHashTable_str_str(response, metadata);
	my_alp_ipc_message_pack_GHashTable_str_list_GHashTable_str_str(response, properties);
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

AlpBundle alp_prv_ghost_image_install(const char * provide_filename, const char * provide_mimetype, int fd, const char * destination_store, int flags, GHashTable ** metadata_p, GHashTable ** properties_p,  AlpBundleProgressiveData * progress, AlpConnection * connection)
{
	AlpBundle package = {0};
	int			ret = 0, overwrite = 1, needReset;
	uint32_t	type, creator;
	char		db_name[PATH_MAX]; 
	char        bundle_name[PATH_MAX];
	char 		path[PATH_MAX], tmp[PATH_MAX];
	
	// identify the folder going with a store: if no store was supplied, pick one
	S_STMT s;
	int ghost_storage;
	gchar * store_name = NULL;
	if (store_name) {
		S_PREPARE(s, "SELECT ghost_storage FROM stores WHERE store_name = ?1");
		S_BIND_TEXT(s, 1, destination_store);

		if (!S_STEP_ROW(s)) {
			S_FINALIZE(s);
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return ALP_BUNDLE_NULL;
		}
		ghost_storage = S_COL_INT(s,0);
		store_name = g_strdup(store_name);
		S_FINALIZE(s);
	} else {
		S_PREPARE(s, "SELECT ghost_storage,store_name FROM stores WHERE ghost_storage NOTNULL AND (is_rom ISNULL OR NOT is_rom) ORDER BY install_priority DESC, store_id ASC LIMIT 1");
		
		if (!S_STEP_ROW(s)) {
			S_FINALIZE(s);
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return ALP_BUNDLE_NULL;
		}
		ghost_storage = S_COL_INT(s,0);
		store_name = g_strdup(S_COL_TEXT(s,1));
		ALP_TMTL("Ghost image install picked storage %d", ghost_storage);
		S_FINALIZE(s);
	}
	
	if (ghost_storage != 0) {
		ALP_TMTL("Ghost image install only allowed onto storage 0");
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		g_free(store_name);
		return ALP_BUNDLE_NULL;
	}

	sprintf(path, "/proc/self/fd/%d", fd);

	ret = -1;
	if (ghost_start() == ALP_STATUS_OK)
	{
		ret = alp_gras_db_import(GSESSION, path, (flags &  ALP_BUNDLE_FLAG_OVERWRITE_DUPLICATE) ? 1 : 0, db_name, &type, &creator, &needReset);
		ghost_stop(ret != ALP_STATUS_OK && ALP_STATUS_VALUE(ret) != ALP_STATUS_GRAS_GARNET_ERROR);
	}
	
	if(ret != 0 ) {
		g_free(store_name);
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_NULL ;	
	}
	
	char type_str[5], creator_str[5];
	
	sprintf(bundle_name,"%s:%d_%s_%s_%s", BUNDLE_SCHEME_NAME_GHOST, ghost_storage, int_to_char4(type, type_str),
		int_to_char4(creator, creator_str), db_name);
			
	// GHost will immediately queue up a dbchange notification, which we will put in our idle queue,
	// and process after the install is complete.
	package = alp_prv_bundle_register_bundle(bundle_name, BUNDLE_SCHEME_NAME_GHOST, store_name, 0);
	
	g_free(store_name);
				
	ALP_TMTL(	"GHost installation gave us:\n  name:       [%s]\n"
			"  type:       '%c%c%c%c'\n"
			"  creator:    '%c%c%c%c'\n"
			"  need reset: %s\n"
		, db_name
		, ((char*)&type)[3], ((char*)&type)[2], ((char*)&type)[1], ((char*)&type)[0]
		, ((char*)&creator)[3], ((char*)&creator)[2], ((char*)&creator)[1], ((char*)&creator)[0]
		, needReset?"Yes":"No"
	);
			
	return package;
}

AlpBundle alp_prv_java_image_install(const char * provide_filename, const char * provide_mimetype, int fd, const char * destination_store, int flags, GHashTable ** metadata_p, GHashTable ** properties_p,  AlpBundleProgressiveData * progress, AlpConnection * connection)
{
	AlpBundle package = {0};
	char	path[PATH_MAX], tmp[PATH_MAX];
	char	bundle_name[PATH_MAX];

	if (!provide_filename) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_NULL;
	}

	// identify the folder going with a store: if no store was supplied, pick one
	S_STMT s;
	int jam_store;
	
	gchar * store_name = NULL;
	if (destination_store) {
		S_PREPARE(s, "SELECT 1 FROM stores WHERE store_name = ?1 AND jam_store NOTNULL");
		S_BIND_TEXT(s, 1, destination_store);
	
		if (!S_STEP_ROW(s)) {
			S_FINALIZE(s);
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return ALP_BUNDLE_NULL;
		}
		store_name = g_strdup(destination_store);
		S_FINALIZE(s);
		
	} else {
		S_PREPARE(s, "SELECT store_name FROM stores WHERE jam_store NOTNULL ORDER BY install_priority DESC");
		
		if (!S_STEP_ROW(s)) {
			S_FINALIZE(s);
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return ALP_BUNDLE_NULL;
		}
		store_name = g_strdup(S_COL_TEXT(s,0));
		
		ALP_TMTL("Java image install picked storage %d", store_name);
		S_FINALIZE(s);
	}
	
	//call jvlite --install depending upon the mime type 
	sprintf(path, "/proc/self/fd/%d", fd);
	gchar* midlet_name;
	int   midlet_id;
	int jar = -1;
	int ret = -1;
	char *update_url = NULL;
	int len = strlen(provide_filename);
	
	if (len >= 4 ) {
		if(strcasecmp(provide_filename + len - 4, ".jad") == 0) {
			ALP_TMTL("Installing jad File \n");
			jar = 0; 
			
		} else if (strcasecmp(provide_filename + len - 4, ".jar") == 0) {
			ALP_TMTL("Installing jar File \n");
			jar = 1 ;
		}
	} 
	if(-1 != jar) {
		ret = java_install(jar, 0, provide_filename, update_url, &midlet_id, &midlet_name);
		if(update_url) {
			g_free(update_url);
		}
		if(0 == ret ) {
			sprintf(bundle_name, "java:%d_%s",  midlet_id, midlet_name);
			package = alp_prv_bundle_register_bundle(bundle_name, BUNDLE_SCHEME_NAME_JAVA, store_name, 0);
			g_free(midlet_name);
		}
	} else {
		ALP_TMTL("unknown file type\n");
	}
	
	g_free(store_name);
	
	if (alp_bundle_is_NULL(package)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_NULL;
	}

	return package; 
}


// FIXME: basic bug: progress not invoked
AlpBundle alp_prv_bundle_image_install_fd(const char * provide_filename, const char * provide_mimetype, int fd, const char * destination_store, int flags, GHashTable ** metadata_p, GHashTable ** properties_p,  AlpBundleProgressiveData * progress, AlpConnection * connection)
{
	// FIXME: we should attempt to identify from contents, not just filename
	
	gchar * id_scheme;
	 
	if ((id_scheme = alp_bundle_scheme_identify(provide_filename, provide_mimetype))) {
		if (strcmp(id_scheme, BUNDLE_SCHEME_NAME_GHOST) == 0) {
			g_free(id_scheme);
			return alp_prv_ghost_image_install(provide_filename, provide_mimetype, fd, destination_store, 
							flags, metadata_p, properties_p,  progress, connection);
		}else if (strcmp(id_scheme, BUNDLE_SCHEME_NAME_JAVA) == 0) {
			g_free(id_scheme);
			return alp_prv_java_image_install(provide_filename, provide_mimetype, fd, destination_store, 
							flags, metadata_p, properties_p,  progress, connection);	
		}
		g_free(id_scheme);
	}

	alp_status_t status;
	
	static int sequence;
	
	GHashTable * metadata = NULL;
	GHashTable * properties = NULL;

	int bar_prefer_archive;
	gchar * bar_folder = NULL;
	gchar * store_name = g_strdup(destination_store);
	
	XML_Parser parser = NULL;
	
	size_t image_len = lseek(fd, 0, SEEK_END);
	lseek(fd, 0, SEEK_SET);
	
	gboolean verification = verify_bar_file(fd, FALSE);

	if (verification == FALSE) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
       
	struct cramfs_super super;

	size_t len = read(fd, &super, sizeof(super));
	if (len < sizeof(super)) {
		ALP_TMTL("Unable to read super block from package file");
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}

	parser = setup_parser(NULL);

	size_t pos = sizeof(super);
	size_t end = super.root.offset << 2;
	
	// pull the manifest out of the gap in the cramfs image, feeding it to the bar xml parser
	while (pos < end) {
		len = end - pos;
		void * buf = XML_GetBuffer(parser, SIZE);
		
		int r = read(fd, buf, (len > SIZE) ? SIZE : len);
		
		if (r < 0) {
			ALP_TMTL("Error reading manifest");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		int done = (r == 0);
		
		if (!done) {
			char * term = memchr((char*)buf, '\0', r);
			if (term) {
				done = 1;
				r = term - (char*)buf;
			}
			pos += r;
			if (pos >= end)
				done = 1;
		}
		
		if (!XML_ParseBuffer(parser, r, done)) {
			ALP_TMTL("Manifest parser aborted with failure");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		
		if (done)
			break;
	}
	
	// finish up parsing, delivering results to caller
	
	gboolean parse_result = finish_parser(parser, &properties, &metadata);
	parser = NULL;
	
	if (!parse_result) {
		ALP_TMTL("Verification failed after manifest parsing");
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	

	// identify the folder going with a store: if no store was supplied, pick one
	S_STMT s;
	int store_id;
	if (store_name) {
		S_PREPARE(s, "SELECT bar_folder,bar_prefer_archive,store_id FROM stores WHERE store_name = ?1");
		S_BIND_TEXT(s, 1, store_name);

		if (!S_STEP_ROW(s)) {
			S_FINALIZE(s);
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		bar_folder = g_strdup(S_COL_TEXT(s,0));
		bar_prefer_archive = S_COL_INT(s,1);
		store_id = S_COL_INT(s,2);
		S_FINALIZE(s);

	} else {
		S_PREPARE(s, "SELECT store_name,bar_folder,bar_prefer_archive,store_id FROM stores WHERE is_rom ISNULL OR NOT is_rom ORDER BY install_priority DESC, store_id ASC LIMIT 1");
		
		if (!S_STEP_ROW(s)) {
			S_FINALIZE(s);
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		store_name = g_strdup(S_COL_TEXT(s,0));
		bar_folder = g_strdup(S_COL_TEXT(s,1));
		bar_prefer_archive = S_COL_INT(s,2);
		store_id = S_COL_INT(s,3);
		ALP_TMTL("Image install picked store named %s, at %s, archive=%d", store_name, bar_folder, bar_prefer_archive);
		S_FINALIZE(s);
	}
	
	// Now that we've picked a store, if we are being called by a client, check authority
	if (connection) {
		gboolean permission = has_store_permission_by_store(connection, PERM_INSTALL_POLICY_COLUMN, store_name);
		if (!permission) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
	}


	char newfolder[PATH_MAX];
	char tmpname[PATH_MAX];
	
	strcpy(newfolder, bar_folder);
	mkdir_p(newfolder, 0777);
	
	ALP_TMTL("newfolder=%s", newfolder);
	
	char * name = g_hash_table_lookup(metadata, "name");

	//Check for duplicate name
	
	S_PREPARE(s, "SELECT containers.package_id FROM containers JOIN common_metadata ON (containers.package_id=common_metadata.package_id) WHERE containers.store_id = ?1 AND common_metadata.name = ?2");
	S_BIND_INT(s, 1, store_id);
	S_BIND_TEXT(s, 2, name);
	
	int delete_original = 0;
	
	if (S_STEP_ROW(s)) {
		// Bundle of same name found
		if (flags & ALP_BUNDLE_FLAG_OVERWRITE_DUPLICATE) {
			// we won't delete the original until we know that the new one is successfully registered
			delete_original = S_COL_INT(s, 0);
			S_FINALIZE(s);
		} else {
			S_FINALIZE(s);
			// There is a collision, so image install isn't possible onto this store
			ALP_TMTL("Collision, bundle of same name %s already registered", name);
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}		
	} else {
		S_FINALIZE(s);
	}	

	// FIXME: should we use a temp-file generation mechanism? Would it help?
        // FIXME: crude, +4 skips 'bar:'
	sprintf(tmpname, "%s/TMP-%s-%d-%u", newfolder, name ? (name+4) : "no-name", getpid(), sequence++);
	ALP_TMTL("tempname=%s", tmpname);
	
	struct stat stbuf;
	
        // FIXME: crude, +4 skips 'bar:'
	sprintf(newfolder + strlen(newfolder), "/%s", name ? (name+4) : "no-name");
	
	if (stat(newfolder, &stbuf) == 0) {
		int conflict=2;
		
		for (;;) {
		        // FIXME: crude, +4 skips 'bar:'
			sprintf(newfolder + strlen(newfolder), "/%s-%d", name ? (name+4) : "no-name", conflict++);
			ALP_TMTL("Conflict installing bundle named %s, trying name %s", name, newfolder);
			if (stat(newfolder, &stbuf) == -1)
				break;
		}
	}
	
	// FIXME: basic logic problem: we're creating a new name, but don't have exclusive access, so we
	// aren't proving there won't be a race condition. Unfortunately, we can't create the file ourselves,
	// as cramfsck won't unpack to an existing folder name.

	ALP_TMTL("newfolder=%s", newfolder);	
	
	// the choice is up to the store whether the archive should be unpacked into a folder
	// representation, or left packed as a flattened image.
	if (bar_prefer_archive) {

		off_t offset;
		
		//FIXME: if we copy the flattened image, someone needs to check to make sure that the mounted
		//folder contains an identical Manifest.xml to that stored in the gap  
	
		// FIXME: consider perms on new file
		int dest = open(tmpname, O_WRONLY|O_CREAT, 0666);
		if (dest == -1) {
			ALP_TMTL("Unable to open destination archive %s: %s", tmpname, strerror(errno));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		
		lseek(fd, 0, SEEK_SET);
		
		if (sendfile(dest, fd, &offset, image_len) == -1) {
			ALP_TMTL("Failure copying archive with sendfile: %s", strerror(errno));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		
	} else {
	
		char tmpbuf[64];
		sprintf(tmpbuf, "/proc/self/fd/%d", fd);
	
		int child_pid;
		
		sigset_t chld;
		sigemptyset(&chld);
		sigaddset(&chld, SIGCHLD);
	
		sigprocmask(SIG_BLOCK, &chld, &chld);
		
		// decide about SIGINT, HUP
		
		if ((child_pid = fork()) == 0) {
			fcntl(fd, F_SETFD, 0);
		
			int devnull=open("/dev/null", O_RDWR);
			if (devnull != 0) dup2(devnull, 0);
			if (devnull != 1) dup2(devnull, 1);
			if (devnull != 2) dup2(devnull, 2);
			
			int result = execlp("cramfsck", "cramfsck", "-i", "-x", tmpname, tmpbuf, NULL);
			_exit(127);
		}
			
		int child_status, result;
		while ((result = waitpid(child_pid, &child_status, 0)) == -1 && (errno == EINTR))
			;
		
		sigprocmask(SIG_SETMASK, &chld, NULL);
			
		if (result == -1) {
			ALP_TMTL("error waiting for child pid %d: %s\n", child_pid, strerror(errno));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		
		if (WIFSIGNALED(child_status)) {
	 		ALP_TMTL("child %d dies with signal %d\n", child_pid, WTERMSIG(child_status));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
	 		goto fail;
		}
	}
	
	// move temporary file/folder to real name
	if (rename(tmpname, newfolder) == -1) {
		ALP_TMTL("Error renaming temp install location '%s' to destination '%s': %s", tmpname, newfolder, strerror(errno));
		if (bar_prefer_archive) {
			unlink(tmpname);
		} else {
			rmdir_contents(tmpname);
			rmdir(tmpname);
		}
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	//FIXME: great big security hole: we shouldn't just send a 'register' message to the
	//server. Instead the install request needs to go through the server... this will be
	//messy.

	AlpBundle package = {0};
	
	ALP_TMTL("Registering new bundle located at %s", newfolder);
	package = alp_prv_bundle_register_bundle(newfolder, BUNDLE_SCHEME_NAME_BAR, store_name, (delete_original != 0) ? ALP_PRV_BUNDLE_FLAG_DO_NOT_NOTIFY : 0);
	
	if (alp_bundle_is_NULL(package)) {
		ALP_TMTL("Unable to register installed package, removing working material");
		if (bar_prefer_archive) {
			unlink(newfolder);
		} else {
			rmdir_contents(newfolder);
			rmdir(newfolder);
		}
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	if (delete_original) {
		// The new bundle is registered, but is not active and has not taken its name, so there
		// won't be any confusion when the one one is deleted
		AlpBundle b = {delete_original};
		ALP_TMTL("Deleting original conflict bundle %d", delete_original);
		if ((flags & ALP_PRV_BUNDLE_FLAG_LAZY_DELETE) && (flags & ALP_PRV_BUNDLE_FLAG_FORCE_UNREGISTER)) {
			ALP_TMTL("Trying non-lazy delete first");
			// did the caller ask us to force a lazy delete? If so, first just try the force.
			// If that succeeds, we're ready to continue
			if (alp_bundle_delete(b, flags & ~ALP_PRV_BUNDLE_FLAG_LAZY_DELETE) != ALP_STATUS_OK) {
				ALP_TMTL("OK, going on to lazy delete");
				// otherwise, we need to really lazily delete the old one. But in order for
				// the new one to immediately kick in, we'll need to summarily deactivate the name on
				// the old one
				alp_prv_bundle_deactivate(b, flags);
				
				// FIXME: do we also want to send out an unreg notification early?
				//alp_prv_bundle_unreg_notify(b);
				
				// Then invoke the lazy delete
				alp_bundle_delete(b, flags);
				
				// now continue on to the notify of the new one -- the old one will be deleted
				// at some point in the future
			}
			
		} else if (alp_bundle_delete(b, flags) != ALP_STATUS_OK) { // don't delete R/W data
			ALP_TMTL("Delete failed, unwinding");
			// delete failed, unwind install
			alp_bundle_delete(package, flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		ALP_TMTL("Delete worked, now activating new package");
		
		// now we activate the new one
		alp_prv_bundle_reg_notify(package, flags);
	}
	
	if (metadata_p)
		*metadata_p = metadata;
	if (properties_p)
		*properties_p = properties;
	
	// no problems detected
	return package;
fail:
	g_free(store_name);
	g_free(bar_folder);
	
	if (parser)
		finish_parser(parser, NULL, NULL);
	
	if (properties)
		g_hash_table_destroy(properties);
	
	if (metadata)
		g_hash_table_destroy(metadata);
		
	set_error(status);
	return ALP_BUNDLE_NULL;
}


// FIXME: basic bug: progress not invoked
alp_status_t alp_bundle_image_archive(AlpBundle index, char ** desired_filename, char ** desired_mimetype, GIOChannel * image, AlpBundleProgressiveData * progress)
{
	gchar * unpacked_folder = NULL;
	gchar * packed_image = NULL;
	
	alp_status_t status;
	
	S_STMT s;
	S_PREPARE(s, "SELECT open_foldername,scheme_id,ghost_db_storage,ghost_db_type,ghost_db_creator,ghost_db_name FROM containers "
			" LEFT OUTER JOIN ghost_metadata ON (ghost_metadata.package_id = containers.package_id) "
			" LEFT OUTER JOIN java_metadata ON (java_metadata.package_id = containers.package_id) "
			" WHERE containers.package_id = ?1");
	S_BIND_INT(s, 1, index.value);
	
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}

	unpacked_folder = g_strdup(S_COL_TEXT(s, 0));
	int scheme = S_COL_INT(s,1);
	int ghost_db_storage = S_COL_INT(s,2);
	int ghost_db_type = S_COL_INT(s,3);
	int ghost_db_creator = S_COL_INT(s,4);
	gchar * ghost_db_name = g_strdup(S_COL_TEXT(s,5));

	S_FINALIZE(s);
	
	if (scheme == BUNDLE_SCHEME_GHOST)
		goto ghost;

	if (desired_filename) {
		gchar * f = alp_bundle_metadata_value(index, "desired_filename");
		if (!f) {
			gchar * name = alp_bundle_metadata_value(index, "name");
			f = g_strdup_printf("%s.bar", name);
			g_free(name);
		}
		*desired_filename = f;
	}

	if (desired_mimetype) {
		*desired_mimetype = g_strdup(MIME_TYPE_BAR);
	}

	// FIXME: want way to return name/type without doing anything, but this is
	// baroque.
	if (!image)
		goto success;

	if (!unpacked_folder && !packed_image) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}

	int fd = -1;

	if (unpacked_folder && !packed_image) {
		char buf[PATH_MAX];
		sprintf(buf, "%s/tmp/archive.XXXXXX", getenv("ALP_RUN_TOP") ?: "");
		fd = mkstemp(buf);
		
		char manifest[PATH_MAX];
		sprintf(manifest, "%s%s", unpacked_folder, ALP_BUNDLE_BAR_MANIFEST);
		
		int child_pid;
		
		sigset_t chld;
		sigemptyset(&chld);
		sigaddset(&chld, SIGCHLD);
	
		sigprocmask(SIG_BLOCK, &chld, &chld);
		
		// decide about SIGINT, HUP
		
		if ((child_pid = fork()) == 0) {
			int devnull=open("/dev/null", O_RDWR);
			if (devnull != 0) dup2(devnull, 0);
			if (devnull != 1) dup2(devnull, 1);
			if (devnull != 2) dup2(devnull, 2);
			int result = execlp("mkcramfs", "mkcramfs", "-i", manifest, "-n", "Binder Archive 1", unpacked_folder, buf, NULL);
			_exit(127);
		}
		
		int child_status, result;
		while ((result = waitpid(child_pid, &child_status, 0)) == -1 && (errno == EINTR))
			;
		
		sigprocmask(SIG_SETMASK, &chld, NULL);
		
		if (result == -1) {
			ALP_TMTL("error waiting for child pid %d: %s\n", child_pid, strerror(errno));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		
		if (WIFSIGNALED(child_status)) {
	 		ALP_TMTL("child %d dies with signal %d\n", child_pid, WTERMSIG(child_status));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
	 		goto fail;
		}
		
		ALP_TMTL("Result of system: %d\n", result);
		
		lseek(fd, 0, SEEK_SET);
		
		packed_image = g_strdup(buf);
	}
	
	if (fd == -1)
		fd = open(packed_image, O_RDONLY);
	
	// check fd != -1
	
	gchar * buffer = g_malloc(SIZE);
	// check buffer != NULL
	
	size_t len = lseek(fd, 0, SEEK_END);
	lseek(fd, 0, SEEK_SET);
	
	if (progress) {
		progress->new_state |= ALP_BUNDLE_PROGRESSIVE_HAVE_TOTAL_BYTES;
		progress->current_state |= ALP_BUNDLE_PROGRESSIVE_HAVE_TOTAL_BYTES;
		progress->callback(progress);
		// abort if TRUE
	}
		
	size_t pos = 0;
	while (pos < len) {
		int r = read(fd, buffer, SIZE);
		size_t written;
		if (r <= 0) {
			break;
		}
		GIOStatus result = g_io_channel_write_chars(image, buffer, r, &written, NULL);
		
		pos += written;
	}
	
	g_free(buffer);
	
	close(fd);
	
	// complete
	
	if (packed_image && unpacked_folder) {
		unlink(packed_image);
	}

success:	
 	g_free(packed_image);
	g_free(unpacked_folder);
	g_free(ghost_db_name);
	
	return ALP_STATUS_OK;

ghost:

	// Can only export from internal store
	if (ghost_db_storage != 0) {
		goto fail;
	}

	if (desired_filename) {
		gchar * f = alp_bundle_metadata_value(index, "desired_filename");
		if (!f) {
			f = g_strdup_printf("%s.prc", ghost_db_name);
		}
		*desired_filename = f;
	}

	if (desired_mimetype) {
		*desired_mimetype = g_strdup(MIME_TYPE_PRC);
	}

	// FIXME: want way to return name/type without doing anything, but this is
	// baroque.
	if (!image)
		return ALP_STATUS_OK;
	
	char tmpdir[] = "/tmp/ghostexportXXXXXX";
	if (!mkdtemp(tmpdir)) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	gchar * dest_filename = g_strdup_printf("%s/%s.prc", tmpdir, ghost_db_name);

	int ret = -1;
	if (ghost_start() == ALP_STATUS_OK)
	{
		ret = alp_gras_db_export(GSESSION, ghost_db_name, ghost_db_type, ghost_db_creator, tmpdir);
		ghost_stop(ret != ALP_STATUS_OK && ALP_STATUS_VALUE(ret) != ALP_STATUS_GRAS_GARNET_ERROR);
	}	
	
	if (ret != 0) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		unlink(dest_filename);
		rmdir(tmpdir);
		goto fail;
	}
	
	fd = open(dest_filename, O_RDONLY);
	
	if (fd == -1) {
		// Guessed wrong filename, clean up after it
		rmdir_contents(tmpdir);
		rmdir(tmpdir);

		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	// check fd != -1
	
	buffer = g_malloc(SIZE);
	// check buffer != NULL
	
	len = lseek(fd, 0, SEEK_END);
	lseek(fd, 0, SEEK_SET);
	
	if (progress) {
		progress->new_state |= ALP_BUNDLE_PROGRESSIVE_HAVE_TOTAL_BYTES;
		progress->current_state |= ALP_BUNDLE_PROGRESSIVE_HAVE_TOTAL_BYTES;
		progress->callback(progress);
		// abort if TRUE
	}
		
	pos = 0;
	while (pos < len) {
		int r = read(fd, buffer, SIZE);
		size_t written;
		if (r <= 0) {
			break;
		}
		GIOStatus result = g_io_channel_write_chars(image, buffer, r, &written, NULL);
		
		pos += written;
	}
	
	g_free(buffer);
	
	close(fd);
	
	// complete
	unlink(dest_filename);
	rmdir(tmpdir);	
	
	goto success;

fail:
	g_free(packed_image);
	g_free(unpacked_folder);
	g_free(ghost_db_name);
	
	ALP_TMTL("Archive failure");
	return set_error(status);
}
