/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>

#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_msg.h"


void handle_connect(AlpConnection * con, gpointer data)
{
	alp_ipc_connection_register_receive_callback(con, handle_receive, NULL);
	
	GArray * ary = g_array_new(FALSE, FALSE, sizeof(AlpBundleRef));
	
	alp_ipc_connection_set_custom_data(con, (gpointer)ary);
}

static void do_disconnect_deref(gpointer key, gpointer value, gpointer user_data)
{
	int i;
	for (i=(int)value; i>=0; i--)
		; //alp_container_unref((AlpContainer*)key);
}

void handle_disconnect(AlpConnection* con, gpointer data)
{
	GArray * ary = alp_ipc_connection_get_custom_data(con);
	if (!ary)
		return;
	
	unsigned int i;
	for (i=0; i<ary->len; i++)
		if (g_array_index(ary, AlpBundleRef, i) != NULL)
			alp_bundle_close(g_array_index(ary, AlpBundleRef, i));
	
	g_array_free(ary, TRUE);
}

void handle_receive(AlpConnection * con, AlpMessage * msg, gpointer data)
{
	// push global error state
	alp_status_t save_error = alp_bundle_error();
	
	switch (alp_ipc_message_get_message_ID(msg)) {
		case MSG_PACKAGE_OPEN:
			return handle_client_alp_bundle_open(con, msg, data); 
		case MSG_PACKAGE_CLOSE:
			return handle_client_alp_bundle_close(con, msg, data); 
		case MSG_PACKAGE_INCREF:
			return handle_client_alp_prv_bundle_incref(con, msg, data); 
		case MSG_PACKAGE_DECREF:
			return handle_client_alp_prv_bundle_decref(con, msg, data); 
		case MSG_PACKAGE_RW_PATHNAME:
			return handle_client_alp_bundle_rw_pathname(con, msg, data); 
		case MSG_PACKAGE_REGISTER:
			return handle_client_alp_prv_bundle_register_bundle(con, msg, data);
		case MSG_PACKAGE_DELETE:
			return handle_client_alp_bundle_delete(con, msg, data);
		case MSG_PACKAGE_UNREGISTER:
			return handle_client_alp_prv_bundle_unregister_bundle(con, msg, data);
		case MSG_PACKAGE_ADD_STORE:
			return handle_client_alp_prv_bundle_add_store(con, msg, data);
		case MSG_PACKAGE_DELETE_STORE:
			return handle_client_alp_prv_bundle_delete_store(con, msg, data);
		case MSG_PACKAGE_MOVE_TO_STORE:
			return handle_client_alp_bundle_move_to_store(con, msg, data);
		case MSG_PACKAGE_COPY_TO_STORE:
			return handle_client_alp_bundle_copy_to_store(con, msg, data);
		case MSG_PACKAGE_WIPE:
			return handle_client_alp_prv_package_wipe(con, msg, data);
		case MSG_PACKAGE_RW_DELETE:
			return handle_client_alp_bundle_rw_delete(con, msg, data);
		case MSG_PACKAGE_SCAN_STORE:
			return handle_client_alp_prv_bundle_scan_store(con, msg, data);
		case MSG_PRV_BUNDLE_IMAGE_INSTALL_TMPNAME:
			return handle_client_alp_prv_bundle_image_install_tmpname(con, msg, data);
		case MSG_PRV_BUNDLE_HOLD_NOTIFICATIONS:
			return handle_client_alp_prv_bundle_hold_notifications(con, msg, data);
		default:
			ALP_TMTL("Server got unknown message %d\n", alp_ipc_message_get_message_ID(msg));
			alp_ipc_message_destroy(msg);
			;
	}
	
	// pop global error state
	set_error(save_error);	
}
