/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>

#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"

GHashTable * alp_bundle_metadata(AlpBundle index)
{
	if (alp_bundle_is_NULL(index)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	GList * l = NULL;
	GHashTable * results = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);

	S_STMT s;
	S_PREPARE(s, "SELECT * FROM merged_metadata WHERE package_id=?1");
	S_BIND_INT(s, 1, index.value);
	S_BEGIN_SELECT(s) {
		int i=0;
		for (i=1;i<sqlite3_column_count(s);i++) {
			gchar * key = g_strdup(sqlite3_column_name(s, i));
			gchar * value = g_strdup(S_COL_TEXT(s,i));
			g_hash_table_insert(results, key, value);
		}
	}
	S_END_SELECT(s);

	return results;
}

gchar * alp_bundle_metadata_value(AlpBundle index, const char * key)
{
	gchar * result = NULL;
	int scheme_id = 0;
	
	S_STMT s;
	char cmd[] = "SELECT ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ FROM merged_metadata WHERE package_id=?1";
	
	if (safe_sql_fillin(cmd, key)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	if (!S_PREPARE_P(s, cmd)) { // if preparation error, must be invalid column
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	S_BIND_INT(s, 1, index.value);
	S_BEGIN_SELECT(s) {
		result = g_strdup(S_COL_TEXT(s, 0));
		// FIXME: detect and report out of memory error here
	}
	S_END_SELECT(s);
	
	if (!result) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	return result;
}

GList * alp_bundle_all_properties(AlpBundle index)
{
	if (alp_bundle_is_NULL(index)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	GList * l = NULL;
	GHashTable * h = NULL;
					
	sqlite3_stmt * s;
	int result = sqlite3_prepare(db, "SELECT property, sequence, key, value FROM properties WHERE package_id = ?1 ORDER BY property ASC, sequence ASC, key ASC;", -1, &s, NULL);
	if (result != SQLITE_OK) {
		fprintf(stderr, "Error preparing command: %s\n", sqlite3_errmsg(db));
		exit(1);
	}
	sqlite3_bind_int(s, 1, index.value);
	int seq = 0;
	while ((result = sqlite3_step(s)) == SQLITE_ROW) {
		if (!h || sqlite3_column_int(s, 1) != seq) {
			h = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
			l = g_list_append(l, h);
			seq = sqlite3_column_int(s, 1);
			
			g_hash_table_insert(h, g_strdup(""), g_strdup(sqlite3_column_text(s, 0)));
		}
		gchar * key = g_strdup(sqlite3_column_text(s, 2));
		gchar * value = g_strdup(sqlite3_column_text(s, 3));
		g_hash_table_insert(h, key, value);
	}
	if (result != SQLITE_DONE) {
		fprintf(stderr, "Can't step command: %s\n", sqlite3_errmsg(db));
		exit(1);
	} else {
		sqlite3_finalize(s);
	}
	
	return l;
}

GHashTable * alp_bundle_properties(AlpBundle index, const char * property, int which)
{
	if (alp_bundle_is_NULL(index)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	GList * l = NULL;
	GHashTable * results = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);

	S_STMT s;
	S_PREPARE(s, "SELECT key,value FROM properties WHERE (package_id = ?1 AND property = ?2 AND sequence = ?3) ORDER BY sequence ASC, key ASC");
	S_BIND_INT(s, 1, index.value);
	S_BIND_TEXT(s, 2, property);
	S_BIND_INT(s, 3, which);
	S_BEGIN_SELECT(s) {
		gchar * key = g_strdup(S_COL_TEXT(s,0));
		gchar * value = g_strdup(S_COL_TEXT(s,1));
		g_hash_table_insert(results, key, value);
	}
	S_END_SELECT(s);
	if (g_hash_table_size(results) == 0)
	{
		g_hash_table_destroy(results);
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		results = NULL;
	}

	return results;
}

gchar * alp_bundle_property_value(AlpBundle index, const char * property, int which, const char * key)
{
	gchar * result = NULL;

	if (alp_bundle_is_NULL(index)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	S_STMT s;
	S_PREPARE(s, "SELECT value FROM properties WHERE (package_id = ?1 AND property = ?2 AND sequence = ?3 AND key = ?4) LIMIT 1 OFFSET 0;");
	S_BIND_INT(s, 1, index.value);
	S_BIND_TEXT(s, 2, property);
	S_BIND_INT(s, 3, which);
	S_BIND_TEXT(s, 4, key);
	S_BEGIN_SELECT(s) {
		result = g_strdup(S_COL_TEXT(s, 0));
		// FIXME: detect and report out of memory error here
	}
	S_END_SELECT(s);
	
	if (!result) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	return result;
}
