/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#define _GNU_SOURCE // for dladdr() from dlfcn.h

#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <hiker/notifymgr.h>
#include <pthread.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <sqlite3.h>
#include <dlfcn.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <unistd.h>
#include <hiker/global_settings.h>

#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"
#include "pm_refimpl.h"

AlpBundle alp_bundle_me(void)
{
	void * addr  = __builtin_return_address(0);
	
	Dl_info i;
	memset(&i, '\0', sizeof(i));
	
	if (dladdr(addr, &i) == 0) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_NULL;
	}
	
	return alp_bundle_locate_file(i.dli_fname);
}

AlpBundle alp_bundle_locate_file(const char * file)
{
	S_STMT s;

	// given a package open on a foldername of "/foo/bar", match a given path of either:
	//   "/foo/bar" (exactly), or
	//   "/foo/bar/biz/baz" (anything that starts with "/foo/bar/").
	// that way, we can't accidentally match
	//   "/foo/barsoom/biz/baz" to "/foo/bar"
	// but can match any file or subfolder within the package
	
	// first check a simple match against the given names
	S_PREPARE(s, "SELECT containers.package_id FROM containers JOIN live_metadata ON "
			" (containers.package_id = live_metadata.package_id) WHERE "
			" ((live_metadata.real_folder == ?1) OR "
			" (substr(?1, 1, length(live_metadata.real_folder)+1) == (live_metadata.real_folder || '/'))) LIMIT 1");
	S_BIND_TEXT(s, 1, file);

	AlpBundle result = ALP_BUNDLE_NULL;
	
	S_BEGIN_SELECT(s) {
		result.value = S_COL_INT(s, 0);
	}
	S_END_SELECT(s);

	
	// next, use a canonical version of the path on both sides, in case someone has dereferenced
	// some symlinks, or has their own links 
	if (!result.value) {
		char buf[PATH_MAX];
		
		//FIXME: file might be partially hypothetical: we want "partrealpath" which will resolve
		//as much of the path as possible, but not abort if it couldn't realize the entire path.
		if (!realpath(file, buf))
			return result;
		
		S_PREPARE(s, "SELECT containers.package_id FROM containers JOIN live_metadata ON "
				"(containers.package_id = live_metadata.package_id) WHERE "
				" ((resolved_real_folder == ?1) OR "
				" (substr(?1, 1, length(resolved_real_folder)+1) == (resolved_real_folder || '/'))) LIMIT 1");
		S_BIND_TEXT(s, 1, buf);

		result = ALP_BUNDLE_NULL;
	
		S_BEGIN_SELECT(s) {
			result.value = S_COL_INT(s, 0);
		}
		S_END_SELECT(s);
	}
	
	// then check the underlying folder, for BAR packages, to see if this is a reference into
	// a closed package
	
	if (!result.value) {
		char buf[PATH_MAX];
		
		//FIXME: file might be partially hypothetical: we want "partrealpath" which will resolve
		//as much of the path as possible, but not abort if it couldn't realize the entire path.
		if (!realpath(file, buf)) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return ALP_BUNDLE_NULL;
		}

		// FIXME: might open_foldername not be fully resolved? This seems an insignificant
		// issue.
		S_PREPARE(s, "SELECT containers.package_id FROM containers WHERE "
				" ((containers.open_foldername == ?1) OR "
				" (substr(?1, 1, length(containers.open_foldername)+1) == (containers.open_foldername || '/'))) LIMIT 1");
		S_BIND_TEXT(s, 1, buf);

		result = ALP_BUNDLE_NULL;
	
		S_BEGIN_SELECT(s) {
			result.value = S_COL_INT(s, 0);
		}
		S_END_SELECT(s);
	}
	
	if (alp_bundle_is_NULL(result)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_NULL;
	}
	
	return result;
}

static AlpBundle sApplication_Package = ALP_BUNDLE_NULL;
static AlpBundleRef sApplication_Package_Ref = NULL;
static gboolean sApplication_Package_Ref_dec = FALSE;

AlpBundle alp_bundle_application(void)
{
	return sApplication_Package;
}

void alp_prv_bundle_set_application(AlpBundle index)
{
	sApplication_Package = index;
	if (sApplication_Package_Ref) {
		if (sApplication_Package_Ref_dec)
			alp_bundle_close(sApplication_Package_Ref);
		sApplication_Package_Ref = NULL;
		sApplication_Package_Ref_dec = FALSE;
	}
}

void alp_prv_bundle_set_application_ref(AlpBundleRef ref)
{
	sApplication_Package = alp_bundle_ref_bundle(ref);
	if (sApplication_Package_Ref && sApplication_Package_Ref_dec) {
		alp_bundle_close(sApplication_Package_Ref);
	}
	sApplication_Package_Ref = ref;
	sApplication_Package_Ref_dec = FALSE;
}

AlpBundleRef alp_bundle_ref_application(void)
{
	if (!sApplication_Package_Ref) {
		if (alp_bundle_is_NULL(sApplication_Package)) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		sApplication_Package_Ref = alp_bundle_open(sApplication_Package);
		sApplication_Package_Ref_dec = TRUE;
	}
	return sApplication_Package_Ref;
}



// This hash table maps between executable file names and open package refs.
// There is no mechanism to delete the hash, or close the refs, other then shutting
// down the package manager client. As noted in the documentation for alp_bundle_ref_me(),
// it effectively leaks when used as intented, and only process exit will clean it all up.
// Use alp_bundle_me() yourself if this leakage is unacceptable.

static GHashTable * sMeRefTable;

AlpBundleRef alp_bundle_ref_me(void)
{
	void * addr  = __builtin_return_address(0);
	
	Dl_info i;
	memset(&i, '\0', sizeof(i));
	
	if (dladdr(addr, &i) == 0) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_REF_NULL;
	}
		
	if (!sMeRefTable) {
		sMeRefTable = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, (GDestroyNotify)alp_bundle_close);
	}
	
	gpointer find = g_hash_table_lookup(sMeRefTable, i.dli_fname);
	
	if (!find) {
		AlpBundle p = alp_bundle_locate_file(i.dli_fname);

		if (alp_bundle_is_NULL(p)) {
			// propagate potential error from alp_bundle_locate_file
			return ALP_BUNDLE_REF_NULL;
		}
			
		AlpBundleRef r = alp_bundle_open(p);

		if (!r) {
			// propagate potential error from alp_bundle_open
			return NULL;
		}
		
		g_hash_table_insert(sMeRefTable, (gpointer)g_strdup(i.dli_fname), (gpointer)r);
		
		return r;
	} else {
		return (AlpBundleRef)find;
	}
}

void alp_prv_package_clear_refs(void)
{
	alp_prv_bundle_set_application(ALP_BUNDLE_NULL);

	if (sMeRefTable) {
		g_hash_table_destroy(sMeRefTable);
		sMeRefTable = NULL;
	}
}

const char * alp_bundle_ref_gettextdomain(AlpBundleRef ref)
{
	if (!ref) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	return ref->textdomain_name;
}

const char * alp_bundle_ref_name(AlpBundleRef ref)
{
	if (!ref) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	return ref->package_name;
}

gchar * alp_bundle_name(AlpBundle index)
{
	// propagate potential error
	return alp_bundle_metadata_value(index, "name");
}

struct xfer {
	char * appID;
	char * notifyType;
	void * details;
	int length;
	int queue;
	int flags;
	int timeout;
	int * handled;
	int sync;
	GMainLoop * loop;
	alp_status_t result;
	int fd;
};

gboolean do_idle(gpointer data)
{
	ALP_TMTL("Got IDLE");
        g_main_quit((GMainLoop*)data);
        return FALSE;
}

void do_notify(gpointer data, gpointer userdata)
{
	struct xfer * x = data;

	if (!x->sync) {
		alp_notify_broadcast(x->appID, x->notifyType, x->details, x->length,
			x->queue, x->flags);
		
	}
	
	g_free(x->appID);
	g_free(x->notifyType);
	g_free(x->details);
	g_free(x);
}

/* Currently unused */
void * thread_func(void * param)
{
	struct xfer * x = param;
	
	ALP_TMTL("-1-");
	
	if (x->sync) {
		x->result =  alp_notify_broadcast_sync(x->appID, x->notifyType, x->details, x->length,
			x->queue, x->flags, x->timeout, x->handled);
	} else {
		x->result =  alp_notify_broadcast(x->appID, x->notifyType, x->details, x->length,
			x->queue, x->flags);
	}
	
	ALP_TMTL("-2-");
	
	ALP_TMTL("x pointer is %p", x);

	GSource * s = g_idle_source_new();
	g_source_set_callback(s, do_idle, x->loop, NULL);
	g_source_attach(s, g_main_loop_get_context(x->loop));
	
	ALP_TMTL("-3-");

	return NULL;
}


static GThreadPool * sNotifyPool;

static void create_notify_pool(void)
{
	if (!sNotifyPool)
		sNotifyPool = g_thread_pool_new(do_notify, NULL, 1, TRUE, NULL);
}

void destroy_notify_pool(void)
{
	if (sNotifyPool) {
		g_thread_pool_free(sNotifyPool, FALSE, TRUE);
		sNotifyPool = NULL;
	}
}

/* Machinery to perform a synchonous notification without blocking: a recursive main loop
   is entered until the broadcast is complete. */

/* Currently we're having a lot of troubles with sync notification, so turn them into async down at this level */   
alp_status_t my_sync_notify(const char * appID, const char * notifyType, void * details, int length, int queue, int flags, int timeout, int * handled)
{
	return my_async_notify(appID, notifyType, details, length, queue, flags);
	//return alp_notify_broadcast_sync(appID, notifyType, details, length, queue, flags, timeout, handled);
#if 0
	create_notify_pool();

	struct xfer x;
	x.appID = appID;
	x.notifyType = notifyType;
	x.details = details;
	x.length = length;
	x.queue = queue;
	x.flags = flags;
	x.timeout = timeout;
	x.handled = handled;
	x.sync = 1;

	ALP_TMTL("x is %p", &x);	
	
	ALP_TMTL("-4- type=%s", notifyType);

	x.loop = g_main_loop_new(NULL, TRUE);

	ALP_TMTL("-5- loop=%p", x.loop);
	
	ALP_TMTL("-5.5-");
	

	ALP_TMTL("-5.8-");
	
	GThread * t = g_thread_create(thread_func, &x, FALSE, NULL);
	
	ALP_TMTL("-6- %p", t);

	g_main_loop_run(x.loop);
	
	ALP_TMTL("-6.5-");
	
	ALP_TMTL("-7-");
	
	g_main_loop_unref(x.loop);
	
	ALP_TMTL("-9- handled=%d", *x.handled);

	return x.result;
#endif
}

/* Machinery to perform an asynchonous notification without blocking: we trigger the notification
   in a separate thread. */
   
alp_status_t my_async_notify(const char * appID, const char * notifyType, void * details, int length, int queue, int flags)
{
//	return alp_notify_broadcast(appID, notifyType, details, length, queue, flags);

	create_notify_pool();

	struct xfer * x = g_malloc(sizeof(struct xfer));
	
	x->appID = g_strdup(appID);
	x->notifyType = g_strdup(notifyType);
	x->details = g_memdup(details, length);
	x->length = length;
	x->queue = queue;
	x->flags = flags;
	x->sync = 0;
	
	// This will create the worker thread if it doesn't already exist, reusing it if is does.
	// If the thread is busy, the data will go onto a queue until it is ready to be processed.
	
	g_thread_pool_push(sNotifyPool, x, NULL);

	return 0;
}


static alp_status_t sGlobalError = ALP_STATUS_BUNDLEMGR_FAILURE;

alp_status_t set_error(alp_status_t status)
{
	if (status != ALP_STATUS_OK)
		sGlobalError = status;
	return status;
}

alp_status_t alp_bundle_error(void)
{
	return sGlobalError;
}

gchar * alp_bundle_saved_settings(AlpBundle index, const char * key)
{
	if (key && key[0])
		return g_strdup_printf("/bundles/%s/saved/%s", alp_bundle_name(index), key);
	else
		return g_strdup_printf("/bundles/%s/saved", alp_bundle_name(index));
}

gchar * alp_bundle_unsaved_settings(AlpBundle index, const char * key)
{
	if (key && key[0])
		return g_strdup_printf("/bundles/%s/unsaved/%s", alp_bundle_name(index), key);
	else
		return g_strdup_printf("/bundles/%s/unsaved", alp_bundle_name(index));
}

// Routine to allow a SQL symbol to be inserted into a string avoiding
// potential injection attacks. str should be a writable C string containing
// a single consecutive run of '^' characters. Those characters will be replaced
// by the contents of fillin. Additional '^' characters will be replaced
// by spaces. If fillin runs up against the length of the '^' run, then failure
// will be returned due to overrun. Failure will also be returned if the filling
// string contains any potentially dangerous characters.
int safe_sql_fillin(char * str, const char * fillin)
{
	char * c = strchr(str, '^');
	
	if (!c)
		return 0;

	while (*fillin) {
		if (!isalnum(*fillin) && (*fillin != '_')) {
			ALP_TMTL("Potential SQL injection: fillin {%s} to cmd {%s} contains illegal character", fillin, str);
			return 2;
		}
		*c++ = *fillin++;
		if (*c != '^') {
			ALP_TMTL("Potential SQL injection: fillin {%s} to cmd {%s} is too long", fillin, str);
			return 1; // fail, fillin too long
		}
	}
	while (*c == '^')
		*c++ = ' ';
	
	return 0;
}
