/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <unistd.h>
#include <hiker/fail.h>
#include <libintl.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/mount.h>

//#include <linux/loop.h>
//#include <linux/cramfs_fs.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
                
#define MAX_LOOPDEV 256

#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"
#include "pm_refimpl.h"
#include "pm_msg.h"
#include "pm_perms.h"
#include "ghost_support.h"

#include "loop_interface.h"
#include "cramfs_interface.h"

GTree * sRecordTree;

struct precord {
	int refcount;
	int scheme;
	int ghost_db_type;
	int ghost_db_creator;
	int ghost_db_storage;
	int java_midlet_id;
	gchar * ghost_db_name;
	gchar * underlying_foldername;
	gchar * underlying_filename;
	gchar * folder_name;
	gchar * package_name;
	gchar * textdomain_name;
	gchar * textdomain_dir;
	gchar * classpath;
	gchar * searchpath;
	gchar * mounted_foldername;
	gchar * loopback_device;
	//gchar * java_midlet_name;
	GList * solibs;
};

// Map from bound textdomain to a structure
typedef struct old_boundtextdomain_t
{
	gchar * old_path;
	int refcount;
	
} old_boundtextdomain_t;
static GHashTable * sBoundtextdomains;

static const int debug = 0; // disable some extra traces which would be too noisy

void alp_prv_bundle_clear_local_effects()
{
	if (sBoundtextdomains)
		g_hash_table_destroy(sBoundtextdomains);
	sBoundtextdomains = NULL;
}

static void clearBounddomainRecord(gpointer p)
{
	old_boundtextdomain_t * t = p;
	g_free(t->old_path);
	g_free(t);
}

static void apply_local_open_effects(struct _AlpBundleRef * ref)
{
	if (ref->textdomain_name && ref->textdomain_dir) {
	
		if (!sBoundtextdomains)
			sBoundtextdomains = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, clearBounddomainRecord);
		
		gpointer h;	
		if (!(h = g_hash_table_lookup(sBoundtextdomains, ref->textdomain_name))) {
			old_boundtextdomain_t * t = g_new0(old_boundtextdomain_t, 1);
			
			t->old_path = g_strdup(bindtextdomain(ref->textdomain_name, NULL));
			t->refcount = 1;
			
			char buf[PATH_MAX];
			sprintf(buf, "%s/%s", ref->folderpath, ref->textdomain_dir);
			
			bindtextdomain(ref->textdomain_name, buf); 
			
			if (debug)
				ALP_TMTL("Binding text domain {%s} to path {%s}, recording old path as {%s}",
					ref->textdomain_name, buf,
					t->old_path);
			
			g_hash_table_insert(sBoundtextdomains, g_strdup(ref->textdomain_name), t);
			
		} else {
			old_boundtextdomain_t * t = h;
			t->refcount++;
		}
	}
	
	if (ref->classpath) {
		char * classpath_env;
		if ((classpath_env = getenv("CLASSPATH"))) {
			gchar * new_classpath = g_strdup_printf("%s:%s", classpath_env, ref->classpath);
			setenv("CLASSPATH", new_classpath, 1);
			g_free(new_classpath);
		} else {
			setenv("CLASSPATH", ref->classpath, 1);
		}

		if (debug)
			ALP_TMTL("Binding classpath {%s} into environment, now {%s}", ref->classpath, getenv("CLASSPATH"));
	}
	
	ref->applied_local_effects = 1;
}

static void unapply_local_open_effects(struct _AlpBundleRef * ref)
{
	if (ref->textdomain_name && ref->textdomain_dir) {
	
		if (!sBoundtextdomains) {
			ALP_TMTL("Very odd");
		}
		
		gpointer h;	
		if ((h = g_hash_table_lookup(sBoundtextdomains, ref->textdomain_name))) {
			old_boundtextdomain_t * t = h;
			
			t->refcount--;

			if (t->refcount == 0) {

				if (debug)
					ALP_TMTL("Unbinding text domain {%s} from path {%s}, restoring to {%s}",
						ref->textdomain_name, ref->textdomain_dir, t->old_path);
				
				bindtextdomain(ref->textdomain_name, t->old_path); 
				g_hash_table_remove(sBoundtextdomains, ref->textdomain_name);
			}
			
		} else {
			ALP_TMTL("Again, very odd");
		}
	}

	if (ref->classpath) {
		char * classpath_env = g_strdup(getenv("CLASSPATH"));
		if (classpath_env) {
			if (debug)
				ALP_TMTL("Unbinding classpath {%s} from environment {%s}", ref->classpath, classpath_env);

			// look for the fragment we added to the classpath, basically matching /(^|:)$FRAGMENT($|:)/
			char * str = classpath_env;
			while ((str = strstr(str, ref->classpath))) {
				if ((str == classpath_env || str[-1] == ':') &&
				    ((str + strlen(ref->classpath)) == (classpath_env+strlen(classpath_env)) || str[strlen(ref->classpath)] == ':')) {
				    	memmove(str, str+strlen(ref->classpath), strlen(str) - strlen(ref->classpath) + 1);
				    	break;
				    }
			}
			if (str) {
				if (debug)
					ALP_TMTL("Unbinding classpath, environment restored to {%s}", classpath_env);
				setenv("CLASSPATH", classpath_env, 1);
			} else {
				ALP_TMTL("Odd, couldn't undo CLASSPATH changes (unable to remove %s from %s)",
					ref->classpath, classpath_env);
			}
			g_free(classpath_env);
		}
	}
}

AlpBundleRef alp_bundle_open(AlpBundle index)
{
	return alp_prv_bundle_open(index, ALP_PRV_BUNDLE_FLAG_APPLY_LOCAL_EFFECTS|ALP_PRV_BUNDLE_FLAG_RECURSE);
}

AlpBundleRef alp_prv_bundle_open(AlpBundle index, int flags)
{
	alp_status_t status;
	
	if (alp_bundle_is_NULL(index)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	GList * refs = NULL;

	struct _AlpBundleRef * result = g_new0(struct _AlpBundleRef, 1);	

	if (!result) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}

	struct ref {
		gchar * name;
		int optional;
	}; 
	

	if (flags & ALP_PRV_BUNDLE_FLAG_RECURSE) {
		// first, alp_bundle_open() all packages that this package depends on

		S_STMT s;
		S_PREPARE(s, "SELECT refer_to, optional FROM bar_references WHERE package_id = ?1");
		S_BIND_INT(s, 1, index.value);
		S_BEGIN_SELECT(s) {
			struct ref * r = g_new0(struct ref, 1);
			r->name = g_strdup(S_COL_TEXT(s,0));
			r->optional = S_COL_INT(s,1);
			
			refs = g_list_prepend(refs, r);
		}
		S_END_SELECT(s);
	
		refs = g_list_reverse(refs);
	
		// collected references in a select statement, then do the open outside, so we won't
		// deadlock	
		while (refs) {
			struct ref * r = refs->data;
			AlpBundleRef referred = alp_bundle_open(alp_bundle_by_name(r->name));
			int optional = r->optional;
			
			g_free(r->name);
			g_free(r);
			
			refs = g_list_delete_link(refs, refs);
			
			if (!referred && !optional) {
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail_refs;
			}
			
			result->references = g_list_prepend(result->references, (gpointer)referred);
		}
		
		if (0) {
	
		fail_refs:
			while (refs) {
				struct ref * r = refs->data;
				g_free(r->name);
				g_free(r);
				refs = g_list_delete_link(refs, refs);
			}
		
			goto fail;
		
		}

	}

	if (pm_status == STATUS_CLIENT) {

		// client implementation -- ask the server to do the actual open

		AlpMessage * message, *response;
		
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_OPEN);
		alp_ipc_message_pack_int32(message, index.value);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			// some failure of IPC
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		alp_ipc_message_destroy(message);

		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result->cookie);

		if (result->cookie == -1) {
			alp_ipc_message_unpack_int32(response, &status);
			alp_ipc_message_unpack_end(response);
			alp_ipc_message_destroy(response);
			goto fail;
		}

		alp_ipc_message_unpack_int32(response, &result->scheme);
		alp_ipc_message_unpack_int32(response, &result->ghost_db_type);
		alp_ipc_message_unpack_int32(response, &result->ghost_db_creator);
		alp_ipc_message_unpack_int32(response, &result->ghost_db_storage);
		alp_ipc_message_unpack_string(response, &result->ghost_db_name);
		alp_ipc_message_unpack_string(response, &result->folderpath);
		alp_ipc_message_unpack_string(response, &result->package_name);
		alp_ipc_message_unpack_string(response, &result->textdomain_name);
		alp_ipc_message_unpack_string(response, &result->textdomain_dir);
		alp_ipc_message_unpack_string(response, &result->classpath);
		int count;
		alp_ipc_message_unpack_int32(response, &count);
		int i;
		result->searchpath = NULL;
		for (i=0;i<count;i++) {
			gchar * ptr;
			alp_ipc_message_unpack_string(response, &ptr);
			result->searchpath = g_list_append(result->searchpath, ptr);
		}
		alp_ipc_message_unpack_int32(response, &result->java_midlet_id);
		//alp_ipc_message_unpack_string(response, &result->java_midlet_name);
		
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);
	
		result->refcount = 1;
		result->package_id = index.value;

	} else {

		// server implementation

		// incref ourself, and set up handle	

		alp_status_t incresult = alp_prv_bundle_incref(index, flags);
		if (incresult != ALP_STATUS_OK) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
			
		result->refcount = 1;
		result->package_id = index.value;
			
		struct precord * record;

		record = g_tree_lookup(sRecordTree, (gpointer)index.value);
	
		result->scheme = record->scheme;
		result->ghost_db_type = record->ghost_db_type;
		result->ghost_db_creator = record->ghost_db_creator;
		result->ghost_db_storage = record->ghost_db_storage;
		result->ghost_db_name = g_strdup(record->ghost_db_name);
		result->package_name = g_strdup(record->package_name);
		result->folderpath = g_strdup(record->folder_name);
		result->searchpath = alp_prv_bundle_expand_searchpath(record->searchpath);
		result->classpath = g_strdup(record->classpath);
		result->textdomain_name = g_strdup(record->textdomain_name);
		result->textdomain_dir = g_strdup(record->textdomain_dir);
		result->java_midlet_id = record->java_midlet_id;
		//result->java_midlet_name = g_strdup(record->java_midlet_name);
		
	}
	
	if (flags & ALP_PRV_BUNDLE_FLAG_APPLY_LOCAL_EFFECTS)
		apply_local_open_effects(result);

	return result;
fail:

	while (refs) {
		struct ref * r = refs->data;
		g_free(r->name);
		g_free(r);
		refs = g_list_delete_link(refs, refs);
	}
	
	if (result->references) {
		GList * iter;
		for (iter = result->references; iter; iter = g_list_next(iter)) {
			alp_prv_bundle_close((AlpBundleRef)iter->data, flags);
		}
		
		g_list_free(result->references);
	}
	if (result->ghost_db_name)
		g_free(result->ghost_db_name);
	if (result->folderpath)
		g_free(result->folderpath);
	if (result->classpath)
		g_free(result->classpath);
	if (result->textdomain_name)
		g_free(result->textdomain_name);
	if (result->textdomain_dir)
		g_free(result->textdomain_dir);
	//if(result->java_midlet_name)
	// 	g_free(result->java_midlet_name);	
	if (result)
		g_free(result);

	set_error(status);
	return NULL;
}

void handle_client_alp_bundle_open(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	AlpBundle index;
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &index.value);
	alp_ipc_message_unpack_end(message);

	struct _AlpBundleRef * result;
	
	if (has_permission(connection, PERM_OPEN_POLICY)) {
		result = alp_prv_bundle_open(index, 0);
	} else {
		result = NULL;
	}

	int cookie;

	if (result) {
		gpointer p = alp_ipc_connection_get_custom_data(connection);
		ALP_FAIL_FATAL_IF(!p, "missing hash table");
		GArray * data = (GArray*)p;
		unsigned int i;
		
		// find a NULL slot in the table, or add a new entry at the end
		for (i=0;i<data->len;i++)
			if (g_array_index(data, AlpBundleRef, i) == NULL)
				break;
		
		if (i < data->len)
			g_array_index(data, AlpBundleRef, i) = result;
		else
			g_array_append_val(data, result);
		
		cookie = i;
	} else
		cookie = -1;

	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_OPEN_RESPONSE, message);
	alp_ipc_message_pack_int32(response, cookie); // pack result pointer as cookie
	if (cookie != -1) {
		alp_ipc_message_pack_int32(response, result->scheme);
		alp_ipc_message_pack_int32(response, result->ghost_db_type);
		alp_ipc_message_pack_int32(response, result->ghost_db_creator);
		alp_ipc_message_pack_int32(response, result->ghost_db_storage);
		alp_ipc_message_pack_string(response, result->ghost_db_name);
		alp_ipc_message_pack_string(response, result->folderpath);
		alp_ipc_message_pack_string(response, result->package_name);
		alp_ipc_message_pack_string(response, result->textdomain_name);
		alp_ipc_message_pack_string(response, result->textdomain_dir);
		alp_ipc_message_pack_string(response, result->classpath);
		alp_ipc_message_pack_int32(response, g_list_length(result->searchpath));
		unsigned int i=0;
		GList * iter = result->searchpath;
		for (i=0;i<g_list_length(result->searchpath);i++,iter = g_list_next(iter)) {
			alp_ipc_message_pack_string(response, (char*)iter->data);
		}
		alp_ipc_message_pack_int32(response, result->java_midlet_id);
		//alp_ipc_message_pack_string(response, result->java_midlet_name);
		
	} else {
		alp_ipc_message_pack_int32(response, alp_bundle_error());
	}
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

// used by package_unregister, this will make sure a package is closed.
// Any AlpBundleRef's will remain allocated, but will be non-functional
// once the underlying AlpBundle index becomes invalid.
alp_status_t alp_prv_bundle_force_close(AlpBundle index, int flags)
{
	struct precord * record;
	
	record = g_tree_lookup(sRecordTree, (gpointer)index.value);

	if (!record)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
	alp_status_t result;

	int refcount = record->refcount;
	
	while (refcount > 0) {
		result = alp_prv_bundle_decref(index, flags);
		// propagate potential error
		if (result != ALP_STATUS_OK)
			return result;
		refcount--;
	}
	
	return ALP_STATUS_OK;
}

alp_status_t alp_bundle_close(struct _AlpBundleRef * ref)
{
	return alp_prv_bundle_close(ref, 0);
}

alp_status_t alp_prv_bundle_close(struct _AlpBundleRef * ref, int flags)
{
	alp_status_t result;
	
	if (!ref)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);

	if (ref->refcount > 1) {
		ref->refcount--;
		return ALP_STATUS_OK;
	}
	
	if (ref->applied_local_effects)
		unapply_local_open_effects(ref);
	
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_CLOSE);
		alp_ipc_message_pack_int32(message, ref->cookie);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			// some failure of IPC
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		alp_ipc_message_destroy(message);

		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);
		
		g_free(ref->folderpath);
		g_list_foreach(ref->searchpath, (GFunc)g_free, NULL);
		g_list_free(ref->searchpath);
		g_free(ref);
		
		return set_error(result);
	}
	
	// server implementation
	AlpBundle index = {ref->package_id};
	result = alp_prv_bundle_decref(index, flags);

	if (result == ALP_STATUS_OK) {

		// close any referred-to packages we opened
		GList * iter;
		for (iter = ref->references; iter; iter = g_list_next(iter)) {
			alp_prv_bundle_close((AlpBundleRef)iter->data, flags);
		}
		
		g_list_free(ref->references);

		g_free(ref->ghost_db_name);
		//g_free(ref->java_midlet_name);
		
		g_free(ref->folderpath);
		g_free(ref->package_name);
		g_list_foreach(ref->searchpath, (GFunc)g_free, NULL);
		g_list_free(ref->searchpath);
		g_free(ref);
	}

	// propagate potential error from alp_prv_bundle_recref
	return result;
}

void handle_client_alp_bundle_close(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	unsigned int cookie;
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &cookie);
	alp_ipc_message_unpack_end(message);
	
	// Note that we don't have any explicit permission checks here; this call
	// can only do something with a bundle previously opened on the same connection

	gpointer p = alp_ipc_connection_get_custom_data(connection);
	ALP_FAIL_FATAL_IF(!p, "missing hash table");
	GArray * data = (GArray*)p;
	unsigned int i;
	
	alp_status_t result;
	
	if (cookie < data->len && g_array_index(data, AlpBundleRef, cookie) != 0) {
		result = alp_bundle_close(g_array_index(data, AlpBundleRef, cookie));
		
		// NULL out this entry in the table
		g_array_index(data, AlpBundleRef, cookie) = NULL;
		
		// truncate the array while the last entry is NULL
		while(data->len > 0 && g_array_index(data, AlpBundleRef,data->len-1) == NULL)
			g_array_remove_index(data, data->len-1);
	} else {
		result = ALP_STATUS_BUNDLEMGR_FAILURE;
	}
		
	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_CLOSE_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result); 
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

AlpBundle alp_bundle_ref_bundle(struct _AlpBundleRef * ref)
{
	if (!ref) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_NULL;
	} else {
		AlpBundle result;
		result.value = ref->package_id;
		return result;
	}
}

alp_status_t alp_prv_bundle_ref_incref(struct _AlpBundleRef * ref)
{
	if (!ref)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
	ref->refcount++;
	
	return ALP_STATUS_OK;
}

alp_status_t alp_prv_bundle_ref_decref(AlpBundleRef ref)
{
	// propagate potential error
	return alp_bundle_close(ref);
}

static int compare(gconstpointer a, gconstpointer b)
{
	if (a<b)
		return -1;
	else if (a>b)
		return 1;
	else
		return 0;
}

alp_status_t alp_prv_bundle_incref(AlpBundle index, int flags)
{
	if (alp_bundle_is_NULL(index))
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;
		
		alp_status_t result;

		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_INCREF);
		alp_ipc_message_pack_int32(message, index.value);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			// some failure of IPC
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		alp_ipc_message_destroy(message);

		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);

		return set_error(result);
	}

	S_STMT s;
	S_PREPARE(s, "SELECT package_id FROM containers WHERE package_id = ?1");
	S_BIND_INT(s, 1, index.value);
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}
	S_FINALIZE(s);

	// all work is server side
	
	struct precord * record;

	if (sRecordTree) {
		record = g_tree_lookup(sRecordTree, (gpointer)index.value);
	
		if (record) {
			if (record->refcount < 1) {
				ALP_TMTL("Odd, there was an empty record");
				return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			}
				
			record->refcount++;
		
			S_STMT s;
			S_PREPARE(s, "UPDATE live_metadata SET refcount = refcount + 1 WHERE package_id = ?1");
			S_BIND_INT(s, 1, index.value);
			S_DO(s);
			return ALP_STATUS_OK;
		}
	}	
	
	record = g_new0(struct precord, 1);
		
	if (!sRecordTree) {
		sRecordTree = g_tree_new(compare);
	}
	g_tree_insert(sRecordTree, (gpointer)index.value, record);
	
	
	char buf[PATH_MAX];
	

	S_PREPARE(s,"SELECT common_metadata.name,open_foldername,archive_filename,bar_searchpath,bar_textdomain_name,bar_textdomain_dir,containers.scheme_id,ghost_metadata.ghost_db_type,ghost_metadata.ghost_db_creator,ghost_metadata.ghost_db_storage,ghost_metadata.ghost_db_name, java_metadata.midlet_name, java_metadata.midlet_id FROM containers JOIN common_metadata ON "
		    " (containers.package_id=common_metadata.package_id) LEFT OUTER JOIN bar_metadata ON "
		    " (containers.package_id=bar_metadata.package_id) LEFT OUTER JOIN ghost_metadata ON "
		    " (containers.package_id=ghost_metadata.package_id)LEFT OUTER JOIN java_metadata ON "
		    " (containers.package_id=java_metadata.package_id) WHERE containers.package_id = ?1");	
		    
	S_BIND_INT(s, 1, index.value);

	if (S_STEP_ROW(s)) {
		record->package_name = g_strdup(S_COL_TEXT(s,0));
		const char * folder = S_COL_TEXT(s, 1);
		if (folder && folder[0] != '/') {
			char buf[PATH_MAX];
			realpath(folder, buf);
			record->underlying_foldername = g_strdup(buf);
		} else
			record->underlying_foldername = g_strdup(folder);
		record->underlying_filename = g_strdup(S_COL_TEXT(s,2));
		record->searchpath = g_strdup(S_COL_TEXT(s,3));
		record->scheme = S_COL_INT(s,6);
		record->ghost_db_type = S_COL_INT(s,7);
		record->ghost_db_creator = S_COL_INT(s,8);
		record->ghost_db_storage = S_COL_INT(s,9);
		record->ghost_db_name = g_strdup(S_COL_TEXT(s,10));
		//record->java_midlet_name  = g_strdup(S_COL_TEXT(s,11));
		record->java_midlet_id = S_COL_INT(s,12);
		//printf("alp_prv_bundle_incref record->java_midlet_id %d ", record->java_midlet_id);	
		if (record->scheme == BUNDLE_SCHEME_BAR) {
		char * t = (char*)S_COL_TEXT(s,4) ?: BAR_DEFAULT_TEXTDOMAIN_NAME;
		
		// FIXME: as, in other places, guard against length overflow
		char work[1024];
		strcpy(work, t);
		
		char * p, *c;
		char * end = work + strlen(work) + 1; // include NUL
		int pname_len = strlen(record->package_name); // exclude NUL
		
		p = work;
		
		while ((c=strchr(p, '^'))) {
			if (c[1] == 'n') {
				char * subst = record->package_name+4;
				int subst_len = pname_len - 4;
				memmove(c+subst_len, c+2, end-(c+2)); // strip out '^n', replace with bundle name post 'bar:'
				memcpy(c, record->package_name+4, subst_len);
				end = end - 2 + subst_len;
				p = c+subst_len; // restart escape scan after replacement
			} else if (c[1] == 'N') {
				char * subst = record->package_name;
				int subst_len = pname_len ;
				memmove(c+subst_len, c+2, end-(c+2)); // strip out '^n', replace with bundle name
				memcpy(c, record->package_name, subst_len);
				end = end - 2 + subst_len;
				p = c+subst_len; // restart escape scan after replacement
			} else {
				memmove(c, c+1, end-(c+1)); // move escaped character up one
				if (c[0]) // restart escape scan after escaped character
					p=c+1; // but if the string ended in an escape mark, don't run off the end!
				else
					break;
			}
		}

		record->textdomain_name = g_strdup(work);
		record->textdomain_dir = g_strdup((char*)S_COL_TEXT(s,5) ?: BAR_DEFAULT_TEXTDOMAIN_DIR);
		} else {
			record->textdomain_name = NULL;
			record->textdomain_dir = NULL;
		}
		S_FINALIZE(s);
	} else {
		S_FINALIZE(s);
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}
	
	record->refcount++;

	if (record->scheme == BUNDLE_SCHEME_BAR) {

	sprintf(buf, "%s%s%s", getenv("ALP_RUN_TOP") ?: "", gBundleWorkPath, BAR_LINK_PATH);
		
	mkdir_p(buf, 0777);
	sprintf(buf + strlen(buf), "/bundle-%d", index.value);
	
	record->folder_name = g_strdup(buf);
	
	alp_status_t status;
	
	// mount package if needed	
	if (record->underlying_filename && !record->underlying_foldername) {
		int loopfd = -1;
		
		sprintf(buf, "%s%s%s/mnt-bundle-%d", getenv("ALP_RUN_TOP") ?: "", gBundleWorkPath, BAR_MOUNT_PATH, index.value);
		mkdir_p(buf, 0777);
		
		record->mounted_foldername = g_strdup(buf);
		
		int archivefd = open(record->underlying_filename, O_RDONLY);
		if (archivefd == -1) {
			ALP_TMTL("Unable to open underlying bundle file %s: %s", record->underlying_filename, strerror(errno));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_mount;
		}

		char loopname[32];
		int i;
	
		struct loop_info read_info, write_info;
		memset(&write_info, '\0', sizeof(write_info));
		strncpy(write_info.lo_name, record->package_name, 63); // should only take 63 characters -- from end?
	
		int tries=0;
		
		i = 0;
	
		// FIXME: this loop should start at 1+ the last allocated loop device, not
		// at zero each time.
	
find_device:	
		for (;i<MAX_LOOPDEV;i++) {

			sprintf(loopname,"/dev/loop%d", i);
			loopfd = open(loopname, O_RDONLY);
			if (loopfd == -1) {
				ALP_TMTL("Couldn't find an available loop device");
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail_mount;
			}

			int result = ioctl(loopfd, LOOP_GET_STATUS, &read_info);
			if (result == -1 && errno == ENXIO)
				break;
			
			close(loopfd);
		}
	
		if (i==MAX_LOOPDEV) {
			ALP_TMTL("Couldn't find an available loop device");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_mount;
		}
		
		int result = ioctl(loopfd, LOOP_SET_FD, archivefd);
		if (result == -1) {
			// this ioctl() should be atomic -- so if someone else (losetup?) is trying to
			// get a loop device, we'll not be able to SET_FD on it if someone else got it first.
			ALP_TMTL("Couldn't set loop fd: %s. Trying to find another loop device.", strerror(errno));
		
			close(loopfd);
			if (tries++ > MAX_LOOPDEV) {
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail_mount;
			}
			goto find_device; // continue to find another loop device
		}
	
		result = ioctl(loopfd, LOOP_SET_STATUS, &write_info);
		if (result == -1) {
			fprintf(stderr, "[Package] Couldn't set loop info: %s. Trying to find another loop device.\n", strerror(errno));
		
			ioctl(loopfd, LOOP_CLR_FD, 0); // return it to the pool
			close(loopfd);
			goto find_device; // continue to find another loop device
		}
	
		ALP_TMTL("Got a loop device named %s", loopname);
		ALP_TMTL("mount(\"%s\", \"%s\", \"cramfs\", RDONLY|NOSUID|NODEV, \"\")\n",
				loopname, record->mounted_foldername );
	
		result = mount(loopname, record->mounted_foldername, "cramfs", MS_RDONLY|MS_NOSUID|MS_NODEV, "");
		if (result == -1) {
			ALP_TMTL("Error mounting package file '%s' onto folder '%s': %s\n", record->underlying_filename, record->mounted_foldername, strerror(errno));
 			ioctl(loopfd, LOOP_CLR_FD, 0); // unwind everything
 			close(loopfd);
 			close(archivefd);
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
 			goto fail_mount;
		}
	
		close(archivefd);
		close(loopfd);
		
		record->loopback_device = g_strdup(loopname);
		
		// simple cheat: if we mounted it successfully, pretend it was there all along
		record->underlying_foldername = g_strdup(record->mounted_foldername);
		
		if (0) {
		fail_mount:
			// FIXME: code up failure -- but it might not be fatal
			// Really, FIXME!
			;
		}
	}
	
	// set up symlink to package
	
	// FIXME: there shouldn't be anything there...
	unlink(record->folder_name);
	
	if (record->mounted_foldername) {
		symlink(record->mounted_foldername, record->folder_name);
	} else {
		symlink(record->underlying_foldername, record->folder_name);
	}
	
	// identify shared libraries exported from this package, and add symlinks to them to
	// the single system path, which should be in LD_LIBRARY_PATH.
	// FIXME: decide on collision behavior
	
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("BEGIN");
	
	S_PREPARE(s,"SELECT path,name FROM bar_exports WHERE package_id = ?1 AND kind = 'LD_LIBRARY_PATH'");
	S_BIND_INT(s, 1, index.value);
	S_BEGIN_SELECT(s) {
		char newpath[PATH_MAX];
		char oldpath[PATH_MAX];
		
		// if a name isn't supplied, take the filename from the supplied path
		const char * name = S_COL_TEXT(s,1);
		if (!name) {
			name = strrchr(S_COL_TEXT(s,0), '/');
			if (name)
				name++;
			else
				name = S_COL_TEXT(s,0);
		}

		sprintf(newpath, "%s%s%s", getenv("ALP_RUN_TOP") ?: "", gBundleWorkPath, BAR_LD_PATH);
		mkdir_p(newpath, 0777);
		sprintf(newpath, "%s%s%s/%s", getenv("ALP_RUN_TOP") ?: "", gBundleWorkPath, BAR_LD_PATH, name);
		sprintf(oldpath, "%s/%s", record->folder_name, S_COL_TEXT(s,0));
		
		symlink(oldpath, newpath);
		
		record->solibs = g_list_append(record->solibs, g_strdup(newpath));
	}
	S_END_SELECT(s);

	// identify java classpath entries exported from this package, record them as a string to
	// add to CLASSPATH
	GString * classpath = g_string_new("");

	S_PREPARE(s,"SELECT path FROM bar_exports WHERE package_id = ?1 AND kind = 'CLASSPATH'");
	S_BIND_INT(s, 1, index.value);
	S_BEGIN_SELECT(s) {
		char buf[PATH_MAX];
		sprintf(buf, "%s/%s", record->folder_name, S_COL_TEXT(s,0));
		
		if (classpath->len)
			g_string_append_c(classpath, ':');
		g_string_append(classpath, buf);
	}
	S_END_SELECT(s);
	
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("COMMIT");
	
	record->classpath = classpath->str;

	g_string_free(classpath, FALSE);
	
	} // if BAR

	// FIXME: !! Figure out why these aren't getting cleaned out!
	S_PREPARE(s, "DELETE FROM live_metadata WHERE package_id = ?1");
	S_BIND_INT(s, 1, index.value);
	S_DO(s);
	
	S_PREPARE(s, "INSERT INTO live_metadata (package_id, refcount, gettextdomain, real_folder, resolved_real_folder) VALUES (?1, 1, ?2, ?3, ?4)");
	S_BIND_INT(s, 1, index.value);
	S_BIND_TEXT(s, 2, record->textdomain_name);
	S_BIND_TEXT(s, 3, record->folder_name);
	
	char resolved[PATH_MAX];
	S_BIND_TEXT(s, 4, realpath(record->folder_name, resolved));
	S_DO(s);

	
	return ALP_STATUS_OK;
}

void handle_client_alp_prv_bundle_incref(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	AlpBundle index;
	
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &index.value);
	alp_ipc_message_unpack_end(message);

	alp_status_t result;
	
	if (has_permission(connection,PERM_INCREF_POLICY)) {
		result = alp_prv_bundle_incref(index, 0);
	} else {
		result = ALP_STATUS_BUNDLEMGR_FAILURE;
	}

	int cookie;

	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_INCREF_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result);
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

alp_status_t alp_prv_bundle_decref(AlpBundle index, int flags)
{
	if (alp_bundle_is_NULL(index))
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;
		
		alp_status_t result;
		
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_DECREF);
		alp_ipc_message_pack_int32(message, index.value);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			// some failure of IPC
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		alp_ipc_message_destroy(message);

		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);

		return set_error(result);
	}

	if (!sRecordTree)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
	struct precord * record;
	record = g_tree_lookup(sRecordTree, (gpointer)index.value);

	if (!record)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			
	// bulk of work is server side

	record->refcount--;
	
	if (record->refcount > 0) {
		S_STMT s;
		S_PREPARE(s, "UPDATE live_metadata SET refcount = refcount - 1 WHERE package_id = ?1");
		S_BIND_INT(s, 1, index.value);
		S_DO(s);
		return ALP_STATUS_OK;
	}

	g_tree_remove(sRecordTree, (gpointer)index.value);
	
	struct precord * r2 = g_tree_lookup(sRecordTree, (gpointer)index.value);
	
	if (record->scheme == BUNDLE_SCHEME_BAR) {

	// remove symlink
	unlink(record->folder_name);

	g_free(record->folder_name);
	
	g_free(record->underlying_foldername);
	g_free(record->underlying_filename);
	
	// unmount
	
	if (record->loopback_device) {
	
		ALP_TMTL("Unmount of loopbacked package, device='%s', mounted foldername='%s'",
			record->loopback_device,
			record->mounted_foldername);
	
		// FIXME: recode this so the close fails if it cannot unmount, unless
		// ALP_PRV_BUNDLE_FLAG_FORCE_UNMOUNT
		
		int loopfd = open(record->loopback_device, O_RDONLY);
		
		if (loopfd == -1) {
			ALP_TMTL("Unable to open loop device %s: %s", record->loopback_device, strerror(errno));
		}
		
		int result = umount(record->mounted_foldername);
		
		if (result == -1 && errno == EBUSY) {
			ALP_TMTL("Unable to unmount package on %s, unmounting lazily", record->mounted_foldername);
			result = umount2(record->mounted_foldername, 2/*MNT_DETACH*/);
		}
		
		if (result == -1) {
			ALP_TMTL("Unable to unmount package on %s: %s", record->mounted_foldername, strerror(errno));
			close(loopfd);
			goto skip_unmount;
		}
		
		result = ioctl(loopfd, LOOP_CLR_FD, 0);
		
		if (result == -1) {
			ALP_TMTL("Unable to clear loop device %s: %s", record->loopback_device, strerror(errno));
		}
		
	skip_unmount:	;
	
		unlink(record->mounted_foldername);
	
		g_free(record->mounted_foldername);
		g_free(record->loopback_device);	
	}
	
	} // if bar

	g_free(record->package_name);
	
	GList * iter;
	for (iter = record->solibs; iter; iter = g_list_next(iter)) {
		unlink((char*)iter->data);
		g_free(iter->data);
	}
	g_list_free(record->solibs);
	

	g_free(record->classpath);
	
	g_free(record->searchpath);
	
	g_free(record);

	S_STMT s;
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("BEGIN");
	S_PREPARE(s, "DELETE FROM live_metadata WHERE package_id = ?1");
	S_BIND_INT(s, 1, index.value);
	S_DO(s);
	S_PREPARE(s, "SELECT recycle FROM common_metadata WHERE package_id = ?1");
	S_BIND_INT(s, 1, index.value);
	int do_delete = S_STEP_ROW(s) && S_COL_INT(s, 0);
	S_FINALIZE(s);
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("COMMIT");
	
	if (do_delete) {
		alp_status_t result = alp_bundle_delete(index, 0);
		// ignore result
	}
	
	return ALP_STATUS_OK;
}

void handle_client_alp_prv_bundle_decref(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	AlpBundle index;
	int flags;
	
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &index.value);
	alp_ipc_message_unpack_int32(message, &flags);
	alp_ipc_message_unpack_end(message);

	alp_status_t result;
	
	// FIXME: decide whether flags should be passed, or filtered
	if (has_permission(connection, PERM_DECREF_POLICY)) {
		result = alp_prv_bundle_decref(index, flags);
	} else {
		result = ALP_STATUS_BUNDLEMGR_FAILURE;
	}

	int cookie;

	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_DECREF_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result);
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}
