/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#define _GNU_SOURCE // for dladdr() from dlfcn.h

#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <hiker/security.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <dlfcn.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <unistd.h>

#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"
#include "pm_perms.h"

static const int debug = 0; // disable some extra traces which would be too noisy

gboolean has_permission(AlpConnection * con, const char * policy_name)
{
  if (policy_name == NULL) // no policy established, let anyone through
    return TRUE;

  uid_t uid = alp_ipc_connection_get_peer_uid(con);
  
  // Root caller is allowed to do anything
  // FIXME: this isn't sensitive to a root caller which has diminished capabilities.
  // Is that serious?
  if (uid == 0) {
    if (debug)
    	ALP_TMTL("Access allowed to pid %d for policy %s because they are root", alp_ipc_connection_get_peer_pid(con), policy_name);
    return TRUE;
  }
      
  pid_t pid = alp_ipc_connection_get_peer_pid(con);
  
  alp_status_t status = alp_spf_query_open();
  if (status != ALP_STATUS_SPF_E_NOERR) {
    ALP_TMTL("Access disallowed to pid %d for policy %s because the SPF could not be initialized (error %x)", pid, policy_name, status);
    return FALSE;
  }
  
  alp_spf_query_set_subject_pid(pid);
  
  uint policy_result = 0;
  status = alp_spf_query_get_assertion_boolean((char*)policy_name, strlen(policy_name), &policy_result);
  if (status != ALP_STATUS_SPF_E_NOERR) {
    ALP_TMTL("Access disallowed to pid %d for policy %s because the policy could not be queried (error %x)", pid, policy_name, status);
    goto fail;
  }
  
  alp_spf_query_close();
  
  if (policy_result == 0) {
    ALP_TMTL("Access disallowed to pid %d for policy %s because the policy returned false", pid, policy_name);
  } else {
  	if (debug)
	    ALP_TMTL("Access allowed to pid %d for policy %s because the policy returned true", pid, policy_name);
  }
  
  return (policy_result != 0) ? TRUE : FALSE;
    
fail:
  alp_spf_query_close();
  return FALSE;
}

gboolean has_store_permission_by_store(AlpConnection * con, const char * policy_name, const char * store_name)
{
        gboolean permission = FALSE;

	S_STMT s;
	char buf[256];
	// SQL injection not a problem, as the policy name is hard-coded in the calling code
	sprintf(buf, "SELECT %s FROM stores WHERE stores.store_name = ?1", policy_name);
	S_PREPARE(s, buf);
	S_BIND_TEXT(s, 1, store_name);
	if (S_STEP_ROW(s)) {
		permission = has_permission(con, S_COL_TEXT(s,0));
	} else {
		ALP_TMTL("Access disallowed to pid %d for store policy %s because the store %s was invalid", alp_ipc_connection_get_peer_pid(con), policy_name, store_name);
		permission = FALSE; // invalid store name, should disallow
	}
	S_FINALIZE(s);
	
	return permission;
}

gboolean has_store_permission_by_bundle(AlpConnection * con, const char * policy_name, AlpBundle index)
{
        gboolean permission = FALSE;

	S_STMT s;
	char buf[256];
	// SQL injection not a problem, as the policy name is hard-coded in the calling code
	sprintf(buf, "SELECT %s FROM stores JOIN containers ON (containers.store_id = stores.store_id) WHERE containers.package_id = ?1", policy_name);
	S_PREPARE(s, buf);
	S_BIND_INT(s, 1, index.value);
	if (S_STEP_ROW(s)) {
		permission = has_permission(con, S_COL_TEXT(s,0));
	} else {
		ALP_TMTL("Access disallowed to pid %d for store policy %s because the bundle index %d was invalid", alp_ipc_connection_get_peer_pid(con), policy_name, index.value);
		permission = FALSE; // invalid bundle, should disallow
	}
	S_FINALIZE(s);
	
	return permission;
}
