/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <hiker/syspaths.h>
#include <hiker/global_settings.h>

#define DEFAULT_BUNDLE_WORK_PATH "/tmp/bundlemgr/work/"
#define BAR_LINK_PATH "links"
#define BAR_MOUNT_PATH "mounts"
#define BAR_LD_PATH "libs"
#define DEFAULT_BUNDLE_RW_PATH ALP_SYSPATH_BUNDLE_DATA
#define BAR_RW_PATH "bar/%s"
#define BAR_DEFAULT_SEARCHPATH "/rsc/^l/:/"
#define BAR_DEFAULT_TEXTDOMAIN_NAME "bar-^n"
#define BAR_DEFAULT_TEXTDOMAIN_DIR "rsc/locale"

#define NOTIFY_NAME "bundlemgr_d" 
#define BAR_LIBPATH "/libalp_%s.so"
#define BAR_DEFAULT_SEARCH "alp_main"

#define DEFAULT_IPC_CHANNEL "ALPBundleMgr"
#define DEFAULT_DATABASE ALP_SYSPATH_SYSTEM_CACHE "bundlemgr/bundle_mgr.db"
#define DEFAULT_LIVE_DATABASE "/tmp/bundlemgr/live_bundle_mgr.db"

#define SQLITE_RETRY_MS 10000

#define PM_CURRENT_DB_VERSION 16

#define BUNDLE_SCHEME_BAR 1
#define BUNDLE_SCHEME_GHOST 2
#define BUNDLE_SCHEME_JAVA 3

#define BUNDLE_SCHEME_NAME_BAR "bar"
#define BUNDLE_SCHEME_NAME_GHOST "GHost"
#define BUNDLE_SCHEME_NAME_JAVA "java"

// From:
//  http://www.iana.org/assignments/media-types/application/vnd.palm
//  http://developers.sun.com/techtopics/mobility/midp/articles/deploy/
#define MIME_TYPE_PRC	"application/vnd.palm"
#define MIME_TYPE_PRC_2	"application/x-pilot"
#define MIME_TYPE_BAR	"x-application/access-bar" //FIXME: Still need to decide on this
#define MIME_TYPE_JAR	"application/java-archive"
#define MIME_TYPE_JAD	"text/vnd.sun.j2me.app-descriptor"


enum {
	STATUS_UNINIT = 0,
	STATUS_CLIENT = 1,
	STATUS_SERVER = 2
};

extern alp_status_t register_volsvc_hooks();

extern int pm_status;

extern sqlite3 * db;

extern AlpChannel * pm_server_channel;
extern AlpConnection * pm_client_connection;

extern GHashTable * gConnectionTable;

extern AlpGlobalSettingsContext * gGSContext;

extern GList * pm_prv_searchpath(const char * path);

extern gboolean gNotify;
extern const char * gBundleRWPath;
extern const char * gBundleWorkPath;

extern gboolean gHoldNotifications;
extern gboolean gHoldNotificationInstall;
extern gboolean gHoldNotificationRegister;

extern int mkdir_p(const char * path, int mode);
extern int rmdir_contents(const char * path);
extern gchar * hypotheticalpath(const char * path);

extern void wipe_locales();

extern int alp_prv_associated_data(const char * name);

extern int alp_prv_folder_empty(const char * path, int status_on_missing);

extern alp_status_t alp_prv_bundle_setup_schema();

extern void alp_prv_package_clear_refs();

extern int lock;

extern alp_status_t my_sync_notify(const char * appID, const char * notifyType, void * details, int length, int queue, int flags, int timeout, int * handled);
extern alp_status_t my_async_notify(const char * appID, const char * notifyType, void * details, int length, int queue, int flags);
extern void destroy_notify_pool(void);

extern AlpBundle alp_prv_bundle_image_install_fd(const char * provide_filename, const char * provide_mimetype, int fd, const char * destination_store, int flags, GHashTable ** metadata_p, GHashTable ** properties_p,  AlpBundleProgressiveData * progress, AlpConnection * connection);

extern void alp_prv_bundle_clear_local_effects();

extern void reset_hold();

extern alp_status_t set_error(alp_status_t status);

extern int safe_sql_fillin(char * str, const char * fillin);
