/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright � 2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/notifymgr.h>
#include <hiker/ipc.h>
#include <hiker/fail.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <expat.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
//#include <linux/cramfs_fs.h>
#include <sys/types.h>
#include <dirent.h>
#include <hiker/global_settings.h>

#include <alp/pdk/gras.h>
#include <hiker/security.h>
#include <hiker/prv/apas.h>
#include <hiker/pdk/notifymgr_pdk.h>


#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"
#include "pm_bar.h"
#include "pm_msg.h"
#include "pm_perms.h"
#include "ghost_support.h"
#include "java_support.h"

#include "cramfs_interface.h"

/*! Update the names table: the mechanism to determine which package should answer to a particular
 *  name is not automatic, and requires invocation of this routine to update the names table which
 *  keeps track of that information.
 *
 *  @param[in]	name	If NULL, update the entire table for all names. Otherwise, only update the table
 * 			for that specific name.
 *
 *  Updates to the table may remove one or more entries if there are no longer packages with such names.
 */
void alp_prv_bundle_update_names(const char * name, int flags)
{
	// FIXME: reworking general naming of ghost bundles
	
	struct notification {
		const char * name;
		const char * type;
		void * data;
		int len;
	};
	GList * notifications = NULL;
	gboolean do_install_notify = FALSE;
	gboolean do_register_notify = FALSE;
	
	const int debug = 0;

	if (debug)
		ALP_TMTL("Begin update names %s", name);
	
	alp_status_t notify_result;

	// We need to record the notifications to send out, and then only process them afterwards,
	// so that we send them outside of the SQL transaction, otherwise we'd be preventing some
	// reentrant operations

	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
	        S_EXEC("BEGIN");
	if (name) {
                S_STMT s;
		S_PREPARE(s, "CREATE TEMPORARY TABLE orig_names AS SELECT names.package_id as package_id, name FROM names WHERE names.name = ?1");
		S_BIND_TEXT(s, 1, name);
		S_DO(s);
                S_PREPARE(s, "DELETE FROM names WHERE name = ?1;");
                S_BIND_TEXT(s, 1, name);
                S_DO(s);
 		S_PREPARE(s, "INSERT OR REPLACE INTO names (package_id, name, register_notify) "
                             " SELECT containers.package_id, containers.name, containers.register_notify FROM containers "
                             "    WHERE containers.name = ?1 AND containers.active "
                             "    ORDER BY containers.name_version ASC, containers.store_id ASC, containers.scheme_id ASC, containers.package_id ASC LIMIT 1");
                S_BIND_TEXT(s, 1, name);
                S_DO(s);
		S_PREPARE(s, "SELECT (orig_names.name ISNULL),names.name,names.package_id,orig_names.package_id,names.register_notify FROM names LEFT OUTER JOIN orig_names ON (names.name = ?1)");
		S_BIND_TEXT(s, 1, name);
		S_BEGIN_SELECT(s) {
			if (debug)
				ALP_TMTL("Pass 1, p=%d, name=%s, newid=%d, oldid=%d",
					S_COL_INT(s,0), S_COL_TEXT(s,1), S_COL_INT(s,2), S_COL_INT(s,3));
			if (S_COL_INT(s,2) == S_COL_INT(s,3))
				continue;
			if (S_COL_INT(s, 0)) {
				const char * name = S_COL_TEXT(s,1);
				AlpNotifyEventBundleNameAdded * data = g_malloc(4 + strlen(name) + 1);
				data->index.value = S_COL_INT(s,2);
				strcpy(data->name, name);

				if (gNotify)
					ALP_TMTL("Notifying of new name: '%s', index %d", data->name, data->index.value);

 				struct notification * n = g_new0(struct notification, 1);
 				n->name = NOTIFY_NAME;
 				n->type = ALP_NOTIFY_EVENT_BUNDLE_NAME_ADDED;
 				n->data = data;
 				n->len = 4+strlen(name)+1;

 				notifications = g_list_prepend(notifications, n);
 				
 				if (gNotify && S_COL_INT(s,4)) {
 					if (S_COL_INT(s,4) & 1) {
 						ALP_TMTL("Registering %s for one-shot %s\n", name, ALP_NOTIFY_EVENT_NOTIFY_INSTALL);
		 				notify_result = alp_notify_register(name, ALP_NOTIFY_EVENT_NOTIFY_INSTALL, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_ONETIME|ALP_NOTIFY_REGISTER_FORCE);
						do_install_notify = TRUE;
					}
 					if (S_COL_INT(s,4) & 2) {
 						ALP_TMTL("Registering %s for one-shot %s\n", name, ALP_NOTIFY_EVENT_NOTIFY_REGISTER);
		 				notify_result = alp_notify_register(name, ALP_NOTIFY_EVENT_NOTIFY_REGISTER, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_ONETIME|ALP_NOTIFY_REGISTER_FORCE);
						do_register_notify = TRUE;
					}
				}
			} else {
				const char * name = S_COL_TEXT(s,1);
				AlpNotifyEventBundleNameChanged * data = g_malloc(8 + strlen(name) + 1);
				data->old_index.value = S_COL_INT(s,3);
				data->new_index.value = S_COL_INT(s,2);
				strcpy(data->name, name);

				if (gNotify)
					ALP_TMTL("Notifying of changed name: '%s', old index was %d, new index is %d", data->name, data->old_index.value, data->new_index.value);

 				struct notification * n = g_new0(struct notification, 1);
 				n->name = NOTIFY_NAME;
 				n->type = ALP_NOTIFY_EVENT_BUNDLE_NAME_CHANGED;
 				n->data = data;
 				n->len = 8+strlen(name)+1;

 				notifications = g_list_prepend(notifications, n);

 				if (gNotify && S_COL_INT(s,4)) {
 					if (S_COL_INT(s,4) & 1) {
 						ALP_TMTL("Registering %s for one-shot %s\n", name, ALP_NOTIFY_EVENT_NOTIFY_INSTALL);
		 				notify_result = alp_notify_register(name, ALP_NOTIFY_EVENT_NOTIFY_INSTALL, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_ONETIME|ALP_NOTIFY_REGISTER_FORCE);
						do_install_notify = TRUE;
					}
 					if (S_COL_INT(s,4) & 2) {
 						ALP_TMTL("Registering %s for one-shot %s\n", name, ALP_NOTIFY_EVENT_NOTIFY_REGISTER);
		 				notify_result = alp_notify_register(name, ALP_NOTIFY_EVENT_NOTIFY_REGISTER, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_ONETIME|ALP_NOTIFY_REGISTER_FORCE);
						do_register_notify = TRUE;
					}
				}
			}
		}
		S_END_SELECT(s);
		S_PREPARE(s, "SELECT orig_names.name,orig_names.package_id FROM orig_names LEFT OUTER JOIN names ON (names.name = ?1) WHERE names.name ISNULL");
		S_BIND_TEXT(s, 1, name);
		S_BEGIN_SELECT(s) {
			const char * name = S_COL_TEXT(s,0);
			if (debug)
				ALP_TMTL("Pass 2, oldname=%s, oldid=%d",
					S_COL_TEXT(s,0), S_COL_INT(s,1));

			AlpNotifyEventBundleNameRemoved * data = g_malloc(4 + strlen(name) + 1);
			data->index.value = S_COL_INT(s,1);
			strcpy(data->name, name);

			if (gNotify)
				ALP_TMTL("Notifying of removed name: '%s', index was %d", data->name, data->index.value);

			struct notification * n = g_new0(struct notification, 1);
			n->name = NOTIFY_NAME;
			n->type = ALP_NOTIFY_EVENT_BUNDLE_NAME_REMOVED;
			n->data = data;
			n->len = 4+strlen(name)+1;

			notifications = g_list_prepend(notifications, n);
		}
		S_END_SELECT(s);
		S_PREPARE(s, "DROP TABLE orig_names");
		S_DO(s);
	} else {
	        //TODO: can the operation over all names be combined into a single INSERT statement?
	        S_STMT s;
		S_PREPARE(s, "CREATE TEMPORARY TABLE orig_names AS SELECT names.package_id as package_id, name FROM names");
		S_DO(s);
		S_PREPARE(s, "DELETE FROM names;");
                S_DO(s);
                S_PREPARE(s, "SELECT DISTINCT (common_metadata.name) FROM common_metadata");
                S_BEGIN_SELECT(s) {
                        name = S_COL_TEXT(s, 0);
                        S_STMT s2;
	 		S_PREPARE(s2, "INSERT OR REPLACE INTO names (package_id, name, register_notify) "
        	                     " SELECT containers.package_id, containers.name, containers.register_notify FROM containers "
                	             "    WHERE containers.name = ?1 AND containers.active "
                        	     "    ORDER BY containers.name_version ASC, containers.store_id ASC, containers.scheme_id ASC, containers.package_id ASC LIMIT 1");
			S_BIND_TEXT(s2, 1, name);
                            S_DO(s2);

                            if (debug)
	                            ALP_TMTL("Redoing names, saw name %s", name);
                }
                S_END_SELECT(s);
		S_PREPARE(s, "SELECT (orig_names.name ISNULL),names.name,names.package_id,orig_names.package_id,names.register_notify "
		             " FROM names LEFT OUTER JOIN orig_names ON (orig_names.name=names.name) "
		             "  ");
		S_BEGIN_SELECT(s) {
			if (debug)
				ALP_TMTL("Pass 1, p=%d, name=%s, newid=%d, oldid=%d",
					S_COL_INT(s,0), S_COL_TEXT(s,1), S_COL_INT(s,2), S_COL_INT(s,3));
			if (S_COL_INT(s,2) == S_COL_INT(s,3))
				continue;
			if (S_COL_INT(s, 0)) {
				const char * name = S_COL_TEXT(s,1);
				AlpNotifyEventBundleNameAdded * data = g_malloc(4 + strlen(name) + 1);
				data->index.value = S_COL_INT(s,2);
				strcpy(data->name, name);

				struct notification * n = g_new0(struct notification, 1);
				n->name = NOTIFY_NAME;
				n->type = ALP_NOTIFY_EVENT_BUNDLE_NAME_ADDED;
				n->data = data;
				n->len = 4+strlen(name)+1;

				notifications = g_list_prepend(notifications, n);

 				if (gNotify && S_COL_INT(s,4)) {
 					if (S_COL_INT(s,4) & 1) {
 						ALP_TMTL("Registering %s for one-shot %s\n", name, ALP_NOTIFY_EVENT_NOTIFY_INSTALL);
		 				notify_result = alp_notify_register(name, ALP_NOTIFY_EVENT_NOTIFY_INSTALL, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_ONETIME|ALP_NOTIFY_REGISTER_FORCE);
						do_install_notify = TRUE;
					}
 					if (S_COL_INT(s,4) & 2) {
 						ALP_TMTL("Registering %s for one-shot %s\n", name, ALP_NOTIFY_EVENT_NOTIFY_REGISTER);
		 				notify_result = alp_notify_register(name, ALP_NOTIFY_EVENT_NOTIFY_REGISTER, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_ONETIME|ALP_NOTIFY_REGISTER_FORCE);
						do_register_notify = TRUE;
					}
				}

			} else {
				const char * name = S_COL_TEXT(s,1);
				AlpNotifyEventBundleNameChanged * data = g_malloc(8 + strlen(name) + 1);
				data->old_index.value = S_COL_INT(s,3);
				data->new_index.value = S_COL_INT(s,2);
				strcpy(data->name, name);

				struct notification * n = g_new0(struct notification, 1);
				n->name = NOTIFY_NAME;
				n->type = ALP_NOTIFY_EVENT_BUNDLE_NAME_CHANGED;
				n->data = data;
				n->len = 8+strlen(name)+1;

				notifications = g_list_prepend(notifications, n);

 				if (gNotify && S_COL_INT(s,4)) {
 					if (S_COL_INT(s,4) & 1) {
 						ALP_TMTL("Registering %s for one-shot %s\n", name, ALP_NOTIFY_EVENT_NOTIFY_INSTALL);
		 				notify_result = alp_notify_register(name, ALP_NOTIFY_EVENT_NOTIFY_INSTALL, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_ONETIME|ALP_NOTIFY_REGISTER_FORCE);
						do_install_notify = TRUE;
					}
 					if (S_COL_INT(s,4) & 2) {
 						ALP_TMTL("Registering %s for one-shot %s\n", name, ALP_NOTIFY_EVENT_NOTIFY_REGISTER);
		 				notify_result = alp_notify_register(name, ALP_NOTIFY_EVENT_NOTIFY_REGISTER, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_ONETIME|ALP_NOTIFY_REGISTER_FORCE);
						do_register_notify = TRUE;
					}
				}
			}
		}
		S_END_SELECT(s);
		S_PREPARE(s, "SELECT orig_names.name,orig_names.package_id FROM orig_names "
			     " LEFT OUTER JOIN names ON (orig_names.name=names.name) WHERE names.name ISNULL");
		S_BEGIN_SELECT(s) {
			if (debug)
				ALP_TMTL("Pass 2, oldname=%s, oldid=%d",
					S_COL_TEXT(s,0), S_COL_INT(s,1));
			const char * name = S_COL_TEXT(s,0);
			AlpNotifyEventBundleNameRemoved * data = g_malloc(4 + strlen(name) + 1);
			data->index.value = S_COL_INT(s,1);
			strcpy(data->name, name);

			struct notification * n = g_new0(struct notification, 1);
			n->name = NOTIFY_NAME;
			n->type = ALP_NOTIFY_EVENT_BUNDLE_NAME_REMOVED;
			n->data = data;
			n->len = 4+strlen(name)+1;

			notifications = g_list_prepend(notifications, n);
		}
		S_END_SELECT(s);
		S_PREPARE(s, "DROP TABLE orig_names");
		S_DO(s);
	}

	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("COMMIT");

	if (debug)
		ALP_TMTL("End of recompute for names");

	notifications = g_list_reverse(notifications);

	while (notifications) {
		struct notification * n = notifications->data;

		alp_status_t err = 0;

		if (debug)
			ALP_TMTL("Bundle Manager sending name notification %s", n->type);
			
		if (gNotify) {
			if (gHoldNotifications) {
				S_STMT s;
				S_PREPARE(s, "INSERT INTO notifications (key, data) VALUES (?1, ?2);");
				S_BIND_TEXT(s, 1, n->type);
				S_BIND_BLOBLEN(s, 2, n->data, n->len);
				S_DO(s);
			} else {
				if (flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION) {
					ALP_TMTL("*************** Notification sent in nested transaction *****************");
				}
				err = my_async_notify(n->name, n->type, n->data, n->len, ALP_NOTIFY_QUEUE_DEFAULT, 0);
			}
		}
		
		if (err)
			ALP_TMTL("Err of notification send was %d", err);

		g_free(n->data);
		g_free(n);

		notifications = g_list_delete_link(notifications, notifications);
	}
	
	if (do_install_notify) {
		alp_status_t err;

		if (!gHoldNotifications || !gHoldNotificationInstall) {
			if (gHoldNotifications) {
				gHoldNotificationInstall = TRUE;
			} else {		
				if (flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION) {
					ALP_TMTL("*************** Notification sent in nested transaction *****************");
				}

				ALP_TMTL("Sending broadcast on %s", ALP_NOTIFY_EVENT_NOTIFY_INSTALL);
				err = my_async_notify(NOTIFY_NAME, ALP_NOTIFY_EVENT_NOTIFY_INSTALL, NULL, 0, ALP_NOTIFY_QUEUE_DEFAULT, ALP_NOTIFY_BROADCAST_FORCE);
			}
		}
	}
	if (do_register_notify) {
		alp_status_t err;
		
		if (!gHoldNotifications || !gHoldNotificationRegister) {
			if (gHoldNotifications) {
				gHoldNotificationRegister = TRUE;
			} else {
				if (flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION) {
					ALP_TMTL("*************** Notification sent in nested transaction *****************");
				}
		
				ALP_TMTL("Sending broadcast on %s", ALP_NOTIFY_EVENT_NOTIFY_REGISTER);
				err = my_async_notify(NOTIFY_NAME, ALP_NOTIFY_EVENT_NOTIFY_REGISTER, NULL, 0, ALP_NOTIFY_QUEUE_DEFAULT, ALP_NOTIFY_BROADCAST_FORCE);
			}
		}
	}

}

struct propdata {
	long long id;
	gchar * property;
	int sequence;
};

void register_props_2(gpointer key, gpointer value, gpointer user_data)
{
	struct propdata * p = user_data;
	S_STMT s;
	S_PREPARE(s, "INSERT INTO properties (package_id, sequence, property, key, value) VALUES(?1,?2,?3,?4,?5);");
	S_BIND_INT(s, 1, p->id);
	S_BIND_INT(s, 2, p->sequence);
	S_BIND_TEXT(s, 3, p->property);
	S_BIND_TEXT(s, 4, key);
	S_BIND_TEXT(s, 5, value);
	S_DO(s);
}

void register_props_3(int id, int seq, char *prop, char *key, char *value)
{
	S_STMT s;
	S_PREPARE(s, "INSERT INTO properties (package_id, sequence, property, key, value) VALUES(?1,?2,?3,?4,?5);");
	S_BIND_INT(s, 1, id);
	S_BIND_INT(s, 2, seq);
	S_BIND_TEXT(s, 3, prop);
	S_BIND_TEXT(s, 4, key);
	S_BIND_TEXT(s, 5, value);
	S_DO(s);
}

void register_props(gpointer key, gpointer value, gpointer user_data)
{
	struct propdata * p = user_data;
	p->property = (gchar*)key;
	p->sequence = 0;
	GList * iter;
	for (iter = value; iter; iter = g_list_next(iter)) {
		g_hash_table_foreach(iter->data, register_props_2, user_data);
		p->sequence++;
	}
}

#if 0
void dump2(gpointer key, gpointer value, gpointer data)
{
	printf("\t%s -> %s\n", key, value);
}

void dump1(gpointer key, gpointer value, gpointer data)
{
	GList * iter;
	printf("%s:\n", key);
	for (iter = value; iter; iter=g_list_next(iter)) {
	        printf("\t(list %p):\n", iter->data);
		g_hash_table_foreach(iter->data, dump2, NULL);
	}
}
#endif

static unsigned long register_bar(const char * filename, const char * store, int flags, gchar ** name, int * stub_id)
{
	int store_id;
	
	alp_status_t status;

	struct stat stbuf;

	if (stat(filename, &stbuf) == -1)
		return 0; // failure, not present

	S_STMT s;
	S_PREPARE(s, "SELECT store_id FROM stores WHERE store_name = ?1");
	S_BIND_TEXT(s, 1, store);
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return 0; // failure, unknown store
	}
	store_id = S_COL_INT(s, 0);
	S_FINALIZE(s);

	XML_Parser parser = NULL;
	int SIZE=65536;
	char buf[SIZE];

	GHashTable * properties = NULL;
	GHashTable * metadata = NULL;

	gboolean archive = FALSE;

	char fbuf[PATH_MAX];
	if (!realpath(filename, fbuf)) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}

	if (!S_ISDIR(stbuf.st_mode)) {
		struct cramfs_super super;

		archive = TRUE;

		int fd = open(fbuf, O_RDONLY);

		if (fd == -1) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_flat;
		}
		
		if (lseek(fd, 0, SEEK_SET) == -1) {
			ALP_TMTL("Seek error on package file: %s", strerror(errno));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_flat;
		}

		int len = read(fd, &super, sizeof(super));
		if (len < (int)sizeof(super)) {
			ALP_TMTL("Unable to read superblock from package file");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_flat;
		}

		if (super.magic != CRAMFS_MAGIC ||
		    memcmp(super.signature, CRAMFS_SIGNATURE, 16) != 0)
		{
			ALP_TMTL("Superblock mismatch");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
		    	goto fail_flat;
		}

		if (memcmp(super.name, "Binder Archive 1", 16) != 0)
		{
		    	ALP_TMTL("Superblock doesn't match expected cramfs name");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
		    	goto fail_flat;
		}

		if ((size_t)(super.root.offset << 2) >= super.size) {
			ALP_TMTL("Root of FS is outsize of FS bounds");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_flat;
		}

		if (super.size > (size_t)stbuf.st_size) {
			ALP_TMTL("Superblock length is longer than file length\n");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_flat;
		}

		parser = setup_parser(NULL);

		size_t pos = sizeof(super);
		size_t end = super.root.offset << 2;

		//FIXME: could the caller pass us the pre-processed metadata and properties?

		// pull the manifest out of the gap in the cramfs image, feeding it to the bar xml parser
		while (pos < end) {
			len = end - pos;
			void * buf = XML_GetBuffer(parser, SIZE);

			int r = read(fd, buf, (len > SIZE) ? SIZE : len);

			if (r < 0) {
				ALP_TMTL("Error reading manifest");
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail_flat;
			}
			int done = (r == 0);

			if (!done) {
				char * term = memchr((char*)buf, '\0', r);
				if (term) {
					done = 1;
					r = term - (char*)buf;
				}
				pos += r;
				if (pos >= end)
					done = 1;
			}

			if (!XML_ParseBuffer(parser, r, done)) {
				ALP_TMTL("Manifest parser aborted with failure");
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail_flat;
			}

			if (done)
				break;
		}
		// finish up parsing, delivering results to caller

		gboolean parse_result = finish_parser(parser, &properties, &metadata);

		if (!parse_result) {
			ALP_TMTL("Parse failed at end");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_flat;
		}

		close(fd);

		if (0) {
			fail_flat:
			if (fd != -1)
				close(fd);
			goto fail;
		}
	} else {
  		// folder

		char manifest_path[PATH_MAX];

		sprintf(manifest_path, "%s/%s", fbuf, ALP_BUNDLE_BAR_MANIFEST);

		int manifest_fd = open(manifest_path, O_RDONLY);
		if (manifest_fd == -1) {
			ALP_TMTL("Unable to register open manifest for package '%s': %s", manifest_path, strerror(errno));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_folder;
		}

		parser = setup_parser(NULL);

		// run the parser over the manifest. That will fill in the rest of the metadata for this
		// container
		for (;;) {
	  		int r_count;
			void * buf = XML_GetBuffer(parser, SIZE);
			if (!buf) {
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail_folder;
			}
			r_count = read(manifest_fd, buf, SIZE);
			if (r_count < 0) {
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail_folder;
			}
			if (! XML_ParseBuffer(parser, r_count, r_count == 0)) {
				ALP_TMTL("registration failure in XML parsing");
				status = ALP_STATUS_BUNDLEMGR_FAILURE;
				goto fail_folder;
			}
			if (r_count == 0)
				break;
		}

		close(manifest_fd);
		manifest_fd = -1;

		gboolean parse_result = finish_parser(parser, &properties, &metadata);

		if (!parse_result) {
			ALP_TMTL("Parse failed at end");
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_folder;
		}

		if (0) {
			fail_folder:

			if (manifest_fd != -1)
				close(manifest_fd);
			goto fail;
		}

	}

	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("BEGIN TRANSACTION");

	S_PREPARE(s,"INSERT INTO containers (scheme_id, store_id, open_foldername, archive_filename, did_register_notify, did_unregister_notify, name, register_notify, name_version) VALUES(?4,?3,?1,?2,0,0,?5,?6,?7) ;");
	if (!archive)
		S_BIND_TEXT(s, 1, fbuf);
	else
		S_BIND_TEXT(s, 2, fbuf);

	S_BIND_INT(s, 4, BUNDLE_SCHEME_BAR);
	S_BIND_INT(s, 3, store_id);
	S_BIND_TEXT(s, 5, g_hash_table_lookup(metadata, "name"));
	{
		int r = 0;
		GList * reg;
		if ((reg = (GList*)g_hash_table_lookup(properties, "notifications"))) {
			// Note: notifications/install and notifications/register have identical semantics.
			// The former triggers /alp/notify/install, the latter /alp/notify/register. Install
			// is deprecated
			// FIXME: remove 'install', leaving just 'register'
			if (g_hash_table_lookup(reg->data, "install"))
				r |= 1;
			if (g_hash_table_lookup(reg->data, "register"))
				r |= 2;
			S_BIND_INT(s, 6, r);
		}
	}
	// Name version is sorted ascending, but we want higher versions to come first, so we need to invert them.
	// SQLite3 apparently doesn't support DESC columns in INDEXes, so we juggle the data,
	// so it can be ASC. We really want to be able to use a multi-column index
	// in the update names query, so it's worth the bother to do this.
	S_BIND_UINT(s, 7, 0xFFFFFFFF - strtoul(g_hash_table_lookup(metadata, "version") ?: "0", NULL, 10) );
	S_DO(s);

#if 0
	printf("metadata: %p\n", metadata);
	g_hash_table_foreach(metadata, dump2, NULL);
	printf("properties: %p\n", properties);
	g_hash_table_foreach(properties, dump1, NULL);
#endif

	long long int container_id = sqlite3_last_insert_rowid(db);

	S_PREPARE(s, "INSERT INTO bar_metadata (package_id, bar_version, bar_searchpath, bar_filename, bar_textdomain_name, bar_textdomain_dir) VALUES (?1, ?2, ?3, ?4, ?5, ?6);");
	S_BIND_INT(s, 1, container_id);
	S_BIND_TEXT(s, 2, g_hash_table_lookup(metadata, "version"));
	S_BIND_TEXT(s, 3, g_hash_table_lookup(metadata, "searchpath") ?: BAR_DEFAULT_SEARCHPATH);
	S_BIND_TEXT(s, 4, filename);
	S_BIND_TEXT(s, 5, g_hash_table_lookup(metadata, "textdomain") ?: BAR_DEFAULT_TEXTDOMAIN_NAME);
	S_BIND_TEXT(s, 6, g_hash_table_lookup(metadata, "bindtextdomain") ?: BAR_DEFAULT_TEXTDOMAIN_DIR);
	S_DO(s);

	S_PREPARE(s, "INSERT INTO common_metadata (package_id, name, register_notify) VALUES (?1, ?2, ?3);");
	S_BIND_INT(s, 1, container_id);
	S_BIND_TEXT(s, 2, g_hash_table_lookup(metadata, "name"));
	{
		int r = 0;
		GList * reg;
		if ((reg = (GList*)g_hash_table_lookup(properties, "notifications"))) {
			// Note: notifications/install and notifications/register have identical semantics.
			// The former triggers /alp/notify/install, the latter /alp/notify/register. Install
			// is deprecated
			// FIXME: remove 'install', leaving just 'register'
			if (g_hash_table_lookup(reg->data, "install"))
				r |= 1;
			if (g_hash_table_lookup(reg->data, "register"))
				r |= 2;
			S_BIND_INT(s, 3, r);
		}
	}
	S_DO(s);

	GList * iter;
	for (iter = g_hash_table_lookup(properties, "export"); iter; iter = g_list_next(iter)) {
		GHashTable * e = iter->data;
                S_PREPARE(s, "INSERT INTO bar_exports (path, name, kind,  package_id) VALUES (?1,?2,?3, ?4)");
                gchar * library = g_hash_table_lookup(e, "library");
                gchar * classpath = g_hash_table_lookup(e, "classpath");
                if (library) {
			S_BIND_TEXT(s, 1, library);
			S_BIND_TEXT(s, 2, g_hash_table_lookup(e, "as"));
			S_BIND_TEXT(s, 3, "LD_LIBRARY_PATH");
		} else if (classpath) {
			S_BIND_TEXT(s, 1, classpath);
			S_BIND_TEXT(s, 3, "CLASSPATH");
		}
		S_BIND_INT(s, 4, container_id);
		S_DO(s);
	}

	for (iter = g_hash_table_lookup(properties, "reference"); iter; iter = g_list_next(iter)) {
		GHashTable * r = iter->data;
                S_PREPARE(s, "INSERT INTO bar_references (refer_to, optional,  package_id) VALUES (?1,?2, ?3)");
		S_BIND_TEXT(s, 1, g_hash_table_lookup(r, "to"));
		S_BIND_INT(s, 2, g_hash_table_lookup(r, "optional") ? 1 : 0);
		S_BIND_INT(s, 3, container_id);
		S_DO(s);
	}


	struct propdata p;
	p.id = container_id;

	// use subroutine to INSERT all properties
	g_hash_table_foreach(properties, register_props, &p);

	S_PREPARE(s,"UPDATE containers SET active = 1 WHERE package_id = ?1");
	S_BIND_INT(s, 1, container_id);
	S_DO(s);

	// We're inserting a package named 'foo': check to see if there is a stub package
	// named 'foo'. We'll delete it if it exists, and make sure appropriate notifications
	// get sent out.
	S_PREPARE(s, "SELECT containers.package_id FROM containers JOIN common_metadata ON (containers.package_id = common_metadata.package_id) WHERE common_metadata.name = ?1 AND containers.stub");
	S_BIND_TEXT(s, 1, g_hash_table_lookup(metadata, "name"));

	*stub_id = 0;

	if (S_STEP_ROW(s)) {
		*stub_id = S_COL_INT(s,0);
	  	S_FINALIZE(s);

		if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		  	S_EXEC("COMMIT");

		AlpBundle result2;
		result2.value = *stub_id;

		// FIXME: Not sending DELETE notification on the stub... I think that's right.

	  	alp_prv_bundle_unreg_notify(result2, flags);

	} else {
		S_FINALIZE(s);
		if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
			S_EXEC("COMMIT");
	}

    *name = g_strdup(g_hash_table_lookup(metadata, "name"));
    
    int uid = -1;
	int gid = -1;
	
	gpointer sec;
	if ((sec = g_hash_table_lookup(properties, "security"))) {

		GHashTable * security = ((GList*)sec)->data;

		gpointer present = security ? g_hash_table_lookup(security, "present") : 0;

		if (present) {
			// if there is a 'security' property with a 'present' key, then
			// hand the open package off to the SPF to register it and check
			// the security.xml

			AlpBundle b = {container_id};
			AlpBundleRef r = alp_prv_bundle_open(b, flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
			gchar * base = alp_bundle_ref_ro_pathname(r, "");

			if (base) {
				unsigned long handle = 0; // call SPF to get a security handle
				unsigned int pkg_count = 0;
				long spf_status = 0;

				/* open connection to SPF */
				spf_status = alp_spf_update_open();	// should return ALP_STATUS_SPF_E_NOERR

				if (spf_status != ALP_STATUS_SPF_E_NOERR) {
					ALP_TMTL("alp_spf_update_open returned error %x", spf_status);
				}

				/* enroll the package with SPF (should return ALP_STATUS_SPF_E_NOERR) */
				if (spf_status == ALP_STATUS_SPF_E_NOERR) {

					spf_status =  alp_spf_apas_enroll((char*) base,  (unsigned int*) &handle);  /* note handle's cast from long* to int*...portability concerns? */

					if (spf_status != ALP_STATUS_SPF_E_NOERR) {
						ALP_TMTL("alp_prv_spf_ps_enroll_package returned error %x", spf_status);
					}
					ALP_TMTL("handle is %x", handle);
				}

				/* increment the SPF package count (should return ALP_STATUS_SPF_E_NOERR) */
				if (spf_status == ALP_STATUS_SPF_E_NOERR) {
					ALP_TMTL("pkg_count starts out at %d", pkg_count);
					spf_status = alp_spf_ps_packageid_increment(
									handle,			// unsigned int packageid
									&pkg_count);	// unsigned int *out_count
					ALP_TMTL("pkg_count ends up as %d", pkg_count);
					if (spf_status != ALP_STATUS_SPF_E_NOERR) {
						ALP_TMTL("alp_prv_spf_packageid_increment returned error %x", spf_status);
					}
				}

				if (spf_status == ALP_STATUS_SPF_E_NOERR) {
					spf_status = alp_spf_apas_packageid_uid_get(handle, &uid);   
					if (spf_status != ALP_STATUS_SPF_E_NOERR) {
						ALP_TMTL("alp_spf_apas_packageid_uid_get(%d) returned error %x", handle, spf_status);
						uid = -1;
					}
				}

				if (spf_status == ALP_STATUS_SPF_E_NOERR) {
					spf_status = alp_spf_apas_packageid_gid_get(handle, &gid);   
					if (spf_status != ALP_STATUS_SPF_E_NOERR) {
						ALP_TMTL("alp_spf_apas_packageid_uid_get(%d) returned error %x", handle, spf_status);
						gid = -1;
					}
				}

				int an_error = (spf_status != ALP_STATUS_SPF_E_NOERR);

				/* close connection to SPF */
				spf_status = alp_spf_ps_close();	// should return ALP_STATUS_SPF_E_NOERR
				if (spf_status != ALP_STATUS_SPF_E_NOERR) {
					ALP_TMTL("alp_spf_ps_close returned error %x", spf_status);
				}


				if (!an_error && spf_status == ALP_STATUS_SPF_E_NOERR) {
					/* If we enrolled sucessfully, then put the handle in our table */
					S_PREPARE(s, "UPDATE containers SET security_handle = ?1 WHERE containers.package_id = ?2;");
					S_BIND_INT(s, 1, handle);
					S_BIND_INT(s, 2, container_id);
					S_DO(s);
				}


			}
			g_free(base);
			alp_prv_bundle_close(r, flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
		}
	}

	if (uid == -1 || gid == -1) {
	
		/* open connection to SPF */
		long spf_status = alp_spf_update_open();	// should return ALP_STATUS_SPF_E_NOERR
		
		unsigned int default_packageid = -1;
		
		if (spf_status == ALP_STATUS_SPF_E_NOERR) {
			spf_status = alp_spf_apas_packageid_default( &default_packageid );
		}
		
		if (spf_status == ALP_STATUS_SPF_E_NOERR && uid == -1) {
			// Get uid for unsigned bundle
			spf_status = alp_spf_apas_packageid_uid_get(default_packageid, &uid);   
			if (spf_status != ALP_STATUS_SPF_E_NOERR) {
				ALP_TMTL("alp_spf_apas_packageid_uid_get(%d) returned error %x", default_packageid, spf_status);
			}
		}

		if (spf_status == ALP_STATUS_SPF_E_NOERR && gid == -1) {
			// Get gid for unsigned bundle
			spf_status = alp_spf_apas_packageid_gid_get(default_packageid, &gid);   
			if (spf_status != ALP_STATUS_SPF_E_NOERR) {
				ALP_TMTL("alp_spf_apas_packageid_uid_get(%d) returned error %x", default_packageid, spf_status);
			}
		}
	
		alp_spf_update_close();
	}

	if (uid == -1) {
		ALP_TMTL("Root fallback for uid of global settings for bundle %d", container_id);
		uid = 0; // fallback to root -- this isn't expected to happen
	}
		
	if (gid == -1) {
		ALP_TMTL("Root fallback for gid of global settings for bundle %d", container_id);
		gid = 0;
	}

	// Create global settings entry for this bundle, if it doesn't already exist,
	// and give it the correct permissions
	
	gchar * key = g_strdup_printf("/bundles/%s", *name);
	
	
	if (gGSContext) {	
                uid_t current_uid;
		// Create new bundle entry, if it doesn't already exist
		alp_global_settings_set_key_directory(gGSContext, key);
		// Set permissions and ownership -- even if it already existed
		alp_global_settings_set_mode(gGSContext, key, 0775); // user rxw, group rwx, other rx
		alp_global_settings_get_uid(gGSContext, key, &current_uid);
		if (current_uid == 0)
		{
 		    alp_global_settings_set_uid(gGSContext, key, uid);
		    alp_global_settings_set_gid(gGSContext, key, gid);
		}
	}
	
	g_free(key);
	
	g_hash_table_destroy(metadata);
	g_hash_table_destroy(properties);

	metadata = NULL;
	properties = NULL;

	return container_id;

fail:
	if (parser)
		finish_parser(parser, NULL, NULL);
	if (properties)
		g_hash_table_destroy(properties);
	if (metadata)
		g_hash_table_destroy(metadata);

	set_error(status);
	return 0;
}

static unsigned long register_ghost(const char * filename, const char * store, int flags, gchar ** name, int * stub_id)
{
   //ALP_TMTL("register_ghost:  called \n");

	int store_id;
	int ghost_storage;

	char bundlename[64];
	
	char type_str[64], creator_str[64], dbname[64];
	int storage;
	
	memset(dbname, '\0', sizeof(dbname));
		
	int ret = sscanf(filename, "GHost:%d_%4s_%4s_%32c", &storage, type_str, creator_str, dbname);
		
	ALP_TMTL("register_ghost: storage %d type_str %s creator_str %s  dbname %s \n", storage, type_str, creator_str, dbname);
	
	S_STMT s;
	S_PREPARE(s, "SELECT store_id,ghost_storage FROM stores WHERE store_name = ?1");
	S_BIND_TEXT(s, 1, store);
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		return 0; // failure, unknown store
	}
	store_id = S_COL_INT(s, 0);
	ghost_storage = S_COL_INT(s,1);
	S_FINALIZE(s);

	sprintf(bundlename, "%s", filename);
	
	ALP_TMTL("register_ghost: inserting data into containers \n");

	S_EXEC("BEGIN TRANSACTION");

	S_PREPARE(s,"INSERT INTO containers (scheme_id, store_id, did_register_notify, did_unregister_notify, name) VALUES(?4,?3,0,0,?5) ;");

	S_BIND_INT(s, 4, BUNDLE_SCHEME_GHOST); // scheme_id
	S_BIND_INT(s, 3, store_id);
	S_BIND_TEXT(s, 5, bundlename);
	S_DO(s);

	long long int container_id = sqlite3_last_insert_rowid(db);
	
	ALP_TMTL("register_ghost: container_id %d \n", container_id);
	ALP_TMTL("register_ghost: inserting data into ghost_metadata \n");
    
	S_PREPARE(s, "INSERT INTO ghost_metadata (package_id, ghost_db_name, ghost_db_storage, ghost_db_type, ghost_db_creator, ghost_db_type_str, ghost_db_creator_str) VALUES (?1, ?2, ?3, ?4, ?5, ?6, ?7);");
	S_BIND_INT(s, 1, container_id);
	S_BIND_TEXT(s, 2, dbname);
	S_BIND_INT(s, 3, storage);
	S_BIND_INT(s, 4, char4_to_int(type_str));
	S_BIND_INT(s, 5, char4_to_int(creator_str));
	S_BIND_TEXT(s, 6, type_str);
	S_BIND_TEXT(s, 7, creator_str);
	S_DO(s);
	
	ALP_TMTL("register_ghost: inserting data into common_metadata bundlename %s  \n", bundlename);
	S_PREPARE(s, "INSERT INTO common_metadata (package_id, name) VALUES (?1, ?2);");
	S_BIND_INT(s, 1, container_id);
	S_BIND_TEXT(s, 2, bundlename);
	S_DO(s);

	S_PREPARE(s,"UPDATE containers SET active = 1 WHERE package_id = ?1");
	S_BIND_INT(s, 1, container_id);
	S_DO(s);

	
	if (strcmp(type_str, "appl") == 0) {
	
		//put name 
		register_props_3(container_id, 0, "application", "name", "$name");
    
		//put resource name which will retrieve icon
		register_props_3(container_id, 0, "application", "icon", "$largeicon.xpm");
	}

	if (ghost_storage != 0) {
		// Only apps on the internal store can be deleted with alp_bundle_delete
		register_props_3(container_id, 0, "application", "prevent-delete", "true");
	}

	// alp_copy_to_store and alp_move_to_store are not currently supported for ghost bundles
	register_props_3(container_id, 0, "application", "prevent-copy", "true");

	
	// We're inserting a package named 'foo': check to see if there is a stub package
	// named 'foo'. We'll delete it if it exists, and make sure appropriate notifications
	// get sent out.
	S_PREPARE(s, "SELECT containers.package_id FROM containers JOIN common_metadata ON (containers.package_id = common_metadata.package_id) WHERE common_metadata.name = ?1 AND containers.stub");
	S_BIND_TEXT(s, 1, bundlename);

	*stub_id = 0;
    
	
	if (S_STEP_ROW(s)) {
		*stub_id = S_COL_INT(s,0);
	  	S_FINALIZE(s);

	  	S_EXEC("COMMIT");

		AlpBundle result2;
		result2.value = *stub_id;

		// FIXME: Not sending DELETE notification on the stub... I think that's right.

	  	alp_prv_bundle_unreg_notify(result2, flags);

	} else {
		S_FINALIZE(s);
		S_EXEC("COMMIT");
	}

	*name = g_strdup(bundlename);

	// Create global settings key for ghost bundle, for benefit of GHost app preferences
	// There's no true owner of GHost, so set the key owner to root for now
	
	gchar * key = g_strdup_printf("/bundles/%s", *name);

	if (gGSContext) {	

		// Create new bundle entry, if it doesn't already exist
		alp_global_settings_set_key_directory(gGSContext, key);
		// Set permissions and ownership -- even if it already existed
		// FIXME: set this to an explicit GHost owner
		alp_global_settings_set_mode(gGSContext, key, 0777); // user rxw, group rwx, other wrx
		alp_global_settings_set_uid(gGSContext, key, 0);
		alp_global_settings_set_gid(gGSContext, key, 0);
	}
	
	g_free(key);
	

	return container_id;

fail:
	return 0;

}

static unsigned long register_java(const char * filename, const char * store, int flags, gchar ** name, int * stub_id)
{
   char midlet_name[PATH_MAX];
   int  midlet_id;
   int store_id;
	
   int ret = sscanf(filename, "java:%d_%s", &midlet_id, midlet_name);
   
   if (ret != 2) {
   	ALP_TMTL("Unable to parse java register name '%s'", filename);
   	return 0; // failure, invalid registratio name
   }
   
   char bundlename[PATH_MAX];
	
   	S_STMT s;
	S_PREPARE(s, "SELECT store_id FROM stores WHERE store_name = ?1");
	S_BIND_TEXT(s, 1, store);
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		return 0; // failure, unknown store
	}
	store_id = S_COL_INT(s, 0);
	S_FINALIZE(s);
   		
	sprintf(bundlename, "%s", filename);

	S_EXEC("BEGIN TRANSACTION");

	S_PREPARE(s,"INSERT INTO containers (scheme_id, store_id, did_register_notify, did_unregister_notify, name) VALUES(?4,?3,0,0,?5) ;");

	S_BIND_INT(s, 4, BUNDLE_SCHEME_JAVA); // scheme_id
	S_BIND_INT(s, 3, store_id);
	S_BIND_TEXT(s, 5, bundlename);
	S_DO(s);

	long long int container_id = sqlite3_last_insert_rowid(db);
			
	S_PREPARE(s, "INSERT INTO java_metadata (package_id, midlet_name, midlet_id) VALUES (?1, ?2, ?3);");
	S_BIND_INT(s, 1, container_id);
	S_BIND_TEXT(s, 2, midlet_name);
	S_BIND_INT(s, 3, midlet_id);
	S_DO(s);
	
	S_PREPARE(s, "INSERT INTO common_metadata (package_id, name) VALUES (?1, ?2);");
	S_BIND_INT(s, 1, container_id);
	S_BIND_TEXT(s, 2, bundlename);
	S_DO(s);

	S_PREPARE(s,"UPDATE containers SET active = 1 WHERE package_id = ?1");
	S_BIND_INT(s, 1, container_id);
	S_DO(s);
	//put name 
	gchar * tname = g_strdup_printf("^t/%s", midlet_name);
	register_props_3(container_id, 0, "application", "name", tname);
	g_free(tname);
	//put resource name which will retrieve icon
	register_props_3(container_id, 0, "application", "icon", "icon.png");
	register_props_3(container_id, 0, "application", "prevent-copy", "true");
	
	char update_str[PATH_MAX];
	char settings_str[PATH_MAX];
	
	// a b are work arounds 
	sprintf(update_str, "%s --update a b", filename);
	sprintf(settings_str, "%s --preferences a b", filename);
	
	register_props_3(container_id, 0, "application", "update-menu", update_str);
	register_props_3(container_id, 0, "application", "settings-menu", settings_str);
	
	// We're inserting a package named 'foo': check to see if there is a stub package
	// named 'foo'. We'll delete it if it exists, and make sure appropriate notifications
	// get sent out.
	S_PREPARE(s, "SELECT containers.package_id FROM containers JOIN common_metadata ON (containers.package_id = common_metadata.package_id) WHERE common_metadata.name = ?1 AND containers.stub");
	S_BIND_TEXT(s, 1, bundlename);

	*stub_id = 0;
    
	
	if (S_STEP_ROW(s)) {
		*stub_id = S_COL_INT(s,0);
	  	S_FINALIZE(s);

	  	S_EXEC("COMMIT");

		AlpBundle result2;
		result2.value = *stub_id;

		// FIXME: Not sending DELETE notification on the stub... I think that's right.

	  	alp_prv_bundle_unreg_notify(result2, flags);

	} else {
		S_FINALIZE(s);
		S_EXEC("COMMIT");
	}

	*name = g_strdup(bundlename);

	// Create global settings key for java bundle, for benefit of java app preferences
	// There's no true owner of java, so set the key owner to root for now
	
	if (gGSContext) {

		gchar * key = g_strdup_printf("/bundles/%s", *name);

		// Create new bundle entry, if it doesn't already exist
		alp_global_settings_set_key_directory(gGSContext, key);
		// Set permissions and ownership -- even if it already existed
		// FIXME: set this to an explicit java owner
		alp_global_settings_set_mode(gGSContext, key, 0777); // user rxw, group rwx, other wrx
		alp_global_settings_set_uid(gGSContext, key, 0);
		alp_global_settings_set_gid(gGSContext, key, 0);
		
		g_free(key);
	}

	return container_id;
 
fail:
	return 0;

}

AlpBundle alp_prv_bundle_register_bundle(const char * filename, const char * scheme, const char * store, int flags)
{
	// NOTE: the store and scheme must be set: this routine will not guess an appropriate store

	// FIXME: figure out security implications/limitations.
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;

		char fbuf[PATH_MAX];
		if (filename && scheme && strcmp(scheme,BUNDLE_SCHEME_NAME_BAR)==0)
			filename = realpath(filename, fbuf);

		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_REGISTER);
		alp_ipc_message_pack_string(message, filename);
		alp_ipc_message_pack_string(message, scheme);
		alp_ipc_message_pack_string(message, store);
		alp_ipc_message_pack_int32(message, flags);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			ALP_TMTL("some failure of IPC");
			// some failure of IPC
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return ALP_BUNDLE_NULL;
		}
		alp_ipc_message_destroy(message);

		AlpBundle result;
		alp_status_t status;
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result.value);
		alp_ipc_message_unpack_int32(response, &status);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);
		
		set_error(status);

		return result;
	}

	if (!filename) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_NULL; // failure, no name
	}

	if (!scheme || !strlen(scheme)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_NULL; // failure, invalid scheme
	}

	if (!store) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
  		return ALP_BUNDLE_NULL; // failure, invalid store
	}

  	gchar * name = NULL;
  	int stub_id = 0;

  	int id;

  	if (strcmp(scheme, BUNDLE_SCHEME_NAME_BAR) == 0)
		id = register_bar(filename, store,flags, &name, &stub_id);
	else if (strcmp(scheme, BUNDLE_SCHEME_NAME_GHOST) == 0)
		id = register_ghost(filename, store,flags, &name, &stub_id);
	else if (strcmp(scheme, BUNDLE_SCHEME_NAME_JAVA) == 0)
		id = register_java(filename, store,flags, &name, &stub_id);	
	else
		id = 0; // failure, unknown scheme

	if (id == 0) {
		g_free(name);
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		ALP_TMTL("id is zero registtartion failed\n");
		return ALP_BUNDLE_NULL;
	}

	AlpBundle result;

	result.value = id;

	if (!(flags & ALP_PRV_BUNDLE_FLAG_DO_NOT_NOTIFY)) {

		alp_prv_bundle_reg_notify(result, flags);

		// update the names table to figure out which package gets elected to use the name of this package.
		// It might or might not be the one we just registered.
		alp_prv_bundle_update_names(name, flags);
	}

	g_free(name);

	if (stub_id != 0) {
		if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
			S_EXEC("BEGIN");
			
		// unregister stub package
		S_STMT s;
		S_PREPARE(s,	"DELETE FROM ghost_metadata WHERE package_id = ?1;");
		S_BIND_INT(s, 1, stub_id);
		S_DO(s);
		S_PREPARE(s,	"DELETE FROM java_metadata WHERE package_id = ?1;");
		S_BIND_INT(s, 1, stub_id);
		S_DO(s);
		S_PREPARE(s,	"DELETE FROM bar_metadata WHERE package_id = ?1;");	
		S_BIND_INT(s, 1, stub_id);
		S_DO(s);
		S_PREPARE(s,	"DELETE FROM common_metadata WHERE package_id = ?1;");
		S_BIND_INT(s, 1, stub_id);
		S_DO(s);
		S_PREPARE(s,	"DELETE FROM containers WHERE package_id = ?1;");
		S_BIND_INT(s, 1, stub_id);
		S_DO(s);

		if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
			S_EXEC("COMMIT");

	}

	return result;
}

void handle_client_alp_prv_bundle_register_bundle(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	gchar * filename, * scheme, *store;
	int flags;

	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_string(message, &filename);
	alp_ipc_message_unpack_string(message, &scheme);
	alp_ipc_message_unpack_string(message, &store);
	alp_ipc_message_unpack_int32(message, &flags);
	alp_ipc_message_unpack_end(message);

	gboolean permission = has_store_permission_by_store(connection, PERM_REGISTER_POLICY_COLUMN, store);
	
	// FIXME: should do additional check on whether registered filename is bar, and within bar
	// folder, and apply a separate policy check on this -- or generally disallow it.
	
	AlpBundle result;
	
	if (permission) {
		result = alp_prv_bundle_register_bundle(filename, scheme, store, flags);
	} else {
		result = ALP_BUNDLE_NULL;
	}

	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_REGISTER_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result.value);
	alp_ipc_message_pack_int32(response, result.value ? ALP_STATUS_OK : alp_bundle_error());
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}


alp_status_t alp_prv_bundle_reg_notify(AlpBundle index, int flags)
{
	S_STMT s;

	const int debug = 0;

	ALP_FAIL_FATAL_IF(pm_status != STATUS_SERVER, "alp_prv_bundle_reg_notify is only server-side");

	if (debug)
		ALP_TMTL("Checking for registration notify of package %d", index.value);

	// notify about package registration
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("BEGIN");
	S_PREPARE(s, "SELECT 1 FROM containers WHERE package_id = ?1 AND did_register_notify = 0");
	S_BIND_INT(s, 1, index.value);
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
			S_EXEC("COMMIT");
		if (debug)
			ALP_TMTL("Nope, must already have been notified");
		return 0;
	}
	S_FINALIZE(s);
	if (debug)
		ALP_TMTL("Updating and notifying");
	S_PREPARE(s, "UPDATE containers SET did_register_notify = 1, did_unregister_notify = 0, active = 1 WHERE package_id = ?1");
	S_BIND_INT(s, 1, index.value);
	S_DO(s);
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
	  	S_EXEC("COMMIT");

	if (gNotify) {
		if (gHoldNotifications) {
			S_STMT s;
			S_PREPARE(s, "INSERT INTO notifications (key, data) VALUES (?1,?2);");
			S_BIND_TEXT(s, 1, ALP_NOTIFY_EVENT_BUNDLE_REGISTER);
			S_BIND_BLOBLEN(s, 2, &index, sizeof(AlpBundle));
			S_DO(s);
		} else {
			alp_status_t err = my_async_notify(NOTIFY_NAME, ALP_NOTIFY_EVENT_BUNDLE_REGISTER, &index, sizeof(AlpBundle), ALP_NOTIFY_QUEUE_DEFAULT, 0);
			if (err != ALP_STATUS_OK) {
				ALP_TMTL("Broadcast of package register failed");
			}
			//ALP_FAIL_NON_FATAL_IF(err != ALP_STATUS_OK, "broadcast of package registration failed (err=%x)", err);
		}
	}

	return 0;
}

alp_status_t alp_prv_bundle_unreg_notify(AlpBundle index, int flags)
{
	S_STMT s;

	const int debug = 0;

	ALP_FAIL_FATAL_IF(pm_status != STATUS_SERVER, "alp_prv_bundle_unreg_notify is only server-side");

	if (debug)
		ALP_TMTL("Checking for unregistration notify of package %d", index.value);

	// notify about package unregistration
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("BEGIN");
	S_PREPARE(s, "SELECT 1 FROM containers WHERE package_id = ?1 AND did_unregister_notify = 0");
	S_BIND_INT(s, 1, index.value);
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
			S_EXEC("COMMIT");
		if (debug)
			ALP_TMTL("Nope, must already have been notified");
		return 0;
	}
	S_FINALIZE(s);
	if (debug)
		ALP_TMTL("Updating and notifying");
	S_PREPARE(s, "UPDATE containers SET did_unregister_notify = 1, did_register_notify = 0, active = 0 WHERE package_id = ?1");
	S_BIND_INT(s, 1, index.value);
	S_DO(s);
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
	  	S_EXEC("COMMIT");

	if (gNotify) {
		if (gHoldNotifications) {
			S_STMT s;
			S_PREPARE(s, "INSERT INTO notifications (key, data) VALUES (?1,?2);");
			S_BIND_TEXT(s, 1, ALP_NOTIFY_EVENT_BUNDLE_UNREGISTER);
			S_BIND_BLOBLEN(s, 2, &index, sizeof(AlpBundle));
			S_DO(s);
		} else {
			if (flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION) {
				ALP_TMTL("*************** Potentially synchronous notification sent in nested transaction *****************");
			}
				
			// should be sync
			int handled = 0;
			alp_status_t err = my_async_notify(NOTIFY_NAME, ALP_NOTIFY_EVENT_BUNDLE_UNREGISTER, &index, sizeof(AlpBundle), ALP_NOTIFY_QUEUE_DEFAULT, 0);
			if (err != ALP_STATUS_OK) {
				ALP_TMTL("Broadcast of package unregister failed");
			}
			//ALP_FAIL_NON_FATAL_IF(err != ALP_STATUS_OK, "broadcast of package unregistration failed (err=%x)", err);
		}
	}
	return 0;
}

alp_status_t alp_prv_bundle_deactivate(AlpBundle index, int flags)
{
	S_STMT s;

	ALP_FAIL_FATAL_IF(pm_status != STATUS_SERVER, "alp_prv_bundle_unreg_notify is only server-side");

	const int debug = 0;

	if (debug)
		ALP_TMTL("Checking for unregistration notify of package %d", index.value);

	// notify about package unregistration
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("BEGIN");
	S_PREPARE(s, "SELECT 1 FROM containers WHERE package_id = ?1 AND active = 1");
	S_BIND_INT(s, 1, index.value);
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
			S_EXEC("COMMIT");
		if (debug)
			ALP_TMTL("Nope, must already have been deactivated");
		return 0;
	}
	S_FINALIZE(s);
	if (debug)
		ALP_TMTL("Deactivating");
	S_PREPARE(s, "UPDATE containers SET active = 0 WHERE package_id = ?1");
	S_BIND_INT(s, 1, index.value);
	S_DO(s);
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
	  	S_EXEC("COMMIT");

	return 0;
}

alp_status_t alp_prv_bundle_unregister_bundle(AlpBundle index, int flags)
{
	// FIXME: figure out security implications/limitations.
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;

		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_UNREGISTER);
		alp_ipc_message_pack_int32(message, index.value);
		alp_ipc_message_pack_int32(message, flags);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			ALP_TMTL("some failure of IPC");
			// some failure of IPC
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		alp_ipc_message_destroy(message);

		alp_status_t result;
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);

		return set_error(result);
	}

  if (alp_bundle_is_NULL(index))
    return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);


  gchar * name = NULL;

  S_STMT(s);
  if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
    S_EXEC("BEGIN");

  int refcount = 0;

  // check to make sure package isn't open
  S_PREPARE(s, "SELECT refcount FROM live_metadata WHERE package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  if (S_STEP_ROW(s)) {
  	refcount = S_COL_INT(s, 0);
  }
  S_FINALIZE(s);

  // if it is open, decide to give up, or force it closed
  if (refcount > 0) {
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
  		S_EXEC("COMMIT");
  	if (flags & ALP_PRV_BUNDLE_FLAG_FORCE_UNREGISTER) {
  		// pass flags, particularly for currently hypothetical ALP_PRV_BUNDLE_FLAG_FORCE_UNMOUNT
  		alp_status_t result = alp_prv_bundle_force_close(index, flags);
  		// propagate potential error
  		if (result != ALP_STATUS_OK)
  			return result;
  	} else {
  		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
  	}
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
	  	S_EXEC("BEGIN");
  }

  // not names.name, because we want the name that the package _might_ be registered with, even
  // if it doesn't own that name at the moment
  S_PREPARE(s,  "SELECT common_metadata.name FROM common_metadata WHERE package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  S_BEGIN_SELECT(s) {
	g_free(name);
	  name = g_strdup(S_COL_TEXT(s, 0));
  }
  S_END_SELECT(s);

  int need_notify;
  if (flags & ALP_PRV_BUNDLE_FLAG_DO_NOT_NOTIFY) {
  	need_notify = 0;
  } else {
	S_PREPARE(s,"SELECT 1 FROM containers WHERE package_id = ?1 AND did_unregister_notify = 0");
	S_BIND_INT(s, 1, index.value);
	need_notify = S_STEP_ROW(s);
	S_FINALIZE(s);
  }

  int remaining_names = 0;

  S_PREPARE(s, "SELECT COUNT(*) FROM containers JOIN common_metadata ON (containers.package_id = common_metadata.package_id) WHERE common_metadata.name = ?1");
  S_BIND_TEXT(s, 1, name);
  if (S_STEP_ROW(s)) {
  	remaining_names = S_COL_INT(s, 0);
  }
  S_FINALIZE(s);


  int stub_id = 0;
// We're deleting the last package named 'foo': check to see if there is any associated
// rw data. If there is, install a new stub package named 'foo', and send out notification.
  if (remaining_names == 1 && alp_prv_associated_data(name) && !(flags & ALP_PRV_BUNDLE_FLAG_DO_NOT_MAKE_STUB)) {

	// FIXME: hardcoded scheme id
	S_PREPARE(s,"INSERT INTO containers (scheme_id, stub, active, name) VALUES(1, 1, 1, ?2) ;");
	S_BIND_TEXT(s, 2, name);
	S_DO(s);

	long long int container_id = sqlite3_last_insert_rowid(db);

  	// FIXME: we probably shouldn't need bar_metadata for this package, but I'm
  	// worried about harming various joins by not having an appropriate entry
	S_PREPARE(s, "INSERT INTO bar_metadata (package_id) VALUES (?1);");
	S_BIND_INT(s, 1, container_id);
	S_DO(s);

	S_PREPARE(s, "INSERT INTO common_metadata (package_id, name) VALUES (?1, ?2);");
	S_BIND_INT(s, 1, container_id);
	S_BIND_TEXT(s, 2, name);
	S_DO(s);

	S_PREPARE(s,"UPDATE containers SET active = 1, did_register_notify = 1, did_unregister_notify = 0 WHERE package_id = ?1");
	S_BIND_INT(s, 1, container_id);
	S_DO(s);

	stub_id = container_id;

	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("COMMIT");


  } else {
	if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
		S_EXEC("COMMIT");
  }

  if (need_notify) {
  	alp_prv_bundle_deactivate(index, flags);

	  //update the name table, possibly sending out notifications
	  alp_prv_bundle_update_names(name, flags);
  }

  if (stub_id != 0) {
	AlpBundle index2;

	index2.value = stub_id;

  	alp_prv_bundle_reg_notify(index2, flags);
  }

  //send out unregister notification about the package index itself

  if (need_notify) {
  	alp_prv_bundle_unreg_notify(index, flags);
  }

  if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
  	S_EXEC("BEGIN");

  S_PREPARE(s, "SELECT security_handle FROM containers WHERE containers.package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  if (S_STEP_ROW(s)) {
  	if (!S_COL_ISNULL(s, 0)) {
  		long handle = S_COL_INT(s, 0);
  		S_FINALIZE(s);
  		if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
			S_EXEC("COMMIT");

  		// tell SPF to decrement refcount on handle
		int spf_status = 0;
		unsigned int pkg_count = 0;

		/* open connection to SPF */
		spf_status = alp_spf_update_open();	// should return ALP_STATUS_SPF_E_NOERR

		if (spf_status != ALP_STATUS_SPF_E_NOERR) {
			ALP_TMTL("alp_spf_update_open returned error %x", spf_status);
		}

		/* SPF decrement package refcount */
		spf_status = alp_spf_ps_packageid_decrement(
							handle,			// unsigned int packageid
							&pkg_count);	// unsigned int *out_count

		ALP_TMTL("alp_spf_ps_packageid_decrement set pkg_count to %d", pkg_count);

		if (spf_status != ALP_STATUS_SPF_E_NOERR) {
			ALP_TMTL("alp_spf_ps_packageid_decrement returned error %x", spf_status);
		}

		/* close connection to SPF */
		spf_status = alp_spf_ps_close();	// should return ALP_STATUS_SPF_E_NOERR

		if (spf_status != ALP_STATUS_SPF_E_NOERR) {
			ALP_TMTL("alp_spf_ps_close returned error %x", spf_status);
		}

		if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
  			S_EXEC("BEGIN");
  	} else {
  		S_FINALIZE(s);
	}
  } else {
  	S_FINALIZE(s);
  }

  S_PREPARE(s,	"DELETE FROM properties WHERE package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  S_DO(s);
  S_PREPARE(s,	"DELETE FROM bar_metadata WHERE package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  S_DO(s);
  S_PREPARE(s,	"DELETE FROM ghost_metadata WHERE package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  S_DO(s);
  S_PREPARE(s,	"DELETE FROM java_metadata WHERE package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  S_DO(s);
  S_PREPARE(s,	"DELETE FROM bar_exports WHERE package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  S_DO(s);
  S_PREPARE(s,	"DELETE FROM bar_references WHERE package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  S_DO(s);
  S_PREPARE(s,	"DELETE FROM common_metadata WHERE package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  S_DO(s);
  S_PREPARE(s,	"DELETE FROM containers WHERE package_id = ?1;");
  S_BIND_INT(s, 1, index.value);
  S_DO(s);
  if (!(flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION))
  	S_EXEC("COMMIT");

  if (name)
    g_free(name);

  return ALP_STATUS_OK;
}

void handle_client_alp_prv_bundle_unregister_bundle(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	AlpBundle index;
	int flags;

	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &index.value);
	alp_ipc_message_unpack_int32(message, &flags);
	alp_ipc_message_unpack_end(message);

	gboolean permission = has_store_permission_by_bundle(connection, PERM_UNREGISTER_POLICY_COLUMN, index);
	
	alp_status_t result;
	
	if (permission) {
		result = alp_prv_bundle_unregister_bundle(index, flags);
	} else {
		result = ALP_STATUS_BUNDLEMGR_FAILURE;
	}

	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_UNREGISTER_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result);
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

alp_status_t alp_bundle_delete(AlpBundle index, int flags)
{
	// NOTE: API is a cheat: the 1/0 everything parameter is really a bit-field
	// FIXME: figure out security implications/limitations.
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;

		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_DELETE);
		alp_ipc_message_pack_int32(message, index.value);
		alp_ipc_message_pack_int32(message, flags);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			ALP_TMTL("some failure of IPC");
			// some failure of IPC
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		alp_ipc_message_destroy(message);

		alp_status_t result;
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);

		return set_error(result);
	} else if (pm_status == STATUS_SERVER) {
		S_STMT s;
retry:		S_PREPARE(s, "SELECT open_foldername,archive_filename,refcount,"
				" common_metadata.name,names.package_id,scheme_id,ghost_db_type,ghost_db_creator,ghost_db_storage, midlet_id FROM containers "
				" OUTER LEFT JOIN live_metadata ON (containers.package_id=live_metadata.package_id) "
				" LEFT OUTER JOIN ghost_metadata ON (containers.package_id=ghost_metadata.package_id) "
				" LEFT OUTER JOIN java_metadata ON (containers.package_id=java_metadata.package_id) "
				" LEFT JOIN common_metadata ON (containers.package_id=common_metadata.package_id) "
				" LEFT OUTER JOIN names ON (common_metadata.name=names.name) WHERE containers.package_id = ?1;");
		S_BIND_INT(s, 1, index.value);
		if (S_STEP_ROW(s)) {
			int opened = S_COL_INT(s,2) > 0;

			if (opened && (flags & ALP_PRV_BUNDLE_FLAG_LAZY_DELETE)) {
				S_FINALIZE(s);

				ALP_TMTL("Marking it for lazy delete");

				// FIXME: consider moving the underlying file/folder for a bar, giving
				// it a 'RECYCLE.' name so that it is automatically deleted on rescan/boot.
				// But this gets complicated if it is a folder, as we'll need to update the
				// symlink simultaneously (!) with the folder rename.

				S_PREPARE(s, "UPDATE common_metadata SET recycle = 1 WHERE common_metadata.package_id = ?1");
				S_BIND_INT(s, 1, index.value);
				S_DO(s);
				return ALP_STATUS_OK; // Well, sort-of
			}
			if (opened && !(flags & ALP_PRV_BUNDLE_FLAG_FORCE_UNREGISTER)) {
				S_FINALIZE(s);
				// don't bother trying, it won't work
				return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			}
			gchar * foldername = g_strdup(S_COL_TEXT(s, 0));
			gchar * filename = g_strdup(S_COL_TEXT(s, 1));
			gchar * package_name = g_strdup(S_COL_TEXT(s, 3));
			int named = S_COL_INT(s,4) != 0;
			int scheme = S_COL_INT(s,5);
			int ghost_db_type = S_COL_INT(s,6);
			int ghost_db_creator = S_COL_INT(s,7);
			int ghost_db_storage = S_COL_INT(s,8);
			int midlet_id = S_COL_INT(s,9);
			
			S_FINALIZE(s);
			
			if (scheme == BUNDLE_SCHEME_GHOST &&
			    (ghost_db_type != (int)char4_to_int("appl") ||
			    ghost_db_storage != 0)) {
				// Only applications on the internal storage can be safely/sensible
				// deleted. 
			    	ALP_TMTL("Disallowing deletion of ghost bundle %d. It can only be unregistered.", index.value);
				g_free(foldername);
				g_free(filename);
				g_free(package_name);

				return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			}
					

			// Step 1: ask whether the delete should be allowed
			if (gNotify) {
				int handled = 0;

				AlpNotifyEventBundleDelete * d = g_malloc0(4 + 4 + 4+ strlen(package_name) + 1);

				d->index = index;
				d->named = named;
				d->flags = flags;
				strcpy(d->name, package_name);

				ALP_TMTL("Delete notification for package index %d, flags=%d, named=%d, name=%s", d->index, d->flags, d->named, d->name);

				if (gHoldNotifications) {
					// Sorry, I can't see a reason to hold this notification for later
				} else {
					alp_status_t err = my_sync_notify(NOTIFY_NAME, ALP_NOTIFY_EVENT_BUNDLE_DELETE, d, 4 + 4 + 4 + strlen(package_name) + 1, ALP_NOTIFY_QUEUE_DEFAULT, 0, 0, &handled);
					//ALP_FAIL_NON_FATAL_IF(err != ALP_STATUS_OK, "broadcast of package unregistration failed (err=%x)", err);
				}

				g_free(d);

				if (handled & 1 && !(flags & ALP_PRV_BUNDLE_FLAG_FORCE_DELETE)) {
					g_free(foldername);
					g_free(filename);
					g_free(package_name);

					return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
				}

			}
			
	
			// keep for step 4
			gchar * rw_folder = alp_bundle_rw_pathname(index, "", FALSE);
			gchar * unsaved_settings = alp_bundle_unsaved_settings(index, NULL);
			gchar * saved_settings = alp_bundle_saved_settings(index, NULL);

			// Step 2: Unregister the package

			// pass flags to unregister, particularly for ALP_PRV_BUNDLE_FLAG_FORCE_UNREGISTER
			int unreg_flags = flags;

			// If we're trying to get rid of the RW data, don't make a stub package to
			// preserve it!
			if (unreg_flags & 1)
				unreg_flags |= ALP_PRV_BUNDLE_FLAG_DO_NOT_MAKE_STUB;

			alp_status_t result = alp_prv_bundle_unregister_bundle(index, unreg_flags);
			if (result != ALP_STATUS_OK) {
				// if the unregister failed, bail out, leaving everything the way it was
				
				g_free(foldername);
				g_free(filename);
				g_free(package_name);

				// propagate error from alp_prv_bundle_unregister_bundle
				return result;
			}

			// Step 3: remove underlying material

			if (scheme == BUNDLE_SCHEME_GHOST ) {
				int ret = -1;
				if (ghost_start() == ALP_STATUS_OK) 
				{
					ret = alp_gras_app_delete(GSESSION, ghost_db_type, ghost_db_creator, ghost_db_storage);
					ghost_stop(ret != ALP_STATUS_OK && ALP_STATUS_VALUE(ret) != ALP_STATUS_GRAS_GARNET_ERROR);
				}

				if (ret) {
					ALP_TMTL("alp_gras_app_delete() failed");
				}
			} else if (scheme == BUNDLE_SCHEME_JAVA) {
				//call remove of JVlite
				
				int ret = java_delete(midlet_id);
				if (ret != 0) {
					// FIXME: quick hack: if the java UI refuses deletion, reregister it with
					// ourselves. This will give a new bundle index which the caller will
					// have to cope with.
					
					ALP_TMTL("java_delete() failed");
					
					// FIXME: shouldn't hard-code 'internal' store
					AlpBundle rebundle = alp_prv_bundle_register_bundle(package_name, BUNDLE_SCHEME_NAME_JAVA, "internal", flags);
					// Not much point in checking the result: either the reregister
					// worked or it didn't.
					ALP_TMTL("After java_delete(), reregistering '%s' as bundle index %d", package_name, rebundle.value);

					g_free(foldername);
					g_free(filename);
					g_free(package_name);
					g_free(rw_folder);
					g_free(unsaved_settings);
					g_free(saved_settings);
					
					// Return error, meaning that the bundle couldn't be deleted
					return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);				
				}
				
			} else {
				if (foldername) {
					rmdir_contents(foldername);
					rmdir(foldername);
				} else if (filename) {
					unlink(filename);
				}
			}

			// Step 4: remove associated data

			if (flags & 1) { //ALP_PACKAGE_FLAG_DELETE_EVERYTHING
				if (rw_folder) {
					rmdir_contents(rw_folder);
					rmdir(rw_folder);
				}
				
				if (gGSContext) {
				
					gchar * base = g_strdup_printf("/bundles/%s", package_name);
				
					// Delete everything under unsaved prefs
					// E.g. /bundles/bar:com.foo/unsaved/fred=george
					alp_global_settings_delete_tree( gGSContext, unsaved_settings );
				
					// Delete saved settings directory, _if empty_
					alp_global_settings_delete( gGSContext, saved_settings );
					
					// Delete entire bundle directory, _if now empty_
					alp_global_settings_delete( gGSContext, base );
				
					g_free(base);
				}
			}


			g_free(foldername);
			g_free(filename);
			g_free(package_name);
			g_free(rw_folder);
			g_free(unsaved_settings);
			g_free(saved_settings);

		} else {
			S_FINALIZE(s);
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
	}

	return ALP_STATUS_OK;
}

void handle_client_alp_bundle_delete(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	AlpBundle index;
	int everything;

	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &index.value);
	alp_ipc_message_unpack_int32(message, &everything);
	alp_ipc_message_unpack_end(message);

	gboolean permission = has_store_permission_by_bundle(connection, PERM_DELETE_POLICY_COLUMN, index);
	
	alp_status_t result;
	
	if (permission) {
		result = alp_bundle_delete(index, everything);
	} else {
		result = ALP_STATUS_BUNDLEMGR_FAILURE;
	}

	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_DELETE_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result);
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

gboolean gHoldNotifications = TRUE;
gboolean gHoldNotificationInstall = FALSE;
gboolean gHoldNotificationRegister = FALSE;

alp_status_t alp_prv_bundle_hold_notifications(gboolean hold)
{
	// FIXME: figure out security implications/limitations.
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;

		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PRV_BUNDLE_HOLD_NOTIFICATIONS);
		alp_ipc_message_pack_int32(message, hold);
		alp_ipc_message_pack_end(message);

		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			ALP_TMTL("some failure of IPC");
			// some failure of IPC
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		alp_ipc_message_destroy(message);

		alp_status_t result;
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);

		return set_error(result);
	}

	if (hold) {
		if (gHoldNotifications)
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			
		gHoldNotifications = TRUE;
		return ALP_STATUS_OK;
	} // else

	if (!gHoldNotifications)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);


	/* FIXME: fold notifications
	
		Convert bundle name DELETE/.../ADD -> .../CHANGE
		Convert bundle name ADD/.../[CHANGE/.../]DELETE -> .../...
		Convert bundle index ADD/.../DELETE -> ...
		Convert bundle index
	*/

	struct n {
		char * key;
		void * data;
		int data_len;
	};
	
	GList * ns = NULL;
	
	S_EXEC("BEGIN");
 	S_STMT s;
	S_PREPARE(s, "SELECT key,data FROM notifications ORDER BY seq ASC");
	S_BEGIN_SELECT(s) {
		const char * key = S_COL_TEXT(s,0);
		const void * data = S_COL_BLOB(s,1);
		int data_len = S_COL_BYTES(s,1);
		
		struct n * np = g_malloc(sizeof(struct n));
		np->key = g_strdup(key);
		np->data = g_memdup(data, data_len);
		np->data_len = data_len;
		
		ns = g_list_prepend(ns, np);
		
	}
	S_END_SELECT(s);
	S_PREPARE(s, "DELETE FROM notifications;");
	S_DO(s);
	S_EXEC("COMMIT");
	
	ns = g_list_reverse(ns);
	
	while (ns) {
		struct n * np = ns->data;
		ns = g_list_delete_link(ns,ns);
		
		alp_status_t err = ALP_STATUS_OK;
		
		while (g_main_context_pending(NULL)) {
			g_main_context_iteration(NULL, FALSE);
		}
		
		if (gNotify) {
			err = my_async_notify(NOTIFY_NAME, np->key, np->data, np->data_len, ALP_NOTIFY_QUEUE_DEFAULT, 0);
		}
		
		
		g_free(np->data);
		g_free(np->key);
		
		g_free(np);
	}
	
	if (gHoldNotificationInstall) {
		alp_status_t err;

		ALP_TMTL("Sending broadcast on %s", ALP_NOTIFY_EVENT_NOTIFY_INSTALL);
		err = my_async_notify(NOTIFY_NAME, ALP_NOTIFY_EVENT_NOTIFY_INSTALL, NULL, 0, ALP_NOTIFY_QUEUE_DEFAULT, ALP_NOTIFY_BROADCAST_FORCE);

	}

	if (gHoldNotificationRegister) {
		alp_status_t err;

		ALP_TMTL("Sending broadcast on %s", ALP_NOTIFY_EVENT_NOTIFY_REGISTER);
		err = my_async_notify(NOTIFY_NAME, ALP_NOTIFY_EVENT_NOTIFY_REGISTER, NULL, 0, ALP_NOTIFY_QUEUE_DEFAULT, ALP_NOTIFY_BROADCAST_FORCE);
	}

	gHoldNotifications = FALSE;
	gHoldNotificationInstall = FALSE;
	gHoldNotificationRegister = FALSE;
	
	return ALP_STATUS_OK;
}

void handle_client_alp_prv_bundle_hold_notifications(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	gboolean hold;

	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &hold);
	alp_ipc_message_unpack_end(message);
	
	alp_status_t result;
	
	if (has_permission(connection, PERM_HOLD_NOTIFICATIONS_POLICY)) {
		result = alp_prv_bundle_hold_notifications(hold);
	} else {
		result = ALP_STATUS_BUNDLEMGR_FAILURE;
	}

	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PRV_BUNDLE_HOLD_NOTIFICATIONS_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result);
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

void reset_hold()
{
	gHoldNotifications = TRUE;
	gHoldNotificationInstall = FALSE;
	gHoldNotificationRegister = FALSE;
}
