/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/sendfile.h>
#include <glib.h>
#include <libintl.h>
#include <alp/pdk/gras.h>

#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"
#include "ghost_support.h"
#include "java_support.h"

#include "pm_refimpl.h"

#include <alp/pdk/gras.h>

// FIXME: general note: the ro_* routines badly need to be refactored, so
// that data coming in in common ways goes out through a single path. Right
// now there are many redundant code paths, especially for ghost and java
// resource handling.

GList * alp_prv_bundle_expand_locale(const char * locale)
{
	if (!locale)
		return NULL;
	
	const char * start = locale;
	const char * end = locale + strlen(locale);
	const char * modifier_start;
	const char * modifier_end;
	const char * codeset_start;
	const char * codeset_end;
	const char * territory_start;
	const char * territory_end;
	
	char * colon;
again:	
	if ((colon = strchr(locale, ':'))) {
		end = colon;
	}
	
	// parse the locale string, based on XPG/4 specification into fragments
	
	if ((modifier_start = strchr(start, '@'))) {
		modifier_end = end;
		end = modifier_start;
	} else {
		modifier_start = modifier_end = NULL;
	}
	
	if ((codeset_start = strchr(start, '.')) && codeset_start < end) {
		codeset_end = end;
		end = codeset_start;
	} else {
		codeset_start = codeset_end = NULL;
	}
	
	if ((territory_start = strchr(start, '_')) && territory_start < end) {
		territory_end = end;
		end = territory_start;
	} else {
		territory_start = territory_end = NULL;
	}
	
	char buf[PATH_MAX];
	GList * results = NULL;
	
	
	for (;;) {
		// First, paste fragments back together
		int pos = 0;
		memcpy(buf + pos, start, end-start);
		pos += end-start;
		
		if (territory_start && territory_end) {
			memcpy(buf + pos, territory_start, territory_end-territory_start);
			pos += territory_end-territory_start;
		}
				
		if (codeset_start && codeset_end) {
			memcpy(buf + pos, codeset_start, codeset_end-codeset_start);
			pos += codeset_end-codeset_start;
		}
				
		if (modifier_start && modifier_end) {
			memcpy(buf + pos, modifier_start, modifier_end-modifier_start);
			pos += modifier_end-modifier_start;
		}
		buf[pos] = '\0';
		
		// record the result
		results = g_list_prepend(results, g_strdup(buf)); // prepend each, then reverse before the return
		
		// Then chop off the next least important fragment, for the next pass.
		// If everything has been chopped off which can be, then we're done
		// The ordering is based on the gettext logic
		
		if (codeset_start && codeset_end) {
			codeset_start = codeset_end = NULL;
		} else if (territory_start && territory_end) {
			territory_start = territory_end = NULL;
		} else if (modifier_start && modifier_end) {
			modifier_start = modifier_end = NULL;
		} else
			break;
	}
	
	if (colon) {
		locale = colon+1;
		goto again;
	}
	
	results = g_list_reverse(results); // prepend each, then reverse before the return
	
	return results;
}

static int sGotLocales;
static GList * sLocales;

static GList * get_locales(void)
{
	if (!sGotLocales) {
		sGotLocales = 1;
		
		const char * l = NULL;
		
		if (!l || l[0] == '\0')
			l = getenv("LANG");
		if (!l || l[0] == '\0')
			l = getenv("LANGUAGE");
		
		return sLocales = alp_prv_bundle_expand_locale(l);
	} else {
		return sLocales;
	}
}

void wipe_locales(void)
{
	if (sLocales) {
		g_list_foreach(sLocales, (GFunc)g_free, NULL);
		g_list_free(sLocales);
	}
	
	sLocales = NULL;
	sGotLocales = 0;
}

GList * alp_prv_bundle_expand_searchpath(const char * path)
{
	const char * start;
	const char * end;
	GList * results = NULL;
	
	if (!path)
		return NULL;

	char buf[PATH_MAX];
	
	GList * locales = get_locales();
	
	start = path;

	for (;;) {
		end = strchr(start, ':');
		if (!end)
			end = start + strlen(start);
		
		const char * c;
		int locale_point = -1;
		
		if (end-start >= PATH_MAX) // skip fragment if it's too long
			goto next;
		
		char * r = buf;
		
		for (c=start; c<end; ) {
			if (*c == '^') {
				if (c[1] == 'l') {
					c+=2;
					locale_point = r-buf;
				} else {
					*r++ = c[1];
					c+=2;
				}
			} else
				*r++ = *c++;
		}
		*r++ = '\0';
		
		if (locale_point != -1) {
			char lbuf[PATH_MAX];
			GList * iter;
			for (iter = locales; iter; iter = g_list_next(iter)) {
				memcpy(lbuf, buf, locale_point);
				strcpy(lbuf + locale_point, (char*)iter->data);
				strcpy(lbuf + strlen(lbuf), buf+locale_point);
				
				results = g_list_prepend(results, g_strdup(lbuf)); // prepend each, then reverse before the return
			}
			
			// and one without any locale material, as last-case fallback
			results = g_list_prepend(results, g_strdup(buf)); // prepend each, then reverse before the return
			
		} else {
			
			results = g_list_prepend(results, g_strdup(buf)); // prepend each, then reverse before the return
		}
		
		// stop if this was the last one
next:		if (end[0] == '\0')
			break;

		start = end+1;
	}
	
	results = g_list_reverse(results); // prepend each, then reverse before the return
	
	return results;
}

static GIOChannel * channel_from_data(char * data, int length)
{
	// FIXME: figure out how to allocate a fixed buffer giochannel

	// Meanwhile, this is a very messy hack to make it work
	char tmpbuf[] = "/tmp/rochannel.XXXXXX";
	int tmpfd = mkstemp(tmpbuf);
	
	int pos = 0;
	while (pos < length) {
		int wrote = write(tmpfd, data + pos, length - pos);
		if (wrote <= 0) {
			close(tmpfd);
			unlink(tmpbuf);
			return NULL;
		}
		pos += wrote;
	}
	
	lseek(tmpfd, 0, SEEK_SET);
	
	GIOChannel * result = g_io_channel_unix_new(tmpfd);
	if (result)
		g_io_channel_set_close_on_unref(result, TRUE);

	unlink(tmpbuf);
	
	return result;
}

gchar * alp_bundle_ref_ro_pathname(struct _AlpBundleRef * ref, const char * path)
{
	if (!ref) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	if (ref->scheme == BUNDLE_SCHEME_GHOST) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	// We could implement pathname for java bundle icons -- but is there a reason?
	if (ref->scheme == BUNDLE_SCHEME_JAVA) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	

	if (!ref->folderpath) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
		
	if (!path) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	if (path[0] == '/' || path[0] == '\0') {
		gchar * result = g_malloc(strlen(ref->folderpath) + strlen(path) + 1);
		if (!result) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		strcpy(result, ref->folderpath);
		strcat(result, path);
		return result;
	}

	if (strncmp(path, "^t/", 3) == 0) {
		// text localization can't have a pathname
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	GList * iter;
	for (iter = ref->searchpath; iter; iter = g_list_next(iter)) {
		char buf[PATH_MAX];
		
		strcpy(buf, ref->folderpath);
		strcat(buf, (char*)iter->data);
		strcat(buf, path);
		
		// FIXME: uses real UID/GID instead of effective. Should I use another operation?
		if (access(buf, F_OK) == 0) {
			//success
			gchar * result = g_strdup(buf);

			if (!result) {
				set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
				return NULL;
			}
			
			return result;
		}
	}

	set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	return NULL;
}

static alp_status_t get_ghost_data(AlpBundleRef ref, const char * path, gchar ** data, size_t * length)
{
	if (strcmp(path, "$name") == 0) 
	{
		int 	ret = -1;
		char 	*name = NULL;
		
		if (ghost_start() == ALP_STATUS_OK)
		{
			ret = alp_gras_app_display_name(GSESSION, ref->ghost_db_creator, ref->ghost_db_name, ref->ghost_db_storage, &name);
			ghost_stop(ret != ALP_STATUS_OK && ALP_STATUS_VALUE(ret) != ALP_STATUS_GRAS_GARNET_ERROR);
		}
		
		if (ret != 0) 
		{
			ALP_TMTL("alp_gras_app_display_name for database %s failed", ref->ghost_db_name);
			if (name)
				free(name);

			// Fall back to the database name
			*data = g_strdup(ref->ghost_db_name);
			if (length)
				*length = strlen(*data);
				
			return ALP_STATUS_OK;
		}
	
		if (g_mem_is_system_malloc()) 
			*data = name;
		else
		{
			*data = g_strdup(name);
			free(name);
		}
		if (length)
			*length = strlen(*data);
		
		return ALP_STATUS_OK;
	
	} 
	else 
	{
		int icon_id = 0;
		
		if (strcmp(path, "$largeicon.xpm") == 0)
			icon_id = 1000;
		else
		if (strcmp(path, "$smallicon.xpm") == 0)
			icon_id = 1001;
			
		if (icon_id != 0)
		{
			char ** xpm = NULL;
			if (ghost_start() == ALP_STATUS_OK)
			{
				int ret = alp_gras_app_create_icon(GSESSION, icon_id, ref->ghost_db_creator, 0, ref->ghost_db_storage, &xpm);
			
				// If large icon failed, try small icon
				if (ret != ALP_STATUS_OK)
				{
					ALP_TMTL("alp_gras_app_create_icon for %s/%d failed", ref->ghost_db_name, icon_id);
					if (icon_id == 1000)
					{
						icon_id = 1001;
						ret = alp_gras_app_create_icon(GSESSION, icon_id, ref->ghost_db_creator, 0, ref->ghost_db_storage, &xpm);
						if (ret != ALP_STATUS_OK)
						{
							ALP_TMTL("alp_gras_app_create_icon for %s/%d failed", ref->ghost_db_name, icon_id);
							// If this is a logical Garnet error, don't force immediate shutdown
							ghost_stop(ret != ALP_STATUS_OK && ALP_STATUS_VALUE(ret) != ALP_STATUS_GRAS_GARNET_ERROR);
							return -1;
						}
					}
				}
			
				ghost_stop(0);
			}
			else
				// Couldn't start - fail
				return -1;
	
			/* Code adapted from gras-client to parse and regenerate 'xpm' */
			
			int width, height, ncolors, bytespp;
			int i;
			
			GString * str = g_string_new("/* XPM */\n");
			sscanf(xpm[0],"%d %d %d %d",&width, &height, &ncolors, &bytespp);
			g_string_append(str, "static char * appIcon[] = {\n");
			g_string_append_printf(str, "\"%s\"",xpm[0]);
			for (i=0; i<ncolors; i++) {
				g_string_append_printf(str,",\n\"%s\"",xpm[i+1]);
			}
			for (i=0; i<height; i++) {
				g_string_append_printf(str,",\n\"%s\"",xpm[i+1+ncolors]);
			}
			g_string_append(str,"\n};\n");
	
	                free(xpm);
	                
			*data = str->str;
			if (length)
				*length = strlen(*data);
	
			g_string_free(str, FALSE);
			return ALP_STATUS_OK;
		}	
		else
			// Not an icon - continue to strcmp(path)
			return -1;
	}
}

// FIXME: the java icon is actually a file, and it's wasteful to read it in, then push it back
// out to a temp-file: refactor the java stuff in a different way so it's not overly copied.
static alp_status_t get_java_data(AlpBundleRef ref, const char * path, gchar ** data, size_t * length)
{
	ALP_TMTL("get_java_data path %s \n", path);
	int ret =  -1;
	ALP_TMTL("midlet_id %d \n", ref->java_midlet_id);
	gchar *icon_name; 
	if (strncmp(path, "^t/", 3) == 0) {
		
		const char * translation = path + 3;
		
		int l = strlen(translation);
		
		*data = g_malloc(l + 1);
		
		if (length)
			*length = l;
		
		memcpy(*data, translation, l+1);
		ret = 0;
				
	}else if(strcmp(path , "icon.png") == 0) {
	
		int midlet_id = ref->java_midlet_id;
		icon_name = java_path(midlet_id,".ico");
		
		if (!icon_name) {
			ALP_TMTL("Error retrieving icon name for java midlet %d", midlet_id);
			return -1;
		}
			
  		gchar * contents = NULL;
		g_file_get_contents(icon_name, &contents, length, NULL);
		
		if (!contents) {
			g_free(icon_name);
			ALP_TMTL("Error retrieving icon contents for java midlet %d, icon filename %s", midlet_id, icon_name);
			return -1;
		}
		
		*data = contents;
		g_free(icon_name);
		ret  = 0 ; 		
	}
	return ret;	
}



alp_status_t alp_bundle_ref_ro_data(struct _AlpBundleRef * ref, const char * path, gchar ** data, size_t * length)
{
	if (!ref) 
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);

	if (ref->scheme == BUNDLE_SCHEME_GHOST) {
		gchar * local_data = NULL;
		int local_length = 0;
		if (get_ghost_data(ref, path, &local_data, &local_length) != ALP_STATUS_OK) {
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		
		if (data)
			*data = local_data;
		if (length)
			*length = local_length;
		
		return ALP_STATUS_OK;
	}else if (ref->scheme == BUNDLE_SCHEME_JAVA) {
		gchar * local_data = NULL;
		int local_length = 0;
		if (get_java_data(ref, path, &local_data, &local_length) != ALP_STATUS_OK) {
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		
		if (data)
			*data = local_data;
		if (length)
			*length = local_length;
		
		return ALP_STATUS_OK;
	}

	if (!ref->folderpath)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
	if (!data)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
	if (!path)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	

	char buf[PATH_MAX];
	int fd;
	
	if (path[0] == '/' || path[0] == '\0') {
		strcpy(buf, ref->folderpath);
		strcat(buf, path);
		
		fd = open(buf, O_RDONLY);
		if (fd != -1)
			goto do_data;
		else 
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}
	
	if (strncmp(path, "^t/", 3) == 0) {
		
		char * translation = alp_bundle_ref_gettext(ref, path+3);
		
		int l = strlen(translation);
		
		*data = g_malloc(l + 1);
		
		if (length)
			*length = l;
		
		memcpy(*data, translation, l+1);
		
		return ALP_STATUS_OK;
	}

	GList * iter;
	for (iter = ref->searchpath; iter; iter = g_list_next(iter)) {
		strcpy(buf, ref->folderpath);
		strcat(buf, (char*)iter->data);
		strcat(buf, path);
		
		fd = open(buf, O_RDONLY);
		if (fd != -1)
			goto do_data;
	}

	return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
do_data:	;
	
	unsigned int l = lseek(fd, 0, SEEK_END);
	lseek(fd, 0, SEEK_SET);
	*data = g_malloc(l + 1);
	
	if (length)
		*length = l;
	
	if (!*data) {
		close(fd);
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}
	
	unsigned long pos = 0;
	while (pos < l) {
		int r = read(fd, *data + pos, l - pos);
		if (r <= 0) {
			close(fd);
			g_free(*data);
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		
		pos += r;
	}
	
	(*data)[l] = '\0';
	
	close(fd);
	
	return ALP_STATUS_OK;
}

gchar * alp_bundle_ref_ro_tempfile(struct _AlpBundleRef * ref, const char * path, gboolean keep, const char * ext)
{
	if (!ref) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	if (ref->scheme == BUNDLE_SCHEME_GHOST) 
	{
		// FIXME: fold this code path into the do_data section

		gchar * data = NULL;
		int length = 0;

		char tmppath[PATH_MAX];
		strcpy(tmppath, "/tmp/roghost.XXXXXX");

		int tmpfd = mkstemp(tmppath);
	
		if (tmpfd == -1) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
	
		if (get_ghost_data(ref, path, &data, &length) != ALP_STATUS_OK) {
			close(tmpfd);
			unlink(tmppath);
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		
		int pos = 0;
		
		while (pos < length) {
			int wrote = write(tmpfd, data + pos, length - pos);
			if (wrote <= 0) {
				// failure
				close(tmpfd);
				unlink(tmppath);
				g_free(data);
				set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
				return NULL;
			}
			pos += wrote;
		}
		
		g_free(data);
	
		// ...unlink temp file, leaving its descriptor open...
		unlink(tmppath);
	
		// ...then use /proc fs to make a filename for that descriptor
		
		gchar * result = g_strdup_printf("/proc/%d/fd/%d", getpid(), tmpfd);
		
		if (!result) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		
		return result;
	} else if(ref->scheme == BUNDLE_SCHEME_JAVA) {
	
		gchar * data = NULL;
		int length = 0;

		char tmppath[PATH_MAX];
		strcpy(tmppath, "/tmp/rotxt.XXXXXX");

		int tmpfd = mkstemp(tmppath);
	
		if (tmpfd == -1) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
	
		if (get_java_data(ref, path, &data, &length) != ALP_STATUS_OK) {
			close(tmpfd);
			unlink(tmppath);
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		
		int pos = 0;
		
		while (pos < length) {
			int wrote = write(tmpfd, data + pos, length - pos);
			if (wrote <= 0) {
				// failure
				close(tmpfd);
				unlink(tmppath);
				g_free(data);
				set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
				return NULL;
			}
			pos += wrote;
		}
		
		g_free(data);
	
		// ...unlink temp file, leaving its descriptor open...
		unlink(tmppath);
	
		// ...then use /proc fs to make a filename for that descriptor
				
		gchar * result = g_strdup_printf("/proc/%d/fd/%d", getpid(), tmpfd);
		if (!result) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		return result;
	}
		
	if (!ref->folderpath) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	
	if (!path) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
		
	if (ext) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}



	if (strncmp(path, "^t/", 3) == 0) {
		// FIXME: fold this code path into the do_data section

		char tmppath[PATH_MAX];
		strcpy(tmppath, "/tmp/rotxt.XXXXXX");

		int tmpfd = mkstemp(tmppath);
	
		if (tmpfd == -1) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
	
		char * translation = alp_bundle_ref_gettext(ref, path+3);
		
		int w = 0;
		int l = strlen(translation);
		
		while (w < l) {
			int result = write(tmpfd, translation + w, l - w);
			if (result <= 0) {
				// failure
				close(tmpfd);
				unlink(tmppath);
				set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
				return NULL;
			}
			w += result;
		}
	
		// ...unlink temp file, leaving its descriptor open...
		unlink(tmppath);
	
		// ...then use /proc fs to make a filename for that descriptor
		
		gchar * result = g_strdup_printf("/proc/%d/fd/%d", getpid(), tmpfd);
		
		if (!result) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		
		return result;
	}

	char buf[PATH_MAX];
	int fd;
	
	if (path[0] == '/' || path[0] == '\0') {
		strcpy(buf, ref->folderpath);
		strcat(buf, path);
		
		fd = open(buf, O_RDONLY);
		if (fd != -1)
			goto do_data;
		else {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
	}

	GList * iter;
	for (iter = ref->searchpath; iter; iter = g_list_next(iter)) {
		strcpy(buf, ref->folderpath);
		strcat(buf, (char*)iter->data);
		strcat(buf, path);
		
		fd = open(buf, O_RDONLY);
		if (fd != -1)
			goto do_data;
	}

	set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	return NULL;
	
do_data:
	
	if (keep) { // case for resources which aren't directly accessible, or which should be accessible after the package is closed
	
		// create a temporary file...
		char tmppath[PATH_MAX];
		strcpy(tmppath, "/tmp/ro.XXXXXX");

		size_t len = lseek(fd, 0, SEEK_END);
	
		int tmpfd = mkstemp(tmppath);
	
		if (tmpfd == -1) {
			close(fd);
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
	
		// ...copy the resource into that temp file...
		off_t off = 0;
		if ((size_t)sendfile(tmpfd, fd, &off, len) != len) {
			close(fd);
			close(tmpfd);
			unlink(tmppath);
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		
		
		// ..close the resource we were reading from...
		close(fd);
	
		// ...unlink temp file, leaving its descriptor open...
		unlink(tmppath);
	
		// ...then use /proc fs to make a filename for that descriptor
		
		gchar * result = g_strdup_printf("/proc/%d/fd/%d", getpid(), tmpfd);
		
		if (!result) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		
		return result;
	} else {
		close(fd);
		
		gchar * result = g_strdup(buf);
		
		if (!result) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		
		return result;
	}

}

alp_status_t alp_bundle_tempfile_release(gchar * tempfile)
{
	if (!tempfile)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);

	int x,y;

	if (strncmp(tempfile, BAR_LINK_PATH, strlen(BAR_LINK_PATH)) == strlen(BAR_LINK_PATH)) {
		// reference to resource directly inside opened BAR package
		// free string, don't do anything to file
		g_free(tempfile);
	} else if (sscanf(tempfile, "/proc/%d/fd/%d", &x, &y) == 2) {
		// reference to resource copied into a (deleted) temporary file
		// if it's ours, close the descriptor, then free the string
		// if it's someone else's, there's not much we can do.
		if (x == getpid())
			close(y);
		g_free(tempfile);
	} else {
		// Huh? Maybe this would be a real tempfile, which we should unlink?
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}
	
	return ALP_STATUS_OK;
}

GIOStatus  (*my_orig_close)         (GIOChannel   *channel, GError      **err);
GIOFuncs * my_channel_funcs; // = NULL


struct myChannel {
	GIOChannel parent;
	int fd; // from GIOUnixChannel
	AlpBundleRef ref;
};

GIOStatus my_close(GIOChannel * channel, GError ** err)
{
	GIOStatus result = my_orig_close(channel, err);
	struct myChannel * c = (struct myChannel*)channel;
	alp_prv_bundle_ref_decref(c->ref);
	return result;
}

GIOChannel * alp_bundle_ref_ro_channel(struct _AlpBundleRef * ref, const char * path)
{
	if (!ref) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	if (ref->scheme == BUNDLE_SCHEME_GHOST) 
	{
		gchar * data = NULL;
		int length = 0;
		if (get_ghost_data(ref, path, &data, &length) != ALP_STATUS_OK) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}

		GIOChannel * result = channel_from_data(data, length);
		g_free(data);
		return result;
	} else if (ref->scheme == BUNDLE_SCHEME_JAVA) {
		gchar * data = NULL;
		int length = 0;
		if (get_java_data(ref, path, &data, &length) != ALP_STATUS_OK) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}

		GIOChannel * result = channel_from_data(data, length);
		g_free(data);
		return result;
	}

	if (!ref->folderpath) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	if (!path) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	char buf[PATH_MAX];

	if (strncmp(path, "^t/", 3) == 0) {
		// FIXME: figure out how to allocate a fixed buffer giochannel
		
		// Meanwhile, this is a very messy hack to make it work
		gchar * tmpfile = alp_bundle_ref_ro_tempfile(ref, path, TRUE, NULL);
		// propagate potential error from alp_bundle_ref_ro_tempfile
		if (!tmpfile)
			return NULL;
		int fd = open(tmpfile, O_RDONLY);
		GIOChannel * result = g_io_channel_unix_new(fd);
		if (result)
			g_io_channel_set_close_on_unref(result, TRUE);
		alp_bundle_tempfile_release(tmpfile);

		if (!result) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		
		return result;
	}
	
	if (path[0] == '/' || path[0] == '\0') {
		strcpy(buf, ref->folderpath);
		strcat(buf, path);
		goto do_open;
	} else {
		GList * iter;
		for (iter = ref->searchpath; iter; iter = g_list_next(iter)) {
			strcpy(buf, ref->folderpath);
			strcat(buf, (char*)iter->data);
			strcat(buf, path);
			
			// FIXME: uses real UID/GID instead of effective. Should I use another operation?
			if (access(buf, F_OK) == 0) {
				//success
				goto do_open;
			}
		}
	}
	
	//failure
	set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	return NULL;
	
do_open:	;

	GIOChannel * result = g_io_channel_new_file(buf, "r", NULL);
		
	if (!result)
		return result;
	
	if (!my_channel_funcs) {
		my_channel_funcs = g_new0(GIOFuncs, 1);
		memcpy(my_channel_funcs, result->funcs, sizeof(GIOFuncs));
		my_orig_close = my_channel_funcs->io_close;
		my_channel_funcs->io_close = my_close;
	}

	struct myChannel * c = g_realloc(result, sizeof(struct myChannel));

	if (!c) {
		g_io_channel_unref(result);
		return NULL;
	}
	
	alp_prv_bundle_ref_incref(ref);
	
	c->ref = ref;
	c->parent.funcs = my_channel_funcs;
	
	return (GIOChannel*)c;	
}

GList * alp_bundle_ref_ro_directory(struct _AlpBundleRef * ref, const char * path, gboolean recurse)
{
	if (!ref) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	// Directory queries not suppored for GHost bundles		
	if (ref->scheme == BUNDLE_SCHEME_GHOST) 
	{
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	// Directory queries not suppored for JVLite bundles		
	if (ref->scheme == BUNDLE_SCHEME_JAVA) 
	{
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	

	if (!ref->folderpath) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	if (!path) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	char buf[PATH_MAX];
	DIR * dir;
	
	GList * results = NULL;
	
	struct dirent * d;
	struct stat stbuf;
	
	GList * descend = NULL;
		
	if (path[0] == '/' || path[0] == '\0') {
		strcpy(buf, ref->folderpath);
		strcat(buf, path);
		
		while (buf[0] && buf[strlen(buf)-1] == '/')
			buf[strlen(buf)-1] = '\0';
		
		if ((dir = opendir(buf)))
			goto do_dir;
		
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	if (strncmp(path, "^t/", 3) == 0) {
		// Directories don't do text translation
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	GList * iter;
	for (iter = ref->searchpath; iter; iter = g_list_next(iter)) {
		strcpy(buf, ref->folderpath);
		strcat(buf, (char*)iter->data);
		strcat(buf, path);
		
		while (buf[0] && buf[strlen(buf)-1] == '/')
			buf[strlen(buf)-1] = '\0';
		
		if ((dir = opendir(buf)))
			goto do_dir;
	}

	set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	return NULL;
	
			
do_dir: ;
	
	char * end = buf + strlen(buf);
	

	*end++ = '/';
	while ((d = readdir(dir))) {
		if (d->d_name[0] == '.')
			continue;
		strcpy(end, d->d_name);
		if (recurse) {
			if (stat(buf, &stbuf)==0 && S_ISDIR(stbuf.st_mode)) {
				descend = g_list_prepend(descend, g_strdup(buf));
			}
		}
		results = g_list_prepend(results, g_strdup(buf + strlen(ref->folderpath))); // prepend each, then reverse before the return
	}
	closedir(dir);
	
again:	if (descend) {
		strcpy(buf, (char*)descend->data);
		g_free(descend->data);
		descend = g_list_delete_link(descend, descend);
		
		dir = opendir(buf);
		if (!dir)
			goto again;
		goto do_dir;
	}
	
	results = g_list_reverse(results); // prepend each, then reverse before the return
	
	return results;
}
