/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright � 2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <dirent.h>
#include <hiker/fail.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <hiker/security.h>
#include <hiker/prv/apas.h>
#include <hiker/pdk/appmgr_pdk.h>

               
#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"
#include "pm_refimpl.h"
#include "pm_msg.h"
#include "pm_perms.h"

static int  alp_get_uid( AlpBundle bundle, uid_t *uid, gid_t *gid) 
{
	
	int ret = 0;
	char *security_string = NULL;
	unsigned int security_id = 0;
	security_string = alp_bundle_metadata_value( bundle, "security_handle");
	if (security_string == NULL) {
		//printf("security string is NULL\n");
		security_id = ALP_APP_DEFAULT_SECURITY_ID ;
	} else {
		int inputs_read = 0;
		inputs_read = sscanf(security_string, "%u", &security_id);
		if (inputs_read != 1){
			//printf("security string is NULL\n");
			security_id = ALP_APP_DEFAULT_SECURITY_ID ;
		}
		g_free(security_string);
	}
	alp_status_t status;
	status = alp_spf_apas_packageid_uid_get(security_id, uid);
	if ( status != ALP_STATUS_SPF_E_NOERR) {	
		printf("cannot get the uid of the package please sign you app using signtool %i \n", security_id);
		return -1;	
	}
	status = alp_spf_apas_packageid_gid_get( security_id, gid);
	if ( status != ALP_STATUS_SPF_E_NOERR) {	
		printf("cannot get the gid of the pacakge please sign you app using signtool %i \n", security_id);
		return -1;	
	}
	return 0; 
}

gchar * alp_bundle_rw_pathname(AlpBundle index, const char * path, gboolean vivify)
{
        gchar * basepath;
        
        if (alp_bundle_is_NULL(index)) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
        	return NULL;
        }
        
        if (!path) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
	    }
        
        if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;
		
		alp_status_t status;
		
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_RW_PATHNAME);
		alp_ipc_message_pack_int32(message, index.value);
		alp_ipc_message_pack_int32(message, vivify);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			// some failure of IPC
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return NULL;
		}
		alp_ipc_message_destroy(message);

		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_string(response, &basepath);
		alp_ipc_message_unpack_int32(response, &status);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);
		
		if (!basepath) {
		  set_error(status);
		  return NULL;
		}

        } else if (pm_status == STATUS_SERVER) {

	  S_STMT s;
	  S_PREPARE(s, "SELECT common_metadata.name,scheme_id FROM containers JOIN common_metadata ON (containers.package_id=common_metadata.package_id) WHERE containers.package_id = ?1");
	  S_BIND_INT(s, 1, index.value);
	  if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
          }
          
	  // RW datafolder not supported for GHost bundles
          if (S_COL_INT(s, 1) == BUNDLE_SCHEME_GHOST) {
          	S_FINALIZE(s);
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
          }
          
          char base[PATH_MAX];

	  // FIXME: crude, +4 skips 'bar:'
          sprintf(base, "%s" "%s" BAR_RW_PATH, getenv("ALP_RUN_TOP") ?: "", gBundleRWPath, S_COL_TEXT(s, 0) + 4);
		
          S_FINALIZE(s);
          
          if (!vivify) {
	    	// Try to access it before creating the directory
          	int fd = open(base, O_RDONLY);
	    	//ALP_TMTL("!vivify rw_folder, base is %s, fd is %d", base, fd);
          	if (fd == -1) {
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
          		return NULL;
		}
		close(fd);
          }
          
          // on the server side, create the base folder if it does not already exist
		  //printf(" %s calling mkdir_p \n",__FUNCTION__);	
          mkdir_p(base, 0777);
          //added chmod to change permission because mkdir_p is setting to default
          chmod(base, 0777);
          uid_t uid;
          gid_t gid;
          //printf("calling alp_get_uid \n");
          if( 0 == alp_get_uid(index, &uid, &gid) ) {
          		//printf("changing the owner \n");
          	if(0 != chown(base, uid, gid)) {
          		//printf("chown failed %d \n", errno);	
          	}
          } 
          strcat(base, "/");
	
          basepath = g_strdup(base);
	}
	
	// shortcut on empty path -- particularly to shortcut usage of server by client
	if (path[0] == '\0')
	  return basepath;
	
	char fullpath[PATH_MAX];
	
	// section shared by client and server
	
	// this section creates subfolders within the base package folder -- this needs to be done on the client side

        strcpy(fullpath, basepath);

	char * r = strrchr(path, '/');
	
	if (r) {
	    memcpy(fullpath + strlen(basepath), path, r-path);
	    fullpath[strlen(basepath) + r-path] = '\0';
	    
	    if (!vivify) {
	    	// Try to access it before creating the directory
	    	int fd = open(fullpath, O_RDONLY);
	    	//ALP_TMTL("!vivify rw_folder, fullpath is %s, fd is %d", fullpath, fd);
	    	if (fd == -1) {
	    		g_free(basepath);
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	    		return NULL;
		}
		close(fd);
	    }
	    mkdir_p(fullpath, 0777);
	    //added chmod to change permission because mkdir_p is setting to default
	    chmod(fullpath, 0777);
	    uid_t uid;
	    gid_t gid;
	    //printf("calling alp_get_uid \n");
	    if( 0 == alp_get_uid(index, &uid, &gid) ) {
	    	//printf("changing the owner \n");
          	if(0 != chown(fullpath, uid, gid)){
          		//printf("chown failed %d \n", errno);	
          	} 	
        }
	    
        }


        strcpy(fullpath + strlen(basepath), path);
        g_free(basepath);

        //if (!vivify) {
	//	ALP_TMTL("!vivify rw_folder, fullpath is %s, successfull", fullpath);
	//}
        
        gchar * result = g_strdup(fullpath);
        
        if (!result) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;        	
        }
        return result;
}			

void handle_client_alp_bundle_rw_pathname(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	AlpBundle index;
	int vivify;
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &index.value);
	alp_ipc_message_unpack_int32(message, &vivify);
	alp_ipc_message_unpack_end(message);
	
	gchar * result;
	
	if (has_permission(connection, PERM_RW_PATHNAME_POLICY)) {
		result = alp_bundle_rw_pathname(index, "", vivify);
	} else {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		result = NULL;
	}	

	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_RW_PATHNAME_RESPONSE, message);
	alp_ipc_message_pack_string(response, result);
	alp_ipc_message_pack_int32(response, result ? ALP_STATUS_OK : alp_bundle_error());
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

alp_status_t alp_bundle_rw_delete(AlpBundle index, const char * folder_path, gboolean itself)
{
	if (alp_bundle_is_NULL(index))
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
  
	 if (!folder_path)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);

	S_STMT s;
	S_PREPARE(s, "SELECT common_metadata.name,scheme_id FROM containers JOIN common_metadata ON (containers.package_id=common_metadata.package_id) WHERE containers.package_id = ?1");
	S_BIND_INT(s, 1, index.value);
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}

	// RW datafolder not supported for GHost bundles
	if (S_COL_INT(s, 1) == BUNDLE_SCHEME_GHOST) {
		S_FINALIZE(s);
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}
	
	char base[PATH_MAX], root[PATH_MAX];
	
	  // FIXME: crude, +4 skips 'bar:'
	sprintf(base, "%s" "%s" BAR_RW_PATH, getenv("ALP_RUN_TOP") ?: "", gBundleRWPath, S_COL_TEXT(s,0) + 4);
	strcpy(root, base);
	
	strcat(base, folder_path);

	S_FINALIZE(s);

	int result;
	
	result = rmdir_contents(base);
	if (itself && result == 0)
		result = rmdir(base);

	// If the client didn't have permission to delete what it is interested in,
	// send the request on to the server. Yes, this means any client can request
	// R/W material to be _deleted_ for any package.	
	if (result == -1 && (errno == EPERM || errno == EACCES) && pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;
		
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_RW_DELETE);
		alp_ipc_message_pack_int32(message, index.value);
		alp_ipc_message_pack_string(message, folder_path);
		alp_ipc_message_pack_int32(message, itself);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			// some failure of IPC
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		alp_ipc_message_destroy(message);
			
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);
		
		return set_error(result);
	}

	// We want to remove the folder entirely if it is present but empty, but
	// the client side may not have appropriate permissions to remove that folder.
	// So we'll bounce to a server request in that case.	
	if (alp_prv_folder_empty(root, 0)) {
		if (pm_status == STATUS_SERVER) {
			rmdir(root);
		} else {
			AlpMessage * message, *response;
			
			message = alp_ipc_message_create();
			alp_ipc_message_pack_start(message, MSG_PACKAGE_RW_DELETE);
			alp_ipc_message_pack_int32(message, index.value);
			alp_ipc_message_pack_string(message, "");
			alp_ipc_message_pack_int32(message, TRUE);
			alp_ipc_message_pack_end(message);
			if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
				alp_ipc_message_destroy(message);
				// some failure of IPC
				return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			}
			alp_ipc_message_destroy(message);
			
			alp_status_t result;

			alp_ipc_message_unpack_start(response);
			alp_ipc_message_unpack_int32(response, &result);
			alp_ipc_message_unpack_end(response);
			alp_ipc_message_destroy(response);
		
			return set_error(result);
		}
	}
	
	return ALP_STATUS_OK;
}

void handle_client_alp_bundle_rw_delete(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	AlpBundle index;
	gchar * path;
	gboolean itself;
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &index.value);
	alp_ipc_message_unpack_string(message, &path);
	alp_ipc_message_unpack_int32(message, &itself);
	alp_ipc_message_unpack_end(message);
	
	alp_status_t result;
	
	if (has_permission(connection, PERM_RW_DELETE_POLICY)) {
	
		char testpath[PATH_MAX];
		sprintf(testpath, "/%s/", path);
	
		// check for security violation attempting to delete upwards from RW path
		if (strstr(testpath, "/../")) {
			result = ALP_STATUS_BUNDLEMGR_FAILURE; // security violation
		} else {
			result = alp_bundle_rw_delete(index, path, itself);
		}
	} else {
		result = ALP_STATUS_BUNDLEMGR_FAILURE;
	}

	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_RW_DELETE_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result);
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
	
	g_free(path);
}

int alp_prv_associated_data(const char * name)
{
	if (strncmp(name, BUNDLE_SCHEME_NAME_BAR ":", strlen(BUNDLE_SCHEME_NAME_BAR ":")) != 0) {
		return 0;
	}

	char path[PATH_MAX];
	
	// FIXME: crude, +4 skips 'bar:'
	sprintf(path, "%s" BAR_RW_PATH, gBundleRWPath, name+4);
	
	// FIXME: Also look at global data
	return alp_prv_folder_empty(path, 1) ? 0 : 1;
} 
