/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>

#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"

GHashTable * alp_bundle_scheme_values(const char * scheme)
{
	GList * l = NULL;
	GHashTable * results = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);

	S_STMT s;
	S_PREPARE(s, "SELECT * FROM schemes WHERE scheme_name=?1");
	S_BIND_TEXT(s, 1, scheme);
	if (S_STEP_ROW(s)) {
		int i=0;
		// start at 2 to skip scheme_id and scheme_name
		for (i=2;i<sqlite3_column_count(s);i++) {
			gchar * key = g_strdup(sqlite3_column_name(s, i));
			gchar * value = g_strdup(S_COL_TEXT(s,i));
			g_hash_table_insert(results, key, value);
		}
	} else {
		g_hash_table_destroy(results);
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		results = NULL;
	}
	S_FINALIZE(s);

	return results;
}

gchar * alp_bundle_scheme_value(const char * scheme, const char * key)
{
	gchar * result = NULL;
	int scheme_id = 0;
	
	S_STMT s;
	char cmd[] = "SELECT ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ FROM schemes WHERE scheme_name=?1";
	if (safe_sql_fillin(cmd, key)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	if (!S_PREPARE_P(s, cmd)) { // if preparation error, must be invalid column
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}	
	
	S_BIND_TEXT(s, 1, scheme);
	S_BEGIN_SELECT(s) {
		result = g_strdup(S_COL_TEXT(s, 0));
	}
	S_END_SELECT(s);
	
	if (!result) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	return result;
}

gchar * alp_bundle_scheme_identify(const char * filename, const char * mimetype)
{
	if (!filename && !mimetype) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	if (mimetype) {
		const char * mimetypes[] = {
			MIME_TYPE_BAR, 	BUNDLE_SCHEME_NAME_BAR,
			MIME_TYPE_PRC,	BUNDLE_SCHEME_NAME_GHOST,
			MIME_TYPE_PRC_2,BUNDLE_SCHEME_NAME_GHOST,
			MIME_TYPE_JAD,	BUNDLE_SCHEME_NAME_JAVA,
			MIME_TYPE_JAR,	BUNDLE_SCHEME_NAME_JAVA,
			NULL, NULL
		};
		
		const char ** c;
		
		// Check for exact match
		for (c=mimetypes; *c; c+=2) {
			if (strcmp(mimetype, c[0]) == 0)
				return g_strdup(c[1]);
		}
		
		// Then check for prefix match: application/vnd.palm.prc
		for (c=mimetypes; *c; c+=2) {
			int l = strlen(c[0]);
			if ((strncmp(mimetype, c[0], l) == 0) &&
				(mimetype[l] == '.'))
				return g_strdup(c[1]);
		}
				
		// No luck, try the filename
	}
	
	if (!filename) {
		// Give up without a filename
		return NULL;
	}
	
	int len = strlen(filename);
	gchar * result = NULL;
	
	// FIXME: identify relevant mimetypes
	
	if (len >= 4 && (strcasecmp(filename + len - 4, ".bar") == 0)) {
		result = g_strdup(BUNDLE_SCHEME_NAME_BAR);
	}
	
	if (len >= 4 && ( 
		(strcasecmp(filename + len - 4, ".prc") == 0) ||
		(strcasecmp(filename + len - 4, ".pdb") == 0)
		)) {
		result = g_strdup(BUNDLE_SCHEME_NAME_GHOST);
	}

	if (len >= 5 && ( 
		(strcasecmp(filename + len - 5, ".oprc") == 0) ||
		(strcasecmp(filename + len - 5, ".bprc") == 0) 
		)) {
		result = g_strdup(BUNDLE_SCHEME_NAME_GHOST);
	}

	if (len >= 4 && ( 
		(strcasecmp(filename + len - 4, ".jar") == 0) ||
		(strcasecmp(filename + len - 4, ".jad") == 0) 
		)) {
		result = g_strdup(BUNDLE_SCHEME_NAME_JAVA);
	}
	
	if (!result) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	return result;
}

