/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/ipc.h>
#include <hiker/notifymgr.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <dlfcn.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/sendfile.h>
#include <sys/types.h>
#include <sys/wait.h>
       
#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"
#include "pm_msg.h"
#include "pm_perms.h"
#include "ghost_support.h"
#include "java_support.h"

#include <alp/pdk/gras.h>

static int sMoving;

// store: 
//           	bar_folder
//           	bar_prefer_archive
//		register_policy
//		unregister_policy
//		delete_policy
//		install_policy
//		scan_policy


// FIXME: move could be more efficient on stores living on the same underlying
// file-system (but that seems unlikely).
// FIXME: we should keep name-changes pending until the move operation is complete.
AlpBundle alp_bundle_move_to_store(AlpBundle package, const char * store, int flags)
{
	alp_status_t status;
	
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;
			
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_MOVE_TO_STORE);
		alp_ipc_message_pack_int32(message, package.value);
		alp_ipc_message_pack_string(message, store);
		alp_ipc_message_pack_int32(message, flags);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			ALP_TMTL("some failure of IPC");
			// some failure of IPC
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return ALP_BUNDLE_NULL;
		}
		alp_ipc_message_destroy(message);
		
		AlpBundle result;
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result.value);
		alp_ipc_message_unpack_int32(response, &status);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);
		
		if (!result.value)
			set_error(status);
		
		return result;
		
	} else if (pm_status != STATUS_SERVER) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_NULL;
	}

	S_STMT s;
	S_PREPARE(s, "SELECT scheme_id FROM containers WHERE containers.package_id = ?1");
	S_BIND_INT(s, 1, package.value);
	if(!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	int scheme = S_COL_INT(s,0);
	S_FINALIZE(s);
	
	if (scheme == BUNDLE_SCHEME_GHOST) {
		// GHost bundles cannot be moved
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	if (gNotify) {	
		int handled = 0;
	
		AlpNotifyEventBundleMove * d = g_malloc0(4 + strlen(store) + 1);
				
		d->index = package;
		strcpy(d->store, store);

		ALP_TMTL("Move notification for package index %d to store %s", d->index, d->store);

		if (gHoldNotifications) {
			// Sorry, there's no good reason to hold this notification pending
		} else {

			alp_status_t err = my_sync_notify(NOTIFY_NAME, ALP_NOTIFY_EVENT_BUNDLE_DELETE, d, 4 + strlen(store) + 1, ALP_NOTIFY_QUEUE_DEFAULT, 0, 0, &handled);
			//ALP_FAIL_NON_FATAL_IF(err != ALP_STATUS_OK, "broadcast of package unregistration failed (err=%x)", err);
		}

		g_free(d);

		if (handled & 1) {
			// someone said it can't be moved
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
	}

	sMoving++;
	AlpBundle copy_result = alp_bundle_copy_to_store(package, store, flags);
	sMoving--;
	
	if (alp_bundle_is_NULL(copy_result)) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	alp_status_t delete_result = alp_bundle_delete(package, FALSE);
	if (delete_result != ALP_STATUS_OK) {
		// FIXME: ick, make a guess that the delete completely failed
		// unwind the copy. This should check the result, and either
		// unwind the copy or possibly do a pending delete on the source
		alp_bundle_delete(copy_result, FALSE);
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
		
	return copy_result;

fail:
	set_error(status);
	return ALP_BUNDLE_NULL;
}

void handle_client_alp_bundle_move_to_store(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	gchar * store;
	AlpBundle package;
	int flags;
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &package.value);
	alp_ipc_message_unpack_string(message, &store);
	alp_ipc_message_unpack_int32(message, &flags);
	alp_ipc_message_unpack_end(message);
			
	AlpBundle result;

	gboolean permission;
	
	permission = has_store_permission_by_store(connection, PERM_INSTALL_POLICY_COLUMN, store) &&
			has_store_permission_by_bundle(connection, PERM_DELETE_POLICY_COLUMN, package);
			
	if (permission) {
		result = alp_bundle_move_to_store(package, store, flags);
	} else {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		result = ALP_BUNDLE_NULL;
	}
	
	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_MOVE_TO_STORE_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result.value); 
	alp_ipc_message_pack_int32(response, result.value ? ALP_STATUS_OK : alp_bundle_error()); 
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

AlpBundle alp_bundle_copy_to_store(AlpBundle package, const char * store, int flags)
{
	static int sequence;
	
	alp_status_t status;
	
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;
			
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_COPY_TO_STORE);
		alp_ipc_message_pack_int32(message, package.value);
		alp_ipc_message_pack_string(message, store);
		alp_ipc_message_pack_int32(message, flags);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			ALP_TMTL("some failure of IPC");
			// some failure of IPC
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return ALP_BUNDLE_NULL;
		}
		alp_ipc_message_destroy(message);
		
		AlpBundle result;
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result.value);
		alp_ipc_message_unpack_int32(response, &status);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);
		
		if (!result.value)
			set_error(status);
		
		return result;
		
	} else if (pm_status != STATUS_SERVER) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return ALP_BUNDLE_NULL;
	}
	
	S_STMT s;
	S_PREPARE(s, "SELECT scheme_id FROM containers WHERE containers.package_id = ?1");
	S_BIND_INT(s, 1, package.value);
	if(!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	int scheme = S_COL_INT(s,0);
	S_FINALIZE(s);
	
	if (scheme == BUNDLE_SCHEME_GHOST) {
		// GHost bundles cannot be copied
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}


	if (gNotify && !sMoving) {
		int handled = 0;
				
		AlpNotifyEventBundleCopy * d = g_malloc0(4 + strlen(store) + 1);
				
		d->index = package;
		strcpy(d->store, store);

		ALP_TMTL("Copy notification for package index %d to store %s", d->index, d->store);
		
		if (gHoldNotifications) {
			// Sorry, there's not a good reason to hold this notification for later
		} else {

			alp_status_t err = my_sync_notify(NOTIFY_NAME, ALP_NOTIFY_EVENT_BUNDLE_COPY, d, 4 + strlen(store) + 1, ALP_NOTIFY_QUEUE_DEFAULT, 0, 0, &handled);
			//ALP_FAIL_NON_FATAL_IF(err != ALP_STATUS_OK, "broadcast of package unregistration failed (err=%x)", err);
		}

		g_free(d);
	
		if (handled & 1) {
			// someone said it can't be coped
			set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
			return ALP_BUNDLE_NULL;
		}
	}
	
	// FIXME: wrap in transaction if not ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION

	gchar * bar_folder = NULL;
	gchar * package_name = NULL;
	gchar * package_location = NULL;
	int bar_prefer_archive;
	int package_is_archive;
	int store_id;
	
	S_PREPARE(s, "SELECT common_metadata.name,open_foldername,archive_filename FROM containers JOIN common_metadata ON (containers.package_id = common_metadata.package_id) WHERE containers.package_id=?1");
	S_BIND_INT(s, 1, package.value);
	if(!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	package_name = g_strdup(S_COL_TEXT(s,0));
	if (S_COL_TEXT(s,1)) {
		package_location = g_strdup(S_COL_TEXT(s,1));
		package_is_archive = 0;
	} else {
		package_location = g_strdup(S_COL_TEXT(s,2));
		package_is_archive = 1;
	}
	S_FINALIZE(s);
	
	S_PREPARE(s, "SELECT bar_folder,bar_prefer_archive,store_id FROM stores WHERE store_name = ?1");
	S_BIND_TEXT(s, 1, store);
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	bar_folder = g_strdup(S_COL_TEXT(s, 0));
	bar_prefer_archive = S_COL_INT(s, 1);
	store_id = S_COL_INT(s,2);
	S_FINALIZE(s);
	
	char target[PATH_MAX];
	char tmpname[PATH_MAX];
	
	//Check for duplicate name
	
	S_PREPARE(s, "SELECT containers.package_id FROM containers JOIN common_metadata ON (containers.package_id=common_metadata.package_id) WHERE containers.store_id = ?1 AND common_metadata.name = ?2");
	S_BIND_INT(s, 1, store_id);
	S_BIND_TEXT(s, 2, package_name);
	
	int delete_original = 0;
	
	if (S_STEP_ROW(s)) {
		// Bundle of same name found
		if (flags & ALP_BUNDLE_FLAG_OVERWRITE_DUPLICATE) {
			// we won't delete the original until we know that the new one is successfully registered
			delete_original = S_COL_INT(s, 0);
			S_FINALIZE(s);
		} else {
			S_FINALIZE(s);
			// There is a collision, so copy isn't possible onto this store
			ALP_TMTL("Collision, bundle of same name %s already registered", package_name);
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}		
	} else {
		S_FINALIZE(s);
	}
	
	// FIXME: consider permissions
	int mkdir_result = mkdir_p(bar_folder, 0777);
	if (mkdir_result == -1 && errno != EEXIST) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	// FIXME: crude, +4 skips 'bar:'
	sprintf(target, "%s/%s", bar_folder, package_name + 4);
	
	struct stat stbuf;
	
	if (stat(target, &stbuf) == 0) {
		int conflict=2;
		
		// FIXME: what if this gets stuck in a loop?
		for (;;) {
			// FIXME: crude, +4 skips 'bar:'
			sprintf(target, "%s/%s-%d", bar_folder, package_name + 4, conflict++);
			ALP_TMTL("Conflict copying bundle named %s, trying name %s", package_name, target);
			if (stat(target, &stbuf) == -1)
				break;
		}
	}

	// FIXME: crude, +4 skips 'bar:'
	sprintf(tmpname, "%s/TMP-%s-%d-%u", bar_folder, package_name+4, getpid(), sequence++);	
	
	if (package_is_archive && bar_prefer_archive) {
		// copy file, using sendfile

		int rd = open(package_location, O_RDONLY);
		// FIXME: consider permissions
		int wd = open(tmpname, O_WRONLY|O_CREAT, 0666);
		
		if (rd == -1) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_copy;
		}
		if (wd == -1) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_copy;
		}
			
		size_t len = lseek(rd, 0, SEEK_END);
		lseek(rd, 0, SEEK_SET);
		
		off_t o;
		size_t result = sendfile(wd, rd, &o, len);
		if (result < len) {
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail_copy;
		}
		
		close(rd);
		close(wd);
		rd = -1;
		wd = -1;
		
		if (0) {
	fail_copy:
		if (rd != -1)
			close(rd);
		if (wd != -1)
			close(wd);
		goto fail;
		}
	} else if (!package_is_archive && !bar_prefer_archive) {
		// copy folder, using `cp`

		int child_pid;
		
		sigset_t chld;
		sigemptyset(&chld);
		sigaddset(&chld, SIGCHLD);
	
		sigprocmask(SIG_BLOCK, &chld, &chld);
		
		// decide about SIGINT, HUP
		
		if ((child_pid = fork()) == 0) {
			int devnull=open("/dev/null", O_RDWR);
			if (devnull != 0) dup2(devnull, 0);
			if (devnull != 1) dup2(devnull, 1);
			if (devnull != 2) dup2(devnull, 2);
			int result = execlp("cp", "cp", "-a", package_location, tmpname, NULL);
			_exit(127);
		}
			
		int status, result;
		while ((result = waitpid(child_pid, &status, 0)) == -1 && (errno == EINTR))
			;
		
		sigprocmask(SIG_SETMASK, &chld, NULL);
			
		if (result == -1) {
			ALP_TMTL("error waiting for child pid %d: %s\n", child_pid, strerror(errno));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		
		if (WIFSIGNALED(status)) {
	 		ALP_TMTL("child %d dies with signal %d\n", child_pid, WTERMSIG(status));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
	 		goto fail;
		}
	} else if (package_is_archive && !bar_prefer_archive) {
		// unpack archive to folder, using `cramfsck`
		
		// FIXME: decide on permissions
		mkdir_p(tmpname, 0666);
	
		int child_pid;
		
		sigset_t chld;
		sigemptyset(&chld);
		sigaddset(&chld, SIGCHLD);
	
		sigprocmask(SIG_BLOCK, &chld, &chld);
		
		// decide about SIGINT, HUP
		
		if ((child_pid = fork()) == 0) {
			int devnull=open("/dev/null", O_RDWR);
			if (devnull != 0) dup2(devnull, 0);
			if (devnull != 1) dup2(devnull, 1);
			if (devnull != 2) dup2(devnull, 2);
			int result = execlp("cramfsck", "cramfsck", "-i", "-x", package_location, tmpname, NULL);
			_exit(127);
		}
			
		int status, result;
		while ((result = waitpid(child_pid, &status, 0)) == -1 && (errno == EINTR))
			;
		
		sigprocmask(SIG_SETMASK, &chld, NULL);
			
		if (result == -1) {
			ALP_TMTL("error waiting for child pid %d: %s\n", child_pid, strerror(errno));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		
		if (WIFSIGNALED(status)) {
	 		ALP_TMTL("child %d dies with signal %d\n", child_pid, WTERMSIG(status));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
	 		goto fail;
		}
		
	} else if (!package_is_archive && bar_prefer_archive) {
		// package folder to archive, using `mkcramfs`
		
		char manifest[PATH_MAX];
		sprintf(manifest, "%s%s%s", package_location, package_name, ALP_BUNDLE_BAR_MANIFEST);
		
		int child_pid;
		
		sigset_t chld;
		sigemptyset(&chld);
		sigaddset(&chld, SIGCHLD);
	
		sigprocmask(SIG_BLOCK, &chld, &chld);
		
		// decide about SIGINT, HUP
		
		if ((child_pid = fork()) == 0) {
			int devnull=open("/dev/null", O_RDWR);
			if (devnull != 0) dup2(devnull, 0);
			if (devnull != 1) dup2(devnull, 1);
			if (devnull != 2) dup2(devnull, 2);
			int result = execlp("mkcramfs", "mkcramfs", "-i", manifest, "-n", "Binder Archive 1", tmpname, package_location, NULL);
			_exit(127);
		}
		
		int status, result;
		while ((result = waitpid(child_pid, &status, 0)) == -1 && (errno == EINTR))
			;
		
		sigprocmask(SIG_SETMASK, &chld, NULL);
		
		if (result == -1) {
			ALP_TMTL("error waiting for child pid %d: %s\n", child_pid, strerror(errno));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		
		if (WIFSIGNALED(status)) {
	 		ALP_TMTL("child %d dies with signal %d\n", child_pid, WTERMSIG(status));
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
	 		goto fail;
		}
		
		ALP_TMTL("Result of system: %d\n", result);
		
	}

	// move temporary file/folder to real name
	if (rename(tmpname, target) == -1) {
		ALP_TMTL("Error renaming temp copy location '%s' to destination '%s': %s", tmpname, target, strerror(errno));
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}
	
	
	// FIXME: it'd be nice to directly copy the metadata instead of having to re-parse it
	AlpBundle result = alp_prv_bundle_register_bundle(target, BUNDLE_SCHEME_NAME_BAR, store, (delete_original != 0) ? ALP_PRV_BUNDLE_FLAG_DO_NOT_NOTIFY : 0);
	
	if (alp_bundle_is_NULL(result)) {
		status = ALP_STATUS_BUNDLEMGR_FAILURE;
		goto fail;
	}

	if (delete_original) {
		// The new bundle is registered, but is not active and has not taken its name, so there
		// won't be any confusion when the one one is deleted
		AlpBundle b = {delete_original};
		ALP_TMTL("Deleting original conflict bundle %d", delete_original);
		if ((flags & ALP_PRV_BUNDLE_FLAG_LAZY_DELETE) && (flags & ALP_PRV_BUNDLE_FLAG_FORCE_UNREGISTER)) {
			ALP_TMTL("Trying non-lazy delete first");
			// did the caller ask us to force a lazy delete? If so, first just try the force.
			// If that succeeds, we're ready to continue
			if (alp_bundle_delete(b, flags & ~ALP_PRV_BUNDLE_FLAG_LAZY_DELETE) != ALP_STATUS_OK) {
				ALP_TMTL("OK, going on to lazy delete");
				// otherwise, we need to really lazily delete the old one. But in order for
				// the new one to immediately kick in, we'll need to summarily deactivate the name on
				// the old one
				alp_prv_bundle_deactivate(b, flags);
				
				// FIXME: do we also want to send out an unreg notification early?
				//alp_prv_bundle_unreg_notify(b);
				
				// Then invoke the lazy delete
				alp_bundle_delete(b, flags);
				
				// now continue on to the notify of the new one -- the old one will be deleted
				// at some point in the future
			}
			
		} else if (alp_bundle_delete(b, flags) != ALP_STATUS_OK) { // don't delete R/W data
			ALP_TMTL("Delete failed, unwinding");
			// delete failed, unwind install
			alp_bundle_delete(result, flags & ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
			status = ALP_STATUS_BUNDLEMGR_FAILURE;
			goto fail;
		}
		ALP_TMTL("Delete worked, now activating copied package");
		
		// now we activate the new one
		alp_prv_bundle_reg_notify(result, flags);
	}
	
	return result;

fail:
	// unwind any partial copy
	rmdir_contents(tmpname);
	rmdir(tmpname);
	unlink(tmpname);
	
	rmdir_contents(target);
	rmdir(target);
	unlink(target);

	g_free(package_location);
	g_free(package_name);
	g_free(bar_folder);
	
	set_error(status);
	return ALP_BUNDLE_NULL;
}


void handle_client_alp_bundle_copy_to_store(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	gchar * store;
	AlpBundle package;
	int flags;
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_int32(message, &package.value);
	alp_ipc_message_unpack_string(message, &store);
	alp_ipc_message_unpack_int32(message, &flags);
	alp_ipc_message_unpack_end(message);
			
	AlpBundle result;
	
	gboolean permission = has_store_permission_by_store(connection, PERM_INSTALL_POLICY_COLUMN, store);

	if (permission) {
		result = alp_bundle_copy_to_store(package, store, flags);
	} else {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		result = ALP_BUNDLE_NULL;
	}
	
	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_COPY_TO_STORE_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result.value); 
	alp_ipc_message_pack_int32(response, result.value ? ALP_STATUS_OK : alp_bundle_error()); 
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

alp_status_t alp_prv_bundle_add_store(const char * store, const char * arguments[])
{
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;
		
		int i=0, j;
		// Look for NULL keys -- NULL values are OK
		while (arguments[i])
			i+=2;
			
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_ADD_STORE);
		alp_ipc_message_pack_string(message, store);
		alp_ipc_message_pack_int32(message, i);
		for (j=0;j<i;j++)
			alp_ipc_message_pack_string(message, arguments[j]);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			ALP_TMTL("some failure of IPC");
			// some failure of IPC
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		
		alp_status_t result;
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);
		
		return set_error(result);
		
	} else if (pm_status != STATUS_SERVER)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	

	char names[1024];
	char values[1024];
	
	sprintf(names, "store_name");
	sprintf(values, "'%s'", store);
	
	int i;
	for (i=0; arguments[i]; i+=2) {
		sprintf(names + strlen(names), ",%s", arguments[i]);
		if (arguments[i+1]) {
			sprintf(values + strlen(values), ",'%s'", arguments[i+1]);
		} else {
			sprintf(values + strlen(values), ",NULL");
		}
	}
	
	char statement[2048];
	
	sprintf(statement, "INSERT OR ABORT INTO stores (%s) VALUES (%s)", names, values);
	S_STMT s;
	S_PREPARE(s, statement);
	if (S_DO_OR_ABORT(s)) {
		// abort, presumably due to integrity violation because of store of same
		// name
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}
		
	return ALP_STATUS_OK;
}

void handle_client_alp_prv_bundle_add_store(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	gchar * store;
	int count;
	char ** arguments;
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_string(message, &store);
	alp_ipc_message_unpack_int32(message, &count);
	arguments = g_new(char*, count+1);
	int i;
	for (i=0;i<count;i++)
		alp_ipc_message_unpack_string(message, &arguments[i]);
	arguments[i] = NULL;
	alp_ipc_message_unpack_end(message);
			
	alp_status_t result;

	gboolean permission;
	
	permission = has_permission(connection, PERM_ADD_STORE_POLICY);
	
	if (permission) {
		result = alp_prv_bundle_add_store(store, (const char**)arguments);
	} else {
		result = ALP_STATUS_BUNDLEMGR_FAILURE;
	}
	
	for (i=0;i<count;i++)
		g_free(arguments[i]);
	g_free(arguments);
	
	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_ADD_STORE_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result); 
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

alp_status_t alp_prv_bundle_delete_store(const char * store)
{
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;
			
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_DELETE_STORE);
		alp_ipc_message_pack_string(message, store);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			ALP_TMTL("some failure of IPC");
			// some failure of IPC
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		alp_ipc_message_destroy(message);

		alp_status_t result;
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);
		
		return set_error(result);
		
	} else if (pm_status != STATUS_SERVER)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
	// first, locate packages which are on that store
	GArray * array = g_array_new(FALSE, FALSE, sizeof(AlpBundle));
	
	if (!array)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);

	S_STMT s;
	S_PREPARE(s, "SELECT containers.package_id FROM containers JOIN stores ON (containers.store_id = stores.store_id) WHERE stores.store_name = ?1");
	S_BIND_TEXT(s, 1, store);
	S_BEGIN_SELECT(s) {
		AlpBundle p;
		p.value = S_COL_INT(s, 0);
		g_array_append_val(array, p);
	}
	S_END_SELECT(s);
	
	unsigned int i;
	for (i=0;i<array->len;i++) {
		AlpBundle p = g_array_index(array, AlpBundle, i);
		alp_prv_bundle_deactivate(p, 0);
	}
	
	// update names, sending out notifications
	alp_prv_bundle_update_names(NULL, 0);

	// unregister them (not delete!)
	for (i=0;i<array->len;i++) {
		AlpBundle p = g_array_index(array, AlpBundle, i);
		alp_prv_bundle_unreg_notify(p, 0);
	}

	int fail=0;
	for (i=0;i<array->len;i++) {
		AlpBundle p = g_array_index(array, AlpBundle, i);
		alp_status_t result = alp_prv_bundle_unregister_bundle(p, ALP_PRV_BUNDLE_FLAG_DO_NOT_NOTIFY);
		if (result != ALP_STATUS_OK) {
			// put it back
			alp_prv_bundle_reg_notify(p, 0);
			fail = 1;
		}
	}

	g_array_free(array, FALSE);
	
	if (!fail) {
	
		// finally remove the record for the storeif (g_array_index(ary, AlpBundleRef, i) != NULL)
		S_PREPARE(s, "DELETE FROM stores WHERE store_name = ?1");
		S_BIND_TEXT(s, 1, store);
		S_DO(s);

		return ALP_STATUS_OK;
	} else {
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	}
}

void handle_client_alp_prv_bundle_delete_store(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	gchar *store;
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_string(message, &store);
	alp_ipc_message_unpack_end(message);
			
	alp_status_t result;

	gboolean permission;
	
	permission = has_permission(connection, PERM_DELETE_STORE_POLICY);
	
	if (permission) {
		result = alp_prv_bundle_delete_store(store);
	} else {
		result = ALP_STATUS_BUNDLEMGR_FAILURE;
	}
	
	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_DELETE_STORE_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result); 
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}

GHashTable * alp_bundle_store(const char * store)
{
	GHashTable * results = g_hash_table_new_full(g_str_hash, g_str_equal, g_free, g_free);
	
	if (!results) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	S_STMT s;
	S_PREPARE(s, "SELECT * FROM stores WHERE stores.store_name = ?1");
	S_BIND_TEXT(s, 1, store);
	S_BEGIN_SELECT(s) {
		int i=0;
		for (i=0;i<sqlite3_column_count(s);i++) {
			gchar * key = g_strdup(sqlite3_column_name(s, i));
			gchar * value = g_strdup(S_COL_TEXT(s,i));
			g_hash_table_insert(results, key, value);
		}
	}
	S_END_SELECT(s);
	
	return results;
}

gchar * alp_bundle_store_value(const char * store, const char * key)
{
	char cmd[] = "SELECT ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^ FROM stores WHERE store_name = ?1";
	
	if (safe_sql_fillin(cmd, key)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}
	
	gchar * result = NULL;
	
	S_STMT s;
	if (!S_PREPARE_P(s, cmd)) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL; // must be an invalid column
	}
	
	S_BIND_TEXT(s, 1, store);
	S_BEGIN_SELECT(s) {
		result = g_strdup(S_COL_TEXT(s, 0));
		// FIXME: detect allocation failure
	}
	S_END_SELECT(s);
	
	if (!result) {
		set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		return NULL;
	}

	return result;
}

// FIXME: fix code to put everything into as few transactions as possible, using nested_transaction flag
void alp_prv_ghost_scan_store(const char * store, int store_id, int ghost_storage, int wipe_invalid)
{
	
	GList * added = NULL;
	GList * removed = NULL;
	GList * recycle = NULL;
	
	const int debug = 0;
	
	Alp68KDBInfo* 	db_list = 0;
	unsigned int 	n_db, quoted = 0, i;
	unsigned int	db_type = 0;
	int				ret = 0;
	
	// If GHost isn't available, unregister everything

	if (!gGHost) {
	
		n_db = 0;
		db_list = NULL;
		
		ret = 0;

	} else {
		ret = -1;
		if (ghost_start() == ALP_STATUS_OK)
		{
			//get the list of ghost apps from the ghost data base
			ret = alp_gras_db_create_list(GSESSION, /* type */ char4_to_int("appl"), ghost_storage, &n_db, &db_list);
			ghost_stop(ret != ALP_STATUS_OK && ALP_STATUS_VALUE(ret) != ALP_STATUS_GRAS_GARNET_ERROR);
		}
	}

	if (ret != 0) {
		ALP_TMTL("alp_gras_db_create_list failed \n");
		return ;	
	}

	if (debug) {
		ALP_TMTL("Apps in  in ghost data base");
		for(i=0; i<n_db; i++) {
			ALP_TMTL("\"%s\"\n", db_list[i].name);
		}
	}		

	//initailize sql stm
	S_EXEC("BEGIN");
	S_STMT s;


	// Generate list of databases which are on store in ghost
	
	S_PREPARE(s, "CREATE TEMPORARY TABLE present_apps (app_name TEXT, app_type_str TEXT, app_creator_str TEXT, existing_container INTEGER)");
	S_DO(s);
	S_PREPARE(s, "INSERT INTO present_apps VALUES (?1, ?2, ?3, NULL)");
	
	for(i = 0 ; i < n_db ; i ++) {
/*		
		// FIXME: Internal apps ("Ghost Shell", "Ghost Quit Launcher") should be hidden in some other fashion
		if (ghost_storage == 0 && strncmp(db_list[i].name, "Ghost ", 6) == 0) {
			continue;
		}

		if (ghost_storage == 0 && strcmp(db_list[i].name, "SyncAuto") == 0) {
			continue;
		}
		
		// FIXME: as a boon to efficiency, don't register anything which isn't an application
		if (db_list[i].type != char4_to_int("appl")) {
			continue;
		}
*/
		if (db_list[i].attributes & dmHdrAttrHidden || !(db_list[i].attributes & dmHdrAttrResDB))
		{
//			printf("******* [%s] is hidden or not resource db - skipping\"n, ", db_list[i].name);
			continue;
		}

		char creator_name[5], type_name[5];
		S_BIND_TEXT(s, 1, db_list[i].name);
		S_BIND_TEXT(s, 2, int_to_char4(db_list[i].type, type_name));
		S_BIND_TEXT(s, 3, int_to_char4(db_list[i].creator, creator_name));
		S_STEP_ROW(s);
		S_RESET(s);
	}
	S_FINALIZE(s);
	
	S_PREPARE(s, "UPDATE present_apps SET existing_container = (SELECT containers.package_id FROM containers "
			" JOIN ghost_metadata ON (containers.package_id = ghost_metadata.package_id) "
			" WHERE containers.store_id = ?1 AND containers.scheme_id = 2 AND "
			" ghost_db_name = app_name AND ghost_db_type_str = app_type_str AND "
			" ghost_db_creator_str = app_creator_str)");
	S_BIND_INT(s, 1, store_id);
	S_STEP_ROW(s);
	S_FINALIZE(s);

	if (debug) {
		S_PREPARE(s, "SELECT app_name,app_type_str,app_creator_str,existing_container FROM present_apps");
		S_BIND_INT(s, 1, store_id);

		S_BEGIN_SELECT(s) {
			ALP_TMTL("ghost_db_name %s, type %s, creator %s, existing container %d (NULL=%d) \n", S_COL_TEXT(s,0), S_COL_TEXT(s,1), S_COL_TEXT(s,2), S_COL_INT(s,3), S_COL_ISNULL(s,3));
		}S_END_SELECT(s);
	}
	

	//find apps which aren't registered, and mark them to be added
	S_PREPARE(s, "SELECT app_name,app_type_str,app_creator_str FROM present_apps WHERE existing_container ISNULL");
	
	S_BEGIN_SELECT(s) {
		char buffer[255];
		sprintf(buffer, "%s:%d_%s_%s_%s", BUNDLE_SCHEME_NAME_GHOST, ghost_storage, S_COL_TEXT(s, 1), S_COL_TEXT(s,2), S_COL_TEXT(s,0)  );

		if (debug)
			ALP_TMTL("Adding GHOST APP %s  ", buffer);
	
		added = g_list_prepend(added, g_strdup(buffer));
	}
	S_END_SELECT(s);
	
	//find apps which are registered but not in the dblist, and mark them as removed
	S_PREPARE(s, "SELECT ghost_db_name, ghost_db_type_str, ghost_db_creator_str, containers.package_id FROM containers "
			" JOIN ghost_metadata ON (containers.package_id = ghost_metadata.package_id) "
			" WHERE containers.store_id = ?1 "
			"  EXCEPT SELECT app_name,app_type_str,app_creator_str,existing_container FROM present_apps");
		
	S_BIND_INT(s, 1, store_id);
	
	S_BEGIN_SELECT(s) {
		char buffer[255];
		sprintf(buffer, "%s:%d_%s_%s_%s", BUNDLE_SCHEME_NAME_GHOST, ghost_storage, S_COL_TEXT(s, 1), S_COL_TEXT(s,2), S_COL_TEXT(s,0) );

		if (debug)
			ALP_TMTL("Removing GHOST APP %s, container id %d  ", buffer, S_COL_INT(s,3));
			
		removed = g_list_prepend(removed, (gpointer) S_COL_INT(s,3));
	}
	S_END_SELECT(s);
	
	//clean up 
	//delete temp table 
	S_PREPARE(s, "DROP TABLE present_apps");
	S_DO(s);
	S_EXEC("COMMIT");
	
	
	//find apps which are to be recycled 
	
	// FIXME: does ghost need any triggering for Garnet "recycle on reboot" functionality?

	added = g_list_reverse(added);
	removed = g_list_reverse(removed);	
	
	//register from added list and send notification
	GList * iter;
	if (debug)
		ALP_TMTL("STORE, STAGE 1 REGISTER NEW FOR GHOST");

	for (iter = added; iter; iter = g_list_next(iter)) {
		gchar * new_path = iter->data;
		if (debug)
			ALP_TMTL("Register %s", new_path);
		AlpBundle result = alp_prv_bundle_register_bundle(new_path, BUNDLE_SCHEME_NAME_GHOST, store, ALP_PRV_BUNDLE_FLAG_DO_NOT_NOTIFY);
		if (debug)
			ALP_TMTL(" Got %d", result.value);
		if (alp_bundle_is_NULL(result)) {

		}
		
		g_free(iter->data);
		iter->data = (void*)result.value;
	}
	
	// Step 3: notify about prior registration of added packages

	if (debug)
		ALP_TMTL("STORE, STAGE 2 NOTIFY OF REGISTER FOR GHOST");

	for (; added; added = g_list_delete_link(added,added)) {
		AlpBundle p;
		p.value = (int)added->data;
		alp_prv_bundle_reg_notify(p, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
	 }
	
	
	//unregister from remove list and send unregister notifictaion 
	//deactivate packages which are being deleted

	if (debug)
		ALP_TMTL("STORE, STAGE 3 DEACTIVATE OLD PACKAGES FOR GHOST");

	for (iter = removed; iter; iter = g_list_next(iter)) {
		AlpBundle p;
		p.value = (int)iter->data;
		if (debug)
				ALP_TMTL("Deactivating package %d", p.value);
		alp_prv_bundle_deactivate(p, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
	 }
	
	//Update names, generating add/change/remove notifications

	if (debug)
		ALP_TMTL("STORE, STAGE 4 UPDATE NAMES FOR GHOST");

	alp_prv_bundle_update_names(NULL, 0);
	

	//notify about impending unregistration of removed packages
	if (debug)
		ALP_TMTL("STORE, STAGE 5 NOTIFY OF UNREGISTER FOR GHOST");

	for (iter = removed; iter; iter = g_list_next(iter)) {
		AlpBundle p;
		p.value = (int)iter->data;
		alp_prv_bundle_unreg_notify(p, 0);
	}

	
	if (debug)
		ALP_TMTL("STORE, STAGE 6 REMOVE OLD FOR GHOST");
	
	int names_again = 0;

	for (; removed; removed = g_list_delete_link(removed,removed)) {
		AlpBundle p;
		p.value = (int)removed->data;
		if (alp_prv_bundle_unregister_bundle(p, ALP_PRV_BUNDLE_FLAG_DO_NOT_NOTIFY) != ALP_STATUS_OK) {
			//oops, the unregister failed -- notify it as being registered again
			ALP_TMTL("Unregister of package %d failed, putting it back", p.value);
			// reactivate it, and notify of its presence
			alp_prv_bundle_reg_notify(p, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
			names_again = 1;
		}
	}

	//fix up names, if problems occured
	
	if (debug)
		ALP_TMTL("STORE, STAGE 7 FIX UP NAMES FOR GHOST");

	if (names_again) 
		alp_prv_bundle_update_names(NULL, 0);
	
	if (db_list)
		free(db_list);
	
}


// FIXME: fix code to put everything into as few transactions as possible, using nested_transaction flag
void alp_prv_java_scan_store(const char * store, int store_id, int wipe_invalid)
{
	GList * added = NULL;
	GList * removed = NULL;
	GList * recycle = NULL;
	
	int debug = 1; 
	
	
	//Get list of apps from java support
	struct java_program* java_app_data_p;
	int java_app_count;
	if(java_sync(&java_app_count, &java_app_data_p) != 0 ) {
		ALP_TMTL("java_sync failed \n");
		return; 		
	}
	
	//initailize sql stm
	S_EXEC("BEGIN");
	S_STMT s;	
	S_PREPARE(s, "CREATE TEMPORARY TABLE present_java_apps (app_name TEXT, app_id INTEGER, existing_container INTEGER)");
	S_DO(s);
	S_PREPARE(s, "INSERT INTO present_java_apps VALUES (?1, ?2, NULL)");
	int i = 0;
	for(i = 0; i < java_app_count; i++) {
		ALP_TMTL("midlet_name %s midlet_id %d \n", java_app_data_p[i].name, java_app_data_p[i].id );
		S_BIND_TEXT(s, 1, java_app_data_p[i].name);
		S_BIND_INT(s, 2, java_app_data_p[i].id);
		S_STEP_ROW(s);
		S_RESET(s);
	}
	S_FINALIZE(s);
	
	
	S_PREPARE(s, "UPDATE present_java_apps SET existing_container = (SELECT containers.package_id FROM containers "
			" JOIN java_metadata ON (containers.package_id = java_metadata.package_id) "
			" WHERE containers.store_id = ?1 AND containers.scheme_id = 3 AND "
			" midlet_name = app_name AND midlet_id = app_id)");
			
	S_BIND_INT(s, 1, store_id);
	S_STEP_ROW(s);
	S_FINALIZE(s);
	
	//find apps which aren't registered, and mark them to be added
	S_PREPARE(s, "SELECT app_id, app_name  FROM present_java_apps WHERE existing_container ISNULL");
	
	S_BEGIN_SELECT(s) {
		char buffer[PATH_MAX];
		sprintf(buffer, "%s:%d_%s", BUNDLE_SCHEME_NAME_JAVA, S_COL_INT(s,0), S_COL_TEXT(s, 1));

		if (debug)
			ALP_TMTL("Adding JAVA APP %s  ", buffer);
	
		added = g_list_prepend(added, g_strdup(buffer));
	}
	S_END_SELECT(s);
	
	//find apps which are registered but not in the dblist, and mark them as removed
	S_PREPARE(s, "SELECT midlet_name, midlet_id, containers.package_id FROM containers "
			" JOIN java_metadata ON (containers.package_id = java_metadata.package_id) "
			" WHERE containers.store_id = ?1 "
			" EXCEPT SELECT app_name,app_id, existing_container FROM present_java_apps");
		
	S_BIND_INT(s, 1, store_id);
	
	S_BEGIN_SELECT(s) {
		if (debug) {
			char buffer[PATH_MAX];
			sprintf(buffer, "%s:%d_%s", BUNDLE_SCHEME_NAME_JAVA,  S_COL_INT(s,1), S_COL_TEXT(s, 0));
			ALP_TMTL("Removing JAVA APP %s, container id %d  ", buffer, S_COL_INT(s,2));
		}
			
		removed = g_list_prepend(removed, (gpointer) S_COL_INT(s,2));
	}
	S_END_SELECT(s);
	
	//clean up 
	//delete temp table 
	S_PREPARE(s, "DROP TABLE present_java_apps");
	S_DO(s);
	S_EXEC("COMMIT");
	
	

	added = g_list_reverse(added);
	removed = g_list_reverse(removed);	
	
	//register from added list and send notification
	GList * iter;
	if (debug)
		ALP_TMTL("STORE, STAGE 1 REGISTER NEW FOR JAVA");

	for (iter = added; iter; iter = g_list_next(iter)) {
		gchar * new_path = iter->data;
		if (debug)
			ALP_TMTL("Register %s", new_path);
		AlpBundle result = alp_prv_bundle_register_bundle(new_path, BUNDLE_SCHEME_NAME_JAVA, store, 0);
		if (debug)
			ALP_TMTL(" Got %d", result.value);
		if (alp_bundle_is_NULL(result)) {

		}
		
		g_free(iter->data);
		iter->data = (void*)result.value;
	}
	
	// Step 3: notify about prior registration of added packages

	if (debug)
		ALP_TMTL("STORE, STAGE 2 NOTIFY OF REGISTER FOR JAVA");

	for (; added; added = g_list_delete_link(added,added)) {
		AlpBundle p;
		p.value = (int)added->data;
		alp_prv_bundle_reg_notify(p, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
	 }
	
	
	//unregister from remove list and send unregister notifictaion 
	//deactivate packages which are being deleted

	if (debug)
		ALP_TMTL("STORE, STAGE 3 DEACTIVATE OLD PACKAGES FOR JAVA");

	for (iter = removed; iter; iter = g_list_next(iter)) {
		AlpBundle p;
		p.value = (int)iter->data;
		if (debug)
				ALP_TMTL("Deactivating package %d", p.value);
		alp_prv_bundle_deactivate(p, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
	 }
	
	//Update names, generating add/change/remove notifications

	if (debug)
		ALP_TMTL("STORE, STAGE 4 UPDATE NAMES FOR JAVA");

	alp_prv_bundle_update_names(NULL, 0);
	

	//notify about impending unregistration of removed packages
	if (debug)
		ALP_TMTL("STORE, STAGE 5 NOTIFY OF UNREGISTER FOR JAVA");

	for (iter = removed; iter; iter = g_list_next(iter)) {
		AlpBundle p;
		p.value = (int)iter->data;
		alp_prv_bundle_unreg_notify(p, 0);
	}

	
	if (debug)
		ALP_TMTL("STORE, STAGE 6 REMOVE OLD FOR JAVA");
	
	int names_again = 0;

	for (; removed; removed = g_list_delete_link(removed,removed)) {
		AlpBundle p;
		p.value = (int)removed->data;
		if (alp_prv_bundle_unregister_bundle(p, ALP_PRV_BUNDLE_FLAG_DO_NOT_NOTIFY) != ALP_STATUS_OK) {
			//oops, the unregister failed -- notify it as being registered again
			ALP_TMTL("Unregister of package %d failed, putting it back", p.value);
			// reactivate it, and notify of its presence
			alp_prv_bundle_reg_notify(p, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
			names_again = 1;
		}
	}

	//fix up names, if problems occured
	
	if (debug)
		ALP_TMTL("STORE, STAGE 7 FIX UP NAMES FOR JAVA");

	if (names_again) 
		alp_prv_bundle_update_names(NULL, 0);
	
}

alp_status_t alp_prv_bundle_scan_store(const char * store, int wipe_invalid, int scheme_skip)
{
	// NOTE: this routine does not notice modified packages: they need to be manually
	// unregistered and registered to get new metadata into the system.
	
	if (pm_status == STATUS_CLIENT) {
		AlpMessage * message, *response;
		
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message, MSG_PACKAGE_SCAN_STORE);
		alp_ipc_message_pack_string(message, store);
		alp_ipc_message_pack_int32(message, wipe_invalid);
		alp_ipc_message_pack_int32(message, scheme_skip);
		alp_ipc_message_pack_end(message);
		if (alp_ipc_connection_send_with_response(pm_client_connection, message, 0, &response)) {
			alp_ipc_message_destroy(message);
			ALP_TMTL("some failure of IPC");
			// some failure of IPC
			return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
		}
		
		alp_status_t result;
		alp_ipc_message_unpack_start(response);
		alp_ipc_message_unpack_int32(response, &result);
		alp_ipc_message_unpack_end(response);
		alp_ipc_message_destroy(response);
		
		return set_error(result);
		
	} else if (pm_status != STATUS_SERVER)
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE);
	
	const int debug = 0;

#define BEGIN do { S_EXEC("BEGIN"); \
	if (gGSContext) alp_global_settings_begin_transaction(gGSContext); } while (0)
#define COMMIT do { if (gGSContext) alp_global_settings_complete_transaction(gGSContext); \
	S_EXEC("COMMIT");  } while (0)

	BEGIN;

	// first, collect list of files actually in the bar folder associated with the store
	
	if (debug)
		ALP_TMTL("Scanning store '%s'", store);

	S_STMT s;
	S_PREPARE(s, "SELECT bar_folder,store_id,wipe_by_default,ghost_storage,jam_store FROM stores WHERE store_name = ?1");
	S_BIND_TEXT(s, 1, store);
	if (!S_STEP_ROW(s)) {
		S_FINALIZE(s);
		S_EXEC("COMMIT");
		ALP_TMTL("Scan of store %s failed, no such store!", store);
		return set_error(ALP_STATUS_BUNDLEMGR_FAILURE); 	
	}
	gchar * path = g_strdup(S_COL_TEXT(s, 0));
	int store_id = S_COL_INT(s,1);
	int wipe_by_default = S_COL_INT(s,2);
	int ghost_storage = S_COL_ISNULL(s,3) ? -1 : S_COL_INT(s,3);
	int jam_store = S_COL_INT(s,4);
	S_FINALIZE(s);
	
	// If bar_folder was NULL, then this store doesn't have bars
	
	if (!path || (scheme_skip & (1<<BUNDLE_SCHEME_BAR))) {
		COMMIT;
		goto bar_skip;
	}
	
	S_PREPARE(s, "CREATE TEMPORARY TABLE present_files (saw_filename TEXT)");
	S_DO(s);
	S_PREPARE(s, "INSERT INTO present_files VALUES (?1)");

	GList * added = NULL;
	GList * removed = NULL;
	GList * wipe = NULL;
	GList * recycle = NULL;
	
	char dir_buf[PATH_MAX];
	
	// We want to resolve any symlinks in the bundle path first
	if (realpath(path, dir_buf) == NULL) {
		//fallback if realpath complains -- the opendir will just fail in turn
		strcpy(dir_buf, path);
	} 

	DIR * dir = opendir(dir_buf);

	if (dir) {
		struct dirent * d;
		while ((d = readdir(dir))) {
		
			if (d->d_name[0] == '.')
				continue;
		
			char buf[PATH_MAX];
			
			sprintf(buf, "%s/%s", dir_buf, d->d_name);
	
			// Nasty hard-coded logic: any folder or file starting with 'TMP-'
			// is assumed to be the result of a prior failed attempt to install
			// something, and is fair game to be deleted
			if (strncmp(d->d_name, "TMP-", 4) == 0) {
				wipe = g_list_prepend(wipe, g_strdup(path));
				continue;
			}
			
			// Do the same for bundles marked for recycling by the 'lazy delete'
			// mechanism.
			if (strncmp(d->d_name, "RECYCLE-", 8) == 0) {
				wipe = g_list_prepend(wipe, g_strdup(path));
				continue;
			}
			
			struct stat stbuf;
			if ((lstat(buf, &stbuf) == 0) && S_ISLNK(stbuf.st_mode)) {
			
				// If the directory entry is itself a symlink, resolve it before
				// we record it. This is important because the entries in the
				// database are fully resolved. If we don't resolve our entries,
				// they won't textually match the ones already in the table.

				char result[PATH_MAX];
				if (realpath(buf, result) == NULL)
					strcpy(result, buf);
				strcpy(buf, result);
			}
			
			if (debug)
				ALP_TMTL("Saw real file '%s' in store %s", buf, store);

			S_BIND_TEXT(s, 1, buf);
			S_STEP_ROW(s);
			S_RESET(s);
		}
	
		closedir(dir);
	} else {
		ALP_TMTL("In scan, no such dir %s", path);
	}

	S_FINALIZE(s);
	
	// next look for any files which aren't already registered, and record their names
		
	S_PREPARE(s, "SELECT saw_filename FROM present_files EXCEPT SELECT coalesce(open_foldername,archive_filename) FROM containers WHERE containers.store_id = ?1 AND containers.scheme_id = 1");
	S_BIND_INT(s, 1, store_id);
	S_BEGIN_SELECT(s) {
		if (debug)
			ALP_TMTL("Adding file %s to store %s", S_COL_TEXT(s,0), store);
		added = g_list_prepend(added, g_strdup(S_COL_TEXT(s,0)));
	}
	S_END_SELECT(s);

	// then look for any packages which are registered, but no longer in the file list
	
	S_PREPARE(s, "SELECT containers.package_id,coalesce(containers.open_foldername,containers.archive_filename) FROM containers LEFT OUTER JOIN present_files ON coalesce(containers.open_foldername,containers.archive_filename) = present_files.saw_filename WHERE containers.store_id = ?1 AND containers.scheme_id = 1 AND present_files.saw_filename ISNULL");
	S_BIND_INT(s, 1, store_id);
	S_BEGIN_SELECT(s) {
		if (debug)
			ALP_TMTL("Removing file %s (id %d) from store %s", S_COL_TEXT(s,1), S_COL_INT(s,0), store);
		removed = g_list_prepend(removed, (gpointer) S_COL_INT(s,0));
	}
	S_END_SELECT(s);
	
	// clean up
	
	S_PREPARE(s, "DROP TABLE present_files");
	S_DO(s);

	// now find anything marked for recycling
	
	S_PREPARE(s, "SELECT containers.package_id FROM containers JOIN common_metadata ON (containers.package_id = common_metadata.package_id) WHERE containers.store_id = ?1 AND containers.scheme_id = 1 AND common_metadata.recycle");
	S_BIND_INT(s, 1, store_id);
	S_BEGIN_SELECT(s) {
		recycle = g_list_prepend(recycle, (gpointer) S_COL_INT(s, 0));
	}
	S_END_SELECT(s);
	
	// do the work: register new packages, unregister removed packages, and wipe any temps
	
	// Step 1: wipe out any junk
	
	if (debug)
		ALP_TMTL("STORE, STAGE 1 WIPE");
	
	for (wipe = g_list_reverse(wipe); wipe; wipe = g_list_delete_link(wipe, wipe)) {
		gchar * wipe_path = added->data;
		if (unlink(wipe_path) == -1 && errno == EISDIR) {
			rmdir_contents(wipe_path);
			rmdir(wipe_path);
		}
		g_free(wipe_path);
	}

	added = g_list_reverse(added);
	removed = g_list_reverse(removed);	
	recycle = g_list_reverse(recycle);
	
	GList * iter;
	
	// Step 2: register new packages
	
	if (debug)
		ALP_TMTL("STORE, STAGE 2 REGISTER NEW");

	for (iter = added; iter; iter = g_list_next(iter)) {
		gchar * new_path = iter->data;
		if (debug)
			ALP_TMTL("Register %s", new_path);
		AlpBundle result = alp_prv_bundle_register_bundle(new_path, BUNDLE_SCHEME_NAME_BAR, store, ALP_PRV_BUNDLE_FLAG_DO_NOT_NOTIFY|ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
		if (debug)
			ALP_TMTL(" Got %d", result.value);
		if (alp_bundle_is_NULL(result)) {
			//FIXME: ok, we need more refinement of the error code, so we know what is
			//going on
			if (wipe_invalid == 2 || (wipe_invalid == 1 && wipe_by_default)) {
				ALP_TMTL("In scan, invalid bundle '%s', erasing it!", new_path);
				if (unlink(new_path) == -1 && errno == EISDIR) {
					rmdir_contents(new_path);
					rmdir(new_path);
				}
			}
		}
		
		g_free(iter->data);
		iter->data = (void*)result.value;
	}
	
	// Step 3: notify about prior registration of added packages

	if (debug)
		ALP_TMTL("STORE, STAGE 3 NOTIFY OF REGISTER");

	for (; added; added = g_list_delete_link(added,added)) {
		AlpBundle p;
		p.value = (int)added->data;
		alp_prv_bundle_reg_notify(p, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
	}

	// Step 4: deactivate packages which are being deleted

	if (debug)
		ALP_TMTL("STORE, STAGE 4 DEACTIVATE OLD PACKAGES");

	for (iter = removed; iter; iter = g_list_next(iter)) {
		AlpBundle p;
		p.value = (int)iter->data;
		if (debug)
			ALP_TMTL("Deactivating package %d", p.value);
		alp_prv_bundle_deactivate(p, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
	}
	
	// Step 5: Update names, generating add/change/remove notifications

	if (debug)
		ALP_TMTL("STORE, STAGE 5 UPDATE NAMES");

	// If notifications aren't held, then we shouldn't hold a transaction while we send
	// notifiactions, or the notified parties may be unable to reenter the bundle manager.
	if (gHoldNotifications) {
		alp_prv_bundle_update_names(NULL, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
	} else {
		COMMIT;

		alp_prv_bundle_update_names(NULL, 0);

		BEGIN;
	}

	// Step 6: notify about impending unregistration of removed packages
	if (debug)
		ALP_TMTL("STORE, STAGE 6 NOTIFY OF UNREGISTER");
	
	if (removed) {
		// Unregister notifications are intended to be synchronous. We
		// need to close our transaction, otherwise we'll cause a deadock
		// when a notification recipient tries to access the bundlemgr database.
		COMMIT;
		
		for (iter = removed; iter; iter = g_list_next(iter)) {
			AlpBundle p;
			p.value = (int)iter->data;
			alp_prv_bundle_unreg_notify(p, 0);
		}

		BEGIN;
	}

	// Step 7: unregister removed packages
	
	if (debug)
		ALP_TMTL("STORE, STAGE 7 REMOVE OLD");
	
	int names_again = 0;

	for (; removed; removed = g_list_delete_link(removed,removed)) {
		AlpBundle p;
		p.value = (int)removed->data;
		if (alp_prv_bundle_unregister_bundle(p, ALP_PRV_BUNDLE_FLAG_DO_NOT_NOTIFY|ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION) != ALP_STATUS_OK) {
			//oops, the unregister failed -- notify it as being registered again
			ALP_TMTL("Unregister of package %d failed, putting it back", p.value);
			// reactivate it, and notify of its presence
			alp_prv_bundle_reg_notify(p, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
			names_again = 1;
		}
	}

	// Step 8: fix up names, if problems occured
	
	if (debug)
		ALP_TMTL("STORE, STAGE 8 FIX UP NAMES");

	if (names_again) {
		// If notifications aren't held, then we shouldn't hold a transaction while we send
		// notifiactions, or the notified parties may be unable to reenter the bundle manager.
		if (gHoldNotifications) {
			alp_prv_bundle_update_names(NULL, ALP_PRV_BUNDLE_FLAG_NESTED_TRANSACTION);
		} else {
			COMMIT;

			alp_prv_bundle_update_names(NULL, 0);

			BEGIN;
		}
	}
	
	// alp_bundle_delete uses a synchronous notifications, so we close our transaction beforehand
	S_EXEC("COMMIT");
		
	// Step 9: delete any bundles marked for recycling
	
	if (debug)
		ALP_TMTL("STORE, STAGE 9 RECYCLE");
	
	for (; recycle; recycle = g_list_delete_link(recycle, recycle)) {
		AlpBundle b;
		b.value = (int)recycle->data;
		
		if (alp_bundle_delete(b, 0) != ALP_STATUS_OK) {
			ALP_TMTL("On rescan of store %s, recycled bundle %d could not be deleted", store, b.value); 
		}
	}
	
	g_free(path);

bar_skip:

	// FIXME: pull ghost scanning into primary routine, or push bar scanning out to separate function.
	// If fewer transactions are better, we might want to consider merging the phases of bar and ghost
	// scanning so they can do similar steps in parallel and avoid duplicate work.
	if ((ghost_storage != -1) && !(scheme_skip & (1<<BUNDLE_SCHEME_GHOST)))
		alp_prv_ghost_scan_store(store, store_id, ghost_storage, wipe_invalid);
		
	if (jam_store && !(scheme_skip & (1<<BUNDLE_SCHEME_JAVA)))
		alp_prv_java_scan_store(store, store_id, wipe_invalid);
		
		
		
		
		

	return ALP_STATUS_OK;
}

void handle_client_alp_prv_bundle_scan_store(AlpConnection * connection, AlpMessage * message, gpointer cbData)
{
	gchar *store;
	int wipe_invalid;
	int scheme_skip;
	
	alp_ipc_message_unpack_start(message);
	alp_ipc_message_unpack_string(message, &store);
	alp_ipc_message_unpack_int32(message, &wipe_invalid);
	alp_ipc_message_unpack_int32(message, &scheme_skip);
	alp_ipc_message_unpack_end(message);
			
	alp_status_t result;

	gboolean permission;
	
	permission = has_store_permission_by_store(connection, PERM_SCAN_POLICY_COLUMN, store);
	
	if (permission) {
		result = alp_prv_bundle_scan_store(store, wipe_invalid, scheme_skip);
	} else {
		result = ALP_STATUS_BUNDLEMGR_FAILURE;
	}
	
	AlpMessage * response = alp_ipc_message_create();
	alp_ipc_message_response_pack_start(response, MSG_PACKAGE_SCAN_STORE_RESPONSE, message);
	alp_ipc_message_pack_int32(response, result); 
	alp_ipc_message_pack_end(response);
	alp_ipc_connection_send(connection, response);
	alp_ipc_message_destroy(response);

	alp_ipc_message_destroy(message);
}
