/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
// Includes ------------------------------------------------------------------
#include <hiker/unit_test.h>
#include <cppunit/extensions/HelperMacros.h>

#include <hiker/sysclass.h>
#include <hiker/types.h>
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <sys/types.h>
#include <sys/wait.h>

       #include <sys/types.h>
       #include <sys/socket.h>


#define TEST_DB "/usr/local/alp/tests/test-bundlemgr.db"
#define TEST_LIVE_DB "/tmp/test-live-bundlemgr.db"

// Typedefs ------------------------------------------------------------------
class BundleMgrBasicTests : public CPPUNIT_NS::TestFixture
{
	CPPUNIT_TEST_SUITE(BundleMgrBasicTests);
	CPPUNIT_TEST(init_1);
	CPPUNIT_TEST(init_2);
	CPPUNIT_TEST(init_3);
	CPPUNIT_TEST(init_4);
	CPPUNIT_TEST(fini_1);
	CPPUNIT_TEST(fini_2);
	CPPUNIT_TEST_SUITE_END();

	public:
		void setUp();
		void tearDown();
		void init_1();
		void init_2();
		void init_3();
		void init_4();
		void fini_1();
		void fini_2();
		
};

ALP_ROOT_TEST_SUITE_REGISTRATION( "AP-002", CPPUNIT_REGISTRY_TYPE_AUTO );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( BundleMgrBasicTests, "AP-002" );

void BundleMgrBasicTests::setUp()
{
	chdir(getenv("ALP_RUN_TOP") ?: "/");
	chdir("usr/local/alp/tests");
}

void BundleMgrBasicTests::tearDown()
{
}

/* PM starts out uninitialized */
void BundleMgrBasicTests::init_1(void)
{
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	
}

/* We can initialize the PM server, and then shut it down */
void BundleMgrBasicTests::init_2(void)
{
	unlink(TEST_DB);
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	
	CPPUNIT_ASSERT(alp_prv_bundle_init(TRUE, TEST_DB, TEST_LIVE_DB, "test-ALP-bundlemgr", "/tmp/bundlemgr/work/", "/tmp/bundlemgr/rw/", FALSE) == 0);
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 2); // server	
	CPPUNIT_ASSERT(alp_prv_bundle_fini() == 0);
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0);
}


gboolean do_hup(GIOChannel * source, GIOCondition cond, gpointer data)
{
	g_main_quit((GMainLoop*)data);
	return FALSE;
}


/* We can initialize the PM server in a separate process, then initialize the client
   in our own process, and then tear all of this down */
void BundleMgrBasicTests::init_3(void)
{
	unlink(TEST_DB);
	pid_t pid;

	int sv[2];

	socketpair(AF_UNIX, SOCK_STREAM, 0, &sv[0]);

	if ((pid = fork()) == 0) {


		close(sv[0]);

		GMainLoop * loop = g_main_loop_new(NULL, TRUE);

		GIOChannel * socket_channel = g_io_channel_unix_new(sv[1]);

		g_io_channel_set_encoding(socket_channel, NULL, NULL);

		g_io_add_watch(socket_channel, G_IO_HUP, do_hup, loop);

		alp_prv_bundle_init(TRUE, TEST_DB, TEST_LIVE_DB, "test-ALP-bundlemgr", "/tmp/bundlemgr/work/", "/tmp/bundlemgr/rw/", FALSE);

		write(sv[1], "1", 1);

		g_main_loop_run(loop);

		alp_prv_bundle_fini();
		
		exit(0);

	} else {
		close(sv[1]);
		char buf[1];

		read(sv[0], buf, 1);

		CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	

		alp_prv_bundle_init(FALSE, TEST_DB, TEST_LIVE_DB, "test-ALP-bundlemgr", "/tmp/bundlemgr/work/", "/tmp/bundlemgr/rw/", FALSE);

		CPPUNIT_ASSERT(alp_prv_bundle_state() == 1); // client

		alp_prv_bundle_fini();


		close(sv[0]);
		
		waitpid(pid, NULL, 0);
	}
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	
}

/* We can initialize the PM server, and then shut it down */
void BundleMgrBasicTests::init_4(void)
{
	unlink(TEST_DB);
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	
	CPPUNIT_ASSERT(alp_prv_bundle_init_playground(TRUE, "test", FALSE) == 0);
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 2); // server	
	CPPUNIT_ASSERT(alp_prv_bundle_fini() == 0);
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0);
}


/* The shutdown call works */
void BundleMgrBasicTests::fini_1(void)
{
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	
	CPPUNIT_ASSERT(alp_prv_bundle_fini() == 0); // OK to try to close it twice
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	
}

/* We can initialize the PM server, and then shut it down */
void BundleMgrBasicTests::fini_2(void)
{
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	
	CPPUNIT_ASSERT(alp_prv_bundle_init(TRUE, TEST_DB, TEST_LIVE_DB, "test-ALP-bundlemgr", "/tmp/bundlemgr/work/", "/tmp/bundlemgr/rw/", FALSE) == 0);
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 2); // server	
	CPPUNIT_ASSERT(alp_prv_bundle_fini() == 0);
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0);
}


