/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
// Includes ------------------------------------------------------------------
#include <hiker/unit_test.h>
#include <cppunit/extensions/HelperMacros.h>

#include <hiker/sysclass.h>
#include <hiker/types.h>
#include <hiker/bundlemgr.h>
#include <hiker/appmgr.h>
#include <hiker/notifymgr.h>
#include <hiker/pdk/appmgr_pdk.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>

#include "../pm_refimpl.h"

       #include <sys/types.h>
       #include <sys/socket.h>


// Typedefs ------------------------------------------------------------------
class BundleMgrClientTests : public CPPUNIT_NS::TestFixture
{
	public:
		void setUp();
		void tearDown();
		void a_test();
		void wipe_test();
		void dump_test();
		void alp_bundle_by_name_test();
		void alp_bundle_name_test();
		void alp_bundle_metadata_value_test();
		void alp_bundle_metadata_value_test_2();
		void alp_bundle_metadata_test();
		void alp_bundle_property_value_test();
		void alp_bundle_open_test();
		void alp_bundle_close_test();
		void alp_bundle_ref_ro_pathname_test();
		void alp_bundle_ref_ro_channel_test();
		void alp_bundle_ref_ro_data_test();
		void alp_bundle_ref_ro_directory_test();
		void alp_bundle_rw_pathname_test();
		void alp_bundle_locate_file_test();
		void scheme_value_test();
		void scheme_values_test();
		void scheme_identify_test();
		void all_properties_test();
		void alp_bundle_search_test();
		void alp_bundle_me_test();
		void alp_bundle_ref_bundle_test();
		void alp_bundle_list_names_test();
		void alp_bundle_list_properties_test();
		void alp_bundle_list_schemes_test();
		void alp_bundle_list_stores_test();
		void alp_bundle_ref_name_test();
		void alp_bundle_store_test();
		void alp_bundle_store_value_test();
		void alp_bundle_acquire_ro_channel_test();
		void alp_bundle_acquire_ro_tempfile_test();	
		void alp_bundle_acquire_ro_data_test();	
		void alp_bundle_acquire_ro_pathname_test();
		void alp_bundle_application_test();
		void alp_bundle_ref_gettextdomain_test();	
		void alp_bundle_properties_test();
		void install_image_test();
		void install_image_test_2();
		void install_image_test_3();
		void baseline_timing_test();
		void property_value_timing_test();
		void entrypoint_timing_test();
		void alp_bundle_ref_ro_tempfile_test();
		void image_archive_test();
		void alp_bundle_copy_to_store_test();
		void alp_bundle_move_to_store_test();
		void alp_bundle_delete_cancelling_test();
                void alp_bundle_scheme_identify_test();
		void unregister_twice_test();
		void delete_running_bundle();
		void delete_open_bundle();

		// utility 
		typedef GList*(*listFunc)(void) ;
		bool list_test(listFunc,const char *);
		void bundle_copy_move_util(AlpBundle (*)(AlpBundle, const char*, int));
		
		// to count our notification types
		static unsigned int NotificationsHandled; // needs to be static for outside access from broadcast func
		static unsigned int NotificationCountRegister;
		static unsigned int NotificationCountUnregister;
		static unsigned int NotificationCountCopy;
		static unsigned int NotificationCountMove;
		static unsigned int NotificationCountDelete;
		static unsigned int NotificationCountNameAdded;
		static unsigned int NotificationCountNameRemoved;
		static unsigned int NotificationCountNameChanged;
		static unsigned int NotificationCountCancel;

		// to trigger a delete in one test situation
		static unsigned int NotificationCancelDelete;
	public:

		int sv[2];
		pid_t pid;
		char * old_LANG;
		static gboolean do_hup(GIOChannel * source, GIOCondition cond, gpointer data);
};

class BundleMgrClientTests_AP : public BundleMgrClientTests 
{ // CPPUNIT definition class only
	CPPUNIT_TEST_SUITE(BundleMgrClientTests_AP);
	CPPUNIT_TEST(a_test);
	CPPUNIT_TEST(wipe_test);
	CPPUNIT_TEST(dump_test);
	CPPUNIT_TEST(alp_bundle_by_name_test);
	CPPUNIT_TEST(alp_bundle_name_test);
	CPPUNIT_TEST(alp_bundle_metadata_value_test);
	CPPUNIT_TEST(alp_bundle_metadata_value_test_2);
	CPPUNIT_TEST(alp_bundle_metadata_test);
	CPPUNIT_TEST(alp_bundle_property_value_test);
	CPPUNIT_TEST(alp_bundle_open_test);
	CPPUNIT_TEST(alp_bundle_close_test);
	CPPUNIT_TEST(alp_bundle_ref_ro_pathname_test);
	CPPUNIT_TEST(alp_bundle_ref_ro_channel_test);
	CPPUNIT_TEST(alp_bundle_ref_ro_data_test);
	CPPUNIT_TEST(alp_bundle_ref_ro_directory_test);
	CPPUNIT_TEST(alp_bundle_rw_pathname_test);
	CPPUNIT_TEST(alp_bundle_locate_file_test);
	CPPUNIT_TEST(scheme_value_test);
	CPPUNIT_TEST(scheme_values_test);
	CPPUNIT_TEST(scheme_identify_test);
	CPPUNIT_TEST(all_properties_test);
	CPPUNIT_TEST(alp_bundle_search_test);
	CPPUNIT_TEST(alp_bundle_list_names_test);
	CPPUNIT_TEST(alp_bundle_list_properties_test);
	CPPUNIT_TEST(alp_bundle_list_schemes_test);
	CPPUNIT_TEST(alp_bundle_list_stores_test);
	CPPUNIT_TEST(alp_bundle_me_test);
	CPPUNIT_TEST(alp_bundle_ref_name_test);
	CPPUNIT_TEST(alp_bundle_ref_bundle_test);	
	CPPUNIT_TEST(alp_bundle_store_test);	
	CPPUNIT_TEST(alp_bundle_store_value_test);	
	CPPUNIT_TEST(alp_bundle_acquire_ro_channel_test);	
	CPPUNIT_TEST(alp_bundle_acquire_ro_tempfile_test);	
	CPPUNIT_TEST(alp_bundle_acquire_ro_data_test);	
	CPPUNIT_TEST(alp_bundle_acquire_ro_pathname_test);	
	CPPUNIT_TEST(alp_bundle_ref_ro_tempfile_test);	
	CPPUNIT_TEST(alp_bundle_application_test);	
	CPPUNIT_TEST(alp_bundle_ref_gettextdomain_test);	
	CPPUNIT_TEST(alp_bundle_properties_test);
	CPPUNIT_TEST(install_image_test);	
	CPPUNIT_TEST(baseline_timing_test);
	CPPUNIT_TEST(property_value_timing_test);
	CPPUNIT_TEST(entrypoint_timing_test);
	CPPUNIT_TEST(image_archive_test);	
	CPPUNIT_TEST(alp_bundle_copy_to_store_test);	
	CPPUNIT_TEST(alp_bundle_move_to_store_test);	
//	CPPUNIT_TEST(alp_bundle_delete_cancelling_test);
	CPPUNIT_TEST(alp_bundle_scheme_identify_test);	
	CPPUNIT_TEST_SUITE_END();
};
class BundleMgrClientTests_AN : public BundleMgrClientTests 
{ // CPPUNIT definition class only
	CPPUNIT_TEST_SUITE(BundleMgrClientTests_AN);
	CPPUNIT_TEST(install_image_test_2);	
	CPPUNIT_TEST(install_image_test_3);	
	CPPUNIT_TEST(unregister_twice_test);	
//	CPPUNIT_TEST(delete_running_bundle);	
	CPPUNIT_TEST(delete_open_bundle);	
	CPPUNIT_TEST_SUITE_END();
};
ALP_ROOT_TEST_SUITE_REGISTRATION( "AP-002", CPPUNIT_REGISTRY_TYPE_AUTO );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( BundleMgrClientTests_AP, "AP-002" );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( BundleMgrClientTests_AN, "AP-002" );

unsigned int BundleMgrClientTests::NotificationsHandled = 0;
unsigned int BundleMgrClientTests::NotificationCountRegister = 0;
unsigned int BundleMgrClientTests::NotificationCountUnregister = 0;
unsigned int BundleMgrClientTests::NotificationCountCopy = 0;
unsigned int BundleMgrClientTests::NotificationCountMove = 0;
unsigned int BundleMgrClientTests::NotificationCountDelete = 0;
unsigned int BundleMgrClientTests::NotificationCountNameAdded = 0;
unsigned int BundleMgrClientTests::NotificationCountNameRemoved = 0;
unsigned int BundleMgrClientTests::NotificationCountNameChanged = 0;
unsigned int BundleMgrClientTests::NotificationCountCancel = 0;
unsigned int BundleMgrClientTests::NotificationCancelDelete = 0;

gboolean BundleMgrClientTests::do_hup(GIOChannel * source, GIOCondition cond, gpointer data)
{
	g_main_quit((GMainLoop*)data);
	return FALSE;
}

/*
 * @brief Catch-all notification handler.
 */
int bundle_notification_for_any(const char *notifyType, void *details, int length, int *handled, const char *appID, void *userData)
{
	++*handled;
	BundleMgrClientTests::NotificationsHandled++;
	printf("Notification arriving: %s number %d\n", notifyType, BundleMgrClientTests::NotificationsHandled);

	if(strcmp(notifyType, ALP_NOTIFY_EVENT_BUNDLE_REGISTER) == 0)
	{
		BundleMgrClientTests::NotificationCountRegister++;
		// cast, verify, etc.
		printf("%s notification, count %d\n", notifyType, BundleMgrClientTests::NotificationCountRegister);
	}
	if(strcmp(notifyType, ALP_NOTIFY_EVENT_BUNDLE_UNREGISTER) == 0)
	{
		BundleMgrClientTests::NotificationCountUnregister++;
		// cast, verify, etc.
		printf("%s notification, count %d\n", notifyType, BundleMgrClientTests::NotificationCountUnregister);
	}
	if(strcmp(notifyType, ALP_NOTIFY_EVENT_BUNDLE_NAME_ADDED) == 0)
	{
		BundleMgrClientTests::NotificationCountNameAdded++;
		// cast, verify, etc.
		printf("%s notification, count %d\n", notifyType, BundleMgrClientTests::NotificationCountNameAdded); 
	}
	if(strcmp(notifyType, ALP_NOTIFY_EVENT_BUNDLE_NAME_CHANGED) == 0)
	{
		BundleMgrClientTests::NotificationCountNameChanged++;
		// cast, verify, etc.
		printf("%s notification, count %d\n", notifyType, BundleMgrClientTests::NotificationCountNameChanged);
	}
	if(strcmp(notifyType, ALP_NOTIFY_EVENT_BUNDLE_NAME_REMOVED) == 0)
	{
		BundleMgrClientTests::NotificationCountNameRemoved++;
		// cast, verify, etc.
		printf("%s notification, count %d\n", notifyType, BundleMgrClientTests::NotificationCountNameRemoved);
	}
	if(strcmp(notifyType, ALP_NOTIFY_EVENT_BUNDLE_COPY) == 0)
	{
		BundleMgrClientTests::NotificationCountCopy++;
		// cast, verify, etc.
		printf("%s notification, count %d\n", notifyType, BundleMgrClientTests::NotificationCountCopy);
	}
	if(strcmp(notifyType, ALP_NOTIFY_EVENT_BUNDLE_MOVE) == 0)
	{
		BundleMgrClientTests::NotificationCountMove++;
		// cast, verify, etc.
		printf("%s notification, count %d\n", notifyType, BundleMgrClientTests::NotificationCountMove);
	}
	if(strcmp(notifyType, ALP_NOTIFY_EVENT_BUNDLE_DELETE) == 0)
	{
		BundleMgrClientTests::NotificationCountDelete++;
		// see if we're trying to cancel this delete, and intervene
		printf("%s notification, count %d\n", notifyType, BundleMgrClientTests::NotificationCountDelete);
		if(BundleMgrClientTests::NotificationCancelDelete == 1)
		{
			*handled = 0x1; // should trigger a cancellation
		}
		// cast, verify, etc.
	}
	// not including reset everything test, as it can't be done from client
	return 0;
}

void BundleMgrClientTests::setUp()
{
	chdir(getenv("ALP_RUN_TOP") ?: ".");
	chdir("usr/local/alp/tests");

	old_LANG = getenv("LANG") ? strdup(getenv("LANG")) : NULL;
	
	setenv("LANG", "en_US", 1);

	socketpair(AF_UNIX, SOCK_STREAM, 0, &sv[0]);

	if ((pid = fork()) == 0) {

		close(sv[0]);

		GMainLoop * loop = g_main_loop_new(NULL, TRUE);

		GIOChannel * socket_channel = g_io_channel_unix_new(sv[1]);

		g_io_channel_set_encoding(socket_channel, NULL, NULL);

		g_io_add_watch(socket_channel, G_IO_HUP, do_hup, loop);

		alp_prv_bundle_init_playground(TRUE, "test", TRUE);

		alp_prv_bundle_hold_notifications(FALSE);

		alp_prv_bundle_wipe();
        
	        const char * store_args[] = {
	        	"bar_folder", "test-packages",
			"bar_prefer_archive", NULL,
			NULL, NULL
		};
        
		alp_prv_bundle_add_store("internal", store_args);

		alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);

		alp_prv_bundle_register_bundle("new_test_packages/package.test-entries", "bar", "internal", 0);

		write(sv[1], "1", 1);

		g_main_loop_run(loop);

		alp_prv_bundle_fini();
		
		exit(0);

	} else {
 
		close(sv[1]);
		char buf[1];

		// wait until child says server is ready
		read(sv[0], buf, 1);

		CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	

		alp_prv_bundle_init_playground(FALSE, "test", TRUE);

		CPPUNIT_ASSERT(alp_prv_bundle_state() == 1); // client

	}
}

void BundleMgrClientTests::tearDown()
{

	// shut down client
	alp_prv_bundle_fini();

	// tell child to shut down server
	close(sv[0]);
	
	// wait for child to exit	
	waitpid(pid, NULL, 0);

	if (old_LANG) {
		setenv("LANG", old_LANG, 1);
		free(old_LANG);
	}
	else
		unsetenv("LANG");
}


void BundleMgrClientTests::a_test()
{
	// nothing
}

void BundleMgrClientTests::wipe_test()
{
	// in the client, wipe shouldn't be allowed
	//CPPUNIT_ASSERT(alp_prv_bundle_wipe() == ALP_STATUS_BUNDLEMGR_FAILURE);
	//Right now, we don't have security enabled, so wipe is enabled from the client
}

void BundleMgrClientTests::dump_test(void)
{
	// FIXME: this test should verify that there is dump output... but I'm
	// not sure I have a way to capture trace output
	alp_prv_bundle_dump();
}

void BundleMgrClientTests::alp_bundle_by_name_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
}

void BundleMgrClientTests::alp_bundle_name_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	const char* pname = alp_bundle_name(p);
	CPPUNIT_ASSERT(strcmp(pname, "bar:package.test") == 0);
}

void BundleMgrClientTests::alp_bundle_metadata_value_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	char * value = alp_bundle_metadata_value(p, "scheme_name");
	CPPUNIT_ASSERT(value);
	CPPUNIT_ASSERT(strcmp(value, "bar") == 0);
	g_free(value);
}

void BundleMgrClientTests::alp_bundle_metadata_value_test_2(void)
{

	// Test that (unfortunately) a numeric SQL injection is possible
	AlpBundle p = alp_bundle_by_name("bar:package.test");

	char * value = alp_bundle_metadata_value(p, "45");
	CPPUNIT_ASSERT(value);
	CPPUNIT_ASSERT(strcmp(value, "45") == 0);
	g_free(value);

	// Test that a true SQL injection won't work

	value = alp_bundle_metadata_value(p, "COUNT(*)");
	CPPUNIT_ASSERT(value == NULL);

	value = alp_bundle_metadata_value(p, "1; SELECT 'ok'");
	CPPUNIT_ASSERT(value == NULL);
}


void BundleMgrClientTests::alp_bundle_property_value_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	char * value = alp_bundle_property_value(p, "testCase", 0, "name");
	CPPUNIT_ASSERT(value);
	CPPUNIT_ASSERT(strcmp(value, "Samuel") == 0);
	g_free(value);
	
	value = alp_bundle_property_value(p, "testCase", 1, "name");
	CPPUNIT_ASSERT(value == NULL);
}

void BundleMgrClientTests::alp_bundle_open_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);
	alp_bundle_close(r);
	CPPUNIT_ASSERT(alp_bundle_open(ALP_BUNDLE_NULL) == NULL);
}

void BundleMgrClientTests::alp_bundle_close_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);
	CPPUNIT_ASSERT(alp_bundle_close(r) == ALP_STATUS_OK);
	//CPPUNIT_ASSERT(alp_bundle_close(r) == ALP_STATUS_BUNDLEMGR_FAILURE); // no, can't test this, it's an invalid pointer
	CPPUNIT_ASSERT(alp_bundle_close(NULL) == ALP_STATUS_BUNDLEMGR_FAILURE);
}

void BundleMgrClientTests::alp_bundle_ref_ro_pathname_test(void)
{

	AlpBundle p = alp_bundle_by_name("bar:package.test");
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);

	char * root = alp_bundle_ref_ro_pathname(r, "/"); // get root of package
	char * pathname = alp_bundle_ref_ro_pathname(r, "resource.txt"); // get localized resource path

	CPPUNIT_ASSERT(root != NULL);
	CPPUNIT_ASSERT(pathname != NULL);

	CPPUNIT_ASSERT(strlen(pathname) > strlen(root));
	
	// pathname is under root
	CPPUNIT_ASSERT(strncmp(root, pathname, strlen(root)) == 0);
	
	// and it is specificaly in this location
	CPPUNIT_ASSERT(strcmp(pathname + strlen(root), "rsc/en_US/resource.txt") == 0);	
	
	g_free(root);
	g_free(pathname);

}

void BundleMgrClientTests::alp_bundle_ref_ro_channel_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);

	GIOChannel * channel = alp_bundle_ref_ro_channel(r, "resource.txt");
	
	CPPUNIT_ASSERT(channel);
	g_io_channel_unref(channel);
	
}

void BundleMgrClientTests::alp_bundle_ref_ro_data_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);

	gchar * data;
	gsize length;
	
	CPPUNIT_ASSERT(alp_bundle_ref_ro_data(r, "resource.txt", &data, &length) == ALP_STATUS_OK);
	CPPUNIT_ASSERT(length == 25);
	CPPUNIT_ASSERT(strlen(data) == 25);
	CPPUNIT_ASSERT(strcmp(data, "This is a resource file.\n") == 0);
	
	g_free(data);
	
}

void BundleMgrClientTests::alp_bundle_ref_ro_directory_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);

	GList * dir = alp_bundle_ref_ro_directory(r, "/", TRUE);
	
	CPPUNIT_ASSERT(dir);
}

void BundleMgrClientTests::alp_bundle_rw_pathname_test()
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");

	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	gchar * path = alp_bundle_rw_pathname(p, "", FALSE);
	
	CPPUNIT_ASSERT(path == NULL);
	
	path = alp_bundle_rw_pathname(p, "", TRUE);
	
	CPPUNIT_ASSERT(path);

	g_free(path);

	path = alp_bundle_rw_pathname(p, "", FALSE);
	
	CPPUNIT_ASSERT(path);
	
	char buf[1024];
	sprintf(buf, "%s%s", path, "somefile.txt");

	FILE * f = fopen(buf, "w");
	CPPUNIT_ASSERT(f);
	
	fputs("This is a test", f);
	fclose(f);
	
	g_free(path);

	path = alp_bundle_rw_pathname(p, "a/", TRUE);
	CPPUNIT_ASSERT(path);
	g_free(path);
	
	path = alp_bundle_rw_pathname(p, "a/b/c/foo", FALSE);
	CPPUNIT_ASSERT(path == NULL);

	path = alp_bundle_rw_pathname(p, "a/b/c/foo", TRUE);
	CPPUNIT_ASSERT(path);
	g_free(path);	

	path = alp_bundle_rw_pathname(p, "a/b/c/foo", FALSE);
	CPPUNIT_ASSERT(path);
	g_free(path);	
	
	path = alp_bundle_rw_pathname(p, "a/other/somedir", TRUE);
	g_free(path);

	CPPUNIT_ASSERT(alp_bundle_rw_delete(p, "", TRUE) == ALP_STATUS_OK);
}

void BundleMgrClientTests::alp_bundle_locate_file_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	AlpBundleRef r = alp_bundle_open(p);
	
	CPPUNIT_ASSERT(r);
	
	gchar * testpath1 = alp_bundle_ref_ro_pathname(r, "resource.txt");

	CPPUNIT_ASSERT(testpath1);
	
	AlpBundle p2 = alp_bundle_locate_file(testpath1);

	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p2));
	CPPUNIT_ASSERT(p2.value == p.value);
	
	char testpath2[PATH_MAX];
	
	CPPUNIT_ASSERT(realpath(testpath1, testpath2) != NULL);
	
	AlpBundle p3 = alp_bundle_locate_file(testpath1);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p3));
	CPPUNIT_ASSERT(p3.value == p.value);

	// Try a path which can't directly go through realpath()
	// because it doesn't completely exist -- it should still
	// be able to work out that this belongs to the bundle
	// Currently it fails due to the use of realpath() inside
	// the locate_file routine
	char testpath3[PATH_MAX];
	
	strcpy(testpath3, testpath2);
	strcat(testpath3, "/somedir/somefile/all-of-this-doesn't-exit");
	
	AlpBundle p4 = alp_bundle_locate_file(testpath3);
	CPPUNIT_ASSERT(alp_bundle_is_NULL(p4));
	
	alp_bundle_close(r);

	//symlink will have gone when package was closed
	p2 = alp_bundle_locate_file(testpath1);
	CPPUNIT_ASSERT(alp_bundle_is_NULL(p2));
	
	//but underlying resolved path should still work
	p3 = alp_bundle_locate_file(testpath2);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p3));
	CPPUNIT_ASSERT(p3.value == p.value);

	g_free(testpath1);
}

void BundleMgrClientTests::scheme_value_test()
{
	gchar * str = alp_bundle_scheme_value("bar", "extension");
	
	CPPUNIT_ASSERT(str != NULL);
	CPPUNIT_ASSERT(strcmp(str, ".bar") == 0);
	
	g_free(str);
	
	str = alp_bundle_scheme_value("-nonexistant-", "extension");
	
	CPPUNIT_ASSERT(str == NULL);
}
void printEm(gpointer key, gpointer value, gpointer skip)
{
	char *processed;
	if(value == NULL)
	{
		processed = "NULL";
	}
	else
		processed = (char*)value;
	printf("\t\"%s\", \"%s\",\n", (char*)key, processed);
}
bool hash_match(GHashTable * h, const char ** match)
{
	int i, count, matched;
	
	for (i=0; match[i*2] != NULL; i++)
		;
	count = i;
	g_hash_table_foreach(h, &printEm, NULL);
	matched = 0;
	
	for (i=0; i<count; i++) {
		gpointer p = g_hash_table_lookup(h, match[i*2]);
		if (p) {
			//printf("\n%s <- compared to -> %s\n",static_cast<char*>(p), match[i*2+1]);
			if (strcmp((char*)p, match[i*2+1]) == 0)
			{
				matched++;
				continue;
			}
			else 
			{
				//printf("Unmatched situation. %s not equal to %s\n", (char*)p, match[i*2+1]);
				continue;

			}
		}
		else if(p==NULL && match[i*2+1] == NULL) 
		{
			//printf("NULL matched\n");
			matched++;
			continue;
		}
		//printf("No value. %s not equal to %s\n", (char*)p, match[i*2+1]);
	}

	if (matched == count && (int)g_hash_table_size(h) == count)
		return true; // success

	return false; // failure
}

void BundleMgrClientTests::scheme_values_test()
{
	GHashTable * results = alp_bundle_scheme_values("bar");

	CPPUNIT_ASSERT(results != NULL);

	const char *expected[] = {
		"extension", ".bar",
		"mimetype", "x-application/access-bar",
		NULL, NULL,
	};

	CPPUNIT_ASSERT(hash_match(results, expected));

	g_hash_table_destroy(results);

	results = alp_bundle_scheme_values("-nonexistant-");

	CPPUNIT_ASSERT(results == NULL);
}

void BundleMgrClientTests::scheme_identify_test()
{
	gchar * scheme = alp_bundle_scheme_identify("something.bar", NULL);

	CPPUNIT_ASSERT(scheme);

	CPPUNIT_ASSERT(strcmp(scheme, "bar") == 0);

	g_free(scheme);

	scheme = alp_bundle_scheme_identify(NULL, NULL);

	CPPUNIT_ASSERT(scheme == NULL);
}

void BundleMgrClientTests::all_properties_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-d", "bar", "internal", 0);

	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));

	GList * results = alp_bundle_all_properties(p);

	CPPUNIT_ASSERT(results != NULL);
	CPPUNIT_ASSERT(g_list_length(results) == 3);

	const char *app_props[] = {
		"", "application",
		"icon", "Missing-icon",
		"name", "",
		"fred", "test",
		NULL, NULL,
	};

	const char *other1_props[] = {
		"", "other",
		"somekey", "",
		NULL, NULL,
	};

	const char *other2_props[] = {
		"", "other",
		"key1","value1",
		NULL, NULL,
	};

	CPPUNIT_ASSERT(hash_match((GHashTable*)g_list_nth_data(results, 0), app_props));
	CPPUNIT_ASSERT(hash_match((GHashTable*)g_list_nth_data(results, 1), other1_props));
	CPPUNIT_ASSERT(hash_match((GHashTable*)g_list_nth_data(results, 2), other2_props));

	g_list_foreach(results, (GFunc)g_hash_table_destroy, NULL);

	results = alp_bundle_all_properties(ALP_BUNDLE_NULL);

	CPPUNIT_ASSERT(results == NULL);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}
void BundleMgrClientTests::alp_bundle_search_test()
{
	GArray *grr = alp_bundle_search(NULL,TRUE,NULL,NULL,"application");
	CPPUNIT_ASSERT(grr->len == 1); 
	if(grr) g_array_free(grr,true);

	grr = alp_bundle_search(NULL,FALSE,NULL,NULL,"application");
	CPPUNIT_ASSERT(grr->len == 1);
	if(grr) g_array_free(grr,true);

	grr = alp_bundle_search("bar:package.test",TRUE,NULL,NULL,NULL);
	CPPUNIT_ASSERT(grr->len == 1);

	AlpBundle p = g_array_index(grr, AlpBundle, 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	char * value = alp_bundle_metadata_value(p, "scheme_name");
	CPPUNIT_ASSERT(value);
	CPPUNIT_ASSERT(strcmp(value, "bar") == 0);
	if(grr) g_array_free(grr, true);
	g_free(value);
}

bool BundleMgrClientTests::list_test(listFunc which, const char *compareMe)
{
	GList *gl = which();
	CPPUNIT_ASSERT(g_list_length(gl) > 0);

	// prove something...
	// be aware a success in any single comparer run passes the whole test
	void *p = NULL;
	bool success = false;
	int len = g_list_length(gl);
	CPPUNIT_ASSERT(len > 0);
	for(int i = 0; i < len; i++)
	{
		p = g_list_nth_data(gl, i);
		CPPUNIT_ASSERT(p);
		if(strstr(static_cast<gchar*>(p),compareMe) != 0) success = true;
		printf("%s\n",static_cast<gchar*>(p));
	}
	CPPUNIT_ASSERT(success);
	if(gl)
	{
		g_list_foreach(gl, (GFunc)g_free, NULL); 
		g_list_free(gl); 
	}
	return success;
}

void BundleMgrClientTests::alp_bundle_list_properties_test(void)
{
	CPPUNIT_ASSERT(list_test(&alp_bundle_list_properties, "testCase"));
}

void BundleMgrClientTests::alp_bundle_list_names_test(void)
{
	CPPUNIT_ASSERT(list_test(alp_bundle_list_names, "bar:package.test"));
}

void BundleMgrClientTests::alp_bundle_list_schemes_test(void)
{
	CPPUNIT_ASSERT(list_test(alp_bundle_list_schemes, "bar"));
}

void BundleMgrClientTests::alp_bundle_list_stores_test(void)
{
	CPPUNIT_ASSERT(list_test(alp_bundle_list_stores, "internal"));
}

void BundleMgrClientTests::alp_bundle_me_test(void)
{
	// yes, I know these are failing because we can't really do this without a native bundle. We'll have to find another way.
	AlpBundle myself = alp_bundle_me();
	CPPUNIT_ASSERT(alp_bundle_is_NULL(myself));
	AlpBundleRef myrefself = alp_bundle_ref_me();
	CPPUNIT_ASSERT(myrefself == NULL);
}

void BundleMgrClientTests::alp_bundle_ref_name_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);

	const char* rname = alp_bundle_ref_name(r);
	CPPUNIT_ASSERT(strcmp(rname, "bar:package.test") == 0);
}

void BundleMgrClientTests::alp_bundle_ref_bundle_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);
	AlpBundle p2 = alp_bundle_ref_bundle(r);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
}

void BundleMgrClientTests::alp_bundle_metadata_test(void)
{
	const char *expected[] = {
		"store_name", "internal",
		"resolved_real_folder", NULL,
		"gettextdomain", NULL,
		"recycle", NULL,
		"bar_version", NULL,
		"bar_filename", "new_test_packages/package.test",
		"bar_textdomain_name", "bar-^n",
		"name", "bar:package.test",
		"open", NULL,
		"refcount", NULL,
		"real_folder", NULL,
		"bar_textdomain_dir", "rsc",
		"scheme_name", "bar",
		"security_handle", NULL,
		NULL, NULL
	};
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	GHashTable* results = alp_bundle_metadata(p);
	CPPUNIT_ASSERT(results);
	CPPUNIT_ASSERT(hash_match(results, expected));
	g_hash_table_destroy(results);
}

void BundleMgrClientTests::alp_bundle_store_test(void)
{
	const char *expected[] = {
		"store_id", "1",
		"store_name", "internal",
		"bar_prefer_archive", NULL,
		"register_policy", NULL,
		"unregister_policy", NULL,
		"delete_policy", NULL,
		"install_policy", NULL,
		"scan_policy", NULL,
		"install_priority", "0",
		"wipe_by_default", NULL,
		"bar_folder", "test-packages",
		"ghost_storage", NULL,
		"ghost_path", NULL,
		"vs_udi", NULL,
		"is_rom", NULL,
		"jam_store", NULL,
		NULL, NULL
	};
	GHashTable* results = alp_bundle_store("internal");
	CPPUNIT_ASSERT(results);
	//g_hash_table_foreach(results, &printEm, NULL);
	CPPUNIT_ASSERT(hash_match(results, expected));
	g_hash_table_destroy(results);
}

void BundleMgrClientTests::alp_bundle_store_value_test(void)
{
	const char *expected = "test-packages";
	gchar* results = alp_bundle_store_value("internal", "bar_folder");
	CPPUNIT_ASSERT(results);
	CPPUNIT_ASSERT(strcmp(results, expected)==0);
}

/*! Try to get a valid response out of acquire_ro_channel
 *
 * @return void
 *
 * @remarks Not convinced this is working yet.
 *
 * 	@since ALP iSDK 1.0
 *
*/

void BundleMgrClientTests::alp_bundle_acquire_ro_channel_test()
{	
	gchar buffer[128];
	gsize didread = 0;
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundleRef myRef = alp_bundle_open(p);
	AlpBundleRef outRef;
	GIOChannel * channel = alp_bundle_acquire_ro_channel("resource.txt", myRef);
	// prove we can read from it
	g_io_channel_read_chars(channel, buffer, 128, &didread, NULL);
	CPPUNIT_ASSERT(didread == 25);
	CPPUNIT_ASSERT(memcmp(buffer,"This is a resource file.\n", 25) == 0);
	alp_bundle_close(myRef);

	g_io_channel_unref(channel);
}

/*! Try to get a valid response out of acquire_ro_tempfile
 *
 * @return void
 *
 * @remarks Not convinced this is working yet.
 *
 * 	@since ALP iSDK 1.0
 *
*/

void BundleMgrClientTests::alp_bundle_acquire_ro_tempfile_test()
{
	char buffer[128];
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundleRef myRef = alp_bundle_open(p);
	AlpBundleRef outRef;
	gchar *pathname = alp_bundle_acquire_ro_tempfile("resource.txt", &outRef, FALSE, NULL, myRef);
	CPPUNIT_ASSERT(outRef);
	CPPUNIT_ASSERT(pathname);
//	printf("%s\n", pathname);
	FILE * f = fopen(pathname, "r");
	// process f
	fgets(buffer, 128, f);
//	printf("%s\n", buffer);
	CPPUNIT_ASSERT(strcmp(buffer,"This is a resource file.\n") == 0);

	alp_bundle_close(myRef);
	fclose(f);
	alp_bundle_close(outRef);
}

/*! Try to get a valid response out of acquire_ro_data. 
 *
 * @return void
 *
 * @remarks Not convinced this is working yet.
 *
 * 	@since ALP iSDK 1.0
 *
*/

void BundleMgrClientTests::alp_bundle_acquire_ro_data_test()
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundleRef myRef = alp_bundle_open(p);
	AlpBundleRef outRef;

	gchar * buffer;
	int refcount1 = myRef->refcount;
	alp_status_t result = alp_bundle_acquire_ro_data("resource.txt", &buffer, NULL, myRef);
	int refcount2 = myRef->refcount;

	alp_bundle_close(myRef);

	//printf("ref=%p, refcount1=%d, refcount=%d\n", myRef, refcount1, refcount2);
	
	CPPUNIT_ASSERT( refcount1 == refcount2);
        
	// process data
	CPPUNIT_ASSERT(strcmp(buffer,"This is a resource file.\n") == 0);

	g_free(buffer);
}

void BundleMgrClientTests::alp_bundle_acquire_ro_pathname_test()
{
	char buffer[128];
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundleRef myRef = alp_bundle_open(p);
	AlpBundleRef outRef;
	gchar * pathname = alp_bundle_acquire_ro_pathname("resource.txt", &outRef, myRef);
	FILE * f = fopen(pathname, "r");
	// process f
//	printf("%s\n", pathname);
	fgets(buffer, 128, f);
//	printf("%s\n", buffer);
	CPPUNIT_ASSERT(strcmp(buffer,"This is a resource file.\n") == 0);

	fclose(f);
	alp_bundle_close(myRef);
	alp_bundle_close(outRef);
}

void BundleMgrClientTests::install_image_test(void)
{
	GIOChannel * channel = g_io_channel_new_file("new_test_packages/images/invalid_image.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundle package;
	
	package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
//	printf("package.value = %d\n", package.value);
	
	CPPUNIT_ASSERT(alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	// valid installation
	channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(! alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	// valid installation to secondary store
	const char * store_args[] = {
		"bar_folder", "/tmp/secondary_store",
		"bar_prefer_archive", NULL,
		NULL, NULL
	};
	
	system("rm -rf /tmp/secondary_store");

	mkdir("/tmp/secondary_store", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
	alp_status_t status = alp_prv_bundle_add_store("secondary", store_args);
	CPPUNIT_ASSERT(status == 0);

	CPPUNIT_ASSERT(list_test(alp_bundle_list_stores, "secondary"));

	channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, "secondary"/*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(! alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	// see that it arrived safely 
	GArray * arr = alp_bundle_search(NULL, false, NULL, "secondary", NULL);
	if(arr) 
	{
		//printf("%d found\n", arr->len);
		CPPUNIT_ASSERT(arr->len == 1); 
		g_array_free(arr,true);
	}

	// get rid of secondary store
	status = alp_prv_bundle_delete_store("secondary");
	CPPUNIT_ASSERT(status == 0);
}

/*! install an empty image
 *
 * @return void
 *
 * @remarks 
 *
 * 	@since ALP iSDK 1.0
 *
*/

void BundleMgrClientTests::install_image_test_2(void)
{
	GIOChannel * channel = g_io_channel_new_file("new_test_packages/images/invalid_image.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundle package;
	
	package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, "internal" /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	printf("package.value = %d\n", package.value);
	
	CPPUNIT_ASSERT(alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, "internal" /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(! alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	CPPUNIT_ASSERT(alp_bundle_delete(package, FALSE) == ALP_STATUS_OK);	
}

/*! install a truncated image
 *
 * @return void
 *
 * @remarks 
 *
 * 	@since ALP iSDK 1.0
 *
*/

void BundleMgrClientTests::install_image_test_3(void)
{
	system("cp new_test_packages/images/test_launcher.bar /tmp/truncated_bundle.bar");
	truncate("/tmp/truncated_bundle.bar",2048); // clip off the end of the previously valid image

	GIOChannel * channel = g_io_channel_new_file("/tmp/truncated_bundle.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundle package;
	
	package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, "internal" /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
//	printf("package.value = %d\n", package.value);
	
	CPPUNIT_ASSERT(alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	return;
}

void BundleMgrClientTests::baseline_timing_test()
{
	int i;
	for (i=0;i<1000;i++) {
		;
	}
}

void BundleMgrClientTests::property_value_timing_test()
{
	int i;
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	for (i=0;i<1000;i++) {
		char * value = alp_bundle_property_value(p, "testCase", 0, "name");
		g_free(value);
	}
}

void BundleMgrClientTests::entrypoint_timing_test()
{
	int i;
	for (i=0;i<100;i++) {
		GClosure * entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/libalp_entry_lib.so", NULL);
	
		CPPUNIT_ASSERT(entry != NULL);
	
		g_closure_unref(entry);
	}
}

/*! Try to get a valid response out of ref_ro_tempfile
 *
 * @return void
 *
 * @remarks Not convinced this is working yet.
 *
 * 	@since ALP iSDK 1.0
 *
*/

void BundleMgrClientTests::alp_bundle_ref_ro_tempfile_test()
{
	char buffer[128];
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundleRef myRef = alp_bundle_open(p);
	gchar *pathname = alp_bundle_ref_ro_tempfile(myRef, "resource.txt", FALSE, NULL);
	CPPUNIT_ASSERT(pathname);
//	printf("%s\n", pathname);
	FILE * f = fopen(pathname, "r");
	// process f
	fgets(buffer, 128, f);
//	printf("%s\n", buffer);
	CPPUNIT_ASSERT(strcmp(buffer,"This is a resource file.\n") == 0);

	alp_bundle_close(myRef);
	fclose(f);
	alp_bundle_tempfile_release(pathname);
}

void BundleMgrClientTests::alp_bundle_application_test()
{
	AlpAppContext *app;
	char *appName;
	int count;
	alp_status_t err;
	GMainLoop *loo = g_main_loop_new(NULL, TRUE);
	appName = "com.access.testapps.BundleAppTest";
	app = alp_app_setup(appName, ALP_APP_LAUNCH_FLAG_PRIMARY, NULL);
	alp_app_set_default_context(app);
	alp_app_flush(app);
	AlpBundle me = alp_bundle_application();
	CPPUNIT_ASSERT(alp_bundle_is_NULL(me));
	AlpBundleRef meToo = alp_bundle_ref_application();
	CPPUNIT_ASSERT(meToo == NULL);
	alp_app_destroy(app,0);
	return;
}

void BundleMgrClientTests::alp_bundle_ref_gettextdomain_test()
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundleRef myRef = alp_bundle_open(p);
	CPPUNIT_ASSERT(myRef != NULL);
	const char* domain = alp_bundle_ref_gettextdomain(myRef);
//	printf("%s\n",domain);
	CPPUNIT_ASSERT(strcmp("bar-package.test",domain) == 0);
	alp_bundle_close(myRef);
}
/*
 * @brief utility function borrowed from notification tests
 */
void delay(int div)
{
	struct timespec time;

	time.tv_sec = 0;
	time.tv_nsec = 1000000000 / div;
	nanosleep(&time,NULL);
}

/*
 * @brief Test alp_bundle_properties and registration notification
 */
void BundleMgrClientTests::alp_bundle_properties_test()
{
	// piggybacking a register/unregister notification test here
	BundleMgrClientTests::NotificationsHandled = 0;
	BundleMgrClientTests::NotificationCountCopy = 0; 
	BundleMgrClientTests::NotificationCountMove = 0; 
	BundleMgrClientTests::NotificationCountDelete = 0; 
	BundleMgrClientTests::NotificationCountNameAdded = 0;
	BundleMgrClientTests::NotificationCountNameRemoved = 0;
	BundleMgrClientTests::NotificationCountNameChanged = 0;
	BundleMgrClientTests::NotificationCountRegister = 0;
	BundleMgrClientTests::NotificationCountUnregister = 0;

	alp_status_t err = alp_notify_register_server(ALP_NOTIFY_EVENT_BUNDLE_REGISTER, bundle_notification_for_any, NULL, 1, 0);
	CPPUNIT_ASSERT(err == ALP_OK);
	err = alp_notify_register_server(ALP_NOTIFY_EVENT_BUNDLE_UNREGISTER, bundle_notification_for_any, NULL, 1, 0);
	CPPUNIT_ASSERT(err == ALP_OK);
	delay(4);

	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-d", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));

	GHashTable * results = alp_bundle_properties(p,"application",0);

	CPPUNIT_ASSERT(results != NULL);

	const char *app_props[] = {
		"fred", "test",
		"name", "",
		"icon", "Missing-icon",
		NULL, NULL,
	};

	CPPUNIT_ASSERT(hash_match(results, app_props));

	g_hash_table_destroy(results);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);

	// Check our notifications, with a reset step

	delay(4);
	// unregister notifications
	do {
		delay(4);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_REGISTER, 0);
		CPPUNIT_ASSERT(err == ALP_OK || err == ALP_NOTIFY_ERR_ACTIVE_BROADCAST);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_UNREGISTER, 0);
		CPPUNIT_ASSERT(err == ALP_OK || err == ALP_NOTIFY_ERR_ACTIVE_BROADCAST);
	} while (err == ALP_NOTIFY_ERR_ACTIVE_BROADCAST);
	
	delay(3);
	CPPUNIT_ASSERT(BundleMgrClientTests::NotificationsHandled >= 2);
	CPPUNIT_ASSERT(BundleMgrClientTests::NotificationCountRegister == 1);
	CPPUNIT_ASSERT(BundleMgrClientTests::NotificationCountUnregister == 1);
}

/* do the reverse of image_install test - alp_bundle_image_archive */
void BundleMgrClientTests::image_archive_test(void)
{
	GIOChannel * channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundle package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	CPPUNIT_ASSERT(! alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);

	// reverse the process
	channel = g_io_channel_new_file("bar:com.access.test.guy_walks_into_a.bar", "w", NULL); // you had to see it coming
	CPPUNIT_ASSERT(channel);
	if(!channel) return;

	g_io_channel_set_encoding(channel, NULL, NULL);

	alp_status_t status = alp_bundle_image_archive(package,NULL,NULL,channel,NULL);
	CPPUNIT_ASSERT(status == 0);
	if(status != 0) return;

	// TODO: binary compare the two
	// system(comp) later
	
	// remove the extra launcher
	CPPUNIT_ASSERT(alp_bundle_delete(package, FALSE) == ALP_STATUS_OK);	

}

/*
 * @brief Move or copy a bundle
 * @description Used to copy a bundle to a neutral location and recopy or move to a separate store
 * Also piggybacking notifications for appropriate events
 */
void BundleMgrClientTests::bundle_copy_move_util(AlpBundle(*which)(AlpBundle, const char*, int))
{
	// find a store 
	const char * store_args[] = {
		"bar_folder", "/tmp/secondary_store",
		"bar_prefer_archive", NULL,
		NULL, NULL
	};
	
	system("rm -rf /tmp/secondary_store");
	system("rm -rf /tmp/tertiary_store");

	mkdir("/tmp/secondary_store", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
	alp_status_t status = alp_prv_bundle_add_store("secondary", store_args);
	CPPUNIT_ASSERT(status == 0);

	mkdir("/tmp/tertiary_store", S_IRWXU | S_IRWXG | S_IROTH | S_IXOTH);
	store_args[1] = "/tmp/tertiary_store";
	status = alp_prv_bundle_add_store("tertiary", store_args);
	CPPUNIT_ASSERT(status == 0);
	
	//printf("\nadd status = %x\n", status);
	CPPUNIT_ASSERT(list_test(alp_bundle_list_stores, "secondary"));
	CPPUNIT_ASSERT(list_test(alp_bundle_list_stores, "tertiary"));

	// piggybacking a register/unregister notification test here
	BundleMgrClientTests::NotificationsHandled = 0;
	BundleMgrClientTests::NotificationCountCopy = 0; 
	BundleMgrClientTests::NotificationCountMove = 0; 
	BundleMgrClientTests::NotificationCountDelete = 0; 
	BundleMgrClientTests::NotificationCountNameAdded = 0;
	BundleMgrClientTests::NotificationCountNameRemoved = 0;
	BundleMgrClientTests::NotificationCountNameChanged = 0;
	BundleMgrClientTests::NotificationCountRegister = 0;
	BundleMgrClientTests::NotificationCountUnregister = 0;

	alp_status_t err = alp_notify_register_server(ALP_NOTIFY_EVENT_BUNDLE_COPY, bundle_notification_for_any, NULL, 0, 0);
	CPPUNIT_ASSERT(err == ALP_OK);
	err = alp_notify_register_server(ALP_NOTIFY_EVENT_BUNDLE_MOVE, bundle_notification_for_any, NULL, 0, 0);
	CPPUNIT_ASSERT(err == ALP_OK);
	err = alp_notify_register_server(ALP_NOTIFY_EVENT_BUNDLE_NAME_ADDED, bundle_notification_for_any, NULL, 0, 0);
	CPPUNIT_ASSERT(err == ALP_OK);
	err = alp_notify_register_server(ALP_NOTIFY_EVENT_BUNDLE_DELETE, bundle_notification_for_any, NULL, 0, 0);
	CPPUNIT_ASSERT(err == ALP_OK);
	err = alp_notify_register_server(ALP_NOTIFY_EVENT_BUNDLE_NAME_CHANGED, bundle_notification_for_any, NULL, 0, 0);
	CPPUNIT_ASSERT(err == ALP_OK);
	err = alp_notify_register_server(ALP_NOTIFY_EVENT_BUNDLE_NAME_REMOVED, bundle_notification_for_any, NULL, 0, 0);
	CPPUNIT_ASSERT(err == ALP_OK);
	delay(4);

	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	// start with a throwaway copy
	AlpBundle q = alp_bundle_copy_to_store(p, "secondary", ALP_BUNDLE_FLAG_OVERWRITE_DUPLICATE);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(q));

	// pick our new source
	GArray * arr = alp_bundle_search(NULL, false, NULL, "secondary", NULL);
	p = g_array_index(arr, AlpBundle, 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));

	// try our operation
	q = which(p, "tertiary", ALP_BUNDLE_FLAG_OVERWRITE_DUPLICATE);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(q));

	if(arr) g_array_free(arr,true);

	// see that it arrived safely 
	arr = alp_bundle_search(NULL, false, NULL, "tertiary", NULL);
	//printf("%d found\n", arr->len);
	CPPUNIT_ASSERT(arr->len == 1); 

	// make sure it's on tertiary store
	AlpBundle r = g_array_index(arr, AlpBundle, 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(r));
	char * value = alp_bundle_metadata_value(r, "store_name");
	CPPUNIT_ASSERT(value);
	CPPUNIT_ASSERT(strcmp(value, "tertiary") == 0);
	
	if(arr) g_array_free(arr,true);
	g_free(value);
	delay(4);

	// delete spare stores - this triggers delete notifications ?
	status = alp_prv_bundle_delete_store("secondary");
	CPPUNIT_ASSERT(status == 0);
	status = alp_prv_bundle_delete_store("tertiary");
	CPPUNIT_ASSERT(status == 0);

	// unregister notifications
	do {
		delay(4);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_REGISTER, 0);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_UNREGISTER, 0);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_NAME_ADDED, 0);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_NAME_CHANGED, 0);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_NAME_REMOVED, 0);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_COPY, 0);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_MOVE, 0);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_DELETE, 0);
	} while (err == ALP_NOTIFY_ERR_ACTIVE_BROADCAST);
	
	delay(4);
	// now verify our counts of notifications that were due
	if(which == &alp_bundle_copy_to_store)
	{
		printf("\n\nCOPY: %d\n",BundleMgrClientTests::NotificationsHandled);
//		CPPUNIT_ASSERT(BundleMgrClientTests::NotificationsHandled == 2); 
		CPPUNIT_ASSERT(BundleMgrClientTests::NotificationCountCopy == 2); // once to temp, once to new home
	} 
	else
	{
		printf("\n\nMOVE: %d\n",BundleMgrClientTests::NotificationsHandled);
//		CPPUNIT_ASSERT(BundleMgrClientTests::NotificationsHandled == 3); 
		CPPUNIT_ASSERT(BundleMgrClientTests::NotificationCountCopy == 1); // once to temp
//		CPPUNIT_ASSERT(BundleMgrClientTests::NotificationCountMove == 1); // once to new home
		CPPUNIT_ASSERT(BundleMgrClientTests::NotificationCountDelete == 2);  // from the copy
	}
}	
	
void BundleMgrClientTests::alp_bundle_copy_to_store_test(void)
{
	this->bundle_copy_move_util(alp_bundle_copy_to_store);
}

void BundleMgrClientTests::alp_bundle_move_to_store_test(void)
{
	this->bundle_copy_move_util(alp_bundle_move_to_store);
}

void BundleMgrClientTests::alp_bundle_delete_cancelling_test(void)
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	// register notification for delete
	BundleMgrClientTests::NotificationsHandled = 0;
	alp_status_t err = alp_notify_register_server(ALP_NOTIFY_EVENT_BUNDLE_DELETE, bundle_notification_for_any, NULL, 0, 0);
	CPPUNIT_ASSERT(err == ALP_OK);
	delay(4);
	
	// set the cancel notification var
	BundleMgrClientTests::NotificationCancelDelete = 1;
	
	// delete and don't succeed
	CPPUNIT_ASSERT(alp_bundle_delete(p, FALSE) == ALP_STATUS_BUNDLEMGR_FAILURE);	
	delay(4);
	
	// unset the var
	BundleMgrClientTests::NotificationCancelDelete = 0;
	
	// delete and succeed
	CPPUNIT_ASSERT(alp_bundle_delete(p, FALSE) == ALP_STATUS_OK);	
	
	// unregister notifications
	do {
		delay(4);
		err = alp_notify_unregister_server(ALP_NOTIFY_EVENT_BUNDLE_DELETE, 0);
		CPPUNIT_ASSERT(err == ALP_OK || err == ALP_NOTIFY_ERR_ACTIVE_BROADCAST);
	} while (err == ALP_NOTIFY_ERR_ACTIVE_BROADCAST);
}

/*
 * @brief Generate a failure when deleting a bundle more than once
 * @description 
 * 
 */
void BundleMgrClientTests::unregister_twice_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-d", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_BUNDLEMGR_FAILURE);
}
void BundleMgrClientTestsWillNotTest()
{
}

void BundleMgrClientTests::alp_bundle_scheme_identify_test(void)
{
        gchar * result;
        
        result = alp_bundle_scheme_identify(NULL, NULL);

        CPPUNIT_ASSERT(result == NULL);

        result = alp_bundle_scheme_identify("foo.bar", NULL);
        CPPUNIT_ASSERT(result != NULL && strcmp(result, "bar") == 0);

        result = alp_bundle_scheme_identify("foo.jar", NULL);
        CPPUNIT_ASSERT(result != NULL && strcmp(result, "java") == 0);

        result = alp_bundle_scheme_identify("foo.prc", NULL);
        CPPUNIT_ASSERT(result != NULL && strcmp(result, "GHost") == 0);

        result = alp_bundle_scheme_identify(NULL, "application/java-archive");
        CPPUNIT_ASSERT(result != NULL && strcmp(result, "java") == 0);
       
        result = alp_bundle_scheme_identify(NULL, "application/java-archive.variant");
        CPPUNIT_ASSERT(result != NULL && strcmp(result, "java") == 0);
       
        result = alp_bundle_scheme_identify(NULL, "application/java-archive."); // mangled variant
        CPPUNIT_ASSERT(result != NULL && strcmp(result, "java") == 0);

        result = alp_bundle_scheme_identify(NULL, "application/pdp-11"); 
        CPPUNIT_ASSERT(result == NULL);
}

/*
 * @brief Attempt to delete a running app
 * @description 
 * 
 */
void BundleMgrClientTests::delete_running_bundle()
{
	// 
	AlpAppContext *ctxt; 
	int status = 0;
	alp_status_t err ;
	AlpBundle p; 
	// need an app that runs and stays running, not like any of our current test apps
	const char* pathName = "/opt/alp/bundles/com.access.apps.testpreflauncher";
	const char* bundleName = "bar:com.access.apps.testpreflauncher";
	const char* bundleNameNoBar = "com.access.apps.testpreflauncher";

	p = alp_bundle_by_name(bundleName);
//	printf("%s 0%x\n", bundleName, p.value);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));

//	alp_prv_bundle_unregister_bundle(p, 0);
//	p = alp_prv_bundle_register_bundle(pathName, "bar", "internal", 0);
//	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));

	pid_t apppid = -1;
	pid_t child = fork();
	if(child == 0)
	{
		// set up a 'me' context
		ctxt = alp_app_setup("package.testapp1Launcher",ALP_APP_LAUNCH_FLAG_NONE,NULL);
		char *argv[1];
		argv[0] = ALP_APP_PRIMARY;
		// launch our target
		alp_status_t err = alp_app_launch(bundleName,1,argv,&apppid);
		// wait for something to happen and then quietly exit
		waitpid(apppid, &status, 0);
		exit(0);
	}
	delay(4);
	err = alp_prv_bundle_unregister_bundle(p, 0);
	CPPUNIT_ASSERT(err != ALP_STATUS_OK);
	CPPUNIT_ASSERT(err == ALP_STATUS_BUNDLEMGR_FAILURE);
	alp_app_exit(bundleName);
	alp_app_destroy(ctxt,0);
	waitpid(child, &status, 0);
}
/*
 * @brief Attempt to delete an open bundle
 * @description Note that this is a registered and open bundle, but not a running program at the time of the delete.
 * 
 */
void BundleMgrClientTests::delete_open_bundle()
{
	AlpBundle p = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_BUNDLEMGR_FAILURE);
	alp_bundle_close(r);
}

