/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
// Includes ------------------------------------------------------------------
#include <hiker/unit_test.h>
#include <cppunit/extensions/HelperMacros.h>

#include <hiker/sysclass.h>
#include <hiker/types.h>
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <sys/types.h>
#include <sys/wait.h>

       #include <sys/types.h>
       #include <sys/socket.h>



// Typedefs ------------------------------------------------------------------
class BundleMgrNotifyTests : public CPPUNIT_NS::TestFixture
{
	CPPUNIT_TEST_SUITE(BundleMgrNotifyTests);
	CPPUNIT_TEST_SUITE_END();

	public:
		void setUp();
		void tearDown();

	public:

		int sv[2];
		pid_t pid;
		static gboolean do_hup(GIOChannel * source, GIOCondition cond, gpointer data);
		
};

ALP_ROOT_TEST_SUITE_REGISTRATION( "AP-002", CPPUNIT_REGISTRY_TYPE_AUTO );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( BundleMgrNotifyTests, "AP-002" );

gboolean BundleMgrNotifyTests::do_hup(GIOChannel * source, GIOCondition cond, gpointer data)
{
	g_main_quit((GMainLoop*)data);
	return FALSE;
}


void BundleMgrNotifyTests::setUp()
{
	chdir(getenv("ALP_RUN_TOP") ?: ".");
	chdir("usr/local/alp/tests");

	socketpair(AF_UNIX, SOCK_STREAM, 0, &sv[0]);

	if ((pid = fork()) == 0) {

		close(sv[0]);

		GMainLoop * loop = g_main_loop_new(NULL, TRUE);

		GIOChannel * socket_channel = g_io_channel_unix_new(sv[1]);

		g_io_channel_set_encoding(socket_channel, NULL, NULL);

		g_io_add_watch(socket_channel, G_IO_HUP, do_hup, loop);

		alp_prv_bundle_init_playground(TRUE, "test", TRUE);
		
		alp_prv_bundle_wipe();
        
		alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", NULL, 0);

		write(sv[1], "1", 1);

		g_main_loop_run(loop);

		alp_prv_bundle_fini();
		
		exit(0);

	} else {
 
		close(sv[1]);
		char buf[1];

		// wait until child says server is ready
		read(sv[0], buf, 1);

		CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	

		alp_prv_bundle_init_playground(FALSE, "test", TRUE);

		CPPUNIT_ASSERT(alp_prv_bundle_state() == 1); // client

	}
}

void BundleMgrNotifyTests::tearDown()
{

	// shut down client
	alp_prv_bundle_fini();

	// tell child to shut down server
	close(sv[0]);
	
	// wait for child to exit	
	waitpid(pid, NULL, 0);
}


