/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
// Includes ------------------------------------------------------------------
#include <hiker/unit_test.h>
#include <cppunit/extensions/HelperMacros.h>

#include <hiker/sysclass.h>
#include <hiker/types.h>
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>

#include <signal.h>

// Typedefs ------------------------------------------------------------------
class BundleMgrRealTests : public CPPUNIT_NS::TestFixture
{
	CPPUNIT_TEST_SUITE(BundleMgrRealTests);
	CPPUNIT_TEST(wipe_test);
	CPPUNIT_TEST(dump_test);
	CPPUNIT_TEST(add_store_test);
	CPPUNIT_TEST(add_store_test_2);
	CPPUNIT_TEST(register_test);
	CPPUNIT_TEST(unregister_test);
	CPPUNIT_TEST(unregister_test_2);
	CPPUNIT_TEST(alp_bundle_by_name_test);
	CPPUNIT_TEST(alp_bundle_metadata_value_test);
	CPPUNIT_TEST(alp_bundle_property_value_test);
	CPPUNIT_TEST(alp_bundle_open_test);
	CPPUNIT_TEST(alp_bundle_close_test);
	CPPUNIT_TEST(alp_bundle_ref_ro_pathname_test);
	CPPUNIT_TEST(alp_bundle_ref_ro_channel_test);
	CPPUNIT_TEST(alp_bundle_ref_ro_data_test);
	CPPUNIT_TEST(alp_bundle_ref_ro_directory_test);
	CPPUNIT_TEST(references_test);
	CPPUNIT_TEST(alp_bundle_rw_pathname_test);
	CPPUNIT_TEST(invalid_package_test);
	CPPUNIT_TEST(verify_image_test);
	CPPUNIT_TEST(install_image_test);
	CPPUNIT_TEST(install_image_test_2);
	CPPUNIT_TEST(install_image_test_3);
	CPPUNIT_TEST(delete_test);
	CPPUNIT_TEST(delete_test_2);
	CPPUNIT_TEST(delete_test_3);
	CPPUNIT_TEST(entrypoint_1_test);
	CPPUNIT_TEST(entrypoint_2_test);
	CPPUNIT_TEST(entrypoint_3_test);
	CPPUNIT_TEST(entrypoint_4_test);
	CPPUNIT_TEST(entrypoint_5_test);
	CPPUNIT_TEST(entrypoint_6_test);
	CPPUNIT_TEST(entrypoint_7_test);
	CPPUNIT_TEST(entrypoint_8_test);
	CPPUNIT_TEST(entrypoint_9_test);
	CPPUNIT_TEST(alp_bundle_locate_file_test);
	CPPUNIT_TEST(scheme_value_test);
	CPPUNIT_TEST(scheme_values_test);
	CPPUNIT_TEST(scheme_identify_test);
	CPPUNIT_TEST(all_properties_test);
	CPPUNIT_TEST(baseline_timing_test);
	CPPUNIT_TEST(property_value_timing_test);
	CPPUNIT_TEST(entrypoint_timing_test);
	CPPUNIT_TIMED_TEST(security_test, 30);
	CPPUNIT_TEST_SUITE_END();

	public:
		void setUp();
		void wipe_test();
		void dump_test();
		void add_store_test();
		void add_store_test_2();
		void register_test();
		void unregister_test();
		void unregister_test_2();
		void alp_bundle_by_name_test();
		void alp_bundle_metadata_value_test();
		void alp_bundle_property_value_test();
		void alp_bundle_open_test();
		void alp_bundle_close_test();
		void alp_bundle_ref_ro_pathname_test();
		void alp_bundle_ref_ro_channel_test();
		void alp_bundle_ref_ro_data_test();
		void alp_bundle_ref_ro_directory_test();
		void alp_bundle_rw_pathname_test();
		void references_test();
		void invalid_package_test();
		void verify_image_test();
		void install_image_test();
		void install_image_test_2();
		void install_image_test_3();
		void delete_test();
		void delete_test_2();
		void delete_test_3();
		void entrypoint_1_test();
		void entrypoint_2_test();
		void entrypoint_3_test();
		void entrypoint_4_test();
		void entrypoint_5_test();
		void entrypoint_6_test();
		void entrypoint_7_test();
		void entrypoint_8_test();
		void entrypoint_9_test();
		void alp_bundle_locate_file_test();
		void scheme_value_test();
		void scheme_values_test();
		void scheme_identify_test();
		void all_properties_test();
		void baseline_timing_test();
		void property_value_timing_test();
		void entrypoint_timing_test();
		void security_test();
		void tearDown();
		
		private:
			char * old_LANG;
		
};

ALP_ROOT_TEST_SUITE_REGISTRATION( "AP-002", CPPUNIT_REGISTRY_TYPE_AUTO );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( BundleMgrRealTests, "AP-002" );


void BundleMgrRealTests::setUp()
{
	chdir(getenv("ALP_RUN_TOP") ?: ".");
	chdir("usr/local/alp/tests");
	char path[PATH_MAX];
	getcwd(path, sizeof(path));
	setenv("PWD", path, 1);

	old_LANG = getenv("LANG") ? strdup(getenv("LANG")) : NULL;
	
	setenv("LANG", "en_US", 1);

	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0); // shouldn't be initialized	
	CPPUNIT_ASSERT(alp_prv_bundle_init_playground(TRUE, "test", FALSE) == 0);
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 2); // server	
}

void BundleMgrRealTests::tearDown()
{
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 2); // server	
	CPPUNIT_ASSERT(alp_prv_bundle_fini() == 0);
	CPPUNIT_ASSERT(alp_prv_bundle_state() == 0);

	if (old_LANG) {
		setenv("LANG", old_LANG, 1);
		free(old_LANG);
	}
	else
		unsetenv("LANG");
}

void BundleMgrRealTests::wipe_test(void)
{
	// in the server, wipe should succeed -- first time, tables won't exist
	CPPUNIT_ASSERT(alp_prv_bundle_wipe() == ALP_STATUS_OK);
	// twice in a row, even -- second time, tables will exist
	CPPUNIT_ASSERT(alp_prv_bundle_wipe() == ALP_STATUS_OK);
}

void BundleMgrRealTests::dump_test(void)
{
	// FIXME: this test should verify that there is dump output... but I'm
	// not sure I have a way to capture trace output
	alp_prv_bundle_dump();
}

void BundleMgrRealTests::add_store_test(void)
{
        const char * store_args[] = {
        	"bar_folder", "test-packages",
		"bar_prefer_archive", 0,
		NULL, NULL
		};
        
	CPPUNIT_ASSERT(alp_prv_bundle_add_store("internal", store_args) == ALP_STATUS_OK);
}


void BundleMgrRealTests::add_store_test_2(void)
{
        const char * store_args[] = {
        	"bar_folder", "test-packages",
		"bar_prefer_archive", 0,
		NULL, NULL
		};
        
	CPPUNIT_ASSERT(alp_prv_bundle_add_store("x", store_args) == ALP_STATUS_OK);
        
	CPPUNIT_ASSERT(alp_prv_bundle_add_store("y", store_args) == ALP_STATUS_OK);

	// cannot add store with duplicate name
	CPPUNIT_ASSERT(alp_prv_bundle_add_store("x", store_args) == ALP_STATUS_BUNDLEMGR_FAILURE);
	
	CPPUNIT_ASSERT(alp_prv_bundle_delete_store("x") == ALP_STATUS_OK);

	CPPUNIT_ASSERT(alp_prv_bundle_delete_store("y") == ALP_STATUS_OK);
}

void BundleMgrRealTests::register_test(void)
{
	CPPUNIT_ASSERT(alp_bundle_is_NULL(alp_prv_bundle_register_bundle(NULL, "bar", NULL, 0)));
	CPPUNIT_ASSERT(alp_bundle_is_NULL(alp_prv_bundle_register_bundle("new_test_packages/package.test", "Sidney", NULL, 0)));
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	alp_prv_bundle_dump();
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::unregister_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);

	p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundleRef ref = alp_bundle_open(p);
	CPPUNIT_ASSERT(ref);
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) != ALP_STATUS_OK);
	alp_bundle_close(ref);
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::unregister_test_2(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	AlpBundleRef ref = alp_bundle_open(p);
	
	CPPUNIT_ASSERT(ref);
	
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) != ALP_STATUS_OK);
	
	// this will leak the AlpBundleRef structure, but shouldn't cause any crashes

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, ALP_PRV_BUNDLE_FLAG_FORCE_UNREGISTER) == ALP_STATUS_OK);

	CPPUNIT_ASSERT(alp_bundle_close(ref) != ALP_STATUS_OK);
	
}

void BundleMgrRealTests::alp_bundle_by_name_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	AlpBundle p2 = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(p.value == p2.value);
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
	AlpBundle p3 = alp_bundle_by_name("bar:package.test");
	CPPUNIT_ASSERT(alp_bundle_is_NULL(p3));
}

void BundleMgrRealTests::alp_bundle_metadata_value_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	char * value = alp_bundle_metadata_value(p, "scheme_name");
	CPPUNIT_ASSERT(value);
	CPPUNIT_ASSERT(strcmp(value, "bar") == 0);
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
	g_free(value);
}

void BundleMgrRealTests::alp_bundle_property_value_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	char * value = alp_bundle_property_value(p, "testCase", 0, "name");
	CPPUNIT_ASSERT(value);
	CPPUNIT_ASSERT(strcmp(value, "Samuel") == 0);
	g_free(value);
	
	value = alp_bundle_property_value(p, "testCase", 1, "name");
	CPPUNIT_ASSERT(value == NULL);
	
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::alp_bundle_open_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);
	alp_bundle_close(r);


	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
	CPPUNIT_ASSERT(alp_bundle_open(p) == NULL);

}

void BundleMgrRealTests::alp_bundle_close_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);
	CPPUNIT_ASSERT(alp_bundle_close(r) == ALP_STATUS_OK);
	//CPPUNIT_ASSERT(alp_bundle_close(r) == ALP_STATUS_BUNDLEMGR_FAILURE); // reads undefined memory, not safe
	CPPUNIT_ASSERT(alp_bundle_close(NULL) == ALP_STATUS_BUNDLEMGR_FAILURE);

	r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);
	
	AlpBundleRef r2 = alp_bundle_open(p);
	
	CPPUNIT_ASSERT(r2);
	CPPUNIT_ASSERT(r != r2);
	CPPUNIT_ASSERT(alp_bundle_close(r) == ALP_STATUS_OK);
	//CPPUNIT_ASSERT(alp_bundle_close(r) == ALP_STATUS_BUNDLEMGR_FAILURE); // reads undefined memory, not safe
	CPPUNIT_ASSERT(alp_bundle_close(r2) == ALP_STATUS_OK);
	//CPPUNIT_ASSERT(alp_bundle_close(r2) == ALP_STATUS_BUNDLEMGR_FAILURE); // reads undefined memory, not safe
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::alp_bundle_ref_ro_pathname_test(void)
{

	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);

	char * root = alp_bundle_ref_ro_pathname(r, "/"); // get root of package
	char * pathname = alp_bundle_ref_ro_pathname(r, "resource.txt"); // get localized resource path

	CPPUNIT_ASSERT(root != NULL);
	CPPUNIT_ASSERT(pathname != NULL);

	CPPUNIT_ASSERT(strlen(pathname) > strlen(root));
	
	// pathname is under root
	CPPUNIT_ASSERT(strncmp(root, pathname, strlen(root)) == 0);

	printf("pathname=%s\n", pathname);
	fflush(stdout);
	
	// and it is specificaly in this location
	CPPUNIT_ASSERT(strcmp(pathname + strlen(root), "rsc/en_US/resource.txt") == 0);	
	
	alp_bundle_close(r);
	
	g_free(root);
	g_free(pathname);
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
	
}

void BundleMgrRealTests::alp_bundle_ref_ro_channel_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);

	GIOChannel * channel = alp_bundle_ref_ro_channel(r, "resource.txt");
	
	CPPUNIT_ASSERT(channel);
	g_io_channel_unref(channel);
	
	alp_bundle_close(r);
	
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::alp_bundle_ref_ro_data_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);

	gchar * data;
	gsize length;
	
	CPPUNIT_ASSERT(alp_bundle_ref_ro_data(r, "resource.txt", &data, &length) == ALP_STATUS_OK);
	CPPUNIT_ASSERT(length == 25);
	CPPUNIT_ASSERT(strlen(data) == 25);
	CPPUNIT_ASSERT(strcmp(data, "This is a resource file.\n") == 0);
	
	g_free(data);
	
	alp_bundle_close(r);
	
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::alp_bundle_ref_ro_directory_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	AlpBundleRef r = alp_bundle_open(p);
	CPPUNIT_ASSERT(r);

	GList * dir = alp_bundle_ref_ro_directory(r, "/", TRUE);
	
	CPPUNIT_ASSERT(dir);
	
	alp_bundle_close(r);
	
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::references_test(void)
{
	AlpBundle p_a = alp_prv_bundle_register_bundle("new_test_packages/package.test-a", "bar", "internal", 0);
	AlpBundle p_b = alp_prv_bundle_register_bundle("new_test_packages/package.test-b", "bar", "internal", 0);
	AlpBundle p_c = alp_prv_bundle_register_bundle("new_test_packages/package.test-c", "bar", "internal", 0);
	
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p_a));
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p_b));
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p_c));
	
	gchar * result;
	
	alp_prv_bundle_dump();
	
	result = alp_bundle_metadata_value(p_a, "refcount");
	CPPUNIT_ASSERT(result == NULL);

	result = alp_bundle_metadata_value(p_b, "refcount");
	CPPUNIT_ASSERT(result == NULL);

	result = alp_bundle_metadata_value(p_c, "refcount");
	CPPUNIT_ASSERT(result == NULL);
	
	AlpBundleRef ref1 = alp_bundle_open(p_a);
	CPPUNIT_ASSERT(ref1 != NULL);
	
	result = alp_bundle_metadata_value(p_a, "refcount");
	CPPUNIT_ASSERT(result != NULL && strcmp(result, "1") == 0);
	g_free(result);

	result = alp_bundle_metadata_value(p_b, "refcount");
	CPPUNIT_ASSERT(result == NULL);

	result = alp_bundle_metadata_value(p_c, "refcount");
	CPPUNIT_ASSERT(result == NULL);

	AlpBundleRef ref2 = alp_bundle_open(p_b);
	CPPUNIT_ASSERT(ref2 != NULL);

	result = alp_bundle_metadata_value(p_a, "refcount");
	CPPUNIT_ASSERT(result != NULL && strcmp(result, "1") == 0);
	g_free(result);

	result = alp_bundle_metadata_value(p_b, "refcount");
	CPPUNIT_ASSERT(result != NULL && strcmp(result, "1") == 0);
	g_free(result);

	result = alp_bundle_metadata_value(p_c, "refcount");
	CPPUNIT_ASSERT(result != NULL && strcmp(result, "1") == 0);
	g_free(result);
	
	CPPUNIT_ASSERT(alp_bundle_close(ref1) == ALP_STATUS_OK);
	CPPUNIT_ASSERT(alp_bundle_close(ref2) == ALP_STATUS_OK);
	
	alp_prv_bundle_unregister_bundle(p_a, 0);
	alp_prv_bundle_unregister_bundle(p_b, 0);
	alp_prv_bundle_unregister_bundle(p_c, 0);
}

void BundleMgrRealTests::alp_bundle_rw_pathname_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	gchar * path = alp_bundle_rw_pathname(p, "", FALSE);
	
	CPPUNIT_ASSERT(path == NULL);

	path = alp_bundle_rw_pathname(p, "", TRUE);

	CPPUNIT_ASSERT(path);
	g_free(path);
	
	path = alp_bundle_rw_pathname(p, "", FALSE);
	
	CPPUNIT_ASSERT(path);
	
	printf("path is %s\n", path);
	
	char buf[1024];
	sprintf(buf, "%s%s", path, "somefile.txt");

	printf("buf is %s\n", buf);
	
	FILE * f = fopen(buf, "w");
	CPPUNIT_ASSERT(f);
	
	fputs("This is a test", f);
	fclose(f);
	
	g_free(path);

	path = alp_bundle_rw_pathname(p, "a/", TRUE);
	CPPUNIT_ASSERT(path);
	printf("path2 = %s\n", path);
	g_free(path);
	
	path = alp_bundle_rw_pathname(p, "a/b/c/foo", FALSE);
	CPPUNIT_ASSERT(path == NULL);

	path = alp_bundle_rw_pathname(p, "a/b/c/foo", TRUE);
	CPPUNIT_ASSERT(path);
	g_free(path);

	path = alp_bundle_rw_pathname(p, "a/b/c/foo", FALSE);
	CPPUNIT_ASSERT(path);
	printf("path3 = %s\n", path);
	g_free(path);	
	
	path = alp_bundle_rw_pathname(p, "a/other/somedir", TRUE);
	printf("path4 = %s\n", path);
	g_free(path);

	CPPUNIT_ASSERT(alp_bundle_rw_delete(p, "", TRUE) == ALP_STATUS_OK);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::invalid_package_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-invalid", "bar", "internal", 0);
	CPPUNIT_ASSERT(alp_bundle_is_NULL(p));
}

void BundleMgrRealTests::verify_image_test(void)
{
	GIOChannel * channel = g_io_channel_new_file("new_test_packages/images/invalid_image.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	int result = alp_bundle_image_verify(NULL, NULL, channel, TRUE, NULL, NULL, NULL);

	CPPUNIT_ASSERT(result == ALP_STATUS_BUNDLEMGR_FAILURE);
	
	g_io_channel_unref(channel);
	
	
	channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	result = alp_bundle_image_verify(NULL, NULL, channel, TRUE, NULL, NULL, NULL);

	CPPUNIT_ASSERT(result == ALP_STATUS_OK);
	
	g_io_channel_unref(channel);
	
}

void BundleMgrRealTests::install_image_test(void)
{
	GIOChannel * channel = g_io_channel_new_file("new_test_packages/images/invalid_image.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundle package;
	
	package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(! alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	CPPUNIT_ASSERT(alp_bundle_delete(package, FALSE) == ALP_STATUS_OK);	
}

void BundleMgrRealTests::install_image_test_2(void)
{
	GIOChannel * channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundle package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(! alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	// installing the same bundle a second time should fail, as it is a duplicate
	
	package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE /* flags */, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);

	channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	// but give it the ALP_BUNDLE_FLAG_OVERWRITE_DUPLICATE, and it should work
	
	AlpBundle package2 = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, ALP_BUNDLE_FLAG_OVERWRITE_DUPLICATE /* flags */, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(package2));
	
	g_io_channel_unref(channel);

	// Now we shouldn't be able to delete the original one	
	CPPUNIT_ASSERT(alp_bundle_delete(package, FALSE) != ALP_STATUS_OK);	
	
	// But we should be able to delete the new one
	CPPUNIT_ASSERT(alp_bundle_delete(package2, FALSE) == ALP_STATUS_OK);	
}

void BundleMgrRealTests::install_image_test_3(void)
{
	GIOChannel * channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundle package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(! alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	// installing the same bundle a second time should fail, as it is a duplicate
	
	AlpBundle package2 = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE /* flags */, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(alp_bundle_is_NULL(package2));
	
	g_io_channel_unref(channel);

	channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundleRef ref = alp_bundle_open(package);
	CPPUNIT_ASSERT(ref);
	
	// we'lll try to ALP_BUNDLE_FLAG_OVERWRITE_DUPLICATE, but this shouldn't work as the bundle is open
	
	package2 = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, ALP_BUNDLE_FLAG_OVERWRITE_DUPLICATE /* flags */, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(alp_bundle_is_NULL(package2));
	
	g_io_channel_unref(channel);
	

	channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	// let's force it
	
	package2 = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, ALP_BUNDLE_FLAG_OVERWRITE_DUPLICATE|ALP_PRV_BUNDLE_FLAG_FORCE_UNREGISTER /* flags */, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	// that should have worked
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(package2));
	
	g_io_channel_unref(channel);
		
	// Now we shouldn't be able to delete the original one	
	CPPUNIT_ASSERT(alp_bundle_delete(package, FALSE) != ALP_STATUS_OK);	
	
	// And the ref should be invalid (but not crash)
	CPPUNIT_ASSERT(alp_bundle_close(ref) != ALP_STATUS_OK);
	
	// And we should be able to delete the new one
	CPPUNIT_ASSERT(alp_bundle_delete(package2, FALSE) == ALP_STATUS_OK);	
	
	// note that we should also test the LAZY_DELETE logic, but a proper test will involve catching
	// notifications and making sure everything is sequenced usefully
}

void BundleMgrRealTests::delete_test(void)
{
	GIOChannel * channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundle package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(! alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	AlpBundleRef ref = alp_bundle_open(package);
	
	CPPUNIT_ASSERT(ref);
	
	CPPUNIT_ASSERT(alp_bundle_delete(package, FALSE) != ALP_STATUS_OK);	
	
	alp_bundle_close(ref);

	CPPUNIT_ASSERT(alp_bundle_delete(package, FALSE) == ALP_STATUS_OK);	
}

void BundleMgrRealTests::delete_test_2(void)
{
	GIOChannel * channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundle package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(! alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	AlpBundleRef ref = alp_bundle_open(package);
	
	CPPUNIT_ASSERT(ref);
	
	// Simple delete cannot work
	CPPUNIT_ASSERT(alp_bundle_delete(package, FALSE) != ALP_STATUS_OK);	
	
	// A force delete will not actually work, because the bundle cannot be unregistered
	CPPUNIT_ASSERT(alp_bundle_delete(package, ALP_PRV_BUNDLE_FLAG_FORCE_DELETE) != ALP_STATUS_OK);	

	// Forcing both should work
	CPPUNIT_ASSERT(alp_bundle_delete(package, ALP_PRV_BUNDLE_FLAG_FORCE_DELETE|ALP_PRV_BUNDLE_FLAG_FORCE_UNREGISTER) == ALP_STATUS_OK);	

	// And now it should not be there
	
	AlpBundleRef ref2 = alp_bundle_open(package);
	CPPUNIT_ASSERT(ref2 == NULL);
	
	// But now the ref is invalid, so close should not work. But also not crash.
	CPPUNIT_ASSERT(alp_bundle_close(ref) != ALP_STATUS_OK);
	
	// (this will leak ref)
}

void BundleMgrRealTests::delete_test_3(void)
{
	GIOChannel * channel = g_io_channel_new_file("new_test_packages/images/test_launcher.bar", "r", NULL);
	CPPUNIT_ASSERT(channel);
	
	g_io_channel_set_encoding(channel, NULL, NULL);
	
	AlpBundle package = alp_bundle_image_install(NULL /*filename*/, NULL /*mimetype*/, channel, NULL /*destination*/, FALSE, NULL /*metadata*/, NULL /*properties*/, NULL /*progress*/);
	
	CPPUNIT_ASSERT(! alp_bundle_is_NULL(package));
	
	g_io_channel_unref(channel);
	
	AlpBundleRef ref = alp_bundle_open(package);
	
	CPPUNIT_ASSERT(ref);
	
	// Cannot delete it, because it is open
	
	CPPUNIT_ASSERT(alp_bundle_delete(package, FALSE) != ALP_STATUS_OK);
	
	// But can ask it to be lazily deleted

	CPPUNIT_ASSERT(alp_bundle_delete(package, ALP_PRV_BUNDLE_FLAG_LAZY_DELETE) == ALP_STATUS_OK);	
	
	// It should still exist
	
	AlpBundleRef ref2 = alp_bundle_open(package);
	CPPUNIT_ASSERT(ref2 != NULL);
	CPPUNIT_ASSERT(alp_bundle_close(ref2) == ALP_STATUS_OK);
	
	// Until we close the last ref
	
	CPPUNIT_ASSERT(alp_bundle_close(ref) == ALP_STATUS_OK);
	
	// And then it should go away

	AlpBundleRef ref3 = alp_bundle_open(package);
	CPPUNIT_ASSERT(ref3 == NULL);
}	

//! Can we acquire an entry point for a shared library using a package URL?
void BundleMgrRealTests::entrypoint_1_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-entries", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	GClosure * entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/libalp_entry_lib.so", NULL);
	
	CPPUNIT_ASSERT(entry != NULL);
	
	g_closure_unref(entry);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
	
	// now check to make sure we get a failure for a missing name
	
	entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/libalp_entry_lib.so-nothere.so", NULL);
	
	CPPUNIT_ASSERT(entry == NULL);
}

//! Can we acquire an entry point for an executable using a package URL?
void BundleMgrRealTests::entrypoint_2_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-entries", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	GClosure * entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/entry_executable", NULL);
	
	CPPUNIT_ASSERT(entry != NULL);
	
	g_closure_unref(entry);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);

	// now check to make sure we get a failure for a missing name
	
	entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/entry_executable-notthere.exe", NULL);
	
	CPPUNIT_ASSERT(entry == NULL);
}

static void pack(GValue * return_value, GValue *params, int argc, char ** argv, char ** envp)
{
	g_type_init();

	memset(return_value, '\0', sizeof(return_value));
	memset(&params[0], '\0', sizeof(GValue));
	memset(&params[1], '\0', sizeof(GValue));
	memset(&params[2], '\0', sizeof(GValue));

	g_value_init(return_value, G_TYPE_INT);
	g_value_init(&params[0], G_TYPE_INT);
	g_value_init(&params[1], G_TYPE_POINTER);
	g_value_init(&params[2], G_TYPE_POINTER);

	g_value_set_int(&params[0], argc);
	g_value_set_pointer(&params[1], argv);
	g_value_set_pointer(&params[2], envp);
                                                                                                                                                                                                      
}

static int unpack(GValue * return_value)
{
	return  g_value_get_int(return_value);
}

//! Can we invoke a specific entrypoint in a shared library with arguments, and get a result?
void BundleMgrRealTests::entrypoint_3_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-entries", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	GClosure * entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/libalp_entry_lib.so", NULL);
	
	CPPUNIT_ASSERT(entry != NULL);

	GValue result, params[3];
	
	char * args[] = {"bar:package.test-entries","-return","42",NULL};
	char * env[] = {NULL};
	
	pack(&result, params, 3, args, env);

	g_closure_invoke(entry, &result, 3, params, NULL);

	int status = unpack(&result);

	CPPUNIT_ASSERT(status == 42);
	
	g_closure_unref(entry);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

//! Can we invoke an executable with arguments, and get a result?
void BundleMgrRealTests::entrypoint_4_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-entries", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	GClosure * entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/entry_executable", NULL);
	
	CPPUNIT_ASSERT(entry != NULL);

	GValue result, params[3];
	
	char * args[] = {"bar:package.test-entries","-return","16",NULL};
	char * env[] = {NULL};
	
	pack(&result, params, 3, args, env);

	g_closure_invoke(entry, &result, 3, params, NULL);

	int status = unpack(&result);

	CPPUNIT_ASSERT(status == 16);
	CPPUNIT_ASSERT(ALP_BUNDLE_ENTRYPOINT_CLOSURE_RESULT_SIGNAL(entry) == 0);
	
	g_closure_unref(entry);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

//! Can we invoke an executable with arguments, and catch a signal?
void BundleMgrRealTests::entrypoint_5_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-entries", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	GClosure * entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/entry_executable", NULL);
	
	CPPUNIT_ASSERT(entry != NULL);

	GValue result, params[3];
	
	char * args[] = {"bar:package.test-entries","-SIGINT",NULL};
	char * env[] = {NULL};
	
	pack(&result, params, 2, args, env);

	g_closure_invoke(entry, &result, 3, params, NULL);

	int status = unpack(&result);

	CPPUNIT_ASSERT(status == -1);
	CPPUNIT_ASSERT(ALP_BUNDLE_ENTRYPOINT_CLOSURE_RESULT_SIGNAL(entry) == SIGINT);
	
	g_closure_unref(entry);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

//! Can we acquire an entry point for a shared library using a file URL?
void BundleMgrRealTests::entrypoint_6_test()
{
	char buf[PATH_MAX];
	sprintf(buf, "file://%s/new_test_packages/package.test-entries/libalp_entry_lib.so", getenv("PWD"));
	
	GClosure * entry = alp_bundle_acquire_entrypoint(buf, NULL);
	
	CPPUNIT_ASSERT(entry != NULL);
	
	g_closure_unref(entry);
	
	strcat(buf, "-notthere.so");

	// now check to make sure we get a failure for a missing name
	
	entry = alp_bundle_acquire_entrypoint(buf, NULL);
	
	CPPUNIT_ASSERT(entry == NULL);
}

//! Can we acquire an entry point for an executable using a package URL?
void BundleMgrRealTests::entrypoint_7_test()
{
	char buf[PATH_MAX];
	sprintf(buf, "file://%s/new_test_packages/package.test-entries/entry_executable", getenv("PWD"));
	
	GClosure * entry = alp_bundle_acquire_entrypoint(buf, NULL);
	
	CPPUNIT_ASSERT(entry != NULL);
	
	g_closure_unref(entry);
	
	strcat(buf, "-notthere.exe");
	
	// now check to make sure we get a failure for a missing name
	
	entry = alp_bundle_acquire_entrypoint(buf, NULL);
	
	CPPUNIT_ASSERT(entry == NULL);
}

//! See if we can use the convenience function alp_bundle_entrypoint_closure_execl
void BundleMgrRealTests::entrypoint_8_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-entries", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	GClosure * entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/libalp_entry_lib.so", NULL);
	CPPUNIT_ASSERT(entry != NULL);

	int status = alp_bundle_entrypoint_closure_execl(entry, "bar:package.test-entries", "-return", "42", 0);

	CPPUNIT_ASSERT(status == 42);
	
	g_closure_unref(entry);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

//! See if we can use the convenience function alp_bundle_entrypoint_closure_execve
void BundleMgrRealTests::entrypoint_9_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-entries", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	GClosure * entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/libalp_entry_lib.so", NULL);
	
	CPPUNIT_ASSERT(entry != NULL);

	GValue result, params[3];
	
	char * args[] = {"bar:package.test-entries","-return","42",NULL};
	char * env[] = {NULL};
	
	int status = alp_bundle_entrypoint_closure_execve(entry, args, env);
	CPPUNIT_ASSERT(status == 42);
	
	g_closure_unref(entry);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::alp_bundle_locate_file_test(void)
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	AlpBundleRef r = alp_bundle_open(p);
	
	CPPUNIT_ASSERT(r);
	
	gchar * testpath1 = alp_bundle_ref_ro_pathname(r, "/resource.txt");

	CPPUNIT_ASSERT(testpath1);
	
	AlpBundle p2 = alp_bundle_locate_file(testpath1);

	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p2));
	CPPUNIT_ASSERT(p2.value == p.value);
	
	char testpath2[PATH_MAX];
	
	CPPUNIT_ASSERT(realpath(testpath1, testpath2) != NULL);
	
	AlpBundle p3 = alp_bundle_locate_file(testpath1);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p3));
	CPPUNIT_ASSERT(p3.value == p.value);

	// Try a path which can't directly go through realpath()
	// because it doesn't completely exist -- it should still
	// be able to work out that this belongs to the bundle
	// Currently it fails due to the use of realpath() inside
	// the locate_file routine
	char testpath3[PATH_MAX];
	
	strcpy(testpath3, testpath2);
	strcat(testpath3, "/somedir/somefile/all-of-this-doesn't-exit");
	
	AlpBundle p4 = alp_bundle_locate_file(testpath3);
	CPPUNIT_ASSERT(alp_bundle_is_NULL(p4));
	
	alp_bundle_close(r);

	//symlink will have gone when package was closed
	p2 = alp_bundle_locate_file(testpath1);
	CPPUNIT_ASSERT(alp_bundle_is_NULL(p2));
	
	//but underlying resolved path should still work
	p3 = alp_bundle_locate_file(testpath2);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p3));
	CPPUNIT_ASSERT(p3.value == p.value);

	g_free(testpath1);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::scheme_value_test()
{
	gchar * str = alp_bundle_scheme_value("bar", "extension");
	
	CPPUNIT_ASSERT(str != NULL);
	CPPUNIT_ASSERT(strcmp(str, ".bar") == 0);
	
	g_free(str);
	
	str = alp_bundle_scheme_value("-nonexistant-", "extension");
	
	CPPUNIT_ASSERT(str == NULL);
}

bool hash_match(GHashTable * h, const char ** match)
{
	unsigned int i, count, matched;
	
	for (i=0; match[i*2] != NULL; i++)
		;
		
	count = i;
	
	matched = 0;
	
	for (i=0; i<count; i++) {
		gpointer p = g_hash_table_lookup(h, match[i*2]);
		if (p) {
			if (strcmp((char*)p, match[i*2+1]) == 0)
				matched++;
		}
	}
	
	if ((matched == count) && (g_hash_table_size(h) == count))
		return true; // success
	
	return false; // failure
}

void BundleMgrRealTests::scheme_values_test()
{
	GHashTable * results = alp_bundle_scheme_values("bar");
	
	CPPUNIT_ASSERT(results != NULL);
	
	const char *expected[] = {
		"extension", ".bar",
		"mimetype", "x-application/access-bar",
		NULL, NULL,
	};
	
	CPPUNIT_ASSERT(hash_match(results, expected));
	
	g_hash_table_destroy(results);
	
	results = alp_bundle_scheme_values("-nonexistant-");
	
	CPPUNIT_ASSERT(results == NULL);
}

void BundleMgrRealTests::scheme_identify_test()
{
	gchar * scheme = alp_bundle_scheme_identify("something.bar", NULL);
	
	CPPUNIT_ASSERT(scheme);
	
	CPPUNIT_ASSERT(strcmp(scheme, "bar") == 0);
	
	g_free(scheme);
	
	scheme = alp_bundle_scheme_identify(NULL, NULL);
	
	CPPUNIT_ASSERT(scheme == NULL);
}

void BundleMgrRealTests::all_properties_test()
{
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-d", "bar", "internal", 0);
	
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));

	GList * results = alp_bundle_all_properties(p);
	
	CPPUNIT_ASSERT(results != NULL);
	CPPUNIT_ASSERT(g_list_length(results) == 3);

	const char *app_props[] = {
		"", "application",
		"icon", "Missing-icon",
		"name", "",
		"fred", "test",
		NULL, NULL,
	};
	
	const char *other1_props[] = {
		"", "other",
		"somekey", "",
		NULL, NULL,
	};
	
	const char *other2_props[] = {
		"", "other",
		"key1","value1",
		NULL, NULL,
	};
	
	CPPUNIT_ASSERT(hash_match((GHashTable*)g_list_nth_data(results, 0), app_props));
	CPPUNIT_ASSERT(hash_match((GHashTable*)g_list_nth_data(results, 1), other1_props));
	CPPUNIT_ASSERT(hash_match((GHashTable*)g_list_nth_data(results, 2), other2_props));
	
	g_list_foreach(results, (GFunc)g_hash_table_destroy, NULL);

	results = alp_bundle_all_properties(ALP_BUNDLE_NULL);
	
	CPPUNIT_ASSERT(results == NULL);

	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::baseline_timing_test()
{
	int i;
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	for (i=0;i<1000;i++) {
		;
	}
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::property_value_timing_test()
{
	int i;
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	for (i=0;i<1000;i++) {
		char * value = alp_bundle_property_value(p, "testCase", 0, "name");
		g_free(value);
	}
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::entrypoint_timing_test()
{
	int i;
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-entries", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	
	for (i=0;i<100;i++) {
		GClosure * entry = alp_bundle_acquire_entrypoint("bar:package.test-entries/libalp_entry_lib.so", NULL);
	
		CPPUNIT_ASSERT(entry != NULL);
	
		g_closure_unref(entry);
	}
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}

void BundleMgrRealTests::security_test()
{
	int i;
	AlpBundle p = alp_prv_bundle_register_bundle("new_test_packages/package.test-security", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
}



