/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright � 2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/prv/bundlemgr_prv.h>
#include <hiker/notifymgr.h>
#include <hiker/volumesvcs.h>
#include <hiker/volumesvcs_types.h>
#include <hiker/appmgr.h>
#include <hiker/pdk/appmgr_pdk.h>
#include <hiker/ipc.h>
#include <string.h>
#include <stdlib.h>
#include <sqlite3.h>
#include <dlfcn.h>
#include <sys/types.h>
#include <dirent.h>
#include <sys/types.h>
#include <dirent.h>
#include <errno.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>
#include <sys/sendfile.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <expat.h>
#include <alp/pdk/gras.h>
#include <alp/pdk/gras_settings.h>
#include <alp/pdk/gras_notif.h>
            
#include "pm_traces.h"
#include "pm_globals.h"
#include "pm_ipc.h"
#include "pm_sql.h"
#include "pm_msg.h"
#include "pm_perms.h"
#include "ghost_support.h"

GTree * sRemoveBundleIndex;
GTree * sRemoveBundleLaunch;
GTree * sRemoveBundleName;

struct cardUserData_t
{
  XML_Parser parser;
  int state;
  int depth;
  GString * text;
  gchar * launchArgument;
  int autodelete;
  GList * OnInsertCopy; // points to gchar*
  GList * OnRemoveDelete; // points to gchar*
  GList * OnInsertLaunch; // points to char*[2]: name, argument
  GList * OnRemoveLaunch; // points to char*[2]: name, argument
  
  int sawOnInsert;
  int sawOnRemove;
};

static void XMLCALL
startElement(void *cardUserData_v, const char *name, const char **atts)
{
	int i;
	struct cardUserData_t * cardUserData = cardUserData_v;
	
	ALP_TMTL("state=%d,depth=%d,name=%s", cardUserData->state, cardUserData->depth,name);
	
	if (cardUserData->state == 0) {
	  if (strcmp(name, "card") != 0) {
            /* no other root element is acceptable */
            XML_StopParser(cardUserData->parser, XML_FALSE);
            return;
	  }
	  cardUserData->state = 1;
        } else if (cardUserData->state == 1) {
          if (cardUserData->depth == 1) {
            if (strcmp(name, "OnInsert") == 0) {
              if (cardUserData->sawOnInsert) {
                ALP_TMTL("Can't have more then one OnInsert block in card.xml");
                XML_StopParser(cardUserData->parser, XML_FALSE);
                return;
              }
              cardUserData->state = 2;
            } else if (strcmp(name, "OnRemove") == 0) {
              if (cardUserData->sawOnRemove) {
                ALP_TMTL("Can't have more then one OnRemove block in card.xml");
                XML_StopParser(cardUserData->parser, XML_FALSE);
                return;
              }
              cardUserData->state = 3;
            }
          }
	} else if (cardUserData->state == 2) {
	  if (cardUserData->depth == 2) {
	    if (strcmp(name, "copy") == 0)
	    {
	        cardUserData->text = g_string_new("");
            }
	    if (strcmp(name, "launch") == 0)
	    {
	        cardUserData->text = g_string_new("");
	        if (atts) {
	          int i;
	          for (i=0; atts[i]; i+=2) {
	            if (strcmp(atts[i], "argument") == 0) {
	              g_free(cardUserData->launchArgument);
	              cardUserData->launchArgument = g_strdup(atts[i+1]);
	            }
	          }
	        }
            }
	  }
	} else if (cardUserData->state == 3) {
	  if (cardUserData->depth == 2) {
	    if (strcmp(name, "delete") == 0)
	    {
	        cardUserData->text = g_string_new("");
            }
	    if (strcmp(name, "launch") == 0)
            {
	        cardUserData->text = g_string_new("");
	        if (atts) {
	          int i;
	          for (i=0; atts[i]; i+=2) {
	            if (strcmp(atts[i], "argument") == 0) {
	              g_free(cardUserData->launchArgument);
	              cardUserData->launchArgument = g_strdup(atts[i+1]);
	            }
	          }
	        }
            }
	  }
	}
	
	cardUserData->depth++;
}

static void XMLCALL
endElement(void *cardUserData_v, const char *name)
{
	struct cardUserData_t * cardUserData = cardUserData_v;

	cardUserData->depth--;

	ALP_TMTL("end state=%d,depth=%d,name=%s", cardUserData->state, cardUserData->depth,name);
	
	if (cardUserData->depth == 1 && cardUserData->state == 2) {
	  cardUserData->sawOnInsert = 1;
	  cardUserData->state = 1;
	} else if (cardUserData->depth == 1 && cardUserData->state == 3) {
	  cardUserData->sawOnRemove = 1;
	  cardUserData->state = 1;
        }
        
        if (cardUserData->depth == 2 && cardUserData->state == 2) {
          if (strcmp(name, "copy") == 0) 
          {
            cardUserData->OnInsertCopy = g_list_append(cardUserData->OnInsertCopy, g_strdup(cardUserData->text->str));
            
            g_string_free(cardUserData->text, TRUE);
            cardUserData->text = NULL;
          }

          if (strcmp(name, "launch") == 0) 
          {
            char ** l = g_new0(char*, 2);
            l[0] = g_strdup(cardUserData->text->str);
            l[1] = cardUserData->launchArgument ?: g_strdup("--alp-insert-card");
            cardUserData->OnInsertLaunch = g_list_append(cardUserData->OnInsertLaunch, l);
            cardUserData->launchArgument = NULL;
            
            g_string_free(cardUserData->text, TRUE);
            cardUserData->text = NULL;
          }
        }
        
        if (cardUserData->depth == 2 && cardUserData->state == 3) {

          if (strcmp(name, "delete") == 0) 
          {
            cardUserData->OnRemoveDelete = g_list_append(cardUserData->OnRemoveDelete, g_strdup(cardUserData->text->str));
            
            g_string_free(cardUserData->text, TRUE);
            cardUserData->text = NULL;
          }

          if (strcmp(name, "launch") == 0) 
          {
            char ** l = g_new0(char*, 2);
            l[0] = g_strdup(cardUserData->text->str);
            l[1] = cardUserData->launchArgument ?: g_strdup("--alp-remove-card");
            cardUserData->OnRemoveLaunch = g_list_append(cardUserData->OnRemoveLaunch, l);
            cardUserData->launchArgument = NULL;
            
            g_string_free(cardUserData->text, TRUE);
            cardUserData->text = NULL;
          }
        }

	
}

static void XMLCALL
Text(void * cardUserData_v, const char * name, int len)
{
	struct cardUserData_t * cardUserData = cardUserData_v;

	if (cardUserData->text)
	{
               g_string_append_len(cardUserData->text, name, len);
        }
}

void launch(const char * appid, const char * argument)
{
  void * appmgr = dlopen("libalp_appmgr.so", RTLD_LAZY);
  
  if (!appmgr) {
    ALP_TMTL("Unable to dlopen libalp_appmgr.so");
    return;
  }
  
  typeof (&alp_app_setup) f1 = dlsym(appmgr, "alp_app_setup");
  typeof (&alp_app_launch_async) f2 = dlsym(appmgr, "alp_app_launch_async");
  typeof (&alp_app_set_default_context) f3 = dlsym(appmgr, "alp_app_set_default_context");
  typeof (&alp_app_abandon) f4 = dlsym(appmgr, "alp_app_abandon");
  
  if (!f1) {
    ALP_TMTL("Unable to acquite alp_app_setup symbol");
    return;
  }

  if (!f2) {
    ALP_TMTL("Unable to acquite alp_app_launch_async symbol");
    return;
  }

  if (!f3) {
    ALP_TMTL("Unable to acquite alp_app_set_default_context symbol");
    return;
  }

  if (!f4) {
    ALP_TMTL("Unable to acquite alp_app_abandon symbol");
    return;
  }

  AlpAppContext * cntx = f1("bundlemgr_d", 0, NULL);
  f3(cntx);

  const char * argv[2];
  argv[0] = argument;
  argv[1] = NULL;

  f2((char*)appid, 1, (char**)argv);

  f4(cntx);
  
  dlclose(appmgr);
}



#define SIZE 65536

void load_card_xml(const char * store, int fd)
{
  size_t end = lseek(fd, SEEK_END, 0);
  lseek(fd, SEEK_SET, 0);
  
  size_t pos = 0;
  size_t len;
  
  struct cardUserData_t * d = g_new0(struct cardUserData_t, 1);
	
  memset(d, '\0', sizeof(d));
  
  d->state = 0;
  d->text = NULL;

  XML_Parser parser = XML_ParserCreate(NULL);
  XML_SetUserData(parser, d);
  XML_SetElementHandler(parser, startElement, endElement);
  XML_SetCharacterDataHandler(parser, Text);
  
  d->parser = parser;
  
  while (pos < end) {
    len = end - pos;
    void * buf = XML_GetBuffer(parser, SIZE);
		
    int r = read(fd, buf, (len > SIZE) ? SIZE : len);
		
    if (r < 0) {
	ALP_TMTL("Error reading card.xml");
	goto fail;
    }
    
    int done = (r == 0);
		
    if (!XML_ParseBuffer(parser, r, done)) {
      ALP_TMTL("Card.xml parser aborted with failure");
      goto fail;
    }
		
    if (done)
      break;
  }
  
  GList * iter;
  
  for (iter = d->OnInsertCopy; iter; iter = g_list_delete_link(iter, iter))
  {
    ALP_TMTL("Card insert, copy '%s'", (char*)iter->data);

    GArray * a = alp_bundle_search((char*)iter->data, TRUE, NULL, store, NULL);
    
    if (a->len == 1) {
      // FIXME: consider flags to use to deal with conflicts
      AlpBundle b = alp_bundle_copy_to_store(g_array_index(a, AlpBundle, 0), "internal", 0);

#if 0
      // FIXME: Disabling auto-delete for now, pending further consideration
      if (!sRemoveBundleIndex)
        sRemoveBundleIndex = g_tree_new_full((GCompareDataFunc)strcmp, NULL, g_free, NULL);
      
      GList * list = g_tree_lookup(sRemoveBundleIndex, store);
      list = g_list_append(list, (void*)b.value);
      g_tree_replace(sRemoveBundleIndex, g_strdup(store), list);
#endif
      
    } else {
      ALP_TMTL("Unable to locate bundle named '%s' on inserted card to copy", (char*)iter->data);
    }
  }

  for (iter = d->OnInsertLaunch; iter; iter = g_list_delete_link(iter, iter))
  {
    char ** l = iter->data;
    ALP_TMTL("Card insert, launch '%s', argument '%s'", l[0], l[1]);
    
    launch(l[0], l[1]);
  }

  for (iter = d->OnRemoveLaunch; iter; iter = g_list_delete_link(iter, iter))
  {
    char ** l = iter->data;
    ALP_TMTL("Card remove, launch '%s', argument '%s'", l[0], l[1]);

      if (!sRemoveBundleLaunch)
        sRemoveBundleLaunch = g_tree_new_full((GCompareDataFunc)strcmp, NULL, g_free, NULL);
      
      GList * list = g_tree_lookup(sRemoveBundleLaunch, store);
      list = g_list_append(list, iter->data);
      g_tree_replace(sRemoveBundleLaunch, g_strdup(store), list);

  }

  for (iter = d->OnRemoveDelete; iter; iter = g_list_delete_link(iter, iter))
  {
    ALP_TMTL("Card remove, delete '%s'", (char*)iter->data);

      if (!sRemoveBundleName)
        sRemoveBundleName = g_tree_new_full((GCompareDataFunc)strcmp, NULL, g_free, NULL);
      
      GList * list = g_tree_lookup(sRemoveBundleName, store);
      list = g_list_append(list, iter->data);
      g_tree_replace(sRemoveBundleName, g_strdup(store), list);

  }
  

fail:
  XML_ParserFree(parser);
  
  g_free(d);
}

static int add_volume(AlpVolumeSvcsFSinfo * e)
{
        
        char path[PATH_MAX];
        sprintf(path, "%s%s", e->fs_path, ALP_EXT_SYSPATH_BUNDLES);
        
        struct stat stbuf;
        
        char storage_str[20] = { 0 };
        int ret = -1;
        uint32_t	start_prc_creator;
        
        if (gGHost) 
        {
			if (ghost_start() == ALP_STATUS_OK)
			{
				int storage_value = -1, auto_start;
				
				ret = alp_gras_volume_mount(GSESSION, e->fs_path, 0, &storage_value, &start_prc_creator);
				
				sprintf(storage_str, "%d", storage_value);
				if (ret == ALP_STATUS_OK)
					ret = alp_gras_settings_retrieve_autostart(&auto_start);
				if (ret == ALP_STATUS_OK && start_prc_creator != 0 && auto_start)
				{
					Alp68KDBInfo	db_info;
					
				    printf("Volume %d mounted. start.prc was available. Getting info.\n", storage_value);
				    
					ret = alp_gras_db_info(GSESSION, 0, 'appl', start_prc_creator, 0, &db_info);
					if (ret == ALP_STATUS_OK)
					{
						char 	bundle_name[64], creator_str[5];

						// Stop any current session right now to start the new one.
						ghost_stop(1);
						
						// #### Escaping should be done on DBName, here and everywhere else in that file...
					    sprintf(bundle_name, "%s:%d_%s_%s_%s",
					      BUNDLE_SCHEME_NAME_GHOST,
					      0,
					      "appl",
					      int_to_char4(start_prc_creator, creator_str),
					      db_info.name);
						
						launch(bundle_name, ALP_APP_PRIMARY);
					}
					else
					{
						ghost_stop(ret != ALP_STATUS_OK && ALP_STATUS_VALUE(ret) != ALP_STATUS_GRAS_GARNET_ERROR);
						ALP_TMTL("alp_gras_db_info failed %d\n", ret);
					}
				}
				else
					ghost_stop(ret != ALP_STATUS_OK && ALP_STATUS_VALUE(ret) != ALP_STATUS_GRAS_GARNET_ERROR);
/*
				if(ret != ALP_STATUS_OK)
					ALP_TMTL("alp_gras_volume_mount failed %d\n", ret);
				ghost_stop(ret != ALP_STATUS_OK && ALP_STATUS_VALUE(ret) != ALP_STATUS_GRAS_GARNET_ERROR);
*/
			}	
        }
        
        const char * args[] = {
          "bar_folder", path,
          "vs_udi", e->udi,
          "ghost_storage", ret == 0 ? storage_str : NULL,
          "ghost_path", ret == 0 ? e->fs_path : NULL,
          NULL, NULL
        };
        
        ALP_TMTL("Adding store named '%s' from mountpoint '%s'", e->name, e->fs_path);
        alp_prv_bundle_add_store(e->name, args);
        
        alp_prv_bundle_scan_store(e->name, TRUE, 0);
        
        // Look for XML media card control file
        sprintf(path, "%s%s", e->fs_path, ALP_EXT_SYSPATH_CARD_XML);
        
        int fd = open(path, O_RDONLY);
        
        ALP_TMTL("Opening xml '%s', fd %d", path, fd);
        
        if (fd != -1) {
          load_card_xml(e->name, fd);
          
          close(fd);
        }
        
        return ALP_STATUS_OK;
}

static int remove_volume(const char * udi)
{
  gchar * store = NULL;
  int ghost_storage = -1;
  gchar * ghost_path = NULL;
  ALP_TMTL("Volume removed: %s", udi);
  
  S_STMT s;
  S_PREPARE(s, "SELECT store_name,ghost_storage,ghost_path FROM stores WHERE stores.vs_udi = ?1;");
  S_BIND_TEXT(s, 1, udi);
  if (S_STEP_ROW(s)) {
    store = g_strdup(S_COL_TEXT(s, 0));
    ghost_storage = S_COL_ISNULL(s,1) ? -1 : S_COL_INT(s, 1);
    ghost_path = g_strdup(S_COL_TEXT(s,2));
    S_FINALIZE(s);
  } else {
    S_FINALIZE(s);
    ALP_TMTL("Unable to resolve volume udi %s to store name", udi);
    return 0;
  }
  
  alp_prv_bundle_delete_store(store);

  if (ghost_storage != -1 && gGHost) {

    if (ghost_start() == ALP_STATUS_OK) {
        
  	  int ret = alp_gras_volume_umount(GSESSION, ghost_storage, ghost_path);
	
	  if(ret)
		    ALP_TMTL("alp_gras_volume_umount failed %d\n", ret);
	
		ghost_stop(ret != ALP_STATUS_OK && ALP_STATUS_VALUE(ret) != ALP_STATUS_GRAS_GARNET_ERROR);
    }
  }

  // FIXME: ick, consider issues of deleting bundle in use.
  // probably we need a synchronous launch, then a "delete bundle after
  // last close" flag.
  if (sRemoveBundleIndex) {
    GList * list = g_tree_lookup(sRemoveBundleIndex, store);
    for (;list;list=g_list_delete_link(list,list)) {
      AlpBundle b;
      b.value = (int)list->data;
      ALP_TMTL("On removal of store %s, deleting bundle %d", store, b.value);
      alp_bundle_delete(b, 0);
    }
    g_tree_remove(sRemoveBundleIndex, store);
  }

  if (sRemoveBundleName) {
    GList * list = g_tree_lookup(sRemoveBundleName, store);
    for (;list;list=g_list_delete_link(list,list)) {
      GArray * a = alp_bundle_search((char*)list->data, TRUE, NULL, "internal", NULL);
      ALP_TMTL("On removal of store %s, deleting internal bundle named %s", store, (char*)list->data);
      if (a->len == 1) {
        alp_bundle_delete(g_array_index(a, AlpBundle, 0), 0);
      } else {
        ALP_TMTL("Couldn't find bundle to delete");
      }
      g_free(list->data);
    }
    g_tree_remove(sRemoveBundleName, store);
  }

  if (sRemoveBundleLaunch) {
    GList * list = g_tree_lookup(sRemoveBundleLaunch, store);
    for (;list;list=g_list_delete_link(list,list)) {
      char ** l = list->data;
      ALP_TMTL("On removal of store %s, launching '%s' with argument '%s'", store, l[0], l[1]);
      launch(l[0], l[1]);
      g_free(list->data);
    }
    g_tree_remove(sRemoveBundleLaunch, store);
  }

  g_free(store);

  return 0;
} 

struct packet_t {
  gchar * notifyType;
  void * details;
  int length;
  gchar * appID;
  AlpNotifyCallbackProc * proc;
  void  * userData;
};

static gboolean idler(gpointer data)
{
  struct packet_t * p = data;
  
  if (!p) {
    // memory allocation failed?
    ALP_TMTL("NULL idler!");
    return FALSE;
  }
  
  int handled = 0;
  p->proc(p->notifyType, p->details, p->length, &handled, p->appID, p->userData);
  g_free(p->notifyType);
  g_free(p->details);
  g_free(p->appID);
  g_free(p);
  
  return FALSE;
}

static int VolumeAddCallback_at_idle(const char * notifyType, void * details, int length, int * handled, const char * appID, void * userData)
{
  alp_status_t status, as;
  // push global error state
  status = alp_bundle_error();

  char * vol = (char*)details;
  ALP_TMTL("Volume added: %s", vol);
  
  AlpVolumeSvcsFSinfo *e=NULL;

  as = alp_volumesvcs_fsinfo_for_udi( &e, vol );
       
  // its possible that this volume has already been unmounted in which case e will remain NULL
  if ( (as == ALP_STATUS_OK) && (e != NULL) ) {
	add_volume(e);
    alp_volumesvcs_fsinfo_free( e );
  }
   
  //pop
  set_error(status);
  return 0;
}

static int VolumeAddCallback(const char * notifyType, void * details, int length, int * handled, const char * appID, void * userData)
{
  struct packet_t p = { g_strdup(notifyType), g_memdup(details,length), length, g_strdup(appID), VolumeAddCallback_at_idle , userData};
  g_idle_add(idler, g_memdup(&p, sizeof(p)) );
  return 0;
}

static int VolumeRemoveCallback_at_idle(const char * notifyType, void * details, int length, int * handled, const char * appID, void * userData)
{
  alp_status_t status;
  // push global error state
  status = alp_bundle_error();
  char * vol = (char*)details;
  
  int result = remove_volume(vol);
  
  //pop
  set_error(status);
  return result;
}

static int VolumeRemoveCallback(const char * notifyType, void * details, int length, int * handled, const char * appID, void * userData)
{
  struct packet_t p = { g_strdup(notifyType), g_memdup(details,length), length, g_strdup(appID), VolumeRemoveCallback_at_idle, userData};
  g_idle_add(idler, g_memdup(&p, sizeof(p)) );
  return 0;
}

static int GHostDBChangeCallback_at_idle(const char * notifyType, void * details, int length, int * handled, const char * appID, void * userData)
{
  AlpGRASDBChangeEventDetails * change = details;

  alp_status_t status = alp_bundle_error(); //save error status
  
  char type_str[5], creator_str[5];
  
  ALP_TMTL("Ghost DB Change callback: type=%s, creator=%s, name=%s, storage=%d, creation=%d",
    int_to_char4(change->db_type,type_str),
    int_to_char4(change->db_creator,creator_str),
    change->db_name,
    change->storage,
    change->creation);
  
  // For now, assume this applies to storage 0
  if (change->storage == -1)
    change->storage = 0;
  
  if (change->creation) {
    // Database created
    
    char buf[64];
    sprintf(buf, "%s:%d_%s_%s_%s", 
      BUNDLE_SCHEME_NAME_GHOST,
      change->storage,
      int_to_char4(change->db_type, type_str),
      int_to_char4(change->db_creator, creator_str),
      change->db_name);
    
    S_STMT s;
    S_PREPARE(s, "SELECT package_id FROM ghost_metadata WHERE ghost_db_type = ?1 AND ghost_db_creator = ?2 "
                  " AND ghost_db_name = ?3 AND ghost_db_storage = ?4");
    S_BIND_INT(s, 1, change->db_type);
    S_BIND_INT(s, 2, change->db_creator);
    S_BIND_TEXT(s, 3, change->db_name);
    S_BIND_INT(s, 4, change->storage);
    
    if (S_STEP_ROW(s)) {
      // oops, this bundle was already registered. Don't register it again.
      // This is actually a common occurance if the alp_bundle_install()
      // is used.
      AlpBundle index;
      index.value = S_COL_INT(s, 0);
      S_FINALIZE(s);
      
      ALP_TMTL("From GHost DB change, not re-registering bundle %d", index.value);
      
      goto done;
      
    } else {
      S_FINALIZE(s);
    }

    S_PREPARE(s, "SELECT store_name FROM stores WHERE ghost_storage = ?1");
    S_BIND_INT(s, 1, change->storage);
    if (S_STEP_ROW(s)) {
      gchar * store_name = g_strdup(S_COL_TEXT(s, 0));
      S_FINALIZE(s);
      
      ALP_TMTL("From GHost DB change, registering '%s' on store '%s'", buf, store_name);
      AlpBundle result = alp_prv_bundle_register_bundle(buf, BUNDLE_SCHEME_NAME_GHOST, store_name, 0);
      
    } else {
      S_FINALIZE(s);
    }
  } else {
    // Database removed
    
    S_STMT s;
    S_PREPARE(s, "SELECT package_id FROM ghost_metadata WHERE ghost_db_type = ?1 AND ghost_db_creator = ?2 "
                  " AND ghost_db_name = ?3 AND ghost_db_storage = ?4");
    S_BIND_INT(s, 1, change->db_type);
    S_BIND_INT(s, 2, change->db_creator);
    S_BIND_TEXT(s, 3, change->db_name);
    S_BIND_INT(s, 4, change->storage);
    
    if (S_STEP_ROW(s)) {
      AlpBundle index;
      index.value = S_COL_INT(s, 0);
      S_FINALIZE(s);
      
      ALP_TMTL("From GHost DB change, unregistering bundle %d", index.value);
      alp_status_t result = alp_prv_bundle_unregister_bundle(index, 0);
      
    } else {
      S_FINALIZE(s);
    }
    
  }
  
done:

  set_error(status);
  return 0;
}

static int GHostDBChangeCallback(const char * notifyType, void * details, int length, int * handled, const char * appID, void * userData)
{
  struct packet_t p = { g_strdup(notifyType), g_memdup(details,length), length, g_strdup(appID), GHostDBChangeCallback_at_idle, userData };
  g_idle_add(idler, g_memdup(&p, sizeof(p)) );
  return 0;
}

static int GHostPathChangeCallback_at_idle(const char * notifyType, void * details, int length, int * handled, const char * appID, void * userData)
{
  gchar * path = details;

  alp_status_t status = alp_bundle_error(); //save error status
  
  ALP_TMTL("Ghost Path change callback: path = %s", path);  
  
  if (path && strlen(path) > 1) {

    S_STMT s;
    S_PREPARE(s, "SELECT store_name FROM stores WHERE ghost_path NOTNULL AND "
                  " substr(?1, 1, length(ghost_path)) = ghost_path");
    S_BIND_TEXT(s, 1, path);
    
    if (S_STEP_ROW(s)) {
      gchar * store_name = g_strdup(S_COL_TEXT(s, 0));
      S_FINALIZE(s);
      
      ALP_TMTL("FIXME: scan store on path change not implemented");
      
    } else {
      S_FINALIZE(s);
    }
  }

  set_error(status);
  return 0;
}

static int GHostPathChangeCallback(const char * notifyType, void * details, int length, int * handled, const char * appID, void *userData)
{
  struct packet_t p = { g_strdup(notifyType), g_memdup(details,length), length, g_strdup(appID), GHostPathChangeCallback_at_idle, userData };
  g_idle_add(idler, g_memdup(&p, sizeof(p)) );
  return 0;
}

static int GHostDBEnvelopChangeCallback_at_idle(const char * notifyType, void * details, int length, int * handled, const char * appID, void * userData)
{
  AlpGRASDBEnvelopChangeEventDetails * change = details;

  alp_status_t status = alp_bundle_error(); //save error status

  gchar * store_name = NULL;
  
  const int debug = 0;
  
  // Find bundle manager store mapped to GHost internal storage, if any.
  S_STMT s;
  S_PREPARE(s, "SELECT store_name FROM stores WHERE stores.ghost_storage NOTNULL AND stores.ghost_storage = 0;");
  S_BEGIN_SELECT(s) {
    g_free(store_name);
    store_name = g_strdup(S_COL_TEXT(s, 0));
  }
  S_END_SELECT(s);
  
  if (!store_name)
    goto done;
  
  // For any items which aren't changed, make sure we know the new is the same as the old
  if (!(change->changes & ALP_GRAS_DBENVELOP_NAME))
    strcpy(change->name, change->old_name);
  if (!(change->changes & ALP_GRAS_DBENVELOP_TYPE))
    change->type = change->old_type;
  if (!(change->changes & ALP_GRAS_DBENVELOP_CREATOR))
    change->creator = change->old_creator;
  if (!(change->changes & ALP_GRAS_DBENVELOP_ATTRIBUTES))
    change->attributes = change->old_attributes;
    
  int wasRegistered;
  int doUnregister;
  int shouldBeRegistered;

  if (debug) {
    char a[5], b[5], c[5], d[5];
    ALP_TMTL("DBEnvelop change, name {%s}<=>{%s}, type %s<=>%s, creator %s<=>%s, attr %x<=>%x", 
      change->old_name, change->name,
      int_to_char4(change->old_type, a), int_to_char4(change->type, b),
      int_to_char4(change->old_creator, c), int_to_char4(change->creator, d),
      change->old_attributes, change->attributes
      );
  }
  
  // Databases should only be registered if they aren't hidden, are resource databases, and have type 'appl'.

  // If any vital specs changed (type, creator, name) we need to fix our bundle. We cannot alter a bundle after
  // it has been registered, so we unregister it, then reregister it again. This _will_ change the bundle index.
  
  // three booleans: was it likely to already be registered. Should it be unregistered in any case. and, should
  // it still be registered when we're done.
  wasRegistered = (!(change->old_attributes & dmHdrAttrHidden)) &&
                   (change->old_attributes & dmHdrAttrResDB) &&
                    (change->old_type == char4_to_int("appl"));
  doUnregister = ((change->changes & (ALP_GRAS_DBENVELOP_NAME|ALP_GRAS_DBENVELOP_TYPE|ALP_GRAS_DBENVELOP_CREATOR)) != 0) ||
                  ((change->old_attributes & dmHdrAttrHidden) != (change->attributes & dmHdrAttrHidden)) ||
                  ((change->old_attributes & dmHdrAttrResDB) != (change->attributes & dmHdrAttrResDB));
  shouldBeRegistered = (!(change->attributes & dmHdrAttrHidden)) &&
                       (change->attributes & dmHdrAttrResDB) &&
                        (change->type == char4_to_int("appl"));

  if (debug)
    ALP_TMTL("wasRegistered = %d, doUnregister = %d, shouldBeRegistered = %d", wasRegistered, doUnregister, 
      shouldBeRegistered);
  
  if (wasRegistered && doUnregister) {
    char bundle_name[PATH_MAX];
    char type_str[5], creator_str[5];
    sprintf(bundle_name, "%s:%d_%s_%s_%s",
      BUNDLE_SCHEME_NAME_GHOST,
      0,
      int_to_char4(change->old_type, type_str),
      int_to_char4(change->old_creator, creator_str),
      change->old_name);
    AlpBundle index = alp_bundle_by_name(bundle_name);
    if (!alp_bundle_is_NULL(index)) {
      if (debug) ALP_TMTL("Going to unregister %s", bundle_name);
      alp_status_t result = alp_prv_bundle_unregister_bundle(index, ALP_PRV_BUNDLE_FLAG_FORCE_UNREGISTER);
      if (debug) ALP_TMTL("Unregistered %s, result = %d", bundle_name, result);
    } else {
      if (debug) ALP_TMTL("Unable to unregister %s, as it wasn't there", bundle_name);
    }
  }
  
  if ((!wasRegistered || doUnregister) && shouldBeRegistered) {
    char bundle_name[PATH_MAX];
    char type_str[5], creator_str[5];
    sprintf(bundle_name, "%s:%d_%s_%s_%s",
      BUNDLE_SCHEME_NAME_GHOST,
      0,
      int_to_char4(change->type, type_str),
      int_to_char4(change->creator, creator_str),
      change->name);
    if (debug) ALP_TMTL("Going to register %s", bundle_name);
    AlpBundle index = alp_prv_bundle_register_bundle(bundle_name, BUNDLE_SCHEME_NAME_GHOST, store_name, 0);
    if (debug) ALP_TMTL("Registered %s, index = %d", bundle_name, index.value);
  }
  
  g_free(store_name);

done:
  
  set_error(status);
  return 0;
}

static int GHostDBEnvelopChangeCallback(const char * notifyType, void * details, int length, int * handled, const char * appID, void * userData)
{
  struct packet_t p = { g_strdup(notifyType), g_memdup(details,length), length, g_strdup(appID), GHostDBEnvelopChangeCallback_at_idle, userData };
  g_idle_add(idler, g_memdup(&p, sizeof(p)) );
  return 0;
}


// This is invoked if GHost needs to reformat it's internal storage, meaning that any information about
// the databases in the internal store will be discarded.
static int GHostStorageRemovedCallback_at_idle(const char * notifyType, void * details, int length, int * handled, const char * appID, void * userData)
{
  alp_status_t status = alp_bundle_error(); //save error status

  gchar * store_name = NULL;
  int store_id = -1;
  
  // Find bundle manager store mapped to GHost internal storage, if any.
  S_STMT s;
  S_PREPARE(s, "SELECT store_name,store_id FROM stores WHERE stores.ghost_storage NOTNULL AND stores.ghost_storage = 0;");
  S_BEGIN_SELECT(s) {
    g_free(store_name);
    store_name = g_strdup(S_COL_TEXT(s, 0));
    store_id = S_COL_INT(s,1);
  }
  S_END_SELECT(s);
  

  if (store_name != NULL) {
    // Rescan only GHost portion of that store
  
    ALP_TMTL("Resyncing store %s due to removed ghost storage", store_name);
    alp_status_t result = alp_prv_bundle_scan_store(store_name, FALSE, ~(1<<BUNDLE_SCHEME_GHOST)); // skip anything NOT ghost
  } 
  
  g_free(store_name); 

  set_error(status);
  return 0;
}

static int GHostStorageRemovedCallback(const char * notifyType, void * details, int length, int * handled, const char * appID, void *userData)
{
  struct packet_t p = { g_strdup(notifyType), g_memdup(details,length), length, g_strdup(appID), GHostStorageRemovedCallback_at_idle, userData };
  g_idle_add(idler, g_memdup(&p, sizeof(p)) );
  return 0;
}

alp_status_t register_volsvc_hooks()
{
  alp_notify_register_server(ALP_NOTIFY_EVENT_VOLUME_ADDED, VolumeAddCallback, NULL, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_DONTSAVE);
  alp_notify_register_server(ALP_NOTIFY_EVENT_VOLUME_REMOVED, VolumeRemoveCallback, NULL, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_DONTSAVE);
  
  if (gGHost) {
    alp_notify_register_server(ALP_NOTIFY_EVENT_GRAS_DB_CHANGE, GHostDBChangeCallback, NULL, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_DONTSAVE);
    alp_notify_register_server(ALP_NOTIFY_EVENT_GRAS_PATH_CHANGE, GHostPathChangeCallback, NULL, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_DONTSAVE);
    alp_notify_register_server(ALP_NOTIFY_EVENT_GRAS_STORAGE_REMOVED, GHostStorageRemovedCallback, NULL, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_DONTSAVE);
    alp_notify_register_server(ALP_NOTIFY_EVENT_GRAS_DBENVELOP_CHANGE, GHostDBEnvelopChangeCallback, NULL, ALP_NOTIFY_PRIORITY_NORMAL, ALP_NOTIFY_REGISTER_FLAG_DONTSAVE);
  }
  
  return 0;
}


alp_status_t alp_prv_bundle_rescan_volumes()
{
  GArray * fsinfo_array = NULL;

  alp_volumesvcs_fsinfo_array( &fsinfo_array);
  
  if (!fsinfo_array) {
    ALP_TMTL("Rescan volumes unable to get volumes list from volume services");
    return ALP_STATUS_BUNDLEMGR_FAILURE;
  }

  AlpVolumeSvcsFSinfo *e;
  
  const int debug = 0;
  
  S_EXEC("BEGIN");
  S_EXEC("CREATE TEMPORARY TABLE present_volumes(vs_udi TEXT, vs_idx INTEGER)");
  S_STMT s;
  S_PREPARE(s, "INSERT INTO present_volumes VALUES(?1, ?2)");
       
  unsigned int i;
  for (i=0; i<fsinfo_array->len; i++) {
      e = &g_array_index( fsinfo_array, AlpVolumeSvcsFSinfo, i);
      
      if (debug)
        ALP_TMTL("Current volumes, %d: %s", i, e->udi);
        
      if (e->fs_path) { // unmounted volumes can be present but not have an fs_path: ignore them

        S_BIND_TEXT(s, 1, e->udi);
        S_BIND_INT(s, 2, i);
        S_STEP_ROW(s);
        S_RESET(s);
      }

  }
  
  S_FINALIZE(s);
  
  GList * add_list = NULL;
  GList * remove_list = NULL;

  S_PREPARE(s, "SELECT vs_udi FROM stores WHERE stores.vs_udi NOTNULL AND stores.vs_udi NOT IN (SELECT vs_udi FROM present_volumes)");
  S_BEGIN_SELECT(s) {
    if (debug)
      ALP_TMTL("Removing volume with UDI %s", S_COL_TEXT(s,0));
    remove_list = g_list_prepend(remove_list, g_strdup(S_COL_TEXT(s, 0)));
  }
  S_END_SELECT(s);

  S_PREPARE(s, "SELECT vs_idx FROM present_volumes WHERE present_volumes.vs_udi NOT IN (SELECT vs_udi FROM stores WHERE vs_udi NOTNULL)");
  S_BEGIN_SELECT(s) {
    if (debug)
      ALP_TMTL("Adding volume with UDI index %d", S_COL_INT(s,0));
    add_list = g_list_prepend(add_list, (gpointer)S_COL_INT(s, 0));
  }
  S_END_SELECT(s);

  S_EXEC("DROP TABLE present_volumes");
  
  S_EXEC("COMMIT");

  for (remove_list = g_list_reverse(remove_list); 
      remove_list;
      remove_list = g_list_delete_link(remove_list, remove_list)) {
      remove_volume(remove_list->data);
      g_free(remove_list->data);
  }

  for (add_list = g_list_reverse(add_list); 
      add_list;
      add_list = g_list_delete_link(add_list, add_list)) {
      add_volume(&g_array_index(fsinfo_array, AlpVolumeSvcsFSinfo, (int)add_list->data));
  }

  alp_volumesvcs_fsinfo_array_free( fsinfo_array);
   
  return 0;
}
