/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*! @file exgmgr.c 
 *
 * @brief Exchange Manager Library
 *
*/

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <dlfcn.h>
#include <sys/un.h>
#include <sys/socket.h>
#include <errno.h>
#include <fcntl.h>
#include <glib/gmain.h>

#include <hiker/sysclass.h>
#include <hiker/traces.h>
#include <hiker/exgmgr.h>
#include <exgtransport.h>
#include "prvexgmgr.h"
#include <hiker/prv/exgmgr_prv.h>
#include <hiker/bundlemgr.h>

/******************** PRIVATE EXGMGR FUNCTIONS IMPLEMENTATION ********************/

// g_alp_exg_transport_infoP holds the info for all transport libraries
static AlpExgMgrTransportInfoType *g_alp_exg_transport_infoP = NULL;
// g_alp_exg_transport_info_count is the number of transport libraries
static int g_alp_exg_transport_info_count = 0;

static GSourceFuncs g_alp_exg_mgr_async_source_funcs;
static AlpExgClientExecGSource *g_alp_exg_mgr_async_gsource = NULL;
static guint g_alp_exg_mgr_async_source_id;

// These variables are used to transform a synchronous execute into an asynchronous one.
// As there can be only one synchronous call running at a given time, it's safe to use globals.
static int g_synchronous_call_done;
alp_status_t g_synchronous_call_result;

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_enumerate_transport_libs: enumerate transports using bundlemgr and load all the info, but do not open them.
 * If something fails, return an error and nothing is loaded*/
static alp_status_t prv_alp_exg_enumerate_transport_libs ()
{
	guint theArrayIndex;
	alp_status_t theErr;
	
	if (g_alp_exg_transport_infoP)
		return 0;

	(void) alp_bundle_init ();
	/*
	theErr = alp_bundle_init ();
	if (theErr)
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_prv_bundle_init returned error = 0x%lx", theErr));
		return theErr;
	}
	*/
	GArray *theArray = NULL;
	theArray = alp_bundle_search(NULL, 0, NULL, NULL, "exchange");
	if (theArray == NULL)
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_bundle_search returned NULL array"));
		return ALP_EXG_MGR_ERR_NO_SUCH_TRANSPORT_ERROR;
	}
	ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_bundle_search found %d transports", theArray->len));
	
	// if no transports, exit
	if (theArray->len == 0)
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : No transport found, exiting."));
		return ALP_EXG_MGR_ERR_NO_SUCH_TRANSPORT_ERROR;
	}
	
	//	Allocate the transport table
	g_alp_exg_transport_infoP = (AlpExgMgrTransportInfoType *) malloc (theArray->len * sizeof (AlpExgMgrTransportInfoType));
	if (g_alp_exg_transport_infoP == NULL)
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : cannot allocate AlpExgMgrTransportInfoType table"));
		return ALP_EXGMGR_ERR_OUT_OF_MEMORY_ERROR;
	}
	
	// set the table fields to all zeros
	memset(g_alp_exg_transport_infoP, 0, theArray->len * sizeof (AlpExgMgrTransportInfoType));

	// load each transport info. If anything fails, cleanup and return error.
	for (theArrayIndex = 0; theArrayIndex < theArray->len; theArrayIndex++)
	{
		AlpBundle theBundle = g_array_index(theArray, AlpBundle, theArrayIndex);
		g_alp_exg_transport_infoP[g_alp_exg_transport_info_count].itsBundleRef = alp_bundle_open (theBundle);
		gchar *theLibNameP;
		
		// Find the library name from the bundle Manifest, and build the full path
		theLibNameP = alp_bundle_property_value (theBundle, "exchange", 0, "libraryclient");
		//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : Library name = %s", theLibNameP));
		if (theLibNameP)
		{
			g_alp_exg_transport_infoP[g_alp_exg_transport_info_count].itsLibPath = alp_bundle_ref_ro_pathname(g_alp_exg_transport_infoP[g_alp_exg_transport_info_count].itsBundleRef, theLibNameP);
			g_free (theLibNameP);
			//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : Library path = %s", g_alp_exg_transport_infoP[g_alp_exg_transport_info_count].itsLibPath));
		}
		if (g_alp_exg_transport_infoP[g_alp_exg_transport_info_count].itsLibPath == NULL)
			goto cleanExit;
		
		// Get the transport internal nameId
		g_alp_exg_transport_infoP[g_alp_exg_transport_info_count].itsNameId = alp_bundle_property_value (theBundle, "exchange", 0, "nameid");
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_bundle_property_value (nameid) = %s", g_alp_exg_transport_infoP[g_alp_exg_transport_info_count].itsNameId));
		if (g_alp_exg_transport_infoP[g_alp_exg_transport_info_count].itsNameId == NULL)
			goto cleanExit;

		// Count this lib as loaded
		g_alp_exg_transport_info_count++;
	}
	
	g_array_free (theArray, 1);
	return 0;

cleanExit:
	// clean the entries correctly built, and also the one partly built (index = g_alp_exg_transport_info_count) which failed
	g_array_free (theArray, 1);
	for (theArrayIndex = 0; theArrayIndex <= (guint) g_alp_exg_transport_info_count; theArrayIndex++)
	{			
		if (g_alp_exg_transport_infoP[theArrayIndex].itsNameId)
			g_free (g_alp_exg_transport_infoP[theArrayIndex].itsNameId);
			
		if (g_alp_exg_transport_infoP[theArrayIndex].itsLibPath)
			g_free (g_alp_exg_transport_infoP[theArrayIndex].itsLibPath);
		
		if (g_alp_exg_transport_infoP[g_alp_exg_transport_info_count].itsBundleRef != 0)
			alp_bundle_close (g_alp_exg_transport_infoP[g_alp_exg_transport_info_count].itsBundleRef);
	}
	
	free (g_alp_exg_transport_infoP);
	g_alp_exg_transport_infoP = NULL;

	return ALP_EXGMGR_ERR_OUT_OF_MEMORY_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_find_transport_lib: returns a pointer to the AlpExgMgrTransportInfoType for the transport whose params are passed.
 * if the transport is not in the table, it is opened*/
static alp_status_t prv_alp_exg_find_transport_lib (const char *iTransportParams, AlpExgMgrTransportInfoType **oTransportInfo)
{
	int theIndex;
	char theNameId[ALP_EXG_TRANSPORT_NAME_MAX_LEN + 1];
	alp_status_t theErr;

	// if transports not loaded yet, do it now
	if (g_alp_exg_transport_infoP == NULL)
	{
		theErr = prv_alp_exg_enumerate_transport_libs();
		if (theErr)
			return theErr;
	}

	// extract the nameid from the transport params. The nameId is the first word in the transport params (until ';' separator is encountered)
	theIndex = 0;
	while ((theNameId[theIndex] = iTransportParams[theIndex]) != ';')
	{
		theIndex++;
		if (theIndex == ALP_EXG_TRANSPORT_NAME_MAX_LEN)
			return ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR;
	}
	theNameId[theIndex] = 0;

	//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : Looking for Transport %s.", theNameId));
	
	// try to find the entry with the given transport name
	for (theIndex = 0; theIndex < g_alp_exg_transport_info_count; theIndex++)
		if (strcmp ((char *) g_alp_exg_transport_infoP[theIndex].itsNameId, theNameId) == 0)
		{
			//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : Transport %s found.", theNameId));
			*oTransportInfo = g_alp_exg_transport_infoP + theIndex;
			
			// if this lib has already been used, we're done
			if ((*oTransportInfo)->itsLibRef != 0)
				return ALP_EXGMGR_ERR_NO_ERROR;
			
			// otherwise, open the lib now
			//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : Transport %s not opened. Opening %s.", theNameId, (*oTransportInfo)->itsLibPath));
			(*oTransportInfo)->itsLibRef = dlopen ((*oTransportInfo)->itsLibPath, RTLD_LAZY);
			if ((*oTransportInfo)->itsLibRef == 0)
			{
				ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : Cannot open %s, reason is '%s'", (*oTransportInfo)->itsLibPath, dlerror()));
				return ALP_EXG_MGR_ERR_CANNOT_OPEN_TRANSPORT_LIB_ERROR;
			}
			if (((*oTransportInfo)->itsTransportExecuteRequestFunc = (alp_exg_transport_execute_request_func) dlsym ((*oTransportInfo)->itsLibRef, "alp_exg_transport_execute_request")) == NULL)
			{
				ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : Cannot get 'alp_exg_transport_execute_request' function pointer."));
				(void) dlclose ((*oTransportInfo)->itsLibRef);
				(*oTransportInfo)->itsLibRef = 0;
				return ALP_EXG_MGR_ERR_CANNOT_GET_TRANSPORT_FUNC_ERROR;
			}
			return ALP_EXGMGR_ERR_NO_ERROR;
		}
		
	return ALP_EXG_MGR_ERR_NO_SUCH_TRANSPORT_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_check_parameter_buffer: check there is at least iNeededSpace available in Info field and grow the buffer as necessary */
static alp_status_t prv_alp_exg_check_parameter_buffer (AlpExgRequestType *iRequestP, size_t iNeededSpace)
{
	size_t theAllocSize = 0;
	if (iRequestP->itsInfoLen + iNeededSpace > iRequestP->itsInfoSize)
	{
		void *theP;
		theAllocSize = (((iRequestP->itsInfoLen + iNeededSpace - 1) / ALP_EXGMGR_ITSINFO_GROW_SIZE) + 1) * ALP_EXGMGR_ITSINFO_GROW_SIZE;
		theP = realloc (iRequestP->itsInfo, theAllocSize);
		if (theP == NULL)
			return ALP_EXGMGR_ERR_OUT_OF_MEMORY_ERROR;
		//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : prv_alp_exg_check_parameter_buffer : reallocated Info, oldsize = %ld, used = %ld, needed = %ld, newsize = %ld",
		//																			iRequestP->itsInfoSize, iRequestP->itsInfoLen, iNeededSpace, theAllocSize));
		iRequestP->itsInfo = (char *) theP;
		iRequestP->itsInfoSize = theAllocSize;
	}

	return ALP_EXGMGR_ERR_NO_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_find_parameter: returns the offset to the paramtype if found */
static alp_status_t prv_alp_exg_find_parameter (char *iParameterBuf, char iParameterType, char *iParameterTag, int *oTagOffset)
{
	// It is OK to not have the buffer allocated.
	if (iParameterBuf == NULL)
		return ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR;
		
	// go through each parameter and see if it's the one we want
	// format is Xtag\0value\0Xtag\0value\0\0
	*oTagOffset = 0;
	while (*iParameterBuf != 0)
	{
		if (*iParameterBuf == iParameterType)
			if (strcmp (iParameterBuf + 1, iParameterTag) == 0)
				return ALP_EXGMGR_ERR_NO_ERROR;

		// bad type or param. skip tag
		while (*iParameterBuf++ != 0)
			*oTagOffset += 1;
		*oTagOffset += 1;
		
		// skip value
		while (*iParameterBuf++ != 0)
			*oTagOffset += 1;
		*oTagOffset += 1;
	}
	// we reached the end
	return ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_remove_parameter */
static alp_status_t prv_alp_exg_remove_parameter (AlpExgRequestType *iRequestP, char iParameterType, char *iParameterTag)
{	
	int theOffset;
	int theStartOffset;
	alp_status_t theErr;
	
	// It is OK to not have the Info buffer allocated.
	if (iRequestP->itsInfo == NULL)
		return ALP_EXGMGR_ERR_NO_ERROR;
		
	// Try to find the parameter.
	theErr = prv_alp_exg_find_parameter (iRequestP->itsInfo, iParameterType, iParameterTag, &theOffset);
	if (theErr == ALP_EXGMGR_ERR_NO_ERROR)
	{
		//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : prv_alp_exg_remove_parameter : prv_alp_exg_find_parameter: returned offset = %ld (len = %ld)", theOffset, iRequestP->itsInfoLen));
		//ALP_TM(B(ALP_CLASS_EXCHANGE, iRequestP->itsInfo, iRequestP->itsInfoLen));
		// remove it. theOffset points to the paramtype character. 
		theStartOffset = theOffset;
		// count the tag len
		theOffset += (strlen (iRequestP->itsInfo + theOffset) + 1);
		// count the value len
		theOffset += (strlen (iRequestP->itsInfo + theOffset) + 1);
		memmove (iRequestP->itsInfo + theStartOffset, iRequestP->itsInfo + theOffset, iRequestP->itsInfoLen - theOffset - 1);
		iRequestP->itsInfoLen -= (theOffset - theStartOffset);
		return ALP_EXGMGR_ERR_NO_ERROR;
	}
	else if (theErr == ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR)
	{
		// OK, nothing to remove
		return ALP_EXGMGR_ERR_NO_ERROR;
	}
	else
		return theErr;
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_init_parameter_space: iParameterLen is the actual data len */
static alp_status_t prv_alp_exg_init_parameter_space (AlpExgRequestType *iRequestP, char iParameterType, char *iParameterTag, size_t iParameterLen, char **oParameterValue)
{
	// This function adds the parameter tag into the request and reserves the space to put the data. The return pointer
	// oParameterValue tells where to put the data. The function guarantees that there will be iParameterLen + 1 bytes of space at this address.
	alp_status_t theErr;
	size_t theNeededSpace;
	size_t theTagLen;
	
	// remove the parameter if it already exists
	theErr = prv_alp_exg_remove_parameter (iRequestP, iParameterType, iParameterTag);
	if (theErr)
		return theErr;
		
	// calculate the needed space : type char + param tag + null + param value + null
	theTagLen = strlen (iParameterTag);
	theNeededSpace = 1 + theTagLen + 1 + iParameterLen + 1;

	// make sure we have an Info buffer with enough space
	theErr = prv_alp_exg_check_parameter_buffer (iRequestP, theNeededSpace);
	if (theErr)
		return theErr;

	//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : prv_alp_exg_init_parameter_space : before insert: infoLen = %d", iRequestP->itsInfoLen));
	//ALP_TM(B(ALP_CLASS_EXCHANGE, iRequestP->itsInfo, iRequestP->itsInfoLen));

	// insert the parameter at the end, right before the final null
	iRequestP->itsInfo[iRequestP->itsInfoLen - 1] = iParameterType;
	memcpy (iRequestP->itsInfo + iRequestP->itsInfoLen, iParameterTag, theTagLen + 1);
	*oParameterValue = iRequestP->itsInfo + iRequestP->itsInfoLen + theTagLen + 1;

	iRequestP->itsInfoLen += theNeededSpace;
	// Set the final NULL that marks the end of parameters.
	iRequestP->itsInfo[iRequestP->itsInfoLen - 1] = 0;
	//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : prv_alp_exg_init_parameter_space : after insert: infoLen = %d", iRequestP->itsInfoLen));
	//ALP_TM(B(ALP_CLASS_EXCHANGE, iRequestP->itsInfo, iRequestP->itsInfoLen));

	return ALP_EXGMGR_ERR_NO_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_set_string_parameter */
static alp_status_t prv_alp_exg_set_parameter (AlpExgRequestType *iRequestP, char iParameterType, char *iParameterTag, char *iParameterValue)
{
	alp_status_t theErr;
	char *theParameterValueP;
	size_t theValueLen;

	// Set the parameter tag and reserve space to copy the parameter value in it.
	theValueLen = strlen (iParameterValue);
	theErr = prv_alp_exg_init_parameter_space (iRequestP, iParameterType, iParameterTag, theValueLen, &theParameterValueP);
	if (theErr)
		return theErr;
	
	// OK, we have space to copy the string and its final NULL.
	strcpy (theParameterValueP, iParameterValue);

	return ALP_EXGMGR_ERR_NO_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_get_parameter */
static alp_status_t prv_alp_exg_get_parameter (AlpExgRequestType *iRequestP, char iParameterType, char *iParameterTag, char *oParameterValue, size_t iParameterValueBufLen)
{
	alp_status_t theErr;
	int theOffset;
	size_t theValueLen;
	char *theInfo;
	
	// parameters can be retrieved either from the RcvInfo (for received request only), or from the normal Info.	
	theInfo = iRequestP->itsRcvInfo;
	theErr = prv_alp_exg_find_parameter (theInfo, iParameterType, iParameterTag, &theOffset);
	if (theErr == ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR)
	{
		theInfo = iRequestP->itsInfo;
		theErr = prv_alp_exg_find_parameter (theInfo, iParameterType, iParameterTag, &theOffset);
	}
	
	if (theErr == ALP_EXGMGR_ERR_NO_ERROR)
	{
		// theOffset points to the paramtype character. jump to the value
		theOffset += (strlen (theInfo + theOffset) + 1);
		theValueLen = strlen (theInfo + theOffset);
		if (theValueLen + 1 > iParameterValueBufLen)
			return ALP_EXGMGR_ERR_BUF_TOO_SMALL_ERROR;
		memcpy (oParameterValue, theInfo + theOffset, theValueLen + 1);
	}
	
	return theErr;
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_connect_to_daemon */
static alp_status_t prv_alp_exg_connect_to_daemon (int *oFd)
{
	struct sockaddr_un 	theAddr;
	
	memset(&theAddr, 0, sizeof(theAddr));
	theAddr.sun_family	= AF_UNIX;
	strcpy (theAddr.sun_path, ALP_EXG_DAEMON_SOCKET_NAME);

	// Create the socket
	if ((*oFd = socket(AF_UNIX, SOCK_STREAM, 0)) == -1)
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : prv_alp_exg_connect_to_daemon: Socket failed with error (0x%08lX).", errno));
		return ALP_EXG_MGR_ERR_SOCKET_ERROR;
	}
	
	// Connect the socket
	if (connect(*oFd, (struct sockaddr*) &theAddr, strlen(theAddr.sun_path) + sizeof(theAddr.sun_family)) == -1)
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : prv_alp_exg_connect_to_daemon: Connect failed with error (0x%08lX).", errno));
		(void) close(*oFd);
		return ALP_EXG_MGR_ERR_CONNECT_ERROR;
	}

	//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : prv_alp_exg_connect_to_daemon: Socket (%ld) created and connected", *oFd));

	return ALP_EXGMGR_ERR_NO_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_send_message_to_daemon_with_reply */
static alp_status_t prv_alp_exg_send_message_to_daemon_with_reply (AlpExgMgrdMessageType *iMessage, AlpExgMgrdMessageReplyType *oMessageReply)
{
	// Connect to the daemon, send the message, wait for the reply status, disconnect and return it.
	alp_status_t theErr;
	int theFd;
	
	theErr = prv_alp_exg_connect_to_daemon (&theFd);
	if (theErr)
		return theErr;
		
	theErr = alp_exg_prv_sync_full_write(theFd, iMessage, sizeof(AlpExgMgrdMessageType));
	if (theErr)
		goto exitClose;

	// daemon replies with a AlpExgMgrdMessageReplyType.
	theErr = alp_exg_prv_sync_full_read(theFd, oMessageReply, sizeof(AlpExgMgrdMessageReplyType));
		
exitClose:
	close (theFd);
	return theErr;
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_send_message_to_daemon */
static alp_status_t prv_alp_exg_send_message_to_daemon (AlpExgMgrdMessageType *iMessage)
{
	// Connect to the daemon, send the message, wait for the reply status, disconnect and return it.
	alp_status_t theErr;
	int theFd;
	AlpExgMgrdMessageReplyType theMsgReply;
	
	theErr = prv_alp_exg_send_message_to_daemon_with_reply (iMessage, &theMsgReply);
	if (theErr == 0)
		theErr = theMsgReply.itsReplyStatus;
		
	return theErr;
}

/******************** RESERVED EXGMGR FUNCTIONS IMPLEMENTATION ********************
 * These functions are for use by the transports and daemon only. */
 
void alp_exg_request_prv_get_version (AlpExgRequest iRequest, int *oVersion)
{
	AlpExgRequestType *theRequestP = (AlpExgRequestType *) iRequest;
	*oVersion = theRequestP->itsExgVersion;
}

void alp_ext_request_prv_get_info (AlpExgRequest iRequest, char **oInfo, size_t *oInfoLen)
{
	AlpExgRequestType *theRequestP = (AlpExgRequestType *) iRequest;
	if (oInfo)
		*oInfo = theRequestP->itsInfo;
	if (oInfoLen)
		*oInfoLen = theRequestP->itsInfoLen;
}

/* alp_exg_request_prv_set_info is the request info field setter, for use by the transport or daemon.
 * If iInfo is NULL, the content of the request info field is cleared, but the field stays allocated.
 * if iInfo is not NULL, it must be a malloc-ed chunck.
 *    if it is the currently allocated itsInfo, then only the used len is set
 *    if it is a different chunck, then it is stored in place of the original itsInfo.
 *    iInfoLen must be <= the size of this chunck */
void alp_exg_request_prv_set_info (AlpExgRequest iRequest, char *iInfo, size_t iInfoLen)
{
	AlpExgRequestType *theRequestP = (AlpExgRequestType *) iRequest;
	
	if (iInfo == NULL)
	{
		// clear content of Info field
		if (theRequestP->itsInfo != NULL)
		{
			*(theRequestP->itsInfo) = 0;
			theRequestP->itsInfoLen = 1;
		}
	}
	else
	{
		// if iInfo is the buffer in the request, just set the len
		if (iInfo == theRequestP->itsInfo)
			theRequestP->itsInfoLen = iInfoLen;
		else
		{
			// replace the Info field in the request with iInfo.
			if (theRequestP->itsInfo != NULL)
				free (theRequestP->itsInfo);
			theRequestP->itsInfo = iInfo;
			theRequestP->itsInfoLen = iInfoLen;
			theRequestP->itsInfoSize = iInfoLen;
		}
	}
}

/* alp_exg_request_prv_alloc_info reallocates the info field if necessary to make sure it can hold iInfoLen bytes.
 * if the info buffer is already large enough, nothing is done. The Info field is emptied in all cases */
alp_status_t alp_exg_request_prv_alloc_init_info (AlpExgRequest iRequest, size_t iInfoLen)
{
	AlpExgRequestType *theRequestP = (AlpExgRequestType *) iRequest;
	
	// iInfoLen cannot be zero because there is always at least one zero in the buffer.
	if (iInfoLen > 0 && theRequestP->itsInfoSize < iInfoLen)
	{
		// existing chunck too small. Allocate a new one.
		void *theP = malloc (iInfoLen);
		if (theP == NULL)
			return ALP_EXGMGR_ERR_OUT_OF_MEMORY_ERROR;
		
		if (theRequestP->itsInfo)
			free (theRequestP->itsInfo);
			
		theRequestP->itsInfo = (char *) theP;
		theRequestP->itsInfoSize = iInfoLen;
	}
	
	theRequestP->itsInfo[0] = 0;
	theRequestP->itsInfoLen = 1;
	
	return ALP_EXGMGR_ERR_NO_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_prv_sync_full_write 
 * this function loops writing until everything has been sent or an error occurs */
alp_status_t alp_exg_prv_sync_full_write (int iSocket, void *iBuf, size_t iBufLen)
{
	size_t theLenDone = 0;
	while (iBufLen)
	{
		ssize_t theLenSent = write (iSocket, ((char *) iBuf) + theLenDone, iBufLen - theLenDone);
		if (theLenSent < 0)
		{
			ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_local_sync_full_write: cannot write to socket %d, error = %s", iSocket, sys_errlist[errno]));
			return ALP_EXG_MGR_ERR_TRANSPORT_WRITE_ERROR;
		}
		theLenDone += theLenSent;
		iBufLen -= theLenSent;
	}

	return ALP_EXGMGR_ERR_NO_ERROR;	
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_prv_sync_full_read 
 * this function loops reading until everything has been received or an error occurs */
alp_status_t alp_exg_prv_sync_full_read (int iSocket, void *iBuf, size_t iBufLen)
{
	size_t theLenDone = 0;
	while (iBufLen)
	{
		ssize_t theLenRcvd = read (iSocket, ((char *) iBuf) + theLenDone, iBufLen - theLenDone);
		if (theLenRcvd < 0)
		{
			ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_local_sync_full_read: cannot read from socket %d, error = %s", iSocket, sys_errlist[errno]));
			return ALP_EXG_MGR_ERR_TRANSPORT_READ_ERROR;
		}
		theLenDone += theLenRcvd;
		iBufLen -= theLenRcvd;
	}

	return ALP_EXGMGR_ERR_NO_ERROR;	
}

typedef struct msghdr   MsgHdr;
typedef struct cmsghdr  CMsgHdr;
typedef struct iovec    IOVec;
typedef union                   /* force alignment of trailing data */
{
    CMsgHdr     cm;
    char        control[ CMSG_SPACE( sizeof( int ) ) ];
} CMsgHdrUnion; 

ssize_t alp_exg_prv_read_fd(int sockFD, void* payload, size_t maxPL, int* recvFD)
{
    MsgHdr          msg;    /* recvmsg() message buffer                     */
    IOVec           iov[1]; /* Scatter-gather payload vector                */
    ssize_t         nPL;    /* Size of actual payload received              */
    CMsgHdrUnion    controlX;

    CMsgHdr*        cmptr;

    msg.msg_name        = NULL; /* name not used for stream sockets         */
    msg.msg_namelen     = 0;

    iov[0].iov_base     = payload;  /* set up scatter-gather                */
    iov[0].iov_len      = maxPL;
    msg.msg_iov         = iov;
    msg.msg_iovlen      = 1;

    /*
     *  XXX:  Older Unixen may use msg_accrights instead of msg_control.
     */
    msg.msg_control     = controlX.control;
    msg.msg_controllen  = sizeof( controlX.control );

    /*
     *  XXX:  Stevens does'nt pre-initialize cm, but RH 5.2 acts flaky
     *        without it.
     */
    cmptr               = CMSG_FIRSTHDR( &msg );
    cmptr->cmsg_len     = CMSG_LEN( sizeof( int ) );
    cmptr->cmsg_level   = SOL_SOCKET;
    cmptr->cmsg_type    = SCM_RIGHTS;

    *( (int*) CMSG_DATA( cmptr ) ) = -1;

	while (1)
	{
		nPL = recvmsg( sockFD, &msg, 0 );

		if ( nPL <= 0 )
		{
			if ((errno == EAGAIN) || (errno == EWOULDBLOCK))
			{
				ALP_TM(TL(ALP_CLASS_EXCHANGE, "read_fd: wait a few..."));
				usleep(500);
				continue;
			}

			ALP_TM(TL(ALP_CLASS_EXCHANGE, "read_fd: recvmsg failed with error (0x%08lX).", errno));
			return( nPL );
		}

		break;
	}

    /*
     *  XXX:    Might realloc, so reseat?
     */
    cmptr = CMSG_FIRSTHDR( &msg );

    if ( cmptr != NULL
      && cmptr->cmsg_len    == CMSG_LEN( sizeof( int ) )
      && cmptr->cmsg_level  == SOL_SOCKET
      && cmptr->cmsg_type   == SCM_RIGHTS
       )
        *recvFD = *( (int*) CMSG_DATA( cmptr ) );
    else
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "read_fd: descriptor was not passed."));
        *recvFD = -1;       /* descriptor was not passed */
	}

    return( nPL );
}

ssize_t alp_exg_prv_write_fd(int sockFD, void* payload, size_t nPL, int sendFD)
{
    MsgHdr          msg;    /* sendmsg() buffer                             */
    IOVec           iov[1]; /* scatter-gather payload vector                */

    CMsgHdrUnion    controlX;

    CMsgHdr*        cmptr;

    msg.msg_name        = NULL;     /* name not used for stream sockets.    */
    msg.msg_namelen     = 0;

    iov[0].iov_base     = payload;  /* set up scatter-gather.               */
    iov[0].iov_len      = nPL;
    msg.msg_iov         = iov;
    msg.msg_iovlen      = 1;

    /*
     *  XXX:  Older Unixen may use msg_accrights instead of msg_control.
     */
    msg.msg_control     = controlX.control;
    msg.msg_controllen  = sizeof( controlX.control );

    cmptr               = CMSG_FIRSTHDR( &msg );
    cmptr->cmsg_len     = CMSG_LEN( sizeof( int ) );
    cmptr->cmsg_level   = SOL_SOCKET;
    cmptr->cmsg_type    = SCM_RIGHTS;

    *( (int*) CMSG_DATA( cmptr ) ) = sendFD;

    return( sendmsg( sockFD, &msg, 0 ) );
}

char *alp_exg_make_tmpfile (char *iPrefix, int *oFd)
{
	size_t thePrefixLen = strlen(iPrefix);
	char *theFilename = (char *) malloc (thePrefixLen + 15);
	if (theFilename == NULL)
		return NULL;
	strcpy (theFilename, iPrefix);
	int theTryCount = 20;
	while (theTryCount--)
	{
		long int theRandom = random();
		sprintf (theFilename + thePrefixLen, "%lx", theRandom);
		*oFd = open (theFilename, O_RDWR | O_CREAT | O_EXCL, 0640);
		if (*oFd >= 0)
			return theFilename;
	}
	free (theFilename);
	return NULL;
}

/* alp_exg_irda_prv_fd_to_file
 * creates a tmp file, and copy all the data from iFd into that file. Close iFd and return the filename in a malloced string
 */
alp_status_t alp_exg_prv_fd_to_file (int iDataFd, char *iPrefix, char **oFilename)
{
	ssize_t theLen;
	int theFd;
	uint8_t theBuf[1024];
	
	*oFilename = alp_exg_make_tmpfile(iPrefix, &theFd);
	if (*oFilename == NULL)
		return ALP_EXGMGR_ERR_CANNOT_CREATE_TMP_FILE_ERROR;

	do
	{
		alp_status_t theErr;
		
		theLen = read (iDataFd, theBuf, sizeof(theBuf));
		if (theLen < 0)
		{
			ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_prv_fd_to_file: read error on data fd, err = %s.", sys_errlist[errno]));
			close (iDataFd);
			close (theFd);
			unlink (*oFilename);
			free (*oFilename);
			*oFilename = NULL;
			return ALP_EXG_MGR_ERR_WRITE_ERROR;
		}
		
		if (theLen > 0)
		{
			theErr = alp_exg_prv_sync_full_write (theFd, theBuf, theLen);
			if (theErr)
			{
				ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_prv_fd_to_file: write error on tmp file fd, err = 0x%lx.", theErr));
				close (iDataFd);
				close (theFd);
				unlink (*oFilename);
				free (*oFilename);
				*oFilename = NULL;
				return theErr;
			}
		}
	} while (theLen > 0);
	
	// close tmp file
	close (theFd);
	// close the request file data descriptor
	close (iDataFd);
	
	return 0;
}

int alp_exg_prv_daemon_is_alive ()
{
	AlpExgMgrdMessageType theMessage;
	
	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_IS_ALIVE;
	if (prv_alp_exg_send_message_to_daemon(&theMessage) == 0)
		return 1;
	
	return 0;
}

alp_status_t alp_exg_prv_extract_transport_params_item_value (char *iTransportParams, char *iItemName, char *oValue, size_t iValueLen)
{
	// iTransportParams must be a string in the form "transportNameId;itemName1:itemValue1;itemNameN:itemValueN;

	int theIndex;
	int theItemIndex;
	char theItem[20];
	
	// skip the nameid
	theIndex = 0;
	while (iTransportParams[theIndex] != ';' && iTransportParams[theIndex] != 0)
		theIndex++;
	
	// end of string ? not good
	if (iTransportParams[theIndex] == 0)
		return ALP_EXG_MGR_ERR_TRANSPORT_DEST_ITEM_NOT_FOUND_ERROR;
	
	while (iTransportParams[theIndex] != 0)
	{
		// point to first byte of next item
		theIndex++;
		
		// extract next item
		theItemIndex = 0;
		while (iTransportParams[theIndex] != ':' && iTransportParams[theIndex] != 0)
			theItem[theItemIndex++] = iTransportParams[theIndex++];
		theItem[theItemIndex] = 0;
		
		if (strcmp (theItem, iItemName) == 0)
		{
			// point to first byte of value
			theIndex++;
			// get the value
			theItemIndex = 0;
			while (iTransportParams[theIndex] != ';' && iTransportParams[theIndex] != 0)
			{
				if (theItemIndex + 1 == (int) iValueLen)
					return ALP_EXGMGR_ERR_BUF_TOO_SMALL_ERROR;
				oValue[theItemIndex++] = iTransportParams[theIndex++];
			}
			oValue[theItemIndex] = 0;
			return 0;
		}
	}
	return ALP_EXG_MGR_ERR_TRANSPORT_DEST_ITEM_NOT_FOUND_ERROR;
}

/******************** PUBLIC FUNCTIONS IMPLEMENTATION ********************/

/* -------------------------------------------------------------------------------------------
 * alp_exg_handler_register */

alp_status_t alp_exg_handler_register (char *iVerb, char *iLocalizedVerb, char *iMIMEType,
									char *iHandlerBundleName,
									int iLocalOnly, int iSerializeMe, int iMustBeUnique, int iAcceptGenericInvocation)
{
	AlpExgMgrdMessageType theMessage;
	
	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_REGISTER_HANDLER;
	
	if (strlen(iVerb) > ALP_EXGMGR_VERB_MAXLEN)
		return ALP_EXGMGR_ERR_VERB_TOO_LONG_ERROR;
	strcpy (theMessage.params.register_handler_bundle.itsVerb, iVerb);

	if (iLocalizedVerb)
	{
		if (strlen(iLocalizedVerb) > ALP_EXGMGR_LOCALIZED_VERB_MAXLEN)
			return ALP_EXGMGR_ERR_LOCALIZED_VERB_TOO_LONG_ERROR;
		strcpy (theMessage.params.register_handler_bundle.itsLocalizedVerb, iLocalizedVerb);
	}
	else
		theMessage.params.register_handler_bundle.itsLocalizedVerb[0] = 0;
		
	if (strlen(iMIMEType) > ALP_EXGMGR_MIMETYPE_MAXLEN)
		return ALP_EXGMGR_ERR_MIMETYPE_TOO_LONG_ERROR;
	strcpy (theMessage.params.register_handler_bundle.itsMIMEType, iMIMEType);

	if (strlen(iHandlerBundleName) > ALP_EXGMGR_PACKAGENAME_MAXLEN)
		return ALP_EXGMGR_ERR_PACKAGENAME_TOO_LONG_ERROR;
	strcpy (theMessage.params.register_handler_bundle.itsHandlerBundleName, iHandlerBundleName);

	theMessage.params.register_handler_bundle.itsLocalOnly = iLocalOnly;
	theMessage.params.register_handler_bundle.itsSerializeMe = iSerializeMe;
	theMessage.params.register_handler_bundle.itsMustBeUnique = iMustBeUnique;
	theMessage.params.register_handler_bundle.itsAcceptGenericInvocation = iAcceptGenericInvocation;

	return prv_alp_exg_send_message_to_daemon(&theMessage);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_handler_register */

alp_status_t alp_exg_handler_register_notification (char *iVerb, char *iLocalizedVerb, char *iMIMEType,
									char *iNotificationType,
									int iLocalOnly, int iSerializeMe, int iMustBeUnique, int iAcceptGenericInvocation)
{
	AlpExgMgrdMessageType theMessage;
	
	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_REGISTER_HANDLER_NOTIFICATION;
	
	if (strlen(iVerb) > ALP_EXGMGR_VERB_MAXLEN)
		return ALP_EXGMGR_ERR_VERB_TOO_LONG_ERROR;
	strcpy (theMessage.params.register_handler_notification.itsVerb, iVerb);

	if (iLocalizedVerb)
	{
		if (strlen(iLocalizedVerb) > ALP_EXGMGR_LOCALIZED_VERB_MAXLEN)
			return ALP_EXGMGR_ERR_LOCALIZED_VERB_TOO_LONG_ERROR;
		strcpy (theMessage.params.register_handler_notification.itsLocalizedVerb, iLocalizedVerb);
	}
	else
		theMessage.params.register_handler_notification.itsLocalizedVerb[0] = 0;
	
	if (strlen(iMIMEType) > ALP_EXGMGR_MIMETYPE_MAXLEN)
		return ALP_EXGMGR_ERR_MIMETYPE_TOO_LONG_ERROR;
	strcpy (theMessage.params.register_handler_notification.itsMIMEType, iMIMEType);

	if (strlen(iNotificationType) > ALP_EXGMGR_NOTIFICATIONTYPE_MAXLEN)
		return ALP_EXGMGR_ERR_NOTIFICATIONTYPE_TOO_LONG_ERROR;
	strcpy (theMessage.params.register_handler_notification.itsHandlerNotificationType, iNotificationType);

	theMessage.params.register_handler_notification.itsLocalOnly = iLocalOnly;
	theMessage.params.register_handler_notification.itsSerializeMe = iSerializeMe;
	theMessage.params.register_handler_notification.itsMustBeUnique = iMustBeUnique;
	theMessage.params.register_handler_notification.itsAcceptGenericInvocation = iAcceptGenericInvocation;

	return prv_alp_exg_send_message_to_daemon(&theMessage);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_handler_unregister */
alp_status_t alp_exg_handler_unregister (char *iVerb, char *iMIMEType, char *iHandlerBundleName)
{
	AlpExgMgrdMessageType theMessage;
	
	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_UNREGISTER_HANDLER;
	
	if (strlen(iVerb) > ALP_EXGMGR_VERB_MAXLEN)
		return ALP_EXGMGR_ERR_VERB_TOO_LONG_ERROR;
	strcpy (theMessage.params.unregister_handler.itsVerb, iVerb);
	
	if (strlen(iMIMEType) > ALP_EXGMGR_MIMETYPE_MAXLEN)
		return ALP_EXGMGR_ERR_MIMETYPE_TOO_LONG_ERROR;
	strcpy (theMessage.params.unregister_handler.itsMIMEType, iMIMEType);

	if (strlen(iHandlerBundleName) > ALP_EXGMGR_PACKAGENAME_MAXLEN)
		return ALP_EXGMGR_ERR_PACKAGENAME_TOO_LONG_ERROR;
	strcpy (theMessage.params.unregister_handler.itsHandlerBundleName, iHandlerBundleName);

	return prv_alp_exg_send_message_to_daemon(&theMessage);	
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_handler_unregister_notification */
alp_status_t alp_exg_handler_unregister_notification (char *iVerb, char *iMIMEType, char *iNotificationType)
{
	AlpExgMgrdMessageType theMessage;
	
	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_UNREGISTER_HANDLER_NOTIFICATION;
	
	if (strlen(iVerb) > ALP_EXGMGR_VERB_MAXLEN)
		return ALP_EXGMGR_ERR_VERB_TOO_LONG_ERROR;
	strcpy (theMessage.params.unregister_handler_notification.itsVerb, iVerb);
	
	if (strlen(iMIMEType) > ALP_EXGMGR_MIMETYPE_MAXLEN)
		return ALP_EXGMGR_ERR_MIMETYPE_TOO_LONG_ERROR;
	strcpy (theMessage.params.unregister_handler_notification.itsMIMEType, iMIMEType);

	if (strlen(iNotificationType) > ALP_EXGMGR_NOTIFICATIONTYPE_MAXLEN)
		return ALP_EXGMGR_ERR_NOTIFICATIONTYPE_TOO_LONG_ERROR;
	strcpy (theMessage.params.unregister_handler_notification.itsHandlerNotificationType, iNotificationType);

	return prv_alp_exg_send_message_to_daemon(&theMessage);	
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_handler_check_registration */
alp_status_t alp_exg_handler_check_registration (char *iVerb, char *iMIMEType)
{
	AlpExgMgrdMessageType theMessage;
	
	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_CHECK_HANDLER_REGISTRATION;
	
	if (strlen(iVerb) > ALP_EXGMGR_VERB_MAXLEN)
		return ALP_EXGMGR_ERR_VERB_TOO_LONG_ERROR;
	strcpy (theMessage.params.check_handler_registration.itsVerb, iVerb);
	
	if (strlen(iMIMEType) > ALP_EXGMGR_MIMETYPE_MAXLEN)
		return ALP_EXGMGR_ERR_MIMETYPE_TOO_LONG_ERROR;
	strcpy (theMessage.params.check_handler_registration.itsMIMEType, iMIMEType);

	return prv_alp_exg_send_message_to_daemon(&theMessage);	
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_handler_query_verbs */
alp_status_t alp_exg_handler_query_verbs (char *iMIMEType, int *oEntryCount, char **oDataP)
{
	AlpExgMgrdMessageType theMessage;
	alp_status_t theErr;
	int theFd;
	AlpExgMgrdMessageReplyType theMsgReply;
	size_t theDataSize;
	char *theDataP;
	
	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_QUERY_GENERIC_VERBS;
	if (strlen(iMIMEType) > ALP_EXGMGR_MIMETYPE_MAXLEN)
		return ALP_EXGMGR_ERR_MIMETYPE_TOO_LONG_ERROR;
	strcpy (theMessage.params.query_generic_verbs.itsMIMEType, iMIMEType);

	// Connect to the daemon, send the message, wait for the reply status,
	theErr = prv_alp_exg_connect_to_daemon (&theFd);
	if (theErr)
		return theErr;
		
	theErr = alp_exg_prv_sync_full_write(theFd, &theMessage, sizeof(AlpExgMgrdMessageType));
	if (theErr)
		goto exitClose;

	// daemon replies first with AlpExgMgrdMessageReplyType to say if it could satisfy our demand
	theErr = alp_exg_prv_sync_full_read(theFd, &theMsgReply, sizeof(theMsgReply));
	if (theErr == 0)
		theErr = theMsgReply.itsReplyStatus;
	
	// if it doesn't, or we couldn't read the status, exit.
	if (theErr)
		goto exitClose;
	
	// if no error, the following data is received next : a size_t for the size of the data to come, and then the data itself
	theErr = alp_exg_prv_sync_full_read(theFd, &theDataSize, sizeof(theDataSize));
	if (theErr)
		goto exitClose;
	
	// The first int in the data is the number of entries.
	theErr = alp_exg_prv_sync_full_read(theFd, oEntryCount, sizeof(int));
	if (theErr)
		goto exitClose;
	theDataSize -= sizeof(int);
	
	// allocate a buffer to receive the data
	theDataP = (char *) malloc (theDataSize);
	if (theDataP == NULL)
	{
		theErr = ALP_EXGMGR_ERR_OUT_OF_MEMORY_ERROR;
		goto exitClose;	
	}
	
	theErr = alp_exg_prv_sync_full_read(theFd, theDataP, theDataSize);
	if (theErr)
	{
		free (theDataP);
		goto exitClose;
	}
	
	*oDataP = theDataP;

exitClose:	
	close (theFd);
	return theErr;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_create */
alp_status_t alp_exg_request_create (AlpExgRequest *oRequest)
{
	AlpExgRequestType *theRequestP;
	
	theRequestP = (AlpExgRequestType *) malloc (sizeof (AlpExgRequestType));
	if (theRequestP == NULL)
		return ALP_EXGMGR_ERR_OUT_OF_MEMORY_ERROR;
	
	theRequestP->itsExgVersion = ALP_EXGMGR_VERSION_1;
	theRequestP->itsInfo = (char *) malloc (ALP_EXGMGR_ITSINFO_GROW_SIZE);
	if (theRequestP->itsInfo == NULL)
	{
		free (theRequestP);
		return ALP_EXGMGR_ERR_OUT_OF_MEMORY_ERROR;
	}
	*(theRequestP->itsInfo) = 0;
	theRequestP->itsInfoLen = 1;
	theRequestP->itsInfoSize = ALP_EXGMGR_ITSINFO_GROW_SIZE;
	theRequestP->itsDataFd = -1;
	theRequestP->itsDaemonFd = -1;
	theRequestP->itsRcvInfo = NULL;
	theRequestP->itsRcvDataFd = -1;
	
	*oRequest = (AlpExgRequest) theRequestP;
	return ALP_EXGMGR_ERR_NO_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_delete */
alp_status_t alp_exg_request_delete (AlpExgRequest iRequest)
{
	AlpExgRequestType *theRequestP = (AlpExgRequestType *) iRequest;
	if (theRequestP != NULL)
	{
		if (theRequestP->itsInfo != NULL)
			free (theRequestP->itsInfo);
		if (theRequestP->itsDataFd != -1)
			close (theRequestP->itsDataFd);
			
		if (theRequestP->itsDaemonFd != -1)
			close (theRequestP->itsDaemonFd);

		if (theRequestP->itsRcvInfo != NULL)
			free (theRequestP->itsRcvInfo);
		if (theRequestP->itsRcvDataFd != -1)
			close (theRequestP->itsRcvDataFd);
			
		free (theRequestP);
		return ALP_EXGMGR_ERR_NO_ERROR;
	}
	else
		return ALP_EXGMGR_ERR_NULL_REQUEST_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_set_verb */
alp_status_t alp_exg_request_set_verb (AlpExgRequest iRequest, char *iVerb)
{
	return prv_alp_exg_set_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, ALP_EXGMGR_VERB_TAG, iVerb);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_set_data_fd : get and take ownership of the data source file descriptor */
alp_status_t alp_exg_request_set_data_fd (AlpExgRequest iRequest, int iFd)
{
	AlpExgRequestType *theRequestP = (AlpExgRequestType *) iRequest;
	if (theRequestP->itsDataFd != -1)
		close (theRequestP->itsDataFd);
	theRequestP->itsDataFd = iFd;
	return 0;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_set_data_url */
alp_status_t alp_exg_request_set_data_url (AlpExgRequest iRequest, char *iURL)
{
	return prv_alp_exg_set_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, ALP_EXGMGR_URL_TAG, iURL);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_set_data_type */
alp_status_t alp_exg_request_set_data_type (AlpExgRequest iRequest, char *iMIMEType)
{
	return prv_alp_exg_set_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, ALP_EXGMGR_MIME_TAG, iMIMEType);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_set_int_parameter */
alp_status_t alp_exg_request_set_int_parameter (AlpExgRequest iRequest, char *iParamTag, int iParamValue)
{
	char theString[20];
	sprintf (theString, "%d", iParamValue);
	return prv_alp_exg_set_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_INT, iParamTag, theString);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_set_string_parameter */
alp_status_t alp_exg_request_set_string_parameter (AlpExgRequest iRequest, char *iParamTag, char *iParamValue)
{
	return prv_alp_exg_set_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, iParamTag, iParamValue);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_set_blob_parameter */
alp_status_t alp_exg_request_set_blob_parameter (AlpExgRequest iRequest, char *iParamTag, uint8_t *iBlob, size_t iBlobSize)
{
	alp_status_t theErr;
	char *theParameterValueP;

	// For now, encode in pseudo-BCD. The resulting string will be twice the blob size. Pro is there is no need to parse the buffer
	// once first to calculate the needed space. Con is it doubles the size.
	// Another possibility is to use NULL escaping. Pro is only a few more bytes, Con is more cpu to calculate the size

	// Set the parameter tag and reserve space to copy the parameter value in it.
	theErr = prv_alp_exg_init_parameter_space (iRequest, ALP_EXGMGR_PARAMETER_TYPE_BLOB, iParamTag, iBlobSize * 2, &theParameterValueP);
	if (theErr)
		return theErr;
	
	// OK, we have space to encode the blob and append a final NULL.
	// Note : we don't need true BCD, all we want is to avoid a NULL nibble.
	while (iBlobSize--)
	{
		*theParameterValueP++ = (uint8_t) ((*iBlob & 0xF0) >> 4) + 0x30;
		*theParameterValueP++ = (*iBlob++ & 0x0F) + 0x30;
	}
	*theParameterValueP = 0;

	return ALP_EXGMGR_ERR_NO_ERROR;	
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_transport_select */
/*
alp_status_t alp_exg_transport_select (AlpExgRequest iRequest, char *iPrompt)
{
	alp_status_t theErr;
	// show UI. iPrompt should be displayed (e.g. "where do you want to lookup Eric's email?")
	
	// for now, hardcode local
//	theErr = alp_exg_request_set_string_parameter (iRequest, ALP_EXGMGR_TRANSPORT_TAG, "local;");
	
	return theErr;
}
*/
	
/* -------------------------------------------------------------------------------------------
 * alp_exg_transport_get_params */
alp_status_t alp_exg_transport_get_params (AlpExgRequest iRequest,
										char *oParamsValue, size_t iParamsValueBufLen)
{
	return prv_alp_exg_get_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, ALP_EXGMGR_TRANSPORT_TAG, oParamsValue, iParamsValueBufLen);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_transport_set_params */
alp_status_t alp_exg_transport_set_params (AlpExgRequest iRequest, char *iParamsValue)
{
	// params are in the form : transportnameid;paramname:paramvalue;paramname:paramvalue;
	return prv_alp_exg_set_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, ALP_EXGMGR_TRANSPORT_TAG, iParamsValue);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_set_generic_mode */
alp_status_t alp_exg_request_set_generic_mode (AlpExgRequest iRequest)
{
	return alp_exg_request_set_int_parameter (iRequest, ALP_EXGMGR_GENERIC_TAG, 1);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_get_generic_mode */
alp_status_t alp_exg_request_get_generic_mode (AlpExgRequest iRequest, int *oGenericMode)
{
	alp_status_t theErr;
	theErr = alp_exg_request_get_int_parameter (iRequest, ALP_EXGMGR_GENERIC_TAG, oGenericMode);
	if (theErr == ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR)
	{
		*oGenericMode = 0;
		theErr = 0;
	}
	
	return theErr;	
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_InsertGuid */
static alp_status_t prv_alp_insertGuid (AlpExgRequest iRequest)
{
	// this function will ask a new guid to the daemon and insert it in the request
	alp_status_t theErr;
	AlpExgMgrdMessageType theMessage;
	AlpExgMgrdMessageReplyType theMessageReply;
		
	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_GET_GUID;

	theErr = prv_alp_exg_send_message_to_daemon_with_reply (&theMessage, &theMessageReply);
	if (theErr == 0)
		theErr = alp_exg_request_set_int_parameter (iRequest, ALP_EXGMGR_REQUEST_GLOBAL_UID_PARAM_NAME, theMessageReply.params.get_request_guid.itsRequestGuid);
	
	return theErr;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_execute */
static alp_status_t prv_exg_request_execute_init (AlpExgRequest iRequest, AlpExgMgrTransportInfoType **oTransportInfo)
{
	#define TRANSPORTPARAMSMAXLEN 200
	alp_status_t theErr;
	char theTransportParams[TRANSPORTPARAMSMAXLEN];

	// Add a unique guid to the request to be able to track it through processes.
	theErr = prv_alp_insertGuid (iRequest);
	if (theErr)
		return theErr;
		
	// Get the transport media parameter. This is a name such as 'local', 'tcp', 'bluetooth'...
	theErr = prv_alp_exg_get_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, ALP_EXGMGR_TRANSPORT_TAG, theTransportParams, TRANSPORTPARAMSMAXLEN);
	if (theErr == ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR)
	{
		// no param set. Use local as default
		strcpy(theTransportParams, "local;");
		theErr = alp_exg_request_set_string_parameter (iRequest, ALP_EXGMGR_TRANSPORT_TAG, theTransportParams);
		if (theErr)
			return theErr;	
	}
	
	if (theErr != ALP_EXGMGR_ERR_NO_ERROR)
		return theErr;
	
	// Find the right transport library from the open libs (open it if not found)
	return prv_alp_exg_find_transport_lib (theTransportParams, oTransportInfo);
}

/* -------------------------------------------------------------------------------------------
 * prv_execute_complete */
static void prv_execute_complete (AlpExgRequest iRequest, alp_status_t iExecuteCompleteErr)
{
	g_synchronous_call_result = iExecuteCompleteErr;
	g_synchronous_call_done = true;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_execute */
alp_status_t alp_exg_request_execute (AlpExgRequest iRequest)
{
	alp_status_t theErr;
	GMainContext *theMainContext;
	
	AlpExgMgrTransportInfoType *theTransportInfo;

	// ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_execute called."));

	// Convert this to an asynchronous call, and run the main gloop ourselves until the completion is called.
	if ((theMainContext = g_main_context_default ()) != NULL)
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_execute called, going to local asynchronous"));
		theErr = alp_exg_request_execute_async (iRequest, prv_execute_complete);
		if (theErr)
			return theErr;

		g_synchronous_call_done = false;
		while (!g_synchronous_call_done)
			g_main_context_iteration (theMainContext, FALSE);

		return g_synchronous_call_result;
	}

	ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_execute called but not GMainContext. Executing synchronously with no progresss"));
	// There is no default context. Fallback to synchronous execute (no progress).
	theErr = prv_exg_request_execute_init (iRequest, &theTransportInfo);
	if (theErr)
		return theErr;

	// call alp_exg_transport_execute_request in the transport lib
	return theTransportInfo->itsTransportExecuteRequestFunc (iRequest, NULL, NULL);
}

/* -------------------------------------------------------------------------------------------
 * GSource event callbacks */
static gboolean alp_exg_mgr_async_func_prepare (GSource *source, gint *timeout)
{
//	ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_mgr_async_func_prepare called."));

	*timeout = -1;
	return FALSE;
}

static gboolean alp_exg_mgr_async_func_check (GSource *source)
{
	AlpExgClientExecGSource *theExgGSource = (AlpExgClientExecGSource *) source;
	int theIndex;
	
	for (theIndex = 0; theIndex < EXG_MAX_ASYNC_EXEC_COUNT; theIndex++)
		if (theExgGSource->itsExecuteData[theIndex].isUsed == 1)
			if (theExgGSource->itsExecuteData[theIndex].itsPollFd.revents != 0)
			{
				ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_mgr_async_func_check called AND FOUND FDs READY."));
				return TRUE;
			}

//	ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_mgr_async_func_check called BUT NO FD READY."));

	return FALSE;
}

static gboolean alp_exg_mgr_async_func_dispatch (GSource *source, GSourceFunc callback, gpointer user_data)
{
	AlpExgClientExecGSource *theExgGSource = (AlpExgClientExecGSource *) source;
	int theIndex;

	ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_mgr_async_func_dispatch called."));
	
	for (theIndex = 0; theIndex < EXG_MAX_ASYNC_EXEC_COUNT; theIndex++)
		if (theExgGSource->itsExecuteData[theIndex].isUsed == 1)
			if (theExgGSource->itsExecuteData[theIndex].itsPollFd.revents != 0)
			{
				// Call transport again to handle events on its fd
				alp_status_t theErr;
				ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_mgr_async_func_dispatch : calling transport again."));
				theErr = theExgGSource->itsExecuteData[theIndex].itsTransportExecuteRequestFunc (theExgGSource->itsExecuteData[theIndex].itsRequestP,
																					&theExgGSource->itsExecuteData[theIndex].itsPollFd,
																					&theExgGSource->itsExecuteData[theIndex].itsState);

				// If state is now ALP_EXG_TRANSPORT_EXECUTE_COMPLETE_STATE, callback the client, free the entry and remove the poll fd from the source
				if(theExgGSource->itsExecuteData[theIndex].itsState == ALP_EXG_TRANSPORT_EXECUTE_COMPLETE_STATE)
				{
					ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_mgr_async_func_dispatch : transport returned COMPLETE."));
					// callback the client. This also returns the request ownership to the client. The execution final result is passed as well.
					theExgGSource->itsExecuteData[theIndex].itsExecuteCompleteFunc (theExgGSource->itsExecuteData[theIndex].itsRequestP, theErr);
					g_source_remove_poll (&theExgGSource->itsGSource, &theExgGSource->itsExecuteData[theIndex].itsPollFd);
					theExgGSource->itsExecuteData[theIndex].isUsed = 0;
				}
			}

	return TRUE;
}

static void alp_exg_mgr_async_func_finalize (GSource *source)
{

}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_execute_async */
alp_status_t alp_exg_request_execute_async (AlpExgRequest iRequest, alp_exg_execute_complete_func iExecuteCompleteFunc)
{
	alp_status_t theErr;
	AlpExgMgrTransportInfoType *theTransportInfo;
	int theIndex;

	// ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_execute_async called."));

	theErr = prv_exg_request_execute_init (iRequest, &theTransportInfo);
	if (theErr)
		return theErr;
	
	// If this is first time we execute a request asynchonously, initialize the mechanism
	if (g_alp_exg_mgr_async_gsource == NULL)
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_execute_async : initialize new gsource."));
		 // Allocate general async gsource
		g_alp_exg_mgr_async_source_funcs.prepare = alp_exg_mgr_async_func_prepare;
		g_alp_exg_mgr_async_source_funcs.check = alp_exg_mgr_async_func_check;
		g_alp_exg_mgr_async_source_funcs.dispatch = alp_exg_mgr_async_func_dispatch;
		g_alp_exg_mgr_async_source_funcs.finalize = alp_exg_mgr_async_func_finalize;
		g_alp_exg_mgr_async_source_funcs.closure_callback = NULL;
		g_alp_exg_mgr_async_source_funcs.closure_marshal = NULL;
		g_alp_exg_mgr_async_gsource = (AlpExgClientExecGSource *) g_source_new (&g_alp_exg_mgr_async_source_funcs, sizeof (AlpExgClientExecGSource));
		if (g_alp_exg_mgr_async_gsource == NULL)
		{
			ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_execute_async called: cannot create gsource"));
			return ALP_EXG_MGR_ERR_CANNOT_CREATE_GSOURCE_ERROR;
		}
		memset (g_alp_exg_mgr_async_gsource->itsExecuteData, 0, sizeof(AlpExgClientExecuteData) * EXG_MAX_ASYNC_EXEC_COUNT);
		// Add the source to the default context. It still has nothing to poll for now
		g_alp_exg_mgr_async_source_id = g_source_attach ((GSource *) g_alp_exg_mgr_async_gsource, NULL);
		if (g_alp_exg_mgr_async_source_id <= 0)
		{
			g_source_destroy ((GSource *) g_alp_exg_mgr_async_gsource);
			g_alp_exg_mgr_async_gsource = NULL;
			ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_execute_async called: cannot attach gsource"));
			return ALP_EXG_MGR_ERR_CANNOT_ATTACH_GSOURCE_ERROR;
		}
	}

	// Initialize an AlpExgClientExecuteData entry for this async execute
	for (theIndex = 0; theIndex < EXG_MAX_ASYNC_EXEC_COUNT; theIndex++)
		if (g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].isUsed == 0)
		{
			g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].isUsed = 1;
			g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsExecuteCompleteFunc = iExecuteCompleteFunc;
			g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsRequestP = (AlpExgRequestType *) iRequest;
			g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsTransportExecuteRequestFunc = theTransportInfo->itsTransportExecuteRequestFunc;
			g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsState = ALP_EXG_TRANSPORT_EXECUTE_INIT_STATE;
			g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsPollFd.fd = -1;
			break;
		}
	if (theIndex == EXG_MAX_ASYNC_EXEC_COUNT)
		return ALP_EXGMGR_ERR_TOO_MANY_ASYNC_EXECUTE_ERROR;

	ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_execute_async: calling transport execute first time"));

	// Call transport to initiate routing the request to destination.
	theErr = theTransportInfo->itsTransportExecuteRequestFunc (iRequest, &g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsPollFd,
																&g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsState);

	// If state is ALP_EXG_TRANSPORT_EXECUTE_COMPLETE_STATE, Execution is already complete (probably with an error)
	// Free async entry immediately and return result.
	if (g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsState == ALP_EXG_TRANSPORT_EXECUTE_COMPLETE_STATE)
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_execute_async : transportExecute returned state == COMPLETE and err = %lx.", theErr));
		goto freeExecuteDataExit;
	}

	// Otherwise, transport must have set a pollfd. It must also have set the correct events it is waiting for.
	if (g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsPollFd.fd == -1)
	{
		ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_execute_async transport execute first call returned NO FD."));
		theErr = ALP_EXGMGR_ERR_API_ERR_ERROR;
		goto freeExecuteDataExit;
	}

	// add it to the gsource.
	g_source_add_poll (&g_alp_exg_mgr_async_gsource->itsGSource, &g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsPollFd);

	return 0;

freeExecuteDataExit:
	g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].isUsed = 0;
	return theErr;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_activate_progress */
alp_status_t alp_exg_request_activate_progress (AlpExgRequest iRequest)
{
	return prv_alp_exg_set_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_INT, ALP_EXGMGR_REQUEST_PROGRESS_ACTIVE_PARAM_NAME, "1");
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_get_verb */
alp_status_t alp_exg_request_get_verb (AlpExgRequest iRequest, char *oVerb, size_t iVerbBufLen)
{
	return prv_alp_exg_get_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, ALP_EXGMGR_VERB_TAG, oVerb, iVerbBufLen);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_get_data_type */
alp_status_t alp_exg_request_get_data_type (AlpExgRequest iRequest, char *oMIMEType, size_t iMIMETypeBufLen)
{
	// the data type can be found in ALP_EXGMGR_MIME_TAG if the client set it, or in ALP_EXGMGR_OBEX_OBJECT_TYPE_BASE
	// in case it was set using the store request utilities
	
	alp_status_t theErr;
	theErr = prv_alp_exg_get_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, ALP_EXGMGR_MIME_TAG, oMIMEType, iMIMETypeBufLen);
	if (theErr == ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR)
		theErr = prv_alp_exg_get_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, ALP_EXGMGR_OBEX_OBJECT_TYPE_BASE, oMIMEType, iMIMETypeBufLen);
	return theErr;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_get_int_parameter */
alp_status_t alp_exg_request_get_int_parameter (AlpExgRequest iRequest, char *iParamTag, int *oParamValue)
{
	alp_status_t theErr;
	char theBuf[20];
	
	theErr = prv_alp_exg_get_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_INT, iParamTag, theBuf, 20);
	if (theErr == ALP_EXGMGR_ERR_NO_ERROR)
		sscanf (theBuf, "%d", oParamValue);
	return theErr;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_get_string_parameter */
alp_status_t alp_exg_request_get_string_parameter (AlpExgRequest iRequest, char *iParamTag, char *oParamValue, size_t iParamValueBufLen)
{
	return prv_alp_exg_get_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, iParamTag, oParamValue, iParamValueBufLen);
}

/* -------------------------------------------------------------------------------------------
 * prv_alp_exg_get_parameter_const */
static alp_status_t prv_alp_exg_get_parameter_const (AlpExgRequestType *iRequest, char iParameterType, char *iParamTag, const char **oParamValue)
{
	// This function returns the address of the parameter value in *oParamValue
	alp_status_t theErr;
	int theOffset;
	char *theInfo;
	
	// parameters can be retrieved either from the RcvInfo (for received request only), or from the normal Info.	
	theInfo = iRequest->itsRcvInfo;
	theErr = prv_alp_exg_find_parameter (theInfo, iParameterType, iParamTag, &theOffset);
	if (theErr == ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR)
	{
		theInfo = iRequest->itsInfo;
		theErr = prv_alp_exg_find_parameter (theInfo, iParameterType, iParamTag, &theOffset);
	}
	
	if (theErr == ALP_EXGMGR_ERR_NO_ERROR)
	{
		// theOffset points to the paramtype character. jump to the value
		theOffset += (strlen (theInfo + theOffset) + 1);
		*oParamValue = theInfo + theOffset;
	}
	
	return theErr;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_get_string_parameter_const */
alp_status_t alp_exg_request_get_string_parameter_const (AlpExgRequest iRequest, char *iParamTag, const char **oParamValue)
{
	return prv_alp_exg_get_parameter_const ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, iParamTag, oParamValue);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_get_blob_parameter */
alp_status_t alp_exg_request_get_blob_parameter (AlpExgRequest iRequest, char *iParamTag, uint8_t **oBlob, size_t *oBlobSize)
{
	alp_status_t theErr;
	const char *theData;
	size_t theBlobLen;
	uint8_t *theP;
	
	theErr = prv_alp_exg_get_parameter_const ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_BLOB, iParamTag, &theData);
	if (theErr)
		return theErr;
		
	// theData now points to the blob encoded in pseudo-BCD
	*oBlobSize = strlen (theData) / 2;
	
	// Allocate a Blob chunk to receive the decoded data and return it to the user
	*oBlob = (uint8_t *) malloc (*oBlobSize);
	if (*oBlob == NULL)
		return ALP_EXGMGR_ERR_OUT_OF_MEMORY_ERROR;
	
	// Decode and store
	theP = *oBlob;
	theBlobLen = *oBlobSize;
	while (theBlobLen--)
	{
		*theP++ = ((*theData - 0x30) << 4) + (*(theData + 1) - 0x30);
		theData += 2;
	}
	
	//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_get_blob_parameter: decoded buf = "));
	//ALP_TM(B(ALP_CLASS_EXCHANGE, *oBlob, *oBlobSize));
	
	return ALP_EXGMGR_ERR_NO_ERROR;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_get_data_fd : Gives ownership of the file descriptor to the caller */
alp_status_t alp_exg_request_get_data_fd (AlpExgRequest iRequest, int *oFd)
{
	AlpExgRequestType *theRequestP = (AlpExgRequestType *) iRequest;
	
	if (theRequestP->itsRcvDataFd != -1)
	{
		*oFd = theRequestP->itsRcvDataFd;
		theRequestP->itsRcvDataFd = -1;
	}
	else
	{
		*oFd = theRequestP->itsDataFd;
		theRequestP->itsDataFd = -1;
	}
	return 0;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_get_data_url */
alp_status_t alp_exg_request_get_data_url (AlpExgRequest iRequest, char *oUrl, size_t iUrlBufLen)
{
	return prv_alp_exg_get_parameter ((AlpExgRequestType *) iRequest, ALP_EXGMGR_PARAMETER_TYPE_STRING, ALP_EXGMGR_URL_TAG, oUrl, iUrlBufLen);
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_receive */
alp_status_t alp_exg_request_receive (AlpExgRequest *oRequest, AlpExgRequestId iRequestId)
{
	AlpExgMgrdMessageType theMessage;
	alp_status_t theErr;
	int theFd;
	AlpExgMgrdMessageReplyType theMsgReply;
	size_t theInfoLen;
	char *theInfo;
	int theHasDataFd;
	int theDataFd;
	
	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_RECEIVE_REQUEST;
	theMessage.params.receive_request.itsRequestId = iRequestId;

	// Connect to the daemon, send the message, wait for the reply status,
	theErr = prv_alp_exg_connect_to_daemon (&theFd);
	if (theErr)
		return theErr;
		
	theErr = alp_exg_prv_sync_full_write(theFd, &theMessage, sizeof(AlpExgMgrdMessageType));
	if (theErr)
		goto exitClose;

	// daemon replies first with alp_status_t to say if it knows the requestId we ask
	theErr = alp_exg_prv_sync_full_read(theFd, &theMsgReply, sizeof(theMsgReply));
	if (theErr == 0)
		theErr = theMsgReply.itsReplyStatus;
	
	// if it doesn't, or we couldn't read the status, exit.
	if (theErr)
		goto exitClose;
	
	// if no error, the following data is received next : the request InfoLen, Info data, the fd_present indication, and the fd if any.
	theErr = alp_exg_prv_sync_full_read(theFd, &theInfoLen, sizeof(theInfoLen));
	if (theErr)
		goto exitClose;
		
	theInfo = (char *) malloc (theInfoLen);
	if (theInfo == NULL)
	{
		theErr = ALP_EXGMGR_ERR_OUT_OF_MEMORY_ERROR;
		goto exitClose;	
	}
	
	theErr = alp_exg_prv_sync_full_read(theFd, theInfo, theInfoLen);
	if (theErr)
		goto exitFreeInfo;

	theErr = alp_exg_prv_sync_full_read(theFd, &theHasDataFd, sizeof(theHasDataFd));
	if (theErr)
		goto exitFreeInfo;
	//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_receive: got theHasDataFd = %d.", theHasDataFd));

	theDataFd = -1;
	if (theHasDataFd != 0)
	{
		// alp_exg_prv_read_fd must return 1 byte read
		if (alp_exg_prv_read_fd (theFd, &theHasDataFd, 1, &theDataFd) != 1)
		{
			theErr = ALP_EXG_MGR_ERR_READ_FD_ERROR;
			goto exitFreeInfo;
		}
		//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_receive: got the dataFd = %d", theDataFd));
	}

	// We have everything..
	// Create the local request to execute.  If error, send back result and abort
	theErr = alp_exg_request_create (oRequest);
	if (theErr)
		goto exitSendErrStatus;
	
	// Set the receive info and fd fields. These will be looked first when the client tries to retrieve parameters or dataFd
	((AlpExgRequestType *) *oRequest)->itsRcvInfo = theInfo;
	((AlpExgRequestType *) *oRequest)->itsRcvInfoLen = theInfoLen;
	((AlpExgRequestType *) *oRequest)->itsRcvDataFd = theDataFd;
	
	// Keep the connection with the daemon open (store fd in the request). It will be
	// used in alp_exg_request_complete to send back the result status and modified request.
	((AlpExgRequestType *) *oRequest)->itsDaemonFd = theFd;
	
	ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_receive: returned 0x00."));
	return 0;

exitSendErrStatus:
	// close local dataFd if we got one
	if (theDataFd != -1)
		close (theDataFd);
	// sends the error to the daemon. This is equivalent to doing an immediate alp_exg_request_complete
	(void) alp_exg_prv_sync_full_write(theFd, &theErr, sizeof(theErr));

exitFreeInfo:
	free (theInfo);

exitClose:
	close (theFd);
	
	ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_receive: returned 0x%lx.", theErr));
	return theErr;
}

/* -------------------------------------------------------------------------------------------
 * alp_exg_request_complete */
alp_status_t alp_exg_request_complete (AlpExgRequest iRequest, alp_status_t iResult)
{
	AlpExgRequestType *theRequestP = (AlpExgRequestType *) iRequest;
	alp_status_t theErr;
	size_t theInfoLen;
	int theHasDataFd;
	
	if (theRequestP->itsDaemonFd == -1)
		return ALP_EXG_MGR_ERR_REQUEST_NOT_RECEIVED_ERROR;
	
	// Send the execution result. If iResult is not null, we're done
	ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_complete: Sending handler result"));
	theErr = alp_exg_prv_sync_full_write(theRequestP->itsDaemonFd, &iResult, sizeof(iResult));
	if (theErr)
		goto exitClose;
	if (iResult)
	{
		shutdown (theRequestP->itsDaemonFd, SHUT_RDWR);
		goto exitClose;
	}
		
	// send the modified request. If the user has set new parameters and data, these are in itsInfo and itsDataFd.
	// Send size of info (if it is <= 2, then there is no parameters)
	theInfoLen = theRequestP->itsInfoLen > 1 ? theRequestP->itsInfoLen : 0;
	//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_complete: Sending infoLen = %d", theInfoLen));
	theErr = alp_exg_prv_sync_full_write (theRequestP->itsDaemonFd, &theInfoLen, sizeof (theInfoLen));
	if (theErr)
		goto exitClose;
	
	// Send info data if there is any
	if (theInfoLen > 0)
	{
		//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_complete: Sending info data"));
		theErr = alp_exg_prv_sync_full_write (theRequestP->itsDaemonFd, theRequestP->itsInfo, theInfoLen);
		if (theErr)
			goto exitClose;
	}
	
	// send data fd only if necessary
	//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_complete: Sending hasDataFd"));
	theHasDataFd = (theRequestP->itsDataFd == -1 ? 0 : 1);
	//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_complete: Sent theHasDataFd = %d.", theHasDataFd));
	theErr = alp_exg_prv_sync_full_write (theRequestP->itsDaemonFd, &theHasDataFd, sizeof(theHasDataFd));
	if (theErr)
		goto exitClose;
	
	if (theHasDataFd == 1)
	{
		//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_complete: Sending fd"));
		// as one byte must be sent along with the control data, use theHasDataFd as dummy data.
		if (alp_exg_prv_write_fd (theRequestP->itsDaemonFd, &theHasDataFd, 1, theRequestP->itsDataFd) < 0)
		{
			theErr = ALP_EXG_MGR_ERR_WRITE_FD_ERROR;
			goto exitClose;
		}
		//ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_complete: Sent theDataFd = %d.", theRequestP->itsDataFd));
	}
	
exitClose:
	// delete the request. This will close the dataFd and daemonFd.
	alp_exg_request_delete(iRequest);
	
	ALP_TM(TL(ALP_CLASS_EXCHANGE, "ExgMgr : alp_exg_request_complete: returned 0x%lx.", theErr));
	return theErr;
}

alp_status_t alp_exg_request_cancel (AlpExgRequest iRequest)
{
	// Get request guid and send cancel msg to daemon.
	alp_status_t theErr;
	AlpExgMgrdMessageType theMessage;

	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_CANCEL_REQUEST;
	theErr = alp_exg_request_get_int_parameter (iRequest, ALP_EXGMGR_REQUEST_GLOBAL_UID_PARAM_NAME, &theMessage.params.cancel_request.itsGuid);
	if (theErr)
		return theErr;

	return prv_alp_exg_send_message_to_daemon (&theMessage);
}

alp_status_t alp_exg_prv_cancel_incoming (int iGuid)
{
	// iGuid is EXG_MGR_BT_INCOMING_UID or EXG_MGR_IR_INCOMING_UID
	AlpExgMgrdMessageType theMessage;

	theMessage.itsOpcode = ALP_EXG_DAEMON_MSG_OPCODE_CANCEL_REQUEST;
	theMessage.params.cancel_request.itsGuid = iGuid;
	return prv_alp_exg_send_message_to_daemon (&theMessage);
}

void __attribute__((constructor)) init_exg_mgr(void)
{
	ALP_TM(TL(ALP_CLASS_EXCHANGE, "init_exg_mgr : exgmgr lib loaded."));
}

void __attribute__((destructor)) cleanup_exg_mgr(void)
{
	int theIndex;
	
	ALP_TM(TL(ALP_CLASS_EXCHANGE, "cleanup_exg_mgr : exgmgr lib unloaded."));

	if (g_alp_exg_mgr_async_gsource != NULL)
	{
		// Make sure there are no async requests currently executing.
		for (theIndex = 0; theIndex < EXG_MAX_ASYNC_EXEC_COUNT; theIndex++)
			if (g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].isUsed == 1)
			{
				// Exg lib is unloading, which means our users are already unloaded and we can't call them.
				// Just kill transport fd. This will correctly abort execution in the daemon or on the remove device.
				close (g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsPollFd.fd);
				//alp_exg_request_delete (AlpExgRequest iRequest)
				g_source_remove_poll (&g_alp_exg_mgr_async_gsource->itsGSource, &g_alp_exg_mgr_async_gsource->itsExecuteData[theIndex].itsPollFd);
			}

		g_source_destroy (&g_alp_exg_mgr_async_gsource->itsGSource);
	}
	
	if (g_alp_exg_transport_infoP)
	{
		for (theIndex = 0; theIndex < g_alp_exg_transport_info_count; theIndex++)
		{
			if (g_alp_exg_transport_infoP[theIndex].itsLibRef != 0)
				dlclose (g_alp_exg_transport_infoP[theIndex].itsLibRef);

			if (g_alp_exg_transport_infoP[theIndex].itsNameId)
				g_free (g_alp_exg_transport_infoP[theIndex].itsNameId);
				
			if (g_alp_exg_transport_infoP[theIndex].itsLibPath)
				g_free (g_alp_exg_transport_infoP[theIndex].itsLibPath);
	
			if (g_alp_exg_transport_infoP[theIndex].itsBundleRef != 0)
				alp_bundle_close (g_alp_exg_transport_infoP[theIndex].itsBundleRef);
		}
		
		free (g_alp_exg_transport_infoP);
	}
}
