/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*! @file exgmgrutils.c 
 *
 * @brief Exchange Manager Utilities Library
 *
*/

#include <string.h>
#include <fcntl.h>
#include <stdlib.h>
#include <hiker/traces.h>
#include <hiker/exgmgrutils.h>
#include <hiker/prv/exgmgr_prv.h>

/* alp_exg_utils_obex_request_create */
alp_status_t alp_exg_utils_store_request_create (AlpExgRequest *oRequest)
{
	alp_status_t theErr;
	
	theErr = alp_exg_request_create (oRequest);
	if (theErr)
		return theErr;

	theErr = alp_exg_request_set_verb (*oRequest, ALP_EXGMGR_STORE_OBJECT_VERB);
	if (theErr)
		(void) alp_exg_request_delete (*oRequest);

	return theErr;
}

/* alp_exg_utils_obex_request_create */
alp_status_t alp_exg_utils_store_request_set_data_fd (AlpExgRequest iRequest, int iFd, char *iMIMEType, char *iDescription, char *iName)
{
	alp_status_t theErr;

	// Set all the required parameters to specify an OBEX object using a file desc.
	theErr = alp_exg_request_set_string_parameter(iRequest, ALP_EXGMGR_OBEX_OBJECT_TYPE_BASE, iMIMEType);
	
	if (theErr == 0)
		theErr = alp_exg_request_set_string_parameter(iRequest, ALP_EXGMGR_OBEX_OBJECT_DESC_BASE, iDescription ? iDescription : "");
	
	if (theErr == 0)
		theErr = alp_exg_request_set_string_parameter(iRequest, ALP_EXGMGR_OBEX_OBJECT_NAME_BASE, iName);

	if (theErr == 0)
		theErr = alp_exg_request_set_int_parameter(iRequest, ALP_EXGMGR_OBEX_OBJECT_COUNT, 1);

	// set file descriptor last because we want to take it in the request only if every thing else succeeded.
	if (theErr == 0)
		theErr = alp_exg_request_set_data_fd (iRequest, iFd);	
	
	return theErr;
}

/* alp_exg_utils_store_request_add_object */
alp_status_t alp_exg_utils_store_request_add_object (AlpExgRequest iRequest, char *iFilename, char *iMIMEType, char *iDescription, char *iName)
{
	alp_status_t theErr;
	int theIntParam;
	char theParamName[40];
	
	// set all the required parameters to add an OBEX object using a filename
	
	// How many objects do we already have? if not found, this is the first one.
	theErr = alp_exg_request_get_int_parameter(iRequest, ALP_EXGMGR_OBEX_OBJECT_COUNT, &theIntParam);	
	if (theErr == ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR)
	{
		theErr = 0;
		theIntParam = 0;
	}

	// add file
	if (theErr == 0)
	{
		strcpy (theParamName, ALP_EXGMGR_OBEX_OBJECT_FILE_BASE);
		theParamName[strlen(theParamName) - 1] += theIntParam;
		theErr = alp_exg_request_set_string_parameter(iRequest, theParamName, iFilename);
	}

	// add type
	if (theErr == 0)
	{
		strcpy (theParamName, ALP_EXGMGR_OBEX_OBJECT_TYPE_BASE);
		theParamName[strlen(theParamName) - 1] += theIntParam;
		theErr = alp_exg_request_set_string_parameter(iRequest, theParamName, iMIMEType);
	}
		
	// add description
	if (theErr == 0 && iDescription)
	{
		strcpy (theParamName, ALP_EXGMGR_OBEX_OBJECT_DESC_BASE);
		theParamName[strlen(theParamName) - 1] += theIntParam;
		theErr = alp_exg_request_set_string_parameter(iRequest, theParamName, iDescription ? iDescription : "");
	}
	
	// add name
	if (theErr == 0 && iName)
	{
		strcpy (theParamName, ALP_EXGMGR_OBEX_OBJECT_NAME_BASE);
		theParamName[strlen(theParamName) - 1] += theIntParam;
		theErr = alp_exg_request_set_string_parameter(iRequest, theParamName, iName);
	}

	if (theErr == 0)
		theErr = alp_exg_request_set_int_parameter(iRequest, ALP_EXGMGR_OBEX_OBJECT_COUNT, theIntParam + 1);

	return theErr;
}

/* alp_exg_utils_is_store_request */
alp_status_t alp_exg_utils_is_store_request (AlpExgRequest iRequest, int *oIsStoreRequest)
{
	alp_status_t theErr;
	char theStringParam[50];
	
	theErr = alp_exg_request_get_verb(iRequest, theStringParam, sizeof (theStringParam));
	if (theErr)
		return theErr;
		
	*oIsStoreRequest = 0;
	if (strcmp (theStringParam, ALP_EXGMGR_STORE_OBJECT_VERB) == 0)
		*oIsStoreRequest = 1;
		
	return 0;
}

/* alp_exg_utils_store_request_get_object_count */
alp_status_t alp_exg_utils_store_request_get_object_count (AlpExgRequest iRequest, int *oObjectCount)
{
	alp_status_t theErr;
	
	theErr = alp_exg_request_get_int_parameter(iRequest, ALP_EXGMGR_OBEX_OBJECT_COUNT, oObjectCount);	
	if (theErr == ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR)
	{
		theErr = 0;
		*oObjectCount = 0;
	}
	return theErr;
}

static alp_status_t prv_alp_exg_utils_get_indexed_obex_param_const (AlpExgRequest iRequest, char *iParamNameBase, int iParamIndex, const char **iBuf)
{
	alp_status_t theErr;
	char theParamName[40];
		
	// params are zero based
	iParamIndex -= 1;
	
	// build param name
	strcpy (theParamName, iParamNameBase);
	theParamName[strlen(theParamName) - 1] += iParamIndex;
	return alp_exg_request_get_string_parameter_const (iRequest, theParamName, iBuf);
}

static alp_status_t prv_alp_exg_utils_get_indexed_obex_param (AlpExgRequest iRequest, char *iParamNameBase, int iParamIndex, char *iBuf, size_t iBufLen)
{
	alp_status_t theErr;
	char theParamName[40];
		
	// params are zero based
	iParamIndex -= 1;
	
	// build param name
	strcpy (theParamName, iParamNameBase);
	theParamName[strlen(theParamName) - 1] += iParamIndex;
	theErr = alp_exg_request_get_string_parameter (iRequest, theParamName, iBuf, iBufLen);
	if (theErr == ALP_EXGMGR_ERR_PARAM_NOT_FOUND_ERROR)
	{
		*iBuf = 0;
		theErr = 0;
	}
	return theErr;
}

/* alp_exg_utils_store_request_get_data_fd */
alp_status_t alp_exg_utils_store_request_get_data_fd (AlpExgRequest iRequest, int *oFd, char *oMIMEType, size_t iMIMETypeBufLen,
																						char *oDescription, size_t iDescriptionBufLen,
																						char *oName, size_t iNameBufLen)
{
	alp_status_t theErr = 0;
	// TODO: DEPRECATED FUNCTION. ADJUST CALLERS

	if (oMIMEType)
		theErr = prv_alp_exg_utils_get_indexed_obex_param (iRequest, ALP_EXGMGR_OBEX_OBJECT_TYPE_BASE, 1, oMIMEType, iMIMETypeBufLen);
	if (theErr == 0 && oDescription)
		theErr = prv_alp_exg_utils_get_indexed_obex_param (iRequest, ALP_EXGMGR_OBEX_OBJECT_DESC_BASE, 1, oDescription, iDescriptionBufLen);
	if (theErr == 0 && oName)
		theErr = prv_alp_exg_utils_get_indexed_obex_param (iRequest, ALP_EXGMGR_OBEX_OBJECT_NAME_BASE, 1, oName, iNameBufLen);
	// give fd to client only if everything else succeeded
	if (theErr == 0)
		theErr = alp_exg_request_get_data_fd(iRequest, oFd);

	return theErr;
}

/* alp_exg_utils_store_request_get_object_fd */
alp_status_t alp_exg_utils_store_request_get_object_fd (AlpExgRequest iRequest, int iObjectNum, int *oFd,
																						char *oMIMEType, size_t iMIMETypeBufLen,
																						char *oDescription, size_t iDescriptionBufLen,
																						char *oName, size_t iNameBufLen)
{
	alp_status_t theErr = 0;
	const char *theFilename;

	if (oMIMEType)
		theErr = prv_alp_exg_utils_get_indexed_obex_param (iRequest, ALP_EXGMGR_OBEX_OBJECT_TYPE_BASE, 1, oMIMEType, iMIMETypeBufLen);
	if (theErr == 0 && oDescription)
		theErr = prv_alp_exg_utils_get_indexed_obex_param (iRequest, ALP_EXGMGR_OBEX_OBJECT_DESC_BASE, 1, oDescription, iDescriptionBufLen);
	if (theErr == 0 && oName)
		theErr = prv_alp_exg_utils_get_indexed_obex_param (iRequest, ALP_EXGMGR_OBEX_OBJECT_NAME_BASE, 1, oName, iNameBufLen);
	if (theErr)
		return theErr;

	// if this this is object index 1, the fd may have been passed with the request
	if (iObjectNum == 1)
	{
		theErr = alp_exg_request_get_data_fd (iRequest, oFd);
		if (theErr)
			return theErr;
		if (*oFd >= 0)
			return 0;
	}

	// open the file, unlink it, remove the request param
	theErr = prv_alp_exg_utils_get_indexed_obex_param_const (iRequest, ALP_EXGMGR_OBEX_OBJECT_FILE_BASE, iObjectNum, &theFilename);
	if (theErr)
		return theErr;
		
	*oFd = open (theFilename, O_RDONLY);
	if (*oFd < 0)
		return ALP_EXG_MGR_ERR_TRANSPORT_OPEN_ERROR;
	unlink (theFilename);
	
	return 0;
}

/* alp_exg_utils_store_request_get_object */
alp_status_t alp_exg_utils_store_request_get_object (AlpExgRequest iRequest, int iObjectNum, char *oFilename, size_t iFilenameBufLen,
																						char *oMIMEType, size_t iMIMETypeBufLen,
																						char *oDescription, size_t iDescriptionBufLen,
																						char *oName, size_t iNameBufLen)
{
	alp_status_t theErr = 0;

	// If this is object index 1, and object data is passed by fd, convert it to a file
	if (iObjectNum == 1)
	{
		int theFd;
		theErr = alp_exg_request_get_data_fd (iRequest, &theFd);
		if (theErr)
			return theErr;

		if (theFd >= 0)
		{
			char *theFilename;
			// copy the data in a temp file. Whatever the result, theFd will have been closed upon return.
			theErr = alp_exg_prv_fd_to_file (theFd, "/tmp/util", &theFilename);
			if (theErr)
				return theErr;
			// Set the filename parameter
			theErr = alp_exg_request_set_string_parameter(iRequest, ALP_EXGMGR_OBEX_OBJECT_FILE_BASE, theFilename);
			if (theErr)
			{
				unlink (theFilename);
				free (theFilename);
				return theErr;
			}
			free (theFilename);
		}
	}
	
	if (oFilename)
		theErr = prv_alp_exg_utils_get_indexed_obex_param (iRequest, ALP_EXGMGR_OBEX_OBJECT_FILE_BASE, iObjectNum, oFilename, iFilenameBufLen);
	if (theErr == 0 && oMIMEType)
		theErr = prv_alp_exg_utils_get_indexed_obex_param (iRequest, ALP_EXGMGR_OBEX_OBJECT_TYPE_BASE, iObjectNum, oMIMEType, iMIMETypeBufLen);
	if (theErr == 0 && oDescription)
		theErr = prv_alp_exg_utils_get_indexed_obex_param (iRequest, ALP_EXGMGR_OBEX_OBJECT_DESC_BASE, iObjectNum, oDescription, iDescriptionBufLen);
	if (theErr == 0 && oName)
		theErr = prv_alp_exg_utils_get_indexed_obex_param (iRequest, ALP_EXGMGR_OBEX_OBJECT_NAME_BASE, iObjectNum, oName, iNameBufLen);

	return theErr;
}
