/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file add_to_default_prefs.c
 *
 * @brief tool to build default preference settings into the initial image
 *
 *****************************************************************************/
#include <stdio.h>
#include <getopt.h>

#include "hiker/traces.h"
#include "gs_imp.h"

#include "global_settings_server.h"



static void show_usage(const char *argv0)
{
    fprintf(stderr, "Usage: %s [--rebuild] default_prefs_file xml_prefs_file [xml_prefs_file...]\n", argv0);
    fprintf(stderr, "Usage: %s --delete-tree absolute_key_path prefs_file\n", argv0);
}


int main(int argc, char *argv[])
{
    gs_svr_context *context = 0;
    alp_status_t status;
    const char *argv0 = argv[0];
    int rebuild_flag = 0;
    int i, c;
    int next_arg = 1;  /* 0 based offset of the next argument to process */
    char key[PATH_MAX];
    int delete_tree = 0;
    static struct option long_options[] =
        {
            { "rebuild", 0, 0, 'r'},
            { "delete-tree", 1, 0, 'D'},
            { 0, 0, 0, 0 }
        } ;



    while (1)
    {
        c = getopt_long(argc, argv, "oD:", long_options, NULL);
        switch (c)
        {
        case 'r':
            rebuild_flag = 1;
            break;
        case 'D':
            strncpy(key, optarg, sizeof(key));

            delete_tree = 1;
            break;
        
        case '?':    
            show_usage(argv[0]);
            exit(1);
            
        default:
            break;
        }
        next_arg = optind;
        if (c == -1)
            break;

    }


    
    if ((delete_tree && rebuild_flag) || (argc - next_arg < 1))
    {
        show_usage(argv0);
        exit(1);
    }

    status = gs_svr_init(argv[next_arg]);
    ALP_FAIL_FATAL_IF(!(status == ALP_STATUS_OK), "Cannot open default pref database file", "");

    status = gs_svr_open(argv[next_arg], &context);
    ALP_FAIL_FATAL_IF(!(status == ALP_STATUS_OK), "Cannot open default pref database file", "");

    status = gs_svr_proc_begin_transaction(context);
    ALP_FAIL_FATAL_IF(!(status == ALP_STATUS_OK), "Cannot begin database transaction", "");

    if (delete_tree)
    {
    
        printf("Delete subtree %s\n", key);
        status = gs_svr_proc_delete_tree(context, key);
        if (status != ALP_STATUS_OK)
        {
            fprintf(stderr, "delete key dir '%s' failed,  %s\n", key, status_to_string(status));
            
        }

    
    }
    else
    {
        if (rebuild_flag)
        {
            printf("Delete existing keys...\n");
            status = gs_svr_proc_delete_tree(context, "/");
            if (status != ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND )
                ALP_FAIL_FATAL_IF(!(status == ALP_STATUS_OK), "Cannot delete existing keys in database", "");
        }

        for (i = next_arg + 1 ; i < argc; i++)
        {
            printf("Importing from %s\n", argv[i]);
            status = gs_svr_proc_import_from_file(context, argv[i]);
            ALP_FAIL_FATAL_IF(!(status == ALP_STATUS_OK), "Error in preference import from XML file %s\n", argv[i]);
        }
    }
    status = gs_svr_proc_complete_transaction(context);
    gs_svr_close(context);

    return 0;
}
