/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file alp_appprefs.c
 *
 * @brief the implementation of the Global Settings service APIs.
 *
 *****************************************************************************/
#include <pthread.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <errno.h>

#include <glib.h>

#include "hiker/traces.h"
#include "hiker/fail.h"
#include "hiker/global_settings.h"
#include "hiker/ipc.h"

#include "gs_imp.h"

#include "serialize.h"

struct _AlpGlobalSettingsContext
{
    AlpConnection *connection;
    char cwd[PATH_MAX];
   
} ;


#define BEGIN_BUSY_LOOP while (1) { 

#define END_BUSY_LOOP if (status != ALP_STATUS_GLOBAL_SETTINGS_BUSY) break; }
    
        
alp_status_t 
alp_global_settings_init(void)
{
    return ALP_STATUS_OK;

}

alp_status_t
alp_global_settings_open(AlpGlobalSettingsContext **context)
{
    alp_status_t status;
    int i;
    CHECK_CONTEXT
    

    *context = calloc(1, sizeof(**context));
    ALP_FAIL_FATAL_IF(*context == 0, "out of memory", "");


    
    status =   alp_ipc_channel_connect_synch(GLOBAL_SETTINGS_CHANNEL_NAME, ALP_IPC_CONNECTION_USAGE_HINT_NONE, GLOBAL_SETTINGS_TIMEOUT,  &( (*context)->connection));

    if (status == ALP_STATUS_OK)
    {
        AlpMessage *msg = 0, *resp = 0;
       
        uint32_t i;


        gs_start_pack(&msg, PROC_open);

        alp_ipc_message_pack_end(msg);
        status = alp_ipc_connection_send_with_response((*context)->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
        if (status == ALP_STATUS_OK)
        {
            alp_ipc_message_unpack_start(resp);
            alp_ipc_message_unpack_uint32(resp, &i);
            if (i != ALP_GLOBAL_SETTINGS_VERSION)
                status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
            else
            {
                alp_ipc_message_unpack_uint32(resp, &i);
                status = i;
            }
            alp_ipc_message_unpack_end(resp);
        }
        alp_ipc_message_destroy(msg);     
        if (resp)
            alp_ipc_message_destroy(resp);     
        
    }
    
    return status;
}

alp_status_t
alp_global_settings_close(AlpGlobalSettingsContext *context)
{
    alp_status_t status ;
    CHECK_CONTEXT

    
    {
        AlpMessage *msg = 0, *resp = 0;
      
        uint32_t i;


        gs_start_pack(&msg, PROC_close);

        alp_ipc_message_pack_end(msg);
        status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
        if (status == ALP_STATUS_OK)
        {
            alp_ipc_message_unpack_start(resp);
            alp_ipc_message_unpack_uint32(resp, &i);
            if (i != ALP_GLOBAL_SETTINGS_VERSION)
                status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
            else
            {
                alp_ipc_message_unpack_uint32(resp, &i);
                status = i;
            }
            alp_ipc_message_unpack_end(resp);
        }
        alp_ipc_message_destroy(msg);     
        if (resp)
            alp_ipc_message_destroy(resp);     
        
    }
    alp_ipc_channel_disconnect(context->connection);
    free(context);
    return status;
}

alp_status_t 
alp_global_settings_set_cwd(AlpGlobalSettingsContext *context, const char *cwd)
{

    char tmp[PATH_MAX];
    form_absolute_path(context->cwd, cwd, tmp);
    strncpy(context->cwd, tmp, sizeof(context->cwd));
    return ALP_STATUS_OK;
}


alp_status_t 
alp_global_settings_get_cwd(AlpGlobalSettingsContext *context, char *buf, size_t buf_size)
{

    if (strlen(context->cwd) + 1 > buf_size)
        return ALP_STATUS_GLOBAL_SETTINGS_BUFFER_TOO_SMALL;
    
    strncpy(buf, context->cwd, buf_size);
    return ALP_STATUS_OK;
}



alp_status_t 
alp_global_settings_set_notification_on_off(AlpGlobalSettingsContext *context, int on_off)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    
    gs_start_pack(&msg, PROC_set_notification_on_off);
    alp_ipc_message_pack_uint32(msg, (uint32_t) on_off);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    
    return status;
}


alp_status_t 
alp_global_settings_set_default_mode(AlpGlobalSettingsContext *context, mode_t mode)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    
    
    gs_start_pack(&msg, PROC_set_default_mode);
    alp_ipc_message_pack_uint32(msg, (uint32_t) mode);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    
    return status;
}

alp_status_t 
alp_global_settings_get_default_mode(AlpGlobalSettingsContext *context, mode_t *mode)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    
    
    gs_start_pack(&msg, PROC_get_default_mode);
    
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
            if (status == ALP_STATUS_OK)
            {

                alp_ipc_message_unpack_uint32(resp, &i);
                *mode = i;
            }
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
       
    
    return status;
}


alp_status_t 
alp_global_settings_begin_transaction(AlpGlobalSettingsContext *context)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    
    
    BEGIN_BUSY_LOOP
    
        gs_start_pack(&msg, PROC_begin_transaction);

        alp_ipc_message_pack_end(msg);
        status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
        if (status == ALP_STATUS_OK)
        {
            alp_ipc_message_unpack_start(resp);
            alp_ipc_message_unpack_uint32(resp, &i);
            if (i != ALP_GLOBAL_SETTINGS_VERSION)
                status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
            else
            {
                alp_ipc_message_unpack_uint32(resp, &i);
                status = i;
            }
            alp_ipc_message_unpack_end(resp);
        }
        alp_ipc_message_destroy(msg);     
        if (resp)
            alp_ipc_message_destroy(resp);     
    
    END_BUSY_LOOP
        
    
    return status;
}


alp_status_t 
alp_global_settings_complete_transaction(AlpGlobalSettingsContext *context)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    
    BEGIN_BUSY_LOOP
    
        gs_start_pack(&msg, PROC_complete_transaction);

        alp_ipc_message_pack_end(msg);
        status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
        if (status == ALP_STATUS_OK)
        {
            alp_ipc_message_unpack_start(resp);
            alp_ipc_message_unpack_uint32(resp, &i);
            if (i != ALP_GLOBAL_SETTINGS_VERSION)
                status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
            else
            {
                alp_ipc_message_unpack_uint32(resp, &i);
                status = i;
            }
            alp_ipc_message_unpack_end(resp);
        }
        alp_ipc_message_destroy(msg);     
        if (resp)
            alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP
    
    return status;

}

alp_status_t 
alp_global_settings_cancel_transaction(AlpGlobalSettingsContext *context)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    
    BEGIN_BUSY_LOOP
    
        gs_start_pack(&msg, PROC_cancel_transaction);

        alp_ipc_message_pack_end(msg);
        status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
        if (status == ALP_STATUS_OK)
        {
            alp_ipc_message_unpack_start(resp);
            alp_ipc_message_unpack_uint32(resp, &i);
            if (i != ALP_GLOBAL_SETTINGS_VERSION)
                status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
            else
            {
                alp_ipc_message_unpack_uint32(resp, &i);
                status = i;
            }
            alp_ipc_message_unpack_end(resp);
        }
        alp_ipc_message_destroy(msg);     
        if (resp)
            alp_ipc_message_destroy(resp);     

    END_BUSY_LOOP

    return status;



}

alp_status_t 
alp_global_settings_set_mode(AlpGlobalSettingsContext *context, const char *key, mode_t mode)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP
    
    gs_start_pack(&msg, PROC_set_mode);
    alp_ipc_message_pack_string(msg, absolute_key);
    alp_ipc_message_pack_uint32(msg, (uint32_t) mode);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP
    
    return status;
}

alp_status_t 
alp_global_settings_get_mode(AlpGlobalSettingsContext *context, const char *key, mode_t *mode)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP
 
    gs_start_pack(&msg, PROC_get_mode);
    alp_ipc_message_pack_string(msg, absolute_key);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
            if (status == ALP_STATUS_OK)
            {

                alp_ipc_message_unpack_uint32(resp, &i);
                *mode = i;
            }
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP
    
    return status;
}

alp_status_t 
alp_global_settings_set_uid(AlpGlobalSettingsContext *context, const char *key, uid_t id)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP
    
    gs_start_pack(&msg, PROC_set_uid);
    alp_ipc_message_pack_string(msg, absolute_key);
    alp_ipc_message_pack_uint32(msg, (uint32_t) id);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP
    
    return status;

}

alp_status_t 
alp_global_settings_get_uid(AlpGlobalSettingsContext *context, const char *key, uid_t *id)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP
    
    gs_start_pack(&msg, PROC_get_uid);
    alp_ipc_message_pack_string(msg, absolute_key);
    
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
            if (status == ALP_STATUS_OK)
            {
                alp_ipc_message_unpack_uint32(resp, &i);
                *id = i;
            }
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP
    
    return status;

}


alp_status_t 
alp_global_settings_set_gid(AlpGlobalSettingsContext *context, const char *key, gid_t id)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP
    
    gs_start_pack(&msg, PROC_set_gid);
    alp_ipc_message_pack_string(msg, absolute_key);
    alp_ipc_message_pack_uint32(msg, (uint32_t) id);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
            
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP    
    
    return status;

}

alp_status_t 
alp_global_settings_get_gid(AlpGlobalSettingsContext *context, const char *key, gid_t *id)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP
    
    gs_start_pack(&msg, PROC_get_gid);
    alp_ipc_message_pack_string(msg, absolute_key);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
            if (status == ALP_STATUS_OK)
            {

                alp_ipc_message_unpack_uint32(resp, &i);
                *id = i;
            }
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
    
    END_BUSY_LOOP
        
    
    return status;
  

}


alp_status_t
alp_global_settings_set( AlpGlobalSettingsContext *context,
                     const gchar *key,
                     const AlpGlobalSettingsValue *value)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP
        gs_start_pack(&msg, PROC_set);
        alp_ipc_message_pack_string(msg, absolute_key);
        gs_pack_value(msg, value);
        alp_ipc_message_pack_end(msg);
        status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
        if (status == ALP_STATUS_OK)
        {
            alp_ipc_message_unpack_start(resp);
            alp_ipc_message_unpack_uint32(resp, &i);
            if (i != ALP_GLOBAL_SETTINGS_VERSION)
                status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
            else
            {
                alp_ipc_message_unpack_uint32(resp, &i);
                status = i;
            }
            alp_ipc_message_unpack_end(resp);
        }
        alp_ipc_message_destroy(msg);     
        if (resp)
            alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP
    
    return status;
}

alp_status_t
alp_global_settings_get( AlpGlobalSettingsContext *context,
                     const gchar *key,
                     AlpGlobalSettingsValue *value)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP


        gs_start_pack(&msg, PROC_get);
        alp_ipc_message_pack_string(msg, absolute_key);
        alp_ipc_message_pack_end(msg);
        status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
        if (status == ALP_STATUS_OK)
        {
            alp_ipc_message_unpack_start(resp);
            alp_ipc_message_unpack_uint32(resp, &i);
            if (i != ALP_GLOBAL_SETTINGS_VERSION)
                status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
            else
            {
                alp_ipc_message_unpack_uint32(resp, &i);
                status = i;
                if (status == ALP_STATUS_OK)
                {
                    gs_unpack_value(resp, value);
                }

            }
            alp_ipc_message_unpack_end(resp);
        }
        alp_ipc_message_destroy(msg);     
        if (resp)
            alp_ipc_message_destroy(resp);     
       
    END_BUSY_LOOP
    
    return status;
}

alp_status_t 
alp_global_settings_get_type( AlpGlobalSettingsContext *context, 
                     const char *key, 
                     AlpGlobalSettingsValueType *type)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP
    
    gs_start_pack(&msg, PROC_get_type);
    alp_ipc_message_pack_string(msg, absolute_key);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
            if (status == ALP_STATUS_OK)
            {
                alp_ipc_message_unpack_uint32(resp, &i);
                *type = (AlpGlobalSettingsValueType) i;
            }
            
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
   
    END_BUSY_LOOP

        
    
    return status;
}
                     

alp_status_t 
alp_global_settings_get_value_size( AlpGlobalSettingsContext *context, 
                     const char *key, 
                     size_t *size)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP
    
    gs_start_pack(&msg, PROC_get_value_size);
    alp_ipc_message_pack_string(msg, absolute_key);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
            if (status == ALP_STATUS_OK)
            {
                alp_ipc_message_unpack_uint32(resp, &i);
                *size = i;
            }
            
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
    
    END_BUSY_LOOP
        
    
    return status;
}


alp_status_t
alp_global_settings_delete( AlpGlobalSettingsContext *context, const gchar *key)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP
    
    gs_start_pack(&msg, PROC_delete);
    alp_ipc_message_pack_string(msg, absolute_key);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
    
    END_BUSY_LOOP
    
    
    return status;
}  

alp_status_t 
alp_global_settings_delete_tree( AlpGlobalSettingsContext *context, 
                           const char *key)  
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP

    gs_start_pack(&msg, PROC_delete_tree);
    alp_ipc_message_pack_string(msg, absolute_key);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP

    
    return status;



}


alp_status_t 
alp_global_settings_delete_tree_keeping_defaults( AlpGlobalSettingsContext *context, 
                           const char *key)  
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_key[PATH_MAX];
    form_absolute_path(context->cwd, key, absolute_key);
    
    BEGIN_BUSY_LOOP

    gs_start_pack(&msg, PROC_delete_tree_keeping_defaults);
    alp_ipc_message_pack_string(msg, absolute_key);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP
    
    return status;



}


alp_status_t
alp_global_settings_import_from_file(AlpGlobalSettingsContext *context, 
                                   const char *file)
{

    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    
    char cwd[PATH_MAX], absolute_file[PATH_MAX];
    getcwd(cwd, sizeof(cwd));
    form_absolute_path(cwd, file, absolute_file);
    
    BEGIN_BUSY_LOOP
    
    gs_start_pack(&msg, PROC_import_from_file);
    alp_ipc_message_pack_string(msg, absolute_file);
    alp_ipc_message_pack_string(msg, context->cwd);
   
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
            
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP
    
    return status;


}
alp_status_t
alp_global_settings_findkeys_start( AlpGlobalSettingsContext *context, const gchar *dir_path, AlpGlobalSettingsEntryIterator **iterator)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_path[PATH_MAX];
    form_absolute_path(context->cwd, dir_path, absolute_path);
    
    BEGIN_BUSY_LOOP

    gs_start_pack(&msg, PROC_findkeys_start);
    alp_ipc_message_pack_string(msg, absolute_path);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
            
            if (status == ALP_STATUS_OK)
            {
                *iterator = calloc(1, sizeof(**iterator));
                gs_svr_unpack_entry_iterator(resp, *iterator);
            }
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP

    return status;
}

alp_status_t
alp_global_settings_list_keys( AlpGlobalSettingsContext *context, const gchar *pattern, AlpGlobalSettingsEntryIterator **iterator)
{
    AlpMessage *msg = 0, *resp = 0;
    alp_status_t status;
    uint32_t i;
    char absolute_path[PATH_MAX];
    form_absolute_path(context->cwd, pattern, absolute_path);
    
    BEGIN_BUSY_LOOP
    
    gs_start_pack(&msg, PROC_list_keys);
    alp_ipc_message_pack_string(msg, absolute_path);
    alp_ipc_message_pack_end(msg);
    status = alp_ipc_connection_send_with_response(context->connection, msg, GLOBAL_SETTINGS_TIMEOUT, &resp);
    if (status == ALP_STATUS_OK)
    {
        alp_ipc_message_unpack_start(resp);
        alp_ipc_message_unpack_uint32(resp, &i);
        if (i != ALP_GLOBAL_SETTINGS_VERSION)
            status = ALP_STATUS_GLOBAL_SETTINGS_VERSION_MISMATCH;
        else
        {
            alp_ipc_message_unpack_uint32(resp, &i);
            status = i;
            
            if (status == ALP_STATUS_OK)
            {
                *iterator = calloc(1, sizeof(**iterator));
                gs_svr_unpack_entry_iterator(resp, *iterator);
            }
        }
        alp_ipc_message_unpack_end(resp);
    }
    alp_ipc_message_destroy(msg);     
    if (resp)
        alp_ipc_message_destroy(resp);     
        
    END_BUSY_LOOP
    
    return status;
}


alp_status_t
alp_global_settings_findkeys_next(AlpGlobalSettingsEntryIterator *iterator)
{
    CHECK_ITERATOR
    do
    {
        iterator->current = iterator->current->next;
    } while (iterator->current && (!strcmp(iterator->current->data, ".") || !strcmp(iterator->current->data, "..")));
    
    if (iterator->current == NULL)
    {
        return ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_AT_END;
    }
    return ALP_STATUS_OK;
}

alp_status_t
alp_global_settings_findkeys_end(AlpGlobalSettingsEntryIterator *iterator)
{
    CHECK_ITERATOR
    GList *tmp;
    for (tmp = iterator->entries; tmp != NULL; tmp = tmp->next)
        free(tmp->data);
    g_list_free(iterator->entries);
    free(iterator->base_path);
    free(iterator);  
    
    
    return ALP_STATUS_OK;
}

alp_status_t
alp_global_settings_findkeys_get_key(AlpGlobalSettingsEntryIterator  *iterator,
                               gchar *key,
                               guint key_size)
{
    CHECK_ITERATOR
    if (key_size <= strlen(((const char*) iterator->current->data)))
        return ALP_STATUS_GLOBAL_SETTINGS_BUFFER_TOO_SMALL ;
    strncpy(key, ((const char*) iterator->current->data), key_size);
    
    return ALP_STATUS_OK;
}


alp_status_t
alp_global_settings_set_key_string_value(AlpGlobalSettingsContext *context, 
                                   const gchar *key, 
                                   const gchar *value)
{
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status = alp_global_settings_value_set_string(&gsvalue, value);
    if (status != ALP_STATUS_OK)
        return status;
    status = alp_global_settings_set(context, key, &gsvalue);
    global_settings_value_clean(&gsvalue);
    return status;

}

alp_status_t
alp_global_settings_get_key_string_value(AlpGlobalSettingsContext *context, 
                                   const gchar *key, 

                                   gchar *value, 
                                   guint value_size)
{                             
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status = alp_global_settings_get(context, key, &gsvalue);
    if (status != ALP_STATUS_OK)
        return status;
    status = alp_global_settings_value_get_string(&gsvalue, value, value_size);
    global_settings_value_clean(&gsvalue);
    return status;

}


alp_status_t
alp_global_settings_set_key_bool_value(AlpGlobalSettingsContext *context, 
                                   const gchar *key, 
                                   gboolean value)
{
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status = alp_global_settings_value_set_bool(&gsvalue, value);
    if (status != ALP_STATUS_OK)
        return status;
    status = alp_global_settings_set(context, key, &gsvalue);
    global_settings_value_clean(&gsvalue);
    return status;

}

alp_status_t
alp_global_settings_get_key_bool_value(AlpGlobalSettingsContext *context, 
                                   const gchar *key, 
                                   gboolean *value)
{                             
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status = alp_global_settings_get(context, key, &gsvalue);
    if (status != ALP_STATUS_OK)
        return status;
    status = alp_global_settings_value_get_bool(&gsvalue, value);
    global_settings_value_clean(&gsvalue);
    return status;

}



alp_status_t
alp_global_settings_set_key_int_value(AlpGlobalSettingsContext *context, 
                                   const gchar *key, 
                                   int value)
{
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status = alp_global_settings_value_set_int(&gsvalue, value);
    if (status != ALP_STATUS_OK)
        return status;
    status = alp_global_settings_set(context, key, &gsvalue);
    global_settings_value_clean(&gsvalue);
    return status;

}

alp_status_t
alp_global_settings_get_key_int_value(AlpGlobalSettingsContext *context, 
                                   const gchar *key, 
                                   int *value)
{                             
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status = alp_global_settings_get(context, key, &gsvalue);
    if (status != ALP_STATUS_OK)
        return status;
    status = alp_global_settings_value_get_int(&gsvalue, value);
    global_settings_value_clean(&gsvalue);
    return status;

}


alp_status_t
alp_global_settings_set_key_float_value(AlpGlobalSettingsContext *context, 
                                   const gchar *key, 
                                   double value)
{
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status = alp_global_settings_value_set_float(&gsvalue, value);
    if (status != ALP_STATUS_OK)
        return status;
    status = alp_global_settings_set(context, key, &gsvalue);
    global_settings_value_clean(&gsvalue);
    return status;

}

alp_status_t
alp_global_settings_get_key_float_value(AlpGlobalSettingsContext *context, 
                                   const gchar *key, 
                                   double *value)
{                             
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status = alp_global_settings_get(context, key, &gsvalue);
    if (status != ALP_STATUS_OK)
        return status;
    status = alp_global_settings_value_get_float(&gsvalue, value);
    global_settings_value_clean(&gsvalue);
    return status;

}

alp_status_t
alp_global_settings_get_key_blob_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   char  *value,
                                   int buffer_size,
                                   int * value_size)
{                             
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status = alp_global_settings_get(context, key, &gsvalue);
    if (status != ALP_STATUS_OK)
        return status;
    if (gsvalue.type !=  ALP_GLOBAL_SETTINGS_TYPE_BLOB)
        status =  ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE ;
    else if ((int)(gsvalue.blob_value.len) > buffer_size)
        status = ALP_STATUS_GLOBAL_SETTINGS_BUFFER_TOO_SMALL ;
    else 
    {
        memcpy(value, gsvalue.blob_value.data, gsvalue.blob_value.len );
        *value_size = gsvalue.blob_value.len;
    }
    global_settings_value_clean(&gsvalue);
    return status;

}


alp_status_t
alp_global_settings_set_key_blob_value(AlpGlobalSettingsContext *context, 
                                   const char *key, 
                                   const char *value,
                                   int value_size)
{
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status ;
    gsvalue.type = ALP_GLOBAL_SETTINGS_TYPE_BLOB;
    gsvalue.blob_value.data = (char*) value; 
    /* note we use blob_value as a carrier to eliminate the need of an extra local copy */
    gsvalue.blob_value.len = value_size;
    status = alp_global_settings_set(context, key, &gsvalue);
    /* so we do not free the value here */
    return status;

}

alp_status_t
alp_global_settings_set_key_directory(AlpGlobalSettingsContext *context, 
                                   const char *key)
{
    AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
    alp_status_t status ;
    gsvalue.type = ALP_GLOBAL_SETTINGS_TYPE_DIR;
    status = alp_global_settings_set(context, key, &gsvalue);
    /* so we do not free the value here */
    return status;

}



                                   
