/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file global_settings_server.h
 *
 * @brief the header file for the Global Settings server
 *
 *****************************************************************************/

#ifndef ALP_GLOBAL_SETTINGS_SERVER_H_
#define ALP_GLOBAL_SETTINGS_SERVER_H_ 1

#ifdef __cplusplus
extern "C" {
#endif

#include <glib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <time.h>

#include <hiker/config.h>
#include <hiker/sysclass.h>
#include <hiker/types.h>
#include <hiker/ipc.h>
#include "hiker/global_settings.h"

#include "sqlfs/sqlfs.h"


#define TRANSACTION_CHECK_INTERVAL 10 /*seconds */

typedef struct gs_svr_context
{
    sqlfs_t *sqlfs;
    int notification_on; /* to broadcast change notification or not */
    GList *node_stack;   
    char *cwd; 
    /*GHashTable *callback_table;
    GHashTable *any_callback_table;*/
    int transaction_level;
    time_t last_op_time;
    pid_t pid;
    int thread_id;
    int to_close;
    char *default_uid_in_import;
    char *default_gid_in_import;
  
    
} gs_svr_context;

int AlpGlobalSettingsServerMain(int argc, char *argv[]);

alp_status_t alp_global_settings_server_wait_started(void);

alp_status_t alp_global_settings_server_running(void);

alp_status_t alp_global_settings_server_stop(void);

alp_status_t Alp_global_settings_server_clean_data(void);

extern alp_status_t 
gs_svr_init(const char *db_file);

extern alp_status_t
gs_svr_open(const char *db_file, gs_svr_context **context);

extern alp_status_t
gs_svr_close(gs_svr_context *context);

extern alp_status_t
gs_svr_proc_import_from_file(gs_svr_context *context, 
                                   const char *file);

extern alp_status_t 
gs_svr_proc_delete_tree(gs_svr_context *context, const gchar *key);


extern alp_status_t 
gs_svr_proc_begin_transaction(gs_svr_context *context);


extern alp_status_t 
gs_svr_proc_complete_transaction(gs_svr_context *context);

extern alp_status_t 
gs_svr_proc_cancel_transaction(gs_svr_context *context);


/*!
@}
*/

#ifdef __cplusplus
}       // extern "C"
#endif

#endif  // #ifndef ALP_GLOBAL_SETTINGS_SERVER_H_
