/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file gs_imp.h
 *
 * @brief data types used in 
 *       the implementation of the Global Settings server.
 *
 *****************************************************************************/

#ifndef __GS_IMP_H__
#define __GS_IMP_H__

#ifdef __cplusplus
extern "C" {
#endif

#include <pthread.h>
#include <assert.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <glib.h>

#include "hiker/global_settings.h"
	 
#include "hiker/ipc.h"
#include "proc_id.h"

#define ALP_STATUS_GLOBAL_SETTINGS_BUSY ((alp_status_t) ALP_CLASS_GLOBAL_SETTINGS | 0x40000000)

struct _AlpGlobalSettingsEntryIterator
{
    GList *entries;
    GList *current;
    char *base_path;
} ;

#if 0
struct _Callback
{
    AlpGlobalSettingsContext *context;
    /*AlpGlobalSettingsCallback func;*/
    int exact_key;
    GString *key;
    int id; /*Gconf callback ID */
};
#endif



#define CHECK_ARG(arg)\
    if ((arg) == NULL)\
    {\
        ALP_TM(T(ALP_CLASS_GLOBAL_SETTINGS, "zero " #arg " pointer supplied"));\
        return ALP_STATUS_GLOBAL_SETTINGS_ARGUMENT_INVALID ;\
    }


#define CHECK_ARG_VOID_RETURN(arg)\
    if ((arg) == NULL)\
    {\
        ALP_TM(T(ALP_CLASS_GLOBAL_SETTINGS, "zero " #arg " pointer supplied"));\
        return ;\
    }

#define CHECK_CONTEXT\
    CHECK_ARG(context)

#define CHECK_ITERATOR\
    CHECK_ARG(iterator)

#define CHECK_TYPE(v, t)\
    if ((v)->type != (t))\
    {\
        ALP_TM(T(ALP_CLASS_GLOBAL_SETTINGS, "incorrect value type"));\
        return ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE ;\
    }
    

    
void free_glist(GList *list);
void free_gslist(GSList *list);
void gstring_copy(GString **gs, const gchar *src);    
void global_settings_value_clean(AlpGlobalSettingsValue *value);
void global_settings_entry_clean(AlpGlobalSettingsEntry *entry);
void global_settings_value_copy(const AlpGlobalSettingsValue *src, AlpGlobalSettingsValue *des);
void global_settings_value_list_to_GSList(const AlpGlobalSettingsValue *value, GSList **list);
void global_settings_list_copy(const GList *src, GList **dst);
AlpGlobalSettingsValue * global_settings_value_duplicate(const AlpGlobalSettingsValue *src);        
AlpGlobalSettingsEntry *alp_global_settings_entry_alloc(void);
void alp_global_settings_entry_free(AlpGlobalSettingsEntry *e);    
AlpGlobalSettingsValue *alp_global_settings_value_alloc(void);
void alp_global_settings_value_free(AlpGlobalSettingsValue *v);
void alp_global_settings_entry_iterator_free(AlpGlobalSettingsEntryIterator *);

const char * gs_type_to_string(AlpGlobalSettingsValueType t);
const char *gs_bool_to_string(gboolean b);

int gs_pack_value(AlpMessage* msg, const AlpGlobalSettingsValue *v);
int gs_unpack_value(AlpMessage* msg, AlpGlobalSettingsValue *v);
void gs_start_pack(AlpMessage** msg, ALP_GLOBAL_SETTINGS_RPC_PROCS);
int gs_end_pack(AlpMessage* msg);
int gs_svr_pack_entry_iterator(AlpMessage* msg, const AlpGlobalSettingsEntryIterator * iterator);
int gs_svr_unpack_entry_iterator(AlpMessage* msg, AlpGlobalSettingsEntryIterator * iterator);

int is_dir(const char *path);
void normalize_path(char *path);
int form_absolute_path(const char *cwd, const char *key, char result[PATH_MAX]);

const char *status_to_string(int status);

#ifdef __cplusplus
}       // extern "C"
#endif

#endif
