/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file gset_tool.c
 *
 * @brief 
 *       the implementation of the Global Settings server command line tool.
 *
 *****************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>
#include <getopt.h>

#include "hiker/global_settings.h"
#include "gs_imp.h"

static AlpGlobalSettingsContext *context = 0;

static void show_usage(const char *argv0)
{
    fprintf(stderr, "Usage: %s [--install file_name] [--base-dir base_dir] [--get key]"
            " [--set key --value value] [--delete-tree key_dir] [--delete key] [--uid uid] [--Get key] [--list pattern]"
            "[--mode mode] [--no-notify]\n", argv0);
}


int main(int argc, char *argv[])
{
    int c, r = 0, processed = 0, i;
    int delete_tree = 0;
    int delete = 0;
    int set
        = 0;
    int get
        = 0;
    int Get = 0;
    int uid = 0;
    int uid_set = 0;
    int mode_set = 0;
    int notification_on = 1;
    unsigned int mode ;
    char xml_file[PATH_MAX] = { 0 } , base_dir[PATH_MAX] = { 0 }, key[PATH_MAX] = { 0 }, *value = 0;
    char *pattern = 0;
    char tmp[1000];
    alp_status_t status;
    static struct option long_options[] =
        {
            { "install", 1, 0, 'i' },
            { "base-dir", 1, 0, 'b' },
            { "get", 1, 0, 'g' },
            { "set", 1, 0, 's' },
            { "value", 1, 0, 'v' },
            { "delete-tree", 1, 0, 'D'} ,
            { "uid", 1, 0, 'u'} ,
            { "Get", 1, 0, 'G'},
            { "delete", 1, 0, 'd'} ,
            { "mode", 1, 0, 'm'},
            { "list", 1, 0, 'l'},
            { "no-notify", 0, 0, 'n'},
            { 0, 0, 0, 0 }
        } ;



    while (1)
    {
        c = getopt_long(argc, argv, "i:b:g:s:u:l:n", long_options, NULL);
        switch (c)
        {
        case 'i':
            strncpy(xml_file, optarg, sizeof(xml_file));
            processed = 1;
            break;
        case 'b':
            strncpy(base_dir, optarg, sizeof(base_dir));
            processed = 1;
            break;
        case 'g':
            strncpy(key, optarg, sizeof(key));
            processed = 1;
            get
                = 1;
            break;
        case 'G':
            strncpy(key, optarg, sizeof(key));
            processed = 1;
            Get
                = 1;
            break;

        case 's':
            strncpy(key, optarg, sizeof(key));
            processed = 1;
            set
                = 1;
            break;

        case 'v':
            value = strdup(optarg);
            processed = 1;
            break;

        case 'D':
            strncpy(key, optarg, sizeof(key));
            processed = 1;
            delete_tree = 1;
            break;
        case 'd':
            strncpy(key, optarg, sizeof(key));
            processed = 1;
            delete = 1;
            break;
        case 'm':
            i = sscanf(optarg, "%o", &mode);
            if ((i != 1) || (mode > 0777))
            {
                fprintf(stderr, "Invalid mode %s\n", optarg);
                exit(1);
            }
            mode_set = 1;
            processed = 1;
            break;
            
        case 'u':
            i = sscanf(optarg, "%d", &uid);
            if (i != 1)
            {
                fprintf(stderr, "Invalid user id %s\n", optarg);
                exit(1);
            }
            uid_set = 1;
            processed = 1;
            break;
        case 'l':
            pattern = strdup(optarg);
            processed = 1;
            break;
            
        case 'n':
            notification_on = 0;
            processed = 1;
            break;
            
        case '?':            
            {
                show_usage(argv[0]);
                exit(1);
            }
            
        default:
            break;
        }
        if (c <= 0)
            break;
    }

    if (processed == 0)
    {
        show_usage(argv[0]);
        exit(1);
    }

    status = alp_global_settings_init();
    if (status == ALP_STATUS_OK)
        status = alp_global_settings_open(&context);

    if (context)
    {
        
        alp_global_settings_set_notification_on_off(context, notification_on);
        
        alp_global_settings_begin_transaction(context);
        alp_global_settings_set_default_mode(context, 0700);
        if (base_dir[0])
        {
            status = alp_global_settings_set_cwd(context, base_dir);
            if (status != ALP_STATUS_OK)
            {
                fprintf(stderr, "cannot set base directory to %s\n", base_dir);
                r = 1;
            }
        }
        
        
        if (status == ALP_STATUS_OK)
            if (xml_file[0])
            {
                r = alp_global_settings_import_from_file(context, xml_file);
            }
        if (status == ALP_STATUS_OK)
        {
            if (delete_tree)
            {
                status = alp_global_settings_delete_tree(context, key);
                if (status != ALP_STATUS_OK)
                {
                    fprintf(stderr, "delete key dir '%s' failed,  %s\n", key, status_to_string(status));
                    r = 1;
                }
            }
        }
        if (status == ALP_STATUS_OK)
        {
            if (key[0] && (value == 0) && (get || Get))
                {
                    int unprintable = 0;
                    AlpGlobalSettingsValue *value = alp_global_settings_value_alloc();
                    status = alp_global_settings_get(context, key, value);
                    if (status == ALP_STATUS_OK)
                    {
                        char *s = tmp;
                        switch (value->type)
                        {
                        case ALP_GLOBAL_SETTINGS_TYPE_STRING:
                            s = value->string_value->str;
                            break;
                        case ALP_GLOBAL_SETTINGS_TYPE_INT:
                            snprintf(s, sizeof(tmp), "%d",  value->int_value);
                            break;
                        case ALP_GLOBAL_SETTINGS_TYPE_FLOAT:
                            snprintf(s, sizeof(tmp), "%lf", value->float_value);
                            break;
                        case ALP_GLOBAL_SETTINGS_TYPE_BOOL:
                            snprintf(s, sizeof(tmp), "%s", gs_bool_to_string(value->bool_value));
                            break;
                        case ALP_GLOBAL_SETTINGS_TYPE_BLOB:
                            s = malloc(sizeof(char) * value->blob_value.len + 1) ;
                            memcpy(s, value->blob_value.data, value->blob_value.len);
                            s[value->blob_value.len] = 0;

                            break;
                        case ALP_GLOBAL_SETTINGS_TYPE_LIST:
                            strncpy(s, "is a list", sizeof(tmp));
                            unprintable = 1;
                            break;
                        case ALP_GLOBAL_SETTINGS_TYPE_DIR:
                            strncpy(s, "is a dir", sizeof(tmp));
                            unprintable = 1;
                            break;
                        default:
                            strncpy(s, "is invalid", sizeof(tmp));
                            unprintable = 1;
                            break;

                        }
                        if (unprintable)
                            fprintf(stderr, "value of '%s' %s\n", key, s);
                        else
                            fprintf(stderr, "%s\n", s);

                        if (value->type == ALP_GLOBAL_SETTINGS_TYPE_BLOB)
                            free(s);
                        if (Get)
                        {
                            mode_t mode = (mode_t) -1;
                            gid_t gid = (gid_t) -1;
                            uid_t uid = (uid_t) -1;
                            i = (ALP_STATUS_OK == alp_global_settings_get_uid(context, key, &uid)) &&
                            (ALP_STATUS_OK == alp_global_settings_get_gid(context, key, &gid)) && 
                            (ALP_STATUS_OK == alp_global_settings_get_mode(context, key, &mode));
                            if (i)
                                fprintf(stderr, "gid: %d, uid: %d, mode: %o\n", gid, uid, mode);
                            else
                                fprintf(stderr, "meta data of key %s not available\n", key);
                        }
                            
                    }
                    else
                    {
                        fprintf(stderr, "getting value of key '%s' failed,  %s\n", key, status_to_string(status));
                        r = 1;
                    }
                    alp_global_settings_value_free(value);
                }
            else if (key[0] && delete)
            {
                status = alp_global_settings_delete(context, key);
                if (status == ALP_STATUS_OK)
                {
                    fprintf(stderr, "key '%s' deleted\n", key);
                }
                else
                {
                    fprintf(stderr, "deleting key '%s' failed,  %s\n", key, status_to_string(status));
                    r = 1;
                }
            }
            else if (key[0]  && set)
            {

                if (value )
                {
                    status = alp_global_settings_set_key_string_value(context, key, value);
                    if (status == ALP_STATUS_OK)
                    {
                        fprintf(stderr, "value of key '%s' is %s\n", key, value);
                    }
                    else
                    {
                        fprintf(stderr, "setting value of key '%s' failed,  %s\n", key, status_to_string(status));
                        r = 1;
                    }
                }
                if ((r != 1) && (mode_set == 1))
                {
                    status = alp_global_settings_set_mode(context, key, (mode_t) mode);
                    if (status == ALP_STATUS_OK)
                    {
                        fprintf(stderr, "mode of key '%s' is 0%o\n", key, mode);
                    }
                    else
                    {
                        fprintf(stderr, "setting mode of key '%s' failed,  %s\n", key, status_to_string(status));
                        r = 1;
                    }
                }
                if ((r != 1) && (uid_set == 1))
                {
                    status = alp_global_settings_set_uid(context, key, (uid_t) uid);
                    if (status == ALP_STATUS_OK)
                    {
                        fprintf(stderr, "uid of key '%s' is 0%o\n", key, uid);
                    }
                    else
                    {
                        fprintf(stderr, "setting uid of key '%s' failed,  %s\n", key, status_to_string(status));
                        r = 1;
                    }
                }
            }
        }
        if (status == ALP_STATUS_OK)
        {
            if (pattern)
            {
            
                AlpGlobalSettingsEntryIterator *iterator = 0;
                status = alp_global_settings_list_keys(context, pattern, &iterator);
                while (status == ALP_STATUS_OK)
                {
                    fprintf(stderr, "%s\n", (const char*) iterator->current->data);
                    status = alp_global_settings_findkeys_next(iterator);

                } 
                if (status == ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_AT_END)
                    alp_global_settings_findkeys_end(iterator);
                
                
            
            }
        
        }
        alp_global_settings_complete_transaction(context);
        alp_global_settings_close(context);
    }
    if (value)
        free(value);
    if (pattern)
	free(pattern);
    return r;

}
