/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file serialize.h
 *
 * @brief fucntions for data type serialization in
 *       the implementation of the Global Settings server.
 *
 *****************************************************************************/


#ifndef __SERIALIZE_H__
#define __SERIALIZE_H__


#include <glib.h>
#include <stdint.h>

#include "hiker/global_settings.h"


struct serialize_buf
{
    void *data;
    void *current;
    int length;
};

typedef struct serialize_buf serialize_buf;

serialize_buf * make_serialize_buf(void);

void clean_serialize_buf(serialize_buf *);
void free_serialize_buf(serialize_buf *);

int serialize_bool(serialize_buf *, gboolean);
int serialize_int(serialize_buf *, int);
int serialize_float(serialize_buf *, double);
int serialize_string(serialize_buf *, const char *);
int serialize_blob(serialize_buf *, const char *, int length);


int deserialize_bool(serialize_buf *, gboolean *);
int deserialize_int(serialize_buf *, int *);
int deserialize_float(serialize_buf *, double *);
int deserialize_string(serialize_buf *, char **, int *length);
int deserialize_blob(serialize_buf *, char **, int *length);

int serialize_gs_value(serialize_buf *, const AlpGlobalSettingsValue *);

int deserialize_gs_value(serialize_buf *, AlpGlobalSettingsValue *);

#endif
