/********************************************************************

Copyright  2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
	
/*
 
Unit Tests for the Global Settings Application Preference Service

*/

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <hiker/unit_test.h>
#include <cppunit/extensions/HelperMacros.h>
#include "hiker/notifymgr.h"
#include <string.h> 
#include "hiker/global_settings.h"
/*#include "global_settings_server.h"*/
#include <hiker/traces.h>
#include <fcntl.h>

#define TEST_TIMEOUT_SEC 120  // Longer timeout for the slow Zylonite

//#define CPPUNIT_ASSERT(x) verify(x)
#define verify(x)   ((x)? 1 : (fprintf(stderr, "%s failed at %s:%d\n", #x, __FILE__, __LINE__), abort(), 1 ))

#define test_directory "/unittest"

class GlobalSettingsTest_AP: public CPPUNIT_NS::TestFixture
{
    CPPUNIT_TEST_SUITE(GlobalSettingsTest_AP);
    
    //CPPUNIT_TEST(service_start);
    CPPUNIT_TEST(modes_get_set);
    /*CPPUNIT_TEST(callbacks_register);*/
    CPPUNIT_TEST(basic_type_values_set_get);
    CPPUNIT_TEST(basic_type_values_set_get_2);
    CPPUNIT_TIMED_TEST(basic_operation_write_performance, TEST_TIMEOUT_SEC);
    CPPUNIT_TIMED_TEST(basic_operation_read_performance, TEST_TIMEOUT_SEC);
    CPPUNIT_TEST(values_set_get_entry);
    CPPUNIT_TEST(values_set_get_list);
    CPPUNIT_TEST(values_set_get_list_gsvalue_str);
    CPPUNIT_TEST(values_set_get_list_gsvalue_float);
    CPPUNIT_TEST(values_set_get_list_gsvalue_mix);
    CPPUNIT_TEST(traversal_dir_entries_test);
    CPPUNIT_TEST(create_complex_tree); 
    CPPUNIT_TEST(delete_complex_tree); 
    CPPUNIT_TEST(tree_deletion);
    CPPUNIT_TEST(tree_deletion_questionable_dir);
    CPPUNIT_TEST(transactions);
    CPPUNIT_TIMED_TEST(run_away_transaction_handling, TEST_TIMEOUT_SEC);
    CPPUNIT_TEST(change_notification);
    CPPUNIT_TEST(import_from_xml_file);

    CPPUNIT_TEST(test_dir_list1);
    CPPUNIT_TEST(test_dir_list2);
    CPPUNIT_TEST(test_dir_list3); // to regress bug148967

    //CPPUNIT_TEST(service_end);
    
	CPPUNIT_TEST_SUITE_END();

public:
    GlobalSettingsTest_AP();

    void setUp(void);
    void tearDown(void);
    
protected:
    void service_start(void);
    void modes_get_set(void);
    /*void callbacks_register(void);*/
    
    void basic_type_values_set_get(void);
    void basic_type_values_set_get_2(void);
    void basic_operation_write_performance(void);
    void basic_operation_read_performance(void);
    void values_set_get_entry(void);
    void values_set_get_list(void);
	void values_set_get_list_gsvalue_str(void);
	void values_set_get_list_gsvalue_float(void);
	void values_set_get_list_gsvalue_mix(void);
    void traversal_dir_entries_test(void);
    void create_complex_tree(void); 
    void tree_deletion(void);
    void tree_deletion_questionable_dir(void);
    void _tree_deletion(const char* dir);
    void transactions(void);
    void run_away_transaction_handling(void);
    void change_notification(void);
    void service_end(void);
    void delete_complex_tree(void);
	void _delete_complex_tree(bool isNegativeTest = false);

    static void change_handler(AlpGlobalSettingsContext *c, const
        AlpGlobalSettingsEntry *entry);
    static void any_change_handler(AlpGlobalSettingsContext *c, const
        AlpGlobalSettingsEntry *entry);
    void glib_event_process(void);
	void import_from_xml_file(void);
	int  _prv_full_write(int fd, void* ibuf, size_t ibuflen);
	void _list_key_test(const char* pattern, int expN=0, const char* expKey=NULL);
	void _create_dir_traverse_entries(const char* keydir);

	void test_dir_list1(void);
	void test_dir_list2(void);
	void test_dir_list3(void);

private:
    
    alp_status_t status;
};

    
AlpGlobalSettingsValue *gsvalue, *gsvalue2, *gsvalue3;
AlpGlobalSettingsContext *context, *context2;
  
static const char *value = "text1", *key = "key", *value2 = "string2";
static const uid_t test_user = 1012;


ALP_ROOT_TEST_SUITE_REGISTRATION( "GS-001_A", CPPUNIT_REGISTRY_TYPE_AUTO );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( GlobalSettingsTest_AP, "GS-001_A" );

#if 0
static int server_main(void)
{
    char db_file[] = "testdb";
    char *argv[] = { "server", db_file, 0 };
    
    return AlpGlobalSettingsServerMain(2, argv);
}
#endif

GlobalSettingsTest_AP::GlobalSettingsTest_AP()
{
    context = 0; 
    context2 = 0;
    gsvalue = 0;
    gsvalue2 = 0;
    gsvalue3 = 0;   
}

void GlobalSettingsTest_AP::setUp()
{
	service_start();

    gsvalue = alp_global_settings_value_alloc();
    gsvalue2 = alp_global_settings_value_alloc();
    gsvalue3 = alp_global_settings_value_alloc();
    CPPUNIT_ASSERT(!((gsvalue == NULL) || (gsvalue2 == NULL) || (gsvalue3 == NULL)));
}

void GlobalSettingsTest_AP::tearDown()
{
	service_end();

    alp_global_settings_value_free(gsvalue);
    alp_global_settings_value_free(gsvalue2);
    alp_global_settings_value_free(gsvalue3);
    context = 0; 
    context2 = 0;
    gsvalue = 0;
    gsvalue2 = 0;
    gsvalue3 = 0;   
}

void GlobalSettingsTest_AP::change_handler(AlpGlobalSettingsContext *c, const AlpGlobalSettingsEntry *entry)
{
    printf("change: key: %s, new value: %s\n", entry->key->str, entry->value.string_value->str);
}

void GlobalSettingsTest_AP::any_change_handler(AlpGlobalSettingsContext *c, const AlpGlobalSettingsEntry *entry)
{
    printf("any change: key: %s, new value: %s\n", entry->key->str, entry->value.string_value->str);
}


void GlobalSettingsTest_AP::glib_event_process(void)
{
    
    while (g_main_context_pending(g_main_context_default()))
        g_main_context_iteration(g_main_context_default(), TRUE);
}



void GlobalSettingsTest_AP::service_start(void)
{

    int i;
    mode_t mode;

#if 0    
    alp_global_settings_server_stop();
    
    i = (int) fork();
    CPPUNIT_ASSERT(i >= 0);
    if (i == 0)
    {
        /* child */
        alp_global_settings_server_stop();
        exit(server_main());
        
    }
    else
    {
	    status = alp_global_settings_server_wait_started();
    }
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
#endif

    status = alp_global_settings_init();
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    status = alp_global_settings_open(&context);
   
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(context != 0);

    status = alp_global_settings_open(&context2);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(context2 != 0);

    alp_global_settings_set_default_mode(context, 0700);
    if (geteuid() == 0)
    {    

        alp_global_settings_delete_tree(context, test_directory);


        status = alp_global_settings_get_mode(context, test_directory, &mode);
        CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
#if 0    
        i = setuid(test_user);
        CPPUNIT_ASSERT(i == 0);
#endif


    }
    
    status = alp_global_settings_set_key_directory(context, test_directory); 
    /* create it if not exists, ignore the result if exists */

    status = alp_global_settings_set_cwd(context, test_directory);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_set_cwd(context2, test_directory);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    /* else the root key must have permissions o+rwx; else test fails */
}

void GlobalSettingsTest_AP::modes_get_set(void)
{
    const char *testkey = "testkey";
    char buf[200];
    mode_t mode;
    gid_t gid;
    uid_t uid;
    size_t size;
    
    status = alp_global_settings_get_mode(context, testkey, &mode);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
    
    status = alp_global_settings_set_gid(context, testkey, 100);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);

    status = alp_global_settings_set_uid(context, testkey, 200);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
    status = alp_global_settings_get_gid(context, testkey, &gid);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
    
    status = alp_global_settings_set_mode(context, testkey, 0123);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);

    status = alp_global_settings_get_uid(context, testkey, &uid);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
   
    status = alp_global_settings_set_key_string_value(context, testkey, "test value");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_set_default_mode(context, 0766);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_set_mode(context, testkey, 0626);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_set_gid(context, testkey, 100);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_set_uid(context, testkey, 200);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_get_default_mode(context, &mode);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    CPPUNIT_ASSERT((mode & 0777) == 0766);

    status = alp_global_settings_get_mode(context, testkey, &mode);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    CPPUNIT_ASSERT((mode & 0777) == 0626);    
    
    status = alp_global_settings_get_gid(context, testkey, &gid);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    CPPUNIT_ASSERT(gid == 100);    
       
    status = alp_global_settings_get_uid(context, testkey, &uid);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    CPPUNIT_ASSERT(uid == 200);
    
    status = alp_global_settings_set_default_mode(context, 0700);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); /* to be ready to allow directory read operations later */
    
    status = alp_global_settings_get_default_mode(context, &mode);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    CPPUNIT_ASSERT((mode & 0777) == 0700);

}
    

#if 0
void GlobalSettingsTest_AP::callbacks_register(void)
{
    
    CPPUNIT_ASSERT(context != 0);
    status = alp_global_settings_keychange_callback_register(context, "key", change_handler);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    status = alp_global_settings_anykeychange_callback_register(context, test_directory, any_change_handler);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

}
#endif

void GlobalSettingsTest_AP::basic_type_values_set_get(void)
{

    double f;
    int i;
    gboolean b;
    char str[100], string[] = "aabvcddfgklggl;dg", base_dir[] = test_directory "/numbers/numbers/numbers";
    char tmp[100];
    char blob[] = { 1, 2, 3, 4, 5, 6, 7, 8};
    const char *testkey2 = "tkey";
    const char *testkey2_default = "tkey.default";
    const char *value_default = "this is some value";
    size_t size;
    AlpGlobalSettingsValueType type;
    CPPUNIT_ASSERT(context != 0);
    
    
    status = alp_global_settings_get_key_string_value(context, "string",  str, sizeof(str));
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);

    status = alp_global_settings_set_cwd(context, base_dir);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_get_cwd(context, tmp, sizeof(tmp));
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(!strcmp(tmp, base_dir));
        
    status = alp_global_settings_value_set_string(gsvalue, value);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set(context, key, gsvalue);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    glib_event_process();
    status = alp_global_settings_get(context, key,  gsvalue2);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_value_get_string(gsvalue2, tmp, sizeof(tmp));
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    printf("result is %s\n", tmp);

    status = alp_global_settings_value_set_string(gsvalue, value2);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);        

    status = alp_global_settings_set(context, key, gsvalue);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    glib_event_process();
    
    status = alp_global_settings_get(context, key,  gsvalue3);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_value_get_string(gsvalue3, tmp, sizeof(tmp));
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    printf("result is %s\n", tmp);



    status = alp_global_settings_value_set_string(gsvalue, value);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set(context, key, gsvalue);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    glib_event_process();
    
    status = alp_global_settings_get(context, key,  gsvalue);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_value_get_string(gsvalue, tmp, sizeof(tmp));
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    printf("result is %s\n", tmp);
    
    status = alp_global_settings_set_key_blob_value(context, "blob", blob, sizeof(blob));
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_set_key_string_value(context, "string", string);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    glib_event_process();

    
    status = alp_global_settings_get_type(context, "string", &type);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(type == ALP_GLOBAL_SETTINGS_TYPE_STRING);
    
    status = alp_global_settings_get_value_size(context, "string", &size);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(size ==  strlen(string) + 1);

    
    status = alp_global_settings_get_key_string_value(context, "string",  str, sizeof(str));
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    CPPUNIT_ASSERT(!strcmp(str, string));

    status = alp_global_settings_set_key_int_value(context, "integer", 100);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    glib_event_process();
    status = alp_global_settings_get_key_int_value(context, "/integer",   &i);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);

    status = alp_global_settings_get_key_int_value(context, "integer",   &i);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    CPPUNIT_ASSERT(i == 100);
    
    //status = alp_global_settings_set_key_float_value(context, "float", 200.0);
    status = alp_global_settings_set_key_float_value(context, "float", 740.740740);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    glib_event_process();
    status = alp_global_settings_get_key_float_value(context, "/float",   &f);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);

    status = alp_global_settings_get_key_float_value(context, "float",   &f);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    //CPPUNIT_ASSERT(f == 200.0);
    fprintf(stderr, "f is %lf\n", f);
    CPPUNIT_ASSERT(fabs(f - 740.740740) < 0.00001 * 740.740740);

    status = alp_global_settings_set_key_bool_value(context, "boolean", TRUE);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    glib_event_process();
    
    status = alp_global_settings_get_key_bool_value(context, "/booleant",   &b);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);

    status = alp_global_settings_get_key_bool_value(context, "boolean",   &b);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    CPPUNIT_ASSERT(b == TRUE);
    
    status = alp_global_settings_set_key_bool_value(context, "boolean", FALSE);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    glib_event_process();

    status = alp_global_settings_get_key_bool_value(context, test_directory "/numbers/numbers/numbers/boolean",   &b);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    CPPUNIT_ASSERT(b == FALSE);
    
    status = alp_global_settings_get_type(context, "blob", &type);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(type == ALP_GLOBAL_SETTINGS_TYPE_BLOB);
    
    status = alp_global_settings_get_value_size(context, "blob", &size);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(size == sizeof(blob));
    
    status = alp_global_settings_get_key_blob_value(context, "blob", tmp, sizeof(tmp), &i);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    CPPUNIT_ASSERT(i == sizeof(blob));
    CPPUNIT_ASSERT(!memcmp(blob, tmp, sizeof(blob)));


    status = alp_global_settings_delete(context, "blob");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_delete(context, "blob");
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);


    status = alp_global_settings_set_key_string_value(context, testkey2_default, value_default);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_get_key_string_value(context, testkey2, tmp, sizeof(tmp));
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(!strcmp(tmp, value_default));


    status = alp_global_settings_get_value_size(context, testkey2, &size);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(size == strlen(value_default) + 1);


}

void GlobalSettingsTest_AP::traversal_dir_entries_test(void)
{
    char key[200];
    char buf[300];
    int length;
    int n;
    AlpGlobalSettingsValue *vt = alp_global_settings_value_alloc();
    alp_status_t status;
    AlpGlobalSettingsEntryIterator *iterator = 0;
    CPPUNIT_ASSERT(context != 0);

    status = alp_global_settings_set_key_string_value(context, "ddd/1", "value1");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    glib_event_process();

    status = alp_global_settings_set_key_string_value(context, "ddd/2", "value2");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    glib_event_process();

    status = alp_global_settings_set_key_string_value(context, "ddd/3", "value3");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    glib_event_process();

    status = alp_global_settings_set_key_string_value(context, "ddd/4", "value4");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    glib_event_process();

    status = alp_global_settings_set_key_string_value(context, "ddd/5/1", "value41");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    glib_event_process();

    status = alp_global_settings_findkeys_start(context, "ddd",  &iterator);
    printf("findkeys_start ddd status = 0x%x\n", status);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    n = 0;
    do
    {
        status = alp_global_settings_findkeys_get_key(iterator, key, sizeof(key));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        sprintf(buf, "ddd/%s", key); 
        printf("key: %s\n", key);
        status = alp_global_settings_get(context, buf, vt);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        if (strcmp(buf, "ddd/5"))
            CPPUNIT_ASSERT(vt->type == ALP_GLOBAL_SETTINGS_TYPE_STRING);
        else
            CPPUNIT_ASSERT(vt->type == ALP_GLOBAL_SETTINGS_TYPE_DIR);
        
        
        status = alp_global_settings_findkeys_next(iterator);
        n++;
    } while (status != ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_AT_END);
    
    status = alp_global_settings_findkeys_end(iterator);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    CPPUNIT_ASSERT(n == 5);

    // might as well test alp_global_settings_list_keys() API
    GlobalSettingsTest_AP::_list_key_test("*ddd/*[23]*", 2);

    GlobalSettingsTest_AP::_list_key_test("*ddd/*", 6);

    alp_global_settings_value_free(vt);
}


void GlobalSettingsTest_AP::_list_key_test(const char* pattern, int expN, const char* expKey)
{
    printf("dbg: _list_key_test(pattern=%s, expN=%d, expKey=%s)\n", pattern, expN, expKey);

	AlpGlobalSettingsEntryIterator* iterator = 0;

	status = alp_global_settings_list_keys(context, pattern, &iterator);
	if (status != ALP_STATUS_OK)
    	printf("dbg alp_global_settings_list_keys() status=0x%x\n", status);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    char key[200];
	int n = 0;
	bool keyMatched = false;
	while (status == ALP_STATUS_OK)
	{
        status = alp_global_settings_findkeys_get_key(iterator, key, sizeof(key));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        printf("n=%d, key: %s\n", n, key);

		char* res;
        // if the expedted key is non-null, see if it is found
		if (expKey != NULL && keyMatched == false)
		{
			//keyMatched = (strcmp(expKey, key) == 0) ? true : false;
			res = strstr(key, expKey); // find expKey in key
        	printf("dbg [%d], res=%s\n", n, res);
			keyMatched = (res != NULL) ? true : false;
		}
		status = alp_global_settings_findkeys_next(iterator);
        n++;
	}
	if (status == ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_AT_END)
		alp_global_settings_findkeys_end(iterator);

	// if given expected number is > 0, validates against it
	if (expN > 0) CPPUNIT_ASSERT(n == expN);
	// if a given expected key is non-NULL it was validated above and 
	// result is asserted here.
	if (expKey != NULL) 
	{
		char msg[100];
		sprintf(msg, "expected key, %s, was not found.", expKey);
		CPPUNIT_ASSERT_MESSAGE(msg, keyMatched == true);
	}
}


void GlobalSettingsTest_AP::basic_operation_write_performance(void)
{
    const char key[] = "test/test/testwrite";
    const char value[] = "12345678901234567890";
    alp_status_t status;
    int i;
    int bound = 200;
    
    for (i = 0; i < bound; i++)
    {
        status = alp_global_settings_set_key_string_value(context, key, value);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    }
}
  
  

void GlobalSettingsTest_AP::basic_operation_read_performance(void)
{
    const char key[] = "test/test/testwrite";
    const char value[] = "12345678901234567890";
    char buf[200] = { 0 };
    alp_status_t status;
    int i;
    int bound = 200;
    
    status = alp_global_settings_set_key_string_value(context, key, value);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    for (i = 0; i < bound; i++)
    {
        status = alp_global_settings_get_key_string_value(context, key, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        CPPUNIT_ASSERT(!strcmp(buf, value));
    }
}
    
void GlobalSettingsTest_AP::tree_deletion(void)
{
    const char dir[] = "ddd/6"; // original test value. straight-forward
	_tree_deletion(dir);
}

void GlobalSettingsTest_AP::tree_deletion_questionable_dir(void)
{
    const char dir[] = " ddd hello !@#$ ddd/6 "; // extra blancs and all...
	_tree_deletion(dir);
}

void GlobalSettingsTest_AP::_tree_deletion(const char* dir)
{
    int n;
    char key[200];
    char buf[300];
    
    alp_status_t status;
    AlpGlobalSettingsValue *vt = alp_global_settings_value_alloc();
    AlpGlobalSettingsEntryIterator *iterator = 0;
    
    status = alp_global_settings_set_key_directory(context, dir);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_directory(context, dir);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_EXISTS);
    
    status = alp_global_settings_findkeys_start(context, dir,  &iterator);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_AT_END);
    
    for (int k = 2; k < 5; k++)
    {
        char keybuf[200];
        char valbuf[20];
        sprintf(keybuf, "%s/%d", dir, k); 
        sprintf(valbuf, "value%d", 60 + k); 
        printf("dbg set_key_str_value : key=(%s), val=(%s)\n", keybuf, valbuf);
        status = alp_global_settings_set_key_string_value(context, keybuf, valbuf);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    }

    status = alp_global_settings_findkeys_start(context, dir,  &iterator);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    n = 0; 
    do
    {
        status = alp_global_settings_findkeys_get_key(iterator, key, sizeof(key));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        sprintf(buf, "%s/%s", dir, key); 
        printf("dbg buf: (%s)\n", buf);
        status = alp_global_settings_get(context, buf, vt);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        CPPUNIT_ASSERT(vt->type == ALP_GLOBAL_SETTINGS_TYPE_STRING);
       
        printf("key: %s\n", key);
        status = alp_global_settings_findkeys_next(iterator);
        n++;
    } while (status != ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_AT_END);
    
    status = alp_global_settings_findkeys_end(iterator);
    printf("dbg (to free iterator) findkeys_end() status=0x%x\n", status);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
	
    CPPUNIT_ASSERT(n == 3);
    status = alp_global_settings_delete(context, dir);
    printf("dbg try delete dir although there're keys under... status = 0x%x\n", status);
    CPPUNIT_ASSERT(status != ALP_STATUS_OK);

    status = alp_global_settings_delete_tree(context, dir);
    printf("dbg delete tree of (%s) status = 0x%x\n", dir, status);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_delete_tree(context, dir);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);

    status = alp_global_settings_get(context, dir, vt);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);

    status = alp_global_settings_set_key_directory(context, dir);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_delete(context, dir);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_delete(context, dir);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);

    alp_global_settings_value_free(vt);
    
    
    {  /* trees with default values */
        const char tree[] = test_directory "/t1/t2/t3";
        const char key1[] = "abcd";
        const char key2[] = "abcd.default";
        const char key3[] = "abcde";
        const char value[] = "123456";
        
        
        status = alp_global_settings_set_key_directory(context, tree);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        status = alp_global_settings_set_cwd(context, tree);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);

        status = alp_global_settings_set_key_string_value(context, key1, value);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        status = alp_global_settings_set_key_string_value(context, key2, value);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        status = alp_global_settings_set_key_string_value(context, key3, value);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);

        status = alp_global_settings_get_key_string_value(context, key1, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        CPPUNIT_ASSERT(!strcmp(buf, value));
        status = alp_global_settings_get_key_string_value(context, key2, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        CPPUNIT_ASSERT(!strcmp(buf, value));
        status = alp_global_settings_get_key_string_value(context, key3, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        CPPUNIT_ASSERT(!strcmp(buf, value));

        status = alp_global_settings_delete_tree(context, tree);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);

        status = alp_global_settings_get_key_string_value(context, tree, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
        
        status = alp_global_settings_get_key_string_value(context, key1, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
        
        status = alp_global_settings_get_key_string_value(context, key2, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
       
        status = alp_global_settings_get_key_string_value(context, key3, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
       
/* again but use delete_tree_keeping_defaults */

        
        status = alp_global_settings_set_key_directory(context, tree);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        status = alp_global_settings_set_cwd(context, tree);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);

        status = alp_global_settings_set_key_string_value(context, key1, value);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        status = alp_global_settings_set_key_string_value(context, key2, value);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        status = alp_global_settings_set_key_string_value(context, key3, value);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);

        status = alp_global_settings_get_key_string_value(context, key1, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        CPPUNIT_ASSERT(!strcmp(buf, value));
        status = alp_global_settings_get_key_string_value(context, key2, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        CPPUNIT_ASSERT(!strcmp(buf, value));
        status = alp_global_settings_get_key_string_value(context, key3, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        CPPUNIT_ASSERT(!strcmp(buf, value));

        status = alp_global_settings_delete_tree_keeping_defaults(context, tree);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        
        status = alp_global_settings_get_key_string_value(context, tree, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_INCORRECT_TYPE);
        
        status = alp_global_settings_get_key_string_value(context, key1, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        CPPUNIT_ASSERT(!strcmp(buf, value));
        status = alp_global_settings_get_key_string_value(context, key2, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        CPPUNIT_ASSERT(!strcmp(buf, value));
       
        status = alp_global_settings_get_key_string_value(context, key3, buf, sizeof(buf));
        CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
    }
        
}
    
    
void GlobalSettingsTest_AP::transactions(void)
{
    const char *testkey = "testkey2", *teststr = "12341234";
    alp_status_t status;
   
    char buf[300];
    AlpGlobalSettingsValue *vt = alp_global_settings_value_alloc();
    
    status = alp_global_settings_delete(context, testkey);
    
    status = alp_global_settings_get(context, testkey, vt);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);

    status = alp_global_settings_begin_transaction(context);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_string_value(context, testkey, teststr);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_get_key_string_value(context, testkey, buf, sizeof(buf));    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(!strcmp(buf, teststr));
    
    status = alp_global_settings_cancel_transaction(context);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_get(context, testkey, vt);
    CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND);
    
    status = alp_global_settings_begin_transaction(context);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_string_value(context, testkey, teststr);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_get_key_string_value(context, testkey, buf, sizeof(buf));    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(!strcmp(buf, teststr));
    
    status = alp_global_settings_complete_transaction(context);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    
    status = alp_global_settings_get_key_string_value(context, testkey, buf, sizeof(buf));    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(!strcmp(buf, teststr));
    
    alp_global_settings_value_free(vt);
}

void GlobalSettingsTest_AP::run_away_transaction_handling(void)
{

    const char *key = "transtest_key";
    const char *value = "1234";
    const char *value2 = "5678";
    alp_status_t status;
    
    status = alp_global_settings_begin_transaction(context);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_string_value(context, key, value);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    sleep(12);
    
    status = alp_global_settings_begin_transaction(context2);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_complete_transaction(context);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_complete_transaction(context2);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_begin_transaction(context2);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_set_key_string_value(context, key, value2);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_begin_transaction(context);    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
}



static int processed = 0;
static int key_changed(const char *type, void *details, int length, int *handled, const char *app, void *userdata)
{
    const char *key = (const char *) details;
    CPPUNIT_ASSERT(!strcmp(app, GLOBAL_SETTINGS_APP_ID));

    *handled = 1;

    fprintf(stderr, "Key %s changed!\n", key);
    processed = 1;
    return ALP_NOTIFY_CALLBACK_CONTINUE;
}

    
void GlobalSettingsTest_AP::change_notification(void)
{
    const char *testkey = "a/b/c/d/e", *testkey1 = "a/b//c/d/e1",  *testkey2 = "a/b/c/d2", *testkey3 = "a/b/d/e", *watch = "a/b/c/";
    char watchdir[200] ;
    
    sprintf(watchdir, "%s%s/%s", ALP_NOTIFY_EVENT_GLOBAL_SETTINGS_KEY_CHANGE, test_directory, watch);
    status = alp_notify_register_server(watchdir, key_changed, NULL, ALP_NOTIFY_PRIORITY_NORMAL, 0);
    
    if (status != ALP_STATUS_OK)
    {
        fprintf(stderr, "failed to connect to the notificaiton manager, error code %x\n", status);
    } 
    
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_notification_on_off(context, 1);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    processed = 0;
    status = alp_global_settings_set_key_string_value(context, testkey, "abcdefg");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    sleep(1);
    CPPUNIT_ASSERT(processed == 1);
    processed = 0;
    status = alp_global_settings_set_key_string_value(context, testkey1, "abcdefg 1");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    sleep(1);
    CPPUNIT_ASSERT(processed == 1);
    processed = 0;
    status = alp_global_settings_set_key_string_value(context, testkey2, "1122334455667");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    sleep(1);
    CPPUNIT_ASSERT(processed == 1);
    
    processed = 0;
    status = alp_global_settings_set_key_string_value(context, testkey3, "xxxyyy");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    sleep(1);
    CPPUNIT_ASSERT(processed == 0);
    
    processed = 0;
    status = alp_global_settings_set_key_string_value(context, testkey2, "abcd");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    sleep(1);
    CPPUNIT_ASSERT(processed == 1);


    status = alp_global_settings_set_notification_on_off(context, 0);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    processed = 0;
    status = alp_global_settings_set_key_string_value(context, testkey, "abcdefg");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    sleep(1);
    CPPUNIT_ASSERT(processed == 0);
    processed = 0;
    status = alp_global_settings_set_key_string_value(context, testkey1, "abcdefg 1");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    sleep(1);
    CPPUNIT_ASSERT(processed == 0);
    processed = 0;
    status = alp_global_settings_set_key_string_value(context, testkey2, "1122334455667");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    sleep(1);
    CPPUNIT_ASSERT(processed == 0);
    
    processed = 0;
    status = alp_global_settings_set_key_string_value(context, testkey3, "xxxyyy");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    sleep(1);
    CPPUNIT_ASSERT(processed == 0);
    
    processed = 0;
    status = alp_global_settings_set_key_string_value(context, testkey2, "abcd");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    sleep(1);
    CPPUNIT_ASSERT(processed == 0);



    status = alp_global_settings_set_notification_on_off(context, 1);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_notify_unregister_server(watchdir, 0);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        
}

/*void GlobalSettingsTest_AP::check_permissions(void) 
{*/
 
        
void GlobalSettingsTest_AP::service_end(void)
{
    CPPUNIT_ASSERT(context != 0);
    
    status = alp_global_settings_close(context);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    status = alp_global_settings_close(context2);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
#if 0    
    alp_global_settings_server_stop();
#endif
}



  




void GlobalSettingsTest_AP::create_complex_tree(void)
{ 
    int n; 
    char key[200];
    char buf1[300], buf2[300], buf3[300], buf4[300], buf5[300], buf6[300], buf7[300], buf8[300]; 
    const char dir[] = "ggf"; 
    const char dir1[] = "ggf/gf0/f00";
    const char dir2[] = "ggf/gf0/f01"; 
    const char dir3[] = "ggf/gf1/f10"; 
    const char dir4[] = "ggf/gf1/f11"; 
    char *int_buf; 
  //  char *itoa (int); 

    alp_status_t status;
    
    AlpGlobalSettingsValue *vt = alp_global_settings_value_alloc();
    AlpGlobalSettingsEntryIterator *iterator = 0;
     
    status = alp_global_settings_set_key_directory(context, dir1);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_directory(context, dir2);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);  

    status = alp_global_settings_set_key_directory(context, dir3);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 

    status = alp_global_settings_set_key_directory(context, dir4);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 
 
    status = alp_global_settings_set_key_string_value(context, "ggf/gf0/f00/s000", "value000");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_string_value(context, "ggf/gf0/f00/s001", "value001");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_set_key_string_value(context, "ggf/gf0/f01/s010", "value010");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
 
    status = alp_global_settings_set_key_string_value(context, "ggf/gf0/f01/s011", "value011");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
  
    status = alp_global_settings_set_key_string_value(context, "ggf/gf1/f10/s100", "value100");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
 
    status = alp_global_settings_set_key_string_value(context, "ggf/gf1/f10/s101", "value101");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_string_value(context, "ggf/gf1/f11/s110", "value110");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_string_value(context, "ggf/gf1/f11/s111", "value111");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
   
    status = alp_global_settings_get_key_string_value(context,"ggf/gf0/f00/s000",buf1,sizeof(buf1)); 
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 

    status = alp_global_settings_get_key_string_value(context,"ggf/gf0/f00/s001",buf2,sizeof(buf2)); 
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 

    status = alp_global_settings_get_key_string_value(context,"ggf/gf0/f01/s010",buf3,sizeof(buf3)); 
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 

    status = alp_global_settings_get_key_string_value(context,"ggf/gf0/f01/s011",buf4,sizeof(buf4)); 
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 

    status = alp_global_settings_get_key_string_value(context,"ggf/gf1/f10/s100",buf5,sizeof(buf5)); 
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 

    status = alp_global_settings_get_key_string_value(context,"ggf/gf1/f10/s101",buf6,sizeof(buf6)); 
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 

    status = alp_global_settings_get_key_string_value(context,"ggf/gf1/f11/s110",buf7,sizeof(buf7)); 
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 

    status = alp_global_settings_get_key_string_value(context,"ggf/gf1/f11/s111",buf8,sizeof(buf8)); 
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 
  
    status = alp_global_settings_findkeys_start(context, dir,  &iterator);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);


    for(int i=0; i<7; i++) 
    {
    
    	if(i==1) 
    	{
		const char dir_f0[] = "ggf/gf0"; 
        	status = alp_global_settings_findkeys_start(context, dir_f0,  &iterator);
        	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    	} 

    	if(i==2) 
    	{
		const char dir_f1[] = "ggf/gf1"; 
        	status = alp_global_settings_findkeys_start(context, dir_f1,  &iterator);
        	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    	}	 
  
    	if(i==3) 
    	{
	
        	status = alp_global_settings_findkeys_start(context, dir1,  &iterator);
        	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    	} 
 
    	if(i==4) 
    	{
	
        	status = alp_global_settings_findkeys_start(context, dir2,  &iterator);
       	 	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    	} 

    	if(i==5) 
    	{
	
    	    	status = alp_global_settings_findkeys_start(context, dir3,  &iterator);
        	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    	} 

    	if(i==6) 
    	{
	
        	status = alp_global_settings_findkeys_start(context, dir4,  &iterator);
        	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    	} 

    	n = 0;
     
        do
        {
    
        	status = alp_global_settings_findkeys_get_key(iterator, key, sizeof(key));
        	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
        	printf("key: %s\n", key);
        	status = alp_global_settings_findkeys_next(iterator);
        	n++;
    	} while (status != ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_AT_END);

    	CPPUNIT_ASSERT(n == 2); 

        status = alp_global_settings_findkeys_end(iterator);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    }

    // might as well test alp_global_settings_list_keys() API
    _list_key_test("*1/f*", 6);

    alp_global_settings_value_free(vt);
}


/**
 * @brief APIs tested: 
 * 		alp_global_setting_value_set_int, _float, and _bool
 * 		alp_global_setting_value_get_int, _float, and _bool
 *
 * @description simply calls the set fucntion followed by get on each data type
 * 		and verify results as expected. (This is a shallow unplugged test).
 */ 		
void GlobalSettingsTest_AP::basic_type_values_set_get_2(void)
{
	gint ivalue = -98765;
	gdouble fvalue = 123.456;
	gboolean bvalue = true;

	gint ivalue_got = 0;
	gdouble fvalue_got = 0.0;
	gboolean bvalue_got = false;

	alp_status_t status;
	AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;

	// call set & get values

	// ALP_TM(TL(ALP_CLASS_GLOBAL_SETTINGS, "Global_Settings_tests : call alp_global_settings_value_set_xxx() int=%d, float=%f, bool=%d", ivalue, fvalue, (int)bvalue ));

	//------ int
		status = alp_global_settings_value_set_int(&gsvalue, ivalue);
		CPPUNIT_ASSERT(status == ALP_STATUS_OK);

		status = alp_global_settings_value_get_int(&gsvalue, &ivalue_got);
		//ALP_TM(TL(ALP_CLASS_GLOBAL_SETTINGS, "GStest : got int=%d", ivalue_got));
		printf("GStest : got int=%d\n", ivalue_got);
		CPPUNIT_ASSERT(status == ALP_STATUS_OK && ivalue_got == ivalue);

	//------ float
		status = alp_global_settings_value_set_float(&gsvalue, fvalue);
		CPPUNIT_ASSERT(status == ALP_STATUS_OK);

		status = alp_global_settings_value_get_float(&gsvalue, &fvalue_got);
		//ALP_TM(TL(ALP_CLASS_GLOBAL_SETTINGS, "GStest: got float=%f", fvalue_got));
		printf("GStest: got float=%f\n", fvalue_got);
		CPPUNIT_ASSERT(status == ALP_STATUS_OK && fvalue_got == fvalue);

	//------ bool
		status = alp_global_settings_value_set_bool(&gsvalue, bvalue);
		CPPUNIT_ASSERT(status == ALP_STATUS_OK);

		status = alp_global_settings_value_get_bool(&gsvalue, &bvalue_got);
		//ALP_TM(TL(ALP_CLASS_GLOBAL_SETTINGS, "GStest: got bool=%d", bvalue_got));
		printf("GStest: got bool=%d\n", bvalue_got);
		CPPUNIT_ASSERT(status == ALP_STATUS_OK && bvalue_got == bvalue);
}


/**
 * @brief APIs tested: 
 * 	alp_global_settings_entry_alloc(), alp_global_settings_entry_free()
 * 	alp_global_settings_entry_set_key_value(), _get_key_value()
 *
 * @description set a string value to GlobalSettinvsValue.
 * 		create a GlobalSettingsEntry and sets the key & str value to it.
 * 		gets key and value-string from entry and verify.
 * 		(this is a shallow unplugged test).
 */
void GlobalSettingsTest_AP::values_set_get_entry(void)
{
	alp_status_t status;
	const char* s_key = "a_test_key";
	const char* the_string_value = "whatever string";
	const int BufSz = 100;
	char buf[BufSz];

	AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
	AlpGlobalSettingsValue gsvalue_recv = GLOBAL_SETTINGS_VALUE_INITIALIZER;

	// preparation - sets strings to gs value
	status = alp_global_settings_value_set_string(&gsvalue, the_string_value);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	// verify gs value
	status = alp_global_settings_value_get_string(&gsvalue, buf, sizeof(buf));
	CPPUNIT_ASSERT(status == ALP_STATUS_OK); printf("gs value is %s\n", buf);

	// thing to be tested, entry
	AlpGlobalSettingsEntry* gsentry = alp_global_settings_entry_alloc();

	// sets the key and value string to entry 
	status = alp_global_settings_entry_set_key_value(gsentry, s_key, &gsvalue);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
	printf("sets key: %s, value: %s\n", gsentry->key->str, gsentry->value.string_value->str);

	// gets key and value-string from entry into buf
	status = alp_global_settings_entry_get_key_value(gsentry, buf, BufSz, &gsvalue_recv);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK && strcmp(buf, s_key)==0);
	char val_buf[BufSz];
	// (extracts string from the received gsvalue)
	status = alp_global_settings_value_get_string(&gsvalue_recv, val_buf, sizeof(val_buf));
	printf("GStest: got key=%s, value=%s\n", buf, val_buf);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK && strcmp(val_buf, the_string_value)==0);

	//
	//----- now, try with integer instead of string for the value
	// 
	int ivalue = 12345;
	status = alp_global_settings_value_set_int(&gsvalue, ivalue);
	// sets the key and value integer to entry 
	status = alp_global_settings_entry_set_key_value(gsentry, s_key, &gsvalue);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
	printf("sets key: %s, int_value: %d\n", gsentry->key->str, gsentry->value.int_value);
	// gets key from entry to buf, and value-int 
	status = alp_global_settings_entry_get_key_value(gsentry, buf, BufSz, &gsvalue_recv);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK && strcmp(buf, s_key)==0);
	printf("GStest: got key=%s, int_value=%d\n", buf, gsvalue_recv.int_value);


	alp_global_settings_entry_free(gsentry);
}


/**
 * @brief APIs tested: 
 * 		alp_global_setting_value_set_list
 * 		alp_global_setting_value_get_list
 *
 * @description simply calls the set fucntion followed by get on list (GList)
 * 		and verify results as expected. (This is a shallow unplugged test).
 */
void GlobalSettingsTest_AP::values_set_get_list(void)
{
	//------ list 
    const char* key = "testlist ";
    const char* key2 = "testlist 2 @#!~^ key";

	AlpGlobalSettingsValue gsvalue = GLOBAL_SETTINGS_VALUE_INITIALIZER;
	AlpGlobalSettingsValue *gsvalue2 = 0, *tgsvalue;

	GList* glist_src = NULL;
	GList* glist_dst = NULL; 

	// prepare setup list for glist_src
	char* dirlist[] = { "d_zero", "d/a_one", "d/b_two", "d/c_three", "d/c/e_four", "d/ca_five" } ;
	
    for (int i=0; i < 6; i++)
    {
        gsvalue2 = alp_global_settings_value_alloc();
        alp_global_settings_value_set_string(gsvalue2, dirlist[i]);
        glist_src = g_list_append(glist_src, gsvalue2);
    }
	int len_s = g_list_length(glist_src);
	int len_d = g_list_length(glist_dst);
	printf("GStest: glist_src len=%d, glist_dst len=%d\n", len_s, len_d);
	CPPUNIT_ASSERT(len_s == 6 && len_d == 0);


	// set from glist_src into gsvalue
	printf("GStest: calls alp_global_settings_value_set_list()\n");
	status = alp_global_settings_value_set_list(&gsvalue, glist_src);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set(context, key, &gsvalue);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	gsvalue2 = alp_global_settings_value_alloc();
    status = alp_global_settings_get(context2, key, gsvalue2);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	// get list from gsvalue into glist_dst
	printf("GStest: calls alp_global_settings_value_get_list()\n");
	status = alp_global_settings_value_get_list(gsvalue2, &glist_dst);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	len_d = g_list_length(glist_dst);
	CPPUNIT_ASSERT(len_d == 6);

	for (int i = 0; i < len_d; i++)
	{
		gpointer gp = g_list_nth_data(glist_dst, i);
        tgsvalue = (AlpGlobalSettingsValue *) gp;
		printf("GStest: glist_dst %d-th element data = %s\n", i, (char*) tgsvalue->string_value->str);
        CPPUNIT_ASSERT(tgsvalue->type == ALP_GLOBAL_SETTINGS_TYPE_STRING);
		CPPUNIT_ASSERT(strcmp((char*)tgsvalue->string_value->str, dirlist[i])==0);
	}
	// ----- now remove a few -----
	gpointer gp = g_list_nth_data(glist_dst, 4);
	glist_dst = g_list_remove(glist_dst, gp);
	gp = g_list_nth_data(glist_dst, 2);
	glist_dst = g_list_remove(glist_dst, gp);

	// ----- show again ----
	// to assure that glist_dst has its own copy, 
	// shared data was not pointed by glist_src 
	printf("GStest: ---- removed 2 elements from glist_dst ----\n");
	len_d = g_list_length(glist_dst);
	len_s = g_list_length(glist_src);
	CPPUNIT_ASSERT(len_d == 4 && len_s == 6);
	for (int i = 0; i < len_d; i++)
	{
		gpointer gp = g_list_nth_data(glist_dst, i);
        tgsvalue = (AlpGlobalSettingsValue *) gp;
		printf("GStest: glist_dst %d-th element data = %s\n", i, (char*) tgsvalue->string_value->str);
	}

	// ----- this time, glist_dst has something (to be freed)
	// ----- call again set / get

	// set from glist_src into gsvalue
	printf("GStest: calls alp_global_settings_value_set_list()\n");
	status = alp_global_settings_value_set_list(&gsvalue, glist_src);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);


	status = alp_global_settings_set(context, key2, &gsvalue);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	status = alp_global_settings_get(context2, key2, gsvalue2);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);


	// get list from gsvalue into glist_dst
	printf("GStest: calls alp_global_settings_value_get_list()\n");
	status = alp_global_settings_value_get_list(gsvalue2, &glist_dst);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	len_d = g_list_length(glist_dst);
	CPPUNIT_ASSERT(len_d == 6);

	for (int i = 0; i < len_d; i++)
	{
		gpointer gp = g_list_nth_data(glist_dst, i);
        tgsvalue = (AlpGlobalSettingsValue *) gp;
		printf("GStest: glist_dst %d-th element data = %s\n", i, (char*) tgsvalue->string_value->str);
        CPPUNIT_ASSERT(tgsvalue->type == ALP_GLOBAL_SETTINGS_TYPE_STRING);
		CPPUNIT_ASSERT(strcmp((char*)tgsvalue->string_value->str, dirlist[i])==0);
	}
	
	// free the lists
	alp_global_settings_value_list_free(glist_dst);
	alp_global_settings_value_list_free(glist_src);
	alp_global_settings_value_free(gsvalue2);
	alp_global_settings_value_clean(&gsvalue);

}


void GlobalSettingsTest_AP::delete_complex_tree(void)
{
    int i, n; 
    char key[300]; 
    char sub_key[200]; 
    
    char buf[500]; 
    const char dr[] = "bbf"; 
    const char dr1[] = "bbf/gf0/f00";
    const char dr2[] = "bbf/gf0/f01"; 
    const char dr3[] = "bbf/gf1/f10"; 
    const char dr4[] = "bbf/gf1/f11"; 

    alp_status_t status;
    AlpGlobalSettingsValue *vt = alp_global_settings_value_alloc();
    AlpGlobalSettingsEntryIterator *iterator = 0;

    status = alp_global_settings_set_key_directory(context, dr1);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 

    status = alp_global_settings_set_key_directory(context, dr2);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);  

    status = alp_global_settings_set_key_directory(context, dr3);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 

    status = alp_global_settings_set_key_directory(context, dr4);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK); 
 
    status = alp_global_settings_set_key_string_value(context2, "bbf/gf0/f00/s000", "value000");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_string_value(context, "bbf/gf0/f00/s001", "value001");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    
    status = alp_global_settings_set_key_string_value(context, "bbf/gf0/f01/s010", "value010");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
 
    status = alp_global_settings_set_key_string_value(context, "bbf/gf0/f01/s011", "value011");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
  
    status = alp_global_settings_set_key_string_value(context2, "bbf/gf1/f10/s100", "value100");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
 
    status = alp_global_settings_set_key_string_value(context, "bbf/gf1/f10/s101", "value101");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_string_value(context, "bbf/gf1/f11/s110", "value110");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    status = alp_global_settings_set_key_string_value(context, "bbf/gf1/f11/s111", "value111");
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

    for(i=0; i<4; i++) 
    { 
	printf("\n\nRound : %d\n",i); 
    	if(i==0)
    	{
    	   	status = alp_global_settings_findkeys_start(context, dr1,  &iterator);
    		CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    		sprintf(key, "%s", (char *)dr1);
		printf("\n  Directory : %s\n",key); 
		n = 0; 
    	}
    	else if(i==1)
    	{
    	    	status = alp_global_settings_findkeys_start(context, dr2,  &iterator);
    		CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    		sprintf(key, "%s", (char *)dr2);
		printf("\n  Directory : %s\n",key); 
		n = 0; 
    	}
    	else if(i==2)
    	{ 
            	status = alp_global_settings_findkeys_start(context2, dr3,  &iterator);
    		CPPUNIT_ASSERT(status == ALP_STATUS_OK);
 		sprintf(key, "%s", (char *)dr3);
		printf("\n  Directory : %s\n",key); 
    		n = 0; 
    	} 
    	else
    	{
	    	status = alp_global_settings_findkeys_start(context, dr4,  &iterator);
    		CPPUNIT_ASSERT(status == ALP_STATUS_OK);
		sprintf(key, "%s", (char *)dr4);
		printf("\n  Directory : %s\n",key); 
    		n = 0; 
    	} 

    	

    	do
    	{
                 
		status = alp_global_settings_findkeys_get_key(iterator, sub_key, sizeof(sub_key));
		CPPUNIT_ASSERT(status == ALP_STATUS_OK);
                sprintf(buf, "%s/%s", key, sub_key); 
        	status = alp_global_settings_get(context, buf, vt);
        	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        	CPPUNIT_ASSERT(vt->type == ALP_GLOBAL_SETTINGS_TYPE_STRING);
      
        	printf("\n  New Key: %s\n", buf);
		status = alp_global_settings_findkeys_next(iterator);
        	n++;

    	} while (status != ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_AT_END);
    
    	CPPUNIT_ASSERT(n == 2);

        status = alp_global_settings_findkeys_end(iterator);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
 
	status = alp_global_settings_delete_tree(context, key);
    	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	printf("\n  Deleting ...  %s\n",key); 

    	status = alp_global_settings_get(context, key, vt);
    	CPPUNIT_ASSERT(status == ALP_STATUS_GLOBAL_SETTINGS_KEY_NOT_FOUND); 
    
    } 

    alp_global_settings_value_free(vt);
} 


/**
 * @brief APIs tested: alp_global_settings_value_set_list, alp_global_settings_value_get_list,
 *    	alp_global_settings_value_list_free, 
 *    	alp_global_settings_value_free, alp_global_settings_value_clean
 *
 * @description 
 * 		After prepared a list of AlpGlobalSettingsValue (which has string value) to form
 * 		GList src, calls alp_global_settings_value_set_list(..), and sets the list w/ key to context.
 * 		Then, specifying the key to get the list from context, and calls _value_get_list(..) 
 * 		to receive the list to GList dst. and verify results are as expected, and cleanup.
 */
void GlobalSettingsTest_AP::values_set_get_list_gsvalue_str(void) // ALTERED WEIRD POSITIVE
{
    const char* key = "a////b/c/testlistkey";   // should become /a/b/c/testlistkey
	static const char* string_vals[] = { "start_zero 0)", "one 1) what\t a # *&^%$!? // \\<{>}[];;;:: weird value!? yet this would be valid -- as far as gstring_copy() takes, which is called by alp_global_settings_value_set_string(), which in turn called by alp_global_settings_set_key_string_value() ... anyway, this is the end of this weird value.", 
"two 2) so, the limit length of the string value is also up to gstring_copy()",
	"three 3", "four 4", "bye_five 5", NULL };

	alp_status_t status;
	const int listLen = 6;
    const int BufSz = 300;
	AlpGlobalSettingsValue gsvalueL = GLOBAL_SETTINGS_VALUE_INITIALIZER;
	AlpGlobalSettingsValue *gsvalue2 = 0;

	GList* glist_src = NULL;
	GList* glist_dst = NULL; 
	// prepare setup list for glist_src
	
    for (int i=0; i < listLen; i++)
    {
        gsvalue2 = alp_global_settings_value_alloc();
        alp_global_settings_value_set_string(gsvalue2, string_vals[i]);
        glist_src = g_list_append(glist_src, gsvalue2);
    }

	int len_s = g_list_length(glist_src);
	int len_d = g_list_length(glist_dst);
	printf("GStest: glist_src len=%d, glist_dst len=%d\n", len_s, len_d);
	CPPUNIT_ASSERT(len_s == listLen && len_d == 0);


	// set from glist_src into gsvalue
	printf("GStest: calls alp_global_settings_value_set_list()\n");
	status = alp_global_settings_value_set_list(&gsvalueL, glist_src);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	// set the GlobalSettingValue (of GList of strings) into context with a key
	status = alp_global_settings_set(context, key, &gsvalueL);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	// ---

	// get GlobalSettingValue from context by key 
	gsvalue2 = alp_global_settings_value_alloc();
	status = alp_global_settings_get(context2, key, gsvalue2);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	// get list from gsvalue into glist_dst
	printf("GStest: calls alp_global_settings_value_get_list()\n");
	status = alp_global_settings_value_get_list(gsvalue2, &glist_dst);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	len_d = g_list_length(glist_dst);
	CPPUNIT_ASSERT(len_d == listLen);

	// show the received glist_dst contents, entries key & value strings
	for (int i = 0; i < len_d; i++)
	{
		gpointer gp = g_list_nth_data(glist_dst, i);
		AlpGlobalSettingsValue* gsvalue_recv = (AlpGlobalSettingsValue*) gp;
        CPPUNIT_ASSERT(gsvalue_recv->type == ALP_GLOBAL_SETTINGS_TYPE_STRING);
		char val_buf[BufSz];
		// (extracts string from the received gsvalue)
		status = alp_global_settings_value_get_string(gsvalue_recv, val_buf, BufSz);
		printf("dbg: value_get_str status=0x%x\n", status);
	    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
		printf("dbg: glist_dst %d-th value=%s\n", i, val_buf);
	}

	// ----- now remove a few -----
	gpointer gp = g_list_nth_data(glist_dst, 3);
	glist_dst = g_list_remove(glist_dst, gp);
	gp = g_list_nth_data(glist_dst, 1);
	glist_dst = g_list_remove(glist_dst, gp);

	// verify that glist_dst has its own copy, not pointed by glist_src 
	printf("dbg: ---- removed 2 elements from glist_dst ----\n");
	len_d = g_list_length(glist_dst);
	len_s = g_list_length(glist_src);
	CPPUNIT_ASSERT(len_d == (listLen - 2) && len_s == listLen);



	// free the lists
	alp_global_settings_value_list_free(glist_dst);
	alp_global_settings_value_list_free(glist_src);

    alp_global_settings_value_free(gsvalue2);
    alp_global_settings_value_clean(&gsvalueL);

}


/**
 * @brief APIs tested: alp_global_settings_value_set_list, alp_global_settings_value_get_list,
 *
 * @description 
 * 		After prepared a list of AlpGlobalSettingsValue (data float type) to form GList src.
 * 		calls alp_global_settings_value_set_list(..), and sets the list w/ key to context.
 * 		Then, specifying the key to get the list from context, and calls _value_get_list(..) 
 * 		to receive the list to GList dst. and verify results are as expected, and cleanup.
 */
void GlobalSettingsTest_AP::values_set_get_list_gsvalue_float(void)
{
    const char* key = "a/b/c/testlistkeyF";

	const double fvalue_seed = -123.456;
	double fvalue_got = 0.0;

	alp_status_t status;
	const int listLen = 8;
	AlpGlobalSettingsValue gsvalue_src = GLOBAL_SETTINGS_VALUE_INITIALIZER;
	AlpGlobalSettingsValue* gsvalue_tmp = 0;

	GList* glist_src = NULL;
	GList* glist_dst = NULL; 

	// prepare setup list for AlpGlobalSettinvsValue of various data type
    for (int i=0; i < listLen; i++)
	{
        gsvalue_tmp = alp_global_settings_value_alloc();
		double fvaltmp = fvalue_seed * i;
		printf("GStest : sets float=%f\n", fvaltmp);
		status = alp_global_settings_value_set_float(gsvalue_tmp, fvaltmp);
		CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        glist_src = g_list_append(glist_src, gsvalue_tmp);
	}

	// Extra - verify g_list is not doing something weird on some double value
	printf("GStest : verify g_list_src contents value\n");
	// show the glist_src contents, float (actually double) data value 
	for (int i = 0; i < listLen; i++)
	{
		gpointer gp = g_list_nth_data(glist_src, i); // <===  Note: src 
		AlpGlobalSettingsValue* gsval = (AlpGlobalSettingsValue*) gp;

		fvalue_got = 0.0; // clear
		double expected = fvalue_seed * i; // same formula used for set
		status = alp_global_settings_value_get_float(gsval, &fvalue_got);
		if ((fvalue_got == expected) || (fabs(fvalue_got - expected) <= fabs(0.00001 * expected)))
			;//printf("GStest :[%d] float=%lf, expected=%lf\n", i, fvalue_got, expected);
		else
			printf("GStest :[%d] float=%lf, expected=%lf <= diffs already!\n", i, fvalue_got, expected);
		CPPUNIT_ASSERT(status == ALP_STATUS_OK && fvalue_got == expected);
#if 0
		// just to show how the value looks like if it was down casted to float once
		// isn't this look same as they do when got back from the list later
		double intentionally_trancated = (double) (float) fvalue_got;
		if (intentionally_trancated != expected)
			printf("  (fyi, intentionally down casted to float=%f <== compare)\n", intentionally_trancated);
#endif 
	}

	int len_s = g_list_length(glist_src);
	int len_d = g_list_length(glist_dst);
	printf("GStest: glist_src len=%d, glist_dst len=%d\n", len_s, len_d);
	CPPUNIT_ASSERT(len_s == listLen && len_d == 0);


	// set from glist_src into gsvalue
	printf("GStest: calls alp_global_settings_value_set_list()\n");
	status = alp_global_settings_value_set_list(&gsvalue_src, glist_src);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	printf("GStest: set GlobalSettingValue (of GList of float) into Context\n");
	status = alp_global_settings_set(context, key, &gsvalue_src);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	printf("GStest: --- --- --- --- ---\n");

	printf("GStest: get GlobalSettingValue (of GList of float) from Context\n");
	gsvalue_tmp = alp_global_settings_value_alloc();
	status = alp_global_settings_get(context, key, gsvalue_tmp);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	// get list from gsvalue into glist_dst
	printf("GStest: calls alp_global_settings_value_get_list()\n");
	status = alp_global_settings_value_get_list(gsvalue_tmp, &glist_dst);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	len_d = g_list_length(glist_dst);
	CPPUNIT_ASSERT(len_d == listLen);


	// show the received glist_dst contents, float data value 
	int failed = 0;
	for (int i = 0; i < len_d; i++)
	{
		gpointer gp = g_list_nth_data(glist_dst, i);
		AlpGlobalSettingsValue* gsvalue_recv = (AlpGlobalSettingsValue*) gp;

		fvalue_got = 0.0; // clear
		double expected = fvalue_seed * i; // same formula used for set
		status = alp_global_settings_value_get_float(gsvalue_recv, &fvalue_got);

		// exact match or slight diffs within a margin
		if ((fvalue_got == expected) || (fabs(fvalue_got - expected) <= fabs(0.00001 * expected)))
			printf("GStest ok:[%d] got float=%lf, expected=%lf\n", i, fvalue_got, expected);
		else
		{
			printf("GStest :[%d] got float=%lf, expected=%lf <== diffs. FAILURE!\n", i, fvalue_got, expected);
			failed++;
		}
	}
	CPPUNIT_ASSERT_MESSAGE("FAILURE. See runlog.txt. Got values from list differ from set ones.", failed==0); 


	// ----- now remove a few -----
	gpointer gp = g_list_nth_data(glist_dst, 3);
	glist_dst = g_list_remove(glist_dst, gp);
	gp = g_list_nth_data(glist_dst, 1);
	glist_dst = g_list_remove(glist_dst, gp);

	// verify that glist_dst has its own copy, not pointed by glist_src 
	printf("GStest: ---- removed 2 elements from glist_dst ----\n");
	len_d = g_list_length(glist_dst);
	len_s = g_list_length(glist_src);
	CPPUNIT_ASSERT(len_d == (listLen - 2) && len_s == listLen);


	// free the lists
	alp_global_settings_value_list_free(glist_dst);
	alp_global_settings_value_list_free(glist_src);

    alp_global_settings_value_free(gsvalue_tmp);
    alp_global_settings_value_clean(&gsvalue_src);

}


/**
 * @brief APIs tested: alp_global_settings_value_set_list, alp_global_settings_value_get_list,
 *
 * @description 
 * 		After prepared a list of AlpGlobalSettingsValue (whose data are: integer, float, 
 * 		boolean, and string value) to form GList src.
 * 		calls alp_global_settings_value_set_list(..), and sets the list w/ key to context.
 * 		Then, specifying the key to get the list from context, and calls _value_get_list(..) 
 * 		to receive the list to GList dst. and verify results are as expected, and cleanup.
 */
void GlobalSettingsTest_AP::values_set_get_list_gsvalue_mix(void)
{
    const char* key = "a/b/c/testlistkey2";

	//const char* strvalue_src = "Hellooooo";
	static const char* string_vals[] = { "start_zero", "hello_one", "how_two", "are_three", "you_four", "bye_five" };
	gint ivalue_src = -98765;
	//gdouble fvalue_src= 123.4567;
	double fvalue_src= 123.4567;
	gboolean bvalue_src = true;

	gint ivalue_got = 0;
	//gdouble fvalue_got = 0.0;
	double fvalue_got = 0.0;
	gboolean bvalue_got = false;

	alp_status_t status;
	const int listLen = 4 * 2 ;
    const int BufSz = 100;
	AlpGlobalSettingsValue gsvalue_src = GLOBAL_SETTINGS_VALUE_INITIALIZER;
	AlpGlobalSettingsValue* gsvalue_tmp = 0;

	GList* glist_src = NULL;
	GList* glist_dst = NULL; 

	// prepare setup list for AlpGlobalSettinvsValue of various data type
    for (int i=0; i < listLen; i++)
	{
        gsvalue_tmp = alp_global_settings_value_alloc();
		switch (i) 
		{
			case 0: 
			case 4: status = alp_global_settings_value_set_string(gsvalue_tmp, string_vals[i]); 
					break;
			case 1: 
			case 5: status = alp_global_settings_value_set_int  (gsvalue_tmp, ivalue_src * i);
					break;
			case 2: 
			case 6: {
					//gdouble fvaltmp = (fvalue_src * i);
					double fvaltmp = (fvalue_src * i);
					printf("GStest : sets float=%f\n", fvaltmp);
					//status = alp_global_settings_value_set_float(gsvalue_tmp, fvalue_src * i); 
					status = alp_global_settings_value_set_float(gsvalue_tmp, fvaltmp);
					CPPUNIT_ASSERT(status == ALP_STATUS_OK);

					//immediately get
					status = alp_global_settings_value_get_float(gsvalue_tmp, &fvaltmp);
					CPPUNIT_ASSERT(status == ALP_STATUS_OK);
					}
					break;

			case 3: bvalue_src = false;
			case 7: status = alp_global_settings_value_set_bool (gsvalue_tmp, bvalue_src); 
					bvalue_src = !bvalue_src;
					break;
		}
		CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        glist_src = g_list_append(glist_src, gsvalue_tmp);
	}

	int len_s = g_list_length(glist_src);
	int len_d = g_list_length(glist_dst);
	printf("GStest: glist_src len=%d, glist_dst len=%d\n", len_s, len_d);
	CPPUNIT_ASSERT(len_s == listLen && len_d == 0);


	// set from glist_src into gsvalue
	printf("GStest: calls alp_global_settings_value_set_list()\n");
	status = alp_global_settings_value_set_list(&gsvalue_src, glist_src);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	// set the GlobalSettingValue (of GList of strings) into context with a key
	status = alp_global_settings_set(context, key, &gsvalue_src);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	// --- --- --- --- 

	// get GlobalSettingValue from context by key 
	gsvalue_tmp = alp_global_settings_value_alloc();
	status = alp_global_settings_get(context, key, gsvalue_tmp);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	// get list from gsvalue into glist_dst
	printf("GStest: calls alp_global_settings_value_get_list()\n");
	status = alp_global_settings_value_get_list(gsvalue_tmp, &glist_dst);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	len_d = g_list_length(glist_dst);
	CPPUNIT_ASSERT(len_d == listLen);


	// show the received glist_dst contents, data value (based on expected type)
	int failed = 0;
	for (int i = 0; i < len_d; i++)
	{
		gpointer gp = g_list_nth_data(glist_dst, i);
		AlpGlobalSettingsValue* gsvalue_recv = (AlpGlobalSettingsValue*) gp;

		switch (i) 
		{
			case 1: 
			case 5: status = alp_global_settings_value_get_int(gsvalue_recv, &ivalue_got);
					printf("GStest : got int=%d\n", ivalue_got);
					CPPUNIT_ASSERT(status == ALP_STATUS_OK && ivalue_got == ivalue_src * i);
					break;
			case 2: 
			case 6: 
			{
					status = alp_global_settings_value_get_float(gsvalue_recv, &fvalue_got);
					//gdouble expected = fvalue_src * i;
					double expected = fvalue_src * (double)i;
		            if ((fvalue_got == expected) || (fabs(fvalue_got - expected)) <= (fabs(0.00001 * expected)))
						printf("GStest ok: got float=%lf, expected=%lf\n", fvalue_got, expected);
					else
					{
						printf("GStest : got float=%lf, expected=%lf <== diffs. would FAIL!\n", fvalue_got, expected);
						failed++;
						CPPUNIT_ASSERT_MESSAGE("FAILURE. See runlog.txt. Got values from list differ from set ones.", failed==0); 
					}
			}
					break;
			case 3:
			case 7: status = alp_global_settings_value_get_bool(gsvalue_recv, &bvalue_got);
					printf("GStest : got boolean=%d, expected=%d\n", bvalue_got, bvalue_src);
					CPPUNIT_ASSERT(status == ALP_STATUS_OK && bvalue_got == bvalue_src);
					bvalue_src = !bvalue_src;
					break;
			case 0: 
			case 4: 
			{
					CPPUNIT_ASSERT(gsvalue_recv->type == ALP_GLOBAL_SETTINGS_TYPE_STRING);
					char val_buf[BufSz];
					status = alp_global_settings_value_get_string(gsvalue_recv, val_buf, BufSz);
					printf("GStest: got string value=%s\n", val_buf);
	    			CPPUNIT_ASSERT(status == ALP_STATUS_OK);
	    			CPPUNIT_ASSERT(strcmp(val_buf, string_vals[i]) == 0);
			}
			break;
		}
	}

	// ----- now remove a few -----
	gpointer gp = g_list_nth_data(glist_dst, 3);
	glist_dst = g_list_remove(glist_dst, gp);
	gp = g_list_nth_data(glist_dst, 1);
	glist_dst = g_list_remove(glist_dst, gp);

	// verify that glist_dst has its own copy, not pointed by glist_src 
	printf("GStest: ---- removed 2 elements from glist_dst ----\n");
	len_d = g_list_length(glist_dst);
	len_s = g_list_length(glist_src);
	CPPUNIT_ASSERT(len_d == (listLen - 2) && len_s == listLen);



	// free the lists
	alp_global_settings_value_list_free(glist_dst);
	alp_global_settings_value_list_free(glist_src);

    alp_global_settings_value_free(gsvalue_tmp);
    alp_global_settings_value_clean(&gsvalue_src);

}

//
// temporary control test flow via environment value
// returns true or false
static bool tmp_testenv_ctrl(const char* envname, const char* expval)
{
	char* enval = getenv(envname);
	if (enval == NULL)
		return false;
	printf("%s = %s\n", envname, enval);
	if (strcmp (enval, expval)== 0)
		return true;
	return false;
}

//
// memory image out of ../example_data/example_defaults.xml
//
const char test1_xml[] =
" <root>\n"
"    <default_mode>0753</default_mode>\n"              // 753 for test
"    <default_gid>106</default_gid>\n"
"    <default_uid>108</default_uid>\n"
"    <keyvalue>\n"
"        <key>a</key>\n"
"        <type>string</type>\n"
"        <value>aaaaaaaaaaa</value>\n"
"    </keyvalue>\n"
"    <keyvalue>\n"
"        <key>b/b/b/b/b</key>\n"
"        <type>string</type>\n"
"        <value>bbbbbbbbbbb bbbbbbbb</value>\n"
"        <mode>0642</mode>\n"                          // 642 for test
"        <uid>global_settings</uid>\n"
"        <gid>trusted</gid>\n"
"    </keyvalue>\n"
"    <keyvalue>\n"
"        <key>iiiiii</key>\n"
"        <type>int</type>\n"
"#        <value>12345678901</value>\n"
"        <value>1234567890</value>\n"
"          <mode>0456</mode>\n"
"    </keyvalue>\n"
"    <keyvalue>\n"
"        <key>1///2///////3//4//5/////float</key>\n"
"        <type>float</type>\n"
"        <value>987654321.123</value>\n"
"        <uid>1060</uid>\n"                            // test
"        <gid>1080</gid>\n"                            // test
"    </keyvalue>\n"
"    <keyvalue>\n"
"        <key>1/2/3/4/5/bool</key>\n"
"        <type>bool</type>\n"
"        <value>true</value>\n"
"    </keyvalue>\n"
"    <keyvalue>\n"
"        <key>1//2//3//4//5//bool2</key>\n"
"        <type>boolean</type> \n"
"        <value>fALse</value> \n"
"    </keyvalue> \n"
"    <keyvalue> \n"
"        <key>1//2//3//4//5//6//7//8</key> \n"
"        <type>dir</type>\n"
"    </keyvalue>   \n"
"     <keyvalue>\n"
"          <key>blob-int</key>\n"
"          <type>blob</type>\n"
"          <value type=\"int\">12345678</value>\n"
"     </keyvalue>\n"
"     <keyvalue>\n"
"          <key>blob-string</key>\n"
"          <type>blob</type>\n"
"          <value type=\"string\">a string</value>\n"
"          <mode>0123</mode>\n"
"     </keyvalue>\n"
"     <keyvalue>\n"
"          <key>blob2</key>\n"
"          <type>blob</type>\n"
"          <value type=\"hex\">00 01 02 03 04</value>\n"
"          <mode>0123</mode>\n"
"     </keyvalue>\n"
"     <keyvalue>\n"
"          <key>blob3</key>\n"
"          <type>blob</type>\n"
"          <value type=\"hex\">05060708090afe0b</value>\n"
"          <mode>0123</mode>\n"
"     </keyvalue>\n"
" </root>";


/**
 * @brief APIs tested: alp_global_settings_import_from_file
 *
 * @description using an example xml file as preference input file, calls
 *		alp_global_settings_import_from_file(...)
 *		then, get some values and compare against the ones in the xml file.
 */ 		
void GlobalSettingsTest_AP::import_from_xml_file(void)
{
    bool test_uid_gid_too = true;
/**
    // temporary playing around ... this way works, tho
    const char* envname = "GSET_EVAL_BUG151411_UID_GID_TOO";
    const char* expval = "TRUE";
    test_uid_gid_too = tmp_testenv_ctrl(envname, expval);
**/

//setegid(5000);
//seteuid(500);
    printf("GStest: import_from_xml_file: euid=%d, egid=%d\n", geteuid(), getegid());


    AlpGlobalSettingsValueType type;
    mode_t mode;
    uid_t uid, gid;
    const unsigned char blob2_value[] = {0x00, 0x01, 0x02, 0x03, 0x04};
    const unsigned char blob3_value[] = {0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0xfe, 0x0b};
    CPPUNIT_ASSERT(context != 0);
    
	// prepare the input test xml file from memory
	const char* xml_filename = "/tmp/test1.xml";
	int fd = open (xml_filename, O_RDWR | O_CREAT | O_TRUNC, 0644);
    CPPUNIT_ASSERT (fd >= 0);
    CPPUNIT_ASSERT(write(fd, (char*) test1_xml, strlen(test1_xml)) > 0);
    close (fd);


	// the API to test
 	status = alp_global_settings_import_from_file(context, xml_filename);
    printf("GStest: import_from_xml_file: status = 0x%x\n", status);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	//
	// get key values and verify
	//
	
    char   svalue[100];
    status = alp_global_settings_get_key_string_value(context, "b/b/b/b/b",  svalue, sizeof(svalue));
    printf("GStest: import_from_xml_file: status = 0x%x, b/b/b/b/b svalue = %s\n", status, svalue);
    CPPUNIT_ASSERT((status == ALP_STATUS_OK) && (strcmp(svalue, "bbbbbbbbbbb bbbbbbbb")==0));

	double fvalue = 0;

    status = alp_global_settings_get_key_float_value(context, "1/2/3/4/5/float", &fvalue); 
    printf("GStest: import_from_xml_file: status = 0x%x, 1/2/3/4/5/float = %lf\n", status, fvalue);
    CPPUNIT_ASSERT((status == ALP_STATUS_OK) && (fabs(fvalue - 987654321.123) < 0.00001 * 987654321.123));

    status = alp_global_settings_get_key_float_value(context, "1//2//3//4//5//float", &fvalue); 
    printf("GStest: import_from_xml_file: status = 0x%x, 1/2/3/4/5/float = %lf\n", status, fvalue);
    CPPUNIT_ASSERT((status == ALP_STATUS_OK) && (fabs(fvalue - 987654321.123) < 0.00001 * 987654321.123));

//NEW verify BUG151411 fix - 
    status = alp_global_settings_get_mode(context, "b/b/b/b/b", &mode);
    printf(" key: b/b/b/b/b mode = 0%o\n", mode);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK && ((mode & 0777) == 0642));

    status = alp_global_settings_get_mode(context, "b/b/b", &mode);
    printf(" dir: b/b/b mode = 0%o\n", mode);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK && ((mode & 0777) == 0753));

    status = alp_global_settings_get_mode(context, "1/2/3/4/5/float", &mode);
    printf(" key: 1/2/3/4/5/float mode = 0%o\n", mode);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK && ((mode & 0777) == 0753));

if (test_uid_gid_too)
{
    status = alp_global_settings_get_uid(context, "1/2/3/4/5/float", &uid);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    status = alp_global_settings_get_gid(context, "1/2/3/4/5/float", &gid);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    printf(" key: 1/2/3/4/5/float uid=%d, gid=%d\n", uid, gid);
    //CPPUNIT_ASSERT(uid==1060 && gid==1080);     // key's specified one
}

    status = alp_global_settings_get_mode(context, "1/2/3/4/5", &mode);
    printf(" dir: 1/2/3/4/5 mode = 0%o\n", mode);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK && ((mode & 0777) == 0753));

if (test_uid_gid_too)
{
    status = alp_global_settings_get_uid(context, "1/2/3/4/5", &uid);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    status = alp_global_settings_get_gid(context, "1/2/3/4/5", &gid);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    printf(" dir: 1/2/3/4/5 uid=%d, gid=%d\n", uid, gid);
    //CPPUNIT_ASSERT(uid==106 && gid==108);      // default's specified one
}

    status = alp_global_settings_get_mode(context, "1", &mode);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK && ((mode & 0777) == 0753));

	int    ivalue = 0;
    status = alp_global_settings_get_key_int_value(context, "iiiiii", &ivalue); 
    printf("GStest: import_from_xml_file: status = 0x%x, iiiiii = %d\n", status, ivalue);
    CPPUNIT_ASSERT((status == ALP_STATUS_OK) && (ivalue == 1234567890));
    //?CPPUNIT_ASSERT((status == ALP_STATUS_OK) && (ivalue == 12345678901));

	gboolean bvalue = true;
    status = alp_global_settings_get_key_bool_value(context, "1//2//3//4//5//bool2", &bvalue); 
    CPPUNIT_ASSERT((status == ALP_STATUS_OK) && (bvalue == false));


	//-----
    status = alp_global_settings_get_type(context, "1//2//3//4//5//6//7//8", &type);
	CPPUNIT_ASSERT((status == ALP_STATUS_OK) && (type == ALP_GLOBAL_SETTINGS_TYPE_DIR));

    AlpGlobalSettingsValue* vt = alp_global_settings_value_alloc();
    CPPUNIT_ASSERT(vt != NULL);
    status = alp_global_settings_get(context, "1//2//3//4//5//6//7//8", vt);
    printf("GStest: import_from_xml_file: status = 0x%x, (dir 1/2/3/4/5/6/7/8)\n", status);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);

	//-----
	char tmp[100];
	int  size, i;
	
	status = alp_global_settings_get_type(context, "blob-int", &type);
	CPPUNIT_ASSERT((status == ALP_STATUS_OK) && (type == ALP_GLOBAL_SETTINGS_TYPE_BLOB));
				    
	status = alp_global_settings_get_key_blob_value(context, "blob-int", tmp, sizeof(tmp), &i);
	ivalue = *((int*)(tmp));
    printf("GStest: import_from_xml_file: status = 0x%x, blob-int = %d, i=%d\n", status, ivalue, i);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
	CPPUNIT_ASSERT(ivalue == 12345678);

	//-----
	status = alp_global_settings_get_type(context, "blob-string", &type);
	CPPUNIT_ASSERT((status == ALP_STATUS_OK) && (type == ALP_GLOBAL_SETTINGS_TYPE_BLOB));
				    
	status = alp_global_settings_get_key_blob_value(context, "blob-string", tmp, sizeof(tmp), &i);
	tmp[i] = '\0';
    printf("GStest: import_from_xml_file: status = 0x%x, blob-string = %s, i=%d\n", status, tmp, i);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
	CPPUNIT_ASSERT(memcmp(tmp, "a string", i) == 0);


    status = alp_global_settings_get_mode(context, "blob-string", &mode);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
	CPPUNIT_ASSERT((mode & 0777) == 0123);
    
    status = alp_global_settings_get_mode(context, "iiiiii", &mode);
	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
	CPPUNIT_ASSERT((mode & 0777) == 0456);
    
    status = alp_global_settings_get_key_blob_value(context, "blob2",  svalue, sizeof(svalue),  &size);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(!memcmp(svalue, blob2_value, size));

    status = alp_global_settings_get_key_blob_value(context, "blob3",  svalue, sizeof(svalue),  &size);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    CPPUNIT_ASSERT(!memcmp(svalue, blob3_value, size));

    alp_global_settings_value_clean(vt);

seteuid(0);
setegid(0);
}


void GlobalSettingsTest_AP::test_dir_list1(void)
{
	_create_dir_traverse_entries("A/B/DDD");

    _list_key_test("*/DDD/*");

    _list_key_test("*/DDD/*[23]*", 0, "/A/B/DDD/3");
}

void GlobalSettingsTest_AP::test_dir_list2(void) // questionable?
{
	_create_dir_traverse_entries("aa/ b b b & ?@B/ddd "); // can a dir have blank?

    _list_key_test("*/ddd*");

    _list_key_test("*/ddd*[23]*", 0, "/aa/ b b b & ?@B/ddd /3");
}

void GlobalSettingsTest_AP::test_dir_list3(void) // will be treated as /a/b/ddd
{
	_create_dir_traverse_entries("a/b/ddd");

    _list_key_test("*/ddd/*");

    _list_key_test("*/ddd/*[23]*", 0, "/a/b/ddd/3");
}

void GlobalSettingsTest_AP::_create_dir_traverse_entries(const char* keydir)
{
    char key[200];
    char keybuf[300];
    char valbuf[200];
    AlpGlobalSettingsValue* vt = alp_global_settings_value_alloc();
    alp_status_t status;
    AlpGlobalSettingsEntryIterator* iterator = 0;
    CPPUNIT_ASSERT(context != 0);

	for (int i = 1; i < 4; i++)
	{
    	sprintf(keybuf, "%s/%d", keydir, i); 
    	sprintf(valbuf, "value%d", i); 
    	status = alp_global_settings_set_key_string_value(context, keybuf, valbuf);
    	CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    	glib_event_process();
	}

    status = alp_global_settings_findkeys_start(context, keydir,  &iterator);
    printf("findkeys_start %s status = 0x%x\n", keydir, status);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
    int n = 0;
    do
    {
        status = alp_global_settings_findkeys_get_key(iterator, key, sizeof(key));
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        sprintf(keybuf, "%s/%s", keydir, key); 
        printf("key: %s\n", key);

        status = alp_global_settings_get(context, keybuf, vt);
        CPPUNIT_ASSERT(status == ALP_STATUS_OK);
        
        status = alp_global_settings_findkeys_next(iterator);
        n++;
    } while (status != ALP_STATUS_GLOBAL_SETTINGS_ENTRY_ITERATOR_AT_END);
    
    alp_global_settings_value_free(vt);
    
    status = alp_global_settings_findkeys_end(iterator);
    CPPUNIT_ASSERT(status == ALP_STATUS_OK);
}

