#include <gtk/gtk.h>

#include <sys/types.h> // used for getpid()
#include <unistd.h>    // used for getpid()
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
                     
                     
#include <string.h>
#include <stdlib.h>

#include <assert.h>

#include <hiker/ipc.h>
                     
                     
#include "alp_ipc_unit_test.h"

static int            parentFD = -1;
static char*          parentPipe = NULL;
static AlpChannel*    ipcChannel = NULL;
static AlpConnection* mostRecentIpcConnection = NULL;
static GMainLoop*     mainLoop = NULL;

void
send_response_message(AlpConnection* connection, int msgID)
{
	AlpMessage* message = alp_ipc_message_create();
	alp_status_t status;

	if (message == NULL)
		return;

	status = alp_ipc_message_pack_start(message, 
								 msgID);

	if (status == ALP_STATUS_OK)
		status = alp_ipc_message_pack_end(message);

	if (status == ALP_STATUS_OK) 
		status = alp_ipc_connection_send(connection, message); 

	alp_ipc_message_destroy(message);  
}

void
send_response_to_message(AlpConnection* connection, int msgID, AlpMessage* messageToRespondTo)
{
	AlpMessage* message = alp_ipc_message_create();
	alp_status_t status;

	if (message == NULL)
		return;

	status = alp_ipc_message_response_pack_start(message, 
										msgID,
										messageToRespondTo);

	if (status == ALP_STATUS_OK)
		status = alp_ipc_message_pack_end(message);

	if (status == ALP_STATUS_OK) 
		status = alp_ipc_connection_send(connection, message); 

	alp_ipc_message_destroy(message);  
}

static void serverReceiveMessageCB(AlpConnection* connection,
							AlpMessage* message, gpointer cbData)
{
	gint helloInt;
	gchar* helloStr;
	gchar* nullStr;
	guint tmpUint;
	float tmpFloat;
	guchar* tmpByteArray;

	static gint goodbyeCount = 0;
	alp_status_t status;
	guint i;
    
	gboolean anyErrors = FALSE;

	status = alp_ipc_message_unpack_start(message);
	if (status != ALP_STATUS_OK)
		return;

	switch (alp_ipc_message_get_message_ID(message)) {
	case UNIT_TEST_MESSAGE_HELLO:
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_string(message, &helloStr);

		if (status == ALP_STATUS_OK) {
			status = alp_ipc_message_unpack_string(message, &nullStr);

			if (nullStr != NULL) {
				send_response_message(connection, UNIT_TEST_MESSAGE_SERVER_ERROR);
				break;
			}
		}


		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_int32(message, &helloInt);

		if (status == ALP_STATUS_OK) {
			send_response_message(connection, UNIT_TEST_MESSAGE_SERVER_OK);

			g_free(helloStr);
		} else {
			send_response_message(connection, UNIT_TEST_MESSAGE_SERVER_ERROR);
		}
		break;

	case UNIT_TEST_MESSAGE_PACK_EACH_TYPE:
		if (status == ALP_STATUS_OK) {
			status = alp_ipc_message_unpack_int32(message, &helloInt);
		}
        
		if (status == ALP_STATUS_OK) {
			if (helloInt != -23) {
				anyErrors = TRUE;
			}
			status = alp_ipc_message_unpack_uint32(message, &tmpUint);
		}

		if (status == ALP_STATUS_OK) {
			if (tmpUint != 23) {
				anyErrors = TRUE;
			}
			status = alp_ipc_message_unpack_float(message, &tmpFloat);
		}

		if (status == ALP_STATUS_OK) {
			if (tmpFloat != -23.0f) {
				anyErrors = TRUE;
			}
			status = alp_ipc_message_unpack_string(message, &helloStr);
		}

		if (status == ALP_STATUS_OK) {
			if (strcmp(helloStr, "-23") != 0) {
				anyErrors = TRUE;
			}
			g_free(helloStr);

			status = alp_ipc_message_unpack_byte_array(message, &tmpByteArray, &tmpUint);
		}

		if (status == ALP_STATUS_OK) {
			assert(tmpUint == 23);
			for (i=0; i<tmpUint; i++) {
				if ((i + tmpByteArray[i]) != 23) {
					anyErrors = TRUE;
	    
					break;
				}
			}
			g_free(tmpByteArray);
		}

		if ((status == ALP_STATUS_OK) && (anyErrors == FALSE)) {
			send_response_message(connection, UNIT_TEST_MESSAGE_SERVER_OK);
		} else {
			send_response_message(connection, UNIT_TEST_MESSAGE_SERVER_ERROR);
		}
		break;


	case UNIT_TEST_MESSAGE_LONG_FORMAT_STRING:
		anyErrors = FALSE;
		for (i=0; i<128; i++) {
			if (status == ALP_STATUS_OK)
				status = alp_ipc_message_unpack_int32(message, &helloInt);
			else
				break;
	
			if (helloInt != (gint)i) {
				anyErrors = TRUE;
				break;
			}
		}

		if ((status == ALP_STATUS_OK) && (anyErrors == FALSE)) {
			send_response_message(connection, UNIT_TEST_MESSAGE_SERVER_OK);
		} else {
			send_response_message(connection, UNIT_TEST_MESSAGE_SERVER_ERROR);
		}
		break;

	case UNIT_TEST_MESSAGE_LONG_MESSAGE:
		status = alp_ipc_message_unpack_string(message, &helloStr);

		anyErrors = FALSE;

		if (status == ALP_STATUS_OK) {
			if (strlen(helloStr) != gLongMessageLength) {
				anyErrors = TRUE;
			}

			if (anyErrors == FALSE) {
				// helloStr consists of 0123456789 over and over
				for (i=0; i<gLongMessageLength; i++) {
					if (helloStr[i] != (gint)('0' + i%10)) {
						anyErrors = TRUE;
						break;
					}
				}
			}
			g_free(helloStr);
		}
	
		if ((status == ALP_STATUS_OK) && (anyErrors == FALSE)) {
			send_response_message(connection, UNIT_TEST_MESSAGE_SERVER_OK);
		} else {
			send_response_message(connection, UNIT_TEST_MESSAGE_SERVER_ERROR);
		} 
		break;

	case UNIT_TEST_MESSAGE_GOODBYE:
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_string(message, &helloStr);
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_int32(message, &helloInt);

		if (status == ALP_STATUS_OK) {
			goodbyeCount++;
			if (goodbyeCount == NUM_GOODBYE_MESSAGES) {
				//	  g_print("GOT ALL %d GOODBYE MESSAGES: %s %d\n", 
				//		  NUM_GOODBYE_MESSAGES, helloStr, helloInt);
				goodbyeCount = 0;
			}
			g_free(helloStr);
		}
		break;

	case UNIT_TEST_MESSAGE_HELLO_WITH_RESPONSE:
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_string(message, &helloStr);
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_int32(message, &helloInt);

		if (status == ALP_STATUS_OK) {
			send_response_to_message(connection, UNIT_TEST_MESSAGE_SERVER_OK, message);

			g_free(helloStr);
		} else {
			send_response_to_message(connection, UNIT_TEST_MESSAGE_SERVER_ERROR, message);
		}
		break;


	case UNIT_TEST_MESSAGE_HELLO_WITH_RESPONSE_TIMEOUT:
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_string(message, &helloStr);
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_int32(message, &helloInt);

		// don't reply to force a timeout condition

		break;

	case UNIT_TEST_MESSAGE_LONG_MESSAGE_WITH_RESPONSE:
		status = alp_ipc_message_unpack_string(message, &helloStr);

		anyErrors = FALSE;

		if (status == ALP_STATUS_OK) {
			if (strlen(helloStr) != gLongMessageLength) {
				anyErrors = TRUE;
			}

			if (anyErrors == FALSE) {
				// helloStr consists of 0123456789 over and over
				for (i=0; i<gLongMessageLength; i++) {
					if (helloStr[i] != (gint)('0' + i%10)) {
						anyErrors = TRUE;
						break;
					}
				}
			}

			g_free(helloStr);
		}

		if ((status == ALP_STATUS_OK) && (anyErrors == FALSE)) {
			send_response_to_message(connection, UNIT_TEST_MESSAGE_SERVER_OK, message);

		} else {
			send_response_to_message(connection, UNIT_TEST_MESSAGE_SERVER_ERROR, message);
		}
		break;

	case	UNIT_TEST_MESSAGE_HELLO_SYNCH_WITH_EXTRA_MESSAGE:
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_string(message, &helloStr);
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_int32(message, &helloInt);

		if (status == ALP_STATUS_OK) {
			// first send an intervening message that the
			// client should cue
			send_response_message(connection, 
							  UNIT_TEST_MESSAGE_SERVER_DEFERRED);

			// then send the OK message
			send_response_message(connection, 
							  UNIT_TEST_MESSAGE_SERVER_OK);

			g_free(helloStr);
		} else {
			send_response_message(connection, 
							  UNIT_TEST_MESSAGE_SERVER_ERROR);
		}
		break;

	case UNIT_TEST_MESSAGE_HELLO_RESPONSE_WITH_EXTRA_MESSAGE:
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_string(message, &helloStr);
		if (status == ALP_STATUS_OK)
			status = alp_ipc_message_unpack_int32(message, &helloInt);

		if (status == ALP_STATUS_OK) {
			send_response_message(connection, UNIT_TEST_MESSAGE_SERVER_DEFERRED);
			
			send_response_to_message(connection, UNIT_TEST_MESSAGE_SERVER_OK, message);

			g_free(helloStr);
		} else {
			send_response_to_message(connection, UNIT_TEST_MESSAGE_SERVER_ERROR, message);
		}
		break;



	case UNIT_TEST_MESSAGE_SHUTDOWN_SERVER:
		if (status == ALP_STATUS_OK) {
			g_main_loop_quit(mainLoop);
		}
		break;

	default:
		g_print("Server received unknown message\n");
	}

	status= alp_ipc_message_unpack_end(message);

	alp_ipc_message_destroy(message);
}

static void serverConnectCB(AlpConnection* connection, gpointer cbData)
{
	mostRecentIpcConnection = connection;

	alp_ipc_connection_register_receive_callback(connection,
										serverReceiveMessageCB,
										NULL);
}

static void serverDisconnectCB(AlpConnection* connection, gpointer cbData)
{
	if (connection == mostRecentIpcConnection) {
		mostRecentIpcConnection = NULL;
	}
}

static gboolean parentHungup(GIOChannel * source, GIOCondition condition, gpointer data)
{
	GMainLoop * loop = data;
	g_main_loop_quit(loop);
	return FALSE;
}

int main( int   argc,
          char *argv[] )
{
	mainLoop = g_main_loop_new(NULL, TRUE);

	// trivial socket to parent to make sure we synchronize, and shut down in the event
	// of a problem
	if (strcmp(argv[1], "-parentFD") == 0) {
		parentPipe = "";
		parentFD = atoi(argv[2]);
		
		if (parentFD == -1) {
			return 1;
		}
		
		GIOChannel * channel = g_io_channel_unix_new(parentFD);
		g_io_add_watch(channel, G_IO_HUP, parentHungup, mainLoop);
		
		argv += 2;
		argc -= 2;
	}

	if (strcmp(argv[1], "-connectTest") == 0)
		ipcChannel = alp_ipc_channel_create(UNIT_TEST_CHANNEL_NAME_CONNECT,
									 ALP_IPC_CHANNEL_ACCESS_OWNER);
	else if (strcmp(argv[1], "-messageTest") == 0)
		ipcChannel = alp_ipc_channel_create(UNIT_TEST_CHANNEL_NAME_MESSAGE,
									 ALP_IPC_CHANNEL_ACCESS_OWNER);
	else
		assert(false);
  
	alp_ipc_channel_register_connect_callback(ipcChannel,
									  serverConnectCB,
									  NULL);

	alp_ipc_channel_register_disconnect_callback(ipcChannel,
										serverDisconnectCB,
										NULL);

	// tell parent we've started up
	if (parentFD != -1)
		write(parentFD, "\x00", 1);

	/* loop and let the fun begin */
	g_main_loop_run(mainLoop);

	alp_ipc_channel_destroy(ipcChannel);
 
	return 0;
}
