/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <stdio.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>

#include <glib.h>

#include <hiker/notifymgr.h>

#include <hiker/attnmgr_common.h> /* pick up attn alert notifications */

int callback_wait(const char * type, void * details, int length, int * handled, const char * appID, void * userData)
{
	return 0;
}



int cmd_wait(void)
{
	alp_status_t result;

	result = alp_notify_register_server("nonexistant-notification", callback_wait, NULL, ALP_NOTIFY_PRIORITY_NORMAL, 0);

	while (result != ALP_STATUS_OK) {
		struct timespec t;
		t.tv_sec = 0;
		t.tv_nsec = 1000000000 / 4;
		
		nanosleep(&t, NULL);

		result = alp_notify_register_server("nonexistant-notification", callback_wait, NULL, ALP_NOTIFY_PRIORITY_NORMAL, 0);

		if (isatty(fileno(stdout))) {
			printf(".");
			fflush(stdout);
		}
	}

	return 0;
}

GMainLoop *main_loop = NULL;

gchar *
decode_details (const char *type, void *details, int length)
{
    gchar *decoded_detail = NULL;

    if (strncmp(type, ALP_NOTIFY_EVENT_ATTN_ALERT,
               sizeof(ALP_NOTIFY_EVENT_ATTN_ALERT) - 1) == 0 &&
        length > (int)sizeof(AlpNotifyEventAttnAlert)) {
        AlpNotifyEventAttnAlert *event = g_malloc0(length);

        if (event) {
            memcpy ((void *)event, details, length);
            decoded_detail = g_strdup_printf("handle %s", event->handle);
            g_free(event);
        }        
    } else {
        int decode_length = length;
        if (decode_length < 0)
            decode_length = 0; /* incase negative length passed in */
        else if (decode_length > 16)
            decode_length = 16; /* decode up to 16 bytes */

        decoded_detail = g_malloc (10 + (decode_length * 3) + 3 + 1);
        if (decoded_detail) {
            int i;
            int offset;
            gchar *cp;

            snprintf(decoded_detail, 10, "len=%-4d ", length);
            cp = (gchar *) details;
            offset = strlen(decoded_detail);
            for (i = 0 ; i < decode_length; i++) {
                snprintf(decoded_detail + offset, 4, "%02x ", cp[i]);
                offset += 3;
            }
            if (length > decode_length) {
                snprintf(decoded_detail + offset, 4, "...");
            }
        }
    }
    return decoded_detail;
}

int callback_listen(const char *type,
                    void       *details,
                    int         length,
                    int        *handled,
                    const char *appID,
                    void       *userData)
{
    struct timeval  tv;
    struct timezone tz;
    struct tm       tm;
    char  *decoded_detail;
    char   buf[16];

    gettimeofday (&tv, &tz);
    localtime_r (&tv.tv_sec, &tm);
    decoded_detail = decode_details(type, details, length);
    printf("%02d:%02d:%02d.%04d %s -> %s: %s\n",
           tm.tm_hour, tm.tm_min, tm.tm_sec, (int)(tv.tv_usec / 1000),
           appID, type, decoded_detail);
    g_free (decoded_detail);
	return ALP_NOTIFY_CALLBACK_CONTINUE;
}

int cmd_listen(char *pattern)
{
	alp_status_t    result;

	result =
        alp_notify_register_server(pattern, callback_listen, NULL, ALP_NOTIFY_PRIORITY_NORMAL, 0);

    main_loop = g_main_loop_new (NULL, FALSE);
    g_main_loop_run (main_loop);

	return 0;
}

char * argv0;

int usage()
{
    printf("Usage: %s	--OPTION [args]\n", argv0);
    printf("Available Options:\n"
           "    --wait                      Don't return until the notifymgr server is available\n"
           "    --listen [notify-pattern]   Listen for notications that match the pattern\n");
  return 2;
}

int main(int argc, char *argv[])
{
  char *opt;
  argv0 = argv[0];


  if (argc < 2)
    return usage();
    
  opt = argv[1];

  if (*opt == '-') {
    opt++;
    // skip second dash 
    if (*opt == '-') opt++;

    if (strcmp(opt, "wait") == 0 && argc == 2)
      return cmd_wait();
    else if (strcmp(opt, "listen") == 0) {
      if (argc == 2)
        return cmd_listen("/");
      else if (argc == 3)
        return cmd_listen(argv[2]);
    }
  }
  return usage();
}
