/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/notifymgr.h>
#include <hiker/prv/notifymgr_prv.h>
#include <hiker/fail.h>

#include <hiker/ipc.h>

#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <string.h>

extern void alp_notify_client_mesg(int * buf, int length);

int alp_notify_client_send(void * buf, int length, int sync, int timeout, int * handled)
{
	AlpConnection * connect = 0;
	AlpMessage * message = 0;
	AlpMessage * responseMessage = 0;
	alp_status_t err;
	int status;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_prv_notify_client_send\n");
#endif
	// connect to the server
	err = alp_ipc_channel_connect_synch(ALP_NOTIFY_MGR_SERVER_CHANNEL,ALP_IPC_CONNECTION_USAGE_HINT_SYNCH_ONLY,0,&connect);
	if (err != ALP_OK) {
#ifdef ALP_NOTIFY_DEBUG
		printf("alp_notify_client_send: alp_ipc_channel_synch err=%d\n",err);
#endif
		goto Done;
	}

	// pack the message
	message = alp_ipc_message_create();
	if (!message) {
#ifdef ALP_NOTIFY_DEBUG
		printf("alp_notify_client_send: alp_ipc_message_create\n");
#endif
		err = ALP_NOTIFY_ERR_OUT_OF_MEMORY;
		goto Done;
	}
	alp_ipc_message_pack_start(message,0);
	alp_ipc_message_pack_byte_array(message,buf,length);
	alp_ipc_message_pack_end(message);

	// send the message to the server
	err = alp_ipc_connection_send_with_response(connect,message,
									    0,&responseMessage);
	if (err != ALP_OK) {
#ifdef ALP_NOTIFY_DEBUG
		printf("alp_notify_client_send: alp_ipc_connection_send_with_response err=%d\n",err);
#endif
		goto Done;
	}

	// unpack the status message from the server
	alp_ipc_message_unpack_start(responseMessage);
	alp_ipc_message_unpack_int32(responseMessage,&status);
	alp_ipc_message_unpack_end(responseMessage);

	// check if sync
	if (status == ALP_OK && sync) {
		int tmp;

		if (responseMessage) {
			alp_ipc_message_destroy(responseMessage);
			responseMessage = NULL;
		}

		// wait to receive a response from the server
        	err = alp_ipc_connection_receive(connect,ALP_IPC_RECEIVE_ANY,timeout,&responseMessage);
        	if (err != ALP_OK) {
#ifdef ALP_NOTIFY_DEBUG
			printf("alp_notify_client_send: alp_ipc_connection_receive err=%d\n",err);
#endif
			goto Done;
		}

		// unpack the status message from the server
		alp_ipc_message_unpack_start(responseMessage);
		alp_ipc_message_unpack_int32(responseMessage,
							    handled ? handled : &tmp);
		alp_ipc_message_unpack_end(responseMessage);
	}
Done:
	// disconnect and cleanup
	if (connect) alp_ipc_channel_disconnect(connect);
	if (message) alp_ipc_message_destroy(message);
	if (responseMessage) alp_ipc_message_destroy(responseMessage);

	// return status
	if (err == ALP_STATUS_IPC_TIMEOUT) return ALP_NOTIFY_ERR_TIMEOUT;
	return err != ALP_OK ? ALP_NOTIFY_ERR_IPC : status;
}

void * alp_notify_wait_for_callbacks(void * ptr)
{
	AlpConnection * connect = 0;
	AlpMessage * message = 0;
	AlpMessage * responseMessage = 0;
	alp_status_t err;
	AlpNotifyCmdCallback request;
	int status;
	guchar * buf;
	gint length;

	// fill out the message header
	request.cmd = ALP_NOTIFY_CMD_CALLBACK;
	request.size = sizeof(AlpNotifyCmdCallback);

	// fill out the message body
	sprintf(request.appID,"process-%d",getpid());
	request.zero = 0;

	// connect to the server
	err = alp_ipc_channel_connect_synch(ALP_NOTIFY_MGR_SERVER_CHANNEL,ALP_IPC_CONNECTION_USAGE_HINT_SYNCH_ONLY,0,&connect);
	if (err != ALP_OK) {
#ifdef ALP_NOTIFY_DEBUG
		printf("alp_notify_wait_for_callback: alp_ipc_channel_sync err=%x\n",err);
#endif
		goto Done;
	}

	// pack the message
	message = alp_ipc_message_create();
	if (!message) {
#ifdef ALP_NOTIFY_DEBUG
		printf("alp_notify_wait_for_callback: alp_ipc_create_message\n");
#endif
		err = ALP_NOTIFY_ERR_OUT_OF_MEMORY;
		goto Done;
	}
	alp_ipc_message_pack_start(message,0);
	alp_ipc_message_pack_byte_array(message,(char *)&request,request.size);
	alp_ipc_message_pack_end(message);

	// send the message to the server
	err = alp_ipc_connection_send_with_response(connect,message,
									    0,&responseMessage);
	if (err != ALP_OK) {
#ifdef ALP_NOTIFY_DEBUG
		printf("alp_notify_wait_for_callback: alp_ipc_connection_send_with_response err=%x\n",err);
#endif
		goto Done;
	}

	// unpack the status message from the server
	alp_ipc_message_unpack_start(responseMessage);
	alp_ipc_message_unpack_int32(responseMessage,&status);
	alp_ipc_message_unpack_end(responseMessage);

	// set pid
	*(int *)ptr = getpid();

	// wait for callback messages
	for (;;) {

		if (responseMessage) {
			alp_ipc_message_destroy(responseMessage);
			responseMessage = NULL;
		}

		// wait to receive a response from the server
        	err = alp_ipc_connection_receive(connect,ALP_IPC_RECEIVE_ANY,0,&responseMessage);
        	if (err != ALP_OK) {
#ifdef ALP_NOTIFY_DEBUG
			printf("alp_notify_wait_for_callback: alp_ipc_connection_receive err=%x\n",err);
#endif
			goto Done;
		}

		// unpack the message from the server
		alp_ipc_message_unpack_start(responseMessage);
		alp_ipc_message_unpack_byte_array(responseMessage,&buf,&length);
		alp_ipc_message_unpack_end(responseMessage);
		ALP_FAIL_NON_FATAL_IF(buf == 0,"unpacked message buffer is null");
		ALP_FAIL_NON_FATAL_IF(length == 0,"unpacked message length is zero\n");

		// process message
		if (buf && length) alp_notify_client_mesg((int *)buf,length);

		// free the buffer from unpack_byte_array
		if (buf) g_free(buf);
	}

Done:
	*(int *)ptr = 0xffffffff;
	if (message) alp_ipc_message_destroy(message);
	if (responseMessage) alp_ipc_message_destroy(responseMessage);
	if (connect) alp_ipc_channel_disconnect(connect);
	return (void *) err;
}
