/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*! @file alpSALPrv.c
   Functions used privately by Data Model.  */
#include <hiker/sal.h>
#include "sal_prv.h"


/******************************************************************************
* DESCRIPTION: translates from SQLITE_ error to alp_status_t
******************************************************************************/
alp_status_t alp_prv_sal_translate_sqlite_err(int32_t sqliteErr)
{
	switch (sqliteErr) {
		case SQLITE_OK: 		 /* No error */
			return ALP_STATUS_OK;
        case SQLITE_BUSY:        /* The datamodel file is locked */
			return ALP_STATUS_SAL_BUSY;
        case SQLITE_LOCKED:      /* A table in the datamodel is locked */
			return ALP_STATUS_SAL_LOCKED;
        case SQLITE_NOMEM:       /* A malloc() failed */
			return ALP_STATUS_SAL_NOMEM;
        case SQLITE_READONLY:    /* Attempt to write a readonly datamodel */
			return ALP_STATUS_SAL_READONLY;
        case SQLITE_INTERRUPT:   /* terminated by sqlite_interrupt() */
			return ALP_STATUS_SAL_INTERRUPT;
        case SQLITE_IOERR:       /* Some kind of disk I/O error occurred */
			return ALP_STATUS_SAL_IOERR;
        case SQLITE_CORRUPT:     /* The datamodel disk image is malformed */
			return ALP_STATUS_SAL_CORRUPT;
        case SQLITE_FULL:        /* Insertion failed because datamodel is full */
			return ALP_STATUS_SAL_FULL;
        case SQLITE_CANTOPEN:    /* Unable to open the datamodel file */
			return ALP_STATUS_SAL_CANTOPEN;
        case SQLITE_TOOBIG:      /* Too much data for one row of a table */
			return ALP_STATUS_SAL_TOOBIG;
        case SQLITE_CONSTRAINT:  /* Abort due to constraint violation */
			return ALP_STATUS_SAL_CONSTRAINT;
        case SQLITE_MISMATCH:    /* Data type mismatch */
			return ALP_STATUS_SAL_MISMATCH;
        case SQLITE_AUTH:        /* Authorization denied */
			return ALP_STATUS_SAL_AUTH;
        case SQLITE_ERROR:       /* SQL error or missing datamodel */
			return ALP_STATUS_SAL_ERROR;
        case SQLITE_MISUSE:      /* Library used incorrectly */
			return ALP_STATUS_SAL_MISUSE;
        case SQLITE_NOLFS:       /* Uses OS features not supported on host */
			return ALP_STATUS_SAL_NOLFS;
        case SQLITE_PROTOCOL:    /* Datamodel lock protocol error */
			return ALP_STATUS_SAL_PROTOCOL;
        case SQLITE_EMPTY:       /* (Internal Only) Datamodel table is empty */
			return ALP_STATUS_SAL_EMPTY;
        case SQLITE_SCHEMA:      /* The datamodel schema changed */
			return ALP_STATUS_SAL_SCHEMA;
        case SQLITE_NOTFOUND:    /* (Internal Only) Table or record not found */
			return ALP_STATUS_SAL_NOTFOUND;
        case SQLITE_INTERNAL:    /* An internal logic error in SQLite */
			return ALP_STATUS_SAL_INTERNAL;
        case SQLITE_PERM:        /* Access permission denied */
			return ALP_STATUS_SAL_PERM;
        case SQLITE_ABORT:       /* Callback routine requested an abort */
			return ALP_STATUS_SAL_ABORT;
	}

	return ALP_STATUS_OK;
}


/******************************************************************************
* DESCRIPTION: Returns true if file exists.  Works by opening file for 
* read - append, then seeking to end and counting bytes in file.  
******************************************************************************/
bool alp_prv_sal_file_exists(const char *fileName)
{
	FILE *fp;
	uint32_t length;

	if ((fp = fopen(fileName, "r")) == NULL) {
		return false;
	}
	/* measure length of file in bytes */
	fseek(fp, 0, SEEK_END);
	length = ftell(fp);
	fclose(fp);

	return length > 0;
}

