/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/

#define APP_E_NOERR 0
#define APP_E_USAGE 1
#define APP_E_OTHER 2

#define APP_A_SET 0
#define APP_A_CHECK 1
#define APP_A_PSET 2
#define APP_A_PGET 3

#include <stdio.h>
#include <string.h>
#include "hiker/security.h"

typedef unsigned int Err;
typedef struct {
  unsigned int operation;
  unsigned int which;
  char *pass;
  char *salt;
  unsigned int iteration;
} App;

static Err cli_parse( App *app, int argc, char **argv );
static void usage_message_emit( char *command );
static unsigned int app_prv_operation_get( char *trial );

int main( int argc, char *argv[] ) {
  Err err = APP_E_NOERR;
  App app;
  alp_status_t status;
  unsigned int correct;
  unsigned int iteration;
  unsigned int active;
  char *salt;
  do {
    if( APP_E_NOERR != ( err = cli_parse( &app, argc, argv ) ) ) break;

    switch( app.operation ) {
    case APP_A_SET:
      err = alp_spf_pkcs5_set( ALP_SPF_PKCS5_PIN, app.pass );
      break;
    case APP_A_CHECK:
      err = alp_spf_pkcs5_check( ALP_SPF_PKCS5_PIN, app.pass, & correct );
      printf( "correct = %d\n", correct );
      break;
    case APP_A_PSET:
      err = alp_spf_pkcs5_params_set( ALP_SPF_PKCS5_PIN, app.iteration, app.salt, 0 );
      break;
    case APP_A_PGET:
      err = alp_spf_pkcs5_params_get( ALP_SPF_PKCS5_PIN, &iteration, &salt, &active );
      printf("iteration = %d, salt = (%s), active = %d\n", iteration, salt, active );
      break;
    default:
      err = APP_E_USAGE;
      break;
    }
  } while( 0 );

  if( APP_E_USAGE == err ) {
    usage_message_emit( argv[0] );
  }
  return( err & 0xFF );
}

static Err cli_parse( App *app, int argc, char **argv ) {
  Err err = APP_E_NOERR;

  do {

    if( argc < 2 ) {
      err = APP_E_USAGE;
      break;
    }

    app->operation = app_prv_operation_get( argv[1] );

    switch( app->operation ) {
    case APP_A_SET:
    case APP_A_CHECK:
      if( argc < 3 ) {
	err = APP_E_USAGE;
	break;
      }
      app->pass = argv[2];
      break;
    case APP_A_PSET:
      if( argc < 4 ) {
	err = APP_E_USAGE;
	break;
      }
      app->salt = argv[2];
      app->iteration = atoi( argv[3] );
      break;
    case APP_A_PGET:
      break;
    default:
      err = APP_E_USAGE;
      break;
    }
  } while( 0 );

  return( err );
}

static void usage_message_emit( char *command ) {
  printf("%%APAST-E-USAGE; Usage is:\n; %s <operation> [options]\n; where operations and options are:\n; set <passphrase>\n; check <passphrase>\n; pset <salt> <iteration count>\n; pget\n", command );
}

static unsigned int app_prv_operation_get( char *trial ) {
  static const char *foo[] = { "set", "check", "pset", "pget" };
  unsigned int i;

  for( i = 0; i < 4; i++ ) {
    if( !strcmp( trial, foo[i] ) ) {
      break;
    }
  }

  return( i );
}
