/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/

#define APP_E_NOERR 0
#define APP_E_USAGE 1
#define APP_E_OTHER 2

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>
#include "hiker/prv/apas.h"

typedef unsigned int Err;
typedef struct {
  unsigned int packageid;
} App;

static Err cli_parse( App *app, int argc, char **argv );
static void usage_message_emit( char *command );

int main( int argc, char *argv[] ) {
  Err err = APP_E_NOERR;
  App app;
  alp_status_t status;
  unsigned int refcount;
  uid_t uid;
  gid_t gid;

  do {
    if( APP_E_NOERR != ( err = cli_parse( &app, argc, argv ) ) ) break;

    if( ALP_STATUS_SPF_APAS_E_NOERR != (status = alp_spf_apas_packageid_refcount_get( app.packageid, &refcount ) ) ){
      err = APP_E_OTHER;
      printf("%%APAST-F-PIDRCQ; Error %08X while trying to get the refcount for\n; package %u.\n", (unsigned int) status, app.packageid );
      break;
    }

    printf("%%APAST-S-PIDRCQ; The refcount for packageid %u is %u.\n", app.packageid, refcount );

    if( ALP_STATUS_SPF_APAS_E_NOERR != (status = alp_spf_apas_packageid_gid_get( app.packageid, &gid ) ) ){
      err = APP_E_OTHER;
      printf("%%APAST-F-PIDRCQ; Error %08X while trying to get the gid for package %u.\n", (unsigned int) status, app.packageid );
      break;
    }

    printf("%%APAST-S-PIDRCQG; The gid for packageid %u is %u.\n", app.packageid, (unsigned int) gid );

    if( ALP_STATUS_SPF_APAS_E_NOERR != (status = alp_spf_apas_packageid_uid_get( app.packageid, &gid ) ) ){
      err = APP_E_OTHER;
      printf("%%APAST-F-PIDRCQ; Error %08X while trying to get the uid for package %u.\n", (unsigned int) status, app.packageid );
      break;
    }

    printf("%%APAST-S-PIDRCQG; The uid for packageid %u is %u.\n", app.packageid, (unsigned int) uid );

  } while( 0 );

  if( APP_E_USAGE == err ) {
    usage_message_emit(argv[0]);
  }

  return( err & 0xFF );
}

static Err cli_parse( App *app, int argc, char **argv ) {
  Err err = APP_E_NOERR;

  do {
    if( argc < 2 ) {
      err = APP_E_USAGE;
      break;
    }

    app->packageid = atoi( argv[1] );

  } while( 0 );

  return( err );
}

static void usage_message_emit( char *command ) {
  printf("%%APAST-E-USAGE; Usage is:\n; %s <packageid>\n; packagid - the id of the package whose refcount will be queried.\n", command );
}

