/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/

#define APP_E_NOERR 0
#define APP_E_USAGE 1
#define APP_E_OTHER 2

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "hiker/prv/apas.h"

typedef unsigned int Err;
typedef struct {
  unsigned int packageid;
  unsigned int tell_refcount;
} App;

static Err cli_parse( App *app, int argc, char **argv );
static void usage_message_emit( char *command );

int main( int argc, char *argv[] ) {
  Err err = APP_E_NOERR;
  App app;
  alp_status_t status;
  unsigned int refcount;
  unsigned int *out_refcount;
  do {
    if( APP_E_NOERR != ( err = cli_parse( &app, argc, argv ) ) ) break;

    if( 0 != app.tell_refcount ) {
      out_refcount = &refcount;
    } else {
      out_refcount = NULL;
    }

    if( ALP_STATUS_SPF_APAS_E_NOERR != (status = alp_spf_apas_packageid_remove(app.packageid, out_refcount ) ) ) {
      err = APP_E_OTHER;
      printf("%%APAST-F-REMOVE; Error %08X while trying to remove package %u / %08X.\n", (unsigned int) status, app.packageid, (unsigned int) out_refcount );
      break;
    }

    printf("%%APAST-S-REMOVE; Package %u was successfully removed.\n", app.packageid );

    if( 0 != app.tell_refcount ) {
      printf("%%APAST-I-REFCOUNT; The reference count is now %u.\n", refcount );
    }

  } while( 0 );

  if( APP_E_USAGE == err ) {
    usage_message_emit(argv[0]);
  }

  return( err & 0xFF );
}

static Err cli_parse( App *app, int argc, char **argv ) {
  Err err = APP_E_NOERR;

  do {
    if( argc < 2 ) {
      err = APP_E_USAGE;
      break;
    }

    app->packageid = atoi( argv[1] );

    if( argc > 2 ) {
      if( !strncmp( "-t", argv[2], 2 ) ) {
	app->tell_refcount = 1;
      } else {
	err = APP_E_USAGE;
	break;
      }
    } else {
      app->tell_refcount = 0;
    }

  } while( 0 );

  return( err );
}

static void usage_message_emit( char *command ) {
  printf("%%APAST-E-USAGE; Usage is:\n; %s <packageid> [-t]\n; <packageid> - the id of the package to remove\n; -t (optional) if specified, it will print the post-removal refcount.\n", command );
}

