/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/

/* Macro Definitions */

#define ALP_PRV_SPF_BREAK_IF_NULL( object, type, err ) if( object == (type *) NULL ) { err = ALP_STATUS_SPF_INVALID_PARAM; break; }

#define ALP_PRV_SL_PACKAGENAME_DEFAULT "default"

/* File Includes */

#include <hiker/security.h>
#include <hiker/prv/apas.h>
#include <dbus/dbus-glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

/* Static Function Declarations */

static alp_status_t alp_prv_spf_dbus_singleton();

/* Global Variables */

static DBusGConnection *bus = (DBusGConnection *) NULL;
static DBusGProxy *remote_object = (DBusGProxy *) NULL;
static DBusGProxy *remote_object_introspectable = (DBusGProxy *)NULL;

static char *default_policy = "default";

static unsigned int global_subject_pid = 0;

static char *error_message = "replace me with error->message, please.";

alp_status_t alp_prv_spf_dbus_singleton() {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR; 
  GError *error = (GError *) NULL;

  do {
    /* Not an error... leave rv set to ALP_STATUS_SPF_E_NOERR */
    if( bus != (DBusGConnection *)NULL ) {
      break;
    }

    g_type_init();

    bus = dbus_g_bus_get( DBUS_BUS_SYSTEM, &error );
    if( !bus ) {
      printf("couldn't connect to session bus (%s)\n", error_message );
      rv = ALP_STATUS_SPF_NO_SERVER;
      break;
    }

    remote_object = dbus_g_proxy_new_for_name( bus,
					       "com.palmsource.security.server.interface",
					       "/SecurityObject",
					       "com.palmsource.security.server.interface" );

  } while ( 0 );

  return( rv );
}

alp_status_t alp_spf_update_open( ) {
  return( ALP_STATUS_SPF_E_NOERR );
}

alp_status_t alp_spf_update( char *message, uint message_len ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  GError *error = (GError *) NULL;

  do {
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, "update", &error,
				     G_TYPE_STRING, message, G_TYPE_INVALID, 
				     G_TYPE_INVALID ) ) {
      printf( "failed on update call (%s)\n", error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }
  } while( 0 );

  return( rv );

}

alp_status_t alp_spf_update_close( ) {
  return( ALP_STATUS_SPF_E_NOERR );
}

//-----------------------------------------------------------------------------
// POLICY STATUS APIs
//-----------------------------------------------------------------------------

alp_status_t alp_spf_ps_open( ) {
  return( ALP_STATUS_SPF_E_NOERR );
}

alp_status_t alp_spf_ps_get_policyname_from_policy( unsigned int policy, char **policyname ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  GError *error = (GError *) NULL;
  char **parentname = (char **)NULL;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( policyname, char *, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, "ps_get_policyname_from_policy", &error,
			     G_TYPE_UINT, policy, G_TYPE_INVALID, 
			     G_TYPE_STRING, policyname,
			     G_TYPE_STRING, parentname, G_TYPE_INVALID ) ) {
      printf( "failed on ps_get_policyname_from_policy call (%s)\n", error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_get_policy_from_policyname( char *policyname, unsigned int * policy ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  GError *error = (GError *) NULL;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( policyname, char, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, "ps_get_policy_from_policyname", &error,
			     G_TYPE_STRING, policyname, G_TYPE_INVALID, 
			     G_TYPE_UINT, policy, G_TYPE_INVALID ) ) {
      printf( "failed on ps_get_policy_from_policyname call (%s)\n", error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }
  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_packageid_increment( unsigned int packageid, unsigned int *out_count ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  unsigned int count_receiver;
  char *call_name = "ps_packageid_increment";
  GError *error = (GError *) NULL;


  do {
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, packageid, G_TYPE_INVALID,
			     G_TYPE_UINT, &count_receiver, G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( out_count != (unsigned int *) NULL ) {
      *out_count = count_receiver;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_packageid_decrement( unsigned int packageid, unsigned int *out_count ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  unsigned int count_receiver;
  char *call_name = "ps_packageid_decrement";
  GError *error = (GError *) NULL;


  do {
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, packageid, G_TYPE_INVALID,
			     G_TYPE_UINT, &count_receiver, G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( out_count != (unsigned int *) NULL ) {
      *out_count = count_receiver;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_packageid_count( unsigned int packageid, unsigned int *out_count ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  unsigned int count_receiver;
  char *call_name = "ps_packageid_count";
  GError *error = (GError *) NULL;

  do {
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, packageid, G_TYPE_INVALID,
			     G_TYPE_UINT, &count_receiver, G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( out_count != (unsigned int *) NULL ) {
      *out_count = count_receiver;
    }
  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_get_policy_by_certificate( char *certificate, unsigned int certificate_length, unsigned int *policy) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  GError *error = (GError *) NULL;

  do {
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ( certificate == NULL ) || ( policy == (unsigned int *)NULL ) ) {
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, "ps_get_policy_from_certificate",
			     &error, G_TYPE_STRING, certificate,
			     G_TYPE_INVALID, G_TYPE_UINT, policy,
			     G_TYPE_INVALID ) ) {
      printf( "failed on ps_get_policy_by_certificate() call (%s)\n",
	      error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }
  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_get_policy_by_gid( gid_t gid, unsigned int *policy) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "ps_get_policy_from_gid";
  GError *error = (GError *) NULL;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( policy, unsigned int, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }
    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, (unsigned int) gid, G_TYPE_INVALID,
			     G_TYPE_UINT, policy, G_TYPE_INVALID ) ) {
      printf( "failed on %s() call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( *policy == 0 ) {
      rv = ALP_STATUS_SPF_INVALID_PARAM;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_get_certificate_by_policy( unsigned int policy, char **certificate, unsigned int *certificate_length, char **certid, unsigned int *certid_length) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "ps_get_certificate_from_policy";
  GError *error = (GError *) NULL;
  char **internal_certid = NULL;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( certificate, char *, rv);
    ALP_PRV_SPF_BREAK_IF_NULL( certificate_length, unsigned int, rv);

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }
    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, policy, G_TYPE_INVALID,
			     G_TYPE_STRING, certificate,
			     G_TYPE_STRING, internal_certid, G_TYPE_INVALID ) ) {
      printf( "failed on %s() call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( certid != (char **)NULL ) {
      *certid = *internal_certid;
    }

    if( certid_length != (unsigned int *) NULL ) {
      *certid_length = strlen( *certid );
    }
  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_get_gid_by_policy( unsigned int policy, gid_t *gid ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "ps_get_gid_from_policy";
  GError *error = (GError *) NULL;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( gid, unsigned int, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }
    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, policy, G_TYPE_INVALID,
			     G_TYPE_UINT, gid, G_TYPE_INVALID ) ) {
      printf( "failed on %s() call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( *gid == 0 ) {
      rv = ALP_STATUS_SPF_INVALID_PARAM;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_get_policy_by_uid( uid_t uid, unsigned int *policy ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "ps_get_policy_from_uid";
  GError *error = (GError *) NULL;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( policy, unsigned int, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }
    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, (unsigned int) uid, G_TYPE_INVALID,
			     G_TYPE_UINT, policy, G_TYPE_INVALID ) ) {
      printf( "failed on %s() call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( *policy == 0 ) {
      rv = ALP_STATUS_SPF_INVALID_PARAM;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_get_policy_by_package( char *package, unsigned int package_length, unsigned int *out_policy) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "ps_get_policy_from_package";
  GError *error = (GError *) NULL;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( package, char, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( out_policy, unsigned int, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_STRING, package, G_TYPE_INVALID,
			     G_TYPE_UINT, out_policy, G_TYPE_INVALID ) ) {
      printf( "failed on %s() call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

  } while( 0 );

  return( rv );
}

/* todo : figure out if this guy is deprecated or what */
alp_status_t alp_spf_ps_get_gids_length( unsigned int policy, unsigned int *size ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( size, unsigned int, rv );

    *size = 0;

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_get_uid_gid_gids_by_policy( unsigned int packageid, uid_t *uid, gid_t *gid, gid_t *sgids[], unsigned int *sgids_length ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "ps_get_uid_gid_sgids";
  GError *error = (GError *) NULL;
  GArray *internal_sgids;
  
  do {
    ALP_PRV_SPF_BREAK_IF_NULL( uid, uid_t, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( gid, gid_t, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( sgids, gid_t *, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( sgids_length, unsigned int, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, packageid,
			     G_TYPE_INVALID,
			     G_TYPE_UINT, uid,
			     G_TYPE_UINT, gid,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s() call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    /* todo : fix this */
    /* *sgids_length = internal_sgids->len; */
    *sgids_length = 0;
    /* *sgids = (gid_t *) internal_sgids->data; */
    *sgids = (gid_t *) NULL;

  } while( 0 );


  return( rv );  
}

alp_status_t alp_prv_spf_ps_enroll_package( char *package, unsigned int package_length, unsigned int *out_packageid ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "ps_enroll_package";
  GError *error = (GError *) NULL;
  char *security_manifest;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( package, char, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( out_packageid, unsigned int, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_STRING, package,
			     G_TYPE_INVALID,	     
			     G_TYPE_UINT, out_packageid,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s() call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_ps_close( ) {
  return( ALP_STATUS_SPF_E_NOERR );
}


//-----------------------------------------------------------------------------
// QUERY APIs
//-----------------------------------------------------------------------------

alp_status_t alp_spf_query_open( ) {
  return( ALP_STATUS_SPF_E_NOERR );
}

alp_status_t alp_spf_query_set_subject_pid( pid_t subject_pid ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char path[80];
  struct stat buf;

  do {
    memset( path, '\0', 80 );
    sprintf( path, "/proc/%d", (int) subject_pid );
    if( 0 != stat( path, &buf ) ) {
      /* Error stat'ing file */
      fprintf( stderr, "%%SPFSL-F-NOSUCHPID; PID %d appears not to exist.\n", (int) subject_pid );
      fflush( stderr );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }
    global_subject_pid = (unsigned int) subject_pid;
  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_query_reset_subject_pid( ) {
  global_subject_pid = 0;
  return( ALP_STATUS_SPF_E_NOERR );
}
  

alp_status_t alp_spf_query_get_assertion_string( char *assertion, uint assertion_len, char **out_assertion_value) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  uint policyid = 0;
  uint client_pid = 0;
  GError *error = (GError *) NULL;
  unsigned int exit_okay = 0;
  unsigned int wait_channel;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( assertion, char , rv );
    ALP_PRV_SPF_BREAK_IF_NULL( out_assertion_value, char *, rv );

    /* Call the singleton to setup the bus */
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    GError *error = (GError *) NULL;

    client_pid = (uint) getpid();

    if( ! dbus_g_proxy_call( remote_object, "query_get_assertion_string", &error,
			     G_TYPE_UINT, policyid,
			     G_TYPE_STRING, assertion,
			     G_TYPE_UINT, global_subject_pid,
			     G_TYPE_UINT, client_pid, G_TYPE_INVALID, 
			     G_TYPE_STRING, out_assertion_value, G_TYPE_INVALID ) ) {
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( ! strncmp( "ask:", *out_assertion_value, 4 ) ) {
      wait_channel = (unsigned int) atoi( & ((*out_assertion_value)[4]) );
      while( exit_okay == 0 ) {
	if( ! dbus_g_proxy_call( remote_object, "query_complete", &error,
				 G_TYPE_UINT, wait_channel,
				 G_TYPE_INVALID,
				 G_TYPE_STRING, out_assertion_value,
				 G_TYPE_UINT, & exit_okay,
				 G_TYPE_INVALID ) ) {
	  rv = ALP_STATUS_SPF_INVALID_PARAM;
	  break;
	}
	sleep(1);
      }
    }
    
  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_query_get_assertion_int( char *assertion, uint assertion_len, int *out_assertion_value ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *internal_assertion_value;

  do {
    if( ( rv = alp_spf_query_get_assertion_string( assertion, assertion_len, &internal_assertion_value ) ) != ALP_STATUS_SPF_E_NOERR ) {
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    *out_assertion_value = atoi( internal_assertion_value );
  } while( 0 );

  return( rv );

}

alp_status_t alp_spf_query_get_assertion_boolean( char *assertion, uint assertion_len, uint *out_assertion_value ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *value;

  do {
    if( ( rv = alp_spf_query_get_assertion_string( assertion, assertion_len, &value ) ) != ALP_STATUS_SPF_E_NOERR ) {
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if ((strcasecmp(value, "true")) == 0) {
	*out_assertion_value = (uint) 1;
    }
    else {
	*out_assertion_value = (uint) 0;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_query_assertion_type( char *assertion, uint assertion_len, AlpSPFAssertionType *out_type ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  uint policyid = 0;
  uint client_pid = 0;
  GError *error = (GError *) NULL;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( assertion, char , rv );
    ALP_PRV_SPF_BREAK_IF_NULL( out_type, AlpSPFAssertionType, rv );

    /* Call the singleton to setup the bus */
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    GError *error = (GError *) NULL;

    client_pid = (uint) getpid();

    if( ! dbus_g_proxy_call( remote_object, "query_get_assertion_type", &error,
			     G_TYPE_UINT, policyid,
			     G_TYPE_STRING, assertion,
			     G_TYPE_UINT, global_subject_pid,
			     G_TYPE_UINT, client_pid, G_TYPE_INVALID, 
			     G_TYPE_UINT, out_type, G_TYPE_INVALID ) ) {
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }
    
  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_query_get_assertion_from_policy( char *policy, char *assertion, uint assertion_len, char **out_assertion_value ) {
  return( alp_spf_query_get_assertion_string_from_policy( policy, assertion, assertion_len, out_assertion_value ) );
}

alp_status_t alp_spf_query_get_assertion_string_from_policy( char *policyname, char *assertion, uint assertion_len, char **out_assertion_value ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;

  GError *error = (GError *) NULL;
  uint policyid;
  uint client_pid = 0;

  unsigned int exit_okay = 0;
  unsigned int wait_channel;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( policyname, char, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( assertion, char , rv );
    ALP_PRV_SPF_BREAK_IF_NULL( out_assertion_value, char *, rv );

    /* Call the singleton to setup the bus */
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    /* Call alp_spf_ps_get_policy_from_policyname() to get the right policy */
    if( ( rv = alp_spf_ps_get_policy_from_policyname( policyname, &policyid ) ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    GError *error = (GError *) NULL;

    client_pid = (uint) getpid();

    if( ! dbus_g_proxy_call( remote_object, "query_get_assertion_string", &error,
			     G_TYPE_UINT, policyid,
			     G_TYPE_STRING, assertion,
			     G_TYPE_UINT, global_subject_pid, 
			     G_TYPE_UINT, client_pid, G_TYPE_INVALID, 
			     G_TYPE_STRING, out_assertion_value, G_TYPE_INVALID ) ) {
      printf( "1: failed on query_get_assertion_string call (%s)\n", error->message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( ! strncmp( "ask:", *out_assertion_value, 4 ) ) {
      wait_channel = (unsigned int) atoi( & ((*out_assertion_value)[4]) );
      while( exit_okay == 0 ) {
	if( ! dbus_g_proxy_call( remote_object, "query_complete", &error,
				 G_TYPE_UINT, wait_channel,
				 G_TYPE_INVALID,
				 G_TYPE_STRING, out_assertion_value,
				 G_TYPE_UINT, & exit_okay,
				 G_TYPE_INVALID ) ) {
	  rv = ALP_STATUS_SPF_INVALID_PARAM;
	  printf( "There was an error %s\n", error->message );
	  break;
	}
	sleep(1);
      }
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_query_get_assertion_int_from_policy( char *policy, char *assertion, uint assertion_len, int *out_assertion_value ) {
  char *value;
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;

  do {
    if( out_assertion_value == (int *) NULL ) {
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( ( rv = alp_spf_query_get_assertion_string_from_policy( policy, assertion, assertion_len, &value ) ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    *out_assertion_value = atoi( value );
  } while ( 0 );

  return( rv );
}

alp_status_t alp_spf_query_get_assertion_boolean_from_policy( char *policy, char *assertion, uint assertion_len, uint *out_assertion_value ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *value;

  do {
    if( out_assertion_value == (uint *) NULL ) {
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    if( ( rv = alp_spf_query_get_assertion_string_from_policy( policy, assertion, assertion_len, &value ) ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if ((strcasecmp(value, "true")) == 0) {
        *out_assertion_value = (uint) 1;
    }
    else {
        *out_assertion_value = (uint) 0;
    }

  } while ( 0 );

  return( rv );
}

alp_status_t alp_spf_query_assertion_type_from_policy( char *policyname, char *assertion, uint assertion_len, AlpSPFAssertionType *out_type ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  unsigned int policy;
  GError *error = (GError *) NULL;
  unsigned int client_pid = 0;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( policyname, char, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( assertion, char, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( out_type, AlpSPFAssertionType, rv );
    
    if( ( rv = alp_spf_ps_get_policy_from_policyname( policyname, &policy ) ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    /* Get Policy ID from Policy Name */

    if( ! dbus_g_proxy_call( remote_object, "query_get_assertion_type", &error,
			     G_TYPE_UINT, policy, 
			     G_TYPE_STRING, assertion, 
			     G_TYPE_UINT, global_subject_pid,
			     G_TYPE_UINT, client_pid,G_TYPE_INVALID,
			     G_TYPE_UINT, out_type, G_TYPE_INVALID ) ) {
      printf( "2: failed on query_get_assertion_type call (%s)\n", error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

  } while( 0 );

  return( rv );  
}

alp_status_t alp_spf_query_close( ) {
  return( ALP_STATUS_SPF_E_NOERR );
}

/* todo : finish implementation */
alp_status_t alp_spf_info( AlpSPFIntroInfo **out_info ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  GError *error = (GError *) NULL;
  uint * version = NULL;
  GHashTable **api_table = NULL;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( out_info, AlpSPFIntroInfo *, rv );
    
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, "info", &error,G_TYPE_INVALID,
			     G_TYPE_UINT, version,
			     DBUS_TYPE_G_STRING_STRING_HASHTABLE, api_table,
			     G_TYPE_INVALID ) ) {
      printf( "failed on info call (%s)\n", error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

    (*out_info)->version = (unsigned int) *version;

  } while( 0 );

  return( rv );  
}

alp_status_t alp_spf_pkcs5_set( unsigned int which, char *phrase ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "pkcs5_set_phrase";
  GError *error = (GError *) NULL;


  do {
    ALP_PRV_SPF_BREAK_IF_NULL( phrase, char, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, which,
			     G_TYPE_STRING, phrase,
			     G_TYPE_INVALID,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_pkcs5_check( unsigned int which, char *phrase, unsigned int *correct ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "pkcs5_check_phrase";
  GError *error = (GError *) NULL;


  do {
    ALP_PRV_SPF_BREAK_IF_NULL( phrase, char, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( correct, unsigned int, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, which,
			     G_TYPE_STRING, phrase,
			     G_TYPE_INVALID,
			     G_TYPE_UINT, correct,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_pkcs5_params_get( unsigned int which, unsigned int *iteration, char **salt, unsigned int *active ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "pkcs5_get_parameters";
  GError *error = (GError *) NULL;


  do {
    ALP_PRV_SPF_BREAK_IF_NULL( iteration, unsigned int, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( salt, char *, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( active, unsigned int, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, which,
			     G_TYPE_INVALID,
			     G_TYPE_UINT, iteration,
			     G_TYPE_UINT, active,
			     G_TYPE_STRING, salt,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_pkcs5_params_set( unsigned int which, unsigned int iteration, char *salt, unsigned int active ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "pkcs5_set_parameters";
  GError *error = (GError *) NULL;


  do {
    ALP_PRV_SPF_BREAK_IF_NULL( salt, char, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, which,
			     G_TYPE_UINT, iteration,
			     G_TYPE_UINT, active,
			     G_TYPE_STRING, salt,
			     G_TYPE_INVALID,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error_message );
      rv = ALP_STATUS_SPF_INVALID_PARAM;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_apas_enroll( char *path, unsigned int *out_packageid ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "apas_enroll";
  GError *error = (GError *) NULL;


  do {
    ALP_PRV_SPF_BREAK_IF_NULL( path, char, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( out_packageid, unsigned int, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_STRING, path,
			     G_TYPE_INVALID,
			     G_TYPE_UINT, out_packageid,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error->message );
      rv = (alp_status_t) error->code;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_apas_path_get_packageid( char *path, unsigned int *out_packageid ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "apas_path_get_packageid";
  GError *error = (GError *) NULL;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( path, char, rv );
    ALP_PRV_SPF_BREAK_IF_NULL( out_packageid, unsigned int, rv );

    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_STRING, path,
			     G_TYPE_INVALID,
			     G_TYPE_UINT, out_packageid,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error->message );
      rv = (alp_status_t) error->code;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_apas_packageid_remove( unsigned int packageid, unsigned int *out_refcount ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "apas_packageid_remove";
  GError *error = (GError *) NULL;


  do {
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, packageid,
			     G_TYPE_INVALID,
			     G_TYPE_UINT, out_refcount,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error->message );
      rv = (alp_status_t) error->code;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_apas_packageid_uid_get( unsigned int packageid, uid_t *out_uid) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "apas_packageid_uid_get";
  GError *error = (GError *) NULL;

  do {
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, packageid,
			     G_TYPE_INVALID,
			     G_TYPE_UINT, out_uid,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error->message );
      rv = (alp_status_t) error->code;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_apas_packageid_gid_get( unsigned int packageid, gid_t *out_gid ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "apas_packageid_gid_get";
  GError *error = (GError *) NULL;


  do {
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, packageid,
			     G_TYPE_INVALID,
			     G_TYPE_UINT, out_gid,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error->message );
      rv = (alp_status_t) error->code;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_apas_packageid_sgids_get( unsigned int packageid, gid_t *out_sgids[], unsigned int *out_sgids_length ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "apas_packageid_sgids_get";
  char *sgids_string = NULL;
  GError *error = (GError *) NULL;
  static gid_t default_sgids[] = { (gid_t) 0 };

  do {
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, packageid,
			     G_TYPE_INVALID,
			     G_TYPE_STRING, &sgids_string,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error->message );
      rv = (alp_status_t) error->code;
      break;
    }

    // Now convert this to an array of gid_t's instead of a comma separated
    // string.

    if( ( NULL == sgids_string ) || ( strlen( sgids_string ) == 0 ) ) {
      *out_sgids = default_sgids;
      *out_sgids_length = 0;
    } else {
      int sgids_string_length;
      int i;
      unsigned int sgids_length = 1;
      gid_t *gid_array;
      sgids_string_length = strlen( sgids_string );
      for( i=0; i<sgids_string_length; i++ ) {
	if( ',' == sgids_string[ i ] ) {
	  sgids_length ++;
	}
      }
      *out_sgids_length = sgids_length;
      gid_array = malloc( ( sgids_length + 1 ) * sizeof( gid_t ) );
      char *current = sgids_string;
      char *next = sgids_string;
      int array_element = 0;
      for( i=0; i < sgids_string_length; i++ ) {
	if( ',' == sgids_string[ i ] ) {
	  next = & ( sgids_string[ i + 1 ] );
	  sgids_string[ i ] = '\0';
	  gid_array[ array_element++ ] = atoi( current );
	  current = next;
	}
      }
      gid_array[ array_element ] = atoi( next );
      gid_array[ sgids_length ] = (gid_t) 0;
      *out_sgids = gid_array;
    }

  } while( 0 );

  return( rv );
}


alp_status_t alp_spf_apas_packageid_refcount_get( unsigned int packageid, unsigned int *out_refcount ) {
  return( alp_spf_ps_packageid_count( packageid, out_refcount) );
}

alp_status_t alp_spf_apas_packageid_refcount_set( unsigned int packageid, unsigned int refcount ) {
  alp_status_t rv = ALP_STATUS_SPF_E_NOERR;
  char *call_name = "apas_packageid_refcount_set";
  GError *error = (GError *) NULL;


  do {
    if( ( rv = alp_prv_spf_dbus_singleton() ) != ALP_STATUS_SPF_E_NOERR ) {
      break;
    }

    if( ! dbus_g_proxy_call( remote_object, call_name, &error,
			     G_TYPE_UINT, packageid,
			     G_TYPE_UINT, refcount,
			     G_TYPE_INVALID,
			     G_TYPE_INVALID ) ) {
      printf( "failed on %s call (%s)\n", call_name, error->message );
      rv = (alp_status_t) error->code;
      break;
    }

  } while( 0 );

  return( rv );
}

alp_status_t alp_spf_apas_packageid_refcount_increment( unsigned int packageid ) {
  alp_status_t err = ALP_STATUS_SPF_APAS_E_NOERR;
  unsigned int out_count;

  do{
    err = alp_spf_ps_packageid_increment( packageid, &out_count );
  } while( 0 );

  return( err );
}

alp_status_t alp_spf_apas_packageid_refcount_decrement( unsigned int packageid ) {
  alp_status_t err = ALP_STATUS_SPF_APAS_E_NOERR;
  unsigned int out_count;

  do{
    err = alp_spf_ps_packageid_decrement( packageid, &out_count );
  } while( 0 );

  return( err );

}

alp_status_t alp_spf_apas_packageid_default( unsigned int *out_packageid ) {
  alp_status_t err = ALP_STATUS_SPF_APAS_E_NOERR;

  do {
    ALP_PRV_SPF_BREAK_IF_NULL( out_packageid, unsigned int, err );

    if( ALP_STATUS_SPF_APAS_E_NOERR != ( err = alp_spf_apas_path_get_packageid( ALP_PRV_SL_PACKAGENAME_DEFAULT, out_packageid ) ) ) {
      break;
    }

  } while( 0 );

  return( err );
}

