/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/* SecurityTest.cpp
**
** $Id$
**
** This file contains the CPPUNIT tests for the SPF Client Library. We 
** currently have 44 tests in this file, each at a slightly different
** level of sophistication ranging from "unimplemented" to "mostly
** "mostly implemented" to "complete." We have both "positive" test cases
** which make sure we don't get an error when we do something that should
** work and "negative" test cases that verify that we get errors when we
** do something bad.
**
** P _ testUpdate
** N _ testUpdateBadCertificate
** N _ testUpdateInvalidCertificate
** N _ testUpdateBadSignature
** N _ testUpdateInvalidXML
** P X testPSPolicynameToID
** N _ testPSInvalidPolicyName
** N _ testPSNullPolicyName
** P X testPSPolicyIDToPolicyname
** N _ testPSInvalidPolicyID
** P X testPSPackageIDIncrementDecrement
** N _ testPSDecrementInvalidPolicyID
** P _ testPSGetPolicyByCertificate
** N _ testPSGetPolicyByBadCertificate
** N _ testPSGetPolicyByInvalidCertificate
** P X testPSGetPolicyByGID
** N _ testPSGetPolicyByInvalidGID
** P X testPSGetPolicyByUID
** N _ testPSGetPolicyByInvalidUID
** P X testPSGetPolicyByPackagename
** N _ testPSGetPolicyByInvalidPackagename
** P _ testPSGetCertificateByPolicy
** N _ testPSGetCertificateByInvalidPolicy
** P X testPSGetGIDByPolicy
** N _ testPSGetGIDByInvalidPolicy
** P X testPSGetGIDSLength
** P X testPSGetUIDGIDGIDSByPolicy
** N _ testPSGetUIDGIDGIDSByInvalidPolicy
** P X testPSEnrollPackage
** N _ testPSEnrollPackageBadCert
** N _ testPSEnrollPackageInvalidCert
** N _ testPSEnrollPackageBadSignature
** N _ testPSEnrollPackageBadFileHash
** N _ testPSEnrollPackageExtraFile
** N _ testPSEnrollPackageInvalidXML
** P _ testPSRemovePackage
** N _ testPSRemoveInvalidPackage
** P X testQueryDefaultPolicy
** P X testQueryPSIPolicy
** P X testQueryKoniaPolicy
** P X testQueryMTobilePolicy
** P X testQueryMobSoftRUsPolicy
** P X testQueryTypes
** P X testQueryInherit
** P X testQueryAsDefault
** P X testQueryAsPSI
** P X testQueryAsKonia
** P X testQueryAsMTobile
** P X testQueryAsMobSoftRUs
** _ _ testInfo
** P _ testPKCS5PIN
** P _ testPKCS5PUK
** P _ testEnforcementSublaunch
*/

/* Macro Definitions */
#define ALP_TRACE ALP_TRACE_ON

/* File Inclusions */
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/wait.h>

#include <cppunit/ui/text/TestRunner.h>
#include <cppunit/extensions/HelperMacros.h>

#include "hiker/security.h" 

#include "hiker/config_defines.h"
#include "hiker/security.h" 
#include "hiker/sysclass.h"
#include "hiker/traces.h"
#include "hiker/types.h"
#include "hiker/unit_test.h"
#include "hiker/bundlemgr.h"
#include "hiker/prv/bundlemgr_prv.h"
#include "hiker/appmgr.h"
#include "hiker/pdk/appmgr_pdk.h"

/* Typedefs, Structs, Unions, Enums, Etc. */

/* Static Function Declarations */

/* Global Variable Declarations */

/* Function Definitions */

class SPFTest : public CPPUNIT_NS::TestFixture{
  /* Tests that may be successfully run as long as we're not root */
  CPPUNIT_TEST_SUITE( SPFTest );
  CPPUNIT_TEST( testUpdate );
//  CPPUNIT_TEST( testUpdateBadCertificate );
//  CPPUNIT_TEST( testUpdateInvalidCertificate );
//  CPPUNIT_TEST( testUpdateBadSignature );
//  CPPUNIT_TEST( testUpdateInvalidXML );
  CPPUNIT_TEST( testPSPolicynameToID );
//  CPPUNIT_TEST( testPSInvalidPolicyName );
//  CPPUNIT_TEST( testPSNullPolicyName );
  CPPUNIT_TEST( testPSPolicyIDToPolicyname );
  CPPUNIT_TEST( testPSInvalidPolicyID );
  CPPUNIT_TEST( testPSPackageIDIncrementDecrement );
//  CPPUNIT_TEST( testPSDecrementInvalidPolicyID );
//  CPPUNIT_TEST( testPSGetPolicyByCertificate );
//  CPPUNIT_TEST( testPSGetPolicyByBadCertificate );
//  CPPUNIT_TEST( testPSGetPolicyByInvalidCertificate );
  CPPUNIT_TEST( testPSGetPolicyByGID );
//  CPPUNIT_TEST( testPSGetPolicyByInvalidGID );
  CPPUNIT_TEST( testPSGetPolicyByUID );
//  CPPUNIT_TEST( testPSGetPolicyByInvalidUID );
  CPPUNIT_TEST( testPSGetPolicyByPackagename );
//  CPPUNIT_TEST( testPSGetPolicyByInvalidPackagename );
  CPPUNIT_TEST( testPSGetCertificateByPolicy );
//  CPPUNIT_TEST( testPSGetCertificateByInvalidPolicy );
  CPPUNIT_TEST( testPSGetGIDByPolicy );
//  CPPUNIT_TEST( testPSGetGIDByInvalidPolicy );
  CPPUNIT_TEST( testPSGetGIDSLength );
  CPPUNIT_TEST( testPSGetUIDGIDGIDSByPolicy );
//  CPPUNIT_TEST( testPSGetUIDGIDGIDSByInvalidPolicy );
  CPPUNIT_TEST( testPSEnrollPackage );
//  CPPUNIT_TEST( testPSEnrollPackageBadCert );
//  CPPUNIT_TEST( testPSEnrollPackageInvalidCert );
//  CPPUNIT_TEST( testPSEnrollPackageBadSignature );
//  CPPUNIT_TEST( testPSEnrollPackageBadFileHash );
//  CPPUNIT_TEST( testPSEnrollPackageExtraFile );
//  CPPUNIT_TEST( testPSEnrollPackageInvalidXML );
  CPPUNIT_TEST( testPSRemovePackage );
//  CPPUNIT_TEST( testPSRemoveInvalidPackage );
  CPPUNIT_TEST( testQueryDefaultPolicy );
  CPPUNIT_TEST( testQueryPSIPolicy );
//  CPPUNIT_TEST( testQueryKoniaPolicy );
//  CPPUNIT_TEST( testQueryMTobilePolicy );
//  CPPUNIT_TEST( testQueryMobSoftRUsPolicy );
  CPPUNIT_TEST( testQueryTypes );
//  CPPUNIT_TEST( testQueryInherit );
  CPPUNIT_NOFORK_TEST( testEnforcementSublaunch );
    CPPUNIT_TEST( testPKCS5PIN );
  //  CPPUNIT_TEST( testPKCS5PUK );
//  CPPUNIT_TEST( testSPF );
	      
    /*     fputs( "%SPFT-S-NONROOT; Added Non Root Test Suite\n", errout ); */
  CPPUNIT_TEST( testInfo );

      /*    if( getuid() == 0 ) { */
  if( 1 == 0 ) {
	/* 	fputs( "%SPFT-S-ROOT; Added Root Test Suite\n", errout ); */
      
    CPPUNIT_TEST( testQueryAsDefault );
    CPPUNIT_TEST( testQueryAsPSI );
//    CPPUNIT_TEST( testQueryAsKonia );
//    CPPUNIT_TEST( testQueryAsMTobile );
//    CPPUNIT_TEST( testQueryAsMobSoftRUs );
    
  } else {
    /* 	fputs( "%SPFT-E-ROOT; Did Not Add Root Test Suite. You're not running as root.\n", errout ); */
  }

  CPPUNIT_TEST_SUITE_END();

public:
  /* Positive Test */
  void testUpdate() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;

/****************************************************************************
 *                                                                          *
 * Update Policy                                                            *
 *                                                                          *
 ****************************************************************************/
    char *update = NULL; 
    char *update_policy_file = "/usr/local/alp/tests/security/update.policy";
    
    int update_length;

    FILE *fp = NULL;

    struct stat buf;
    int filesize;
    int bytes_read;

    if (stat(update_policy_file, & buf ) != 0 ) {
      err =  ALP_STATUS_SPF_INVALID_PARAM;
      goto finalize; 
    }

    update_length = (unsigned int) buf.st_size;
    if ((update = (char *)malloc(update_length + 1)) == NULL) {
      err = ALP_STATUS_SPF_MEM_ERROR;
      goto finalize; 
    }

    if( ( fp = fopen( update_policy_file, "r" ) ) == (FILE *) NULL ) {
      err =  ALP_STATUS_SPF_INVALID_PARAM;
      goto finalize; 
    }

    if ((bytes_read = fread(update, (size_t)1, (size_t)update_length, fp)) != 
        update_length) {
      err = ALP_STATUS_SPF_INVALID_PARAM;
      goto finalize; 
    }

    fclose( fp );
    fp = NULL;

    update[update_length] = '\0';


    /*     fputs( "%SPFT-I-UPDATE; Starting Update Test.\n", errout ); */

    do {
      err = alp_spf_update_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      update_length = (unsigned int) strlen( update );

      err = alp_spf_update( update, update_length );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_update_close();

    } while( 0 );

finalize:
    if (fp != NULL) {
	fclose(fp);
    }

    if (update != NULL) {
	free(update);
    }

    printf("\n\n%x\n\n",err);
    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  void testUpdateBadCertificate() {
  }

  void testUpdateInvalidCertificate() {
  }

  void testUpdateBadSignature() {
  }

  void testUpdateInvalidXML() {
  }

  /* Positive Test */
  void testPSPolicynameToID() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *policyname;

    /*     fputs( "%SPFT-I-PN2PID; Starting Policy Name to Policy ID Test.\n", errout ); */

    do {
      err = alp_spf_ps_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_ps_get_policyname_from_policy( 1, &policyname );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( ! strcmp( "default", policyname ) );

      err = alp_spf_ps_get_policyname_from_policy( 2, &policyname );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( ! strcmp( "psi", policyname ) );

//      err = alp_spf_ps_get_policyname_from_policy( 3, &policyname );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( ! strcmp( "konia", policyname ) );

//      err = alp_spf_ps_get_policyname_from_policy( 4, &policyname );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( ! strcmp( "m-tobile", policyname ) );

//      err = alp_spf_ps_get_policyname_from_policy( 5, &policyname );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( ! strcmp( "mobsoftrus", policyname ) );

      err = alp_spf_ps_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  void testPSInvalidPolicyName() {
  }

  void testPSNullPolicyName() {
  }

  /* Positive Test */
  void testPSPolicyIDToPolicyname() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    unsigned int policyid;

    /*     fputs( "%SPFT-I-PID2PN; Starting Policy Id to Policy Name Test.\n", errout ); */

    do {
      err = alp_spf_ps_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_ps_get_policy_from_policyname( "default", &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( policyid == 1 );

      err = alp_spf_ps_get_policy_from_policyname( "psi", &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( policyid == 2 );

//      err = alp_spf_ps_get_policy_from_policyname( "konia", &policyid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( policyid == 3 );

//      err = alp_spf_ps_get_policy_from_policyname( "m-tobile", &policyid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( policyid == 4 );

//      err = alp_spf_ps_get_policy_from_policyname( "mobsoftrus", &policyid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( policyid == 5 );

      err = alp_spf_ps_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  void testPSInvalidPolicyID() {
  }

  /* Positive Test */
  void testPSPackageIDIncrementDecrement() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    unsigned int count;

    /*     fputs( "%SPFT-I-PSIDC; Starting PS Increment, Decrement, Count Test.\n", errout ); */

    do {
      err = alp_spf_ps_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_ps_packageid_count( 1, &count );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( count == 1 );

      err = alp_spf_ps_packageid_increment( 1, &count );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( count == 1 );

      err = alp_spf_ps_packageid_count( 1, &count );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( count == 2 );

      err = alp_spf_ps_packageid_decrement( 1, &count );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( count == 2 );

      err = alp_spf_ps_packageid_count( 1, &count );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( count == 1 );

      err = alp_spf_ps_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  void testPSDecrementInvalidPolicyID() {
  }

  void testPSGetPolicyByCertificate() {
  }

  void testPSGetPolicyByBadCertificate() {
  }

  void testPSGetPolicyByInvalidCertificate() {
  }

  /* Positive Test */
  void testPSGetPolicyByGID() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    unsigned int policyid;

    /*     fputs( "%SPFT-I-PSPBGID; Starting PS Get Policy by GID.\n", errout ); */

    do {
      err = alp_spf_ps_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_ps_get_policy_by_gid( 1001, &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( policyid == 1 );

      err = alp_spf_ps_get_policy_by_gid( 1002, &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( policyid == 2 );

//      err = alp_spf_ps_get_policy_by_gid( 1003, &policyid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( policyid == 3 );

//      err = alp_spf_ps_get_policy_by_gid( 1004, &policyid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( policyid == 4 );

//      err = alp_spf_ps_get_policy_by_gid( 1005, &policyid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( policyid == 5 );

      err = alp_spf_ps_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  void testPSGetPolicyByInvalidGID() {
  }

  /* Positive Test */
  void testPSGetPolicyByUID() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    unsigned int policyid;

    /*     fputs( "%SPFT-I-PSPBUID; Starting PS Get Policy by UID.\n", errout ); */

    do {
      err = alp_spf_ps_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_ps_get_policy_by_uid( 1001, &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( policyid == 2 );

      err = alp_spf_ps_get_policy_by_uid( 1002, &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( policyid == 2 );

      err = alp_spf_ps_get_policy_by_uid( 1003, &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( policyid == 2 );

//      err = alp_spf_ps_get_policy_by_uid( 1004, &policyid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( policyid == 3 );

//      err = alp_spf_ps_get_policy_by_uid( 1005, &policyid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( policyid == 4 );

//      err = alp_spf_ps_get_policy_by_uid( 1006, &policyid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( policyid == 5 );

//      err = alp_spf_ps_get_policy_by_uid( 1007, &policyid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( policyid == 1 );

      err = alp_spf_ps_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  void testPSGetPolicyByInvalidUID() {
  }

  /* Positive Test */
  void testPSGetPolicyByPackagename() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    unsigned int policyid;

    /*     fputs( "%SPFT-I-PSPBPN; Starting PS Get Policy by Package Name(path).\n", errout ); */

    do {
      err = alp_spf_ps_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_ps_get_policy_by_package( "/usr/packages/PSIMail", strlen( "/usr/packages/PSIMail" ), &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      /* CPPUNIT_ASSERT( policyid == 2 ); */
      /* CPPUNIT_ASSERT( policyid == 1 ); */

      err = alp_spf_ps_get_policy_by_package( "/usr/packages/PSIPhone", strlen( "/usr/packages/PSIPhone" ), &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      /* CPPUNIT_ASSERT( policyid == 2 ); */
      /* CPPUNIT_ASSERT( policyid == 1 ); */

      err = alp_spf_ps_get_policy_by_package( "/usr/packages/PSIBrowser", strlen( "/usr/packages/PSIBrowser" ), &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      /* CPPUNIT_ASSERT( policyid == 2 ); */
      /* CPPUNIT_ASSERT( policyid == 1 ); */

      err = alp_spf_ps_get_policy_by_package( "/usr/packages/KoniaCamera", strlen( "/usr/packages/KoniaCamera" ), &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      /* CPPUNIT_ASSERT( policyid == 3 ); */
      /* CPPUNIT_ASSERT( policyid == 1 ); */

      err = alp_spf_ps_get_policy_by_package( "/usr/packages/MTobileDashboard", strlen( "/usr/packages/MTobileDashboard" ), &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      /* CPPUNIT_ASSERT( policyid == 4 ); */
      /* CPPUNIT_ASSERT( policyid == 1 ); */

      err = alp_spf_ps_get_policy_by_package( "/usr/packages/LunchMoney", strlen( "/usr/packages/LunchMoney" ), &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      /* CPPUNIT_ASSERT( policyid == 5 ); */
      /* CPPUNIT_ASSERT( policyid == 1 ); */

      err = alp_spf_ps_get_policy_by_package( "/usr/packages/DungeonExplorer", strlen( "/usr/packages/DungeonExplorer" ), &policyid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      /* CPPUNIT_ASSERT( policyid == 1 ); */
      /* CPPUNIT_ASSERT( policyid == 1 ); */

      err = alp_spf_ps_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  void testPSGetPolicyByInvalidPackagename() {
  }

  /* Positive Test */
  void testPSGetCertificateByPolicy() {
  }

  void testPSGetCertificateByInvalidPolicy() {
  }

  /* Positive Test */
  void testPSGetGIDByPolicy() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    gid_t gid;

    /*     fputs( "%SPFT-I-PSGIDBP; Starting PS Get GID by policy.\n", errout ); */

    do {
      err = alp_spf_ps_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_ps_get_gid_by_policy( 1, &gid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( gid == 1001 );

      err = alp_spf_ps_get_gid_by_policy( 2, &gid );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( gid == 1002 );

//      err = alp_spf_ps_get_gid_by_policy( 3, &gid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( gid == 1003 );

//      err = alp_spf_ps_get_gid_by_policy( 4, &gid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( gid == 1004 );

//      err = alp_spf_ps_get_gid_by_policy( 5, &gid );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( gid == 1005 );

      err = alp_spf_ps_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  void testPSGetGIDByInvalidPolicy() {
  }

  /* Positive Test */
  void testPSGetGIDSLength() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    unsigned int length;

    /*     fputs( "%SPFT-I-PSGIDSL; Starting PS GIDS Length Test.\n", errout ); */

    do {
      err = alp_spf_ps_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_ps_get_gids_length( 1, &length );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( length == 0 );

      err = alp_spf_ps_get_gids_length( 2, &length );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( length == 0 );

      err = alp_spf_ps_get_gids_length( 3, &length );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( length == 0 );

      err = alp_spf_ps_get_gids_length( 4, &length );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( length == 0 );

      err = alp_spf_ps_get_gids_length( 5, &length );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( length == 0 );

      err = alp_spf_ps_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  /* Positive Test */
  void testPSGetUIDGIDGIDSByPolicy() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    uid_t uid;
    gid_t gid;
    gid_t *gids;
    unsigned int length;

    /*     fputs( "%SPFT-I-PSGUGGBP; Starting PS Get UID, GID, GIDS by Policy Test.\n", errout );     */

    do {
      err = alp_spf_ps_open();

      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_ps_get_uid_gid_gids_by_policy( 1, &uid, &gid, &gids, &length );

      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( length == 0 );
      CPPUNIT_ASSERT( uid == 1001 );
      CPPUNIT_ASSERT( gid == 1002 );

      err = alp_spf_ps_get_uid_gid_gids_by_policy( 2, &uid, &gid, &gids, &length );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( length == 0 );
      CPPUNIT_ASSERT( uid == 1002 );
      CPPUNIT_ASSERT( gid == 1002 );

//      err = alp_spf_ps_get_uid_gid_gids_by_policy( 3, &uid, &gid, &gids, &length );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( length == 0 );
//      CPPUNIT_ASSERT( uid == 1003 );
//      CPPUNIT_ASSERT( gid == 1002 );

//      err = alp_spf_ps_get_uid_gid_gids_by_policy( 4, &uid, &gid, &gids, &length );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( length == 0 );
//      CPPUNIT_ASSERT( uid == 1004 );
//      CPPUNIT_ASSERT( gid == 1003 );

//      err = alp_spf_ps_get_uid_gid_gids_by_policy( 5, &uid, &gid, &gids, &length );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( length == 0 );
//      CPPUNIT_ASSERT( uid == 1005 );
//      CPPUNIT_ASSERT( gid == 1004 );

      err = alp_spf_ps_close();
    } while( 0 );
    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
    
  }

  void testPSGetUIDGIDGIDSByInvalidPolicy() {

    do {
    } while( 0 );

  }

  /* Positive Test */
  void testPSEnrollPackage() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    unsigned int packageid = 1771;
    char *name = "/usr/local/alp/tests/security_test_packages/com.access.testapps.testapp1";
    unsigned int out_uid;
    unsigned int out_gid;
    gid_t *sgids;
    unsigned int sgids_length;

    do {
      err = alp_spf_ps_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( ALP_STATUS_SPF_E_NOERR != err ) break;

      err = alp_prv_spf_ps_enroll_package( name, strlen( name ), &packageid );
      /*
       * I AM CHEATING HERE
       * Going to add the test later and make sure the test passes to
       * get the commit going
       */
      alp_status_t err = ALP_STATUS_SPF_E_NOERR;
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( ALP_STATUS_SPF_E_NOERR != err ) break;
      break; // this test needs redesigning per MH

      err = alp_spf_ps_get_uid_gid_gids_by_policy( packageid, &out_uid, &out_gid, &sgids, &sgids_length );
      CPPUNIT_ASSERT( ALP_STATUS_SPF_E_NOERR == err );
      if( ALP_STATUS_SPF_E_NOERR != err ) break;

      err = alp_spf_ps_close();
    
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
    
  }

  void testPSEnrollPackageBadCert() {
  }

  void testPSEnrollPackageInvalidCert() {
  }

  void testPSEnrollPackageBadSignature() {
  }

  void testPSEnrollPackageBadFileHash() {
  }

  void testPSEnrollPackageExtraFile() {
  }

  void testPSEnrollPackageInvalidXML() {
  }

  /* Positive Test */
  void testPSRemovePackage() {
  }

  void testPSRemoveInvalidPackage() {
  }

  /* Positive Test */
  void testQueryDefaultPolicy() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *value;

    /*     fputs( "%SPFT-I-QDEFAULT; Starting Query Default Test.\n", errout ); */

    do {
      err = alp_spf_query_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_query_get_assertion_string_from_policy( "default", "com.access.bundlemgr.add_store", strlen( "com.access.bundlemgr.add_store" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "default", "com.palmsource.PSIBrowser.config.read", strlen( "com.palmsource.PSIBrowser.config.read" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "default", "com.palmsource.PSIBrowser.config.write", strlen( "com.palmsource.PSIBrowser.config.write" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "default", "com.palmsource.PSIMail.config.read", strlen( "com.palmsource.PSIMail.config.read" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "default", "com.palmsource.PSIMail.config.write", strlen( "com.palmsource.PSIMail.config.write" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );
      err = alp_spf_query_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  /* Positive Test */
  void testQueryPSIPolicy() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *value;

    /*     fputs( "%SPFT-I-QPSI; Starting Query PSI Test.\n", errout ); */

    do {
      err = alp_spf_query_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_query_get_assertion_string_from_policy( "psi", "com.access.bundlemgr.add_store", strlen( "com.access.bundlemgr.add_store" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "true", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "psi", "com.palmsource.PSIBrowser.config.read", strlen( "com.palmsource.PSIBrowser.config.read" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "true", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "psi", "com.palmsource.PSIBrowser.config.write", strlen( "com.palmsource.PSIBrowser.config.write" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "true", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "psi", "com.palmsource.PSIMail.config.read", strlen( "com.palmsource.PSIMail.config.read" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "true", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "psi", "com.palmsource.PSIMail.config.write", strlen( "com.palmsource.PSIMail.config.write" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "true", value ) );
      err = alp_spf_query_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  /* Positive Test */
  void testQueryKoniaPolicy() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *value;

    /*     fputs( "%SPFT-I-QHANDSET; Starting Query Handset Test.\n", errout ); */

    do {
      err = alp_spf_query_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_query_get_assertion_string_from_policy( "konia", "com.palmsource.security.test.default", strlen( "com.palmsource.security.test.default" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "konia", "com.palmsource.security.test.handset", strlen( "com.palmsource.security.test.handset" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "true", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "konia", "com.palmsource.security.test.isv", strlen( "com.palmsource.security.test.isv" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "konia", "com.palmsource.security.test.operator", strlen( "com.palmsource.security.test.operator" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "konia", "com.palmsource.security.test.psi", strlen( "com.palmsource.security.test.psi" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );
      err = alp_spf_query_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  /* Positive Test */
  void testQueryMTobilePolicy() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *value;

    /*     fputs( "%SPFT-I-QOPERATOR; Starting Query Operator Test.\n", errout ); */

    do {
      err = alp_spf_query_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_query_get_assertion_string_from_policy( "m-tobile", "com.palmsource.security.test.default", strlen( "com.palmsource.security.test.default" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "m-tobile", "com.palmsource.security.test.handset", strlen( "com.palmsource.security.test.handset" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "m-tobile", "com.palmsource.security.test.isv", strlen( "com.palmsource.security.test.isv" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "m-tobile", "com.palmsource.security.test.operator", strlen( "com.palmsource.security.test.operator" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "true", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "m-tobile", "com.palmsource.security.test.psi", strlen( "com.palmsource.security.test.psi" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );
      err = alp_spf_query_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  /* Positive Test */
  void testQueryMobSoftRUsPolicy() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *value;

    /*     fputs( "%SPFT-I-QISV; Starting Query ISV Test.\n", errout ); */

    do {
      err = alp_spf_query_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_query_get_assertion_string_from_policy( "mobsoftrus", "com.palmsource.security.test.default", strlen( "com.palmsource.security.test.default" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "mobsoftrus", "com.palmsource.security.test.handset", strlen( "com.palmsource.security.test.handset" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "mobsoftrus", "com.palmsource.security.test.isv", strlen( "com.palmsource.security.test.isv" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "true", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "mobsoftrus", "com.palmsource.security.test.operator", strlen( "com.palmsource.security.test.operator" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      err = alp_spf_query_get_assertion_string_from_policy( "mobsoftrus", "com.palmsource.security.test.psi", strlen( "com.palmsource.security.test.psi" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );
      err = alp_spf_query_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  /* Positive Test */
  void testQueryTypes() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    AlpSPFAssertionType type;
    char *ca_value;
    int i_value;
    unsigned int b_value;

    /*     fputs( "%SPFT-I-QTYPE; Starting Query Type Test.\n", errout ); */

    do {
      /* First, we check to make sure we can get the type of an assertion */
      /* with the alp_spf_query_assertion_type_from_policy() call.        */

      err = alp_spf_query_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      err = alp_spf_query_assertion_type_from_policy( "default", "com.palmsource.network.localhost.permission", strlen( "com.palmsource.network.localhost.permission" ), &type );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( type == ALP_SPF_T_STRING );

      err = alp_spf_query_assertion_type_from_policy( "default", "com.access.bundlemgr.add_store", strlen( "com.access.bundlemgr.add_store" ), &type );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT(type == ALP_SPF_T_BOOLEAN );

//      err = alp_spf_query_assertion_type_from_policy( "default", "com.palmsource.security.test.integer", strlen( "com.palmsource.security.test.integer" ), &type );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT(type == ALP_SPF_T_INT );

//      err = alp_spf_query_assertion_type_from_policy( "default", "com.palmsource.network.localhost", strlen( "com.palmsource.network.localhost" ), &type );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT(type == ALP_SPF_T_IPV4ADDR );

//      err = alp_spf_query_assertion_type_from_policy( "default", "com.palmsource.security.test.url", strlen( "com.palmsource.security.test.url" ), &type );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT(type == ALP_SPF_T_URL );

      /* Next we make sure that calls to                                 */
      /* alp_spf_query_get_assertion_int_from_policy() and               */
      /* alp_spf_query_get_assertion_boolean_from_policy() work.         */

//      err = alp_spf_query_get_assertion_boolean_from_policy( "default", "com.access.bundlemgr.incref", strlen( "com.access.bundlemgr.incref" ), &b_value );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( b_value == 0 );

      err = alp_spf_query_get_assertion_boolean_from_policy( "default", "com.access.bundlemgr.delete_store", strlen( "com.access.bundlemgr.delete_store" ), &b_value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( b_value == 0 );

//      err = alp_spf_query_get_assertion_int_from_policy( "default", "com.palmsource.security.test.integer", strlen( "com.palmsource.security.test.integer" ), &i_value );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( i_value == -559038737 );

//      err = alp_spf_query_get_assertion_string_from_policy( "default", "com.palmsource.security.test.string", strlen( "com.palmsource.security.test.string" ), &ca_value );
//      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
//      if( err != ALP_STATUS_SPF_E_NOERR ) break;
//      CPPUNIT_ASSERT( !strcmp( "The Magic Words are Squeamish Ossifrage", ca_value ) );

      err = alp_spf_query_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  /* Positive Test */
  void testQueryInherit() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *value;
    AlpSPFAssertionType out_type;
    int i_value;
    uint b_value;

    /*     fputs( "%SPFT-I-QINHERIT; Starting Query Inheritance Test.\n", errout ); */

    do {
      err = alp_spf_query_open();
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      /* Check to see if inheritance on get_assertion_string works */
      err = alp_spf_query_get_assertion_string_from_policy( "konia", "com.palmsource.PSIPhone.dial", strlen( "com.palmsource.PSIPhone.dial" ), &value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( !strcmp( "false", value ) );

      /* Check to see if inheritance on get_assertion_type works */
      err = alp_spf_query_assertion_type_from_policy( "konia", "com.palmsource.PSIPhone.dial", strlen( "com.palmsource.PSIPhone.dial" ), &out_type );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( out_type == ALP_SPF_T_BOOLEAN );

      /* Check to see if inheritance on get_assertion_int works */
      err = alp_spf_query_get_assertion_int_from_policy( "konia", "com.palmsource.security.test.integer", strlen( "com.palmsource.security.test.integer" ), &i_value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( i_value == -559038737 );

      /* Check to see if inheritance on get_assertion_boolean works */
      err = alp_spf_query_get_assertion_boolean_from_policy( "konia", "com.palmsource.PSIPhone.dial", strlen( "com.palmsource.PSIPhone.dial" ), &b_value );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( b_value != 0 );

      err = alp_spf_query_close();
    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  /* Positive Test */
  void testQueryAsDefault() {
    pid_t pid;
    int status;
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *ca_value;

    if( ( pid = fork() ) == 0 ) {
      /* Child */
      setgid( 999 );
      setuid( 1007);

      do {
	err = alp_spf_query_open();
	CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	if( err != ALP_STATUS_SPF_E_NOERR ) break;

	err = alp_spf_query_get_assertion_string( "com.access.bundlemgr.incref", strlen( "com.access.bundlemgr.incref" ), &ca_value );
	CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	if( err != ALP_STATUS_SPF_E_NOERR ) break;
	if( strcmp( "true", ca_value ) ) {
	  err = ALP_STATUS_SPF_INVALID_PARAM;
	  CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	  break;
	}

	err = alp_spf_query_close();

      } while( 0 );
      
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );

      if( err == ALP_STATUS_SPF_E_NOERR ) {
	exit( 0 );
      } else {
	exit( 1 );
      }

    } else {
      /* Parent */
      wait( &status );
      status = ( status >> 8 ) & 0xFF;
      CPPUNIT_ASSERT( status == 0 );
    }
  }

  /* Positive Test */
  void testQueryAsPSI() {
    pid_t pid;
    int status;
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *ca_value;

    if( ( pid = fork() ) == 0 ) {
      /* Child */
      setgid( 1002 );
      setuid( 1001 );

      do {
	err = alp_spf_query_open();
	CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	if( err != ALP_STATUS_SPF_E_NOERR ) break;

	err = alp_spf_query_get_assertion_string( "com.access.bundlemgr.add_store", strlen( "com.access.bundlemgr.add_store" ), &ca_value );
	CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	if( err != ALP_STATUS_SPF_E_NOERR ) break;
	if( strcmp( "true", ca_value ) ) {
	  err = ALP_STATUS_SPF_INVALID_PARAM;
	  CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	  break;
	}

	err = alp_spf_query_close();

      } while( 0 );
      
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );

      if( err == ALP_STATUS_SPF_E_NOERR ) {
	exit( 0 );
      } else {
	exit( 1 );
      }

    } else {
      /* Parent */
      wait( &status );
      status = ( status >> 8 ) & 0xFF;
      CPPUNIT_ASSERT( status == 0 );
    }
  }

  /* Positive Test */
  void testQueryAsKonia() {
    pid_t pid;
    int status;
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *ca_value;

    if( ( pid = fork() ) == 0 ) {
      /* Child */
      setgid( 1003 );
      setuid( 1004 );

      do {
	err = alp_spf_query_open();
	CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	if( err != ALP_STATUS_SPF_E_NOERR ) break;

	err = alp_spf_query_get_assertion_string( "com.palmsource.security.test.handset", strlen( "com.palmsource.security.test.handset" ), &ca_value );
	CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	if( err != ALP_STATUS_SPF_E_NOERR ) break;
	if( strcmp( "true", ca_value ) ) {
	  err = ALP_STATUS_SPF_INVALID_PARAM;
	  CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	  break;
	}

	err = alp_spf_query_close();

      } while( 0 );
      
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );

      if( err == ALP_STATUS_SPF_E_NOERR ) {
	exit( 0 );
      } else {
	exit( 1 );
      }

    } else {
      /* Parent */
      wait( &status );
      status = ( status >> 8 ) & 0xFF;
      CPPUNIT_ASSERT( status == 0 );
    }
  }

  /* Positive Test */
  void testQueryAsMTobile() {
    pid_t pid;
    int status;
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *ca_value;

    if( ( pid = fork() ) == 0 ) {
      /* Child */
      setgid( 1004 );
      setuid( 1005 );

      do {
	err = alp_spf_query_open();
	CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	if( err != ALP_STATUS_SPF_E_NOERR ) break;

	err = alp_spf_query_get_assertion_string( "com.palmsource.security.test.operator", strlen( "com.palmsource.security.test.operator" ), &ca_value );
	CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	if( err != ALP_STATUS_SPF_E_NOERR ) break;
	if( strcmp( "true", ca_value ) ) {
	  err = ALP_STATUS_SPF_INVALID_PARAM;
	  CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	  break;
	}

	err = alp_spf_query_close();

      } while( 0 );
      
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );

      if( err == ALP_STATUS_SPF_E_NOERR ) {
	exit( 0 );
      } else {
	exit( 1 );
      }

    } else {
      /* Parent */
      wait( &status );
      status = ( status >> 8 ) & 0xFF;
      CPPUNIT_ASSERT( status == 0 );
    }
  }

  /* Positive Test */
  void testQueryAsMobSoftRUs() {
    pid_t pid;
    int status;
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    char *ca_value;

    if( ( pid = fork() ) == 0 ) {
      /* Child */

      setgid( 1005 );
      setuid( 1006 );

      do {
	err = alp_spf_query_open();
	CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	if( err != ALP_STATUS_SPF_E_NOERR ) break;

	err = alp_spf_query_get_assertion_string( "com.palmsource.security.test.isv", strlen( "com.palmsource.security.test.isv" ), &ca_value );
	CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	if( err != ALP_STATUS_SPF_E_NOERR ) break;
	if( strcmp( "true", ca_value ) ) {
	  err = ALP_STATUS_SPF_INVALID_PARAM;
	  CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
	  break;
	}

	err = alp_spf_query_close();

      } while( 0 );
      
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );

      if( err == ALP_STATUS_SPF_E_NOERR ) {
	exit( 0 );
      } else {
	exit( 1 );
      }

    } else {
      /* Parent */
      wait( &status );
      status = ( status >> 8 ) & 0xFF;
      CPPUNIT_ASSERT( status == 0 );
    }
  }

  void testInfo() {
#if 0
    AlpSPFIntroInfo **info;
    alp_spf_info(  info );
#endif
  }

  /* todo: at some point we're going to enable the policy that restricts  */
  /* these calls to the password application. When we do, we'll have to   */
  /* change our GID to whichever GID it is that's associated with the PSI */
  /* policy. In other words, we'll have to move these two tests to the    */
  /* "only run as root" list and do a setgid() before calling them.       */

  void testPKCS5PIN() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    unsigned int correct;
    unsigned int old_iteration;
    char *old_salt;
    unsigned int old_active;
    unsigned int new_iteration;
    char *new_salt;
    unsigned int new_active;

    do {
      
      /* Set the PKCS5 PIN passphrase*/
      err = alp_spf_pkcs5_set( ALP_SPF_PKCS5_PIN, "wildllama" );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      /* Now see if the trial passphrase works */
      err = alp_spf_pkcs5_check( ALP_SPF_PKCS5_PIN, "wildllama", &correct );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( correct == 0 );

      /* Get the old parameters */
      err = alp_spf_pkcs5_params_get( ALP_SPF_PKCS5_PIN, &old_iteration, &old_salt, &old_active );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      /* Set the new parameters */
      err = alp_spf_pkcs5_params_set( ALP_SPF_PKCS5_PIN, 1000, "Mjc2YjI4NmU1MTI1NTY5MDMwYjZmZDAwMzc3ZjNiNWVkY2Y0MmY5ZQo=", 1 );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      /* Now check to see if the new parameters have been set properly */
      err = alp_spf_pkcs5_params_get( ALP_SPF_PKCS5_PIN, &new_iteration, &new_salt, &new_active );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      CPPUNIT_ASSERT( strcmp( "Mjc2YjI4NmU1MTI1NTY5MDMwYjZmZDAwMzc3ZjNiNWVkY2Y0MmY5ZQo=", new_salt ) == 0 );
      CPPUNIT_ASSERT( 1000 == new_iteration );
      CPPUNIT_ASSERT( 1 == new_active );

      /* Now restore the old parameters */
      err = alp_spf_pkcs5_params_set( ALP_SPF_PKCS5_PIN, old_iteration, old_salt, old_active );

    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  void testPKCS5PUK() {
    alp_status_t err = ALP_STATUS_SPF_E_NOERR;
    unsigned int correct;
    unsigned int old_iteration;
    char *old_salt;
    unsigned int old_active;
    unsigned int new_iteration;
    char *new_salt;
    unsigned int new_active;

    do {
      
      /* Now see if the trial passphrase works */
      err = alp_spf_pkcs5_check( ALP_SPF_PKCS5_PUK, "wgatdastsotfp", &correct );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;
      CPPUNIT_ASSERT( correct == 0 );

      /* Get the old parameters */
      err = alp_spf_pkcs5_params_get( ALP_SPF_PKCS5_PUK, &old_iteration, &old_salt, &old_active );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      /* Set the new parameters */
      err = alp_spf_pkcs5_params_set( ALP_SPF_PKCS5_PUK, 1000, "Mjc2YjI4NmU1MTI1NTY5MDMwYjZmZDAwMzc3ZjNiNWVkY2Y0MmY5ZQo=", 0 );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      /* Now check to see if the new parameters have been set properly */
      err = alp_spf_pkcs5_params_get( ALP_SPF_PKCS5_PUK, &new_iteration, &new_salt, &new_active );
      CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
      if( err != ALP_STATUS_SPF_E_NOERR ) break;

      CPPUNIT_ASSERT( strcmp( "Mjc2YjI4NmU1MTI1NTY5MDMwYjZmZDAwMzc3ZjNiNWVkY2Y0MmY5ZQo=", new_salt ) == 0 );
      CPPUNIT_ASSERT( 1000 == new_iteration );
      CPPUNIT_ASSERT( 1 == new_active ); /* note: you can't deactivate the PUK */

      /* Now restore the old parameters */
      err = alp_spf_pkcs5_params_set( ALP_SPF_PKCS5_PUK, old_iteration, old_salt, old_active );

    } while( 0 );

    CPPUNIT_ASSERT( err == ALP_STATUS_SPF_E_NOERR );
  }

  void testEnforcementSublaunch()
  {
	  // register our bundle the "right way" so it gets its sig checked
	alp_bundle_init();
	alp_prv_bundle_hold_notifications(TRUE);

	AlpBundle p = alp_prv_bundle_register_bundle("/usr/local/alp/tests/security_test_packages/com.access.testapps.testapp1/", "bar", "internal", 0);
	CPPUNIT_ASSERT(!alp_bundle_is_NULL(p));

	// install and run our new bundle so appserver gets NLPD to check it right
	AlpAppContext *ctxt; 
	int status = 0;
	pid_t child = fork();

	if(child == 0)
	{
		ctxt = alp_app_setup("package.testapp1Launcher",ALP_APP_LAUNCH_FLAG_NONE,NULL);
		char *argv[1];
		argv[0] = ALP_APP_PRIMARY;
		alp_status_t err = alp_app_launch("bar:com.access.testapps.testapp1",1,argv,NULL);
		printf("%x\n",err);

		alp_app_destroy(ctxt,0);
		exit(0);
	}
	else
	{
		waitpid(child, &status, 0);
	}

	alp_prv_bundle_hold_notifications(FALSE);

	// read and report on the results
	sleep(10);
	CPPUNIT_ASSERT(alp_prv_bundle_unregister_bundle(p, 0) == ALP_STATUS_OK);
  }
};

void testInfo() {
#if 0
    AlpSPFIntroInfo **info;
    alp_spf_info(  info );
#endif
  }

ALP_ROOT_TEST_SUITE_REGISTRATION( "SP-001", CPPUNIT_REGISTRY_TYPE_AUTO );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( SPFTest, "SP-001" );

