/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*! @file sysutils.c
	This file contains the implementation of various system utility APIs.
*/

// Uncomment the next line to disable traces AT COMPILE TIME 
#define ALP_TRACE ALP_TRACE_OFF		//Enable/disable



#include <signal.h>
#include <hiker/pdk/sysutils_pdk.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>

/*
	Prototypes
*/


/*
	Globals
*/
static int sDaemonFD = -1;


/* ***************************************************
	Public SDK APIs
   *************************************************** */







/* ***************************************************
	Semi-Public PDK APIs (available to licensees only)
   *************************************************** */

/*!
This API is used by system services during boot.  The proper way for a 
system service to be started from alp_init.sh is for it to be run as an
executable in foreground mode (without an '&').  The service then calls
alp_daemon_start() to fork a child process, completes initialization there,
and calls alp_daemon_release() to cause the parent process to exit so the
boot sequence can continue.  This procedure ensures that system services 
are always brought up synchronously and in a consistent manner.

@param[in] nochdir Pass non-zero to prevent the current working directory from being set to "/".
@param[in] noclose Pass non-zero to prevent stdin, stdout, and stderr from being redirected to /dev/null.

@return Yields 0 in the child process on success.  Does not return in the parent process.
*/
int alp_daemon_start(int nochdir, int noclose)
{
	int child;

	int pipe_fd[2];
 
	pipe(pipe_fd);
 
	sDaemonFD = pipe_fd[1];
 
	if ((child = fork()) == 0) 
	{
 		// child
//		printf("Child PID = %d\n", getpid());
//		printf("Parent PID = %d\n", getppid());
		
		close(pipe_fd[0]);
  
		if (!nochdir)
			chdir("/");
   
		if (!noclose) {
			int devnull = open("/dev/null", O_RDWR);
			dup2(devnull, 0);
			dup2(devnull, 1);
			dup2(devnull, 2);
		}
  
		// Allow the child process to continue
		return 0;
	}

	// parent
//	printf("Parent PID = %d\n", getpid());

	close(pipe_fd[1]);
 
	signal(SIGPIPE, SIG_IGN);
 
	// wait for child to send us a message on the pipe. If the child dies, 
	// we'll get EPIPE,otherwise a status response.
 
//	printf("Parent waiting\n");
 
	int status = 0;
	int result = read(pipe_fd[0], &status, sizeof(int));

//	printf("Parent got result %d, status %d\n", result, status);

	// Immediately exit the parent, with success or failure. No code past 
	// alp_daemon_start() is executed in the parent.

	if (result == -1) {
		exit(127);
	}
 
	exit(status);
}

/*!
This API is used by system services during boot, in conjunction with 
alp_daemon_start().  This call is made by the child process after the parent calls alp_daemon_start, and will cause the parent to exit with the indicated status, allowing the boot sequence to continue.  See documentation for alp_daemon_start() for more information.

@param[in] status The status code which should be passe dby the parent to exit().

@return Yields 0 on success.
 */
int alp_daemon_release(int status)
{
//	printf("Childing releasing\n");
 
	// Don't fail if we're not in daemon mode 
	if (sDaemonFD == -1)
		return 0;
//	printf("Child sending status %d\n", status);

	// Allow the parent to continue, and to exit with a particular status.
	int result = write(sDaemonFD, &status, sizeof(int));
 
//	printf("Child result of send was %d\n", result);

	if (result == sizeof(int))
		return 0;
	else
		return -1;
}
