/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*
	Unit tests for the sysutils library
*/

// Uncomment the next line to disable traces AT COMPILE TIME 
#define ALP_TRACE ALP_TRACE_OFF		//Enable/disable

#include <hiker/unit_test.h>
#include <cppunit/extensions/HelperMacros.h>

#include <hiker/pdk/sysutils_pdk.h>

#include <errno.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <unistd.h>





class SysUtilsTest: public CPPUNIT_NS::TestFixture
{
	CPPUNIT_TEST_SUITE( SysUtilsTest );
	
	
	CPPUNIT_TEST( DaemonTest );
	
	
	CPPUNIT_TEST_SUITE_END();
	
	public:
		void setUp();         // overriding TestFixture method
		void tearDown();      // overriding TestFixture method
		
	protected:
		// Suite-level setup & teardown.  Not really unit tests...
		void DaemonTest(void);

		
		
	private:
		
};

#define ALP_PRV_SYSUTILS_SUITE_NAME "SYS-001"

ALP_ROOT_TEST_SUITE_REGISTRATION( ALP_PRV_SYSUTILS_SUITE_NAME, CPPUNIT_REGISTRY_TYPE_AUTO );
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( SysUtilsTest, ALP_PRV_SYSUTILS_SUITE_NAME );


void SysUtilsTest::setUp()
{
	
}

void SysUtilsTest::tearDown()
{
  // if you had allocated resources in setUp, this is where you would
  // deallocate them.
}



// ***********************************************************
// Testing for alp_daemon_start() and alp_daemon_release():
// ***********************************************************
void SysUtilsTest::DaemonTest(void)
{
	pid_t childPID, pid, daemonPID; 
	int status;
	int pipe_fd[2];
	int result;
	pid_t parentPID = getpid();
	
	result = pipe(pipe_fd);
//	printf("Created pipe, result = %d\n", result);
		
	childPID = fork();
	if(childPID == 0)
	{
		// Child of testrunner.
		childPID = getpid(); // get our real PID.
		
		// Close the "reading" end of pipe we use to send result back to parent.
		close(pipe_fd[0]);

		// call alp_daemon_start() to fork a 
		// child process to be our "test daemon"
		alp_daemon_start(1, 1);
		
		// Here we're in the child-child-process, 
		// the parent will not return from alp_daemon_start().
		
		// Allow parent to terminate with exit status 23.
		alp_daemon_release(23);
		
		// If any of these three PIDs match
		daemonPID = getpid();
//		printf("testrunner PID=%d, childPID=%d, daemonPID=%d\n",
//			  	parentPID, childPID, daemonPID);
		if(parentPID == childPID || 
			childPID == daemonPID || 
			parentPID == daemonPID)
			status = 0;
		else status = 42; // test will fail if we do not return 42.
		
		// Write secret message to pipe.
		result = write(pipe_fd[1], &status, sizeof(int));
//		printf("Wrote. result = %d, data = %d\n", result, status);
		
		// Then we exit as well.
		exit(0);
	}
	
	// Close the "writing" end of the pipe.
	close(pipe_fd[1]);

	// Testrunner: wait for child process to terminate, 
	// and make sure we receive the correct "secret message" from the 
	// daemon process.
	// We should see exit code 23, and then 42.
	pid = wait(&status);
	CPPUNIT_ASSERT(WIFEXITED(status));
//	printf("Exit Status: %d, pid=%d, childPID=%d\n", WEXITSTATUS(status), pid, childPID);
	CPPUNIT_ASSERT(WEXITSTATUS(status) == 23);
	CPPUNIT_ASSERT(pid == childPID);
	
	
	// Read result from child process:	
	result = read(pipe_fd[0], &status, sizeof(int));
//	printf("Read. result = %d, data = %d\n", result, status);
	CPPUNIT_ASSERT(result == sizeof(int));
	CPPUNIT_ASSERT(status == 42);
	
	return;
}
