/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/prv/timestamp.h>
#include <sys/types.h>
#include <sys/stat.h>

static FILE *fp = NULL;
static char *oldfilename = NULL;
static int timestamping = 0;
static int init = 0;
static float lasttime = 0.0f;
const float to_ms = 1.0/1.0e6;

// Timestamp writes a timestamp message to the file pointed to by fp.
// It only does this if the enviroment TIMESTAMP is set to something.
// Usage :
// timestamp("starting app", "my_app", "/tmp/myapp.log");
// ...
// timestamp("starting phase2", "my_app", NULL); ...
// ..
// The first timestamp with the 3rd parameter opens the file
//
void timestamp(char *mesg, char *source, char *filename)
{
	if (!init) {
		/// Only call getenv once 
		timestamping = getenv("TIMESTAMP") ? 1 : 0;
		init = 1;
	}
	if (timestamping) {
		struct timeval tv;
		struct timezone tz;
		gettimeofday(&tv, &tz);

		// float should be more than sufficient for this
		float fraction = to_ms * tv.tv_usec;
		fraction += (tv.tv_sec & 0xffff);

		if ( fp == NULL) {
			// Need a file name
			if ( filename == NULL) {
				perror("Source must not be NULL on first call to timestamp()");
				exit(0);
			}
			// open the file
			if ((fp = fopen( filename, "a")) == NULL) {
				perror("Unable to open log file");
				exit(0);
			}
			chmod( filename, 00666 );
			// store filename of currently open file
			oldfilename = strdup(filename);
		} else  if ( filename && strcmp(oldfilename, filename) != 0) {
			fclose(fp);
			if ((fp = fopen( filename, "a")) == NULL) {
				perror("Unable to open log file");
				exit(0);
			}
			chmod( filename, 00666 );
			if (oldfilename)
				free(oldfilename);
			oldfilename = strdup(filename);
		}
		fprintf(fp, "%s. %s at \t\t --time %4.3f (dt=%4.3fs)\n", 
				source, mesg, fraction,
			       	lasttime == 0.0f ? 0.0f : fraction-lasttime);
		lasttime = fraction;
		fflush(fp);
	}
}

// Used if you want to open a different logfile otherwise the file
// will close when the processes exits.
//
void donelog()
{
	if (fp != NULL)
		fclose(fp);
}
