/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <fail.c>
 *
 * @brief Provide implementation for fatal errors
 *
 ******************************************************************************/


#include <stdio.h>
#include <stdarg.h>
#include <signal.h>
#include <string.h>
#include <stdlib.h>
#include <execinfo.h>
#include <hiker/fail.h>

// ---------------------------------------------------------------------------
#define BACKTRACE_MAX_SIZE	32
static void alp_prv_backtrace(int skipLines)
{
	void*	btbuf[BACKTRACE_MAX_SIZE];
	char	**symbols, **psym;
	int		num;

	num = backtrace(btbuf, BACKTRACE_MAX_SIZE);
	if (num >= skipLines)
	{
		symbols = backtrace_symbols(btbuf, num);
		if (symbols != 0)
		{
			psym = symbols + skipLines;
			while (psym != symbols + num)
			{
				if (strstr(*psym, "libc_start") != 0)
					break;
				if (strstr(*psym, "libpthread.so") != 0)
					break;
				fprintf(stderr, "  %s\n", *psym);
				psym++;
			}
			free(symbols);
		}
	}
}

// ---------------------------------------------------------------------------
void alp_fail_raise(const char* file, int line, const char* cond, const char* fmt, ...)
{
	fprintf(stderr, "*** FATAL ERROR ***\n");
	fprintf(stderr, "Condition [%s] is verified.\n", cond);
	fprintf(stderr, "In file [%s], line %d\n", file, line);
	if (fmt != 0)
	{
		va_list arg;

		va_start(arg, fmt);
		vfprintf(stderr, fmt, arg);
		va_end(arg);
	}

	fprintf(stderr, "Back trace:\n");
	alp_prv_backtrace(1);
	raise(SIGTRAP);
}
