/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/

#include <hiker/unit_test.h>
#include <cppunit/extensions/HelperMacros.h>
#include <hiker/types.h>

#include <hiker/config_defines.h>
#include <hiker/traces.h>

/*
#if ALP_TRACE == ALP_TRACE_ON
#warning "ALP_TRACE is ALP_TRACE_ON"
#else
#warning  "ALP_TRACE is ALP_TRACE_OFF"
#endif
*/

#define testTraceClass			ALP_TRACE_DEFINE(ALP_CLASS_UNDEFINED, 7)


class TracesTests : public CPPUNIT_NS::TestFixture
{
	CPPUNIT_TEST_SUITE(TracesTests);

	CPPUNIT_TEST(OutputOnStdErr);
	CPPUNIT_TEST(OutputOnTCP);
	CPPUNIT_TEST(OutputOnSyslog);
	CPPUNIT_TEST(OutputOnXML);
	CPPUNIT_TEST(OutputOnNULL);
	CPPUNIT_TEST(SetLocal);

	CPPUNIT_TEST_SUITE_END();

	public:
		void setUp();
		void tearDown();

	protected:
		void OutputOnStdErr();
		void OutputOnTCP();
		void OutputOnXML();
		void OutputOnSyslog();

		void OutputOnNULL();
		void SetOuput();
		void SetLocal();
	private:
		void DisplayText(void);
		char cOutput[512];
		
}; 


ALP_ROOT_TEST_SUITE_REGISTRATION("OS-030", CPPUNIT_REGISTRY_TYPE_MANUAL);
CPPUNIT_TEST_SUITE_NAMED_REGISTRATION(TracesTests, "OS-030");


void TracesTests::setUp()
{
	alp_trace_getoutput(cOutput, 512);
	printf("\nSETUP: -----------> Get output : %s\n", cOutput);
	

}

void TracesTests::tearDown()
{
	alp_trace_setoutput(cOutput,0);
	printf("\nTEARDOWN:----------->  Set output : %s\n", cOutput);
}

void TracesTests::OutputOnStdErr()
{
	alp_trace_init(0, "stderr",0);
	DisplayText();
	CPPUNIT_ASSERT(true);
}

void TracesTests::OutputOnTCP()
{
	alp_trace_init(0, "tcp","reporter.palmsource.com");
	DisplayText();
	CPPUNIT_ASSERT(true);
}

void TracesTests::OutputOnSyslog()
{
	alp_trace_init(0, "syslog",0);
	DisplayText();
	CPPUNIT_ASSERT(true);
}

void TracesTests::OutputOnXML()
{
	alp_trace_init(0, "XML","testfile.xml");
	DisplayText();
	CPPUNIT_ASSERT(true);
}

void TracesTests::OutputOnNULL()
{
	alp_trace_init(0, "NULL",0);
	DisplayText();
	CPPUNIT_ASSERT(true);
}

void TracesTests::SetOuput()
{
	alp_trace_setoutput("stderr",0);
	DisplayText();
	CPPUNIT_ASSERT(true);
}

void TracesTests::SetLocal()
{
	alp_trace_setoutput("stderr",0);
	alp_trace_local_add(testTraceClass);		//Add the global filter
	ALP_TM(TL(testTraceClass, "1/3 This line must be displayed !"));
	alp_trace_local_remove(testTraceClass);		//Add the global filter
	ALP_TM(TL(testTraceClass, "2/3 This line SHOULD NOT be displayed !"));
	alp_trace_local_add(testTraceClass);		//Add the global filter
	ALP_TM(TL(testTraceClass, "3/3 This line must be displayed !(2/3 is not displayed"));
	CPPUNIT_ASSERT(true);
}

void TracesTests::DisplayText(void)
{
	unsigned char theBuffer[121];
	ALP_TM(TL(testTraceClass, "This is a test."));
	ALP_TM(TL(testTraceClass, "%#2hx.", 4));
// Not working		TraceOutput(TL(0x0100, "[\\n\\r\\t] = [\n\r\t]"));
// Not working 		TraceOutput(TL(0x0100, "Specials: [�]");)
	theBuffer[0] = 0xE9;
	theBuffer[1] = 0xE8;
	ALP_TM(TL(testTraceClass, "%lu.", theBuffer[0]));
	ALP_TM(TL(testTraceClass, "Specials in a buffer:"));
	ALP_TM(B(testTraceClass, theBuffer, 2));

	{
		int i;
		for (i = 0; i < (int)sizeof(theBuffer) - 1; i++)
			theBuffer[i] = i;
		theBuffer[sizeof(theBuffer) - 1] = 0;
		ALP_TM(TL(testTraceClass, "Buffer in ASCII: %s", theBuffer + 1));
		ALP_TM(T(testTraceClass, "Buffer in HEXA"));
		ALP_TM(B(testTraceClass, theBuffer, sizeof(theBuffer)));
	}
}




