/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <traces_conf.c>
 *
 * @brief Provide some functions to access config file for traces library. All settings are stored 
 * 	in a ".traces.conf" file  This file relies on glib library. 
 *
 ******************************************************************************/
	
#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>

#include <sys/time.h>
#include <hiker/traces.h>

#include "traces_conf_util.h"

/* -------------------------------------------------------------------- */
#define envHomeDir          "HOME"

/* -------------------------------------------------------------------- */
/* Key for config file- case sensitive !!! */
#define KEY_CONFIG_LOCATION	"/etc/traces/"				// Directory for config file
#define KEY_LOG_LOCATION		"/var/log/"			// Directory for log file

#define KEY_CONF_FILE			"traces.conf"
#define DEFAULT_OUTPUT 		"file:"KEY_LOG_LOCATION"traces.log"		//Changes done on Jeff P. request

#define KEY_SECTION_SETTING	"SETTINGS"	
#define KEY_PARAM_OUTPUT		"OUTPUT"	
#define KEY_SECTION_FILTER		"FILTERS"

const char *COMMENT_HEADER="#Configuration file for traces\n" ;
const char *COMMENT_FILTER="#Filters: should be like: Bluetooth=0x03000000 (see sysclass.h for values).\n#If no filter are specified, everything will be outputed.\n";

static char g_homeConf[512] = {0};
static char g_CurrentOutput[512] = {0};
static int    g_displayed = 0;

typedef struct _AlpSystemClasses
{
	char    name[64];
	char    valueHexa[16];
	bool    bstatus;

}AlpSystemClasses;




/*--------------------------------------------------------------------*/
/*--------------------------------------------------------------------*/
int  alp_prv_gb_settings_initialize(void)
{
	char *pTmpOutput=NULL;
	int iRet;

	/* Prepare the full path name */
	/* And clear buffer */
	if (g_homeConf[0]== 0)
	{
		snprintf(g_homeConf, sizeof(g_homeConf), "%s%s", KEY_CONFIG_LOCATION,KEY_CONF_FILE);
		prv_conf_close();		//To reload conf file		
	}

	if (-1 == prv_conf_getfile(g_homeConf, CONF_RDONLY))	/* create if needed, prev RDWR*/
	{
		if ( g_displayed == 0)
		{
			g_displayed = 1;
	 		fprintf(stderr,"[TRACES] Warning: Cannot open Traces config file at %s - Traces ignored !!! \n",g_homeConf);
			return -1;
		}
	}
	
	/* Config file found, check for data */
	pTmpOutput = prv_conf_read_string( KEY_SECTION_SETTING, KEY_PARAM_OUTPUT, "!");
	if (pTmpOutput[0]=='!')
	{
		if ( g_displayed == 0)
		{
			g_displayed = 1;
			fprintf(stderr,"[TRACES] Warning: Output defaulted to [%s] \n", DEFAULT_OUTPUT);			
		}
		strcpy(g_CurrentOutput,DEFAULT_OUTPUT);
	}
	else

	/* flush */
	if (pTmpOutput)
		strcpy(g_CurrentOutput, pTmpOutput);
	
exitfct:
	if (pTmpOutput)
		free(pTmpOutput);

	//fprintf(stderr,"[TRACES] xxx - Default to [%s] \n", g_CurrentOutput);			

	return 0;
}


/* -------------------------------------------------------------------- */
/*  Internal */
/* -------------------------------------------------------------------- */
 void alp_prv_traces_conf_Close(void)
{
	prv_conf_close( );
	//free(g_pCurrentOutput);
	strcpy(g_CurrentOutput,"");
}


/* -------------------------------------------------------------------- */
/*  Internal */
/* -------------------------------------------------------------------- */
char *  alp_prv_traces_conf_GetOutput(void)
{

	char *pTmpOutput=NULL;

	if (-1 == alp_prv_gb_settings_initialize())	/* read conf file if needed */
	{	// Failed to get config file
		return NULL;
	}
	
	pTmpOutput = prv_conf_read_string( KEY_SECTION_SETTING, KEY_PARAM_OUTPUT, "");

	if ((pTmpOutput == NULL) ||  (strlen(pTmpOutput) ==0 ))
	{
		strcpy(g_CurrentOutput,DEFAULT_OUTPUT);
	}
	else
	{
		strcpy(g_CurrentOutput, pTmpOutput);
		free(pTmpOutput);
	}	
	return (char *) g_CurrentOutput;

}

/*	------------------------------------------------------------------- */
/* Internal  */
/*	------------------------------------------------------------------- */
int  alp_prv_traces_conf_SetOutput(const char* aOutput)
{		
	char *pTmpOutput=NULL;

	if  (-1 == alp_prv_gb_settings_initialize())	/* read conf file if needed */
	{	// Failed to get config file
		return -1;
	}

	prv_conf_write_string( KEY_SECTION_SETTING, KEY_PARAM_OUTPUT, aOutput);
	/* And reread */
	
	pTmpOutput = prv_conf_read_string( KEY_SECTION_SETTING, KEY_PARAM_OUTPUT, DEFAULT_OUTPUT);
	if (pTmpOutput)
	{
		strcpy(g_CurrentOutput, pTmpOutput);
		free(pTmpOutput);
	}
	else
		strcpy(g_CurrentOutput,"");

	prv_conf_flush();
	return 0;
}

/*	------------------------------------------------------------------- */
/* DOLATER: Add error check */
/* DOLATER: Add check to see if file must be reloaded or not */
/* Load only the first call*/
/*	------------------------------------------------------------------- */
int 	alp_prv_Fill_Filter(int *a_Classes, int iArraySize)
{	
	int iItems =0;
	char **output=NULL;
	int i = 0, j=0;
	int rc ;
	char *pparam[2];
	char bRet[]="Y";
	char **test=NULL;
	int bEnable = 0;

	
	/* Add a check to see if we reload- file size ?  */
	rc = prv_conf_get_keys(KEY_SECTION_FILTER,&iItems, &output);
	if (output == NULL)
		return -1;
	for (i = 0;  ((i < iArraySize) && (i < iItems)) ; i++)
	{	
		rc =prv_conf_read_list(KEY_SECTION_FILTER, output[i], &j, &test);
		bEnable = 1;				/* Default, yes  */
		if (j > 1)							/* Check status */
		{
			char c = test[1][0];
			if ((c == '0') || (c == 'n') || (c == 'N'))
				bEnable = 0;
		}
	
		if (bEnable)
		{
			sscanf(test[0],"%x",&a_Classes[i]);
		}
		{
			char** ptr;
			for (ptr = test; *ptr; ptr++)
				free(*ptr);
		}
		free(test);
		free(output[i]);
	}

	/* clean memory b4 leaving */
	free(output); //DOLATER	
	return iItems;
}

/*-------- EOF -------*/
