/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <traces_file.c>
 *
 * @brief C file for generating traces to a text file.
 *
 ******************************************************************************/
 
#include <stdlib.h>
#include <string.h>

#include "traces_helper.h"
#include "traces_file.h"

/*	------------------------------------------------------------------- */
TTracer* TrFile_Create(void)
{
	TFileTracer*	theTracer;

	theTracer = malloc(sizeof(TFileTracer));
	theTracer->itsOpenProc = TrFile_OpenProc;
	theTracer->itsCloseProc = TrFile_CloseProc;
	theTracer->itsSendProc = TrFile_SendProc;
	theTracer->itsDestroyProc = TrFile_DestroyProc;
	theTracer->itsDisconnectedProc = TrFile_DisconnectedProc;
	theTracer->itsfOpened = 0;

	return (TTracer*)theTracer;
}

/*	------------------------------------------------------------------- */
int TrFile_OpenProc(TTracer* aTracer, const char* filename)
{
	((TFileTracer*) aTracer)->itsFile = 0;
	aTracer->itsfOpened = 0;
	aTracer->itsOutputKind = OutputKind_HumanReadable;
	((TFileTracer*) aTracer)->itsFile = fopen(filename, "a+");
	if (!(((TFileTracer*) aTracer)->itsFile))
	{
		fprintf(stderr,"[Traces file] Cannot open %s", filename);
	}
	
	
	if (((TFileTracer*) aTracer)->itsFile)
	{
		aTracer->itsfOpened = 1;
		//Kenneth req (060721):
		setvbuf(((TFileTracer*) aTracer)->itsFile, (char *)NULL, _IONBF , 0);
	}
	
	return aTracer->itsfOpened;
}

/*	------------------------------------------------------------------- */
void TrFile_CloseProc(TTracer* aTracer)
{
	if (!aTracer->itsfOpened)
		return;
	
	fclose(((TFileTracer*)aTracer)->itsFile);
	((TFileTracer*)aTracer)->itsFile = 0;
	aTracer->itsfOpened = 0;
}

/*	------------------------------------------------------------------- */
#define DUMP_LINE_LEN	16
void TrFile_SendProc(TTracer* aTracer, TTraceSendParams* aParams)
{
	static char	sHexDigits[] = "0123456789ABCDEF";

	if (!aTracer->itsfOpened)
		return;
	
	if (aParams->itsKind == TraceKind_Binary)
	{
		size_t	theSize, i;
		char	theBuffer[DUMP_LINE_LEN * 3 + 2 + DUMP_LINE_LEN + 1];
		unsigned char*	theDataPtr, *theBufferPtr;

		memset(theBuffer, ' ', sizeof(theBuffer));
		theDataPtr = aParams->itsData;
		while (aParams->itsDataLen > 0)
		{
			if (aParams->itsDataLen > DUMP_LINE_LEN)
				theSize = DUMP_LINE_LEN;
			else
				theSize = aParams->itsDataLen;

			theBufferPtr = theBuffer;
			for (i = 0; i < theSize; i++, theBufferPtr += 3)
			{
				theBufferPtr[0] = sHexDigits[theDataPtr[i] >> 4];
				theBufferPtr[1] = sHexDigits[theDataPtr[i] & 0x0F];
			}

			if (theSize < DUMP_LINE_LEN)
			{
				memset(theBufferPtr, ' ', 3 * (DUMP_LINE_LEN - theSize));
				theBufferPtr += 3 * (DUMP_LINE_LEN - theSize);
			}
			theBufferPtr += 2;

			for (i = 0; i < theSize; i++)
			{
				if (theDataPtr[i] < 0x20)
					*theBufferPtr++ = '.';
				else
					*theBufferPtr++ = theDataPtr[i];
			}

			*theBufferPtr = 0;
			fprintf(((TFileTracer*)aTracer)->itsFile, "%s\n", theBuffer);

			aParams->itsDataLen -= theSize;
			theDataPtr += theSize;
		}
	}
	else
	{
		fprintf(((TFileTracer*)aTracer)->itsFile, "%s", aParams->itsData);
		if (aParams->itsKind == TraceKind_EOL)
			fprintf(((TFileTracer*)aTracer)->itsFile, "\n");
	}
}

/*	------------------------------------------------------------------- */
void TrFile_DestroyProc(TTracer* aTracer)
{
	if (aTracer->itsfOpened)
		TrFile_CloseProc(aTracer);
	free(aTracer);
}

/*	------------------------------------------------------------------- */
long TrFile_DisconnectedProc(TTracer* aTracer)
{
	return !aTracer->itsfOpened;
}

