/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <traces_helper.h>
 *
 * @brief Header file for traces library.
 *
 ******************************************************************************/

#ifndef _TRACES_HELPER_H
#define _TRACES_HELPER_H

/*	------------------------------------------------------------------- */
#include "traces_main.h"

/*	------------------------------------------------------------------- */
#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/*	------------------------------------------------------------------- */
typedef	unsigned short  TraceModule;
typedef	unsigned short TraceCode;
typedef	unsigned long	TraceIdent;

typedef	unsigned char	TraceKind;
#define TraceKind_Normal			0
#define TraceKind_EOL				1
#define TraceKind_Binary			2 

#define OutputKind_HumanReadable	0
#define	OutputKind_Binary			1

typedef unsigned short TraceOpcode;
#define TraceOpcode_End				0
/** PMXTracer Opcode: Attributes Opcode (Module, Code, Kind).	*/
#define TraceOpcode_Attributes		1
#define TraceOpcode_IndentLevel		2
#define TraceOpcode_ProcessID		3
#define TraceOpcode_ThreadID		4
#define TraceOpcode_Data			5
#define TraceOpcode_TimeStamp		6
#define TraceOpcode_AppName			128

/*	------------------------------------------------------------------- */
#define SPRINTF_BUFFER_SIZE	4096

/*	------------------------------------------------------------------- */
typedef struct _TTraceSendParams
{
	TraceKind	itsKind;
	TraceIdent	itsIdent;
	unsigned char*		itsData;
	size_t		itsDataLen;
	unsigned char		itsIndentLevel;
	unsigned long		itsTimeStamp;
} TTraceSendParams;

/*	------------------------------------------------------------------- */
typedef struct _TTracer
{
	int		itsfOpened;
	int		itsfFirstTrace;
	/* 
	itsOutputKind must be set to:
		OutputKind_HumanReadable if this tracer does not need binary marshalling.
		OutputKind_BinaryPaquet if this tracer needs binary marshalling.
	*/
	int		itsOutputKind;
	int		(*itsOpenProc)(struct _TTracer* aTracer, const char* config);
	void		(*itsCloseProc)(struct _TTracer* aTracer);
	void		(*itsSendProc)(struct _TTracer* aTracer, TTraceSendParams* aParams);
	long		(*itsDisconnectedProc)(struct _TTracer* aTracer);
	void		(*itsDestroyProc)(struct _TTracer* aTracer);
} TTracer;

/*	------------------------------------------------------------------- */
extern TTracer*			gTracer;
extern int				gfTracesActivated;
extern int				gTracesInited;
extern CRITICAL_SECTION	gSendCS;

/*	------------------------------------------------------------------- */
void Trace_SendText(TTraceSendParams* aParams, const char* aFormat,  va_list anArgs);

/*	------------------------------------------------------------------- */
void Trace_Send(TTraceSendParams* aParams);
void Trace_Close(TTracer* aTracer);

/*	------------------------------------------------------------------- */
#ifdef __cplusplus
	}
#endif /* __cplusplus */

/*	------------------------------------------------------------------- */
#endif /* TrHelp_h */
