/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <traces_main.c>
 *
 * @brief Main file for traces library
 *
 ******************************************************************************/
	
#include <dlfcn.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <unistd.h>
#include <ctype.h>

#include <sys/time.h>
#include <hiker/traces.h>

#include <hiker/fail.h>

#include "traces_conf.h"
#include "traces_helper.h"

#include "traces_init.h"
#include "traces_fac.h"



/*	------------------------------------------------------------------- */
/* Prototypes			*/
static void prv_traceinit2(unsigned long anInstance, const char* aTracerKind, const char* config);


/*	------------------------------------------------------------------- */
/* Get value from env, set localhost if failed 									*/
/*	------------------------------------------------------------------- */
static int  tracer_init_open_default(void)
{
	char*   targetP= NULL;


	/* targetP will be set to the output and stderr if there 's no conf file */
	targetP= alp_prv_traces_conf_GetOutput();
	if (targetP != NULL)
       {		/* Now, process targetP */

		char*   offsetP;
		char *ptemp;

		ptemp = strdup(targetP);

              /* Search for type */
              offsetP = strstr(ptemp, ":");
              if (!offsetP)
              {	/* Init traces */
                	prv_traceinit2( getpid(), ptemp, NULL);
              }
              else
              {	/* Init traces */
                	*offsetP = 0;
                	prv_traceinit2(getpid(), ptemp, offsetP + 1);
              }

		if (ptemp)
		  	free(ptemp);
       }
exitfct:
//	if (targetP)
//		free(targetP);
	return (gTracer!=NULL);
}


/*	------------------------------------------------------------------- */
/* This is the real TraceInit													*/
/*	------------------------------------------------------------------- */
static void prv_traceinit2(unsigned long anInstance, const char* aTracerKind, const char* config)
{
	alp_trace_close();
	if (!gTracer)
	{
		InitializeCriticalSection(&gSendCS);
		InstanciateTracer(anInstance,aTracerKind);
	}

	if (gTracer)
	{
		InitTraces(config);
//		alp_prv_trace_outputTL(0xFFFF0000, "Traces version: %4X started ---\n" , TRACE_VERSION );	
	}
}

/*	------------------------------------------------------------------- */
/* Provides a filtering check, according to configuration file 						*/
/*	------------------------------------------------------------------- */

#define MAJOR_FILTER 	(int) 0xFFFF0000
#define FILTER_ARRAY_SIZE	(int) 512

static int g_aAllowedClasses[FILTER_ARRAY_SIZE]={0,0};
static int g_Items_in_list = -1;


extern int  alp_prv_gb_settings_initialize(void);

static int  alp_prv_filter_check(int aModule)
{
	int i;
	int j ;
//	int jRet = 0;	/* default Not OK */

	//Fill the array ...if needed !
	if (-1 == alp_prv_gb_settings_initialize())	// read conf file if needed 
	{	
		return 0;
	}

	//Init ok
	if (g_Items_in_list == -1)			//load or reload needed...
	{
		g_Items_in_list = alp_prv_Fill_Filter(g_aAllowedClasses, FILTER_ARRAY_SIZE);
	}
	
	//Check the value
	if ((g_Items_in_list <=0) || (g_Items_in_list > FILTER_ARRAY_SIZE))
		return 0; 		// No display if empty or error

	//Parse the array
	for ( j = 0; (j < g_Items_in_list); j++)
	{
		if (((aModule  &  MAJOR_FILTER ) == g_aAllowedClasses[j]) ||
		 	 (MAJOR_FILTER == g_aAllowedClasses[j]))		//Means ALL out !!!
			return 1;
	}
	return 0 ;
}



/* ================ APIs ================================= */

/******************************************************************************
 *	alp_trace_setoutput
 *****************************************************************************/
/*! This API allows to select the default output where traces will be send. This setting is global  	
	 to system.
		
@param[in]		aTracerKind			kind of trace ("tcp","file", "stderr", "null",...)
@param[in]		config				Additionnal settings

@return			
*/
void alp_trace_setoutput(const char* aTracerKind, const char* config)
{
	char workBuffer[255];

	ALP_FAIL_FATAL_IF(aTracerKind == 0, "alp_trace_setoutput: Buffer pointer is null","");

	/* Maybe, extend control to each kind of tracer */
	if (config != 0)
		sprintf(workBuffer,"%s:%s",aTracerKind,config);
	else
		sprintf(workBuffer,"%s",aTracerKind);
		
	alp_prv_traces_conf_SetOutput(workBuffer);
}


/******************************************************************************
 *	alp_trace_getoutput
 *****************************************************************************/
/*! Retrieve the current output settings.
		
@param[in]		nBufferLen			buffer length
@param[in]		aOutput				A pointer to a buffer

@param[out]		aOutput				A pointer to a buffer filled with output setting

@return			
*/
void alp_trace_getoutput( char* aOutput,  int nBufferLen)
{
	char *pTemp;
	ALP_FAIL_FATAL_IF(aOutput == 0, "alp_trace_getoutput: Buffer pointer is null","");

	pTemp = alp_prv_traces_conf_GetOutput() ;
	if (pTemp != NULL)
		strncpy(aOutput, pTemp , nBufferLen);
	else 
		*aOutput=0;
	return ;
}


/******************************************************************************
 *	alp_trace_init
 *****************************************************************************/
/*! Initialize the traces library with output string.
		
@param[in]		anInstance			An instance ID to the process
@param[in]		aTracerKind			kind of trace ("tcp","file", "stderr", "null",...)
@param[in]		config				Additionnal settings

@return
*/
void alp_trace_init(unsigned long anInstance, const char* aTracerKind, const char* config)
{
	alp_trace_close();
	
	if (aTracerKind != NULL)				//if tracerkind specified, change the output ...
	{
		alp_trace_setoutput(aTracerKind, config);
	}

	if (!tracer_init_open_default())	//Call to get the settings
		return;
}


/******************************************************************************
 *	alp_trace_close
 *****************************************************************************/
/*! Close the trace library 
		
@param[in]		
@return			
*/
void alp_trace_close(void)
{

	alp_prv_traces_conf_Close();

	if (!gTracer)
		return;

	CloseTraces();
	alp_trace_reload();  /* This forces a reload */
	
	gTracer->itsDestroyProc(gTracer);
	DeleteCriticalSection(&gSendCS);
	gTracer = 0;
}

/******************************************************************************
 *	alp_trace_local_add
 *****************************************************************************/
/*! Add a trace module ID that will be used locally.
		
@param[in]		traceModule			Add a trace module ID that will be used locally
@return			
*/
void alp_trace_local_add(int traceModule)
{
	if (alp_prv_filter_check( traceModule) != 1)		//not already there
	{
		if (g_Items_in_list < FILTER_ARRAY_SIZE)
			g_aAllowedClasses[g_Items_in_list++]= traceModule;
	}
	return ;
}


/******************************************************************************
 *	alp_trace_local_remove
 *****************************************************************************/
/*! Remove a trace module ID that will be used locally
		
@param[in]		traceModule			Remove a trace module ID that will be used locally

@return			
*/
void alp_trace_local_remove(int traceModule)
{
	int i;

	if (g_Items_in_list <= 0)
		return;
		
	for (i = 0; i < g_Items_in_list; i++)
	{
		if (g_aAllowedClasses[i] == traceModule)
		{
			g_aAllowedClasses[i]  = 0;	//empty one
			break;
		}
	}
	//We don't decrement g_Items_in_list, because we don't care of "array hole"
	return ;
}


/******************************************************************************
 *	alp_trace_reload
 *****************************************************************************/
/*! Force a reload of configuration file.
		
@param[in]		
@return			
*/
void alp_trace_reload(void)
{
	extern  int g_Items_in_list ;
	g_Items_in_list = -1;
}

/*	------------------------------------------------------------------- */
/* API - Obsolete. Not used anymore */
/*	------------------------------------------------------------------- */
long alp_trace_getoutputport_status(void)
{
	if (!gTracer)
		return 1;

	return gTracer->itsDisconnectedProc(gTracer);
}

/*	------------------------------------------------------------------- */
/* API - Obsolete. Not used anymore */
/*	------------------------------------------------------------------- */
void alp_trace_get_capabilities(char *buffer, size_t* len)
{
	char caps[] =	"kind=tcp,name=Palm OS Reporter,paramdescr=Target:,paramdefval=localhost,autoconnect=1\0"
					"kind=file,name=Text file,paramdescr=File name:,paramdefval=traces.txt,autoconnect=0\0"
					"kind=syslog,name=Syslog TraceServer.log,,autoconnect=0\0"
					"kind=xml,name=XML file,paramdescr=XML output file name:,paramdefval=traces.xml,autoconnect=0\0"
					
					"\0";
	
	if (!buffer || sizeof(caps) > *len)
	{
		*len = sizeof(caps);
	}
	else
	{
		memcpy(buffer, caps, sizeof(caps));
	}
}



/*	------------------------------------------------------------------- */
/* API called thru a macro. See ALP_TM  */
/*	------------------------------------------------------------------- */
void alp_prv_trace_outputVT(int aModule, const char* aFormatString,  va_list anArgs)
{
	TTraceSendParams	theParams;


	/* Check if filter is ok */
	if (0 == alp_prv_filter_check(aModule))
		return ;

	if (!gTracer)
	{
		if (!tracer_init_open_default())
			return;
	}

	theParams.itsKind = TraceKind_Normal;
	theParams.itsIdent = aModule << 16;
	
	/* It's valid to try sending traces after a failed Init (this can
	   happen if the Reporter was not running) */
	Trace_SendText(&theParams, aFormatString, anArgs);
}

/*	------------------------------------------------------------------- */
/* API called thru a macro. See ALP_TM  */
/*	------------------------------------------------------------------- */
void alp_prv_trace_outputT(int aModule, const char* aFormatString, ...)
{
	va_list theList;

	/* Check if displayable */
	if (0 == alp_prv_filter_check(aModule))
		return ;
	
	if (!gTracer)
	{
		if (!tracer_init_open_default())
			return;
	}

	va_start(theList, aFormatString);

	alp_prv_trace_outputVT(aModule, aFormatString, theList);

	va_end(theList);
}

/*	------------------------------------------------------------------- */
/* API called thru a macro. See ALP_TM  */
/*	------------------------------------------------------------------- */
void alp_prv_trace_outputVTL(int aModule, const char* aFormatString,  va_list anArgs)
{
	TTraceSendParams	theParams;

	/* Check if displayable */
	if (0 == alp_prv_filter_check(aModule))
		return ;
	
	if (!gTracer)
	{
		if (!tracer_init_open_default())
			return;
	}
	
	theParams.itsKind = TraceKind_EOL;
	theParams.itsIdent = aModule << 16;

	Trace_SendText(&theParams, aFormatString, anArgs);
}

/*	------------------------------------------------------------------- */
/* API called thru a macro. See ALP_TM  */
/*	------------------------------------------------------------------- */
void  alp_prv_trace_outputTL(int aModule, const char* aFormatString, ...)
{
	va_list theList;

	/* Check if displayable */
	if (0 == alp_prv_filter_check(aModule))
		return ;

	if (!gTracer)
	{
		if (!tracer_init_open_default())
			return;
	}
	
	va_start(theList, aFormatString);

	alp_prv_trace_outputVTL(aModule, aFormatString, theList);

	va_end(theList);
}


/*	------------------------------------------------------------------- */
/* API called thru a macro. See ALP_TM  */
/*	------------------------------------------------------------------- */
void alp_prv_trace_outputB(int aModule, unsigned char * aBuffer, size_t aBufferLen)
{
	TTraceSendParams	theParams;
	struct timeval tp;

		/* Check if displayable */
	if (0 == alp_prv_filter_check(aModule))
		return ;
	
	if (!gTracer)
	{
		if (!tracer_init_open_default())
			return;
	}

	if (aBufferLen > 0)
	{
		theParams.itsData = aBuffer;
		theParams.itsKind = TraceKind_Binary;
		theParams.itsIdent = aModule << 16;
		theParams.itsDataLen = aBufferLen;
		theParams.itsIndentLevel = 0;
		gettimeofday(&tp, NULL);
		theParams.itsTimeStamp = (double) tp.tv_sec + tp.tv_usec / 1000.0;

		Trace_Send(&theParams);
	}
}

/*-------- EOF -------*/
