/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <traces_syslog.c>
 *
 * @brief C File for generating syslog traces .
 *
 ******************************************************************************/

#include <string.h>
#include <syslog.h>

#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <syslog.h>

#include "traces_helper.h"
#include "traces_syslog.h"

/*	------------------------------------------------------------------- */
TTracer* TrSyslog_Create(void)
{
	TSYSLOGTracer*		theTracer;

	theTracer = malloc(sizeof(TSYSLOGTracer));

	theTracer->itsOutputKind =OutputKind_HumanReadable ;
	theTracer->itsOpenProc = TrSyslog_OpenProc;
	theTracer->itsCloseProc = TrSyslog_CloseProc;
	theTracer->itsSendProc = TrSyslog_SendProc;
	theTracer->itsDisconnectedProc = TrSyslog_DisconnectedProc;
	theTracer->itsDestroyProc = TrSyslog_DestroyProc;

	theTracer->itsfOpened = 0;

	return (TTracer*)theTracer;
}

/*	------------------------------------------------------------------- */
/* OGU DOLATER: Could use the aParam to select LOG_LOCALx				      */

int TrSyslog_OpenProc(TTracer* aTracer, const char* aParam)
{
	TSYSLOGTracer*		theTracer = (TSYSLOGTracer*) aTracer;

	openlog("Trace", LOG_CONS|LOG_PID, LOG_LOCAL7);
	
	theTracer->itsfOpened = 1;
	return aTracer->itsfOpened;

}

/*	------------------------------------------------------------------- */
void TrSyslog_CloseProc(TTracer* aTracer)
{
	if (!aTracer->itsfOpened)
		return;
	aTracer->itsfOpened = 0;
	closelog();

}

#define DUMP_LINE_LEN	16
/*	------------------------------------------------------------------- */
void TrSyslog_SendProc(TTracer* aTracer, TTraceSendParams* aParams)
{
	int		theResult;
	unsigned char*	thePtr;
	size_t	theLen;

	static char	sHexDigits[] = "0123456789ABCDEF";

	if (!aTracer->itsfOpened)
		return;
	
	if (aParams->itsKind == TraceKind_Binary)
	{
		size_t	theSize, i;
		char	theBuffer[DUMP_LINE_LEN * 3 + 2 + DUMP_LINE_LEN + 1];
		unsigned char*	theDataPtr, *theBufferPtr;

		memset(theBuffer, ' ', sizeof(theBuffer));
		theDataPtr = aParams->itsData;
		while (aParams->itsDataLen > 0)
		{
			if (aParams->itsDataLen > DUMP_LINE_LEN)
				theSize = DUMP_LINE_LEN;
			else
				theSize = aParams->itsDataLen;

			theBufferPtr = theBuffer;
			for (i = 0; i < theSize; i++, theBufferPtr += 3)
			{
				theBufferPtr[0] = sHexDigits[theDataPtr[i] >> 4];
				theBufferPtr[1] = sHexDigits[theDataPtr[i] & 0x0F];
			}

			if (theSize < DUMP_LINE_LEN)
			{
				memset(theBufferPtr, ' ', 3 * (DUMP_LINE_LEN - theSize));
				theBufferPtr += 3 * (DUMP_LINE_LEN - theSize);
			}
			theBufferPtr += 2;

			for (i = 0; i < theSize; i++)
			{
				if (theDataPtr[i] < 0x20)
					*theBufferPtr++ = '.';
				else
					*theBufferPtr++ = theDataPtr[i];
			}

			*theBufferPtr = 0;
			syslog(LOG_DEBUG, "%s",theBuffer);

			aParams->itsDataLen -= theSize;
			theDataPtr += theSize;
		}
	}
	else
	{
		syslog(LOG_DEBUG,"%s", aParams->itsData);
	}
	aTracer->itsfFirstTrace = 0;
}

/*	------------------------------------------------------------------- */
void TrSyslog_DestroyProc(TTracer* aTracer)
{
	if (aTracer->itsfOpened)
		TrSyslog_CloseProc(aTracer);
	aTracer->itsfOpened = 0;
	free(aTracer);
}

/*	------------------------------------------------------------------- */
long TrSyslog_DisconnectedProc(TTracer* aTracer)
{
	return 0;
}
