/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <traces_tcp.c>
 *
 * @brief File for generating traces over TCP (Reporter is needed as receiver).
 *
 ******************************************************************************/

#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "traces_tcp.h"

#define CT_LOCALHOST	"localhost:25998"
/*	------------------------------------------------------------------- */
#define	TCPTracer_Port		(25999-1)
#define 	TCPMaxHostLen		256


/*	------------------------------------------------------------------- k*/
TTracer* TrTCP_Create(void)
{
	TTCPTracer*		theTracer;

//	fprintf(stderr," ************* TraceLib over TCP: CREATE ******************\n");

	theTracer = malloc(sizeof(TTCPTracer));
	theTracer->itsfOpened = 0;
	theTracer->itsOutputKind = OutputKind_Binary;
	theTracer->itsOpenProc = TrTCP_OpenProc;
	theTracer->itsCloseProc = TrTCP_CloseProc;
	theTracer->itsSendProc = TrTCP_SendProc;
	theTracer->itsDisconnectedProc = TrTCP_DisconnectedProc;
	theTracer->itsDestroyProc = TrTCP_DestroyProc;

	theTracer->itsSocket = INVALID_SOCKET;
	theTracer->itsfFailed = 0;

	theTracer->theHost ="";	//If empty, not trying to connect
	

	return (TTracer*)theTracer;
}


/*	------------------------------------------------------------------- */
/*  aTracer: Context specific												      */
/*  aParam: could be null (for an already "tried and failed" call					      */
/*			or filled with param for a first call (like: "127.0.0.1:999)			      */
/*	------------------------------------------------------------------- */
int RealOpenTCPProc(TTracer* aTracer)
{
	TTCPTracer*		theTracer = (TTCPTracer*) aTracer;
	unsigned long			theIP;

	char dbgHost[128];
	strcpy(dbgHost, theTracer->theHost);

	theIP = inet_addr(theTracer->theHost);
	if (theIP == INADDR_NONE)
	{
		struct hostent*	theHE;

		theHE = gethostbyname(theTracer->theHost);
		if (theHE == 0)
		{
			fprintf( stderr, "TraceTCP: Cannot GetHostByName: %s\n",theTracer->theHost);
			return 0;
		}
		theIP = *(unsigned long*)theHE->h_addr;
	}

	theTracer->itsIP = theIP;
	theTracer->itsPort = htons(theTracer->thePort);

	theTracer->itsSocket = socket(AF_INET, SOCK_STREAM, 0);	
	if (theTracer->itsSocket != INVALID_SOCKET)
	{
		struct sockaddr_in	theSin;

		theSin.sin_family = AF_INET;
		theSin.sin_port = theTracer->itsPort;
		theSin.sin_addr.s_addr = theTracer->itsIP;
		if (connect(theTracer->itsSocket, (struct sockaddr*)&theSin, sizeof(struct sockaddr_in)) == -1)
		{
			close(theTracer->itsSocket);
			theTracer->itsSocket = INVALID_SOCKET;
		}
		else
		{
			theTracer->itsfOpened = 1;
		}
	}

	theTracer->itsfFailed = 0;
	return aTracer->itsfOpened;
}


/*	-----------------------------------------------------------	*/
/* aParam is like "127.0.0.1" or "aaa.bbb.com" or "127.0.0.1:1625" 		*/
/*	-----------------------------------------------------------	*/
int TrTCP_OpenProc(TTracer* aTracer, const char* aParam)
{
	TTCPTracer*		theTracer = (TTCPTracer*) aTracer;
	char*			theHost;
	unsigned short			thePort;
	char*			theParam;
	unsigned long			theIP;
	char* 			thePtr;
	int 				iRet; 


	/* First call (or reset a previous one) */
	theTracer->thePort = 0;			/*Default value */
	thePort = 0;
	theTracer->theHost = NULL;			/* Default value */
	theHost = NULL;

	
	//Fix the aParam if needed:
	//aParam should not be NULL but could be empty if output set to "tcp:"
	//if NULL or empty, try PALMTRACE_TARGET first otherwise "localhost"
	if (aParam == NULL || aParam[0] == '\0')
		aParam = getenv("PALMTRACE_TARGET");		/* try environment first */
	if (aParam == NULL || aParam[0] == '\0')
		theParam = strdup(CT_LOCALHOST);			/* else set to localhost */
	else
		theParam = strdup(aParam);
	
	/* Now, check for the port */
	theHost = (char*)theParam;	/* Get the host */
	thePtr = strchr(theHost, ':');  /* Look for the port */
	if (thePtr != NULL)
	{
		*thePtr++ = 0;
		thePort = atoi(thePtr);		
	}
	/* nothing specified, get the default */
	if (thePort == 0)
		thePort = TCPTracer_Port;		//Set default port if not specified

	/* And save the host */
	theTracer->theHost= strdup( theHost);
	theTracer->thePort = thePort;

	iRet =  RealOpenTCPProc( aTracer);

	/* Free memory */
	free(theParam);
	
	return iRet;
}


/*	------------------------------------------------------------------- */
void TrTCP_CloseProc(TTracer* aTracer)
{
	/* Clean Memory */
	if (((TTCPTracer*)aTracer)->theHost != NULL)
		free(((TTCPTracer*)aTracer)->theHost);
	
	((TTCPTracer*)aTracer)->theHost ="";	//If empty, not trying to connect

	if (!aTracer->itsfOpened)
		return;

	/* Close the link */
	close(((TTCPTracer*)aTracer)->itsSocket);
	((TTCPTracer*)aTracer)->itsSocket = INVALID_SOCKET;
	
	aTracer->itsfOpened = 0;
}


/*	------------------------------------------------------------------- */
static void MySendData(TTracer* aTracer,  unsigned char *thePtr, size_t	theLen)
{
	int		theResult;
	
	while (theLen > 0)
	{
		theResult = send(((TTCPTracer*)aTracer)->itsSocket, thePtr, theLen, 0);
		if (theResult == SOCKET_ERROR)
		{
			((TTCPTracer*)aTracer)->itsfFailed = 1;
			break;
		}
		theLen -= theResult;
		thePtr += theResult;
	}
}

/*	------------------------------------------------------------------- */
void TrTCP_SendProc(TTracer* aTracer, TTraceSendParams* aParams)
{

	unsigned char*	thePtr;
	size_t	theLen;
	unsigned char       dbgByte;
	static 	unsigned char* bMemory=NULL;
	static	size_t  wLen =0;
	unsigned char 	*bPos;




	/* Don't send anything if the connection isn't up and not tries occurs*/
	dbgByte = ((TTCPTracer*)aTracer)->theHost [0];
	thePtr =((TTCPTracer*)aTracer)->theHost ;
	
	//Opened ?.....
	if (aTracer->itsfOpened)
	{
		/*...now use the new opened connection */
		thePtr = aParams->itsData;
		theLen = aParams->itsDataLen;
		
		MySendData(aTracer, thePtr,theLen);

		aTracer->itsfFirstTrace = 0;
	}
}


/*	------------------------------------------------------------------- */
void TrTCP_DestroyProc(TTracer* aTracer)
{
	if (aTracer->itsfOpened)
		TrTCP_CloseProc(aTracer);
	free(aTracer);
}


/*	------------------------------------------------------------------- */
long TrTCP_DisconnectedProc(TTracer* aTracer)
{
	return !aTracer->itsfOpened || ((TTCPTracer*)aTracer)->itsfFailed;
}
