/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 * @file <traces_xml.c>
 *
 * @brief  C file for generating XML traces. 
 *
 ******************************************************************************/

#include <unistd.h>
#include <netdb.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <syslog.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "traces_helper.h"
#include "traces_xml.h"

/*	------------------------------------------------------------------- */
TTracer* TrXML_Create(void)
{
	TXMLTracer*		theTracer;

	theTracer = malloc(sizeof(TXMLTracer));

	theTracer->itsOutputKind =OutputKind_HumanReadable ;
	theTracer->itsOpenProc = TrXML_OpenProc;
	theTracer->itsCloseProc = TrXML_CloseProc;
	theTracer->itsSendProc = TrXML_SendProc;
	theTracer->itsDisconnectedProc = TrXML_DisconnectedProc;
	theTracer->itsDestroyProc = TrXML_DestroyProc;
	theTracer->itsfOpened = 0;

	return (TTracer*)theTracer;
}

/*	------------------------------------------------------------------- */
int  TrXML_OpenProc(TTracer* aTracer, const char* filename)
{
	((TXMLTracer*) aTracer)->itsFile = 0;
	aTracer->itsfOpened = 0;
	aTracer->itsOutputKind = OutputKind_HumanReadable;
	((TXMLTracer*) aTracer)->itsFile = fopen(filename, "a+");
	
	if (((TXMLTracer*) aTracer)->itsFile)
	{
		aTracer->itsfOpened = 1;
		fprintf(((TXMLTracer*)aTracer)->itsFile, "<?xml version=\"1.0\" ?>\n "  );
		fprintf(((TXMLTracer*)aTracer)->itsFile, "<!-- This file represents a fragment of a Trace output -->\n");
		fprintf(((TXMLTracer*)aTracer)->itsFile, "<Traces>\n");
		}
	
	return aTracer->itsfOpened;

}

/*	------------------------------------------------------------------- */
void TrXML_CloseProc(TTracer* aTracer)
{
	if (!aTracer->itsfOpened)
		return;

	fprintf(((TXMLTracer*)aTracer)->itsFile, "</Traces>\n");
	fclose(((TXMLTracer*)aTracer)->itsFile);
	((TXMLTracer*)aTracer)->itsFile = 0;
	aTracer->itsfOpened = 0;

}


/*	------------------------------------------------------------------- */
/* Build the message  													
 void FormatMyMessage (int pri, const char *fmt, va_list ap)
 {
   const char *pri_name = "?";
   const char *fac_name = "?";
   const char *pri_end;
   char  *p;
   char   tbuffer [2048];
   int    left = sizeof(tbuffer);
   int    saved_errno = errno;
   time_t t;
   

	//Get the time and format
	time (&t);
	p  = tbuffer;
	p += sprintf (p,  "<%3d>%.15s ", pri, ctime(&t)+4);
	left -= p - tbuffer;
	pri_end = 1 + strchr (tbuffer, '>');

	if (logTag)
	{
 		p += sprintf (p, logTag);
 		left -= p - tbuffer;
   	}
   	if (logStat & LOG_PID)
   	{
     		p += sprintf (p,  "[%d]", getpid());
     		left -= p - tbuffer;
   	}
   	if (logTag)
   	{
     		p += sprintf (p,  ": ");
   	  	left -= p - tbuffer;
   	}
 
   	SOCK_ERRNO (saved_errno);
   	p += sprintf (p,  fmt, ap);
   	if (*(p-1) != '\n')
  	{
     		*p++ = '\n';
     		*p = '\0';
   	}
    
     int   len = strlen (tbuffer);
     const char *err;
 
 }
*/

#define DUMP_LINE_LEN	16

/*	------------------------------------------------------------------- */
void TrXML_SendProc(TTracer* aTracer, TTraceSendParams* aParams)
{
	int		theResult;
	unsigned char*	thePtr;
	size_t	theLen;

	static char	sHexDigits[] = "0123456789ABCDEF";

	if (!aTracer->itsfOpened)
		return;
	
	if (aParams->itsKind == TraceKind_Binary)
	{
		size_t	theSize, i;
		char	theBuffer[DUMP_LINE_LEN * 3 + 2 + DUMP_LINE_LEN + 1];
		unsigned char*	theDataPtr, *theBufferPtr;

		memset(theBuffer, ' ', sizeof(theBuffer));
		theDataPtr = aParams->itsData;
		while (aParams->itsDataLen > 0)
		{
			if (aParams->itsDataLen > DUMP_LINE_LEN)
				theSize = DUMP_LINE_LEN;
			else
				theSize = aParams->itsDataLen;

			theBufferPtr = theBuffer;
			for (i = 0; i < theSize; i++, theBufferPtr += 3)
			{
				theBufferPtr[0] = sHexDigits[theDataPtr[i] >> 4];
				theBufferPtr[1] = sHexDigits[theDataPtr[i] & 0x0F];
			}

			if (theSize < DUMP_LINE_LEN)
			{
				memset(theBufferPtr, ' ', 3 * (DUMP_LINE_LEN - theSize));
				theBufferPtr += 3 * (DUMP_LINE_LEN - theSize);
			}
			theBufferPtr += 2;

			for (i = 0; i < theSize; i++)
			{
				if (theDataPtr[i] < 0x20)
					*theBufferPtr++ = '.';
				else
					*theBufferPtr++ = theDataPtr[i];
			}

			*theBufferPtr = 0;
			fprintf(((TXMLTracer*)aTracer)->itsFile, "  <DataBinary>\n");
			fprintf(((TXMLTracer*)aTracer)->itsFile, "  %s", aParams->itsData);
			fprintf(((TXMLTracer*)aTracer)->itsFile, "  </DataBinary>\n");

			aParams->itsDataLen -= theSize;
			theDataPtr += theSize;
		}
	}
	else
	{
		fprintf(((TXMLTracer*)aTracer)->itsFile, "  <DataText>\n");
		fprintf(((TXMLTracer*)aTracer)->itsFile, "  %s", aParams->itsData);
		fprintf(((TXMLTracer*)aTracer)->itsFile, "  </DataText>\n");
	}
	aTracer->itsfFirstTrace = 0;
}


#ifdef TOTO
/*	------------------------------------------------------------------- */
void TrXML_SendProcX(TTracer* aTracer, TTraceSendParams* aParams)
{
	static char	sHexDigits[] = "0123456789ABCDEF";

	if (!aTracer->itsfOpened)
		return;
	
	if (aParams->itsKind == TraceKind_Binary)
	{
		size_t	theSize, i;
		char	theBuffer[DUMP_LINE_LEN * 3 + 2 + DUMP_LINE_LEN + 1];
		unsigned char*	theDataPtr, *theBufferPtr;

		memset(theBuffer, ' ', sizeof(theBuffer));
		theDataPtr = aParams->itsData;
		while (aParams->itsDataLen > 0)
		{
			if (aParams->itsDataLen > DUMP_LINE_LEN)
				theSize = DUMP_LINE_LEN;
			else
				theSize = aParams->itsDataLen;

			theBufferPtr = theBuffer;
			for (i = 0; i < theSize; i++, theBufferPtr += 3)
			{
				theBufferPtr[0] = sHexDigits[theDataPtr[i] >> 4];
				theBufferPtr[1] = sHexDigits[theDataPtr[i] & 0x0F];
			}

			if (theSize < DUMP_LINE_LEN)
			{
				memset(theBufferPtr, ' ', 3 * (DUMP_LINE_LEN - theSize));
				theBufferPtr += 3 * (DUMP_LINE_LEN - theSize);
			}
			theBufferPtr += 2;

	/*		for (i = 0; i < theSize; i++)
			{
				if (theDataPtr[i] < 0x20)
					*theBufferPtr++ = '.';
				else
					*theBufferPtr++ = theDataPtr[i];
			}
*/
			*theBufferPtr = 0;
			fprintf(((TXMLTracer*)aTracer)->itsFile, "  <DataBinary>\n");
			fprintf(((TXMLTracer*)aTracer)->itsFile, "  %s\n", theBuffer);
			fprintf(((TXMLTracer*)aTracer)->itsFile, "  </DataBinary>\n");

			aParams->itsDataLen -= theSize;l
			theDataPtr += theSize;
		}
	}
	else
	{
		fprintf(((TXMLTracer*)aTracer)->itsFile, "  <DataText>\n");
		fprintf(((TXMLTracer*)aTracer)->itsFile, "  %s", aParams->itsData);
		if (aParams->itsKind == TraceKind_EOL)
			fprintf(((TXMLTracer*)aTracer)->itsFile, "\n");
		fprintf(((TXMLTracer*)aTracer)->itsFile, "  </DataText>\n");
	}
}
#endif
/*	------------------------------------------------------------------- */
void TrXML_DestroyProc(TTracer* aTracer)
{
	if (aTracer->itsfOpened)
		TrXML_CloseProc(aTracer);
	free(aTracer);
}

/*	------------------------------------------------------------------- */
long TrXML_DisconnectedProc(TTracer* aTracer)
{
	return !aTracer->itsfOpened;
}
