/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <hiker/volumesvcs.h>
#include <hiker/notifymgr.h>
#include <hiker/traces.h>

void usage()
{
	printf("%s",
			"usage: alpvsutil <cmd> <args>\n"
			"                info - return volume info array\n"
			"                fstypes - return supported fstypes\n"
			"                mount <udi>\n"
			"                umount <udi>\n"
			"                bind <name> <path> - do mount --bind name path\n"
			"                makefs <udi> <name> <fstype> - initialze the device\n"
			"                rename <udi> <name> - rename the volume\n"
			"                monitor - watch volume servcies notifcations\n"
			"                Monitor - watch volume servcies notifcations, do info after each\n");
	exit(-1);
}

alp_status_t do_info()
{
	alp_status_t as;
	AlpVolumeSvcsFSinfo *e;
	GArray *fsinfo;
	guint i;

	as = alp_volumesvcs_fsinfo_array( &fsinfo );
	if (as != ALP_STATUS_OK) {
		printf("alp_volumesvcs_fsinfo_array failed with error:%x\n",as);
	} else {
		if (fsinfo->len == 0)
			printf("nothing is mounted\n");
		else for (i=0; i<fsinfo->len; i++) {
			e = &g_array_index( fsinfo, AlpVolumeSvcsFSinfo, i);
			printf("\nentry: %d\n",i);
			printf("         udi: %s\n",e->udi);
			printf("    dev_path: %s\n",e->dev_path);
			printf("     fs_path: %s\n",e->fs_path);
			printf("        name: %s\n",e->name);
			printf("        slot: %s\n",e->slot);
			printf("vendor_model: %s\n",e->vendor_model);
			printf("        uuid: %s\n",e->uuid);
			printf("   card_type: %s\n",e->card_type);
			printf("       flags: %s %s %s\n",
					e->flags & ALP_VOLUMESVCS_FLAGS_BINDMOUNT?"BindMount":" ",
					e->flags & ALP_VOLUMESVCS_FLAGS_HIDDEN?"Hidden":" ",
					e->flags & ALP_VOLUMESVCS_FLAGS_PARTITION?"Partition":" ");
			printf(" total space: %llu\n", e->total_space);
			printf("  used space: %llu\n", e->used_space);
			printf(" total files: %llu\n", e->total_files);
			printf("  used files: %llu\n", e->used_files);
		}
		alp_volumesvcs_fsinfo_array_free( fsinfo );
	}
	return as;
}

static int cnt=0;
static int info_flag=0;
int callback( const char * notifyType, void * details, int length,
              int * handled, const char * appID, void * userData)
{
	char *dt;
	time_t t;

	t = time(NULL);
	dt = ctime(&t);
	printf("%3d: %s - %s\t%s\n",cnt++, notifyType, dt, (char*)details );
	if (info_flag)
		do_info();
	return 0;
}

void do_monitor(int info)
{ 
	GMainLoop *loop;
	alp_status_t as;

	info_flag = info;
	loop = g_main_loop_new( NULL, FALSE);

	as = alp_notify_register_server(ALP_NOTIFY_EVENT_VOLUME_ADDED, callback, NULL,0,0);
	ALP_FAIL_FATAL_IF(as != ALP_STATUS_OK, "alp_notify_register_server for %s returned %x",
			ALP_NOTIFY_EVENT_VOLUME_ADDED, as);
	as = alp_notify_register_server(ALP_NOTIFY_EVENT_VOLUME_REMOVED, callback, NULL,0,0);
	ALP_FAIL_FATAL_IF(as != ALP_STATUS_OK, "alp_notify_register_server for %s returned %x",
			ALP_NOTIFY_EVENT_VOLUME_REMOVED, as);
	as = alp_notify_register_server(ALP_NOTIFY_EVENT_DEVICE_ADDED ,callback, NULL,0,0);
	ALP_FAIL_FATAL_IF(as != ALP_STATUS_OK, "alp_notify_register_server for %s returned %x",
			ALP_NOTIFY_EVENT_DEVICE_ADDED, as);
	as = alp_notify_register_server(ALP_NOTIFY_EVENT_DEVICE_REMOVED, callback, NULL,0,0);
			ALP_FAIL_FATAL_IF(as != ALP_STATUS_OK, "alp_notify_register_server for %s returned %x",
			ALP_NOTIFY_EVENT_DEVICE_REMOVED, as);

	g_main_loop_run(loop);
	exit(0); // can't get here
}

int main(int argc, char *argv[])
{
	GPtrArray *fstypes;
	alp_status_t as;
	guint i;

	if (argc < 2)
		usage();
	if (!strcmp(argv[1],"info")) {
		as = do_info();
	} else if (!strcmp(argv[1], "monitor")) {
		do_monitor(0); // doesn't return
	} else if (!strcmp(argv[1], "Monitor")) {
		do_monitor(1); // doesn't return
	} else if (!strcmp(argv[1], "fstypes")) {
		as = alp_volumesvcs_fstypes( &fstypes );
		if (as != ALP_STATUS_OK) {
			printf("alp_volumesvcs_fstypes failed with error:%x\n",as);
		} else if (fstypes->len == 0) {
			printf("alp-volumesvcs_fstyeps returend no supported filesystems\n");
		} else for (i=0; i<fstypes->len; i++) {
			printf("%d: %s\n",i, (char*)g_ptr_array_index( fstypes, i));
		}
	} else if (!strcmp(argv[1], "mount")) {
		if (argc < 3)
			usage();
		as = alp_volumesvcs_mount( argv[2] );
		if (as != ALP_STATUS_OK) {
			printf("alp_volumesvcs_mount failed with error:%x\n",as);
		}
	} else if (!strcmp(argv[1], "umount")) {
		if (argc < 3)
			usage();
		as = alp_volumesvcs_umount( argv[2] );
		if (as != ALP_STATUS_OK) {
			printf("alp_volumesvcs_umount failed with error:%x\n",as);
		}
	} else if (!strcmp(argv[1], "bind")) {
		if (argc < 4)
			usage();
		as = alp_volumesvcs_bind( argv[2], argv[3] );
		if (as != ALP_STATUS_OK) {
			printf("alp_volumesvcs_bind failed with error:%x\n",as);
		}
	} else if (!strcmp(argv[1], "makefs")) {
		if (argc < 5)
			usage();
		as = alp_volumesvcs_makefs( argv[2], argv[3], argv[4] );
		if (as != ALP_STATUS_OK) {
			printf("alp_volumesvcs_makefs failed with error:%x\n",as);
		}
	} else if (!strcmp(argv[1], "rename")) {
		if (argc < 4)
			usage();
		as = alp_volumesvcs_rename_volume( argv[2], argv[3] );
		if (as != ALP_STATUS_OK) {
			printf("alp_volumesvcs_rename failed with error:%x\n",as);
		}
   	} else {
		usage();
	}
	return 0;
}
