/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright � 2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/

#include <stdio.h>
#include <string.h>

#include <hiker/types.h>
#include <hiker/bundlemgr.h>
#include <hiker/global_settings.h>
#include <glib.h>

#include "demo_global.h"

/* demo_global_changer's purpose in life is to change the global settings
 * created by demo_global_setter to see if that app is notified of the change. */

int alp_main(int argc, char** argv)
{
	/* Just to show both methods, create one AlpGlobalSettingsValue on
	 * the stack and allocate the other. */
	AlpGlobalSettingsValue intValueSetting = GLOBAL_SETTINGS_VALUE_INITIALIZER;
	AlpGlobalSettingsValue* strValueSetting = NULL; 
	AlpGlobalSettingsContext* context = NULL;
	char tempBuf[100];
	char* redStr = "red";
	char* greenStr = "green";
	char* colorStr;
	int favoriteNum;
	alp_status_t colorStatus, numberStatus;
	alp_status_t appStatus = ALP_STATUS_OK;

	/* Early exit if global settings just not working. */
	if (alp_global_settings_init() != ALP_STATUS_OK)
		return 1;
	if (alp_global_settings_open(&context) != ALP_STATUS_OK)
		return 2;

	/* First try to read existing global values. It it finds
	 * them, it will change them. */
	colorStatus = alp_global_settings_get_key_string_value(context, FAV_COLOR_KEY, 
														tempBuf, sizeof(tempBuf));
	if (colorStatus == ALP_STATUS_OK)
	{
		/* We need a transaction if we are going to do writing. */
		colorStatus = alp_global_settings_begin_transaction(context);
		if (colorStatus == ALP_STATUS_OK)
		{
			colorStr = (strcmp(tempBuf, redStr)) ? redStr : greenStr;
			
			colorStatus = alp_global_settings_set_key_string_value(context, 
													FAV_COLOR_KEY, colorStr);        
			if (colorStatus == ALP_STATUS_OK)
				alp_global_settings_complete_transaction(context);
			else
				alp_global_settings_cancel_transaction(context);
		}
	}

	/* Treat each global as a separate item. */
	numberStatus = alp_global_settings_get_key_int_value(context, FAV_NUM_KEY, 
														&favoriteNum);
	if (numberStatus == ALP_STATUS_OK)
	{
		/* We need a transaction if we are going to do writing. */
		numberStatus = alp_global_settings_begin_transaction(context);
		if (numberStatus == ALP_STATUS_OK)
		{
			favoriteNum++;
			
			numberStatus = alp_global_settings_set_key_int_value(context, 
													FAV_NUM_KEY, favoriteNum);        
			if (numberStatus == ALP_STATUS_OK)
				alp_global_settings_complete_transaction(context);
			else
				alp_global_settings_cancel_transaction(context);
		}
	}

	/* Only come here after settings opened. */
	alp_global_settings_close(context);

	/* If we had an error, return the first bad status. */
	if (colorStatus != ALP_STATUS_OK)
		appStatus = colorStatus;
	else if (numberStatus != ALP_STATUS_OK)
		appStatus = numberStatus;
	
	return appStatus;
}

