/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright � 2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/

#include <stdio.h>

#include <hiker/types.h>
#include <hiker/bundlemgr.h>
#include <hiker/global_settings.h>
#include <glib.h>


/* Use the bundle manager to find the full path to a file in the app bundle. */
static gchar* get_full_bundle_path(const gchar* filename)
{
	gchar* pname = NULL;
	
	/* If alp_bundle_ref_ro_pathname succeeds, it will create a string
	 * that will have to be freed later. */
	pname = alp_bundle_ref_ro_pathname(alp_bundle_ref_me(), filename); 
	if (g_file_test(pname, G_FILE_TEST_EXISTS))
	{
		return pname;
	}
	else
	{
	    return NULL;
	}
}


int init_demo()
{
	gchar *bundleFilename = "/data/default.xml";
	gchar *defaultsPath;
	FILE *fp;
	char line[100];
	int lcount = 0;
	int error = 0;

	/* Start by using the bundle manager. */
	defaultsPath = get_full_bundle_path(bundleFilename);
	
	if (defaultsPath != NULL)
	{
		printf("Found the defaults file at: %s\n", defaultsPath);
	
		/* Open the file.  If NULL is returned there was an error */
		if ((fp = fopen(defaultsPath, "r")) == NULL) 
		{
			printf("Error Opening File.\n");
			error = 2;
	 	}
	 	else
	 	{
	 		/* Demonstrate it is actually the correct file. */
			while (fgets(line, sizeof(line), fp) != NULL) 
			{
				/* Print the line number and data */
				lcount++;
				printf("Line %d: %s", lcount, line);  
			}
		
			fclose(fp);  /* Close the file */
		}
	
		/* Free the string that was allocated for us by alp_bundle_ref_ro_pathname. */
		if (defaultsPath)
			g_free(defaultsPath);
	}
	else
	{
		printf("Bundle Manager couldn'f find: %s\n", bundleFilename);
		error = 1;
	}
	
	return error;
}


int alp_main(int argc, char** argv)
{
	printf("Running ALP open source middleware sample.\n");
	
	init_demo();

	return 0;
}

