//
//    Copyright © 2002-2006, ACCESS Systems Americas, Inc. All rights reserved.
//
/********************************************************************

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright © 2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <stdlib.h>
#include <string.h>
#include <dlfcn.h>
#include <unistd.h>
#include <signal.h>
#include <errno.h>
#include <pthread.h>
#include <glib-object.h>
#include <time.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>
#include <hiker/traces.h>
#include <hiker/bundlemgr.h>
#ifdef ALP
#include <alp/alpmenubar.h>
#endif
#include <hiker/pdk/appmgr_pdk.h>
#include <hiker/prv/bundlemgr_prv.h>
#ifdef ALP
#include <alp/prv/postgtkinitsurgery.h>
#endif
#include <hiker/prv/nlpd_bits.h>
#ifdef ALP
#include <alp/max.h>
#endif
#include <hiker/traces.h>
#ifdef ALP
#include <alp/prv/theming_prv.h>
#endif
#include <hiker/prv/timestamp.h>

#define USE_SYSTEM_BUG_WORKAROUND 1
#define OVERWRITE_ENV 1
#define NLPD_PATH "/opt/alp/bin/alp_NativeProcessLaunchpad_D"
#define HOME "/var/home"


// Static variable used to determine when Matchbox is
// running so our Matchbox theme can be set without
// getting clobbered 
static gboolean gMatchboxThemeIsSet = FALSE;

//
// This is not used for just relaunching.  (Actually it 
// does not handle relaunch because it does not make sense
// for it.  It just exits())  See comments above function definition below
// for details.
//
static void relaunch_handler(int argc, char *argv[], gpointer cbData);
static void init_pre_fork_app_state(void);

//
// These are just used to give argv and envp file scope
//
static char **gArgv = NULL;
static char **gEnvp = NULL;
static char *Alp_appname= NULL;

int main(int argc, char *argv[], char *envp[])
{
 
	// Set the global version of these.
	gArgv = argv;
	gEnvp = envp;
   
       // THis is a special case of a --direct launch	
	if ((argc >= 3) && (strcmp(argv[1], "--direct") == 0)) 
	{
		gArgv = argv + 2;
		signal(SIGCHLD, SIG_IGN);
		init_pre_fork_app_state();
		// third parameter is 0 to indicate direct launch 
		ALP_TM(TL(ALP_CLASS_NLPD, "NLPD::main() Jumping into handle a --direct")); 
		relaunch_handler(argc - 2, gArgv, NULL); 
		exit(0); // we should never get here
	}
    
	//
	// This makes sure that we are only running one copy of 
	// the cookie cutter (ie = kill -9)
	//
	alp_prv_signal_all( NLPD_PATH, 9);	

	GMainLoop *Main_Loop;
	Main_Loop = g_main_loop_new(NULL, TRUE);

	AlpAppContext *cntx;
	cntx = alp_app_setup("bar:alp.system.nlp", ALP_APP_LAUNCH_FLAG_LAUNCHPAD, "bar");

	//
	//  Unit test start this up without an appserver.  This will return
	//  a null cntxt
	if (cntx != NULL)
	{
		ALP_TM(TL(ALP_CLASS_NLPD, "NLPD::main() set a relaunch handler" ));
		alp_app_set_relaunch_handler( relaunch_handler, cntx );
       
        
	} else 
	{
		ALP_TM(TL(ALP_CLASS_NLPD, "NLPD::main() Did NOT set a relaunch handler"));
	}
	
#ifdef ALP
	// Move to background, allowing boot sequence to continue.
	// We're now ready to handle requests.
	daemon(FALSE,TRUE);
#else
	if (fork()) 
	    exit(0);	
#endif	
	// Ignore children so as to avoid zombies
	signal(SIGCHLD, SIG_IGN);

	ALP_TM(TL(ALP_CLASS_NLPD, "Daemonize done"));
	// Here we do the pre-fork() initialization work.
	// This is all stuff we initialize once, and then each launched app
	// inherits our already-initialized state.
	init_pre_fork_app_state();
	
	g_main_loop_run(Main_Loop);

	// Shutdown and die
	alp_app_destroy(cntx, 0);

	ALP_TM(TL(ALP_CLASS_NLPD, "main done"));

	return(0);	
}


// Here we do the pre-fork() initialization work.
// This is all stuff we initialize once, and then each launched app
// inherits our already-initialized state.
// Many things are dependent on the app being launched, and cannot be initialized here.
static void init_pre_fork_app_state(void)
{
	alp_status_t status;

	g_type_init();
#ifdef ALP	
	// Install MAX specific stuff in GTK+.
	alp_max_gtk_init();

	// initialize the theme data structures and
	// load the current theme
	status = alp_theme_init(TRUE);
#endif
}

static void relaunch_handler(int argc, char *argv[], gpointer cbData)
{                                                       /* cbData is 0 if we launch using --direct */
    
	// argv must exist and it must be terminated by a pointer
	// to null.  argc should equal the number of non-null values
	// in the array.
	if (( argv == NULL ) || (argc == 0)) {
		ALP_TM(TL(ALP_CLASS_NLPD, "Whoopse we got some bogus relaunch arguments dude"));
		return;
	}
#ifdef ALP
	// Verify that the matchbox theme has been set up.  This is
	// necessary due to a race condition where xinit has not 
	// initialized matchbox prior to alp_theme_init.  Once we 
	// verify that it has been set up, we don't need to do
	// it ever again.
	if (gMatchboxThemeIsSet == FALSE) {
		gMatchboxThemeIsSet = alp_theme_verify_matchbox_theme();
	}
#endif	

	// If we are doing a direct launch we do not want to fork otherwise we do
	// cbData == NULL means we did a --direct
	pid_t pid;
	unsigned int security_id = ALP_APP_DEFAULT_SECURITY_ID;
	//if (cbData != NULL)
	{	
		pid = fork();
		// if we are not the child then we reopen bundle_manager and we are done
		if (pid != 0) {
			if (cbData == NULL) _exit(0);
			return;
		}
	if (cbData != NULL) {	
		// Before we get rid of the context, obtain the app's security ID.
		security_id = alp_app_get_security_id((AlpAppContext *)cbData);
		
		// Close our copy of the parent's (NLPD's) app context.
		alp_app_abandon((AlpAppContext *)cbData); 
		timestamp("We have forked", "NLPD", LAUNCHLOG);
	  }
	}
	
	Alp_appname = argv[0];
	
	/*
		It seems there's a problem with initializing BundleMgr in the parent process,
		and then bringing it down & up again to fork a new process for an app.
		It seems to break debugging under some circumstances, and the app exits
		without GDB even being notified.
		Until that issue is resolved, we have a "quick fix", which reverts to the
		old behavior of only ever initializing BundleMgr in the child process.
		This has the unfortunate side affect of leaving all threads except the main
		one running as root.
		
		Also, even before this "quick fix", it seems there was an *extra*
		call to alp_bundle_init() (see below - "Reopen the bundle").
		I'm removing this too.
	*/


	if (alp_set_security_parameters(security_id) == -1)
	{
		ALP_TM(TL(ALP_CLASS_NLPD, "Whoop setting up security failed"));
		exit(0);
	}
	
	timestamp("has set security parameter", "NLPD", LAUNCHLOG );	
	
	// Rename the process.  We rename it to the last n
	// characters of the name.  'n' is the length of the
	// command line.
	//
	alp_prv_rename_process(argv[0], gArgv); 
	timestamp("has renamed processes", "NLPD", LAUNCHLOG);	

	// Initialize bundlemgr, because we need it
	alp_bundle_init();
        
	// get a handle for pulling out meta data	
	AlpBundle package = alp_bundle_by_name(argv[0]);
	AlpBundleRef ref = alp_bundle_open(package);
	if (ref == NULL) {
		ALP_TM(TL(ALP_CLASS_NLPD, "Whoop ref NULL"));
		exit(-1);
	}

	timestamp("has alp_bundle_open", "NLPD", LAUNCHLOG );	

	// Get the r/w dir (ie /var/home/<packagename>), and set
	// BUNDLE_HOME, and HOME to point to it.
	gchar *path = alp_bundle_rw_pathname( package, ".", TRUE);
	setenv("BUNDLE_HOME", path, OVERWRITE_ENV);
	setenv("HOME", HOME, OVERWRITE_ENV) ;
	
	// Also set the current working dir to the bundle's R/W dir in /var.
	chdir(path);
	g_free(path);
	
	// Set BUNDLE_ROOT to point to the bundle's RO area (localized rsrc directory).
	path = alp_bundle_ref_ro_pathname(ref, ".");
	setenv("BUNDLE_ROOT", path, OVERWRITE_ENV);
	g_free(path);
	
	// Set up ALP_APPID since we said we'd add it:
	setenv("ALP_APPID", Alp_appname, OVERWRITE_ENV);
	
	//
	// Set the new environment for the new application.  "Default_env" is a
	// package attribute that contains environment variables to be set before
	// running the user code.
	//
	gchar *NewEnv = NULL;
	NewEnv = alp_bundle_property_value( package, "application", 0, "Default_env");
	if ((NewEnv != NULL) && (strlen(NewEnv) != 0))
		alp_prv_SetUpEnv((char *)NewEnv);
	g_free(NewEnv);

	// retrieve <entry> element of <application> in package manifest, if present
	gchar * entry_name = alp_bundle_property_value(package, "application", 0, "entry");

	// look up entry URL relative to the named package
	// It's ok if entry_name is NULL.
	GClosure * ptr = alp_bundle_acquire_entrypoint(entry_name, ref);
		
	timestamp("has bundle properties and entrypoint(close)", "NLPD", LAUNCHLOG);	
	// Immediately fall over if we couldn't load 
	if (ptr == NULL) {
		ALP_TM(TL(ALP_CLASS_NLPD, "LINK ERROR, ABORTING IN NATIVE LAUNCHPAD, TRYING TO LOAD %s\n",
				       	entry_name ));
		timestamp("We quit due to stupidity", "NLPD", LAUNCHLOG);	
	}
	g_free(entry_name);
	entry_name = NULL;
	
	
	guint32 launchFlags = 0;
	launchFlags = alp_app_parse_launch_flags(ALP_APP_LAUNCH_FLAG_NONE, argc, argv);
	timestamp("We parse launch flags ", "NLPD", LAUNCHLOG);	

	AlpAppContext *cntx = NULL;
	cntx = alp_app_setup(argv[0], launchFlags, NULL);
	alp_app_set_default_context(cntx);
#ifdef ALP		
	
	char *propVal = alp_bundle_property_value(package, ALP_BUNDLE_PROPERTY_APPLICATION, 
									  0, ALP_APP_PROP_ENABLE_THEME);
	if (propVal != NULL && strcmp(propVal, "false") == 0)
	{
		alp_disable_theming();
	}
	else
	{
		// We need to set up the theme update proc for the new app context.
		// Even though ours is already set, they aren't propogated automatically or anything.
		alp_app_set_theme_update_proc(cntx, alp_theme_switch_cb);
	}
#endif

	//
	// the closure will keep a ref on the appropriate package
	//
	alp_bundle_close(ref);
		
	// set the application ref using the ref actually stored in the closure
	alp_prv_bundle_set_application_ref(ALP_BUNDLE_ENTRYPOINT_CLOSURE_REF(ptr));
	timestamp("has set application ref", "NLPD", LAUNCHLOG );	
		
	
	// Set up a couple of Data structures	
	GValue return_value;
	GValue params[3];
		
	memset(&return_value, '\0', sizeof(return_value));
	memset(&params[0], '\0', sizeof(GValue));
	memset(&params[1], '\0', sizeof(GValue));
	memset(&params[2], '\0', sizeof(GValue));
		
	g_value_init(&return_value, G_TYPE_INT);
	g_value_init(&params[0], G_TYPE_INT);
	g_value_init(&params[1], G_TYPE_POINTER);
	g_value_init(&params[2], G_TYPE_POINTER);
		
	g_value_set_int(&params[0], argc);
	g_value_set_pointer(&params[1], argv);
	g_value_set_pointer(&params[2], gEnvp);
		
		
	//
	// If we have a debug launch.  We block until we 
	// get a SIGUSR1 from the dubugger.  Sometimes 
	// sigwait returns for other reasons thus the loop.
	//
	if ((launchFlags & ALP_APP_LAUNCH_FLAG_DEBUG) != 0)
	{
		sigset_t newset;
		sigset_t oldset;
		const int MAXERRS = 10;
		sigemptyset( &newset );
		sigaddset( &newset, SIGUSR1 );
		pthread_sigmask( SIG_SETMASK, &newset, &oldset);
		int output = 0;
		int count = 0;
		while((sigwait(&newset, &output) != 0) && (output != SIGUSR1))
			if (count++ > MAXERRS)
				exit(EXIT_FAILURE);
		pthread_sigmask(SIG_SETMASK, &oldset, NULL);  // restore the signal mask.
	}


	timestamp("Jumping into app.so ", "NLPD", LAUNCHLOG);
	g_closure_invoke(ptr, &return_value, 3, params, NULL);

	timestamp("cleaning up", "NLPD", LAUNCHLOG);
	// Clean up and send back the return value
	int rvalue = g_value_get_int(&return_value);	
	alp_prv_bundle_set_application_ref(NULL);
	g_closure_unref(ptr);
	alp_app_destroy(cntx, rvalue);
	alp_prv_bundle_fini();

	exit(0);  // We are the child process so we need to go away
}
