//
//    Copyright © 2002-2006, ACCESS Systems Americas, Inc. All rights reserved.
//
/********************************************************************

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

// Just a little helper function because
// C strings are a pain in the tail
// This returns a new string created from 
// a portion of the old string.  
// len must be <= strlen(str).  
static char *alp_prv_strcpy2( const char *str, size_t len)
{
	char *rvalue = (char *)malloc(sizeof(char) * (len+1));
	size_t count;
	for(count = 0; count<len; count++)
		rvalue[count] = str[count];
	rvalue[len] = '\0';
	return(rvalue);
}
#define OVERWRITE 1

// 
// Given a null terminated string of the form
// [NAME=VALUE; | NAME;]+ this will set each
// environment NAME = VALUE or in the case of 
// NAME; it will unset that environment variable.
//
// Example "HOME=/home/david;DISTCC;SHELL=/usr/bin/sh"
// Will set $HOME = /home/david 
// Will unset $DISTCC and will
// set $SHELL = /usr/bin/sh
// 
void alp_prv_SetUpEnv(char *env)
{
	size_t inc = 0;
	char *symbol = NULL;
	char *value = NULL;
	char terminators[] = ";=\0";

	while((env != NULL) && (env != '\0'))
	{
		inc = strcspn( env, terminators );
		symbol = alp_prv_strcpy2( env, inc );
		env += inc;

		//
		// This is the case of xxx\0
		// It is malformed so we ignore it
		//
		if ( env[0] == '\0' )
		{
			free(symbol);
			break;
		}

		//
		// This is the case of xxx;
		// This means unset xxx
		//
		if ( env[0] == ';' )
		{
			if ((symbol != NULL) && (strlen(symbol) > 0))
				unsetenv(symbol);
			env++;
			free(symbol);
			continue;
		}

		inc = strcspn( ++env, terminators );
		value = alp_prv_strcpy2( env, inc );
		env += inc;
		if (env[0] == ';')
			env++;
		if ((symbol != NULL) && (strlen(symbol) > 0))
			setenv(symbol, value, OVERWRITE);

		free(symbol);	
		free(value);
	}
}
