//
//    Copyright © 2002-2006, ACCESS Systems Americas, Inc. All rights reserved.
//
/********************************************************************

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/bundlemgr.h>
#include <hiker/security.h>
#include <hiker/prv/lsm_cmd.h>
#include <hiker/prv/apas.h>
#include <hiker/prv/nlpd_bits.h>
#include <hiker/pdk/appmgr_pdk.h>


//
// This turns a bundle into a security_id.  This requires that the
// bundle manager connection is already open (How else would you get
// the bundle).
//
// The security_id has no purpose except to be used by
// alp_set_security_parameters() 
//

unsigned int alp_get_security_id( AlpBundle bundle ) 
{
	// First if the env variable  use_lsm is not set
	// to 1 do nothing. This is normally set in alp_init.sh


	char *use_lsm = NULL;
	use_lsm = getenv( "NLPD_USE_LSM" );

	if (use_lsm == NULL) {
		ALP_TM(TL(ALP_CLASS_NLPD, "NLPD_USE_LSM not set") );
		return(ALP_APP_DEFAULT_SECURITY_ID);
	}

	if (strcmp( "TRUE", use_lsm) != 0)  {
	      ALP_TM(TL(ALP_CLASS_NLPD, "NLPD_USE_LSM set to %s", use_lsm) );
		 return(ALP_APP_DEFAULT_SECURITY_ID);
	}

	ALP_TM(TL(ALP_CLASS_NLPD, "NLPD_USE_LSM set to TRUE") );


	//
	// Next we generate the package ID.  This needs to be scanned out of 
	// a string returned from bundle manager.   If it is not there then 
	// we give it a sentinal value
	//

	char *security_string = NULL;
	unsigned int security_id = 0;
	security_string = alp_bundle_metadata_value( bundle, "security_handle");
	if (security_string == NULL) 
	{
		ALP_TM(TL(ALP_CLASS_NLPD, 
			  "No security handle (SPF's package id) found\n"));
		security_id = ALP_APP_DEFAULT_SECURITY_ID ;
	}
	else // We have a security_string 
	{
		int inputs_read = 0;
		inputs_read = sscanf(security_string, "%u", &security_id);
		if (inputs_read != 1)
		{
			ALP_TM(TL(ALP_CLASS_NLPD,  
				"Invalid package id %s\n", security_string));
			security_id = ALP_APP_DEFAULT_SECURITY_ID ;
		}
		g_free(security_string);
	}

	return(security_id);
}

//
// This acutally does the work of setting the security for the 
// running process.  With non-NPTL threads (ie what we have right now)
// this will only set permission of the current thread and it's children
// Not pre-existing threads of this processes.  (This is due to the
// brain dead way we do pthread_create() i.e. clone()).
//
// If alp_set_security_parameters is disabled it return 0.
// If it is enabled and there is an error it returns -1.  If
// everything works it returns a 1.
//
int alp_set_security_parameters( unsigned int security_id ) 
{ 	 

	// First if the env variable  use_lsm is not set
	// to 1 do nothing. This is normally set in alp_init.sh


	char *use_lsm = NULL;
	use_lsm = getenv( "NLPD_USE_LSM" );

	if (use_lsm == NULL) {
		ALP_TM(TL(ALP_CLASS_NLPD, "NLPD_USE_LSM not set") );
		return(0);
	}

	if (strcmp( "TRUE", use_lsm) != 0)  {
	      ALP_TM(TL(ALP_CLASS_NLPD, "NLPD_USE_LSM set to %s", use_lsm) );
	      return(0);
	}

	ALP_TM(TL(ALP_CLASS_NLPD, "NLPD_USE_LSM set to TRUE") );


	//
	// Can we connect to the SPF?
	//
	
	if ( alp_spf_ps_open() != ALP_STATUS_SPF_E_NOERR  ) {
		ALP_TM(TL(ALP_CLASS_NLPD,  
       "Whoops we were unable to open the security framework.  Is spf running?"
       		));
		return(-1);
	}

	//
	// A ALP_APP_DEFAULT_SECURITY_ID for security_id implies the bundle we're using 
	// isn't signed.  In this case, we call the 
	// alp_spf_apas_packageid_default() routine to get the default 
	// bundle id.
	//
	alp_status_t status;
	if ( security_id == ALP_APP_DEFAULT_SECURITY_ID ) {

	  	ALP_TM(TL(ALP_CLASS_NLPD,  
		   "No valid package id found, security environment not set\n"
		   ));

	  	status = alp_spf_apas_packageid_default( &security_id );

	  	ALP_TM(TL(ALP_CLASS_NLPD, "Using %08X as package id.\n",
			security_id ));
	}
 
	//
	// Get the UID associated with the bundle
	//

	uid_t uid;
	status = alp_spf_apas_packageid_uid_get(security_id, &uid);  
       if ( status != ALP_STATUS_SPF_E_NOERR) {	
		ALP_TM(TL(ALP_CLASS_NLPD, 
		"Cannot get the uid of the pacakge %i", security_id ));
		return(-1);
       }

	//
	// Get the GID associated with the bundle
	//

	gid_t gid;
	status = alp_spf_apas_packageid_gid_get(security_id, &gid);
	if ( status != ALP_STATUS_SPF_E_NOERR ) {
		ALP_TM(TL( ALP_CLASS_NLPD, "Cannot get the gid of package!"));
		return(-1);
	}


	//
	// Get the SGIDs associate with the bundle
	//      

	gid_t *sgids = NULL;
	unsigned int sgids_length = 0;
	status = alp_spf_apas_packageid_sgids_get( security_id, 
			&sgids, &sgids_length );
	if ( status != ALP_STATUS_SPF_E_NOERR ) {
		ALP_TM(TL( ALP_CLASS_NLPD, "Cannot get sgids of package!" ));
		return(-1);
	}
    
	//
	// This just shuts down the connection.  It seems like we should be
	// able to ignore errors here but without the security frame work
	// running do we really want to be running processes?
	//

	if ( alp_spf_ps_close() != ALP_STATUS_SPF_E_NOERR ) {
		ALP_TM(TL( ALP_CLASS_NLPD, 
			"Whoops unable to close connection SPF" ));
		return(-1);
	}

	// Now UID maps to the package identifier, GID maps to Policy
	// Need the PID for use in the SPF-LSM binding
	pid_t pid;
	pid = getpid();

	// open the SPF-LSM interface, and push binding command
	int dev_security_in = -1;
	dev_security_in = open(SPF_LSM_DEV_IN, O_WRONLY);
	if ( dev_security_in < 0 ) {
		ALP_TM(TL( ALP_CLASS_NLPD, 
			"Oops could not open /dev/security_in"));
		return(-1);
	}


	// set up command structure 
	Alp_lsmCmd_unit cmd_unit;

	cmd_unit.command	= ALP_LSMCMD_PID_MAP;
	cmd_unit.subject_id	= uid;
	cmd_unit.object_id1	= gid;
	cmd_unit.object_id2	= 0;
   	cmd_unit.object_id3	= 0;
   	cmd_unit.process_id	= pid;
   	cmd_unit.bit_flag	= 0;
   	cmd_unit.status		= 0;

	// write and close to /dev/security_in
	int results = -1;
	results = write( dev_security_in, &cmd_unit, sizeof(Alp_lsmCmd_unit));
	if ( results == -1 ) {
		ALP_TM(TL( ALP_CLASS_NLPD, 
			"Unable to write to /dev/security_in"));
		return(-1);
	}

	if ( close(dev_security_in) != 0 ) {
		ALP_TM(TL( ALP_CLASS_NLPD, 
			"Unable to close /dev/security_in"));
		return(-1);
	}

	// Check LSM return status
	if ( cmd_unit.status != 0 )  {
		ALP_TM(TL( ALP_CLASS_NLPD, 
			"Ugh, LSM could not bind PID to Policy")); 
		return(-1);
	}

	// Set the groups (different from the group see man page)
	if ( sgids_length != 0 ) {
		if ( setgroups(	sgids_length, sgids ) == -1 ) {
			ALP_TM(TL( ALP_CLASS_NLPD, 
			  "Whoops unable to set supplemental gids"));
			return(-1);
		}
	}


	// Set THE GROUP.
	if ( setregid( gid, gid ) != 0 ) {
		ALP_TM(TL( ALP_CLASS_NLPD, 
	       	   "Whoops unable to set the gid"));
		return(-1);
	}


	// Set the User ID
	if ( setreuid( uid, uid ) != 0 ) {
		ALP_TM(TL( ALP_CLASS_NLPD, 
	       	   "Whoops unable to set the uid"));
		return(-1);
		}


	return(1);  // Yeah everything was copasetic !!
}

