//
//    Copyright © 2002-2006, ACCESS Systems Americas, Inc. All rights reserved.
//
/********************************************************************

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/

#include <hiker/prv/nlpd_bits.h>

//
// This is just a little helper function to change
// all "//" in a string into "/".  It works in place
// so no memory management issues.
//
static char *alp_prv_remove_double_slash(char *exe)
{
	if (exe == NULL)
		return(NULL);
	char *ch = NULL;
	while((ch = strstr(exe, "//")) != NULL)
	{
		while (*ch != '\0')
		{
			*ch = *(ch+1);
			ch++;
		}
		ch = NULL;
	}
	return(exe);
}

//
// This is a function that converts sym link to the file name that the
// sym link points to.
//
static char *alp_prv_convertlink( char *exe )
{
	char newpath[MAX_PATH_LEN];
	struct stat buf;
	lstat( exe, &buf);
	if (S_ISLNK(buf.st_mode))
	{
		int count = readlink(exe, newpath, MAX_PATH_LEN - 1);
		char *rvalue = (char *)malloc(sizeof(char) * (count+1));
		newpath[count] = '\0';
		strcat(rvalue, newpath);
		return(rvalue);
	} 
	return(exe);
}

//
// Given a string of the absolute path to an executeable this send
// a signal (sig) to all instances of that executeable running on
// the system.  FYI the reason we use the absolute path is to make
// sure we don't kill anything with the same name by accident.  
//
int alp_prv_signal_all(const char *exe, int sig)
{
	const char *procfs = "/proc";
	struct dirent **namelist;
	int count = -1;
	pid_t my_pid = getpid();

	// Make a copy of the string.
	char *exe_copy=malloc(sizeof(char)*(strlen(exe)+1));
	strcpy( exe_copy, exe );

	//
	// I have run in to problem where paths contain // which is valid put
	// screws up the string comparison.  So I remove them.
	
	alp_prv_remove_double_slash(exe_copy);	

	//
	// Need to find out if exe is a symlink.  If so expand it to a hard path
	//
	char *temp = alp_prv_convertlink(exe_copy);
	
	if ((count = scandir(procfs, &namelist, alp_prv_file_is_digits, alphasort))
		       	== -1)
	{
		ALP_TM(TL(ALP_CLASS_NLPD, "unable to scan /proc\n"));
		return(0);
	}

	int i;
	int killed = 0;
	for (i=0; i<count; i++)
	{
		char *current_name = alp_prv_image_path( namelist[i]->d_name );
		pid_t current_pid = (pid_t)atoi(namelist[i]->d_name);
		if (current_name != NULL) {
			if (strcmp(temp, current_name) == 0 &&
				current_pid != my_pid)
			{
				kill( current_pid, sig );
				killed++;
			}
		free(current_name);
		}
	}

	if ((temp != NULL) && (temp != exe_copy))
		free(temp);

	if (exe_copy != NULL)
		free(exe_copy);

	return(killed);
}

//
// current->d_name is the name of the entry
// in the directory.  This function returns
// 1 if the name is composed of only digits
// and the entry is a directory.  This sounds weird put
// in the /proc file system there is a directory for each
// process.  The directory name is the PID of that process.
// THere are other directories in /proc which do not 
// correspond to processes and thus this function.
//
static int alp_prv_file_is_digits( const struct dirent *current )
{
	// if it isn't a directory return 0
	if (current->d_type != DT_DIR)
		return(0);

	// if it isn't got a name return 0
	char *name = current->d_name;
	if ((name == NULL) || (name[0] == '\0'))
		return(0);

	while (name[0] != '\0')
		if ((name[0] >= '0') && (name[0] <= '9'))
			name++;
		else
			return(0);

	return(1);
}

// 
// There is a file /proc/$PID/exe which is a symlink
// that points to the binary for the $PID process. 
// Given a $PID as a string this returns the path
// to that binary.  This has a memory leak -- "buf".
// caller needs to release it.
//
static char *alp_prv_image_path(char *current )
{
	char *path = (char *)malloc(sizeof(char) * (strlen(current) + 11));
	strcpy(path, "/proc/");
	strcat(path, current);
	strcat(path, "/exe");

	// directories are very transitive so this is not
	// an error
	char *buf = (char *)malloc(sizeof(char) * (MAX_PATH_LEN +1));
	int count = -1;
	buf[0] = '\0';
	if ((count = readlink(path, buf, MAX_PATH_LEN -1)) <= 0)
	{
		free(path);
		return NULL;
	}
	buf[count] = '\0';
	free(path);
	return(buf);
}
