/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <stdlib.h>
#include <cppunit/extensions/HelperMacros.h>
#include <hiker/unit_test.h>
#include <hiker/prv/nlpd_bits.h>

class test_SetUpEnv : public CPPUNIT_NS::TestFixture
{
	CPPUNIT_TEST_SUITE( test_SetUpEnv );
	CPPUNIT_TEST( test_set1 );
	CPPUNIT_TEST( test_set2 );
	CPPUNIT_TEST( test_set3 );
	CPPUNIT_TEST( test_set4 );
	CPPUNIT_TEST( test_unset1 );
	CPPUNIT_TEST( test_unset2 );
	CPPUNIT_TEST( test_all );
	CPPUNIT_TEST_SUITE_END();

	protected:
		void test_set1();
		void test_set2();
		void test_set3();
		void test_set4();
		void test_unset1();
		void test_unset2();
		void test_all();

	public:
		void setUp();

};

ALP_ROOT_TEST_SUITE_REGISTRATION( "AP-001", CPPUNIT_REGISTRY_TYPE_AUTO );

CPPUNIT_TEST_SUITE_NAMED_REGISTRATION( test_SetUpEnv, "AP-001" );

void test_SetUpEnv::setUp()
{
	unsetenv("TESTSYMBOL1");
	unsetenv("TESTSYMBOL2");
	unsetenv("TESTSYMBOL3");
	setenv("TESTSYMBOL4", "unset", 1);
	setenv("TESTSYMBOL5", "unset", 1);
}

//
// Simple test to verify that we can set a env variable.
//
void test_SetUpEnv::test_set1()
{
	CPPUNIT_ASSERT(getenv("TESTSYMBOL1") == NULL);
	alp_prv_SetUpEnv("TESTSYMBOL1=HelloWorld;");
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL1"), "HelloWorld") == 0);
	alp_prv_SetUpEnv("TESTSYMBOL1=HelloWorld2");
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL1"), "HelloWorld2") == 0);
}

//
// This test check a larger env variables string 
//
void test_SetUpEnv::test_set2()
{
	CPPUNIT_ASSERT(getenv("TESTSYMBOL2") == NULL);
	CPPUNIT_ASSERT(getenv("TESTSYMBOL3") == NULL);
	alp_prv_SetUpEnv("TESTSYMBOL1=Fred;TESTSYMBOL2=Charlie;TESTSYMBOL3=Jose");
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL1"),"Fred") == 0);
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL2"),"Charlie") == 0);
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL3"), "Jose") == 0);
}

//
// This test some malformed set strings;
// foo=bar=bip --> $foo = bar
//
void test_SetUpEnv::test_set3()
{
	unsetenv("TESTSYMBOL1");
	unsetenv("TESTSYMBOL2");
	alp_prv_SetUpEnv("TESTSYMBOL1=TESTSYMBOL2=bip");
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL1"), "TESTSYMBOL2") == 0);
	CPPUNIT_ASSERT(getenv("TESTSYMBOL2") == NULL );
}

// This one test that export FOO= does not set $FOO to NULL
// but sets it to the empty string ""
//
void test_SetUpEnv::test_set4()
{
	unsetenv("TESTSYMBOL1");
	alp_prv_SetUpEnv("TESTSYMBOL1=");
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL1"), "") == 0);
}
//
// This tests if we can unset a variable.
//
void test_SetUpEnv::test_unset1()
{
	alp_prv_SetUpEnv("TESTSYMBOL4;");
	CPPUNIT_ASSERT(getenv("TESTSYMBOL4") == NULL );
}
//
// This tests if we can unset a list of variables
//
void test_SetUpEnv::test_unset2()
{
	setenv("TESTSYMBOL4", "bogus", 1);
	alp_prv_SetUpEnv("TESTSYMBOL4;TESTSYMBOL5;");
	CPPUNIT_ASSERT(getenv("TESTSYMBOL4") == NULL );
	CPPUNIT_ASSERT(getenv("TESTSYMBOL5") == NULL );
}
//
// This test if we can set and unset a mixed group of variables
//
void test_SetUpEnv::test_all()
{
	setenv("TESTSYMBOL1", "bogus", 1);
	setenv("TESTSYMBOL2", "bogus", 1);
	setenv("TESTSYMBOL3", "bogus", 1);
	setenv("TESTSYMBOL4", "bogus", 1);
	setenv("TESTSYMBOL5", "bogus", 1);
	alp_prv_SetUpEnv("TESTSYMBOL1;TESTSYMBOL2=;TESTSYMBOL3=HappyHappy;TESTSYMBOL4=JoyJoy;TESTSYMBOL5");
	CPPUNIT_ASSERT(getenv("TESTSYMBOL1") == NULL );
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL2"),"") == 0 );
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL3"), "HappyHappy") == 0 );
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL4"), "JoyJoy") == 0);
	CPPUNIT_ASSERT(strcmp(getenv("TESTSYMBOL5"), "bogus") == 0);
}
