/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/**
 * @file attnmgrd_prv.h
 *
 * @brief ALP attention manager private structures and defines
 *
 *****************************************************************************/

#ifndef __ALP_ATTNMGRD_PRV_H__
#define __ALP_ATTNMGRD_PRV_H__

#include <paths.h>

#include <hiker/ipc.h>
#include <hiker/attnmgr_common.h>

#define ALP_ATTNMGRD_DEF_PIDFILE	_PATH_VARRUN "attnmgr"

/* TODO: get ALERT bundle id from ruleset / config */
#define ALP_ATTNMGRD_APPID_ALERT   "bar:com.access.apps.attn_alert"
#define ALP_ATTNMGRD_APPID_SELF    "bar:com.access.services.attnmgr"
#define ALP_ATTNMGRD_NOTIFYID_SELF "attnmgrd"

/*
 * A posted alert priority at this level or lower will trigger
 * the user dialog interface.
 */
#define ALP_ATTN_PRIORITY_TRIGGER    3

extern AlpDmlH gAlpAttnMgrDDmlH;
extern bool gAlpAttnMgrDDebug;
extern bool gAlpAttnMgrDTest;

/* ipc */
typedef struct _AlpAttnConnectionEntry AlpAttnConnectionEntry;

struct _AlpAttnConnectionEntry {
	AlpConnection                  *connect;
	char                           *appID;
	struct _AlpAttnConnectionEntry *next;
};

#ifdef __cplusplus
extern "C" {
#endif

    extern alp_status_t alp_attnmgrd_db_setup     (const char *directory,
                                                   const char *filename,
                                                   bool debug);
    extern void alp_attnmgrd_db_teardown  (void);
    extern void alp_attnmgrd_ipc_setup    (const char *channel);
    extern void alp_attnmgrd_ipc_teardown (void);

    extern void alp_attnmgrd_process_post_alert (AlpConnection *connection,
                                                 AlpMessage    *message,
                                                 AlpDmlItemH    itemH,
                                                 gint           procId,
                                                 gint           userId,
                                                 gint           groupId);
    extern void alp_attnmgrd_process_update_alert (AlpConnection *connection,
                                                   AlpMessage    *message,
                                                   AlpDmlItemH   itemH);
    extern void alp_attnmgrd_process_delete_alert (AlpConnection *connection,
                                                   AlpMessage    *message,
                                                   AlpDmlItemH    itemH);
#ifdef __cplusplus
}
#endif

#endif /* __ALP_ATTNMGRD_PRV_H__ */
