/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*! @file exgmgrd.h 
 *
 * @brief Private headers for exgmgrd.c
 *
*/
#ifndef EXGMGRD_H_
#define EXGMGRD_H_

#include "exgtransport.h"
#include <hiker/prv/exgmgr_prv.h>
#include <hiker/bundlemgr.h>

// Different ways a handler can be invoked
#define ALP_EXG_DAEMON_HANDLER_INVOKE_BY_BUNDLE 1
#define ALP_EXG_DAEMON_HANDLER_INVOKE_BY_NOTIFICATION 2

// AlpExgMgrdHandlerInfoType contains the info about one handler.
typedef struct _AlpExgMgrdHandlerInfoType AlpExgMgrdHandlerInfoType;
struct _AlpExgMgrdHandlerInfoType {
	char *itsVerb;
	char *itsLocalizedVerb;
	char *itsMIMEType;
	int itsInvokeType;		// launch package or invoke notification
	union
	{
		struct _byBundle
		{
			char *itsBundleName;
		} byBundle;
		
		struct _byNotification
		{
			char *itsNotificationType;
		} byNotification;
	} params;
	int itsLocalOnly;
	int itsSerializeMe;
	int itsMustBeUnique;
	int itsAcceptGenericInvocation;
	AlpExgMgrdHandlerInfoType *itsNext;
	AlpExgMgrdHandlerInfoType *itsPrev;	// not sure I need to double link...
};

// AlpExgMgrdTransportInfoType contains info about each open transport.
typedef struct _AlpExgMgrdTransportInfoType AlpExgMgrdTransportInfoType;
struct _AlpExgMgrdTransportInfoType {
	AlpBundleRef itsBundleRef;
	gchar *itsNameId;		// internal name, non localizable, as used in the transport params
	void *itsLibRef;
	int itsInited;	// 1 when itsTransportInitFunc has been called successfully 
	alp_exg_transport_init_func itsTransportInitFunc;
	alp_exg_transport_deinit_func itsTransportDeinitFunc;
	alp_exg_transport_dispatch_result_func itsTransportDispatchResultFunc;
	alp_exg_transport_cancel_func itsTransportCancelFunc;
};

// States used to run the dispatcher
#define ALP_EXG_DAEMON_STATE_WAIT_CXN_MESSAGE 1
#define ALP_EXG_DAEMON_STATE_SEND_FINAL_REPLY_STATUS 2
#define ALP_EXG_DAEMON_STATE_SEND_RCV_REQ_REPLY_STATUS 3
#define ALP_EXG_DAEMON_STATE_SEND_REQUEST_DATA_SIZE 4
#define ALP_EXG_DAEMON_STATE_SEND_REQUEST_DATA 5
#define ALP_EXG_DAEMON_STATE_SEND_REQUEST_FD_PRESENT 6
#define ALP_EXG_DAEMON_STATE_WAIT_HANDLER_REPLY 7
#define ALP_EXG_DAEMON_STATE_WAIT_REQUEST_DATA_SIZE 8
#define ALP_EXG_DAEMON_STATE_WAIT_REQUEST_DATA 9
#define ALP_EXG_DAEMON_STATE_WAIT_REQUEST_FD_PRESENT 10
#define ALP_EXG_DAEMON_STATE_WAIT_REQUEST_FD 11
#define ALP_EXG_DAEMON_STATE_SEND_GENERIC_VERBS_REPLY_STATUS 12
#define ALP_EXG_DAEMON_STATE_SEND_GENERIC_VERBS_SIZE 13
#define ALP_EXG_DAEMON_STATE_SEND_GENERIC_VERBS 14

// AlpExgMgrdDispatchInfotype contains the info about each incoming
// request waiting to be executed by its handler. Once the handler has connected,
// the entry is removed from the list, and the request and transportId are stored in the connection
typedef struct _AlpExgMgrdDispatchInfoType AlpExgMgrdDispatchInfoType;
struct _AlpExgMgrdDispatchInfoType {
	int itsDispatchInProgress;	// 0 = waiting in queue, 1 = handler invoked.
	guint itsTimeoutSourceId;	// gsource implementing the handler connection timeout.
	AlpExgRequest itsRequest;	// The request we received
	int itsTransportId;		// Id of the transport which delivered the request to us
	AlpExgMgrdHandlerInfoType *itsLastHandlerInfoP;	// not NULL if handler did not require to be unique
	char *itsTmpDatafile;		// used to save data between invocation of multiple handlers.
	AlpExgMgrdDispatchInfoType *itsNext;
	AlpExgMgrdDispatchInfoType *itsPrev;
};

// The daemon use a list of AlpExgDaemonCxnState entries to manage connections from clients or handlers.
typedef struct _AlpExgDaemonCxnState AlpExgDaemonCxnState;
struct _AlpExgDaemonCxnState {
	int isUsed;
	GPollFD itsPollFD;
	int itsState;
	char itsTempBuf[sizeof(AlpExgMgrdMessageOrReplyType)];
	size_t itsTempBufUsed;
	int itsOpcode;
	union
	{
		struct _clientCxn	// exgmgr lib client connection
		{
			// nothing more than the initial message is sent for client connections.
		} clientCxn;
		
		struct _handlerCxn	// an invoked handler is now running and has connected to the daemon to receive the request
		{
			AlpExgMgrdDispatchInfoType *itsDispatchInfoP;	// The dispatch info that was associated to this handler
			char *itsInfoP;		// pointer to the request info field. will contain the parameters returned by handler
			size_t itsInfoLen2;
			alp_status_t itsReplyStatus;	// the status returned by the handler and forwarded to the transport.
		} handlerCxn;
		
		struct _queryGenericVerbsCxn	// this connection must return a list of generic verbs
		{
			size_t itsDataSize;		// size of itsData
			char *itsData;	// int entrycount + list of verbs and verb descriptions.
		} queryGenericVerbsCxn;
	} params;
	
};

#endif /*EXGMGRD_H_*/
