/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#ifndef NOTIFY_MGR_SERVER_PRV_H
#define NOTIFY_MGR_SERVER_PRV_H

#include <pthread.h>
#include <time.h>
#include <hiker/ipc.h>

//! maximum number of active broadcasts
#define ALP_NOTIFY_MAX_BROADCASTS 100

//! maximum number of registrations
#define ALP_NOTIFY_MAX_REGISTRATIONS 5000

// internal structures for registrations

typedef struct _AlpNotifyRegisterEntry AlpNotifyRegisterEntry;
struct _AlpNotifyRegisterEntry {
	char * notifyType;
        char * appID;
        void * callback;
        void * userData;        
        int priority;
        int flags;
        int pid;
        struct _AlpNotifyRegisterEntry * next;
};

typedef struct _AlpNotifyRegisterHead AlpNotifyRegisterHead;
struct _AlpNotifyRegisterHead {
        int hash;
        AlpNotifyRegisterEntry * entry;
        struct _AlpNotifyRegisterHead * next;
};

// internal structures for broadcasts

typedef struct _AlpNotifyNotificationEntry AlpNotifyNotificationEntry;
struct _AlpNotifyNotificationEntry {
	char * notifyType;
	char * details;
        char * appID;
	int length;
	int queue;
	int flags;
	int busy;
	int handled;
	int pid;
	pthread_t thread;
	AlpConnection * connect;
	AlpNotifyRegisterEntry * entry;
	struct _AlpNotifyNotificationEntry * next;
};

// internal structure for server connections

typedef struct _AlpNotifyConnectionEntry AlpNotifyConnectionEntry;
struct _AlpNotifyConnectionEntry {
	AlpConnection * connect;
	char * appID;
	struct _AlpNotifyConnectionEntry * next;
};

// internal structure for active broadcasts

typedef struct _AlpNotifyBroadcastEntry AlpNotifyBroadcastEntry;
struct _AlpNotifyBroadcastEntry {
	int pid;
	time_t seconds;
	char * appID;
	struct _AlpNotifyBroadcastEntry * next;
};

// internal structure for active package installs

typedef struct _AlpNotifyPackageEntry AlpNotifyPackageEntry;
struct _AlpNotifyPackageEntry {
	int count;
	char * appID;
	struct _AlpNotifyPackageEntry * next;
};

// register
extern int alp_notify_register_add(AlpNotifyCmdRegister * request, int pid);
extern AlpNotifyRegisterHead * alp_notify_register_get(const char * notifyType);
extern void alp_notify_register_cleanup(void);
extern int alp_notify_register_remove(AlpNotifyCmdUnregister * request);
extern int alp_notify_register_remove_all(const char * appID, int server);
extern int alp_notify_register_is_defined(const char * notifyType, const char * appID);

// broadcast
extern int alp_notify_broadcast_get_pending(void);
extern int alp_notify_broadcast_add(AlpNotifyCmdBroadcast * request, AlpConnection * connect);
extern int alp_notify_broadcast_done(AlpNotifyCmdNotificationDone * done);
extern void alp_notify_broadcast_do_pending(void);
extern void alp_notify_broadcast_fixup(AlpNotifyRegisterEntry * old, AlpNotifyRegisterEntry * new);
extern void alp_notify_broadcast_cleanup(void);
extern int alp_notify_broadcast_is_active(AlpNotifyRegisterEntry * entry);
extern void alp_notify_broadcast_set_inactive(const char * appID);
extern void alp_notify_broadcast_cancel_sync(AlpConnection * connect);


// active notification
extern int alp_notify_add(const char * appID, int pid);
extern void alp_notify_remove(const char * appID);
extern int alp_notify_get_pid(const char * appID);
extern void alp_notify_cleanup(void);

// ipc
extern int alp_notify_ipc_send(const char * appID, void * buf, int length);
extern void alp_notify_ipc_start_timer(void);
extern void alp_notify_ipc_init(void);

// base64
extern void alp_notify_base64_encode(unsigned char src[], unsigned char dst[], int length);
extern int alp_notify_base64_encode_size(int length);

// package
extern int alp_notify_package_install(const char * appID);
extern int alp_notify_package_uninstall(const char * appID);
extern int alp_notify_package_cleanup(const char * notifyType);
extern void alp_notify_package_remove(const char * appID);
extern void alp_notify_package_init(void);

// database
extern int alp_notify_database_init(void);
extern int alp_notify_database_add(AlpNotifyRegisterEntry * entry);
extern int alp_notify_database_remove(AlpNotifyRegisterEntry * entry);
extern int alp_notify_database_restore(void);

#endif
