/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
// Encoding from RFC 3548 among others 
static const unsigned char sEncodeBase64[]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";

// Encodes the one, two, or third bytes into four bytes (padded if necessary)
// Assumes length is at least one byte.
static void alp_prv_notify_base64_encode_block(unsigned char src[], unsigned char dst[], int length)
{
	// first character
	dst[0] = sEncodeBase64[ src[0] >> 2 ];

	// second character
	if (length > 1)
		dst[1] = sEncodeBase64[ (src[0] & 0x03) << 4 | (src[1] & 0xf0) >> 4 ];
	else
		dst[1] = sEncodeBase64[ (src[0] & 0x03) << 4 ];

	// third character
	if (length > 1)
		dst[2] = sEncodeBase64[ (src[1] & 0x0f) << 2 | (src[2] & 0xc0) >> 6 ];
	else
		dst[2] = '=';

	// fourth character
	if (length > 2)
		dst[3] = sEncodeBase64[ src[2] & 0x3f ];
	else
		dst[3] = '=';
}

// Encode a buffer of data
// Caller must allocate the dst buffer
void alp_notify_base64_encode(unsigned char src[], unsigned char dst[], int len)
{
	while (len > 0) {
		alp_prv_notify_base64_encode_block(src,dst,len < 3 ? len : 3);
	        src += 3;
		dst += 4;
		len -= 3;
	}
}

// Calculates how space is required to encode the data
// Rounds up to the next four bytes.
int alp_notify_base64_encode_size(int len)
{
	len = (len * 4) / 3;
	if (len % 4) len += 4 - len % 4;
	return len;
}
