/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/notifymgr.h>
#include <hiker/prv/notifymgr_prv.h>
#include <hiker/pdk/notifymgr_pdk.h>
#include "notify_mgr_server_prv.h"

#include <hiker/bundlemgr.h>
#include <hiker/pdk/appmgr_pdk.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

// notification entries keeps track of active broadcasts
static AlpNotifyNotificationEntry * sBroadcastList = 0;
static int sBroadcastListSize = 0;
static int sBroadcastEnabled = 0;

// allocate new notification entry
static AlpNotifyNotificationEntry * alp_prv_notify_broadcast_new(AlpNotifyCmdBroadcast * request, AlpConnection * connect)
{
	AlpNotifyNotificationEntry * entry;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_prv_notify_broadcast_new\n");
#endif
	entry = (AlpNotifyNotificationEntry *)malloc(sizeof(AlpNotifyNotificationEntry));
	if (!entry) return 0;

	entry->notifyType = malloc(strlen(request->notifyType)+1);
	if (!entry->notifyType) {
		free(entry);
		return 0;
	}

	if (request->length) {
		entry->details = malloc(request->length);
		if (!entry->details) {
			free(entry->notifyType);
			free(entry);
			return 0;
		}
	} else {
		entry->details = 0;
	}

	entry->appID = malloc(strlen(request->appID)+1);
	if (!entry->appID) {
		free(entry->notifyType);
		if (entry->details) free(entry->details);
		free(entry);
		return 0;
	}

	strcpy(entry->notifyType,request->notifyType);
	if (request->length) memmove(entry->details,request->details,request->length);
	strcpy(entry->appID,request->appID);
	entry->length = request->length;
	entry->queue = request->queue;
	entry->flags = request->flags;
	entry->busy = 0;
	entry->handled = 0;
	entry->pid = 0;
	entry->connect = connect;
	entry->entry = 0;
	entry->next = 0;

	sBroadcastListSize++;

#ifdef ALP_NOTIFY_DEBUG_MESG
	printf("NotifyMgr: broadcast add: notifyType=%s queue=%d length=%d active=%d\n",entry->notifyType,entry->queue,entry->length,sBroadcastListSize);
#endif
	return entry;
}

// free a notification entry
static void alp_prv_notify_broadcast_free(AlpNotifyNotificationEntry * entry)
{
	sBroadcastListSize--;
#ifdef ALP_NOTIFY_DEBUG_MESG
	printf("NotifyMgr: broadcast remove: notifyType=%s queue=%d length=%d active=%d\n",entry->notifyType,entry->queue,entry->length,sBroadcastListSize);
#endif
	if (entry->connect && entry->flags & ALP_NOTIFY_BROADCAST_FLAG_SYNC) {
		AlpMessage * message;
		message = alp_ipc_message_create();
		alp_ipc_message_pack_start(message,0);
		alp_ipc_message_pack_int32(message,entry->handled);
		alp_ipc_message_pack_end(message);
		alp_ipc_connection_send(entry->connect,message);
		alp_ipc_message_destroy(message);
	}
	if (entry->pid == -1) pthread_join(entry->thread,0);
	free(entry->notifyType);
	if (entry->details) free(entry->details);
	free(entry->appID);
	free(entry);
}

// get the number of active broadcasts
int alp_notify_broadcast_get_pending(void)
{
#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_broadcast_get_pending\n");
#endif
	return sBroadcastListSize;
}

// exported entrypoint to add a new broadcast
int alp_notify_broadcast_add(AlpNotifyCmdBroadcast * request, AlpConnection * connect)
{
	AlpNotifyRegisterHead * head;
	AlpNotifyNotificationEntry * entry, * new;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_broadcast_add\n");
#endif
	// check packet size
	if (request->size != sizeof(AlpNotifyCmdBroadcast))
		return ALP_NOTIFY_ERR_INVALID_CMD;

	// check packet data
	if (request->zero) return ALP_NOTIFY_ERR_INVALID_CMD;
	if (strlen(request->appID) > ALP_NOTIFY_MAX_APPID_SIZE)
		return ALP_NOTIFY_ERR_INVALID_APPID;
	if (strlen(request->notifyType) > ALP_NOTIFY_MAX_NOTIFY_SIZE)
		return ALP_NOTIFY_ERR_INVALID_NOTIFYTYPE;
	if (!request->queue)
		return ALP_NOTIFY_ERR_INVALID_QUEUE;
	if (request->length > ALP_NOTIFY_MAX_DETAILS_SIZE)
		return ALP_NOTIFY_ERR_INVALID_LENGTH;

	// check for special broadcasts
	if (!strcmp(request->notifyType,ALP_NOTIFY_EVENT_BUNDLE_NAME_REMOVED)) {
		char * appID;
		AlpNotifyEventBundleNameRemoved * details;

		// get the appID
		details = (AlpNotifyEventBundleNameRemoved *)request->details;
		appID = details->name;

		// handle the package uninstall
		if (appID) alp_notify_package_uninstall(appID);
	}
	if (!strcmp(request->notifyType,ALP_NOTIFY_EVENT_BUNDLE_NAME_CHANGED)) {
		char * appID;
		AlpNotifyEventBundleNameChanged * details;

		// get the appID
		details = (AlpNotifyEventBundleNameChanged *)request->details;
		appID = details->name;

		// handle the package uninstall
		if (appID) alp_notify_package_uninstall(appID);
		
		// install is handled directly by bundle manager
	}

	/* open source specific addition */
	// check if boot broadcast
	if (!strcmp(request->notifyType,ALP_NOTIFY_EVENT_NOTIFY_BOOT)) sBroadcastEnabled = 1;
	/* open source specific addition */
	//
	// check if there are any registered client for this notification
	head = alp_notify_register_get(request->notifyType);
	if (!head) return ALP_NOTIFY_ERR_NOT_REGISTERED;

	// check if over the limit
	if ((sBroadcastListSize >= ALP_NOTIFY_MAX_BROADCASTS) && !(request->flags & ALP_NOTIFY_BROADCAST_FORCE) && sBroadcastEnabled)
		return ALP_NOTIFY_ERR_TOO_MANY_BROADCASTS;

	// check if boot broadcast
	if (!strcmp(request->notifyType,ALP_NOTIFY_EVENT_NOTIFY_BOOT)) sBroadcastEnabled = 1;

	// allocate new broadcast entry
	new = alp_prv_notify_broadcast_new(request,connect);
	if (!new) return ALP_NOTIFY_ERR_OUT_OF_MEMORY;

	// check if no pending notifications
	if (!sBroadcastList) {

		// add a new notification
		sBroadcastList = new;
		sBroadcastList->entry = head->entry;

	} else {

		// check if add at the beginning or the end
		if (request->flags & ALP_NOTIFY_BROADCAST_FLAG_IMMEDIATE) {

			// add at the beginning of the list
			entry = sBroadcastList;
			sBroadcastList = new;
			new->next = entry;
			new->entry = head->entry;

		} else {

			// add at the end of the list
			entry = sBroadcastList;
			while (entry->next) entry = entry->next;

			// add a new notification
			entry->next = new;
			new->entry = head->entry;
		}
	}
	return ALP_OK;
}

// sync app launch
static int alp_prv_notify_broadcast_launch_app_sync(AlpNotifyNotificationEntry * entry)
{
	char * argv[3],buf[50];
	AlpNotifyLaunch launch;
	int size;
	alp_status_t err;
	AlpAppContext * context = 0;

	// setup launch parameters
	memset(&launch,0,sizeof(AlpNotifyLaunch));
	strcpy(launch.notifyType,entry->notifyType);
	launch.handled = entry->handled;
	launch.length = entry->length;
	strcpy(launch.appID,entry->appID);
	if (entry->length) memmove(launch.details,entry->details,entry->length);

	// allocate argv parameters
	argv[0] = argv[1] = argv[2] = 0;
	argv[0] = malloc(strlen(ALP_APP_NOTIFY)+1);
	if (!argv[0]) {
		err = ALP_NOTIFY_ERR_OUT_OF_MEMORY;
		goto Done;
	}
	size = strlen(ALP_NOTIFY_LAUNCH_DETAILS) + alp_notify_base64_encode_size(sizeof(AlpNotifyLaunch)) + 1;
	argv[1] = malloc(size);
	if (!argv[1]) {
		err = ALP_NOTIFY_ERR_OUT_OF_MEMORY;
		goto Done;
	}
	argv[2] = malloc(strlen(ALP_NOTIFY_LAUNCH_TYPE)+ALP_NOTIFY_MAX_NOTIFY_SIZE+1);
	if (!argv[2]) {
		err = ALP_NOTIFY_ERR_OUT_OF_MEMORY;
		goto Done;
	}

	// setup argv parameters
	strcpy(argv[0],ALP_APP_NOTIFY);
	memset(argv[1],0,size);
	strcpy(argv[1],ALP_NOTIFY_LAUNCH_DETAILS);
	alp_notify_base64_encode((unsigned char *)&launch,&argv[1][strlen(ALP_NOTIFY_LAUNCH_DETAILS)],sizeof(AlpNotifyLaunch));
	sprintf(argv[2],"%s%s",ALP_NOTIFY_LAUNCH_TYPE,entry->notifyType);

	// call the appserver to launch
	sprintf(buf,"com.access.notify_server.pid=%d",getpid());
	context = alp_app_setup(buf, ALP_APP_LAUNCH_FLAG_NONE, NULL);
	err = alp_app_launch_full(context,entry->entry->appID,3,argv,&entry->pid);
	if (context) alp_app_destroy(context,0);

Done:
	if (argv[0]) free(argv[0]);
	if (argv[1]) free(argv[1]);
	if (argv[2]) free(argv[2]);

	return err;
}

// async app launch
static void * alp_prv_notify_broadcast_launch_app_async(void * ptr)
{
	alp_status_t err;
	AlpNotifyNotificationEntry * entry;

	// setup
	entry = (AlpNotifyNotificationEntry *) ptr;

	// launch the app
	err = alp_prv_notify_broadcast_launch_app_sync(entry);
	
	// remove bogus entry
	if (entry->entry) alp_notify_remove(entry->entry->appID);

	// check if the app was actually launched
	if (!err) {
		// need to save the real pid
		if (entry->entry) alp_notify_add(entry->entry->appID,entry->pid);
	} else {
		// not busy anymore
		entry->busy = 0;
	}

	// done
	return (void *) 0;
}

// launch the app to send the notification
static int alp_prv_notify_broadcast_launch_app(AlpNotifyNotificationEntry * entry)
{
#ifndef ALP_NOTIFY_ASYNC_ALP_APP_LAUNCH
	return alp_prv_notify_broadcast_launch_app_sync(entry);
#else
	// set fake pid 
	entry->pid = -1;

	// set busy (while the thread is running)
	entry->busy = 1;
	alp_notify_add(entry->entry->appID,entry->pid);

	// create a private to call alp_app_launch
	pthread_create(&entry->thread,NULL,alp_prv_notify_broadcast_launch_app_async,entry);

	// return launch successfully
	return 0;
#endif
}
// send a notification either an app (via the appserver) or server (via IPC)
static int alp_prv_notify_broadcast_send(AlpNotifyNotificationEntry * entry, int * pid)
{
#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_broadcast_send\n");
#endif
#ifdef ALP_NOTIFY_DEBUG_MESG
	if (entry->entry->callback) {
		printf("NotifyMgr: notify send server: notifyType=%s pid=%d\n",entry->notifyType,entry->entry->pid);
	} else {
		printf("NotifyMgr: notify send app: notifyType=%s appID=%s\n",entry->notifyType,entry->entry->appID);
	}
#endif
	// check if broadcasting to a server or to an application
	if (entry->entry->callback) {
		// broadcasting to a server

		AlpNotifyCmdNotification request;

		memset(&request,0,sizeof(AlpNotifyCmdNotification));
		request.cmd = ALP_NOTIFY_CMD_NOTIFY;
		request.size = sizeof(AlpNotifyCmdNotification);

		strcpy(request.notifyType,entry->notifyType);
		request.callback = entry->entry->callback;
        request.userData = entry->entry->userData;
		request.length = entry->length;
		request.flags = entry->flags;
		request.handled = entry->handled;
		strcpy(request.appID,entry->appID);
		strcpy(request.appID2,entry->entry->appID);
		if (entry->length) memmove(request.details,entry->details,entry->length);

		// return the pid
		*pid = entry->entry->pid;

		return alp_notify_ipc_send(entry->entry->appID,&request,request.size);

	} else {
		// broadcasting to an application
		alp_status_t err;

		err = alp_prv_notify_broadcast_launch_app(entry);
		*pid = entry->pid;
		return err;

	}
}

// prefix string compare function for notification type strings
static int alp_prv_notify_broadcast_strcmp(const char * broadcast, const char * registered)
{
	int broadcast_len, registered_len;
	char * space;

	// get the string compare lengths
	broadcast_len = strlen(broadcast);
	space = strchr(registered,' ');
	registered_len = space ? (int)space - (int)registered : (int)strlen(registered);

	// check if prefix or standard compare
	if (*broadcast == '/' && *registered == '/' && registered_len <= broadcast_len) 

		// prefix compare
		return strncmp(broadcast,registered,registered_len);
	else

		// standard compare
		return strcmp(registered,broadcast);
}

// find something to broadcast
void alp_notify_broadcast_do_pending(void)
{
	AlpNotifyNotificationEntry * entry;
	int pid, queue = 0;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_broadcast_do_pending\n");
#endif
	// check if any pending notifications or if broadcasting is disabled
	if (!sBroadcastList || !sBroadcastEnabled) {
		return;
	}

	// search the list
	entry = sBroadcastList;
	for (;;) {

		// check if a broadcast is not currently in progress and the queue is ready
		if (!entry->busy && !(queue & entry->queue)) {

			// skip notifyTypes that don't match
			while (entry->entry && alp_prv_notify_broadcast_strcmp(entry->notifyType,entry->entry->notifyType))
				entry->entry = entry->entry->next;

			// check appID
			if (entry->entry && !alp_notify_get_pid(entry->entry->appID)) {

				// send notification
				if (alp_prv_notify_broadcast_send(entry,&pid) == 0) {
					alp_notify_add(entry->entry->appID,pid);
					entry->busy = 1;
				}
			}
		}

		// update available queues
		if (entry->entry && entry->busy) queue = queue | entry->queue;

		// check if at the end of the list
		if (!entry->next) break;

		// otherwise go to the next item
		entry = entry->next;
	} 
}

// remove a broadcast
int alp_notify_broadcast_remove(AlpNotifyNotificationEntry * notify)
{
	AlpNotifyNotificationEntry * entry;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_broadcast_remove\n");
#endif
	// check if empty list
	if (!sBroadcastList) return 0;

	// check if first item
	if (sBroadcastList == notify) {
		sBroadcastList = notify->next;
		alp_prv_notify_broadcast_free(notify);
		return 0;
	}

	// otherwise search the list
	entry = sBroadcastList;
	for (;;) {

		// check if found
		if (entry->next == notify) {
			entry->next = entry->next->next;
			alp_prv_notify_broadcast_free(notify);
			return 0;
		}

		// check if at the end of the list
		if (!entry->next) break;

		// otherwise go to the next item
		entry = entry->next;
	}

	// shouldn't ever get here
	free(notify);
	printf("FATAL ERROR: notify_broadcast_remove\n");
	return 1;
}

// cleanup any broadcast that don't have anything left to do
void alp_notify_broadcast_cleanup(void)
{
	AlpNotifyNotificationEntry * entry, * previous, * tmp;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_broadcast_cleanup\n");
#endif
	// search all the entry
	entry = sBroadcastList;
	previous = 0;
	while (entry) {

		// check if empty broadcast
		if (!entry->entry) {

			// check if the first item should be deleted
			tmp = entry;
			if (!previous) {
				entry = sBroadcastList = sBroadcastList->next;
			} else {
				previous->next = entry->next;
				entry = entry->next;
			}

			// free memory
			alp_prv_notify_broadcast_free(tmp);

		} else {

			// goto the next entry
			previous = entry;
			entry = entry->next;
		}
	}
}

// for onetime registrations, remove them when the notification is completed
static void alp_prv_notify_broadcast_remove(AlpNotifyRegisterEntry * entry)
{
	AlpNotifyCmdUnregister request;

	// fill out the message header
	memset(&request,0,sizeof(AlpNotifyCmdUnregister));
	request.cmd = ALP_NOTIFY_CMD_UNREGISTER;
	request.size = sizeof(AlpNotifyCmdUnregister);

	// fill out the message body
	strcpy(request.notifyType,entry->notifyType);
	request.flags = 0;
	strcpy(request.appID,entry->appID);
	request.zero = 0;

	// unregister
	alp_notify_register_remove(&request);
}

// exported entrypoint when a notification is completed
int alp_notify_broadcast_done(AlpNotifyCmdNotificationDone * request)
{
	AlpNotifyNotificationEntry * entry;
	AlpNotifyRegisterEntry * tmp;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_broadcast_done\n");
#endif
	// check packet size
	if (request->size != sizeof(AlpNotifyCmdNotificationDone))
		return ALP_NOTIFY_ERR_INVALID_CMD;

	// check packet data
	if (request->zero) return ALP_NOTIFY_ERR_INVALID_CMD;
	if (strlen(request->appID) > ALP_NOTIFY_MAX_APPID_SIZE)
		return ALP_NOTIFY_ERR_INVALID_APPID;

	// check if any pending notifications
	if (!sBroadcastList)
		return 0;

#ifdef ALP_NOTIFY_DEBUG_MESG
	printf("NotifyMgr: notify done: appID=%s\n",request->appID);
#endif
	// search the list
	entry = sBroadcastList;
	for (;;) {

		// is this a match
		if (entry->busy == 1 && !strcmp(entry->entry->appID,request->appID)) {
			// reset the state
			entry->busy = 0;
			alp_notify_remove(entry->entry->appID);

			// set the handled flag
			entry->handled = request->handled;

			// check if one time broadcast
			tmp = (entry->entry->flags & ALP_NOTIFY_REGISTER_FLAG_ONETIME) ? entry->entry : 0;

			// are we done?
			if (!entry->entry->next || request->status & ALP_NOTIFY_CALLBACK_CANCEL) {
				// no more entries, remove broadcast
				alp_notify_broadcast_remove(entry);

				// remove if one time registration
				if (tmp) alp_prv_notify_broadcast_remove(tmp);
				return 0;
			}

			// goto the next one
			entry->entry = entry->entry->next;

			// remove if one time registration
			if (tmp) alp_prv_notify_broadcast_remove(tmp);
		}

		// check if at the end of the list
		if (!entry->next) break;

		// otherwise go to the next item
		entry = entry->next;
	} 
	return 0;
}

// cancels a sync broadcast if the connection disconnects
void alp_notify_broadcast_cancel_sync(AlpConnection * connect)
{
	AlpNotifyNotificationEntry * entry;

	// search the list
	entry = sBroadcastList;
	while (entry) {

		// check if active
		if (entry->connect == connect) entry->connect = 0;

		// otherwise go to the next item
		entry = entry->next;
	}
}

// check if a particular registration entry is active (so we don't delete them)
int alp_notify_broadcast_is_active(AlpNotifyRegisterEntry * entry)
{
	AlpNotifyNotificationEntry * tmp;

	// search the list
	tmp = sBroadcastList;
	while (tmp) {

		// check if active
		if (tmp->entry == entry && tmp->busy) return 1;

		// otherwise go to the next item
		tmp = tmp->next;
	}

	// not found
	return 0;
}

// set all broadcasts for a given appID to be inactive when the app crashes
void alp_notify_broadcast_set_inactive(const char  * appID)
{
	AlpNotifyNotificationEntry * entry;

	// search the list
	entry = sBroadcastList;
	while (entry) {

		// check if active
		if (entry->entry && entry->busy && !strcmp(entry->entry->appID,appID)) {

			// wait for launch to complete
			if (entry->pid == -1) pthread_join(entry->thread,0);

			// mark broadcast as non-busy
			entry->busy = 0;

			// goto to the next
			entry->entry = entry->entry->next;

			// done
			return;
		}

		// otherwise go to the next item
		entry = entry->next;
	}
}

// fixup any broadcasts that point to removed registrations
void alp_notify_broadcast_fixup(AlpNotifyRegisterEntry * old, AlpNotifyRegisterEntry * new)
{
	AlpNotifyNotificationEntry * entry;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_broadcast_fixup\n");
#endif
	// check if any pending notifications
	if (!sBroadcastList) {
		return;
	}

	// search the list
	entry = sBroadcastList;
	for (;;) {

		// check the registration was deleted
		if (entry->entry == old) entry->entry = new;

		// check if at the end of the list
		if (!entry->next) break;

		// otherwise go to the next item
		entry = entry->next;
	} 
}
