/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
#include <hiker/notifymgr.h>
#include <hiker/prv/notifymgr_prv.h>
#include <hiker/pdk/notifymgr_pdk.h>
#include "notify_mgr_server_prv.h"

#include <hiker/bundlemgr.h>
#include <hiker/security.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#define TELEPHONY_PATH	"/alp/telephony/"

static AlpNotifyRegisterHead * sRegisterHead = 0;
static int sRegisterCount = 0;

// calculate the hash for AlpNotifyRegisterHead structs
static int alp_prv_notify_register_get_hash(const char * notifyType)
{
	return 1;
}

// allocate a new registration entry
// returns a pointer to the new entry or zero for out of memory
static AlpNotifyRegisterEntry * alp_prv_notify_register_new_entry(AlpNotifyCmdRegister * request)
{
	AlpNotifyRegisterEntry * entry;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_prv_notify_register_new_entry\n");
#endif
	// allocate
	entry = (AlpNotifyRegisterEntry *)malloc(sizeof(AlpNotifyRegisterEntry));
	if (!entry) return 0;

	entry->notifyType = malloc(strlen(request->notifyType)+1);
	if (!entry->notifyType) {
		free(entry);
		return 0;
	}

	entry->appID = malloc(strlen(request->appID)+1);
	if (!entry->appID) {
		free(entry->notifyType);
		free(entry);
		return 0;
	}

	// copy the data
	strcpy(entry->notifyType,request->notifyType);
	strcpy(entry->appID,request->appID);
	entry->callback = request->callback;
    entry->userData = request->userData;
	entry->priority = request->priority;
	entry->flags = request->flags;
	entry->pid = request->pid;
	entry->next = 0;

	// update count
	sRegisterCount++;

#ifdef ALP_NOTIFY_DEBUG_MESG
	if (entry->callback) {
		printf("NotifyMgr: register server: notifyType=%s pid=%d priority=%d active=%d userData=%p\n",entry->notifyType,entry->pid,entry->priority,sRegisterCount,entry->userData);
	} else {
		printf("NotifyMgr: register app: notifyType=%s appID=%s priority=%d active=%d\n",entry->notifyType,entry->appID,entry->priority,sRegisterCount);
	}
#endif
	// add to database
	if (!(request->flags & ALP_NOTIFY_REGISTER_FLAG_DONTSAVE))
		alp_notify_database_add(entry);


	return entry;
}

// free register entry
static void alp_prv_notify_register_free_entry(AlpNotifyRegisterEntry * entry)
{
	sRegisterCount--;
#ifdef ALP_NOTIFY_DEBUG_MESG
	if (entry->callback) {
		printf("NotifyMgr: unregister server: notifyType=%s pid=%d priority=%d active=%d userData=%p\n",entry->notifyType,entry->pid,entry->priority,sRegisterCount,entry->userData);
	} else {
		printf("NotifyMgr: unregister app: notifyType=%s appID=%s priority=%d active=%d\n",entry->notifyType,entry->appID,entry->priority,sRegisterCount);
	}
#endif
	free(entry->notifyType);
	free(entry->appID);
	free(entry);
}

// allocate new register head entry
static AlpNotifyRegisterHead * alp_prv_notify_register_new_head(AlpNotifyCmdRegister * request)
{
	AlpNotifyRegisterHead * head;
	AlpNotifyRegisterEntry * entry;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_prv_notify_register_new_head\n");
#endif
	head = (AlpNotifyRegisterHead *)malloc(sizeof(AlpNotifyRegisterHead));
	if (!head) return 0;

	entry = alp_prv_notify_register_new_entry(request);
	if (!entry) {
		free(head);
		return 0;
	}

	head->hash = alp_prv_notify_register_get_hash(request->notifyType);
	head->entry = entry;
	head->next = 0;

	return head;
}

// add a new register entry in priority sorted order
static alp_status_t alp_prv_notify_register_insert_entry(AlpNotifyRegisterHead * head, AlpNotifyCmdRegister * request)
{
	AlpNotifyRegisterEntry * tmp, * entry, * new;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_prv_notify_register_insert_entry\n");
#endif
	// allocate a new entry
	new = alp_prv_notify_register_new_entry(request);
	if (!new) return ALP_NOTIFY_ERR_OUT_OF_MEMORY;

	// check if insert at the beginning
	if (head->entry->priority < request->priority) {
		tmp = head->entry;
		head->entry = new;
		new->next = tmp;
		return ALP_OK;
	}

	// search the list
	entry = head->entry;
	for (;;) {
		// check that highest or lowest priority is not already registered
		if (!strcmp(entry->notifyType,request->notifyType) && (entry->priority == request->priority) && (request->priority == ALP_NOTIFY_PRIORITY_HIGHEST || request->priority == ALP_NOTIFY_PRIORITY_LOWEST)) {
			alp_prv_notify_register_free_entry(new);
			return ALP_NOTIFY_ERR_ALREADY_REGISTERED;
		}

		// check if insert at the end of the list
		if (!entry->next) {
			entry->next = new;
			return ALP_OK;
		}

		// check if insert between this and the next
		if (entry->next->priority < request->priority) {
			tmp = entry->next;
			entry->next = new;
			new->next = tmp;
			return ALP_OK;
		}

		// otherwise go to the next item
		entry = entry->next;
	}
}

// exported entrypoint to add a new registration
int alp_notify_register_add(AlpNotifyCmdRegister * request, int pid)
{
	AlpNotifyRegisterHead * head;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_register_add\n");
#endif
	// check packet size
	if (request->size != sizeof(AlpNotifyCmdRegister))
		return ALP_NOTIFY_ERR_INVALID_CMD;

	// check packet data
	if (request->zero)
		return ALP_NOTIFY_ERR_INVALID_CMD;
	if (strlen(request->notifyType) > ALP_NOTIFY_MAX_NOTIFY_SIZE)
		return ALP_NOTIFY_ERR_INVALID_NOTIFYTYPE;
	if (strlen(request->appID) > ALP_NOTIFY_MAX_APPID_SIZE)
		return ALP_NOTIFY_ERR_INVALID_APPID;

	// check appID
	if (!request->callback && !(request->flags & ALP_NOTIFY_REGISTER_FORCE)) {
		AlpBundle package;
		// initialize the package manager
		alp_notify_package_init();

		// get the package for the appID
		package = alp_bundle_by_name(request->appID);
		if (!package.value) return ALP_NOTIFY_ERR_INVALID_APPID;
	}

	// check if already defined
	if (alp_notify_register_is_defined(request->notifyType, request->appID))
		return ALP_NOTIFY_ERR_ALREADY_REGISTERED;

	// check if the priority is invalid
	if (request->priority > ALP_NOTIFY_PRIORITY_HIGHEST || request->priority < ALP_NOTIFY_PRIORITY_LOWEST)
		return ALP_NOTIFY_ERR_INVALID_PRIORITY;

	// check if already at the limit
	if ((sRegisterCount >= ALP_NOTIFY_MAX_REGISTRATIONS) && !(request->flags & ALP_NOTIFY_REGISTER_FORCE))
		return ALP_NOTIFY_ERR_TOO_MANY_REGISTRATIONS;

	// check security policy
/*	if (!strncmp(request->notifyType,TELEPHONY_PATH,MIN(strlen(request->notifyType),strlen(TELEPHONY_PATH))) && pid) {
		char * value = 0;
		char * policy_name = "com.access.notifymgr.register.telephony";
		int deny = 0;
		alp_status_t err;

		if ((err = alp_spf_query_open()) != ALP_STATUS_SPF_E_NOERR) return ALP_NOTIFY_ERR_PERMISSION_DENIED;
		if ((err = alp_spf_query_set_subject_pid(pid)) != ALP_STATUS_SPF_E_NOERR) goto Done;
		if ((err = alp_spf_query_get_assertion_string(policy_name,strlen(policy_name),&value)) != ALP_STATUS_SPF_E_NOERR) goto Done;
Done:
		alp_spf_query_close();
		if (err != ALP_STATUS_SPF_E_NOERR) return ALP_NOTIFY_ERR_PERMISSION_DENIED;

		if (value) {
			if (!strcmp(value,"false")) deny = 1;
			g_free(value);
			if (deny) return ALP_NOTIFY_ERR_PERMISSION_DENIED;
		}
	}
*/	// check if no registrations
	if (!sRegisterHead) {
		// add a new registration
		sRegisterHead = alp_prv_notify_register_new_head(request);
		return sRegisterHead ? ALP_OK : ALP_NOTIFY_ERR_OUT_OF_MEMORY;
	}

	// search existing registration
	head = sRegisterHead;
	for (;;) {
		// check for a match
		if (head->hash == alp_prv_notify_register_get_hash(request->notifyType)) {
			return alp_prv_notify_register_insert_entry(head,request);
		}

		// check if at the end of the list
		if (!head->next) break;

		// otherwise go to the next item
		head = head->next;
	} 

	// add a new registration at the end of the list
	head->next = alp_prv_notify_register_new_head(request);

	return head->next ? ALP_OK : ALP_NOTIFY_ERR_OUT_OF_MEMORY;
}

// get the register head struct for a given notification type
AlpNotifyRegisterHead * alp_notify_register_get(const char * notifyType)
{
	AlpNotifyRegisterHead * head;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_register_get\n");
#endif
	// search existing registration
	head = sRegisterHead;
	while (head) {

		// check for a match
		if (head->hash == alp_prv_notify_register_get_hash(notifyType)) return head;

		// otherwise go to the next item
		head = head->next;
	}

	// no match
	return 0;
}

// check if the given notification type is already registered for the given appID
int alp_notify_register_is_defined(const char * notifyType, const char * appID)
{
	AlpNotifyRegisterHead * head;
	AlpNotifyRegisterEntry * entry;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_register_is_defined\n");
#endif
	// get the list of apps for the specific notifyType
	head = alp_notify_register_get(notifyType);

	// check if there are any registered apps
	if (!head) return 0;

	// look at all the entries
	entry = head->entry;
	while (entry) {

		// check if the notifyTypes and appIDs match
		if (!strcmp(notifyType,entry->notifyType) && !strcmp(entry->appID,appID)) return 1;

		// goto the next item
		entry = entry->next;
	}

	// no more entries, no match
	return 0;
}

// remove any register head structs with no register entries
void alp_notify_register_cleanup(void)
{
	AlpNotifyRegisterHead * head, * previous, * tmp;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_register_cleanup\n");
#endif
	// check for empty list
	if (!sRegisterHead) return;

	// search existing registration
	head = sRegisterHead;
	previous = 0;

	for (;;) {

		// check for unused registrations
		if (!head->entry) {

			// check if first item should be deleted
			if (!previous) {
				tmp = head;
				head = sRegisterHead = head->next;
			} else {
				tmp = head;
				previous->next = head->next;
				head = head->next;
			}

			// free memory
			free(tmp);

			// check if at the end of the list
			if (!head) break;

			// otherwise do again
			continue;
		}

		// check if at the end of the list
		if (!head->next) break;

		// otherwise go to the next item
		previous = head;
		head = head->next;
	}
}

// exported entrypoint to remove a registration
int alp_notify_register_remove(AlpNotifyCmdUnregister * request)
{
	AlpNotifyRegisterHead * head;
	AlpNotifyRegisterEntry * entry, * previous, * tmp;

#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_register_remove\n");
#endif
	// check packet size
	if (request->size != sizeof(AlpNotifyCmdUnregister))
		return ALP_NOTIFY_ERR_INVALID_CMD;

	// check packet data
	if (request->zero)
		return ALP_NOTIFY_ERR_INVALID_CMD;
	if (strlen(request->appID) > ALP_NOTIFY_MAX_APPID_SIZE)
		return ALP_NOTIFY_ERR_INVALID_APPID;
	if (strlen(request->notifyType) > ALP_NOTIFY_MAX_NOTIFY_SIZE)
		return ALP_NOTIFY_ERR_INVALID_NOTIFYTYPE;

	// check if already defined
	if (!alp_notify_register_is_defined(request->notifyType, request->appID))
		return ALP_NOTIFY_ERR_NOT_REGISTERED;

	// get the register
	if (!(head = alp_notify_register_get(request->notifyType)))
		return 0;

	// look at all the entries
	entry = head->entry;
	previous = 0;
	while(entry) {

		// check if the appIDs match
		if (!strcmp(entry->appID,request->appID) && !strcmp(entry->notifyType,request->notifyType)) {

			// check if any active broadcast using this
			if (alp_notify_broadcast_is_active(entry)) return ALP_NOTIFY_ERR_ACTIVE_BROADCAST;

			// check if first item should be deleted
			if (!previous) {
				head->entry = head->entry->next;
			} else {
				previous->next = entry->next;
			}

			// searching pending notification
			alp_notify_broadcast_fixup(entry,entry->next);

			// remove from database
			alp_notify_database_remove(entry);

			// free memory
			tmp = entry->next;
			alp_prv_notify_register_free_entry(entry);

			// goto the next entry
			entry = tmp;

		} else {

			// goto the next entry
			previous = entry;
			entry = entry->next;
		}
	}

	// cleanup any empty registrations
	alp_notify_register_cleanup();

	// cleanup any empty pending notifications
	alp_notify_broadcast_cleanup();

	return 0;
}

// exported entrypoint to remove all registration for a given appID
int alp_notify_register_remove_all(const char * appID, int server)
{
	AlpNotifyRegisterHead * head;
	AlpNotifyRegisterEntry * entry, * previous, * tmp;

#ifdef ALP_NOTIFY_DEBUG_MESG
	printf("NotifyMgr: cleanup %s\n",appID);
#endif
#ifdef ALP_NOTIFY_DEBUG
	printf("alp_notify_register_remove_all\n");
#endif
	// check active broadcast
	alp_notify_broadcast_set_inactive(appID);

	// search existing registration
	head = sRegisterHead;

	// look at all the notifications
	while (head) {

		// look at all the entries
		entry = head->entry;
		previous = 0;

		while(entry) {

			// check if the appIDs match and if server mode then check for callback
			if (!strcmp(entry->appID,appID) &&
				((server && entry->callback) || !server)) {

				// check if first item should be deleted
				if (!previous) {
					head->entry = head->entry->next;
				} else {
					previous->next = entry->next;
				}

				// searching pending notification
				alp_notify_broadcast_fixup(entry,entry->next);

				// remove from database
				alp_notify_database_remove(entry);

				// free memory
				tmp = entry->next;
				alp_prv_notify_register_free_entry(entry);

				// goto the next entry
				entry = tmp;

			} else {

				// goto the next entry
				previous = entry;
				entry = entry->next;
			}
		}

		// goto the next entry
		head = head->next;
	}

	// cleanup any empty registrations
	alp_notify_register_cleanup();

	// cleanup any empty pending notifications
	alp_notify_broadcast_cleanup();

	return 0;
}
