/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/* SecurityObject.h
**
**
** Security Object Interface
**
** This file defines the interface for the SecurityObject.
**
*/


/* Macro Definitions */
#ifndef _H_ALP_PRV_SECURITYOBJECT_
#define _H_ALP_PRV_SECURITYOBJECT_

#define ALP_PRV_SPF_SO_E_NOERR             0 
#define ALP_PRV_SPF_SO_E_NULL_PARAMETER    1 
#define ALP_PRV_SPF_SO_E_FILE              2
#define ALP_PRV_SPF_SO_E_MALLOC            3
#define ALP_PRV_SPF_SO_E_GID_IN_USE        4
#define ALP_PRV_SPF_SO_E_ID_IN_USE         5
#define ALP_PRV_SPF_SO_E_UID_IN_USE        6
#define ALP_PRV_SPF_SO_E_UNIMPLEMENTED     7
#define ALP_PRV_SPF_SO_E_UNKNOWN_ASSERTION 8
#define ALP_PRV_SPF_SO_E_UNKNOWN_POLICY    9
#define ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE   10
#define ALP_PRV_SPF_SO_E_UNKNOWN_GID       11
#define ALP_PRV_SPF_SO_E_FULL_QUEUE        12
#define ALP_PRV_SPF_SO_E_NOT_FOUND         13
#define ALP_PRV_SPF_SO_E_UNSPECIFIED       14
#define ALP_PRV_SPF_SO_E_PARSE             15
#define ALP_PRV_SPF_SO_E_UNKNOWN_PID       16
#define ALP_PRV_SPF_SO_E_VERIFY            17

#define ALP_PRV_SPF_SO_DEFAULT_VERSION          0x01000000;

#define ALP_PRV_SPF_SO_DEFAULT_POLICY_BASE      "default"
#define ALP_PRV_SPF_SO_DEFAULT_POLICY_DEFAULT   "default"
#define ALP_PRV_SPF_SO_DEFAULT_POLICY_GID_MIN   1001
#define ALP_PRV_SPF_SO_DEFAULT_POLICY_GID_NEXT  1001
#define ALP_PRV_SPF_SO_DEFAULT_POLICY_ID_MIN    1
#define ALP_PRV_SPF_SO_DEFAULT_POLICY_ID_NEXT   1

#define ALP_PRV_SPF_SO_DEFAULT_PACKAGE_UID_MIN  1001
#define ALP_PRV_SPF_SO_DEFAULT_PACKAGE_UID_NEXT 1001
#define ALP_PRV_SPF_SO_DEFAULT_PACKAGE_ID_MIN   1
#define ALP_PRV_SPF_SO_DEFAULT_PACKAGE_ID_NEXT  1

#define ALP_PRV_SPF_SO_DEFAULT_CONFIG "/etc/default.policy"

#define ALP_PRV_SPF_SO_TYPE_STRING  0
#define ALP_PRV_SPF_SO_TYPE_BOOLEAN 1
#define ALP_PRV_SPF_SO_TYPE_INTEGER 2
#define ALP_PRV_SPF_SO_TYPE_ADDRESS 3
#define ALP_PRV_SPF_SO_TYPE_PATH    4
#define ALP_PRV_SPF_SO_TYPE_URL     5
#define ALP_PRV_SPF_SO_TYPE_MAX     5

#define ALP_PRV_SPF_SO_PKCS5_SALT_MAX_LENGTH 40
#define ALP_PRV_SPF_SO_PKCS5_LENGTH 20

#define ALP_PRV_SPF_SO_ATTN_QUEUE_SIZE 10

/* File Includes */
#include <glib.h>
#include <stdio.h>
#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <pwd.h>
#include <grp.h>
#include <sys/stat.h>
#include <hiker/pdk/sysutils_pdk.h>
#include <hiker/attnmgr_common.h>
#include "SecurityUtilities.h"

/* Typedefs, Structs, Unions, Enums, etc. */
typedef unsigned int AlpPrvSpfSoErr;

typedef struct {
  char *issuer;
  char *serial;
  char *encoding;
} AlpPrvSpfSoCertificateReference;

typedef struct {
  char *key;
  char *value;
  unsigned int type;
  unsigned int kernel;
} AlpPrvSpfSoSecurityAssertion;

typedef struct _AlpPrvSpfSoSecurityPolicy {
  unsigned int id;
  unsigned int gid;
  char *name;
  struct _AlpPrvSpfSoSecurityPolicy *base;
  GPtrArray *assertions;
  GPtrArray *certificates;
  char *sgids;
  unsigned int sflag;
  unsigned int bflag;
} AlpPrvSpfSoSecurityPolicy;

typedef struct {
  unsigned int id;
  AlpPrvSpfSoSecurityPolicy *policy;
  AlpPrvSpfSoSecurityAssertion *assertion;
  char *response;
} AlpPrvSpfSoAttn;

typedef struct {
  unsigned int id;
  unsigned int uid;
  unsigned int ref_count;
  char hash[20];
  char *package;
  AlpPrvSpfSoSecurityPolicy *policy;
} AlpPrvSpfSoPackage;

typedef struct {
  unsigned int debug;
  char *config_path;
  FILE *debug_fp;
  char *log_path;
  unsigned int user_set;
  char *user;
  char *group;
  uid_t uid;
  gid_t gid;
  void (*old_signal)(int);
  unsigned int background;
  AlpAttnProp **attn_properties;
  unsigned int attn_properties_size;
} App ;

typedef struct _tAlertStatus {
  struct _tAlertStatus *next;
  char *policy;
  char *assertion;
  char *response;
} tAlertStatus;

typedef struct _SecurityObject {
  /* Object header to make DBUS happy */
  GObject parent;

  App *app;
  GQuark SSQ;
  unsigned int debug;
  FILE *debug_fp;
  char *config_path;

  /* Meta-Data */
  unsigned int version;

  /* Connection Info */
  DBusGConnection *connection;
  DBusGProxy *bus_proxy;

  /* Policy Collection */
  GPtrArray *policies;
  char *policy_base;
  char *policy_default;
  GHashTable *certs_to_policies;
  unsigned int policy_gid_min;
  unsigned int policy_gid_next;
  unsigned int policy_id_min;
  unsigned int policy_id_next;

  /* Package Collection */
  GPtrArray *packages;
  unsigned int package_uid_min;
  unsigned int package_uid_next;
  unsigned int package_id_min;
  unsigned int package_id_next;

  /* Certificates */
  GPtrArray *certificates;

  /* PKCS5 Phrase */
  AlpPrvSpfSuPKCS5 *pin;
  AlpPrvSpfSuPKCS5 *puk;

  AlpPrvSpfSoAttn attn_queue[ ALP_PRV_SPF_SO_ATTN_QUEUE_SIZE ];
  unsigned int attn_queue_depth;
  unsigned int attn_queue_next;

} SecurityObject;

typedef struct _SecurityObjectClass {
  GObjectClass parent;
} SecurityObjectClass;

/* Function Prototypes */
AlpPrvSpfSoErr alp_prv_spf_so_initialize( SecurityObject *object );
AlpPrvSpfSoErr alp_prv_spf_so_uninitialize( SecurityObject *object );

AlpPrvSpfSoErr alp_prv_spf_so_read( SecurityObject *object, char *path );
AlpPrvSpfSoErr alp_prv_spf_so_write( SecurityObject *object, char *path );

AlpPrvSpfSoErr alp_prv_spf_so_package_add( SecurityObject *object, AlpPrvSpfSoPackage *package);
AlpPrvSpfSoErr alp_prv_spf_so_certificate_add( SecurityObject *object, AlpPrvSpfSoCertificateReference *certificate );

AlpPrvSpfSoErr alp_prv_spf_so_policy_find_by_id( SecurityObject *object, unsigned int id, AlpPrvSpfSoSecurityPolicy **policy );
AlpPrvSpfSoErr alp_prv_spf_so_policy_find_by_gid( SecurityObject *object, unsigned int gid, AlpPrvSpfSoSecurityPolicy **policy );
AlpPrvSpfSoErr alp_prv_spf_so_policy_find_by_certificate_reference( SecurityObject *object, AlpPrvSpfSoCertificateReference *certificate, AlpPrvSpfSoSecurityPolicy **policy );
AlpPrvSpfSoErr alp_prv_spf_so_policy_find_by_name( SecurityObject *object, char *name, AlpPrvSpfSoSecurityPolicy **policy );
AlpPrvSpfSoErr alp_prv_spf_so_policy_initialize( AlpPrvSpfSoSecurityPolicy *policy );
AlpPrvSpfSoErr alp_prv_spf_so_policy_uninitialize( AlpPrvSpfSoSecurityPolicy *policy );
AlpPrvSpfSoErr alp_prv_spf_so_policy_write( AlpPrvSpfSoSecurityPolicy *policy, FILE *fp, char *indent );
AlpPrvSpfSoErr alp_prv_spf_so_policy_assertion_add( AlpPrvSpfSoSecurityPolicy *policy, AlpPrvSpfSoSecurityAssertion *assertion );
AlpPrvSpfSoErr alp_prv_spf_so_policy_assertion_find( AlpPrvSpfSoSecurityPolicy *policy, char *key, AlpPrvSpfSoSecurityAssertion **out_assertion );

AlpPrvSpfSoErr alp_prv_spf_so_assertion_write( AlpPrvSpfSoSecurityAssertion *assertion, FILE *fp, char *indent );

AlpPrvSpfSoErr alp_prv_spf_so_certificate_populate( AlpPrvSpfSoCertificateReference *certificate );
AlpPrvSpfSoErr alp_prv_spf_so_package_find_by_id( SecurityObject *object, unsigned int packageid, AlpPrvSpfSoPackage **out_package );
AlpPrvSpfSoErr alp_prv_spf_so_package_find_by_uid( SecurityObject *object, unsigned int uid, AlpPrvSpfSoPackage **out_package );
AlpPrvSpfSoErr alp_prv_spf_so_package_find_by_package( SecurityObject *object, char *package, AlpPrvSpfSoPackage **out_package );
gboolean alp_prv_spf_so_package_find_by_hash( SecurityObject *object, char *digest, AlpPrvSpfSoPackage **out_package );


/* External Variable Declarations */

AlpPrvSpfSoErr alp_prv_spf_so_queue_add( SecurityObject *object, AlpPrvSpfSoSecurityAssertion *assertion, AlpPrvSpfSoSecurityPolicy *policy, unsigned int *id );
AlpPrvSpfSoErr alp_prv_spf_so_queue_remove( SecurityObject *object, unsigned int id );
AlpPrvSpfSoErr alp_prv_spf_so_queue_find( SecurityObject *object, unsigned int id, AlpPrvSpfSoAttn **attn );

#endif
