/********************************************************************/
/* SecuirtyUi.c
**
**
** $Id: //dev/alp/platform/main/servers/security/SecuirtyUi.c#61 $
**
** This file contains the "UI side" of the SPF security service.
*/

/* File Includes */

#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <pwd.h>
#include <grp.h>
#include <sys/stat.h>
#include <hiker/pdk/sysutils_pdk.h>
#include <hiker/types.h>
#include <hiker/bundlemgr.h>
#include <hiker/attnmgr.h>
#include <hiker/security.h>
#include "SecurityService.h"
#include "SecurityObject.h"
#include "SecurityUpdate.h"
#include "SecurityKernel.h"
#include "SecurityManifest.h"
#include "SecurityUtilities.h"
#include "SecurityPKCS5Support.h"
#include "SecurityUi.h"
#include <openssl/sha.h>

#define MAX_USER_INPUT_LEN     80

/* Typedefs, Structs, Unions, Enums, etc. */
static void testme_prv_callback (const char *serverId, 
                                 const char *alertType, 
                                 const char *handle, 
                                 int argc, 
                                 char *argv[], 
                                 void *user_data);

static AlpAttnProp  prop20 = { "title", "Security Alert" };
static AlpAttnProp  prop21 = { "text-1", "insert assertion key here" };
static AlpAttnProp  prop22 = { "text-2", "insert policy here"};
static AlpAttnProp  prop23 = { "button-1", "Allow" };
static AlpAttnProp  prop24 = { "button-2", "Disallow" };
static AlpAttnProp  prop25 = { "button-3", "" };
static AlpAttnProp  prop26 = { "action-1", "def-connect-clear allow" };
static AlpAttnProp  prop27 = { "action-2", "def-connect-clear disallow" };
static AlpAttnProp  prop28 = { "action-goto", "clear" };
static AlpAttnProp *attn_properties_two[] = {
    &prop20,
    &prop21,
    &prop22,
    &prop23,
    &prop24,
    &prop25,
    &prop26,
    &prop27,
    &prop28
};

static AlpAttnProp  prop30 = { "title", "Security Alert" };
static AlpAttnProp  prop31 = { "text-1", "insert assertion key here" };
static AlpAttnProp  prop32 = { "text-2", "insert policy here"};
static AlpAttnProp  prop33 = { "button-1", "Allow" };
static AlpAttnProp  prop34 = { "button-2", "Allow Once" };
static AlpAttnProp  prop35 = { "button-3", "Deny" };
static AlpAttnProp  prop36 = { "action-1", "def-connect-clear allow" };
static AlpAttnProp  prop37 = { "action-2", "def-connect-clear allow-once" };
static AlpAttnProp  prop38 = { "action-3", "def-connect-clear disallow" };
static AlpAttnProp  prop39 = { "action-goto", "clear" };
static AlpAttnProp *attn_properties_three[] = {
    &prop30,
    &prop31,
    &prop32,
    &prop33,
    &prop34,
    &prop35,
    &prop36,
    &prop37,
    &prop38,
    &prop39
 };

static AlpAttnProp  prop40 = { "title", "Security Alert" };
static AlpAttnProp  prop41 = { "text-1", "insert assertion key here" };
static AlpAttnProp  prop42 = { "text-2", "insert policy here"};
static AlpAttnProp  prop43 = { "button-1", "Allow" };
static AlpAttnProp  prop44 = { "button-2", "Allow Always" };
static AlpAttnProp  prop45 = { "button-3", "Deny" };
static AlpAttnProp  prop46 = { "button-4", "Deny Always" };
static AlpAttnProp  prop47 = { "action-1", "def-connect-clear allow-once" };
static AlpAttnProp  prop48 = { "action-2", "def-connect-clear allow" };
static AlpAttnProp  prop49 = { "action-3", "def-connect-clear disallow-once" };
static AlpAttnProp  prop4A = { "action-4", "def-connect-clear disallow" };
static AlpAttnProp  prop4B = { "action-goto", "clear" };
static AlpAttnProp *attn_properties_four[] = {
    &prop40,
    &prop41,
    &prop42,
    &prop43,
    &prop44,
    &prop45,
    &prop46,
    &prop47,
    &prop48,
    &prop49,
    &prop4A,
    &prop4B
 };


void alp_prv_spf_attn_properties_set (App *app, unsigned int attn_prop) {


  switch (attn_prop) {
    case ATTN_PROP_TWO:
        app->attn_properties = attn_properties_two;
        app->attn_properties_size = sizeof(attn_properties_two);
        break;

    case ATTN_PROP_THREE:
        app->attn_properties = attn_properties_three;
        app->attn_properties_size = sizeof(attn_properties_three);
        break;

    case ATTN_PROP_FOUR:
        app->attn_properties = attn_properties_four;
        app->attn_properties_size = sizeof(attn_properties_four);
        break;

    default:
        app->attn_properties = attn_properties_two;
        app->attn_properties_size = sizeof(attn_properties_two);
        break;
  }
}

gboolean alp_prv_spf_ask_the_user (SecurityObject *object, 
                                   App *app,
                                   AlpPrvSpfSoSecurityAssertion *assertion,
                                   AlpPrvSpfSoSecurityPolicy *policy, 
                                   unsigned int id) {

  alp_status_t err = ALP_PRV_SPF_SO_E_NOERR;
  AlpPrvSpfSoAttn *attn;
  char buffer[MAX_USER_INPUT_LEN];
  
  uint32_t nprops = app->attn_properties_size/sizeof(attn_properties_two[0]);
  int one, two;

  d("%SPFSS-D-ATU; Entering ask_the_user.\n");
#if DEBUG
  fprintf(object->debug_fp, "id = %d\n", id );
  fflush( object->debug_fp );
#endif

  /* 
   * Send the request to the attention manager here 
   */
  if (alp_prv_spf_so_queue_find(object, id, &attn) == ALP_PRV_SPF_SO_E_NOERR) {
#if DEBUG
      fprintf(object->debug_fp, "found attn = %08X\n", (unsigned int) attn );
      one = (int) & object->debug_fp;
      two = (int) & object->attn_queue[0];
      fprintf(object->debug_fp, "array = %d\n", ( one - two ) );
      fflush( object->debug_fp );
#endif
      sprintf(buffer, "alpsecurityd%d", id);
      err = alp_attn_alert_post_with_callback("bar:com.access.services.security",
  			    "/alp/security/alert/31337",buffer, NULL, 2, 0,
  			    nprops, app->attn_properties, testme_prv_callback,
  			    (void *) attn);
  }

#if DEBUG
      fprintf(object->debug_fp, "return from alert_post_with_callback = %08X\n", (unsigned int) err );
      fflush( object->debug_fp );
#endif
  
  return (TRUE);
}

static void testme_prv_callback (const char *serverId, 
                                 const char *alertType, 
                                 const char *handle, 
                                 int argc, 
                                 char *argv[], 
                                 void *user_data) {
  AlpPrvSpfSoAttn *attn = (AlpPrvSpfSoAttn *) user_data;
  
  FILE *fp = NULL;

  if (NULL != (fp = fopen("/tmp/security_callback","a"))) {
      fprintf( fp, "server = %08X, alert = %08X, handle = %08X, argc = %d, argv[0] = %s, user_data = %08X\n", (unsigned int) serverId, (unsigned int) alertType, (unsigned int) handle, argc, argv[0], (unsigned int) user_data );
      fclose( fp );
  }

  if (!strcmp( "disallow", argv[0])) {
      attn->response = "false";
  } else {
      attn->response = "true";
  }
}

