/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/* SecurityUpdate.c
**
**
** Security Update Implementation
**
** This file contains functions and data structures that implement the
** Security Object interface.
**
*/

#define TEMP_SMIME_FILE         "/tmp/smime" 
#define TEMP_CERT_FILE          "/tmp/cert" 
#define UPDATED_POLICY_FILE     "/var/opt/alp/data/lib/security/policy"
#define CMD_LINE_LEN            256

#include <stdio.h>
#include <glib.h>
#include <expat.h>
#include <string.h>
#include "SecurityObject.h"
#include "SecurityXml.h"
#include "SecurityUpdate.h"


/* Typedefs, Structs, Unions, Enums, etc. */

/**************************************************************************/

static AlpPrvSpfSoErr alp_prv_spf_su_verify (SecurityObject *object, 
                                             AlpPrvSpfXmlParse *xmlParse) { 
    AlpPrvSpfSoErr err = ALP_PRV_SPF_SO_E_NOERR;
    FILE *certFp;
    FILE *smimeMsgFp;
    char cmdLine[CMD_LINE_LEN];
    int status;
    int bytesWrote;
    int size;

    do {
      if ((object == NULL) || (xmlParse == NULL)) {
          err= ALP_PRV_SPF_SO_E_NULL_PARAMETER;
	  break;
      }

      if ((xmlParse->_certref._encoding == NULL) ||
          (xmlParse->_smimeMsg == NULL)) {
          err = ALP_PRV_SPF_SO_E_VERIFY;
	  break;
      }

      /*
       * !!!WARNING
       * Does not support calling registering of bundles at the same time from
       * different process or threads. Absolutely no protection whatsover.
       * TODO - ABOVE ISSUE HAS TO BE ADDRESSED
       */

      certFp = fopen(TEMP_CERT_FILE, "w+");
      smimeMsgFp = fopen(TEMP_SMIME_FILE, "w+");

      if ((certFp == NULL) || (smimeMsgFp == NULL)) {
          err = ALP_PRV_SPF_SO_E_FILE;  
	  break;
      }

      size = strlen(xmlParse->_certref._encoding);
      bytesWrote = fwrite(xmlParse->_certref._encoding, 1, size, certFp);
      if (bytesWrote != size) { 
          err = ALP_PRV_SPF_SO_E_FILE;
          break; 
      }

      size = strlen(xmlParse->_smimeMsg);
      bytesWrote = fwrite(xmlParse->_smimeMsg, 1, size, smimeMsgFp);
      if (bytesWrote != size) { 
          err = ALP_PRV_SPF_SO_E_FILE;
          break; 
      }

      fclose(certFp);
      fclose(smimeMsgFp);

      certFp = smimeMsgFp = NULL;

      sprintf(cmdLine, "openssl smime -verify -in %s -CAfile %s > %s\n",
                    TEMP_SMIME_FILE, TEMP_CERT_FILE, UPDATED_POLICY_FILE);

      status = system(cmdLine);

      /*
       * !!!!! WARNING
       * status returns 0 if it successful
       * if ALP_PRV_SPF_SO_E_NOERR is changed to other than 0, following
       * code has to be changed
       */
      if (status != ALP_PRV_SPF_SO_E_NOERR) {
          bzero (cmdLine, CMD_LINE_LEN);
          sprintf(cmdLine, "/bin/rm -f %s\n", UPDATED_POLICY_FILE);
          status = system(cmdLine);
          err = ALP_PRV_SPF_SO_E_VERIFY;
          break; 
      }

    } while (0);

    if (certFp != NULL) {
	fclose(certFp);
    }

    if (smimeMsgFp != NULL) {
	fclose(smimeMsgFp);
    }

    bzero (cmdLine, CMD_LINE_LEN);
    sprintf(cmdLine, "cat %s | tr -d \'\015\' > %s\n", UPDATED_POLICY_FILE, 
		                                       UPDATED_POLICY_FILE);
    fprintf(object->debug_fp, "\n%s\n", cmdLine);
    fflush(object->debug_fp);
    status = system(cmdLine);

    bzero (cmdLine, CMD_LINE_LEN);
    sprintf(cmdLine, "/bin/rm -f %s %s\n", TEMP_SMIME_FILE, TEMP_CERT_FILE);
    status = system(cmdLine);

    return( err );
}

AlpPrvSpfSoErr alp_prv_spf_su_update (SecurityObject *object, 
		                      AlpPrvSpfUpdate *update, 
				      unsigned int len) {

    AlpPrvSpfSoErr err = ALP_PRV_SPF_SO_E_NOERR;
    AlpPrvSpfXmlParse xmlParse;

    /*
     * Initialize
     */
    xmlParse._smimeMsg          = NULL;
    xmlParse._name              = NULL;
    xmlParse._certref._issuer   = NULL;
    xmlParse._certref._serial   = NULL;
    xmlParse._certref._encoding = NULL;

    do {
      if ((object == NULL) || (update == NULL)) {
          err = ALP_PRV_SPF_SO_E_NULL_PARAMETER;
	  break;
      }


      /*
       * Allocate the smime message to be the size of security.xml
       */
      xmlParse._smimeMsg = g_malloc(len);
      if (xmlParse._smimeMsg == NULL) {
          err = ALP_PRV_SPF_SO_E_MALLOC;    
	  break;
      }

      bzero(xmlParse._smimeMsg, len);

      err = alp_prv_spf_xml_parse(object, update->_xmlData, &xmlParse, len);
      if (err != ALP_PRV_SPF_SO_E_NOERR) {
          break; 
      }

      err = alp_prv_spf_su_verify(object, &xmlParse); 
      if (err != ALP_PRV_SPF_SO_E_NOERR) {
          break; 
      }


    } while (0);

    if (xmlParse._smimeMsg != NULL) {
        free(xmlParse._smimeMsg);
	xmlParse._smimeMsg = NULL;
    }

    alp_prv_spf_cleanup_xml_parse(&xmlParse);

    return (err);
}
