/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/* security_service.c
**
**
** $Id: //dev/alp/open-source/hiker/main/dist/servers/security/security_service.c#3 $
**
** This file contains the "service side" of the SPF security service.
*/

/* Macro Definitions */
#define DBUS_API_SUBJECT_TO_CHANGE

#ifndef DEBUG
#define DEBUG 1
#endif

#define ALP_PRV_SPF_SO_BREAK_IF_NULL( object, type, err ) if( object == (type *) NULL ) { err = FALSE; break; g_set_error( error, SSQ, ALP_PRV_SPF_SO_E_NULL_PARAMETER, "null parameter"); }

/* File Includes */

#include <dbus/dbus-glib.h>
#include <dbus/dbus-glib-bindings.h>
#include <dbus/dbus-glib-lowlevel.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <errno.h>
#include <sys/types.h>
#include <unistd.h>
#include <time.h>
#include <signal.h>
#include <pwd.h>
#include <grp.h>
#include <sys/stat.h>
#include <hiker/pdk/sysutils_pdk.h>
#include <hiker/types.h>
#include <hiker/bundlemgr.h>
#include <hiker/attnmgr.h>
#include <hiker/security.h>
#include "SecurityService.h"
#include "SecurityObject.h"
#include "SecurityUpdate.h"
#include "SecurityKernel.h"
#include "SecurityManifest.h"
#include "SecurityUtilities.h"
#include "SecurityPKCS5Support.h"
#include "SecurityUi.h"
#include "SecurityQuery.h"
#include <openssl/sha.h>

/* Typedefs, Structs, Unions, Enums, etc. */

G_DEFINE_TYPE(SecurityObject, security_object, G_TYPE_OBJECT);

/* Static Function Prototypes */

GType security_object_get_type( void );

gboolean alp_security_service_update( SecurityObject *obj, char *message, GError **error );

gboolean alp_security_service_ps_get_policyname_from_policy( SecurityObject *obj, unsigned int policy, gchar **out_policyname, gchar **out_parentname, GError **error );
gboolean alp_security_service_ps_get_policy_from_policyname( SecurityObject *obj, char *policyname, unsigned int *out_policy, GError **error );
gboolean alp_security_service_ps_packageid_increment( SecurityObject *obj, unsigned int packageid, unsigned int *out_count, GError **error );
gboolean alp_security_service_ps_packageid_decrement( SecurityObject *obj, unsigned int packageid, unsigned int *out_count, GError **error );
gboolean alp_security_service_ps_packageid_count( SecurityObject *obj, unsigned int packageid, unsigned int *out_count, GError **error );
gboolean alp_security_service_ps_get_policy_from_certificate( SecurityObject *obj, char *certificate, unsigned int *out_policy, GError **error );
gboolean alp_security_service_ps_get_policy_from_gid( SecurityObject *obj, unsigned int gid, unsigned int *out_policy, GError **error );
gboolean alp_security_service_ps_get_certificate_from_policy( SecurityObject *obj, unsigned int policy, gchar **out_certificate, gchar **out_certid, GError **error );
gboolean alp_security_service_ps_get_gid_from_policy( SecurityObject *obj, unsigned int policy, unsigned int *out_gid, GError **error );
gboolean alp_security_service_ps_get_policy_from_uid( SecurityObject *obj, unsigned int uid, unsigned int *out_policy, GError **error );
gboolean alp_security_service_ps_get_policy_from_package( SecurityObject *obj, char *package, unsigned int *out_policy, GError **error );
gboolean alp_security_service_ps_get_supplementary_group_size( SecurityObject *obj, unsigned int policy, unsigned int *out_size, GError **error );
gboolean alp_security_service_ps_get_uid_gid_sgids( SecurityObject *obj, unsigned int policy, unsigned int *out_uid, unsigned int *out_gid, GError **error );
gboolean alp_security_service_ps_enroll_package( SecurityObject *obj, char *package, unsigned int *out_packageid, GError **error );

gboolean alp_security_service_query_get_assertion_string( SecurityObject *obj, uint policy, char *assertion, unsigned int subject_pid, unsigned int client_pid, gchar **out_assertion_value, GError **error );
gboolean alp_security_service_query_get_assertion_type( SecurityObject *obj,unsigned int policy, char *assertion, unsigned int subject_pid, unsigned int client_pid, unsigned int *out_assertion_value, GError **error );

gboolean alp_security_service_info( SecurityObject *obj, gchar **out_version, GHashTable **out_calls, GError **error );

gboolean alp_security_service_pkcs5_set_phrase( SecurityObject *obj, unsigned int which, char *phrase, GError **error );
gboolean alp_security_service_pkcs5_check_phrase( SecurityObject *obj, unsigned int which, char *trial_phrase, unsigned int *correct, GError **error );
gboolean alp_security_service_pkcs5_set_parameters( SecurityObject *obj, unsigned int which, unsigned int iteration, unsigned int active, char *salt, GError **error );
gboolean alp_security_service_pkcs5_get_parameters( SecurityObject *obj, unsigned int which, unsigned int *iteration, unsigned int *active, char **salt, GError **error );

gboolean alp_security_service_apas_enroll( SecurityObject *obj, char *path, unsigned int *out_packageid, GError **error );
gboolean alp_security_service_apas_path_get_packageid( SecurityObject *obj, char *path, unsigned int *out_packageid, GError **error );
gboolean alp_security_service_apas_packageid_remove( SecurityObject *obj, unsigned int packageid, unsigned int *out_refcount, GError **error );
gboolean alp_security_service_apas_packageid_uid_get( SecurityObject *obj, unsigned int packageid, unsigned int *out_uid, GError **error );
gboolean alp_security_service_apas_packageid_gid_get( SecurityObject *obj, unsigned int packageid, unsigned int *out_gid, GError **error );
gboolean alp_security_service_apas_packageid_sgids_get( SecurityObject *obj, unsigned int packageid, char **out_sgids, GError **error );

static void *alp_prv_spf_kernel_interaction_thread( void *arg );

static gboolean alp_prv_security_service_parsecli( int argc, char *argv[], App *app );
static void alp_prv_security_service_usage( char *command );
static gboolean alp_prv_security_service_get_assertion_object( SecurityObject *obj, unsigned int policy, char *key, AlpPrvSpfSoSecurityAssertion **out_assertion, GError **error );

static void open_logs( App *app );
static void install_signal( App *app );
static void change_uid( App *app );
static void signal_close_log( int signum );

static gboolean alp_prv_security_service_get_gid_uid_from_pid( pid_t pid, uid_t *uid, gid_t *gid );

static void alp_prv_security_service_pkcs5_generate( char *data, unsigned int data_length, char *salt, unsigned int salt_length, unsigned int iteration, char *youtput );
static void alp_prv_security_service_sha_this( char *data, unsigned int length, char *output );

static void alp_prv_hash_buffer( char *data, unsigned int length, char *digest );

static void take_paths( App *app, char *paths[] );

gboolean alp_security_service_query_complete( SecurityObject *obj, unsigned int id, gchar **out_assertion_value, unsigned int *out_exit_okay, GError **error );


#include "security_service_glue.h"

/* Global Variable Declarations */

//static App app;
App app;

GQuark SSQ;
static tAlertStatus alert_status;

/* Function Definitions */

static void security_object_init( SecurityObject *obj ) {
  char *config_path = app.config_path;
  GError *error = NULL;
  int config_path_len = 0;

  SSQ = g_quark_from_string( "SecurityService" );
  alp_prv_spf_so_initialize( obj );
  
  obj->debug_fp = app.debug_fp;
  obj->app = &app;
  obj->SSQ = SSQ;

  obj->connection = (DBusGConnection *) NULL;
  obj->bus_proxy = (DBusGProxy *)NULL;
  if( config_path == NULL ) {
    config_path_len = strlen(ALP_PRV_SPF_DEFAULT_CONFIG_FILE_PATH_RO);
    config_path = g_malloc( config_path_len + 1 );
    if (config_path == NULL) {
	return;
    }

    strncpy( config_path, ALP_PRV_SPF_DEFAULT_CONFIG_FILE_PATH_RO, 
            config_path_len );

    config_path[config_path_len + 1] = '\0';
  }
  obj->config_path = config_path;

  alp_prv_spf_so_read( obj, config_path );

  alert_status.next = (tAlertStatus *)NULL;
  alert_status.policy = NULL;
  alert_status.assertion = NULL;
  alert_status.response = NULL;
}

static void security_object_class_init( SecurityObjectClass *klass ) {
  dbus_g_object_type_install_info( SECURITY_TYPE_OBJECT, &dbus_glib_alp_security_service_object_info );
}

gboolean alp_security_service_update( SecurityObject *obj, char *message, GError **error ) {
  size_t message_len;
  gboolean rv = TRUE;
  AlpPrvSpfUpdate update;
  AlpPrvSpfSoErr err = ALP_PRV_SPF_SO_E_NOERR;

  d("%SPFSS-D-UPDATE; Beginning Security Policy Update.\n");

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( message, char, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

    message_len = strlen(message);
    update._xmlData = message;

    err = alp_prv_spf_su_update(obj, &update, (unsigned int)message_len);

    if (err != ALP_PRV_SPF_SO_E_NOERR) {
        g_set_error(error, SSQ, err, "Update Failed");

        rv = FALSE;
        break;
    }
  } while( 0 );

  return( rv );
}

gboolean alp_security_service_ps_get_policyname_from_policy( SecurityObject *obj, unsigned int policy, gchar **out_policyname, gchar **out_parentname, GError **error ) {
  gboolean rv = TRUE;
  char *value = NULL;
  AlpPrvSpfSoSecurityPolicy *found_policy;

#if defined( DEBUG )
  if( obj->debug != 0 ) {
    fprintf( app.debug_fp, "%%SPFSS-D-ENTER; Entering alp_security_service_ps_get_policyname_from_policy\n");
    fflush( app.debug_fp );
  }
#endif

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_policyname, gchar *, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_parentname, gchar *, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

    if( alp_prv_spf_so_policy_find_by_id( obj, policy, &found_policy ) != ALP_PRV_SPF_SO_E_NOERR ) {
      rv = FALSE;
      g_set_error( error, SSQ, ALP_PRV_SPF_SO_E_UNKNOWN_POLICY, "can't find policy %d", policy );
      break;
    }

    if( found_policy == (AlpPrvSpfSoSecurityPolicy *) NULL ) {
      *out_parentname = NULL;
      *out_policyname = NULL;
    } else {
      *out_policyname = g_strdup( found_policy->name );
      if( found_policy->base == NULL ) {
	*out_parentname = NULL;
      } else {
	*out_parentname = g_strdup( (found_policy->base)->name );
      }
    }

  } while( 0 );
  
  return( rv );
}

gboolean alp_security_service_ps_get_policy_from_policyname( SecurityObject *obj, char *policyname, unsigned int *out_policy, GError **error ) {
  gboolean rv = TRUE;
  char *value = NULL;
  AlpPrvSpfSoSecurityPolicy *found_policy;

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( policyname, char, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_policy, unsigned int, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

    if( alp_prv_spf_so_policy_find_by_name( obj, policyname, &found_policy ) != ALP_PRV_SPF_SO_E_NOERR ) {
      rv = FALSE;
      g_set_error( error, SSQ, ALP_PRV_SPF_SO_E_UNKNOWN_POLICY, "can't find policy %s", policyname );
      break;
    }

    if( found_policy == (AlpPrvSpfSoSecurityPolicy *) NULL ) {
      *out_policy = 0;
    } else {
      *out_policy = found_policy->id;
    }

  } while( 0 );
  
  return( rv );
}

gboolean alp_security_service_ps_packageid_increment( SecurityObject *obj, unsigned int packageid, unsigned int *out_count, GError **error ) {
  gboolean rv = TRUE;
  AlpPrvSpfSoPackage *package = NULL;
  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

    /* Find package, error if you can't find it */
    if( alp_prv_spf_so_package_find_by_id( obj, packageid, &package ) != 
	ALP_PRV_SPF_SO_E_NOERR ) {
      rv = FALSE;
      g_set_error( error, SSQ, ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE, "can't find package %d", packageid );

      break;
    }

    /* set out_count */
    if( out_count != (unsigned int *) NULL ) {
      *out_count = package->ref_count;
    }

    /* increment count */
    (package->ref_count)++;

    alp_prv_spf_so_write( obj, obj->config_path );

  } while( 0 );

  return( rv );
}

gboolean alp_security_service_ps_packageid_decrement( SecurityObject *obj, unsigned int packageid, unsigned int *out_count, GError **error ) {
  gboolean rv = TRUE;
  AlpPrvSpfSoPackage *package = NULL;
  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

    /* Find package, error if you can't find it */
    if( alp_prv_spf_so_package_find_by_id( obj, packageid, &package ) != 
	ALP_PRV_SPF_SO_E_NOERR ) {
      rv = FALSE;
      g_set_error( error, SSQ, ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE, "can't find package %d", packageid );

      break;
    }

    /* set out_count */
    if( out_count != (unsigned int *) NULL ) {
      *out_count = package->ref_count;
    }

    /* decrement count */
    (package->ref_count)--;

    alp_prv_spf_so_write( obj, obj->config_path );

  } while( 0 );

  return( rv );
}

gboolean alp_security_service_ps_packageid_count( SecurityObject *obj, unsigned int packageid, unsigned int *out_count, GError **error ) {
  gboolean rv = TRUE;
  AlpPrvSpfSoPackage *package = NULL;

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_count, unsigned int, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

    /* Find package, error if you can't find it */
    if( alp_prv_spf_so_package_find_by_id( obj, packageid, &package ) != 
	ALP_PRV_SPF_SO_E_NOERR ) {
      rv = FALSE;
      g_set_error( error, SSQ, ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE, "can't find package %d", packageid );

      break;
    }

    /* set out_count */
    *out_count = package->ref_count;

  } while( 0 );

  return( rv );
}


gboolean alp_security_service_ps_get_policy_from_certificate( SecurityObject *obj, char *certificate, unsigned int *out_policy, GError **error ) {
  gboolean rv = TRUE;
  char *value = NULL;
  AlpPrvSpfSoSecurityPolicy *found_policy;
  AlpPrvSpfSoCertificateReference *certificate_reference;
  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( certificate, char, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_policy, unsigned int, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

    certificate_reference = (AlpPrvSpfSoCertificateReference *) malloc( sizeof( AlpPrvSpfSoCertificateReference ) );

    ALP_PRV_SPF_SO_BREAK_IF_NULL( certificate_reference, AlpPrvSpfSoCertificateReference, rv );

    certificate_reference->encoding = certificate;

    if( alp_prv_spf_so_certificate_populate( certificate_reference ) != ALP_PRV_SPF_SO_E_NOERR ) {
      /* todo : modify code to properly use GError errors. */
      /* rv = FALSE; */
      break;
    }

    if( alp_prv_spf_so_policy_find_by_certificate_reference( obj, certificate_reference, &found_policy ) != ALP_PRV_SPF_SO_E_NOERR ) {
      /* todo : modify code to properly use GError errors. */
      /* rv = FALSE; */
      break;
    }

    if( found_policy == (AlpPrvSpfSoSecurityPolicy *) NULL ) {
      *out_policy = 0;
    } else {
      *out_policy = found_policy->id;
    }

  } while( 0 );
  
  return( rv );
}

gboolean alp_security_service_ps_get_policy_from_gid( SecurityObject *obj, unsigned int gid, unsigned int *out_policy, GError **error ) {
  gboolean rv = TRUE;
  unsigned int index;
  AlpPrvSpfSoSecurityPolicy *current = NULL;

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_policy, unsigned int, rv );

    *out_policy = 0;

    /* check to see if the gid is below min or above next */
    if( ( gid < obj->policy_gid_min ) || ( gid >= obj->policy_gid_next ) ) {
      break;
    }

    /* for each policy, check to see if it's gid is equal to the one given */
    for( index = 0; index < obj->policies->len; index++ ) {
      current = (AlpPrvSpfSoSecurityPolicy *) obj->policies->pdata[ index ];
      if( current->gid == gid ) {
	break;
      }
    }

    /* if we can't find the gid, set an error and return a zero */
    if( index >= obj->policies->len ) {
      *out_policy = 0;
      break;
    }

    *out_policy = current->id;
  } while( 0 );

  return( rv );
}

gboolean alp_security_service_ps_get_certificate_from_policy( SecurityObject *obj, unsigned int policy, gchar **out_certificate, gchar **out_certid, GError **error ) {
  gboolean rv = TRUE;

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_certificate, gchar *, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_certid, gchar *, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

  } while( 0 );

  return( rv );
}

gboolean alp_security_service_ps_get_gid_from_policy( SecurityObject *obj, unsigned int policy, unsigned int *out_gid, GError **error ) {
  gboolean rv = TRUE;
  unsigned int index;
  AlpPrvSpfSoSecurityPolicy *current = (AlpPrvSpfSoSecurityPolicy *)NULL;

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_gid, unsigned int, rv );

    *out_gid = 0;

    /* check to see if the policy is below min or above next */
    if( ( policy < obj->policy_id_min ) || ( policy >= obj->policy_id_next ) ) {
      break;
    }

    /* for each policy, check to see if it's id is equal to the one given */
    for( index = 0; index < obj->policies->len; index++ ) {
      current = (AlpPrvSpfSoSecurityPolicy *) obj->policies->pdata[ index ];
      if( current->id == policy ) {
	break;
      }
    }

    /* if we can't find the gid, set an error and return a zero */
    if( index >= obj->policies->len ) {
      *out_gid = 0;
      break;
    }

    *out_gid = current->gid;

  } while( 0 );

  return( rv );
}

gboolean alp_security_service_ps_get_policy_from_uid( SecurityObject *obj, unsigned int uid, unsigned int *out_policy, GError **error ) {
  gboolean rv = TRUE;
  unsigned int index;
  AlpPrvSpfSoPackage *current;
  AlpPrvSpfSoSecurityPolicy *current_policy = (AlpPrvSpfSoSecurityPolicy *)NULL;

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_policy, unsigned int, rv );
   
    *out_policy = 0;

    /* if UID is less than minuid or greater that nextuid, break */
    if( ( uid < obj->package_uid_min ) || ( uid >= obj->package_uid_next ) ) {
      break;
    }

    /* Find package with given UID */
    for( index = 0; index < obj->packages->len; index++ ) {
      current = (AlpPrvSpfSoPackage *) obj->packages->pdata[ index ];
      if( current->uid == uid ) {
	current_policy = current->policy;
	break;
      }
    }

    /* if we can't find the uid, set an error and return a zero */
    if( current_policy == (AlpPrvSpfSoSecurityPolicy *) NULL ) {
      *out_policy = 0;
      break;
    }

    *out_policy = current_policy->id;
  } while( 0 );

  return( rv );
}

/* todo : finish implementation */
gboolean alp_security_service_ps_get_policy_from_package( SecurityObject *obj, char *package, unsigned int *out_policy, GError **error ) {
  gboolean rv = TRUE;

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( package, char, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_policy, unsigned int, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

  } while( 0 );

  return( rv );
}

gboolean alp_security_service_ps_get_supplementary_group_size( SecurityObject *obj, unsigned int policy, unsigned int *out_size, GError **error ) {
  gboolean rv = TRUE;

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_size, unsigned int, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );
  } while( 0 );

  return( rv );
}

gboolean alp_security_service_ps_get_uid_gid_sgids( SecurityObject *obj, unsigned int packageid, unsigned int *out_uid, unsigned int *out_gid, GError **error ) {
  gboolean rv = TRUE;

  AlpPrvSpfSoSecurityPolicy *policy_structure;
  AlpPrvSpfSoPackage *package_structure;
  AlpPrvSpfSoErr err = ALP_PRV_SPF_SO_E_NOERR;

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_uid, unsigned int, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_gid, unsigned int, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

    /* Find Package */
    err = alp_prv_spf_so_package_find_by_id( obj, packageid, &package_structure );
    if( ALP_PRV_SPF_SO_E_NOERR != err ) {
      break;
    }

    policy_structure = package_structure->policy;

    if( (AlpPrvSpfSoSecurityPolicy *) NULL == policy_structure ) {
      err = ALP_PRV_SPF_SO_E_NOT_FOUND;
      break;
    }

    /* Set UID */
    *out_uid = package_structure->uid;

    /* Set GID */
    *out_gid = policy_structure->gid;

  } while( 0 );

  return( rv );
}

/* todo : finish implementation */
gboolean alp_security_service_ps_enroll_package( SecurityObject *obj, char *packagename, unsigned int *out_packageid, GError **error ) {
  return( alp_security_service_apas_enroll( obj, packagename, out_packageid, error ) );
}

gboolean alp_security_service_info( SecurityObject *obj, gchar **out_version, GHashTable **out_calls, GError **error ) {
  gboolean rv = TRUE;
  DBusConnection *generic_connection;

  FILE *fp;
  unsigned long pid;
  dbus_bool_t wubba;
  guint npid = 96967;
  const char *proxyname = "unset";
  
  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_calls, GHashTable *, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_version, gchar *, rv );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( error, GError *, rv );

    *out_version = g_strdup("$Revision: #3 $");
    
    *out_calls = g_hash_table_new( NULL, NULL );

    g_hash_table_insert( *out_calls, "update", "0" );
    g_hash_table_insert( *out_calls, "get_policyname_from_policy", "0" );

  } while( 0 );

  return( rv );
}

static gboolean alp_prv_security_service_parsecli( int argc, char *argv[], App *app ) {
  int i;
  gboolean rv = TRUE;

  /* turn debugging off by default */
  app->debug = 0;

  /* look for the default.policies file in the default location */
  app->config_path = ALP_PRV_SPF_DEFAULT_CONFIG_FILE_PATH_RO;

  /* by default we write the debug log into /var/log/security. But up 'til */
  /* the time we open that logfile (or if we can't open it) we write debug */
  /* messages to standard error.                                           */
  app->log_path = ALP_PRV_SPF_DEFAULT_LOG_FILE_PATH;
  app->debug_fp = stderr;

  /* we're evenutally going to try to change the UID of this process to    */
  /* the user "security". This can be changed with the -u command line     */
  /* option. If we can't find the UID for user "security" in /etc/passwd,  */
  /* we will try to change our UID to 105. However.. if we're not running  */
  /* as root, we can't really change our UID, so we won't even try.        */
  app->user_set = 0;
  app->user = ALP_PRV_SPF_DEFAULT_USER;
  app->group = ALP_PRV_SPF_DEFAULT_GROUP;
  app->uid = ALP_PRV_SPF_DEFAULT_UID;
  app->gid = ALP_PRV_SPF_DEFAULT_GID;
  app->background = 1;
  alp_prv_spf_attn_properties_set(app, ATTN_PROP_TWO);

  for( i=1; i<argc; i++ ) {
    /* if we see the -d flag, turn debugging on */
    if( !strncmp( "-d", argv[i], 2 ) ) {
      app->debug = 1;
      continue;
    }

    /* if we see the -f flag, the next argument is the name of the config file */
    if( !strncmp( "-f", argv[i], 2 ) ) {
      /* only change the config path if there is a next argument */
      i++;
      if( i >= argc ) {
	break;
      }

      app->config_path = argv[i];
      continue;
    }

    /* if we see the -h flag, print a usage message and exit */
    if( !strncmp( "-h", argv[i], 2 ) ) {
      rv = FALSE;
      alp_prv_security_service_usage( argv[0] );
      break;
    }

    if( !strncmp( "-l", argv[i], 2 ) ) {
      /* only change the log path if there is a next argument */
      i++;
      if( i >= argc ) {
	break;
      }

      app->log_path = argv[i];
      continue;
    }

    if( !strncmp( "-u", argv[i], 2 ) ) {
      /* only change the user if there is a next argument */
      i++;
      if( i >= argc ) {
	break;
      }

      app->user = argv[i];
      app->user_set = ( app->user_set ) | 0x00000001;
      continue;
    }

    if( !strncmp( "-v", argv[i], 2 ) ) {
      /* only change the uid if there is a next argument */
      i++;
      if( i >= argc ) {
	break;
      }

      app->uid = atoi( argv[i] );
      app->user_set = ( app->user_set ) | 0x00000002;
      continue;
    }

    if( !strncmp( "-g", argv[i], 2 ) ) {
      /* only change the group if there is a next argument */
      i++;
      if( i >= argc ) {
	break;
      }

      app->group = argv[i];
      app->user_set = ( app->user_set ) | 0x00000004;
      continue;
    }

    if( !strncmp( "-h", argv[i], 2 ) ) {
      /* only change the gid if there is a next argument */
      i++;
      if( i >= argc ) {
	break;
      }

      app->uid = atoi( argv[i] );
      app->user_set = ( app->user_set ) | 0x00000008;
      continue;
    }

    if( !strncmp( "-b", argv[i], 2 ) ) {
      app->background = 0;
    }

    if( !strncmp( "-a", argv[i], 2 ) ) {
      i++;
      if( i >= argc ) {
	break;
      }
      
      switch( atoi( argv[i] ) ) {
      case 4:
        alp_prv_spf_attn_properties_set(app, ATTN_PROP_FOUR);
	break;

      case 3:
        alp_prv_spf_attn_properties_set(app, ATTN_PROP_THREE);
	break;

      case 2:
      default:
        alp_prv_spf_attn_properties_set(app, ATTN_PROP_TWO);
	break;
      }

    }

  }

  return( rv );
}

static void alp_prv_security_service_usage( char *command ) {
  printf("Questions are a burden to others, Answers are a prison for oneself.\nUsage : %s [options], where options are:\n  -d               : turn debugging on\n  -f <config_file> : use a different config file\n  -h               : print this help message.\n  -l <log_file>    : use a different log file\n  -u <username>    : run as a different user\n  -v <uid>         : run with a specific UID\n  -g <groupname>   : run with a different group\n  -h <gid>         : run with a specific GID\n", command );
}

static void open_logs( App *app ) {
  FILE *fp;
  time_t hsin = (time_t) 0;
  char *timehack;
  unsigned int time_length;

  do {
    if( ( fp = fopen( app->log_path, "a" ) ) == (FILE *) NULL ) {
      fprintf( app->debug_fp, "%%SPFSS-E-LOG; Error opening log file '%s'.\n;Errno = %d (%s).\n", app->log_path, errno, strerror( errno ) ); fflush( app->debug_fp );
      fflush( app->debug_fp );
      break;
    }
    fprintf( app->debug_fp, "%%SPFSS-I-LOGFILE; Opened log file %s.\n", app->log_path ); fflush( app->debug_fp );
    fflush( app->debug_fp );

    app->debug_fp = fp;

    time( & hsin );
    timehack = ctime( &hsin );;
    time_length = strlen( timehack );
    
    if( hsin == (time_t) 0 ) {
      fprintf( stderr, "yikes, time == zero\n" ); fflush( stderr );
      break;
    }

    fprintf( app->debug_fp, "%%SPFSS-I-HSIN; Logging Begins at %*.*s.\n", time_length - 1, time_length - 1, timehack ); fflush( app->debug_fp );
    fflush( app->debug_fp );
    fprintf( stderr, "%%SPFSS-I-HSIN; Logging Begins at %*.*s.\n", time_length - 1, time_length - 1, timehack ); fflush( stderr );
  } while ( 0 );

}

static void change_uid( App *app ) {
  uid_t uid;
  struct passwd *passwd;
  struct group *group;

  /* If we're not root, then we can't call setuid() successfully, so just */
  /* forget about the whole idea.                                         */
  
  uid = geteuid();

  if( uid != 0 ) {
#if defined( DEBUG)
    if( app->debug != 0 ) {
      fprintf( app->debug_fp, "%%SPFSS-E-SETUID; Can't set UID. Not running as root (%d).\n", uid ); fflush( app->debug_fp );
    }
#endif
  } else {
    /* If we've been given a UID, use that in preference to a username   */
    /* given with the -u command line option.                            */

    if( ( app->user_set & 0x2 ) == 0 ) {
      for( passwd = getpwent(); passwd != (struct passwd *) NULL; passwd = getpwent() ) {
	if( !strcmp( passwd->pw_name, app->user ) ) {
	  app->uid = passwd->pw_uid;
	  break;
	}
      }
      endpwent();
    }
  }

  /* Now do the same for the group */
  if( ( app->user_set & 0x8 ) == 0 ) {
    for( group = getgrent(); group != (struct group *) NULL; group = getgrent() ) {
      if( !strcmp( group->gr_name, app->group ) ) {
	app->gid = group->gr_gid;
	break;
      }
    }
    endgrent();
  }

  setgid( app->gid );
  setuid( app->uid );
}


int main( int argc, char *argv[] ) {
  DBusGConnection *bus;
  DBusGProxy *bus_proxy;
  GError *error = NULL;
  SecurityObject *obj;
  GMainLoop *mainloop;
  guint request_name_result;
  AlpPrvSpfSoErr status;
  static char *paths[] = { "/var/LSMSPF_init", "/etc/default.policies", "/var/log/security", NULL };


  if( alp_prv_security_service_parsecli( argc, argv, &app ) == FALSE ) {
    return(1);
  }

  if( app.background ) {
    alp_daemon_start( 1, 1 );
  }

  paths[1] = app.config_path;
  paths[2] = app.log_path;

  take_paths( &app, paths );

  open_logs( &app );

  install_signal( &app );

  change_uid( &app );

  g_type_init();

  dbus_g_object_type_install_info (SECURITY_TYPE_OBJECT, &dbus_glib_alp_security_service_object_info);

  mainloop = g_main_loop_new (NULL, FALSE);

  bus = dbus_g_bus_get (DBUS_BUS_SYSTEM, &error);
  if (!bus) {
    printf("Couldn't connect to session bus (%s)\n", error->message );
    exit(3);
  }

  bus_proxy = dbus_g_proxy_new_for_name (bus, "org.freedesktop.DBus",
                                         "/org/freedesktop/DBus",
                                         "org.freedesktop.DBus");

  if (!dbus_g_proxy_call (bus_proxy, "RequestName", &error,
                          G_TYPE_STRING, "com.palmsource.security.server.interface",
                          G_TYPE_UINT, 0,
                          G_TYPE_INVALID,
                          G_TYPE_UINT, &request_name_result,
                          G_TYPE_INVALID)) {
    printf ("Failed to acquire com.palmsource.security.server.interface (%s)\n", error->message);
    exit(4);
  }

  obj = g_object_new( SECURITY_TYPE_OBJECT, NULL );

  /* fails if no Meta Data init file present */
  status = alp_prv_spf_so_kernel_load_meta();
  if (status != 0)
  {
	/* initialize LSM from SPF policy */
    alp_prv_spf_so_kernel_initialize( obj );

	/* dump_meta will fail if no Meta Data */
    status = alp_prv_spf_so_kernel_dump_meta();
  }

  /* Enable LSM enforcement only if we have LSM Meta data */
  if (status == 0)
    alp_prv_spf_so_kernel_lsm_enable();

  obj->debug = app.debug;
  obj->connection = bus;
  obj->bus_proxy = bus_proxy;
  dbus_g_connection_register_g_object (bus, "/SecurityObject", G_OBJECT (obj));

  if( app.background ) {
    alp_daemon_release( 0 );
  }

  g_main_loop_run (mainloop);

  exit (0);
}

static void close_log() {
  time_t hsin = (time_t) 0;
  char *timehack;
  unsigned int time_length;

  if( app.debug_fp != stderr ) {
    time( & hsin );
    timehack = ctime( & hsin );
    time_length = strlen( timehack );
    fprintf( app.debug_fp, "%%SPFSS-I-HSINF; Logging Ends at %*.*s.\n", time_length - 1, time_length - 1, timehack );
    fflush( app.debug_fp );
    fclose( app.debug_fp );
  }

}
static void signal_close_log( int signum ) {

  close_log();

  if( signum == SIGTERM ) {
    if( ( app.old_signal != SIG_DFL ) && ( app.old_signal != SIG_IGN ) ) {
      (*(app.old_signal))( signum );
    }
    exit(0);
  }
}

static void install_signal( App *app ) {
  app->old_signal = signal( SIGTERM, signal_close_log );
}


gboolean alp_security_service_pkcs5_set_phrase (SecurityObject *obj, 
		                                unsigned int which, 
					        char *phrase, 
					        GError **error ) {
  gboolean rv = TRUE;

  d("%SPFSS-D-PKCS5SP; Entering PKCS5 Set Phrase.\n");

  if ((obj == NULL) || (phrase == NULL) || (error == NULL)) {
      g_set_error(error, SSQ, ALP_PRV_SPF_SO_E_NULL_PARAMETER, 
		  "null parameter");
      return (FALSE);
  }

  rv = alp_prv_spf_pkcs5_set_phrase(obj, which, phrase, error);

  return (rv);
}


gboolean alp_security_service_pkcs5_check_phrase (SecurityObject *obj, 
		                                  unsigned int which, 
						  char *trial_phrase, 
						  unsigned int *correct, 
						  GError **error ){
  gboolean rv = TRUE;
  AlpPrvSpfSuPKCS5 *pkcs5;

  d("%SPFSS-D-PKCS5CP; Entering PKCS5 Check Phrase.\n");

  if ((obj == NULL) || (trial_phrase == NULL) || 
      (correct == NULL) || (error == NULL)) {
      g_set_error(error, SSQ, ALP_PRV_SPF_SO_E_NULL_PARAMETER, 
		  "null parameter");
      return (FALSE);
  }

  rv = alp_prv_spf_pkcs5_check_phrase(obj, which, trial_phrase, 
		                                   correct, error);

  return (rv);
}


gboolean alp_security_service_pkcs5_set_parameters (SecurityObject *obj, 
		                                    unsigned int which, 
						    unsigned int iteration, 
						    unsigned int active, 
						    char *salt, 
						    GError **error) {
  gboolean rv = TRUE;
  AlpPrvSpfSuPKCS5 *pkcs5;
  void *old_salt;

  d("%SPFSS-D-PKCS5SR; Entering PKCS5 Set Parameters.\n");

  if ((obj == NULL) || (salt == NULL) || (error == NULL)) {
      g_set_error(error, SSQ, ALP_PRV_SPF_SO_E_NULL_PARAMETER, 
		  "null parameter");
      return (FALSE);
  }

  rv = alp_prv_spf_pkcs5_set_parameters(obj, which, iteration, 
		                                     active, salt, error);

  return (rv);
}


gboolean alp_security_service_pkcs5_get_parameters (SecurityObject *obj, 
		                                    unsigned int which, 
						    unsigned int *iteration, 
						    unsigned int *active, 
						    char **salt, 
						    GError **error) {
  gboolean rv = TRUE;
  AlpPrvSpfSuPKCS5 *pkcs5;
  char *local_salt = NULL;
  int bytes_written = 0;

  d("%SPFSS-D-PKCS5GR; Entering PKCS5 Get Parameters.\n");

  if ((obj == NULL) || (iteration == NULL) || (active == NULL) || 
      (salt == NULL) || (error == NULL)) {
      g_set_error(error, SSQ, ALP_PRV_SPF_SO_E_NULL_PARAMETER, 
		  "null parameter");
      return (FALSE);
  }

  rv = alp_prv_spf_pkcs5_get_parameters(obj, which, iteration,
		                                     active, salt, error);

  return (rv);
}

static void alp_prv_security_service_sha_this( char *data, unsigned int length, char *output ) {
  SHA_CTX context;

  SHA_Init( &context );
  SHA_Update( &context, data, (size_t) length );
  SHA_Final( output, &context );

}


gboolean alp_security_service_apas_enroll (SecurityObject *obj, 
		                           char *path, 
                                           unsigned int *out_packageid, 
                                           GError **error) {
  FILE *fp = (FILE *) NULL;
  gboolean rv = TRUE;
  struct stat buf;
  char *real_path = (char *)NULL;
  char *buffer = (char *)NULL;
  size_t bytes_read, length;
  unsigned int packageid;
  AlpPrvSpfManifest manifest;
  AlpPrvSpfSoErr err;
  AlpPrvSpfSoSecurityPolicy *policy;
  AlpPrvSpfSoPackage *new_package = NULL;
  FILE *debug_fp;
  char digest[20];
  gboolean hash_found = FALSE;

  if ((obj == NULL) || (path == NULL) || (out_packageid == NULL) ||
      (error == NULL)) {
      err = ALP_PRV_SPF_SO_E_NULL_PARAMETER;
      g_set_error(error, SSQ, ALP_PRV_SPF_SO_E_NULL_PARAMETER, 
		  "null parameter");
      return FALSE;
  }

  debug_fp = obj->debug_fp;

  d("%SPFSS-D-APASE; Beginning APAS ENROLL.\n");
  fprintf(debug_fp, "; out_packageid = %08X, path = %s\n", (int)out_packageid, 
                                                           path );
  fflush( debug_fp );

  manifest._name           = NULL;
  manifest._cert.issuer    = NULL;
  manifest._cert.serial    = NULL;
  manifest._cert.encoding  = NULL;
  buffer = NULL;

  do {

    /* Does the file pointed to by path exist? */
    if (0 != stat(path, &buf)) {
        rv = FALSE;
        g_set_error(error, SSQ, ALP_PRV_SPF_SO_E_NOT_FOUND, 
                    "security manifest not found in path %s", path);
	break;
    }

    /* 
     * set the path to path/security.xml. security.xml contains 
     * certref which is defines the bundle's policy id is. 
     */
  	
    /* Is it a directory? */
    if (S_ISDIR( buf.st_mode)) {
        length = strlen(path);
        real_path = malloc(length + strlen(ALP_PRV_SPF_SECURITY_XML_FILE) + 1);
        if (real_path == NULL) {
            rv = FALSE;
	    break;
        }

        sprintf(real_path, "%s/%s", path, ALP_PRV_SPF_SECURITY_XML_FILE);
    } else {
        real_path = path; 
    }

    if (0 != stat(real_path, &buf)) {
        rv = FALSE;
        g_set_error(error, SSQ, ALP_PRV_SPF_SO_E_NOT_FOUND, 
                    "security manifest not found in path %s", path);
	break;
    }


    /* if it exists and it's not a directory, try to open it. */
    fp = fopen(real_path, "r");
    if ((FILE *)NULL == fp) {
        rv = FALSE;
        g_set_error(error, SSQ, ALP_PRV_SPF_SO_E_NOT_FOUND, 
                    "security manifest not found in path %s", path);
        break; 
    }

    buffer = g_malloc(buf.st_size + 1);
    if (buffer == NULL) {
        rv = FALSE;
        break; 
    }

    bzero(buffer, (buf.st_size));

    bytes_read = fread(buffer, 1, buf.st_size, fp);
    if (bytes_read != (guint)buf.st_size) {
        rv = FALSE;
        g_set_error(error, SSQ, ALP_PRV_SPF_SO_E_FILE, 
                    "error (%s) while trying to read file (%s)", 
                    strerror(errno), real_path );
        break; 
    }

    /*
     * Buffer contains the security.xml contents. hash the contents and 
     * check if we have seen a similar hash before. If yes, then the package 
     * to policy id binding already exists. 
     */ 
    buffer[bytes_read] = '\0';

    /* hash the security.xml and see if we've seen it before */
    alp_prv_hash_buffer(buffer, bytes_read, digest);

    hash_found = alp_prv_spf_so_package_find_by_hash(obj, digest, &new_package);
    if (hash_found) {
        *out_packageid = new_package->id;
        break; 
    }

    /*
     * package does not have an entry in the policy file. Check if the bundle 
     * has been propely signed and verified
     */
    manifest._xmlData = buffer;

    err = alp_prv_spf_sm_parse(obj, &manifest, (guint)buf.st_size);
    if (ALP_PRV_SPF_SO_E_NOERR != err) { 
        g_set_error( error, SSQ, err, "error parsing %s\n", real_path );
        rv = FALSE;
        break; 
    }

    /* 
     * Now we have to map the certificate to policy
     * find policy by certificate 
     */
    err = alp_prv_spf_so_policy_find_by_certificate_reference(obj,
                                                              &(manifest._cert),
                                                              &policy);
    if (ALP_PRV_SPF_SO_E_NOERR != err) {
        g_set_error(error, SSQ, err, 
	            "error finding policy for certificate %s %s\n", 
                    manifest._cert.issuer, manifest._cert.serial); 
        rv = FALSE;
        break; 
    }

                                                            
    /* 
     * found the policy corresponding to the certificate. Now add the package 
     */
    new_package = (AlpPrvSpfSoPackage *)malloc(sizeof(AlpPrvSpfSoPackage));
    if ((AlpPrvSpfSoPackage *)NULL == new_package) {
        err = ALP_PRV_SPF_SO_E_MALLOC;
        g_set_error(error, SSQ, err, "error allocating space for new package");
        rv = FALSE;
	break;
    }

    new_package->id = 0;
    new_package->uid = 0;
    new_package->ref_count = 1;
    memcpy(new_package->hash, digest, 20);
    new_package->package = g_strdup(manifest._name);
    new_package->policy = policy;

    err = alp_prv_spf_so_package_add(obj, new_package);
    if (ALP_PRV_SPF_SO_E_NOERR != err) {
        rv = FALSE;
        g_set_error(error, SSQ, err, "error adding package %s\n", 
			             manifest._name);
        break; 
    }

    /* return package */
    *out_packageid = new_package->id;

    /* update file */
    alp_prv_spf_so_write(obj, obj->config_path);
  
  } while (0);

  /*
   * cleanup and free 
   */
  if (real_path && (real_path != path)) {
      free(real_path);
  }

  if (manifest._name) {
      free(manifest._name);
  }

  if (manifest._cert.issuer) {
      free(manifest._cert.issuer);
  }

  if (manifest._cert.serial) {
      free(manifest._cert.serial);
  }

  if (manifest._cert.encoding) {
      free(manifest._cert.encoding);
  }

  if (buffer) {
      free(buffer);
  }

  if ((FILE *)NULL != fp) {
      fclose( fp );
  }

  return( rv );
}

gboolean alp_security_service_apas_path_get_packageid( SecurityObject *obj, char *path, unsigned int *out_packageid, GError **error ){
  gboolean rv = TRUE;
  AlpPrvSpfSoPackage *current = NULL;
  unsigned int i;
  AlpPrvSpfSoErr err = ALP_PRV_SPF_SO_E_NOERR;

  fprintf( obj->debug_fp, "%%SPF-I_PGPI; path = %s, out_packageid = %08X\n", path, (int) out_packageid );
  fflush( obj->debug_fp );

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, err );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( path, char, err );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_packageid, unsigned int, err );
    
    fprintf( obj->debug_fp, "; there are %d packages\n", obj->packages->len );
    fflush( obj->debug_fp );

    for( i = 0; i < obj->packages->len; i++ ) {
      current = (AlpPrvSpfSoPackage *) obj->packages->pdata[i];
      fprintf( obj->debug_fp, "; comparing (%s) with (%s)\n", path, current->package );
      fflush( obj->debug_fp );
      if( !strcmp( current->package, path ) ) {
	fprintf( obj->debug_fp, "; found id = %d\n", current->id );
	fflush( obj->debug_fp );
	break;
      }
    }

    if( i < obj->packages->len ) {
      *out_packageid = current->id;
      fprintf( obj->debug_fp, "; current = %d error = %08X\n" , *out_packageid, err );
      fflush( obj->debug_fp );

    } else {
      err = ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE;
      g_set_error( error, SSQ, err, "unknown package: %s", path);
      fprintf( obj->debug_fp, "; error = %08X\n",  err );
      fflush( obj->debug_fp );
      break;
    }

  } while( 0 );

  if( ALP_PRV_SPF_SO_E_NOERR != err ) {
    rv = FALSE;
  }

  return( rv );
}

gboolean alp_security_service_apas_packageid_remove( SecurityObject *obj, unsigned int packageid, unsigned int *out_refcount, GError **error ){
  g_set_error( error, SSQ, ALP_PRV_SPF_SO_E_UNIMPLEMENTED, "method apas_packageid_remove unimplemented");
  return(FALSE);
}

gboolean alp_security_service_apas_packageid_uid_get( SecurityObject *obj, unsigned int packageid, unsigned int *out_uid, GError **error ){
  gboolean rv = FALSE;
  AlpPrvSpfSoErr err = ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE;
  unsigned int i;
  AlpPrvSpfSoPackage *current = NULL;

  fprintf( obj->debug_fp, "%%APAS-I-UIDGET; Asking for the UID for packageid %d\n", packageid );
  fflush( obj->debug_fp );

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, err );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_uid, unsigned int, err );

    for( i=0; i < obj->packages->len; i++ ) {
      current = (AlpPrvSpfSoPackage *) obj->packages->pdata[i];
      if( current->id == packageid ) {
	err = ALP_PRV_SPF_SO_E_NOERR;
	*out_uid = current->uid;
	rv = TRUE;
	break;
      }
    }

    if( ALP_PRV_SPF_SO_E_NOERR != err ) {
      g_set_error( error, SSQ, err, "could not find package with id %d", packageid );
    }
    
  } while( 0 );

  if( ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE != err ) {
    fprintf( obj->debug_fp, "%%APAS-E-UIDGET; Error %08X.\n", err );
  } else {
    fprintf( obj->debug_fp, "%%APAS-S-UIDGET; Found uid %d for packageid %d\n", current->uid, packageid );
  }
  fflush( obj->debug_fp );

  return( rv );
}

gboolean alp_security_service_apas_packageid_gid_get( SecurityObject *obj, unsigned int packageid, unsigned int *out_gid, GError **error ){
  gboolean rv = FALSE;
  AlpPrvSpfSoErr err = ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE;
  AlpPrvSpfSoSecurityPolicy *policy = NULL;
  unsigned int i;
  AlpPrvSpfSoPackage *current;

  fprintf( obj->debug_fp, "%%APAS-I-GIDGET; Asking for the UID for packageid %d\n", packageid );
  fflush( obj->debug_fp );


  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, err );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_gid, unsigned int, err );

    for( i=0; i < obj->packages->len; i++ ) {
      current = (AlpPrvSpfSoPackage *) obj->packages->pdata[i];
      if( current->id == packageid ) {
	policy = current->policy;
	if( policy != NULL ) {
	  err = ALP_PRV_SPF_SO_E_NOERR;
	  rv = TRUE;
	}
	break;
      }
    }

    if( ALP_PRV_SPF_SO_E_NOERR != err ) {
      g_set_error( error, SSQ, err, "could not find package with id %d\n", packageid );
      break;
    }

    *out_gid = policy->gid;
  } while( 0 );

  if( ALP_PRV_SPF_SO_E_NOERR != err ) {
    fprintf( obj->debug_fp, "%%APAS-E-GIDGET; Error %08X.\n", err );
  } else {
    fprintf( obj->debug_fp, "%%APAS-S-GIDGET; Found gid %d for packageid %d\n", policy->gid, packageid );
  }
  fflush( obj->debug_fp );

  return( rv );
}

gboolean alp_security_service_apas_packageid_sgids_get( SecurityObject *obj, unsigned int packageid, char **out_sgids, GError **error ){
  gboolean rv = TRUE;
  AlpPrvSpfSoErr err = ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE;
  AlpPrvSpfSoSecurityPolicy *policy;
  unsigned int i;
  AlpPrvSpfSoPackage *current;

  fprintf( obj->debug_fp, "%%APAS-I-SGIDGET; Asking for the SGIDS for packageid %d\n", packageid );
  fflush( obj->debug_fp );

  do {
    ALP_PRV_SPF_SO_BREAK_IF_NULL( obj, SecurityObject, err );
    ALP_PRV_SPF_SO_BREAK_IF_NULL( out_sgids, char *, err );

    for( i=0; i < obj->packages->len; i++ ) {
      current = (AlpPrvSpfSoPackage *) obj->packages->pdata[i];

      if( current->id == packageid ) {
	policy = current->policy;
	if( policy != NULL ) {
	  err = ALP_PRV_SPF_SO_E_NOERR;
	  rv = TRUE;
	}
	break;
      }
    }

    if( ALP_PRV_SPF_SO_E_NOERR != err ) {
      g_set_error( error, SSQ, err, "could not find package with id %d", packageid );
      break;
    }

    // Now set the sgids string from the policy object
    *out_sgids = g_strdup( policy->sgids );

  } while( 0 );

  if( ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE != err ) {
    fprintf( obj->debug_fp, "%%APAS-E-SGIDSGET; Error %08X.", err );
  } else {
    fprintf( obj->debug_fp, "%%APAS-S-SGIDSGET; Found for packageid %d", packageid );
  }
  
  return( rv );
}

gboolean alp_security_service_query_get_assertion_string (
                                           SecurityObject *obj,
                                           unsigned int policy_id,
                                           char *assertion,
                                           unsigned int subject_pid,
                                           unsigned int client_pid,
                                           gchar **out_assertion_value, 
                                           GError **error) {
  gboolean rv;

  rv = alp_prv_spf_query_get_assertion_string(obj, policy_id, assertion,
                                              subject_pid, client_pid,
                                              out_assertion_value, error);
  
  return (rv);
}


gboolean alp_security_service_query_get_assertion_type (
                                           SecurityObject *obj,
                                           unsigned int policy_id,
                                           char *assertion,
                                           unsigned int subject_pid,
                                           unsigned int client_pid,
                                           unsigned int *out_assertion_type,
                                           GError **error) {

  gboolean rv;

  rv = alp_prv_spf_query_get_assertion_type(obj, policy_id, assertion,
                                            subject_pid, client_pid,
                                            out_assertion_type, error);
  
  return (rv);
}


gboolean alp_security_service_query_complete( SecurityObject *obj, unsigned int id, gchar **out_assertion_value, unsigned int *out_exit_okay, GError **error ) {
  gboolean rv = TRUE;

  rv = alp_prv_spf_query_complete(obj, id, out_assertion_value,
                                  out_exit_okay, error);

  return( rv );  
}

static void alp_prv_hash_buffer( char *data, unsigned int length, char *digest ) {
  alp_prv_security_service_sha_this( data, length, digest );
}

static void take_paths( App *app, char *paths[] ) {
  uid_t owner;
  unsigned int i;
  char *current;

  do {
    if( ( app == NULL ) || ( paths == NULL ) ) break;
    owner = app->uid;
    for( i=0; paths[i] != NULL; i++ ) {
      current = paths[i];
      chmod( current, 0666 );
    }
  } while( 0 );
}

/* 
 * find the policy and/or package structures associated with the uid and gid
 * associated with the subject or client pids. If the out_policy handle is 
 * null, we won't look for the policy. If the out_package handle is null, we 
 * won't look for the package. If we can't find the specified PID, we'll
 * return a ALP_PRV_SPF_SO_E_UNKNOWN_PID. If we can't find the policy
 * associated with the GID of the PID in question, we'll return an
 * ALP_PRV_SPF_SO_E_UNKNOWN_POLICY. If we can't find the package associated 
 * with the UID of the PID in question, we'll return an
 * ALP_PRV_SPF_SO_E_UNKNOWN_PACKAGE. Otherwise we'll return a
 * ALP_PRV_SPF_SO_E_NOERR.                                                   
 */

AlpPrvSpfSoErr alp_prv_get_policy_and_package_from_pids (
		                        SecurityObject *obj, 
					unsigned int client_pid, 
					unsigned int subject_pid, 
					AlpPrvSpfSoSecurityPolicy **out_policy,
				       	AlpPrvSpfSoPackage **out_package) {

  AlpPrvSpfSoErr err = ALP_PRV_SPF_SO_E_NOERR;
  unsigned int pid_to_use;
  uid_t uid;
  gid_t gid;
  gboolean rv = FALSE;

  do {
    /* 
     * Use the subject_pid if it is non-zero, otherwise use the client_pid 
     */ 
    pid_to_use = subject_pid;
    if (pid_to_use == 0) {
        pid_to_use = client_pid;
    }

    /* 
     * Now find the UID and GID associated with the PID 
     */
    err = alp_prv_spf_get_user_group_from_pid(obj, pid_to_use, &uid, &gid);
    if (err != ALP_PRV_SPF_SO_E_NOERR) {
        err = ALP_PRV_SPF_SO_E_UNKNOWN_PID;
        break;
    }

    /* 
     * If out_policy is non-null, try to find the policy based on the GID 
     */ 
    if (out_policy != NULL) {
        err = alp_prv_spf_so_policy_find_by_gid(obj, gid, out_policy);
        if (err != ALP_PRV_SPF_SO_E_NOERR) {
	    break;
        }
    }

    /* 
     * If out_package is non-null, try to find the package based on the UID 
     */ 
    if (out_package != NULL) {
        err = alp_prv_spf_so_package_find_by_uid(obj, uid, out_package);
        if (err != ALP_PRV_SPF_SO_E_NOERR) {
	    break;
        }
    }
    
  } while (0);

  return( err );
}
