/********************************************************************

Copyright 2006, ACCESS Systems Americas, Inc. All rights reserved.

The contents of this file are subject to the Mozilla Public License Version
1.1 (the "License"); you may not use this file except in compliance with
the License. You may obtain a copy of the License at
http://www.mozilla.org/MPL/

Software distributed under the License is distributed on an "AS IS" basis,
WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
for the specific language governing rights and limitations under the
License.

The Original Code is the entire contents of this file.

The Initial Developer of the Original Code is ACCESS Systems Americas, Inc.

Portions created by ACCESS Systems Americas, Inc. are Copyright  2006. All
Rights Reserved.

Contributor(s): none.

********************************************************************/
/*!
 *     @file volumesvcs.h internal definitions for volumesvcs_d.c
 *     @addtogroup ALP_VOLUMESVCS Volume Services
 * @{
 */
#ifndef SERVER_VOLUMESVCS_H
#define SERVER_VOLUMESVCS_H

#include <sys/mount.h>
#include <hiker/volumesvcs_types.h>
#include <hiker/prv/volumesvcs_prv.h>
#include <stdlib.h>
#include <hiker/traces.h>
#include <glib.h>
#include <time.h>

extern int gtracelevel;
#define TRACELEVEL 0  // 0 - none, 1 - some, 2 - more, 3 - you've got to be kidding
#define MAX_TRACELEVEL 3
#if ALP_TRACE == ALP_TRACE_ON
#define ALP_VSTL(level, str, ...) \
	if (level <= gtracelevel)  \
		ALP_TM(TL(ALP_CLASS_VOLUMESVCS, "%s():@%d " str, __PRETTY_FUNCTION__,time(NULL), ## __VA_ARGS__))
#else
#define ALP_VSTL(leve, str, ...)
#endif
#define MEMCHECK(p) ALP_FAIL_FATAL_IF( p == NULL, "out of memory")

#define AisBsParent(a,b) ( a->sys_dev_path && b->sys_dev_path && !strncmp(a->sys_dev_path, b->sys_dev_path, strlen(a->sys_dev_path)) )

#define MOUNTROOT "/media"
#define MOUNT_OPTIONS MS_NOATIME+MS_NODEV+MS_NOSUID+MS_SYNCHRONOUS
#define UMOUNT_OPTIONS MNT_FORCE // MNT_EXPIRE MNT_DETACH are other possible options
#define NOTIFY_TIMEOUT 0

#define ACCESS_UDI_ROOT "/access/ALP/volumesvcs/"

typedef struct _prv_slot {
	char *dev_path;				//!< /dev/name string (can be a substring)
	char *name;					//!< name to use if above key/value match
	int refcnt;					//!< how many entries in gfsinfo are using this
} prv_slot_t;

typedef struct _prv_fsinfo {
	AlpVolumeSvcsFSinfo pub;   //!< the public structure we export to users
	                           //!< additinoal internal only fields
	gchar *sys_dev_path;       //!< sysfs path from udev
	prv_slot_t *slotp;		   //!< pointer to the slot info, if any
	gchar *fstype;			   //!< fstype string (vfat ext3..)
	gchar *parent_udi;
	int	major;
	int minor;
} prv_fsinfo_t;

#endif

/*@}*/

